<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

 /**
  * Mindmap Course Format - displays course content as interactive,
  * personalized mindmap
  *
  * @package    format_mindmap
  * @copyright  2014 onwards Łukasz Sanokowski, Barbara Dębska
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */

$characters = array (
    "ą" => "&#261;",
    "Ą" => "&#260;",
    "ć" => "&#263;",
    "Ć" => "&#262;",
    "ę" => "&#281;",
    "Ę" => "&#280;",
    "ł" => "&#322;",
    "Ł" => "&#321;",
    "ń" => "&#324;",
    "Ń" => "&#323;",
    "ó" => "&#243;",
    "Ó" => "&#211;",
    "ś" => "&#347;",
    "Ś" => "&#346;",
    "ż" => "&#380;",
    "Ż" => "&#379;",
    "ź" => "&#378;",
    "Ź" => "&#377;",
    "Á" => "&#193;",
    "á" => "&#225;",
    "Ą" => "&#260;",
    "ą" => "&#261;",
    "Ä" => "&#196;",
    "ä" => "&#228;",
    "É" => "&#201;",
    "é" => "&#233;",
    "Ę" => "&#280;",
    "ę" => "&#281;",
    "Ě" => "&#282;",
    "ě" => "&#283;",
    "Í" => "&#205;",
    "í" => "&#237;",
    "Ó" => "&#211;",
    "ó" => "&#243;",
    "Ô" => "&#212;",
    "ô" => "&#244;",
    "Ú" => "&#218;",
    "ú" => "&#250;",
    "Ů" => "&#366;",
    "ů" => "&#367;",
    "Ý" => "&#221;",
    "ý" => "&#253;",
    "Č" => "&#268;",
    "č" => "&#269;",
    "ď" => "&#271;",
    "ť" => "&#357;",
    "Ĺ" => "&#313;",
    "ĺ" => "&#314;",
    "Ň" => "&#327;",
    "ň" => "&#328;",
    "Ŕ" => "&#340;",
    "ŕ" => "&#341;",
    "Ř" => "&#344;",
    "ř" => "&#345;",
    "Š" => "&#352;",
    "š" => "&#353;",
    "Ž" => "&#381;",
    "ž" => "&#382;",
    "À" => "&#192;",
    "à" => "&#224;",
    "Â" => "&#194;",
    "â" => "&#226;",
    "Æ" => "&#198;",
    "æ" => "&#230;",
    "Ç" => "&#199;",
    "ç" => "&#231;",
    "È" => "&#200;",
    "è" => "&#232;",
    "É" => "&#201;",
    "é" => "&#233;",
    "Ê" => "&#202;",
    "ê" => "&#234;",
    "Ë" => "&#203;",
    "ë" => "&#235;",
    "Î" => "&#206;",
    "î" => "&#238;",
    "Ï" => "&#207;",
    "ï" => "&#239;",
    "Ô" => "&#212;",
    "ô" => "&#244;",
    "Œ" => "&#140;",
    "œ" => "&#156;",
    "Ù" => "&#217;",
    "ù" => "&#249;",
    "Û" => "&#219;",
    "û" => "&#251;",
    "Ü" => "&#220;",
    "ü" => "&#252;",
    "«" => "&#171;",
    "»" => "&#187;",
    "€" => "&#128;",
    "₣" => "&#8355;",
    "Ä" => "&#196;",
    "ä" => "&#228;",
    "É" => "&#201;",
    "é" => "&#233;",
    "Ö" => "&#214;",
    "ö" => "&#246;",
    "Ü" => "&#220;",
    "ü" => "&#252;",
    "ß" => "&#223;",
    "«" => "&#171;",
    "»" => "&#187;",
    "„" => "&#132;",
    "“" => "&#147;",
    "”" => "&#148;",
    "°" => "&#176;",
    "€" => "&#128;",
    "£" => "&#163;",
    "Α" => "&#913;",
    "α" => "&#945;",
    "Β" => "&#914;",
    "β" => "&#946;",
    "Γ" => "&#915;",
    "γ" => "&#947;",
    "Δ" => "&#916;",
    "δ" => "&#948;",
    "Ε" => "&#917;",
    "ε" => "&#949;",
    "Ζ" => "&#918;",
    "ζ" => "&#950;",
    "Η" => "&#919;",
    "η" => "&#951;",
    "Θ" => "&#920;",
    "θ" => "&#952;",
    "Ι" => "&#921;",
    "ι" => "&#953;",
    "Κ" => "&#922;",
    "κ" => "&#954;",
    "Λ" => "&#923;",
    "λ" => "&#955;",
    "Μ" => "&#924;",
    "μ" => "&#956;",
    "Ν" => "&#925;",
    "ν" => "&#957;",
    "Ξ" => "&#926;",
    "ξ" => "&#958;",
    "Ο" => "&#927;",
    "ο" => "&#959;",
    "Π" => "&#928;",
    "π" => "&#960;",
    "Ρ" => "&#929;",
    "ρ" => "&#961;",
    "Σ" => "&#931;",
    "σ" => "&#963;",
    "Σ" => "&#931;",
    "σ" => "&#963;",
    "ς" => "&#962;",
    "Τ" => "&#932;",
    "τ" => "&#964;",
    "Υ" => "&#933;",
    "υ" => "&#965;",
    "Φ" => "&#934;",
    "φ" => "&#966;",
    "Χ" => "&#935;",
    "χ" => "&#967;",
    "Ψ" => "&#936;",
    "ψ" => "&#968;",
    "Ω" => "&#937;",
    "ω" => "&#969;",
    "Ä" => "&#196;",
    "ä" => "&#228;",
    "Ē" => "&#274;",
    "ē" => "&#275;",
    "Ī" => "&#298;",
    "ī" => "&#299;",
    "Ō" => "&#332;",
    "ō" => "&#333;",
    "Ū" => "&#362;",
    "ū" => "&#363;",
    "ʻ" => "&#699;",
    "À" => "&#192;",
    "à" => "&#224;",
    "Á" => "&#193;",
    "á" => "&#225;",
    "È" => "&#200;",
    "è" => "&#232;",
    "É" => "&#201;",
    "é" => "&#233;",
    "Ì" => "&#204;",
    "ì" => "&#236;",
    "Í" => "&#205;",
    "í" => "&#237;",
    "Ò" => "&#210;",
    "ò" => "&#242;",
    "Ó" => "&#211;",
    "ó" => "&#243;",
    "Ù" => "&#217;",
    "ù" => "&#249;",
    "Ú" => "&#218;",
    "ú" => "&#250;",
    "«" => "&#171;",
    "»" => "&#187;",
    "€" => "&#128;",
    "Ă" => "&#258;",
    "ă" => "&#259;",
    "Â" => "&#194;",
    "â" => "&#226;",
    "Î" => "&#206;",
    "î" => "&#238;",
    "Ș" => "&#x218;",
    "ș" => "&#x219;",
    "Ş" => "&#350;",
    "ş" => "&#351;",
    "Ţ" => "&#354;",
    "ţ" => "&#355;",
    "А" => "&#1040;",
    "а" => "&#1072;",
    "Б" => "&#1041;",
    "б" => "&#1073;",
    "В" => "&#1042;",
    "в" => "&#1074;",
    "Г" => "&#1043;",
    "г" => "&#1075;",
    "Д" => "&#1044;",
    "д" => "&#1076;",
    "Е" => "&#1045;",
    "е" => "&#1077;",
    "Ж" => "&#1046;",
    "ж" => "&#1078;",
    "З" => "&#1047;",
    "з" => "&#1079;",
    "И" => "&#1048;",
    "и" => "&#1080;",
    "Й" => "&#1049;",
    "й" => "&#1081;",
    "К" => "&#1050;",
    "к" => "&#1082;",
    "Л" => "&#1051;",
    "л" => "&#1083;",
    "М" => "&#1052;",
    "м" => "&#1084;",
    "Н" => "&#1053;",
    "н" => "&#1085;",
    "О" => "&#1054;",
    "о" => "&#1086;",
    "П" => "&#1055;",
    "п" => "&#1087;",
    "Р" => "&#1056;",
    "р" => "&#1088;",
    "С" => "&#1057;",
    "с" => "&#1089;",
    "Т" => "&#1058;",
    "т" => "&#1090;",
    "У" => "&#1059;",
    "у" => "&#1091;",
    "Ф" => "&#1060;",
    "ф" => "&#1092;",
    "Х" => "&#1061;",
    "х" => "&#1093;",
    "Ц" => "&#1062;",
    "ц" => "&#1094;",
    "Ч" => "&#1063;",
    "ч" => "&#1095;",
    "Ш" => "&#1064;",
    "ш" => "&#1096;",
    "Щ" => "&#1065;",
    "щ" => "&#1097;",
    "Ъ" => "&#1066;",
    "ъ" => "&#1098;",
    "Ы" => "&#1067;",
    "ы" => "&#1099;",
    "Ь" => "&#1068;",
    "ь" => "&#1100;",
    "Э" => "&#1069;",
    "э" => "&#1101;",
    "Ю" => "&#1070;",
    "ю" => "&#1102;",
    "Я" => "&#1071;",
    "я" => "&#1103;",
    "Á" => "&#193;",
    "á" => "&#225;",
    "É" => "&#201;",
    "é" => "&#233;",
    "Í" => "&#205;",
    "í" => "&#237;",
    "Ñ" => "&#209;",
    "ñ" => "&#241;",
    "Ó" => "&#211;",
    "ó" => "&#243;",
    "Ú" => "&#218;",
    "ú" => "&#250;",
    "Ü" => "&#220;",
    "ü" => "&#252;",
    "«" => "&#171;",
    "»" => "&#187;",
    "¿" => "&#191;",
    "¡" => "&#161;",
    "€" => "&#128;",
    "₧" => "&#8359;",
    "İ" => "&#304;",
    "ı" => "&#305;",
    "Ö" => "&#214;",
    "ö" => "&#246;",
    "Ü" => "&#220;",
    "ü" => "&#252;",
    "Ç" => "&#199;",
    "ç" => "&#231;",
    "Ğ" => "&#286;",
    "ğ" => "&#287;",
    "Ş" => "&#350;",
    "ş" => "&#351;",
    "₤" => "&#8356;",
    "¤" => "&#164;",
    "$" => "&#36;",
    "¢" => "&#162;",
    "£" => "&#163;",
    "¥" => "&#165;",
    "₣" => "&#8355;",
    "₤" => "&#8356;",
    "₧" => "&#8359;",
    "€" => "&#128;",
    "%" => "&#37;",
    "‰" => "&#137;",
    "∞" => "&#8734;",
    "⅛" => "&#8539;",
    "¼" => "&#188;",
    "⅜" => "&#8540;",
    "½" => "&#189;",
    "⅝" => "&#8541;",
    "¾" => "&#190;",
    "⅞" => "&#8542;",
    "∫" => "&#8747;",
    "∂" => "&#8706;",
    "∆" => "&#8710;",
    "∏" => "&#8719;",
    "∑" => "&#8721;",
    "√" => "&#8730;",
    "∟" => "&#8735;",
    "∩" => "&#8745;",
    "∙" => "&#8729;",
    "ƒ" => "&#131;",
    "†" => "&#134;",
    "‡" => "&#135;",
    "™" => "&#153;",
    "›" => "&#155;",
    "¡" => "&#161;",
    "¦" => "&#166;",
    "©" => "&#169;",
    "ª" => "&#170;",
    "«" => "&#171;",
    "¬" => "&#172;",
    "®" => "&#174;",
    "°" => "&#176;",
    "²" => "&#178;",
    "³" => "&#179;",
    "µ" => "&#181;",
    "¶" => "&#182;",
    "·" => "&#183;",
    "¹" => "&#185;",
    "º" => "&#186;",
    "»" => "&#187;",
    "¿" => "&#191;",
    "ⁿ" => "&#8319;",
    "§" => "&#167;",
    "¨" => "&#191;",
    "‣" => "&#8227;",
    "‼" => "&#8252;",
    "№" => "&#8470;",
    "À" => "&#192;",
    "Á" => "&#193;",
    "Â" => "&#194;",
    "Ã" => "&#195;",
    "Ä" => "&#196;",
    "Å" => "&#197;",
    "Æ" => "&#198;",
    "Ç" => "&#199;",
    "È" => "&#200;",
    "É" => "&#201;",
    "Ê" => "&#202;",
    "Ë" => "&#203;",
    "Ì" => "&#204;",
    "Í" => "&#205;",
    "Î" => "&#206;",
    "Ï" => "&#207;",
    "Ð" => "&#208;",
    "Ñ" => "&#209;",
    "Ò" => "&#210;",
    "Ó" => "&#211;",
    "Ô" => "&#212;",
    "Õ" => "&#213;",
    "Ö" => "&#214;",
    "×" => "&#215;",
    "Ø" => "&#216;",
    "Ù" => "&#217;",
    "Ú" => "&#218;",
    "Û" => "&#219;",
    "Ü" => "&#220;",
    "Ý" => "&#221;",
    "Þ" => "&#222;",
    "ß" => "&#223;",
    "à" => "&#224;",
    "á" => "&#225;",
    "â" => "&#226;",
    "ã" => "&#227;",
    "ä" => "&#228;",
    "å" => "&#229;",
    "æ" => "&#230;",
    "ç" => "&#231;",
    "è" => "&#232;",
    "é" => "&#233;",
    "ê" => "&#234;",
    "ë" => "&#235;",
    "ì" => "&#236;",
    "í" => "&#237;",
    "î" => "&#238;",
    "ï" => "&#239;",
    "ð" => "&#240;",
    "ñ" => "&#241;",
    "ò" => "&#242;",
    "ó" => "&#243;",
    "ô" => "&#244;",
    "õ" => "&#245;",
    "ö" => "&#246;",
    "÷" => "&#247;",
    "ø" => "&#248;",
    "ù" => "&#249;",
    "ú" => "&#250;",
    "û" => "&#251;",
    "ü" => "&#252;",
    "ý" => "&#253;",
    "þ" => "&#254;",
    "ÿ" => "&#255;",
    " " => " ",
    "丁" => "&#x4E01;",
    "丂" => "&#x4E02;",
    "七" => "&#x4E03;",
    "丄" => "&#x4E04;",
    "丅" => "&#x4E05;",
    "丆" => "&#x4E06;",
    "万" => "&#x4E07;",
    "丈" => "&#x4E08;",
    "三" => "&#x4E09;",
    "上" => "&#x4E0A;",
    "下" => "&#x4E0B;",
    "丌" => "&#x4E0C;",
    "不" => "&#x4E0D;",
    "与" => "&#x4E0E;",
    "丏" => "&#x4E0F;",
    "丐" => "&#x4E10;",
    "丑" => "&#x4E11;",
    "丒" => "&#x4E12;",
    "专" => "&#x4E13;",
    "且" => "&#x4E14;",
    "丕" => "&#x4E15;",
    "世" => "&#x4E16;",
    "丗" => "&#x4E17;",
    "丘" => "&#x4E18;",
    "丙" => "&#x4E19;",
    "业" => "&#x4E1A;",
    "丛" => "&#x4E1B;",
    "东" => "&#x4E1C;",
    "丝" => "&#x4E1D;",
    "丞" => "&#x4E1E;",
    "丟" => "&#x4E1F;",
    "丠" => "&#x4E20;",
    "両" => "&#x4E21;",
    "丢" => "&#x4E22;",
    "丣" => "&#x4E23;",
    "两" => "&#x4E24;",
    "严" => "&#x4E25;",
    "並" => "&#x4E26;",
    "丧" => "&#x4E27;",
    "丨" => "&#x4E28;",
    "丩" => "&#x4E29;",
    "个" => "&#x4E2A;",
    "丫" => "&#x4E2B;",
    "丬" => "&#x4E2C;",
    "中" => "&#x4E2D;",
    "丮" => "&#x4E2E;",
    "丯" => "&#x4E2F;",
    "丰" => "&#x4E30;",
    "丱" => "&#x4E31;",
    "串" => "&#x4E32;",
    "丳" => "&#x4E33;",
    "临" => "&#x4E34;",
    "丵" => "&#x4E35;",
    "丶" => "&#x4E36;",
    "丷" => "&#x4E37;",
    "丸" => "&#x4E38;",
    "丹" => "&#x4E39;",
    "为" => "&#x4E3A;",
    "主" => "&#x4E3B;",
    "丼" => "&#x4E3C;",
    "丽" => "&#x4E3D;",
    "举" => "&#x4E3E;",
    "丿" => "&#x4E3F;",
    "乀" => "&#x4E40;",
    "乁" => "&#x4E41;",
    "乂" => "&#x4E42;",
    "乃" => "&#x4E43;",
    "乄" => "&#x4E44;",
    "久" => "&#x4E45;",
    "乆" => "&#x4E46;",
    "乇" => "&#x4E47;",
    "么" => "&#x4E48;",
    "义" => "&#x4E49;",
    "乊" => "&#x4E4A;",
    "之" => "&#x4E4B;",
    "乌" => "&#x4E4C;",
    "乍" => "&#x4E4D;",
    "乎" => "&#x4E4E;",
    "乏" => "&#x4E4F;",
    "乐" => "&#x4E50;",
    "乑" => "&#x4E51;",
    "乒" => "&#x4E52;",
    "乓" => "&#x4E53;",
    "乔" => "&#x4E54;",
    "乕" => "&#x4E55;",
    "乖" => "&#x4E56;",
    "乗" => "&#x4E57;",
    "乘" => "&#x4E58;",
    "乙" => "&#x4E59;",
    "乚" => "&#x4E5A;",
    "乛" => "&#x4E5B;",
    "乜" => "&#x4E5C;",
    "九" => "&#x4E5D;",
    "乞" => "&#x4E5E;",
    "也" => "&#x4E5F;",
    "习" => "&#x4E60;",
    "乡" => "&#x4E61;",
    "乢" => "&#x4E62;",
    "乣" => "&#x4E63;",
    "乤" => "&#x4E64;",
    "乥" => "&#x4E65;",
    "书" => "&#x4E66;",
    "乧" => "&#x4E67;",
    "乨" => "&#x4E68;",
    "乩" => "&#x4E69;",
    "乪" => "&#x4E6A;",
    "乫" => "&#x4E6B;",
    "乬" => "&#x4E6C;",
    "乭" => "&#x4E6D;",
    "乮" => "&#x4E6E;",
    "乯" => "&#x4E6F;",
    "买" => "&#x4E70;",
    "乱" => "&#x4E71;",
    "乲" => "&#x4E72;",
    "乳" => "&#x4E73;",
    "乴" => "&#x4E74;",
    "乵" => "&#x4E75;",
    "乶" => "&#x4E76;",
    "乷" => "&#x4E77;",
    "乸" => "&#x4E78;",
    "乹" => "&#x4E79;",
    "乺" => "&#x4E7A;",
    "乻" => "&#x4E7B;",
    "乼" => "&#x4E7C;",
    "乽" => "&#x4E7D;",
    "乾" => "&#x4E7E;",
    "乿" => "&#x4E7F;",
    "亀" => "&#x4E80;",
    "亁" => "&#x4E81;",
    "亂" => "&#x4E82;",
    "亃" => "&#x4E83;",
    "亄" => "&#x4E84;",
    "亅" => "&#x4E85;",
    "了" => "&#x4E86;",
    "亇" => "&#x4E87;",
    "予" => "&#x4E88;",
    "争" => "&#x4E89;",
    "亊" => "&#x4E8A;",
    "事" => "&#x4E8B;",
    "二" => "&#x4E8C;",
    "亍" => "&#x4E8D;",
    "于" => "&#x4E8E;",
    "亏" => "&#x4E8F;",
    "亐" => "&#x4E90;",
    "云" => "&#x4E91;",
    "互" => "&#x4E92;",
    "亓" => "&#x4E93;",
    "五" => "&#x4E94;",
    "井" => "&#x4E95;",
    "亖" => "&#x4E96;",
    "亗" => "&#x4E97;",
    "亘" => "&#x4E98;",
    "亙" => "&#x4E99;",
    "亚" => "&#x4E9A;",
    "些" => "&#x4E9B;",
    "亜" => "&#x4E9C;",
    "亝" => "&#x4E9D;",
    "亞" => "&#x4E9E;",
    "亟" => "&#x4E9F;",
    "亠" => "&#x4EA0;",
    "亡" => "&#x4EA1;",
    "亢" => "&#x4EA2;",
    "亣" => "&#x4EA3;",
    "交" => "&#x4EA4;",
    "亥" => "&#x4EA5;",
    "亦" => "&#x4EA6;",
    "产" => "&#x4EA7;",
    "亨" => "&#x4EA8;",
    "亩" => "&#x4EA9;",
    "亪" => "&#x4EAA;",
    "享" => "&#x4EAB;",
    "京" => "&#x4EAC;",
    "亭" => "&#x4EAD;",
    "亮" => "&#x4EAE;",
    "亯" => "&#x4EAF;",
    "亰" => "&#x4EB0;",
    "亱" => "&#x4EB1;",
    "亲" => "&#x4EB2;",
    "亳" => "&#x4EB3;",
    "亴" => "&#x4EB4;",
    "亵" => "&#x4EB5;",
    "亶" => "&#x4EB6;",
    "亷" => "&#x4EB7;",
    "亸" => "&#x4EB8;",
    "亹" => "&#x4EB9;",
    "人" => "&#x4EBA;",
    "亻" => "&#x4EBB;",
    "亼" => "&#x4EBC;",
    "亽" => "&#x4EBD;",
    "亾" => "&#x4EBE;",
    "亿" => "&#x4EBF;",
    "什" => "&#x4EC0;",
    "仁" => "&#x4EC1;",
    "仂" => "&#x4EC2;",
    "仃" => "&#x4EC3;",
    "仄" => "&#x4EC4;",
    "仅" => "&#x4EC5;",
    "仆" => "&#x4EC6;",
    "仇" => "&#x4EC7;",
    "仈" => "&#x4EC8;",
    "仉" => "&#x4EC9;",
    "今" => "&#x4ECA;",
    "介" => "&#x4ECB;",
    "仌" => "&#x4ECC;",
    "仍" => "&#x4ECD;",
    "从" => "&#x4ECE;",
    "仏" => "&#x4ECF;",
    "仐" => "&#x4ED0;",
    "仑" => "&#x4ED1;",
    "仒" => "&#x4ED2;",
    "仓" => "&#x4ED3;",
    "仔" => "&#x4ED4;",
    "仕" => "&#x4ED5;",
    "他" => "&#x4ED6;",
    "仗" => "&#x4ED7;",
    "付" => "&#x4ED8;",
    "仙" => "&#x4ED9;",
    "仚" => "&#x4EDA;",
    "仛" => "&#x4EDB;",
    "仜" => "&#x4EDC;",
    "仝" => "&#x4EDD;",
    "仞" => "&#x4EDE;",
    "仟" => "&#x4EDF;",
    "仠" => "&#x4EE0;",
    "仡" => "&#x4EE1;",
    "仢" => "&#x4EE2;",
    "代" => "&#x4EE3;",
    "令" => "&#x4EE4;",
    "以" => "&#x4EE5;",
    "仦" => "&#x4EE6;",
    "仧" => "&#x4EE7;",
    "仨" => "&#x4EE8;",
    "仩" => "&#x4EE9;",
    "仪" => "&#x4EEA;",
    "仫" => "&#x4EEB;",
    "们" => "&#x4EEC;",
    "仭" => "&#x4EED;",
    "仮" => "&#x4EEE;",
    "仯" => "&#x4EEF;",
    "仰" => "&#x4EF0;",
    "仱" => "&#x4EF1;",
    "仲" => "&#x4EF2;",
    "仳" => "&#x4EF3;",
    "仴" => "&#x4EF4;",
    "仵" => "&#x4EF5;",
    "件" => "&#x4EF6;",
    "价" => "&#x4EF7;",
    "仸" => "&#x4EF8;",
    "仹" => "&#x4EF9;",
    "仺" => "&#x4EFA;",
    "任" => "&#x4EFB;",
    "仼" => "&#x4EFC;",
    "份" => "&#x4EFD;",
    "仾" => "&#x4EFE;",
    "仿" => "&#x4EFF;",
    "伀" => "&#x4F00;",
    "企" => "&#x4F01;",
    "伂" => "&#x4F02;",
    "伃" => "&#x4F03;",
    "伄" => "&#x4F04;",
    "伅" => "&#x4F05;",
    "伆" => "&#x4F06;",
    "伇" => "&#x4F07;",
    "伈" => "&#x4F08;",
    "伉" => "&#x4F09;",
    "伊" => "&#x4F0A;",
    "伋" => "&#x4F0B;",
    "伌" => "&#x4F0C;",
    "伍" => "&#x4F0D;",
    "伎" => "&#x4F0E;",
    "伏" => "&#x4F0F;",
    "伐" => "&#x4F10;",
    "休" => "&#x4F11;",
    "伒" => "&#x4F12;",
    "伓" => "&#x4F13;",
    "伔" => "&#x4F14;",
    "伕" => "&#x4F15;",
    "伖" => "&#x4F16;",
    "众" => "&#x4F17;",
    "优" => "&#x4F18;",
    "伙" => "&#x4F19;",
    "会" => "&#x4F1A;",
    "伛" => "&#x4F1B;",
    "伜" => "&#x4F1C;",
    "伝" => "&#x4F1D;",
    "伞" => "&#x4F1E;",
    "伟" => "&#x4F1F;",
    "传" => "&#x4F20;",
    "伡" => "&#x4F21;",
    "伢" => "&#x4F22;",
    "伣" => "&#x4F23;",
    "伤" => "&#x4F24;",
    "伥" => "&#x4F25;",
    "伦" => "&#x4F26;",
    "伧" => "&#x4F27;",
    "伨" => "&#x4F28;",
    "伩" => "&#x4F29;",
    "伪" => "&#x4F2A;",
    "伫" => "&#x4F2B;",
    "伬" => "&#x4F2C;",
    "伭" => "&#x4F2D;",
    "伮" => "&#x4F2E;",
    "伯" => "&#x4F2F;",
    "估" => "&#x4F30;",
    "伱" => "&#x4F31;",
    "伲" => "&#x4F32;",
    "伳" => "&#x4F33;",
    "伴" => "&#x4F34;",
    "伵" => "&#x4F35;",
    "伶" => "&#x4F36;",
    "伷" => "&#x4F37;",
    "伸" => "&#x4F38;",
    "伹" => "&#x4F39;",
    "伺" => "&#x4F3A;",
    "伻" => "&#x4F3B;",
    "似" => "&#x4F3C;",
    "伽" => "&#x4F3D;",
    "伾" => "&#x4F3E;",
    "伿" => "&#x4F3F;",
    "佀" => "&#x4F40;",
    "佁" => "&#x4F41;",
    "佂" => "&#x4F42;",
    "佃" => "&#x4F43;",
    "佄" => "&#x4F44;",
    "佅" => "&#x4F45;",
    "但" => "&#x4F46;",
    "佇" => "&#x4F47;",
    "佈" => "&#x4F48;",
    "佉" => "&#x4F49;",
    "佊" => "&#x4F4A;",
    "佋" => "&#x4F4B;",
    "佌" => "&#x4F4C;",
    "位" => "&#x4F4D;",
    "低" => "&#x4F4E;",
    "住" => "&#x4F4F;",
    "佐" => "&#x4F50;",
    "佑" => "&#x4F51;",
    "佒" => "&#x4F52;",
    "体" => "&#x4F53;",
    "佔" => "&#x4F54;",
    "何" => "&#x4F55;",
    "佖" => "&#x4F56;",
    "佗" => "&#x4F57;",
    "佘" => "&#x4F58;",
    "余" => "&#x4F59;",
    "佚" => "&#x4F5A;",
    "佛" => "&#x4F5B;",
    "作" => "&#x4F5C;",
    "佝" => "&#x4F5D;",
    "佞" => "&#x4F5E;",
    "佟" => "&#x4F5F;",
    "你" => "&#x4F60;",
    "佡" => "&#x4F61;",
    "佢" => "&#x4F62;",
    "佣" => "&#x4F63;",
    "佤" => "&#x4F64;",
    "佥" => "&#x4F65;",
    "佦" => "&#x4F66;",
    "佧" => "&#x4F67;",
    "佨" => "&#x4F68;",
    "佩" => "&#x4F69;",
    "佪" => "&#x4F6A;",
    "佫" => "&#x4F6B;",
    "佬" => "&#x4F6C;",
    "佭" => "&#x4F6D;",
    "佮" => "&#x4F6E;",
    "佯" => "&#x4F6F;",
    "佰" => "&#x4F70;",
    "佱" => "&#x4F71;",
    "佲" => "&#x4F72;",
    "佳" => "&#x4F73;",
    "佴" => "&#x4F74;",
    "併" => "&#x4F75;",
    "佶" => "&#x4F76;",
    "佷" => "&#x4F77;",
    "佸" => "&#x4F78;",
    "佹" => "&#x4F79;",
    "佺" => "&#x4F7A;",
    "佻" => "&#x4F7B;",
    "佼" => "&#x4F7C;",
    "佽" => "&#x4F7D;",
    "佾" => "&#x4F7E;",
    "使" => "&#x4F7F;",
    "侀" => "&#x4F80;",
    "侁" => "&#x4F81;",
    "侂" => "&#x4F82;",
    "侃" => "&#x4F83;",
    "侄" => "&#x4F84;",
    "侅" => "&#x4F85;",
    "來" => "&#x4F86;",
    "侇" => "&#x4F87;",
    "侈" => "&#x4F88;",
    "侉" => "&#x4F89;",
    "侊" => "&#x4F8A;",
    "例" => "&#x4F8B;",
    "侌" => "&#x4F8C;",
    "侍" => "&#x4F8D;",
    "侎" => "&#x4F8E;",
    "侏" => "&#x4F8F;",
    "侐" => "&#x4F90;",
    "侑" => "&#x4F91;",
    "侒" => "&#x4F92;",
    "侓" => "&#x4F93;",
    "侔" => "&#x4F94;",
    "侕" => "&#x4F95;",
    "侖" => "&#x4F96;",
    "侗" => "&#x4F97;",
    "侘" => "&#x4F98;",
    "侙" => "&#x4F99;",
    "侚" => "&#x4F9A;",
    "供" => "&#x4F9B;",
    "侜" => "&#x4F9C;",
    "依" => "&#x4F9D;",
    "侞" => "&#x4F9E;",
    "侟" => "&#x4F9F;",
    "侠" => "&#x4FA0;",
    "価" => "&#x4FA1;",
    "侢" => "&#x4FA2;",
    "侣" => "&#x4FA3;",
    "侤" => "&#x4FA4;",
    "侥" => "&#x4FA5;",
    "侦" => "&#x4FA6;",
    "侧" => "&#x4FA7;",
    "侨" => "&#x4FA8;",
    "侩" => "&#x4FA9;",
    "侪" => "&#x4FAA;",
    "侫" => "&#x4FAB;",
    "侬" => "&#x4FAC;",
    "侭" => "&#x4FAD;",
    "侮" => "&#x4FAE;",
    "侯" => "&#x4FAF;",
    "侰" => "&#x4FB0;",
    "侱" => "&#x4FB1;",
    "侲" => "&#x4FB2;",
    "侳" => "&#x4FB3;",
    "侴" => "&#x4FB4;",
    "侵" => "&#x4FB5;",
    "侶" => "&#x4FB6;",
    "侷" => "&#x4FB7;",
    "侸" => "&#x4FB8;",
    "侹" => "&#x4FB9;",
    "侺" => "&#x4FBA;",
    "侻" => "&#x4FBB;",
    "侼" => "&#x4FBC;",
    "侽" => "&#x4FBD;",
    "侾" => "&#x4FBE;",
    "便" => "&#x4FBF;",
    "俀" => "&#x4FC0;",
    "俁" => "&#x4FC1;",
    "係" => "&#x4FC2;",
    "促" => "&#x4FC3;",
    "俄" => "&#x4FC4;",
    "俅" => "&#x4FC5;",
    "俆" => "&#x4FC6;",
    "俇" => "&#x4FC7;",
    "俈" => "&#x4FC8;",
    "俉" => "&#x4FC9;",
    "俊" => "&#x4FCA;",
    "俋" => "&#x4FCB;",
    "俌" => "&#x4FCC;",
    "俍" => "&#x4FCD;",
    "俎" => "&#x4FCE;",
    "俏" => "&#x4FCF;",
    "俐" => "&#x4FD0;",
    "俑" => "&#x4FD1;",
    "俒" => "&#x4FD2;",
    "俓" => "&#x4FD3;",
    "俔" => "&#x4FD4;",
    "俕" => "&#x4FD5;",
    "俖" => "&#x4FD6;",
    "俗" => "&#x4FD7;",
    "俘" => "&#x4FD8;",
    "俙" => "&#x4FD9;",
    "俚" => "&#x4FDA;",
    "俛" => "&#x4FDB;",
    "俜" => "&#x4FDC;",
    "保" => "&#x4FDD;",
    "俞" => "&#x4FDE;",
    "俟" => "&#x4FDF;",
    "俠" => "&#x4FE0;",
    "信" => "&#x4FE1;",
    "俢" => "&#x4FE2;",
    "俣" => "&#x4FE3;",
    "俤" => "&#x4FE4;",
    "俥" => "&#x4FE5;",
    "俦" => "&#x4FE6;",
    "俧" => "&#x4FE7;",
    "俨" => "&#x4FE8;",
    "俩" => "&#x4FE9;",
    "俪" => "&#x4FEA;",
    "俫" => "&#x4FEB;",
    "俬" => "&#x4FEC;",
    "俭" => "&#x4FED;",
    "修" => "&#x4FEE;",
    "俯" => "&#x4FEF;",
    "俰" => "&#x4FF0;",
    "俱" => "&#x4FF1;",
    "俲" => "&#x4FF2;",
    "俳" => "&#x4FF3;",
    "俴" => "&#x4FF4;",
    "俵" => "&#x4FF5;",
    "俶" => "&#x4FF6;",
    "俷" => "&#x4FF7;",
    "俸" => "&#x4FF8;",
    "俹" => "&#x4FF9;",
    "俺" => "&#x4FFA;",
    "俻" => "&#x4FFB;",
    "俼" => "&#x4FFC;",
    "俽" => "&#x4FFD;",
    "俾" => "&#x4FFE;",
    "俿" => "&#x4FFF;",
    "倀" => "&#x5000;",
    "倁" => "&#x5001;",
    "倂" => "&#x5002;",
    "倃" => "&#x5003;",
    "倄" => "&#x5004;",
    "倅" => "&#x5005;",
    "倆" => "&#x5006;",
    "倇" => "&#x5007;",
    "倈" => "&#x5008;",
    "倉" => "&#x5009;",
    "倊" => "&#x500A;",
    "個" => "&#x500B;",
    "倌" => "&#x500C;",
    "倍" => "&#x500D;",
    "倎" => "&#x500E;",
    "倏" => "&#x500F;",
    "倐" => "&#x5010;",
    "們" => "&#x5011;",
    "倒" => "&#x5012;",
    "倓" => "&#x5013;",
    "倔" => "&#x5014;",
    "倕" => "&#x5015;",
    "倖" => "&#x5016;",
    "倗" => "&#x5017;",
    "倘" => "&#x5018;",
    "候" => "&#x5019;",
    "倚" => "&#x501A;",
    "倛" => "&#x501B;",
    "倜" => "&#x501C;",
    "倝" => "&#x501D;",
    "倞" => "&#x501E;",
    "借" => "&#x501F;",
    "倠" => "&#x5020;",
    "倡" => "&#x5021;",
    "倢" => "&#x5022;",
    "倣" => "&#x5023;",
    "値" => "&#x5024;",
    "倥" => "&#x5025;",
    "倦" => "&#x5026;",
    "倧" => "&#x5027;",
    "倨" => "&#x5028;",
    "倩" => "&#x5029;",
    "倪" => "&#x502A;",
    "倫" => "&#x502B;",
    "倬" => "&#x502C;",
    "倭" => "&#x502D;",
    "倮" => "&#x502E;",
    "倯" => "&#x502F;",
    "倰" => "&#x5030;",
    "倱" => "&#x5031;",
    "倲" => "&#x5032;",
    "倳" => "&#x5033;",
    "倴" => "&#x5034;",
    "倵" => "&#x5035;",
    "倶" => "&#x5036;",
    "倷" => "&#x5037;",
    "倸" => "&#x5038;",
    "倹" => "&#x5039;",
    "债" => "&#x503A;",
    "倻" => "&#x503B;",
    "值" => "&#x503C;",
    "倽" => "&#x503D;",
    "倾" => "&#x503E;",
    "倿" => "&#x503F;",
    "偀" => "&#x5040;",
    "偁" => "&#x5041;",
    "偂" => "&#x5042;",
    "偃" => "&#x5043;",
    "偄" => "&#x5044;",
    "偅" => "&#x5045;",
    "偆" => "&#x5046;",
    "假" => "&#x5047;",
    "偈" => "&#x5048;",
    "偉" => "&#x5049;",
    "偊" => "&#x504A;",
    "偋" => "&#x504B;",
    "偌" => "&#x504C;",
    "偍" => "&#x504D;",
    "偎" => "&#x504E;",
    "偏" => "&#x504F;",
    "偐" => "&#x5050;",
    "偑" => "&#x5051;",
    "偒" => "&#x5052;",
    "偓" => "&#x5053;",
    "偔" => "&#x5054;",
    "偕" => "&#x5055;",
    "偖" => "&#x5056;",
    "偗" => "&#x5057;",
    "偘" => "&#x5058;",
    "偙" => "&#x5059;",
    "做" => "&#x505A;",
    "偛" => "&#x505B;",
    "停" => "&#x505C;",
    "偝" => "&#x505D;",
    "偞" => "&#x505E;",
    "偟" => "&#x505F;",
    "偠" => "&#x5060;",
    "偡" => "&#x5061;",
    "偢" => "&#x5062;",
    "偣" => "&#x5063;",
    "偤" => "&#x5064;",
    "健" => "&#x5065;",
    "偦" => "&#x5066;",
    "偧" => "&#x5067;",
    "偨" => "&#x5068;",
    "偩" => "&#x5069;",
    "偪" => "&#x506A;",
    "偫" => "&#x506B;",
    "偬" => "&#x506C;",
    "偭" => "&#x506D;",
    "偮" => "&#x506E;",
    "偯" => "&#x506F;",
    "偰" => "&#x5070;",
    "偱" => "&#x5071;",
    "偲" => "&#x5072;",
    "偳" => "&#x5073;",
    "側" => "&#x5074;",
    "偵" => "&#x5075;",
    "偶" => "&#x5076;",
    "偷" => "&#x5077;",
    "偸" => "&#x5078;",
    "偹" => "&#x5079;",
    "偺" => "&#x507A;",
    "偻" => "&#x507B;",
    "偼" => "&#x507C;",
    "偽" => "&#x507D;",
    "偾" => "&#x507E;",
    "偿" => "&#x507F;",
    "傀" => "&#x5080;",
    "傁" => "&#x5081;",
    "傂" => "&#x5082;",
    "傃" => "&#x5083;",
    "傄" => "&#x5084;",
    "傅" => "&#x5085;",
    "傆" => "&#x5086;",
    "傇" => "&#x5087;",
    "傈" => "&#x5088;",
    "傉" => "&#x5089;",
    "傊" => "&#x508A;",
    "傋" => "&#x508B;",
    "傌" => "&#x508C;",
    "傍" => "&#x508D;",
    "傎" => "&#x508E;",
    "傏" => "&#x508F;",
    "傐" => "&#x5090;",
    "傑" => "&#x5091;",
    "傒" => "&#x5092;",
    "傓" => "&#x5093;",
    "傔" => "&#x5094;",
    "傕" => "&#x5095;",
    "傖" => "&#x5096;",
    "傗" => "&#x5097;",
    "傘" => "&#x5098;",
    "備" => "&#x5099;",
    "傚" => "&#x509A;",
    "傛" => "&#x509B;",
    "傜" => "&#x509C;",
    "傝" => "&#x509D;",
    "傞" => "&#x509E;",
    "傟" => "&#x509F;",
    "傠" => "&#x50A0;",
    "傡" => "&#x50A1;",
    "傢" => "&#x50A2;",
    "傣" => "&#x50A3;",
    "傤" => "&#x50A4;",
    "傥" => "&#x50A5;",
    "傦" => "&#x50A6;",
    "傧" => "&#x50A7;",
    "储" => "&#x50A8;",
    "傩" => "&#x50A9;",
    "傪" => "&#x50AA;",
    "傫" => "&#x50AB;",
    "催" => "&#x50AC;",
    "傭" => "&#x50AD;",
    "傮" => "&#x50AE;",
    "傯" => "&#x50AF;",
    "傰" => "&#x50B0;",
    "傱" => "&#x50B1;",
    "傲" => "&#x50B2;",
    "傳" => "&#x50B3;",
    "傴" => "&#x50B4;",
    "債" => "&#x50B5;",
    "傶" => "&#x50B6;",
    "傷" => "&#x50B7;",
    "傸" => "&#x50B8;",
    "傹" => "&#x50B9;",
    "傺" => "&#x50BA;",
    "傻" => "&#x50BB;",
    "傼" => "&#x50BC;",
    "傽" => "&#x50BD;",
    "傾" => "&#x50BE;",
    "傿" => "&#x50BF;",
    "僀" => "&#x50C0;",
    "僁" => "&#x50C1;",
    "僂" => "&#x50C2;",
    "僃" => "&#x50C3;",
    "僄" => "&#x50C4;",
    "僅" => "&#x50C5;",
    "僆" => "&#x50C6;",
    "僇" => "&#x50C7;",
    "僈" => "&#x50C8;",
    "僉" => "&#x50C9;",
    "僊" => "&#x50CA;",
    "僋" => "&#x50CB;",
    "僌" => "&#x50CC;",
    "働" => "&#x50CD;",
    "僎" => "&#x50CE;",
    "像" => "&#x50CF;",
    "僐" => "&#x50D0;",
    "僑" => "&#x50D1;",
    "僒" => "&#x50D2;",
    "僓" => "&#x50D3;",
    "僔" => "&#x50D4;",
    "僕" => "&#x50D5;",
    "僖" => "&#x50D6;",
    "僗" => "&#x50D7;",
    "僘" => "&#x50D8;",
    "僙" => "&#x50D9;",
    "僚" => "&#x50DA;",
    "僛" => "&#x50DB;",
    "僜" => "&#x50DC;",
    "僝" => "&#x50DD;",
    "僞" => "&#x50DE;",
    "僟" => "&#x50DF;",
    "僠" => "&#x50E0;",
    "僡" => "&#x50E1;",
    "僢" => "&#x50E2;",
    "僣" => "&#x50E3;",
    "僤" => "&#x50E4;",
    "僥" => "&#x50E5;",
    "僦" => "&#x50E6;",
    "僧" => "&#x50E7;",
    "僨" => "&#x50E8;",
    "僩" => "&#x50E9;",
    "僪" => "&#x50EA;",
    "僫" => "&#x50EB;",
    "僬" => "&#x50EC;",
    "僭" => "&#x50ED;",
    "僮" => "&#x50EE;",
    "僯" => "&#x50EF;",
    "僰" => "&#x50F0;",
    "僱" => "&#x50F1;",
    "僲" => "&#x50F2;",
    "僳" => "&#x50F3;",
    "僴" => "&#x50F4;",
    "僵" => "&#x50F5;",
    "僶" => "&#x50F6;",
    "僷" => "&#x50F7;",
    "僸" => "&#x50F8;",
    "價" => "&#x50F9;",
    "僺" => "&#x50FA;",
    "僻" => "&#x50FB;",
    "僼" => "&#x50FC;",
    "僽" => "&#x50FD;",
    "僾" => "&#x50FE;",
    "僿" => "&#x50FF;",
    "儀" => "&#x5100;",
    "儁" => "&#x5101;",
    "儂" => "&#x5102;",
    "儃" => "&#x5103;",
    "億" => "&#x5104;",
    "儅" => "&#x5105;",
    "儆" => "&#x5106;",
    "儇" => "&#x5107;",
    "儈" => "&#x5108;",
    "儉" => "&#x5109;",
    "儊" => "&#x510A;",
    "儋" => "&#x510B;",
    "儌" => "&#x510C;",
    "儍" => "&#x510D;",
    "儎" => "&#x510E;",
    "儏" => "&#x510F;",
    "儐" => "&#x5110;",
    "儑" => "&#x5111;",
    "儒" => "&#x5112;",
    "儓" => "&#x5113;",
    "儔" => "&#x5114;",
    "儕" => "&#x5115;",
    "儖" => "&#x5116;",
    "儗" => "&#x5117;",
    "儘" => "&#x5118;",
    "儙" => "&#x5119;",
    "儚" => "&#x511A;",
    "儛" => "&#x511B;",
    "儜" => "&#x511C;",
    "儝" => "&#x511D;",
    "儞" => "&#x511E;",
    "償" => "&#x511F;",
    "儠" => "&#x5120;",
    "儡" => "&#x5121;",
    "儢" => "&#x5122;",
    "儣" => "&#x5123;",
    "儤" => "&#x5124;",
    "儥" => "&#x5125;",
    "儦" => "&#x5126;",
    "儧" => "&#x5127;",
    "儨" => "&#x5128;",
    "儩" => "&#x5129;",
    "優" => "&#x512A;",
    "儫" => "&#x512B;",
    "儬" => "&#x512C;",
    "儭" => "&#x512D;",
    "儮" => "&#x512E;",
    "儯" => "&#x512F;",
    "儰" => "&#x5130;",
    "儱" => "&#x5131;",
    "儲" => "&#x5132;",
    "儳" => "&#x5133;",
    "儴" => "&#x5134;",
    "儵" => "&#x5135;",
    "儶" => "&#x5136;",
    "儷" => "&#x5137;",
    "儸" => "&#x5138;",
    "儹" => "&#x5139;",
    "儺" => "&#x513A;",
    "儻" => "&#x513B;",
    "儼" => "&#x513C;",
    "儽" => "&#x513D;",
    "儾" => "&#x513E;",
    "儿" => "&#x513F;",
    "兀" => "&#x5140;",
    "允" => "&#x5141;",
    "兂" => "&#x5142;",
    "元" => "&#x5143;",
    "兄" => "&#x5144;",
    "充" => "&#x5145;",
    "兆" => "&#x5146;",
    "兇" => "&#x5147;",
    "先" => "&#x5148;",
    "光" => "&#x5149;",
    "兊" => "&#x514A;",
    "克" => "&#x514B;",
    "兌" => "&#x514C;",
    "免" => "&#x514D;",
    "兎" => "&#x514E;",
    "兏" => "&#x514F;",
    "児" => "&#x5150;",
    "兑" => "&#x5151;",
    "兒" => "&#x5152;",
    "兓" => "&#x5153;",
    "兔" => "&#x5154;",
    "兕" => "&#x5155;",
    "兖" => "&#x5156;",
    "兗" => "&#x5157;",
    "兘" => "&#x5158;",
    "兙" => "&#x5159;",
    "党" => "&#x515A;",
    "兛" => "&#x515B;",
    "兜" => "&#x515C;",
    "兝" => "&#x515D;",
    "兞" => "&#x515E;",
    "兟" => "&#x515F;",
    "兠" => "&#x5160;",
    "兡" => "&#x5161;",
    "兢" => "&#x5162;",
    "兣" => "&#x5163;",
    "兤" => "&#x5164;",
    "入" => "&#x5165;",
    "兦" => "&#x5166;",
    "內" => "&#x5167;",
    "全" => "&#x5168;",
    "兩" => "&#x5169;",
    "兪" => "&#x516A;",
    "八" => "&#x516B;",
    "公" => "&#x516C;",
    "六" => "&#x516D;",
    "兮" => "&#x516E;",
    "兯" => "&#x516F;",
    "兰" => "&#x5170;",
    "共" => "&#x5171;",
    "兲" => "&#x5172;",
    "关" => "&#x5173;",
    "兴" => "&#x5174;",
    "兵" => "&#x5175;",
    "其" => "&#x5176;",
    "具" => "&#x5177;",
    "典" => "&#x5178;",
    "兹" => "&#x5179;",
    "兺" => "&#x517A;",
    "养" => "&#x517B;",
    "兼" => "&#x517C;",
    "兽" => "&#x517D;",
    "兾" => "&#x517E;",
    "兿" => "&#x517F;",
    "冀" => "&#x5180;",
    "冁" => "&#x5181;",
    "冂" => "&#x5182;",
    "冃" => "&#x5183;",
    "冄" => "&#x5184;",
    "内" => "&#x5185;",
    "円" => "&#x5186;",
    "冇" => "&#x5187;",
    "冈" => "&#x5188;",
    "冉" => "&#x5189;",
    "冊" => "&#x518A;",
    "冋" => "&#x518B;",
    "册" => "&#x518C;",
    "再" => "&#x518D;",
    "冎" => "&#x518E;",
    "冏" => "&#x518F;",
    "冐" => "&#x5190;",
    "冑" => "&#x5191;",
    "冒" => "&#x5192;",
    "冓" => "&#x5193;",
    "冔" => "&#x5194;",
    "冕" => "&#x5195;",
    "冖" => "&#x5196;",
    "冗" => "&#x5197;",
    "冘" => "&#x5198;",
    "写" => "&#x5199;",
    "冚" => "&#x519A;",
    "军" => "&#x519B;",
    "农" => "&#x519C;",
    "冝" => "&#x519D;",
    "冞" => "&#x519E;",
    "冟" => "&#x519F;",
    "冠" => "&#x51A0;",
    "冡" => "&#x51A1;",
    "冢" => "&#x51A2;",
    "冣" => "&#x51A3;",
    "冤" => "&#x51A4;",
    "冥" => "&#x51A5;",
    "冦" => "&#x51A6;",
    "冧" => "&#x51A7;",
    "冨" => "&#x51A8;",
    "冩" => "&#x51A9;",
    "冪" => "&#x51AA;",
    "冫" => "&#x51AB;",
    "冬" => "&#x51AC;",
    "冭" => "&#x51AD;",
    "冮" => "&#x51AE;",
    "冯" => "&#x51AF;",
    "冰" => "&#x51B0;",
    "冱" => "&#x51B1;",
    "冲" => "&#x51B2;",
    "决" => "&#x51B3;",
    "冴" => "&#x51B4;",
    "况" => "&#x51B5;",
    "冶" => "&#x51B6;",
    "冷" => "&#x51B7;",
    "冸" => "&#x51B8;",
    "冹" => "&#x51B9;",
    "冺" => "&#x51BA;",
    "冻" => "&#x51BB;",
    "冼" => "&#x51BC;",
    "冽" => "&#x51BD;",
    "冾" => "&#x51BE;",
    "冿" => "&#x51BF;",
    "净" => "&#x51C0;",
    "凁" => "&#x51C1;",
    "凂" => "&#x51C2;",
    "凃" => "&#x51C3;",
    "凄" => "&#x51C4;",
    "凅" => "&#x51C5;",
    "准" => "&#x51C6;",
    "凇" => "&#x51C7;",
    "凈" => "&#x51C8;",
    "凉" => "&#x51C9;",
    "凊" => "&#x51CA;",
    "凋" => "&#x51CB;",
    "凌" => "&#x51CC;",
    "凍" => "&#x51CD;",
    "凎" => "&#x51CE;",
    "减" => "&#x51CF;",
    "凐" => "&#x51D0;",
    "凑" => "&#x51D1;",
    "凒" => "&#x51D2;",
    "凓" => "&#x51D3;",
    "凔" => "&#x51D4;",
    "凕" => "&#x51D5;",
    "凖" => "&#x51D6;",
    "凗" => "&#x51D7;",
    "凘" => "&#x51D8;",
    "凙" => "&#x51D9;",
    "凚" => "&#x51DA;",
    "凛" => "&#x51DB;",
    "凜" => "&#x51DC;",
    "凝" => "&#x51DD;",
    "凞" => "&#x51DE;",
    "凟" => "&#x51DF;",
    "几" => "&#x51E0;",
    "凡" => "&#x51E1;",
    "凢" => "&#x51E2;",
    "凣" => "&#x51E3;",
    "凤" => "&#x51E4;",
    "凥" => "&#x51E5;",
    "処" => "&#x51E6;",
    "凧" => "&#x51E7;",
    "凨" => "&#x51E8;",
    "凩" => "&#x51E9;",
    "凪" => "&#x51EA;",
    "凫" => "&#x51EB;",
    "凬" => "&#x51EC;",
    "凭" => "&#x51ED;",
    "凮" => "&#x51EE;",
    "凯" => "&#x51EF;",
    "凰" => "&#x51F0;",
    "凱" => "&#x51F1;",
    "凲" => "&#x51F2;",
    "凳" => "&#x51F3;",
    "凴" => "&#x51F4;",
    "凵" => "&#x51F5;",
    "凶" => "&#x51F6;",
    "凷" => "&#x51F7;",
    "凸" => "&#x51F8;",
    "凹" => "&#x51F9;",
    "出" => "&#x51FA;",
    "击" => "&#x51FB;",
    "凼" => "&#x51FC;",
    "函" => "&#x51FD;",
    "凾" => "&#x51FE;",
    "凿" => "&#x51FF;",
    "刀" => "&#x5200;",
    "刁" => "&#x5201;",
    "刂" => "&#x5202;",
    "刃" => "&#x5203;",
    "刄" => "&#x5204;",
    "刅" => "&#x5205;",
    "分" => "&#x5206;",
    "切" => "&#x5207;",
    "刈" => "&#x5208;",
    "刉" => "&#x5209;",
    "刊" => "&#x520A;",
    "刋" => "&#x520B;",
    "刌" => "&#x520C;",
    "刍" => "&#x520D;",
    "刎" => "&#x520E;",
    "刏" => "&#x520F;",
    "刐" => "&#x5210;",
    "刑" => "&#x5211;",
    "划" => "&#x5212;",
    "刓" => "&#x5213;",
    "刔" => "&#x5214;",
    "刕" => "&#x5215;",
    "刖" => "&#x5216;",
    "列" => "&#x5217;",
    "刘" => "&#x5218;",
    "则" => "&#x5219;",
    "刚" => "&#x521A;",
    "创" => "&#x521B;",
    "刜" => "&#x521C;",
    "初" => "&#x521D;",
    "刞" => "&#x521E;",
    "刟" => "&#x521F;",
    "删" => "&#x5220;",
    "刡" => "&#x5221;",
    "刢" => "&#x5222;",
    "刣" => "&#x5223;",
    "判" => "&#x5224;",
    "別" => "&#x5225;",
    "刦" => "&#x5226;",
    "刧" => "&#x5227;",
    "刨" => "&#x5228;",
    "利" => "&#x5229;",
    "刪" => "&#x522A;",
    "别" => "&#x522B;",
    "刬" => "&#x522C;",
    "刭" => "&#x522D;",
    "刮" => "&#x522E;",
    "刯" => "&#x522F;",
    "到" => "&#x5230;",
    "刱" => "&#x5231;",
    "刲" => "&#x5232;",
    "刳" => "&#x5233;",
    "刴" => "&#x5234;",
    "刵" => "&#x5235;",
    "制" => "&#x5236;",
    "刷" => "&#x5237;",
    "券" => "&#x5238;",
    "刹" => "&#x5239;",
    "刺" => "&#x523A;",
    "刻" => "&#x523B;",
    "刼" => "&#x523C;",
    "刽" => "&#x523D;",
    "刾" => "&#x523E;",
    "刿" => "&#x523F;",
    "剀" => "&#x5240;",
    "剁" => "&#x5241;",
    "剂" => "&#x5242;",
    "剃" => "&#x5243;",
    "剄" => "&#x5244;",
    "剅" => "&#x5245;",
    "剆" => "&#x5246;",
    "則" => "&#x5247;",
    "剈" => "&#x5248;",
    "剉" => "&#x5249;",
    "削" => "&#x524A;",
    "剋" => "&#x524B;",
    "剌" => "&#x524C;",
    "前" => "&#x524D;",
    "剎" => "&#x524E;",
    "剏" => "&#x524F;",
    "剐" => "&#x5250;",
    "剑" => "&#x5251;",
    "剒" => "&#x5252;",
    "剓" => "&#x5253;",
    "剔" => "&#x5254;",
    "剕" => "&#x5255;",
    "剖" => "&#x5256;",
    "剗" => "&#x5257;",
    "剘" => "&#x5258;",
    "剙" => "&#x5259;",
    "剚" => "&#x525A;",
    "剛" => "&#x525B;",
    "剜" => "&#x525C;",
    "剝" => "&#x525D;",
    "剞" => "&#x525E;",
    "剟" => "&#x525F;",
    "剠" => "&#x5260;",
    "剡" => "&#x5261;",
    "剢" => "&#x5262;",
    "剣" => "&#x5263;",
    "剤" => "&#x5264;",
    "剥" => "&#x5265;",
    "剦" => "&#x5266;",
    "剧" => "&#x5267;",
    "剨" => "&#x5268;",
    "剩" => "&#x5269;",
    "剪" => "&#x526A;",
    "剫" => "&#x526B;",
    "剬" => "&#x526C;",
    "剭" => "&#x526D;",
    "剮" => "&#x526E;",
    "副" => "&#x526F;",
    "剰" => "&#x5270;",
    "剱" => "&#x5271;",
    "割" => "&#x5272;",
    "剳" => "&#x5273;",
    "剴" => "&#x5274;",
    "創" => "&#x5275;",
    "剶" => "&#x5276;",
    "剷" => "&#x5277;",
    "剸" => "&#x5278;",
    "剹" => "&#x5279;",
    "剺" => "&#x527A;",
    "剻" => "&#x527B;",
    "剼" => "&#x527C;",
    "剽" => "&#x527D;",
    "剾" => "&#x527E;",
    "剿" => "&#x527F;",
    "劀" => "&#x5280;",
    "劁" => "&#x5281;",
    "劂" => "&#x5282;",
    "劃" => "&#x5283;",
    "劄" => "&#x5284;",
    "劅" => "&#x5285;",
    "劆" => "&#x5286;",
    "劇" => "&#x5287;",
    "劈" => "&#x5288;",
    "劉" => "&#x5289;",
    "劊" => "&#x528A;",
    "劋" => "&#x528B;",
    "劌" => "&#x528C;",
    "劍" => "&#x528D;",
    "劎" => "&#x528E;",
    "劏" => "&#x528F;",
    "劐" => "&#x5290;",
    "劑" => "&#x5291;",
    "劒" => "&#x5292;",
    "劓" => "&#x5293;",
    "劔" => "&#x5294;",
    "劕" => "&#x5295;",
    "劖" => "&#x5296;",
    "劗" => "&#x5297;",
    "劘" => "&#x5298;",
    "劙" => "&#x5299;",
    "劚" => "&#x529A;",
    "力" => "&#x529B;",
    "劜" => "&#x529C;",
    "劝" => "&#x529D;",
    "办" => "&#x529E;",
    "功" => "&#x529F;",
    "加" => "&#x52A0;",
    "务" => "&#x52A1;",
    "劢" => "&#x52A2;",
    "劣" => "&#x52A3;",
    "劤" => "&#x52A4;",
    "劥" => "&#x52A5;",
    "劦" => "&#x52A6;",
    "劧" => "&#x52A7;",
    "动" => "&#x52A8;",
    "助" => "&#x52A9;",
    "努" => "&#x52AA;",
    "劫" => "&#x52AB;",
    "劬" => "&#x52AC;",
    "劭" => "&#x52AD;",
    "劮" => "&#x52AE;",
    "劯" => "&#x52AF;",
    "劰" => "&#x52B0;",
    "励" => "&#x52B1;",
    "劲" => "&#x52B2;",
    "劳" => "&#x52B3;",
    "労" => "&#x52B4;",
    "劵" => "&#x52B5;",
    "劶" => "&#x52B6;",
    "劷" => "&#x52B7;",
    "劸" => "&#x52B8;",
    "効" => "&#x52B9;",
    "劺" => "&#x52BA;",
    "劻" => "&#x52BB;",
    "劼" => "&#x52BC;",
    "劽" => "&#x52BD;",
    "劾" => "&#x52BE;",
    "势" => "&#x52BF;",
    "勀" => "&#x52C0;",
    "勁" => "&#x52C1;",
    "勂" => "&#x52C2;",
    "勃" => "&#x52C3;",
    "勄" => "&#x52C4;",
    "勅" => "&#x52C5;",
    "勆" => "&#x52C6;",
    "勇" => "&#x52C7;",
    "勈" => "&#x52C8;",
    "勉" => "&#x52C9;",
    "勊" => "&#x52CA;",
    "勋" => "&#x52CB;",
    "勌" => "&#x52CC;",
    "勍" => "&#x52CD;",
    "勎" => "&#x52CE;",
    "勏" => "&#x52CF;",
    "勐" => "&#x52D0;",
    "勑" => "&#x52D1;",
    "勒" => "&#x52D2;",
    "勓" => "&#x52D3;",
    "勔" => "&#x52D4;",
    "動" => "&#x52D5;",
    "勖" => "&#x52D6;",
    "勗" => "&#x52D7;",
    "勘" => "&#x52D8;",
    "務" => "&#x52D9;",
    "勚" => "&#x52DA;",
    "勛" => "&#x52DB;",
    "勜" => "&#x52DC;",
    "勝" => "&#x52DD;",
    "勞" => "&#x52DE;",
    "募" => "&#x52DF;",
    "勠" => "&#x52E0;",
    "勡" => "&#x52E1;",
    "勢" => "&#x52E2;",
    "勣" => "&#x52E3;",
    "勤" => "&#x52E4;",
    "勥" => "&#x52E5;",
    "勦" => "&#x52E6;",
    "勧" => "&#x52E7;",
    "勨" => "&#x52E8;",
    "勩" => "&#x52E9;",
    "勪" => "&#x52EA;",
    "勫" => "&#x52EB;",
    "勬" => "&#x52EC;",
    "勭" => "&#x52ED;",
    "勮" => "&#x52EE;",
    "勯" => "&#x52EF;",
    "勰" => "&#x52F0;",
    "勱" => "&#x52F1;",
    "勲" => "&#x52F2;",
    "勳" => "&#x52F3;",
    "勴" => "&#x52F4;",
    "勵" => "&#x52F5;",
    "勶" => "&#x52F6;",
    "勷" => "&#x52F7;",
    "勸" => "&#x52F8;",
    "勹" => "&#x52F9;",
    "勺" => "&#x52FA;",
    "勻" => "&#x52FB;",
    "勼" => "&#x52FC;",
    "勽" => "&#x52FD;",
    "勾" => "&#x52FE;",
    "勿" => "&#x52FF;",
    "匀" => "&#x5300;",
    "匁" => "&#x5301;",
    "匂" => "&#x5302;",
    "匃" => "&#x5303;",
    "匄" => "&#x5304;",
    "包" => "&#x5305;",
    "匆" => "&#x5306;",
    "匇" => "&#x5307;",
    "匈" => "&#x5308;",
    "匉" => "&#x5309;",
    "匊" => "&#x530A;",
    "匋" => "&#x530B;",
    "匌" => "&#x530C;",
    "匍" => "&#x530D;",
    "匎" => "&#x530E;",
    "匏" => "&#x530F;",
    "匐" => "&#x5310;",
    "匑" => "&#x5311;",
    "匒" => "&#x5312;",
    "匓" => "&#x5313;",
    "匔" => "&#x5314;",
    "匕" => "&#x5315;",
    "化" => "&#x5316;",
    "北" => "&#x5317;",
    "匘" => "&#x5318;",
    "匙" => "&#x5319;",
    "匚" => "&#x531A;",
    "匛" => "&#x531B;",
    "匜" => "&#x531C;",
    "匝" => "&#x531D;",
    "匞" => "&#x531E;",
    "匟" => "&#x531F;",
    "匠" => "&#x5320;",
    "匡" => "&#x5321;",
    "匢" => "&#x5322;",
    "匣" => "&#x5323;",
    "匤" => "&#x5324;",
    "匥" => "&#x5325;",
    "匦" => "&#x5326;",
    "匧" => "&#x5327;",
    "匨" => "&#x5328;",
    "匩" => "&#x5329;",
    "匪" => "&#x532A;",
    "匫" => "&#x532B;",
    "匬" => "&#x532C;",
    "匭" => "&#x532D;",
    "匮" => "&#x532E;",
    "匯" => "&#x532F;",
    "匰" => "&#x5330;",
    "匱" => "&#x5331;",
    "匲" => "&#x5332;",
    "匳" => "&#x5333;",
    "匴" => "&#x5334;",
    "匵" => "&#x5335;",
    "匶" => "&#x5336;",
    "匷" => "&#x5337;",
    "匸" => "&#x5338;",
    "匹" => "&#x5339;",
    "区" => "&#x533A;",
    "医" => "&#x533B;",
    "匼" => "&#x533C;",
    "匽" => "&#x533D;",
    "匾" => "&#x533E;",
    "匿" => "&#x533F;",
    "區" => "&#x5340;",
    "十" => "&#x5341;",
    "卂" => "&#x5342;",
    "千" => "&#x5343;",
    "卄" => "&#x5344;",
    "卅" => "&#x5345;",
    "卆" => "&#x5346;",
    "升" => "&#x5347;",
    "午" => "&#x5348;",
    "卉" => "&#x5349;",
    "半" => "&#x534A;",
    "卋" => "&#x534B;",
    "卌" => "&#x534C;",
    "卍" => "&#x534D;",
    "华" => "&#x534E;",
    "协" => "&#x534F;",
    "卐" => "&#x5350;",
    "卑" => "&#x5351;",
    "卒" => "&#x5352;",
    "卓" => "&#x5353;",
    "協" => "&#x5354;",
    "单" => "&#x5355;",
    "卖" => "&#x5356;",
    "南" => "&#x5357;",
    "単" => "&#x5358;",
    "卙" => "&#x5359;",
    "博" => "&#x535A;",
    "卛" => "&#x535B;",
    "卜" => "&#x535C;",
    "卝" => "&#x535D;",
    "卞" => "&#x535E;",
    "卟" => "&#x535F;",
    "占" => "&#x5360;",
    "卡" => "&#x5361;",
    "卢" => "&#x5362;",
    "卣" => "&#x5363;",
    "卤" => "&#x5364;",
    "卥" => "&#x5365;",
    "卦" => "&#x5366;",
    "卧" => "&#x5367;",
    "卨" => "&#x5368;",
    "卩" => "&#x5369;",
    "卪" => "&#x536A;",
    "卫" => "&#x536B;",
    "卬" => "&#x536C;",
    "卭" => "&#x536D;",
    "卮" => "&#x536E;",
    "卯" => "&#x536F;",
    "印" => "&#x5370;",
    "危" => "&#x5371;",
    "卲" => "&#x5372;",
    "即" => "&#x5373;",
    "却" => "&#x5374;",
    "卵" => "&#x5375;",
    "卶" => "&#x5376;",
    "卷" => "&#x5377;",
    "卸" => "&#x5378;",
    "卹" => "&#x5379;",
    "卺" => "&#x537A;",
    "卻" => "&#x537B;",
    "卼" => "&#x537C;",
    "卽" => "&#x537D;",
    "卾" => "&#x537E;",
    "卿" => "&#x537F;",
    "厀" => "&#x5380;",
    "厁" => "&#x5381;",
    "厂" => "&#x5382;",
    "厃" => "&#x5383;",
    "厄" => "&#x5384;",
    "厅" => "&#x5385;",
    "历" => "&#x5386;",
    "厇" => "&#x5387;",
    "厈" => "&#x5388;",
    "厉" => "&#x5389;",
    "厊" => "&#x538A;",
    "压" => "&#x538B;",
    "厌" => "&#x538C;",
    "厍" => "&#x538D;",
    "厎" => "&#x538E;",
    "厏" => "&#x538F;",
    "厐" => "&#x5390;",
    "厑" => "&#x5391;",
    "厒" => "&#x5392;",
    "厓" => "&#x5393;",
    "厔" => "&#x5394;",
    "厕" => "&#x5395;",
    "厖" => "&#x5396;",
    "厗" => "&#x5397;",
    "厘" => "&#x5398;",
    "厙" => "&#x5399;",
    "厚" => "&#x539A;",
    "厛" => "&#x539B;",
    "厜" => "&#x539C;",
    "厝" => "&#x539D;",
    "厞" => "&#x539E;",
    "原" => "&#x539F;",
    "厠" => "&#x53A0;",
    "厡" => "&#x53A1;",
    "厢" => "&#x53A2;",
    "厣" => "&#x53A3;",
    "厤" => "&#x53A4;",
    "厥" => "&#x53A5;",
    "厦" => "&#x53A6;",
    "厧" => "&#x53A7;",
    "厨" => "&#x53A8;",
    "厩" => "&#x53A9;",
    "厪" => "&#x53AA;",
    "厫" => "&#x53AB;",
    "厬" => "&#x53AC;",
    "厭" => "&#x53AD;",
    "厮" => "&#x53AE;",
    "厯" => "&#x53AF;",
    "厰" => "&#x53B0;",
    "厱" => "&#x53B1;",
    "厲" => "&#x53B2;",
    "厳" => "&#x53B3;",
    "厴" => "&#x53B4;",
    "厵" => "&#x53B5;",
    "厶" => "&#x53B6;",
    "厷" => "&#x53B7;",
    "厸" => "&#x53B8;",
    "厹" => "&#x53B9;",
    "厺" => "&#x53BA;",
    "去" => "&#x53BB;",
    "厼" => "&#x53BC;",
    "厽" => "&#x53BD;",
    "厾" => "&#x53BE;",
    "县" => "&#x53BF;",
    "叀" => "&#x53C0;",
    "叁" => "&#x53C1;",
    "参" => "&#x53C2;",
    "參" => "&#x53C3;",
    "叄" => "&#x53C4;",
    "叅" => "&#x53C5;",
    "叆" => "&#x53C6;",
    "叇" => "&#x53C7;",
    "又" => "&#x53C8;",
    "叉" => "&#x53C9;",
    "及" => "&#x53CA;",
    "友" => "&#x53CB;",
    "双" => "&#x53CC;",
    "反" => "&#x53CD;",
    "収" => "&#x53CE;",
    "叏" => "&#x53CF;",
    "叐" => "&#x53D0;",
    "发" => "&#x53D1;",
    "叒" => "&#x53D2;",
    "叓" => "&#x53D3;",
    "叔" => "&#x53D4;",
    "叕" => "&#x53D5;",
    "取" => "&#x53D6;",
    "受" => "&#x53D7;",
    "变" => "&#x53D8;",
    "叙" => "&#x53D9;",
    "叚" => "&#x53DA;",
    "叛" => "&#x53DB;",
    "叜" => "&#x53DC;",
    "叝" => "&#x53DD;",
    "叞" => "&#x53DE;",
    "叟" => "&#x53DF;",
    "叠" => "&#x53E0;",
    "叡" => "&#x53E1;",
    "叢" => "&#x53E2;",
    "口" => "&#x53E3;",
    "古" => "&#x53E4;",
    "句" => "&#x53E5;",
    "另" => "&#x53E6;",
    "叧" => "&#x53E7;",
    "叨" => "&#x53E8;",
    "叩" => "&#x53E9;",
    "只" => "&#x53EA;",
    "叫" => "&#x53EB;",
    "召" => "&#x53EC;",
    "叭" => "&#x53ED;",
    "叮" => "&#x53EE;",
    "可" => "&#x53EF;",
    "台" => "&#x53F0;",
    "叱" => "&#x53F1;",
    "史" => "&#x53F2;",
    "右" => "&#x53F3;",
    "叴" => "&#x53F4;",
    "叵" => "&#x53F5;",
    "叶" => "&#x53F6;",
    "号" => "&#x53F7;",
    "司" => "&#x53F8;",
    "叹" => "&#x53F9;",
    "叺" => "&#x53FA;",
    "叻" => "&#x53FB;",
    "叼" => "&#x53FC;",
    "叽" => "&#x53FD;",
    "叾" => "&#x53FE;",
    "叿" => "&#x53FF;",
    "吀" => "&#x5400;",
    "吁" => "&#x5401;",
    "吂" => "&#x5402;",
    "吃" => "&#x5403;",
    "各" => "&#x5404;",
    "吅" => "&#x5405;",
    "吆" => "&#x5406;",
    "吇" => "&#x5407;",
    "合" => "&#x5408;",
    "吉" => "&#x5409;",
    "吊" => "&#x540A;",
    "吋" => "&#x540B;",
    "同" => "&#x540C;",
    "名" => "&#x540D;",
    "后" => "&#x540E;",
    "吏" => "&#x540F;",
    "吐" => "&#x5410;",
    "向" => "&#x5411;",
    "吒" => "&#x5412;",
    "吓" => "&#x5413;",
    "吔" => "&#x5414;",
    "吕" => "&#x5415;",
    "吖" => "&#x5416;",
    "吗" => "&#x5417;",
    "吘" => "&#x5418;",
    "吙" => "&#x5419;",
    "吚" => "&#x541A;",
    "君" => "&#x541B;",
    "吜" => "&#x541C;",
    "吝" => "&#x541D;",
    "吞" => "&#x541E;",
    "吟" => "&#x541F;",
    "吠" => "&#x5420;",
    "吡" => "&#x5421;",
    "吢" => "&#x5422;",
    "吣" => "&#x5423;",
    "吤" => "&#x5424;",
    "吥" => "&#x5425;",
    "否" => "&#x5426;",
    "吧" => "&#x5427;",
    "吨" => "&#x5428;",
    "吩" => "&#x5429;",
    "吪" => "&#x542A;",
    "含" => "&#x542B;",
    "听" => "&#x542C;",
    "吭" => "&#x542D;",
    "吮" => "&#x542E;",
    "启" => "&#x542F;",
    "吰" => "&#x5430;",
    "吱" => "&#x5431;",
    "吲" => "&#x5432;",
    "吳" => "&#x5433;",
    "吴" => "&#x5434;",
    "吵" => "&#x5435;",
    "吶" => "&#x5436;",
    "吷" => "&#x5437;",
    "吸" => "&#x5438;",
    "吹" => "&#x5439;",
    "吺" => "&#x543A;",
    "吻" => "&#x543B;",
    "吼" => "&#x543C;",
    "吽" => "&#x543D;",
    "吾" => "&#x543E;",
    "吿" => "&#x543F;",
    "呀" => "&#x5440;",
    "呁" => "&#x5441;",
    "呂" => "&#x5442;",
    "呃" => "&#x5443;",
    "呄" => "&#x5444;",
    "呅" => "&#x5445;",
    "呆" => "&#x5446;",
    "呇" => "&#x5447;",
    "呈" => "&#x5448;",
    "呉" => "&#x5449;",
    "告" => "&#x544A;",
    "呋" => "&#x544B;",
    "呌" => "&#x544C;",
    "呍" => "&#x544D;",
    "呎" => "&#x544E;",
    "呏" => "&#x544F;",
    "呐" => "&#x5450;",
    "呑" => "&#x5451;",
    "呒" => "&#x5452;",
    "呓" => "&#x5453;",
    "呔" => "&#x5454;",
    "呕" => "&#x5455;",
    "呖" => "&#x5456;",
    "呗" => "&#x5457;",
    "员" => "&#x5458;",
    "呙" => "&#x5459;",
    "呚" => "&#x545A;",
    "呛" => "&#x545B;",
    "呜" => "&#x545C;",
    "呝" => "&#x545D;",
    "呞" => "&#x545E;",
    "呟" => "&#x545F;",
    "呠" => "&#x5460;",
    "呡" => "&#x5461;",
    "呢" => "&#x5462;",
    "呣" => "&#x5463;",
    "呤" => "&#x5464;",
    "呥" => "&#x5465;",
    "呦" => "&#x5466;",
    "呧" => "&#x5467;",
    "周" => "&#x5468;",
    "呩" => "&#x5469;",
    "呪" => "&#x546A;",
    "呫" => "&#x546B;",
    "呬" => "&#x546C;",
    "呭" => "&#x546D;",
    "呮" => "&#x546E;",
    "呯" => "&#x546F;",
    "呰" => "&#x5470;",
    "呱" => "&#x5471;",
    "呲" => "&#x5472;",
    "味" => "&#x5473;",
    "呴" => "&#x5474;",
    "呵" => "&#x5475;",
    "呶" => "&#x5476;",
    "呷" => "&#x5477;",
    "呸" => "&#x5478;",
    "呹" => "&#x5479;",
    "呺" => "&#x547A;",
    "呻" => "&#x547B;",
    "呼" => "&#x547C;",
    "命" => "&#x547D;",
    "呾" => "&#x547E;",
    "呿" => "&#x547F;",
    "咀" => "&#x5480;",
    "咁" => "&#x5481;",
    "咂" => "&#x5482;",
    "咃" => "&#x5483;",
    "咄" => "&#x5484;",
    "咅" => "&#x5485;",
    "咆" => "&#x5486;",
    "咇" => "&#x5487;",
    "咈" => "&#x5488;",
    "咉" => "&#x5489;",
    "咊" => "&#x548A;",
    "咋" => "&#x548B;",
    "和" => "&#x548C;",
    "咍" => "&#x548D;",
    "咎" => "&#x548E;",
    "咏" => "&#x548F;",
    "咐" => "&#x5490;",
    "咑" => "&#x5491;",
    "咒" => "&#x5492;",
    "咓" => "&#x5493;",
    "咔" => "&#x5494;",
    "咕" => "&#x5495;",
    "咖" => "&#x5496;",
    "咗" => "&#x5497;",
    "咘" => "&#x5498;",
    "咙" => "&#x5499;",
    "咚" => "&#x549A;",
    "咛" => "&#x549B;",
    "咜" => "&#x549C;",
    "咝" => "&#x549D;",
    "咞" => "&#x549E;",
    "咟" => "&#x549F;",
    "咠" => "&#x54A0;",
    "咡" => "&#x54A1;",
    "咢" => "&#x54A2;",
    "咣" => "&#x54A3;",
    "咤" => "&#x54A4;",
    "咥" => "&#x54A5;",
    "咦" => "&#x54A6;",
    "咧" => "&#x54A7;",
    "咨" => "&#x54A8;",
    "咩" => "&#x54A9;",
    "咪" => "&#x54AA;",
    "咫" => "&#x54AB;",
    "咬" => "&#x54AC;",
    "咭" => "&#x54AD;",
    "咮" => "&#x54AE;",
    "咯" => "&#x54AF;",
    "咰" => "&#x54B0;",
    "咱" => "&#x54B1;",
    "咲" => "&#x54B2;",
    "咳" => "&#x54B3;",
    "咴" => "&#x54B4;",
    "咵" => "&#x54B5;",
    "咶" => "&#x54B6;",
    "咷" => "&#x54B7;",
    "咸" => "&#x54B8;",
    "咹" => "&#x54B9;",
    "咺" => "&#x54BA;",
    "咻" => "&#x54BB;",
    "咼" => "&#x54BC;",
    "咽" => "&#x54BD;",
    "咾" => "&#x54BE;",
    "咿" => "&#x54BF;",
    "哀" => "&#x54C0;",
    "品" => "&#x54C1;",
    "哂" => "&#x54C2;",
    "哃" => "&#x54C3;",
    "哄" => "&#x54C4;",
    "哅" => "&#x54C5;",
    "哆" => "&#x54C6;",
    "哇" => "&#x54C7;",
    "哈" => "&#x54C8;",
    "哉" => "&#x54C9;",
    "哊" => "&#x54CA;",
    "哋" => "&#x54CB;",
    "哌" => "&#x54CC;",
    "响" => "&#x54CD;",
    "哎" => "&#x54CE;",
    "哏" => "&#x54CF;",
    "哐" => "&#x54D0;",
    "哑" => "&#x54D1;",
    "哒" => "&#x54D2;",
    "哓" => "&#x54D3;",
    "哔" => "&#x54D4;",
    "哕" => "&#x54D5;",
    "哖" => "&#x54D6;",
    "哗" => "&#x54D7;",
    "哘" => "&#x54D8;",
    "哙" => "&#x54D9;",
    "哚" => "&#x54DA;",
    "哛" => "&#x54DB;",
    "哜" => "&#x54DC;",
    "哝" => "&#x54DD;",
    "哞" => "&#x54DE;",
    "哟" => "&#x54DF;",
    "哠" => "&#x54E0;",
    "員" => "&#x54E1;",
    "哢" => "&#x54E2;",
    "哣" => "&#x54E3;",
    "哤" => "&#x54E4;",
    "哥" => "&#x54E5;",
    "哦" => "&#x54E6;",
    "哧" => "&#x54E7;",
    "哨" => "&#x54E8;",
    "哩" => "&#x54E9;",
    "哪" => "&#x54EA;",
    "哫" => "&#x54EB;",
    "哬" => "&#x54EC;",
    "哭" => "&#x54ED;",
    "哮" => "&#x54EE;",
    "哯" => "&#x54EF;",
    "哰" => "&#x54F0;",
    "哱" => "&#x54F1;",
    "哲" => "&#x54F2;",
    "哳" => "&#x54F3;",
    "哴" => "&#x54F4;",
    "哵" => "&#x54F5;",
    "哶" => "&#x54F6;",
    "哷" => "&#x54F7;",
    "哸" => "&#x54F8;",
    "哹" => "&#x54F9;",
    "哺" => "&#x54FA;",
    "哻" => "&#x54FB;",
    "哼" => "&#x54FC;",
    "哽" => "&#x54FD;",
    "哾" => "&#x54FE;",
    "哿" => "&#x54FF;",
    "唀" => "&#x5500;",
    "唁" => "&#x5501;",
    "唂" => "&#x5502;",
    "唃" => "&#x5503;",
    "唄" => "&#x5504;",
    "唅" => "&#x5505;",
    "唆" => "&#x5506;",
    "唇" => "&#x5507;",
    "唈" => "&#x5508;",
    "唉" => "&#x5509;",
    "唊" => "&#x550A;",
    "唋" => "&#x550B;",
    "唌" => "&#x550C;",
    "唍" => "&#x550D;",
    "唎" => "&#x550E;",
    "唏" => "&#x550F;",
    "唐" => "&#x5510;",
    "唑" => "&#x5511;",
    "唒" => "&#x5512;",
    "唓" => "&#x5513;",
    "唔" => "&#x5514;",
    "唕" => "&#x5515;",
    "唖" => "&#x5516;",
    "唗" => "&#x5517;",
    "唘" => "&#x5518;",
    "唙" => "&#x5519;",
    "唚" => "&#x551A;",
    "唛" => "&#x551B;",
    "唜" => "&#x551C;",
    "唝" => "&#x551D;",
    "唞" => "&#x551E;",
    "唟" => "&#x551F;",
    "唠" => "&#x5520;",
    "唡" => "&#x5521;",
    "唢" => "&#x5522;",
    "唣" => "&#x5523;",
    "唤" => "&#x5524;",
    "唥" => "&#x5525;",
    "唦" => "&#x5526;",
    "唧" => "&#x5527;",
    "唨" => "&#x5528;",
    "唩" => "&#x5529;",
    "唪" => "&#x552A;",
    "唫" => "&#x552B;",
    "唬" => "&#x552C;",
    "唭" => "&#x552D;",
    "售" => "&#x552E;",
    "唯" => "&#x552F;",
    "唰" => "&#x5530;",
    "唱" => "&#x5531;",
    "唲" => "&#x5532;",
    "唳" => "&#x5533;",
    "唴" => "&#x5534;",
    "唵" => "&#x5535;",
    "唶" => "&#x5536;",
    "唷" => "&#x5537;",
    "唸" => "&#x5538;",
    "唹" => "&#x5539;",
    "唺" => "&#x553A;",
    "唻" => "&#x553B;",
    "唼" => "&#x553C;",
    "唽" => "&#x553D;",
    "唾" => "&#x553E;",
    "唿" => "&#x553F;",
    "啀" => "&#x5540;",
    "啁" => "&#x5541;",
    "啂" => "&#x5542;",
    "啃" => "&#x5543;",
    "啄" => "&#x5544;",
    "啅" => "&#x5545;",
    "商" => "&#x5546;",
    "啇" => "&#x5547;",
    "啈" => "&#x5548;",
    "啉" => "&#x5549;",
    "啊" => "&#x554A;",
    "啋" => "&#x554B;",
    "啌" => "&#x554C;",
    "啍" => "&#x554D;",
    "啎" => "&#x554E;",
    "問" => "&#x554F;",
    "啐" => "&#x5550;",
    "啑" => "&#x5551;",
    "啒" => "&#x5552;",
    "啓" => "&#x5553;",
    "啔" => "&#x5554;",
    "啕" => "&#x5555;",
    "啖" => "&#x5556;",
    "啗" => "&#x5557;",
    "啘" => "&#x5558;",
    "啙" => "&#x5559;",
    "啚" => "&#x555A;",
    "啛" => "&#x555B;",
    "啜" => "&#x555C;",
    "啝" => "&#x555D;",
    "啞" => "&#x555E;",
    "啟" => "&#x555F;",
    "啠" => "&#x5560;",
    "啡" => "&#x5561;",
    "啢" => "&#x5562;",
    "啣" => "&#x5563;",
    "啤" => "&#x5564;",
    "啥" => "&#x5565;",
    "啦" => "&#x5566;",
    "啧" => "&#x5567;",
    "啨" => "&#x5568;",
    "啩" => "&#x5569;",
    "啪" => "&#x556A;",
    "啫" => "&#x556B;",
    "啬" => "&#x556C;",
    "啭" => "&#x556D;",
    "啮" => "&#x556E;",
    "啯" => "&#x556F;",
    "啰" => "&#x5570;",
    "啱" => "&#x5571;",
    "啲" => "&#x5572;",
    "啳" => "&#x5573;",
    "啴" => "&#x5574;",
    "啵" => "&#x5575;",
    "啶" => "&#x5576;",
    "啷" => "&#x5577;",
    "啸" => "&#x5578;",
    "啹" => "&#x5579;",
    "啺" => "&#x557A;",
    "啻" => "&#x557B;",
    "啼" => "&#x557C;",
    "啽" => "&#x557D;",
    "啾" => "&#x557E;",
    "啿" => "&#x557F;",
    "喀" => "&#x5580;",
    "喁" => "&#x5581;",
    "喂" => "&#x5582;",
    "喃" => "&#x5583;",
    "善" => "&#x5584;",
    "喅" => "&#x5585;",
    "喆" => "&#x5586;",
    "喇" => "&#x5587;",
    "喈" => "&#x5588;",
    "喉" => "&#x5589;",
    "喊" => "&#x558A;",
    "喋" => "&#x558B;",
    "喌" => "&#x558C;",
    "喍" => "&#x558D;",
    "喎" => "&#x558E;",
    "喏" => "&#x558F;",
    "喐" => "&#x5590;",
    "喑" => "&#x5591;",
    "喒" => "&#x5592;",
    "喓" => "&#x5593;",
    "喔" => "&#x5594;",
    "喕" => "&#x5595;",
    "喖" => "&#x5596;",
    "喗" => "&#x5597;",
    "喘" => "&#x5598;",
    "喙" => "&#x5599;",
    "喚" => "&#x559A;",
    "喛" => "&#x559B;",
    "喜" => "&#x559C;",
    "喝" => "&#x559D;",
    "喞" => "&#x559E;",
    "喟" => "&#x559F;",
    "喠" => "&#x55A0;",
    "喡" => "&#x55A1;",
    "喢" => "&#x55A2;",
    "喣" => "&#x55A3;",
    "喤" => "&#x55A4;",
    "喥" => "&#x55A5;",
    "喦" => "&#x55A6;",
    "喧" => "&#x55A7;",
    "喨" => "&#x55A8;",
    "喩" => "&#x55A9;",
    "喪" => "&#x55AA;",
    "喫" => "&#x55AB;",
    "喬" => "&#x55AC;",
    "喭" => "&#x55AD;",
    "單" => "&#x55AE;",
    "喯" => "&#x55AF;",
    "喰" => "&#x55B0;",
    "喱" => "&#x55B1;",
    "喲" => "&#x55B2;",
    "喳" => "&#x55B3;",
    "喴" => "&#x55B4;",
    "喵" => "&#x55B5;",
    "営" => "&#x55B6;",
    "喷" => "&#x55B7;",
    "喸" => "&#x55B8;",
    "喹" => "&#x55B9;",
    "喺" => "&#x55BA;",
    "喻" => "&#x55BB;",
    "喼" => "&#x55BC;",
    "喽" => "&#x55BD;",
    "喾" => "&#x55BE;",
    "喿" => "&#x55BF;",
    "嗀" => "&#x55C0;",
    "嗁" => "&#x55C1;",
    "嗂" => "&#x55C2;",
    "嗃" => "&#x55C3;",
    "嗄" => "&#x55C4;",
    "嗅" => "&#x55C5;",
    "嗆" => "&#x55C6;",
    "嗇" => "&#x55C7;",
    "嗈" => "&#x55C8;",
    "嗉" => "&#x55C9;",
    "嗊" => "&#x55CA;",
    "嗋" => "&#x55CB;",
    "嗌" => "&#x55CC;",
    "嗍" => "&#x55CD;",
    "嗎" => "&#x55CE;",
    "嗏" => "&#x55CF;",
    "嗐" => "&#x55D0;",
    "嗑" => "&#x55D1;",
    "嗒" => "&#x55D2;",
    "嗓" => "&#x55D3;",
    "嗔" => "&#x55D4;",
    "嗕" => "&#x55D5;",
    "嗖" => "&#x55D6;",
    "嗗" => "&#x55D7;",
    "嗘" => "&#x55D8;",
    "嗙" => "&#x55D9;",
    "嗚" => "&#x55DA;",
    "嗛" => "&#x55DB;",
    "嗜" => "&#x55DC;",
    "嗝" => "&#x55DD;",
    "嗞" => "&#x55DE;",
    "嗟" => "&#x55DF;",
    "嗠" => "&#x55E0;",
    "嗡" => "&#x55E1;",
    "嗢" => "&#x55E2;",
    "嗣" => "&#x55E3;",
    "嗤" => "&#x55E4;",
    "嗥" => "&#x55E5;",
    "嗦" => "&#x55E6;",
    "嗧" => "&#x55E7;",
    "嗨" => "&#x55E8;",
    "嗩" => "&#x55E9;",
    "嗪" => "&#x55EA;",
    "嗫" => "&#x55EB;",
    "嗬" => "&#x55EC;",
    "嗭" => "&#x55ED;",
    "嗮" => "&#x55EE;",
    "嗯" => "&#x55EF;",
    "嗰" => "&#x55F0;",
    "嗱" => "&#x55F1;",
    "嗲" => "&#x55F2;",
    "嗳" => "&#x55F3;",
    "嗴" => "&#x55F4;",
    "嗵" => "&#x55F5;",
    "嗶" => "&#x55F6;",
    "嗷" => "&#x55F7;",
    "嗸" => "&#x55F8;",
    "嗹" => "&#x55F9;",
    "嗺" => "&#x55FA;",
    "嗻" => "&#x55FB;",
    "嗼" => "&#x55FC;",
    "嗽" => "&#x55FD;",
    "嗾" => "&#x55FE;",
    "嗿" => "&#x55FF;",
    "嘀" => "&#x5600;",
    "嘁" => "&#x5601;",
    "嘂" => "&#x5602;",
    "嘃" => "&#x5603;",
    "嘄" => "&#x5604;",
    "嘅" => "&#x5605;",
    "嘆" => "&#x5606;",
    "嘇" => "&#x5607;",
    "嘈" => "&#x5608;",
    "嘉" => "&#x5609;",
    "嘊" => "&#x560A;",
    "嘋" => "&#x560B;",
    "嘌" => "&#x560C;",
    "嘍" => "&#x560D;",
    "嘎" => "&#x560E;",
    "嘏" => "&#x560F;",
    "嘐" => "&#x5610;",
    "嘑" => "&#x5611;",
    "嘒" => "&#x5612;",
    "嘓" => "&#x5613;",
    "嘔" => "&#x5614;",
    "嘕" => "&#x5615;",
    "嘖" => "&#x5616;",
    "嘗" => "&#x5617;",
    "嘘" => "&#x5618;",
    "嘙" => "&#x5619;",
    "嘚" => "&#x561A;",
    "嘛" => "&#x561B;",
    "嘜" => "&#x561C;",
    "嘝" => "&#x561D;",
    "嘞" => "&#x561E;",
    "嘟" => "&#x561F;",
    "嘠" => "&#x5620;",
    "嘡" => "&#x5621;",
    "嘢" => "&#x5622;",
    "嘣" => "&#x5623;",
    "嘤" => "&#x5624;",
    "嘥" => "&#x5625;",
    "嘦" => "&#x5626;",
    "嘧" => "&#x5627;",
    "嘨" => "&#x5628;",
    "嘩" => "&#x5629;",
    "嘪" => "&#x562A;",
    "嘫" => "&#x562B;",
    "嘬" => "&#x562C;",
    "嘭" => "&#x562D;",
    "嘮" => "&#x562E;",
    "嘯" => "&#x562F;",
    "嘰" => "&#x5630;",
    "嘱" => "&#x5631;",
    "嘲" => "&#x5632;",
    "嘳" => "&#x5633;",
    "嘴" => "&#x5634;",
    "嘵" => "&#x5635;",
    "嘶" => "&#x5636;",
    "嘷" => "&#x5637;",
    "嘸" => "&#x5638;",
    "嘹" => "&#x5639;",
    "嘺" => "&#x563A;",
    "嘻" => "&#x563B;",
    "嘼" => "&#x563C;",
    "嘽" => "&#x563D;",
    "嘾" => "&#x563E;",
    "嘿" => "&#x563F;",
    "噀" => "&#x5640;",
    "噁" => "&#x5641;",
    "噂" => "&#x5642;",
    "噃" => "&#x5643;",
    "噄" => "&#x5644;",
    "噅" => "&#x5645;",
    "噆" => "&#x5646;",
    "噇" => "&#x5647;",
    "噈" => "&#x5648;",
    "噉" => "&#x5649;",
    "噊" => "&#x564A;",
    "噋" => "&#x564B;",
    "噌" => "&#x564C;",
    "噍" => "&#x564D;",
    "噎" => "&#x564E;",
    "噏" => "&#x564F;",
    "噐" => "&#x5650;",
    "噑" => "&#x5651;",
    "噒" => "&#x5652;",
    "噓" => "&#x5653;",
    "噔" => "&#x5654;",
    "噕" => "&#x5655;",
    "噖" => "&#x5656;",
    "噗" => "&#x5657;",
    "噘" => "&#x5658;",
    "噙" => "&#x5659;",
    "噚" => "&#x565A;",
    "噛" => "&#x565B;",
    "噜" => "&#x565C;",
    "噝" => "&#x565D;",
    "噞" => "&#x565E;",
    "噟" => "&#x565F;",
    "噠" => "&#x5660;",
    "噡" => "&#x5661;",
    "噢" => "&#x5662;",
    "噣" => "&#x5663;",
    "噤" => "&#x5664;",
    "噥" => "&#x5665;",
    "噦" => "&#x5666;",
    "噧" => "&#x5667;",
    "器" => "&#x5668;",
    "噩" => "&#x5669;",
    "噪" => "&#x566A;",
    "噫" => "&#x566B;",
    "噬" => "&#x566C;",
    "噭" => "&#x566D;",
    "噮" => "&#x566E;",
    "噯" => "&#x566F;",
    "噰" => "&#x5670;",
    "噱" => "&#x5671;",
    "噲" => "&#x5672;",
    "噳" => "&#x5673;",
    "噴" => "&#x5674;",
    "噵" => "&#x5675;",
    "噶" => "&#x5676;",
    "噷" => "&#x5677;",
    "噸" => "&#x5678;",
    "噹" => "&#x5679;",
    "噺" => "&#x567A;",
    "噻" => "&#x567B;",
    "噼" => "&#x567C;",
    "噽" => "&#x567D;",
    "噾" => "&#x567E;",
    "噿" => "&#x567F;",
    "嚀" => "&#x5680;",
    "嚁" => "&#x5681;",
    "嚂" => "&#x5682;",
    "嚃" => "&#x5683;",
    "嚄" => "&#x5684;",
    "嚅" => "&#x5685;",
    "嚆" => "&#x5686;",
    "嚇" => "&#x5687;",
    "嚈" => "&#x5688;",
    "嚉" => "&#x5689;",
    "嚊" => "&#x568A;",
    "嚋" => "&#x568B;",
    "嚌" => "&#x568C;",
    "嚍" => "&#x568D;",
    "嚎" => "&#x568E;",
    "嚏" => "&#x568F;",
    "嚐" => "&#x5690;",
    "嚑" => "&#x5691;",
    "嚒" => "&#x5692;",
    "嚓" => "&#x5693;",
    "嚔" => "&#x5694;",
    "嚕" => "&#x5695;",
    "嚖" => "&#x5696;",
    "嚗" => "&#x5697;",
    "嚘" => "&#x5698;",
    "嚙" => "&#x5699;",
    "嚚" => "&#x569A;",
    "嚛" => "&#x569B;",
    "嚜" => "&#x569C;",
    "嚝" => "&#x569D;",
    "嚞" => "&#x569E;",
    "嚟" => "&#x569F;",
    "嚠" => "&#x56A0;",
    "嚡" => "&#x56A1;",
    "嚢" => "&#x56A2;",
    "嚣" => "&#x56A3;",
    "嚤" => "&#x56A4;",
    "嚥" => "&#x56A5;",
    "嚦" => "&#x56A6;",
    "嚧" => "&#x56A7;",
    "嚨" => "&#x56A8;",
    "嚩" => "&#x56A9;",
    "嚪" => "&#x56AA;",
    "嚫" => "&#x56AB;",
    "嚬" => "&#x56AC;",
    "嚭" => "&#x56AD;",
    "嚮" => "&#x56AE;",
    "嚯" => "&#x56AF;",
    "嚰" => "&#x56B0;",
    "嚱" => "&#x56B1;",
    "嚲" => "&#x56B2;",
    "嚳" => "&#x56B3;",
    "嚴" => "&#x56B4;",
    "嚵" => "&#x56B5;",
    "嚶" => "&#x56B6;",
    "嚷" => "&#x56B7;",
    "嚸" => "&#x56B8;",
    "嚹" => "&#x56B9;",
    "嚺" => "&#x56BA;",
    "嚻" => "&#x56BB;",
    "嚼" => "&#x56BC;",
    "嚽" => "&#x56BD;",
    "嚾" => "&#x56BE;",
    "嚿" => "&#x56BF;",
    "囀" => "&#x56C0;",
    "囁" => "&#x56C1;",
    "囂" => "&#x56C2;",
    "囃" => "&#x56C3;",
    "囄" => "&#x56C4;",
    "囅" => "&#x56C5;",
    "囆" => "&#x56C6;",
    "囇" => "&#x56C7;",
    "囈" => "&#x56C8;",
    "囉" => "&#x56C9;",
    "囊" => "&#x56CA;",
    "囋" => "&#x56CB;",
    "囌" => "&#x56CC;",
    "囍" => "&#x56CD;",
    "囎" => "&#x56CE;",
    "囏" => "&#x56CF;",
    "囐" => "&#x56D0;",
    "囑" => "&#x56D1;",
    "囒" => "&#x56D2;",
    "囓" => "&#x56D3;",
    "囔" => "&#x56D4;",
    "囕" => "&#x56D5;",
    "囖" => "&#x56D6;",
    "囗" => "&#x56D7;",
    "囘" => "&#x56D8;",
    "囙" => "&#x56D9;",
    "囚" => "&#x56DA;",
    "四" => "&#x56DB;",
    "囜" => "&#x56DC;",
    "囝" => "&#x56DD;",
    "回" => "&#x56DE;",
    "囟" => "&#x56DF;",
    "因" => "&#x56E0;",
    "囡" => "&#x56E1;",
    "团" => "&#x56E2;",
    "団" => "&#x56E3;",
    "囤" => "&#x56E4;",
    "囥" => "&#x56E5;",
    "囦" => "&#x56E6;",
    "囧" => "&#x56E7;",
    "囨" => "&#x56E8;",
    "囩" => "&#x56E9;",
    "囪" => "&#x56EA;",
    "囫" => "&#x56EB;",
    "囬" => "&#x56EC;",
    "园" => "&#x56ED;",
    "囮" => "&#x56EE;",
    "囯" => "&#x56EF;",
    "困" => "&#x56F0;",
    "囱" => "&#x56F1;",
    "囲" => "&#x56F2;",
    "図" => "&#x56F3;",
    "围" => "&#x56F4;",
    "囵" => "&#x56F5;",
    "囶" => "&#x56F6;",
    "囷" => "&#x56F7;",
    "囸" => "&#x56F8;",
    "囹" => "&#x56F9;",
    "固" => "&#x56FA;",
    "囻" => "&#x56FB;",
    "囼" => "&#x56FC;",
    "国" => "&#x56FD;",
    "图" => "&#x56FE;",
    "囿" => "&#x56FF;",
    "圀" => "&#x5700;",
    "圁" => "&#x5701;",
    "圂" => "&#x5702;",
    "圃" => "&#x5703;",
    "圄" => "&#x5704;",
    "圅" => "&#x5705;",
    "圆" => "&#x5706;",
    "圇" => "&#x5707;",
    "圈" => "&#x5708;",
    "圉" => "&#x5709;",
    "圊" => "&#x570A;",
    "國" => "&#x570B;",
    "圌" => "&#x570C;",
    "圍" => "&#x570D;",
    "圎" => "&#x570E;",
    "圏" => "&#x570F;",
    "圐" => "&#x5710;",
    "圑" => "&#x5711;",
    "園" => "&#x5712;",
    "圓" => "&#x5713;",
    "圔" => "&#x5714;",
    "圕" => "&#x5715;",
    "圖" => "&#x5716;",
    "圗" => "&#x5717;",
    "團" => "&#x5718;",
    "圙" => "&#x5719;",
    "圚" => "&#x571A;",
    "圛" => "&#x571B;",
    "圜" => "&#x571C;",
    "圝" => "&#x571D;",
    "圞" => "&#x571E;",
    "土" => "&#x571F;",
    "圠" => "&#x5720;",
    "圡" => "&#x5721;",
    "圢" => "&#x5722;",
    "圣" => "&#x5723;",
    "圤" => "&#x5724;",
    "圥" => "&#x5725;",
    "圦" => "&#x5726;",
    "圧" => "&#x5727;",
    "在" => "&#x5728;",
    "圩" => "&#x5729;",
    "圪" => "&#x572A;",
    "圫" => "&#x572B;",
    "圬" => "&#x572C;",
    "圭" => "&#x572D;",
    "圮" => "&#x572E;",
    "圯" => "&#x572F;",
    "地" => "&#x5730;",
    "圱" => "&#x5731;",
    "圲" => "&#x5732;",
    "圳" => "&#x5733;",
    "圴" => "&#x5734;",
    "圵" => "&#x5735;",
    "圶" => "&#x5736;",
    "圷" => "&#x5737;",
    "圸" => "&#x5738;",
    "圹" => "&#x5739;",
    "场" => "&#x573A;",
    "圻" => "&#x573B;",
    "圼" => "&#x573C;",
    "圽" => "&#x573D;",
    "圾" => "&#x573E;",
    "圿" => "&#x573F;",
    "址" => "&#x5740;",
    "坁" => "&#x5741;",
    "坂" => "&#x5742;",
    "坃" => "&#x5743;",
    "坄" => "&#x5744;",
    "坅" => "&#x5745;",
    "坆" => "&#x5746;",
    "均" => "&#x5747;",
    "坈" => "&#x5748;",
    "坉" => "&#x5749;",
    "坊" => "&#x574A;",
    "坋" => "&#x574B;",
    "坌" => "&#x574C;",
    "坍" => "&#x574D;",
    "坎" => "&#x574E;",
    "坏" => "&#x574F;",
    "坐" => "&#x5750;",
    "坑" => "&#x5751;",
    "坒" => "&#x5752;",
    "坓" => "&#x5753;",
    "坔" => "&#x5754;",
    "坕" => "&#x5755;",
    "坖" => "&#x5756;",
    "块" => "&#x5757;",
    "坘" => "&#x5758;",
    "坙" => "&#x5759;",
    "坚" => "&#x575A;",
    "坛" => "&#x575B;",
    "坜" => "&#x575C;",
    "坝" => "&#x575D;",
    "坞" => "&#x575E;",
    "坟" => "&#x575F;",
    "坠" => "&#x5760;",
    "坡" => "&#x5761;",
    "坢" => "&#x5762;",
    "坣" => "&#x5763;",
    "坤" => "&#x5764;",
    "坥" => "&#x5765;",
    "坦" => "&#x5766;",
    "坧" => "&#x5767;",
    "坨" => "&#x5768;",
    "坩" => "&#x5769;",
    "坪" => "&#x576A;",
    "坫" => "&#x576B;",
    "坬" => "&#x576C;",
    "坭" => "&#x576D;",
    "坮" => "&#x576E;",
    "坯" => "&#x576F;",
    "坰" => "&#x5770;",
    "坱" => "&#x5771;",
    "坲" => "&#x5772;",
    "坳" => "&#x5773;",
    "坴" => "&#x5774;",
    "坵" => "&#x5775;",
    "坶" => "&#x5776;",
    "坷" => "&#x5777;",
    "坸" => "&#x5778;",
    "坹" => "&#x5779;",
    "坺" => "&#x577A;",
    "坻" => "&#x577B;",
    "坼" => "&#x577C;",
    "坽" => "&#x577D;",
    "坾" => "&#x577E;",
    "坿" => "&#x577F;",
    "垀" => "&#x5780;",
    "垁" => "&#x5781;",
    "垂" => "&#x5782;",
    "垃" => "&#x5783;",
    "垄" => "&#x5784;",
    "垅" => "&#x5785;",
    "垆" => "&#x5786;",
    "垇" => "&#x5787;",
    "垈" => "&#x5788;",
    "垉" => "&#x5789;",
    "垊" => "&#x578A;",
    "型" => "&#x578B;",
    "垌" => "&#x578C;",
    "垍" => "&#x578D;",
    "垎" => "&#x578E;",
    "垏" => "&#x578F;",
    "垐" => "&#x5790;",
    "垑" => "&#x5791;",
    "垒" => "&#x5792;",
    "垓" => "&#x5793;",
    "垔" => "&#x5794;",
    "垕" => "&#x5795;",
    "垖" => "&#x5796;",
    "垗" => "&#x5797;",
    "垘" => "&#x5798;",
    "垙" => "&#x5799;",
    "垚" => "&#x579A;",
    "垛" => "&#x579B;",
    "垜" => "&#x579C;",
    "垝" => "&#x579D;",
    "垞" => "&#x579E;",
    "垟" => "&#x579F;",
    "垠" => "&#x57A0;",
    "垡" => "&#x57A1;",
    "垢" => "&#x57A2;",
    "垣" => "&#x57A3;",
    "垤" => "&#x57A4;",
    "垥" => "&#x57A5;",
    "垦" => "&#x57A6;",
    "垧" => "&#x57A7;",
    "垨" => "&#x57A8;",
    "垩" => "&#x57A9;",
    "垪" => "&#x57AA;",
    "垫" => "&#x57AB;",
    "垬" => "&#x57AC;",
    "垭" => "&#x57AD;",
    "垮" => "&#x57AE;",
    "垯" => "&#x57AF;",
    "垰" => "&#x57B0;",
    "垱" => "&#x57B1;",
    "垲" => "&#x57B2;",
    "垳" => "&#x57B3;",
    "垴" => "&#x57B4;",
    "垵" => "&#x57B5;",
    "垶" => "&#x57B6;",
    "垷" => "&#x57B7;",
    "垸" => "&#x57B8;",
    "垹" => "&#x57B9;",
    "垺" => "&#x57BA;",
    "垻" => "&#x57BB;",
    "垼" => "&#x57BC;",
    "垽" => "&#x57BD;",
    "垾" => "&#x57BE;",
    "垿" => "&#x57BF;",
    "埀" => "&#x57C0;",
    "埁" => "&#x57C1;",
    "埂" => "&#x57C2;",
    "埃" => "&#x57C3;",
    "埄" => "&#x57C4;",
    "埅" => "&#x57C5;",
    "埆" => "&#x57C6;",
    "埇" => "&#x57C7;",
    "埈" => "&#x57C8;",
    "埉" => "&#x57C9;",
    "埊" => "&#x57CA;",
    "埋" => "&#x57CB;",
    "埌" => "&#x57CC;",
    "埍" => "&#x57CD;",
    "城" => "&#x57CE;",
    "埏" => "&#x57CF;",
    "埐" => "&#x57D0;",
    "埑" => "&#x57D1;",
    "埒" => "&#x57D2;",
    "埓" => "&#x57D3;",
    "埔" => "&#x57D4;",
    "埕" => "&#x57D5;",
    "埖" => "&#x57D6;",
    "埗" => "&#x57D7;",
    "埘" => "&#x57D8;",
    "埙" => "&#x57D9;",
    "埚" => "&#x57DA;",
    "埛" => "&#x57DB;",
    "埜" => "&#x57DC;",
    "埝" => "&#x57DD;",
    "埞" => "&#x57DE;",
    "域" => "&#x57DF;",
    "埠" => "&#x57E0;",
    "埡" => "&#x57E1;",
    "埢" => "&#x57E2;",
    "埣" => "&#x57E3;",
    "埤" => "&#x57E4;",
    "埥" => "&#x57E5;",
    "埦" => "&#x57E6;",
    "埧" => "&#x57E7;",
    "埨" => "&#x57E8;",
    "埩" => "&#x57E9;",
    "埪" => "&#x57EA;",
    "埫" => "&#x57EB;",
    "埬" => "&#x57EC;",
    "埭" => "&#x57ED;",
    "埮" => "&#x57EE;",
    "埯" => "&#x57EF;",
    "埰" => "&#x57F0;",
    "埱" => "&#x57F1;",
    "埲" => "&#x57F2;",
    "埳" => "&#x57F3;",
    "埴" => "&#x57F4;",
    "埵" => "&#x57F5;",
    "埶" => "&#x57F6;",
    "執" => "&#x57F7;",
    "埸" => "&#x57F8;",
    "培" => "&#x57F9;",
    "基" => "&#x57FA;",
    "埻" => "&#x57FB;",
    "埼" => "&#x57FC;",
    "埽" => "&#x57FD;",
    "埾" => "&#x57FE;",
    "埿" => "&#x57FF;",
    "堀" => "&#x5800;",
    "堁" => "&#x5801;",
    "堂" => "&#x5802;",
    "堃" => "&#x5803;",
    "堄" => "&#x5804;",
    "堅" => "&#x5805;",
    "堆" => "&#x5806;",
    "堇" => "&#x5807;",
    "堈" => "&#x5808;",
    "堉" => "&#x5809;",
    "堊" => "&#x580A;",
    "堋" => "&#x580B;",
    "堌" => "&#x580C;",
    "堍" => "&#x580D;",
    "堎" => "&#x580E;",
    "堏" => "&#x580F;",
    "堐" => "&#x5810;",
    "堑" => "&#x5811;",
    "堒" => "&#x5812;",
    "堓" => "&#x5813;",
    "堔" => "&#x5814;",
    "堕" => "&#x5815;",
    "堖" => "&#x5816;",
    "堗" => "&#x5817;",
    "堘" => "&#x5818;",
    "堙" => "&#x5819;",
    "堚" => "&#x581A;",
    "堛" => "&#x581B;",
    "堜" => "&#x581C;",
    "堝" => "&#x581D;",
    "堞" => "&#x581E;",
    "堟" => "&#x581F;",
    "堠" => "&#x5820;",
    "堡" => "&#x5821;",
    "堢" => "&#x5822;",
    "堣" => "&#x5823;",
    "堤" => "&#x5824;",
    "堥" => "&#x5825;",
    "堦" => "&#x5826;",
    "堧" => "&#x5827;",
    "堨" => "&#x5828;",
    "堩" => "&#x5829;",
    "堪" => "&#x582A;",
    "堫" => "&#x582B;",
    "堬" => "&#x582C;",
    "堭" => "&#x582D;",
    "堮" => "&#x582E;",
    "堯" => "&#x582F;",
    "堰" => "&#x5830;",
    "報" => "&#x5831;",
    "堲" => "&#x5832;",
    "堳" => "&#x5833;",
    "場" => "&#x5834;",
    "堵" => "&#x5835;",
    "堶" => "&#x5836;",
    "堷" => "&#x5837;",
    "堸" => "&#x5838;",
    "堹" => "&#x5839;",
    "堺" => "&#x583A;",
    "堻" => "&#x583B;",
    "堼" => "&#x583C;",
    "堽" => "&#x583D;",
    "堾" => "&#x583E;",
    "堿" => "&#x583F;",
    "塀" => "&#x5840;",
    "塁" => "&#x5841;",
    "塂" => "&#x5842;",
    "塃" => "&#x5843;",
    "塄" => "&#x5844;",
    "塅" => "&#x5845;",
    "塆" => "&#x5846;",
    "塇" => "&#x5847;",
    "塈" => "&#x5848;",
    "塉" => "&#x5849;",
    "塊" => "&#x584A;",
    "塋" => "&#x584B;",
    "塌" => "&#x584C;",
    "塍" => "&#x584D;",
    "塎" => "&#x584E;",
    "塏" => "&#x584F;",
    "塐" => "&#x5850;",
    "塑" => "&#x5851;",
    "塒" => "&#x5852;",
    "塓" => "&#x5853;",
    "塔" => "&#x5854;",
    "塕" => "&#x5855;",
    "塖" => "&#x5856;",
    "塗" => "&#x5857;",
    "塘" => "&#x5858;",
    "塙" => "&#x5859;",
    "塚" => "&#x585A;",
    "塛" => "&#x585B;",
    "塜" => "&#x585C;",
    "塝" => "&#x585D;",
    "塞" => "&#x585E;",
    "塟" => "&#x585F;",
    "塠" => "&#x5860;",
    "塡" => "&#x5861;",
    "塢" => "&#x5862;",
    "塣" => "&#x5863;",
    "塤" => "&#x5864;",
    "塥" => "&#x5865;",
    "塦" => "&#x5866;",
    "塧" => "&#x5867;",
    "塨" => "&#x5868;",
    "塩" => "&#x5869;",
    "塪" => "&#x586A;",
    "填" => "&#x586B;",
    "塬" => "&#x586C;",
    "塭" => "&#x586D;",
    "塮" => "&#x586E;",
    "塯" => "&#x586F;",
    "塰" => "&#x5870;",
    "塱" => "&#x5871;",
    "塲" => "&#x5872;",
    "塳" => "&#x5873;",
    "塴" => "&#x5874;",
    "塵" => "&#x5875;",
    "塶" => "&#x5876;",
    "塷" => "&#x5877;",
    "塸" => "&#x5878;",
    "塹" => "&#x5879;",
    "塺" => "&#x587A;",
    "塻" => "&#x587B;",
    "塼" => "&#x587C;",
    "塽" => "&#x587D;",
    "塾" => "&#x587E;",
    "塿" => "&#x587F;",
    "墀" => "&#x5880;",
    "墁" => "&#x5881;",
    "墂" => "&#x5882;",
    "境" => "&#x5883;",
    "墄" => "&#x5884;",
    "墅" => "&#x5885;",
    "墆" => "&#x5886;",
    "墇" => "&#x5887;",
    "墈" => "&#x5888;",
    "墉" => "&#x5889;",
    "墊" => "&#x588A;",
    "墋" => "&#x588B;",
    "墌" => "&#x588C;",
    "墍" => "&#x588D;",
    "墎" => "&#x588E;",
    "墏" => "&#x588F;",
    "墐" => "&#x5890;",
    "墑" => "&#x5891;",
    "墒" => "&#x5892;",
    "墓" => "&#x5893;",
    "墔" => "&#x5894;",
    "墕" => "&#x5895;",
    "墖" => "&#x5896;",
    "増" => "&#x5897;",
    "墘" => "&#x5898;",
    "墙" => "&#x5899;",
    "墚" => "&#x589A;",
    "墛" => "&#x589B;",
    "墜" => "&#x589C;",
    "墝" => "&#x589D;",
    "增" => "&#x589E;",
    "墟" => "&#x589F;",
    "墠" => "&#x58A0;",
    "墡" => "&#x58A1;",
    "墢" => "&#x58A2;",
    "墣" => "&#x58A3;",
    "墤" => "&#x58A4;",
    "墥" => "&#x58A5;",
    "墦" => "&#x58A6;",
    "墧" => "&#x58A7;",
    "墨" => "&#x58A8;",
    "墩" => "&#x58A9;",
    "墪" => "&#x58AA;",
    "墫" => "&#x58AB;",
    "墬" => "&#x58AC;",
    "墭" => "&#x58AD;",
    "墮" => "&#x58AE;",
    "墯" => "&#x58AF;",
    "墰" => "&#x58B0;",
    "墱" => "&#x58B1;",
    "墲" => "&#x58B2;",
    "墳" => "&#x58B3;",
    "墴" => "&#x58B4;",
    "墵" => "&#x58B5;",
    "墶" => "&#x58B6;",
    "墷" => "&#x58B7;",
    "墸" => "&#x58B8;",
    "墹" => "&#x58B9;",
    "墺" => "&#x58BA;",
    "墻" => "&#x58BB;",
    "墼" => "&#x58BC;",
    "墽" => "&#x58BD;",
    "墾" => "&#x58BE;",
    "墿" => "&#x58BF;",
    "壀" => "&#x58C0;",
    "壁" => "&#x58C1;",
    "壂" => "&#x58C2;",
    "壃" => "&#x58C3;",
    "壄" => "&#x58C4;",
    "壅" => "&#x58C5;",
    "壆" => "&#x58C6;",
    "壇" => "&#x58C7;",
    "壈" => "&#x58C8;",
    "壉" => "&#x58C9;",
    "壊" => "&#x58CA;",
    "壋" => "&#x58CB;",
    "壌" => "&#x58CC;",
    "壍" => "&#x58CD;",
    "壎" => "&#x58CE;",
    "壏" => "&#x58CF;",
    "壐" => "&#x58D0;",
    "壑" => "&#x58D1;",
    "壒" => "&#x58D2;",
    "壓" => "&#x58D3;",
    "壔" => "&#x58D4;",
    "壕" => "&#x58D5;",
    "壖" => "&#x58D6;",
    "壗" => "&#x58D7;",
    "壘" => "&#x58D8;",
    "壙" => "&#x58D9;",
    "壚" => "&#x58DA;",
    "壛" => "&#x58DB;",
    "壜" => "&#x58DC;",
    "壝" => "&#x58DD;",
    "壞" => "&#x58DE;",
    "壟" => "&#x58DF;",
    "壠" => "&#x58E0;",
    "壡" => "&#x58E1;",
    "壢" => "&#x58E2;",
    "壣" => "&#x58E3;",
    "壤" => "&#x58E4;",
    "壥" => "&#x58E5;",
    "壦" => "&#x58E6;",
    "壧" => "&#x58E7;",
    "壨" => "&#x58E8;",
    "壩" => "&#x58E9;",
    "壪" => "&#x58EA;",
    "士" => "&#x58EB;",
    "壬" => "&#x58EC;",
    "壭" => "&#x58ED;",
    "壮" => "&#x58EE;",
    "壯" => "&#x58EF;",
    "声" => "&#x58F0;",
    "壱" => "&#x58F1;",
    "売" => "&#x58F2;",
    "壳" => "&#x58F3;",
    "壴" => "&#x58F4;",
    "壵" => "&#x58F5;",
    "壶" => "&#x58F6;",
    "壷" => "&#x58F7;",
    "壸" => "&#x58F8;",
    "壹" => "&#x58F9;",
    "壺" => "&#x58FA;",
    "壻" => "&#x58FB;",
    "壼" => "&#x58FC;",
    "壽" => "&#x58FD;",
    "壾" => "&#x58FE;",
    "壿" => "&#x58FF;",
    "夀" => "&#x5900;",
    "夁" => "&#x5901;",
    "夂" => "&#x5902;",
    "夃" => "&#x5903;",
    "处" => "&#x5904;",
    "夅" => "&#x5905;",
    "夆" => "&#x5906;",
    "备" => "&#x5907;",
    "夈" => "&#x5908;",
    "変" => "&#x5909;",
    "夊" => "&#x590A;",
    "夋" => "&#x590B;",
    "夌" => "&#x590C;",
    "复" => "&#x590D;",
    "夎" => "&#x590E;",
    "夏" => "&#x590F;",
    "夐" => "&#x5910;",
    "夑" => "&#x5911;",
    "夒" => "&#x5912;",
    "夓" => "&#x5913;",
    "夔" => "&#x5914;",
    "夕" => "&#x5915;",
    "外" => "&#x5916;",
    "夗" => "&#x5917;",
    "夘" => "&#x5918;",
    "夙" => "&#x5919;",
    "多" => "&#x591A;",
    "夛" => "&#x591B;",
    "夜" => "&#x591C;",
    "夝" => "&#x591D;",
    "夞" => "&#x591E;",
    "够" => "&#x591F;",
    "夠" => "&#x5920;",
    "夡" => "&#x5921;",
    "夢" => "&#x5922;",
    "夣" => "&#x5923;",
    "夤" => "&#x5924;",
    "夥" => "&#x5925;",
    "夦" => "&#x5926;",
    "大" => "&#x5927;",
    "夨" => "&#x5928;",
    "天" => "&#x5929;",
    "太" => "&#x592A;",
    "夫" => "&#x592B;",
    "夬" => "&#x592C;",
    "夭" => "&#x592D;",
    "央" => "&#x592E;",
    "夯" => "&#x592F;",
    "夰" => "&#x5930;",
    "失" => "&#x5931;",
    "夲" => "&#x5932;",
    "夳" => "&#x5933;",
    "头" => "&#x5934;",
    "夵" => "&#x5935;",
    "夶" => "&#x5936;",
    "夷" => "&#x5937;",
    "夸" => "&#x5938;",
    "夹" => "&#x5939;",
    "夺" => "&#x593A;",
    "夻" => "&#x593B;",
    "夼" => "&#x593C;",
    "夽" => "&#x593D;",
    "夾" => "&#x593E;",
    "夿" => "&#x593F;",
    "奀" => "&#x5940;",
    "奁" => "&#x5941;",
    "奂" => "&#x5942;",
    "奃" => "&#x5943;",
    "奄" => "&#x5944;",
    "奅" => "&#x5945;",
    "奆" => "&#x5946;",
    "奇" => "&#x5947;",
    "奈" => "&#x5948;",
    "奉" => "&#x5949;",
    "奊" => "&#x594A;",
    "奋" => "&#x594B;",
    "奌" => "&#x594C;",
    "奍" => "&#x594D;",
    "奎" => "&#x594E;",
    "奏" => "&#x594F;",
    "奐" => "&#x5950;",
    "契" => "&#x5951;",
    "奒" => "&#x5952;",
    "奓" => "&#x5953;",
    "奔" => "&#x5954;",
    "奕" => "&#x5955;",
    "奖" => "&#x5956;",
    "套" => "&#x5957;",
    "奘" => "&#x5958;",
    "奙" => "&#x5959;",
    "奚" => "&#x595A;",
    "奛" => "&#x595B;",
    "奜" => "&#x595C;",
    "奝" => "&#x595D;",
    "奞" => "&#x595E;",
    "奟" => "&#x595F;",
    "奠" => "&#x5960;",
    "奡" => "&#x5961;",
    "奢" => "&#x5962;",
    "奣" => "&#x5963;",
    "奤" => "&#x5964;",
    "奥" => "&#x5965;",
    "奦" => "&#x5966;",
    "奧" => "&#x5967;",
    "奨" => "&#x5968;",
    "奩" => "&#x5969;",
    "奪" => "&#x596A;",
    "奫" => "&#x596B;",
    "奬" => "&#x596C;",
    "奭" => "&#x596D;",
    "奮" => "&#x596E;",
    "奯" => "&#x596F;",
    "奰" => "&#x5970;",
    "奱" => "&#x5971;",
    "奲" => "&#x5972;",
    "女" => "&#x5973;",
    "奴" => "&#x5974;",
    "奵" => "&#x5975;",
    "奶" => "&#x5976;",
    "奷" => "&#x5977;",
    "奸" => "&#x5978;",
    "她" => "&#x5979;",
    "奺" => "&#x597A;",
    "奻" => "&#x597B;",
    "奼" => "&#x597C;",
    "好" => "&#x597D;",
    "奾" => "&#x597E;",
    "奿" => "&#x597F;",
    "妀" => "&#x5980;",
    "妁" => "&#x5981;",
    "如" => "&#x5982;",
    "妃" => "&#x5983;",
    "妄" => "&#x5984;",
    "妅" => "&#x5985;",
    "妆" => "&#x5986;",
    "妇" => "&#x5987;",
    "妈" => "&#x5988;",
    "妉" => "&#x5989;",
    "妊" => "&#x598A;",
    "妋" => "&#x598B;",
    "妌" => "&#x598C;",
    "妍" => "&#x598D;",
    "妎" => "&#x598E;",
    "妏" => "&#x598F;",
    "妐" => "&#x5990;",
    "妑" => "&#x5991;",
    "妒" => "&#x5992;",
    "妓" => "&#x5993;",
    "妔" => "&#x5994;",
    "妕" => "&#x5995;",
    "妖" => "&#x5996;",
    "妗" => "&#x5997;",
    "妘" => "&#x5998;",
    "妙" => "&#x5999;",
    "妚" => "&#x599A;",
    "妛" => "&#x599B;",
    "妜" => "&#x599C;",
    "妝" => "&#x599D;",
    "妞" => "&#x599E;",
    "妟" => "&#x599F;",
    "妠" => "&#x59A0;",
    "妡" => "&#x59A1;",
    "妢" => "&#x59A2;",
    "妣" => "&#x59A3;",
    "妤" => "&#x59A4;",
    "妥" => "&#x59A5;",
    "妦" => "&#x59A6;",
    "妧" => "&#x59A7;",
    "妨" => "&#x59A8;",
    "妩" => "&#x59A9;",
    "妪" => "&#x59AA;",
    "妫" => "&#x59AB;",
    "妬" => "&#x59AC;",
    "妭" => "&#x59AD;",
    "妮" => "&#x59AE;",
    "妯" => "&#x59AF;",
    "妰" => "&#x59B0;",
    "妱" => "&#x59B1;",
    "妲" => "&#x59B2;",
    "妳" => "&#x59B3;",
    "妴" => "&#x59B4;",
    "妵" => "&#x59B5;",
    "妶" => "&#x59B6;",
    "妷" => "&#x59B7;",
    "妸" => "&#x59B8;",
    "妹" => "&#x59B9;",
    "妺" => "&#x59BA;",
    "妻" => "&#x59BB;",
    "妼" => "&#x59BC;",
    "妽" => "&#x59BD;",
    "妾" => "&#x59BE;",
    "妿" => "&#x59BF;",
    "姀" => "&#x59C0;",
    "姁" => "&#x59C1;",
    "姂" => "&#x59C2;",
    "姃" => "&#x59C3;",
    "姄" => "&#x59C4;",
    "姅" => "&#x59C5;",
    "姆" => "&#x59C6;",
    "姇" => "&#x59C7;",
    "姈" => "&#x59C8;",
    "姉" => "&#x59C9;",
    "姊" => "&#x59CA;",
    "始" => "&#x59CB;",
    "姌" => "&#x59CC;",
    "姍" => "&#x59CD;",
    "姎" => "&#x59CE;",
    "姏" => "&#x59CF;",
    "姐" => "&#x59D0;",
    "姑" => "&#x59D1;",
    "姒" => "&#x59D2;",
    "姓" => "&#x59D3;",
    "委" => "&#x59D4;",
    "姕" => "&#x59D5;",
    "姖" => "&#x59D6;",
    "姗" => "&#x59D7;",
    "姘" => "&#x59D8;",
    "姙" => "&#x59D9;",
    "姚" => "&#x59DA;",
    "姛" => "&#x59DB;",
    "姜" => "&#x59DC;",
    "姝" => "&#x59DD;",
    "姞" => "&#x59DE;",
    "姟" => "&#x59DF;",
    "姠" => "&#x59E0;",
    "姡" => "&#x59E1;",
    "姢" => "&#x59E2;",
    "姣" => "&#x59E3;",
    "姤" => "&#x59E4;",
    "姥" => "&#x59E5;",
    "姦" => "&#x59E6;",
    "姧" => "&#x59E7;",
    "姨" => "&#x59E8;",
    "姩" => "&#x59E9;",
    "姪" => "&#x59EA;",
    "姫" => "&#x59EB;",
    "姬" => "&#x59EC;",
    "姭" => "&#x59ED;",
    "姮" => "&#x59EE;",
    "姯" => "&#x59EF;",
    "姰" => "&#x59F0;",
    "姱" => "&#x59F1;",
    "姲" => "&#x59F2;",
    "姳" => "&#x59F3;",
    "姴" => "&#x59F4;",
    "姵" => "&#x59F5;",
    "姶" => "&#x59F6;",
    "姷" => "&#x59F7;",
    "姸" => "&#x59F8;",
    "姹" => "&#x59F9;",
    "姺" => "&#x59FA;",
    "姻" => "&#x59FB;",
    "姼" => "&#x59FC;",
    "姽" => "&#x59FD;",
    "姾" => "&#x59FE;",
    "姿" => "&#x59FF;",
    "娀" => "&#x5A00;",
    "威" => "&#x5A01;",
    "娂" => "&#x5A02;",
    "娃" => "&#x5A03;",
    "娄" => "&#x5A04;",
    "娅" => "&#x5A05;",
    "娆" => "&#x5A06;",
    "娇" => "&#x5A07;",
    "娈" => "&#x5A08;",
    "娉" => "&#x5A09;",
    "娊" => "&#x5A0A;",
    "娋" => "&#x5A0B;",
    "娌" => "&#x5A0C;",
    "娍" => "&#x5A0D;",
    "娎" => "&#x5A0E;",
    "娏" => "&#x5A0F;",
    "娐" => "&#x5A10;",
    "娑" => "&#x5A11;",
    "娒" => "&#x5A12;",
    "娓" => "&#x5A13;",
    "娔" => "&#x5A14;",
    "娕" => "&#x5A15;",
    "娖" => "&#x5A16;",
    "娗" => "&#x5A17;",
    "娘" => "&#x5A18;",
    "娙" => "&#x5A19;",
    "娚" => "&#x5A1A;",
    "娛" => "&#x5A1B;",
    "娜" => "&#x5A1C;",
    "娝" => "&#x5A1D;",
    "娞" => "&#x5A1E;",
    "娟" => "&#x5A1F;",
    "娠" => "&#x5A20;",
    "娡" => "&#x5A21;",
    "娢" => "&#x5A22;",
    "娣" => "&#x5A23;",
    "娤" => "&#x5A24;",
    "娥" => "&#x5A25;",
    "娦" => "&#x5A26;",
    "娧" => "&#x5A27;",
    "娨" => "&#x5A28;",
    "娩" => "&#x5A29;",
    "娪" => "&#x5A2A;",
    "娫" => "&#x5A2B;",
    "娬" => "&#x5A2C;",
    "娭" => "&#x5A2D;",
    "娮" => "&#x5A2E;",
    "娯" => "&#x5A2F;",
    "娰" => "&#x5A30;",
    "娱" => "&#x5A31;",
    "娲" => "&#x5A32;",
    "娳" => "&#x5A33;",
    "娴" => "&#x5A34;",
    "娵" => "&#x5A35;",
    "娶" => "&#x5A36;",
    "娷" => "&#x5A37;",
    "娸" => "&#x5A38;",
    "娹" => "&#x5A39;",
    "娺" => "&#x5A3A;",
    "娻" => "&#x5A3B;",
    "娼" => "&#x5A3C;",
    "娽" => "&#x5A3D;",
    "娾" => "&#x5A3E;",
    "娿" => "&#x5A3F;",
    "婀" => "&#x5A40;",
    "婁" => "&#x5A41;",
    "婂" => "&#x5A42;",
    "婃" => "&#x5A43;",
    "婄" => "&#x5A44;",
    "婅" => "&#x5A45;",
    "婆" => "&#x5A46;",
    "婇" => "&#x5A47;",
    "婈" => "&#x5A48;",
    "婉" => "&#x5A49;",
    "婊" => "&#x5A4A;",
    "婋" => "&#x5A4B;",
    "婌" => "&#x5A4C;",
    "婍" => "&#x5A4D;",
    "婎" => "&#x5A4E;",
    "婏" => "&#x5A4F;",
    "婐" => "&#x5A50;",
    "婑" => "&#x5A51;",
    "婒" => "&#x5A52;",
    "婓" => "&#x5A53;",
    "婔" => "&#x5A54;",
    "婕" => "&#x5A55;",
    "婖" => "&#x5A56;",
    "婗" => "&#x5A57;",
    "婘" => "&#x5A58;",
    "婙" => "&#x5A59;",
    "婚" => "&#x5A5A;",
    "婛" => "&#x5A5B;",
    "婜" => "&#x5A5C;",
    "婝" => "&#x5A5D;",
    "婞" => "&#x5A5E;",
    "婟" => "&#x5A5F;",
    "婠" => "&#x5A60;",
    "婡" => "&#x5A61;",
    "婢" => "&#x5A62;",
    "婣" => "&#x5A63;",
    "婤" => "&#x5A64;",
    "婥" => "&#x5A65;",
    "婦" => "&#x5A66;",
    "婧" => "&#x5A67;",
    "婨" => "&#x5A68;",
    "婩" => "&#x5A69;",
    "婪" => "&#x5A6A;",
    "婫" => "&#x5A6B;",
    "婬" => "&#x5A6C;",
    "婭" => "&#x5A6D;",
    "婮" => "&#x5A6E;",
    "婯" => "&#x5A6F;",
    "婰" => "&#x5A70;",
    "婱" => "&#x5A71;",
    "婲" => "&#x5A72;",
    "婳" => "&#x5A73;",
    "婴" => "&#x5A74;",
    "婵" => "&#x5A75;",
    "婶" => "&#x5A76;",
    "婷" => "&#x5A77;",
    "婸" => "&#x5A78;",
    "婹" => "&#x5A79;",
    "婺" => "&#x5A7A;",
    "婻" => "&#x5A7B;",
    "婼" => "&#x5A7C;",
    "婽" => "&#x5A7D;",
    "婾" => "&#x5A7E;",
    "婿" => "&#x5A7F;",
    "媀" => "&#x5A80;",
    "媁" => "&#x5A81;",
    "媂" => "&#x5A82;",
    "媃" => "&#x5A83;",
    "媄" => "&#x5A84;",
    "媅" => "&#x5A85;",
    "媆" => "&#x5A86;",
    "媇" => "&#x5A87;",
    "媈" => "&#x5A88;",
    "媉" => "&#x5A89;",
    "媊" => "&#x5A8A;",
    "媋" => "&#x5A8B;",
    "媌" => "&#x5A8C;",
    "媍" => "&#x5A8D;",
    "媎" => "&#x5A8E;",
    "媏" => "&#x5A8F;",
    "媐" => "&#x5A90;",
    "媑" => "&#x5A91;",
    "媒" => "&#x5A92;",
    "媓" => "&#x5A93;",
    "媔" => "&#x5A94;",
    "媕" => "&#x5A95;",
    "媖" => "&#x5A96;",
    "媗" => "&#x5A97;",
    "媘" => "&#x5A98;",
    "媙" => "&#x5A99;",
    "媚" => "&#x5A9A;",
    "媛" => "&#x5A9B;",
    "媜" => "&#x5A9C;",
    "媝" => "&#x5A9D;",
    "媞" => "&#x5A9E;",
    "媟" => "&#x5A9F;",
    "媠" => "&#x5AA0;",
    "媡" => "&#x5AA1;",
    "媢" => "&#x5AA2;",
    "媣" => "&#x5AA3;",
    "媤" => "&#x5AA4;",
    "媥" => "&#x5AA5;",
    "媦" => "&#x5AA6;",
    "媧" => "&#x5AA7;",
    "媨" => "&#x5AA8;",
    "媩" => "&#x5AA9;",
    "媪" => "&#x5AAA;",
    "媫" => "&#x5AAB;",
    "媬" => "&#x5AAC;",
    "媭" => "&#x5AAD;",
    "媮" => "&#x5AAE;",
    "媯" => "&#x5AAF;",
    "媰" => "&#x5AB0;",
    "媱" => "&#x5AB1;",
    "媲" => "&#x5AB2;",
    "媳" => "&#x5AB3;",
    "媴" => "&#x5AB4;",
    "媵" => "&#x5AB5;",
    "媶" => "&#x5AB6;",
    "媷" => "&#x5AB7;",
    "媸" => "&#x5AB8;",
    "媹" => "&#x5AB9;",
    "媺" => "&#x5ABA;",
    "媻" => "&#x5ABB;",
    "媼" => "&#x5ABC;",
    "媽" => "&#x5ABD;",
    "媾" => "&#x5ABE;",
    "媿" => "&#x5ABF;",
    "嫀" => "&#x5AC0;",
    "嫁" => "&#x5AC1;",
    "嫂" => "&#x5AC2;",
    "嫃" => "&#x5AC3;",
    "嫄" => "&#x5AC4;",
    "嫅" => "&#x5AC5;",
    "嫆" => "&#x5AC6;",
    "嫇" => "&#x5AC7;",
    "嫈" => "&#x5AC8;",
    "嫉" => "&#x5AC9;",
    "嫊" => "&#x5ACA;",
    "嫋" => "&#x5ACB;",
    "嫌" => "&#x5ACC;",
    "嫍" => "&#x5ACD;",
    "嫎" => "&#x5ACE;",
    "嫏" => "&#x5ACF;",
    "嫐" => "&#x5AD0;",
    "嫑" => "&#x5AD1;",
    "嫒" => "&#x5AD2;",
    "嫓" => "&#x5AD3;",
    "嫔" => "&#x5AD4;",
    "嫕" => "&#x5AD5;",
    "嫖" => "&#x5AD6;",
    "嫗" => "&#x5AD7;",
    "嫘" => "&#x5AD8;",
    "嫙" => "&#x5AD9;",
    "嫚" => "&#x5ADA;",
    "嫛" => "&#x5ADB;",
    "嫜" => "&#x5ADC;",
    "嫝" => "&#x5ADD;",
    "嫞" => "&#x5ADE;",
    "嫟" => "&#x5ADF;",
    "嫠" => "&#x5AE0;",
    "嫡" => "&#x5AE1;",
    "嫢" => "&#x5AE2;",
    "嫣" => "&#x5AE3;",
    "嫤" => "&#x5AE4;",
    "嫥" => "&#x5AE5;",
    "嫦" => "&#x5AE6;",
    "嫧" => "&#x5AE7;",
    "嫨" => "&#x5AE8;",
    "嫩" => "&#x5AE9;",
    "嫪" => "&#x5AEA;",
    "嫫" => "&#x5AEB;",
    "嫬" => "&#x5AEC;",
    "嫭" => "&#x5AED;",
    "嫮" => "&#x5AEE;",
    "嫯" => "&#x5AEF;",
    "嫰" => "&#x5AF0;",
    "嫱" => "&#x5AF1;",
    "嫲" => "&#x5AF2;",
    "嫳" => "&#x5AF3;",
    "嫴" => "&#x5AF4;",
    "嫵" => "&#x5AF5;",
    "嫶" => "&#x5AF6;",
    "嫷" => "&#x5AF7;",
    "嫸" => "&#x5AF8;",
    "嫹" => "&#x5AF9;",
    "嫺" => "&#x5AFA;",
    "嫻" => "&#x5AFB;",
    "嫼" => "&#x5AFC;",
    "嫽" => "&#x5AFD;",
    "嫾" => "&#x5AFE;",
    "嫿" => "&#x5AFF;",
    "嬀" => "&#x5B00;",
    "嬁" => "&#x5B01;",
    "嬂" => "&#x5B02;",
    "嬃" => "&#x5B03;",
    "嬄" => "&#x5B04;",
    "嬅" => "&#x5B05;",
    "嬆" => "&#x5B06;",
    "嬇" => "&#x5B07;",
    "嬈" => "&#x5B08;",
    "嬉" => "&#x5B09;",
    "嬊" => "&#x5B0A;",
    "嬋" => "&#x5B0B;",
    "嬌" => "&#x5B0C;",
    "嬍" => "&#x5B0D;",
    "嬎" => "&#x5B0E;",
    "嬏" => "&#x5B0F;",
    "嬐" => "&#x5B10;",
    "嬑" => "&#x5B11;",
    "嬒" => "&#x5B12;",
    "嬓" => "&#x5B13;",
    "嬔" => "&#x5B14;",
    "嬕" => "&#x5B15;",
    "嬖" => "&#x5B16;",
    "嬗" => "&#x5B17;",
    "嬘" => "&#x5B18;",
    "嬙" => "&#x5B19;",
    "嬚" => "&#x5B1A;",
    "嬛" => "&#x5B1B;",
    "嬜" => "&#x5B1C;",
    "嬝" => "&#x5B1D;",
    "嬞" => "&#x5B1E;",
    "嬟" => "&#x5B1F;",
    "嬠" => "&#x5B20;",
    "嬡" => "&#x5B21;",
    "嬢" => "&#x5B22;",
    "嬣" => "&#x5B23;",
    "嬤" => "&#x5B24;",
    "嬥" => "&#x5B25;",
    "嬦" => "&#x5B26;",
    "嬧" => "&#x5B27;",
    "嬨" => "&#x5B28;",
    "嬩" => "&#x5B29;",
    "嬪" => "&#x5B2A;",
    "嬫" => "&#x5B2B;",
    "嬬" => "&#x5B2C;",
    "嬭" => "&#x5B2D;",
    "嬮" => "&#x5B2E;",
    "嬯" => "&#x5B2F;",
    "嬰" => "&#x5B30;",
    "嬱" => "&#x5B31;",
    "嬲" => "&#x5B32;",
    "嬳" => "&#x5B33;",
    "嬴" => "&#x5B34;",
    "嬵" => "&#x5B35;",
    "嬶" => "&#x5B36;",
    "嬷" => "&#x5B37;",
    "嬸" => "&#x5B38;",
    "嬹" => "&#x5B39;",
    "嬺" => "&#x5B3A;",
    "嬻" => "&#x5B3B;",
    "嬼" => "&#x5B3C;",
    "嬽" => "&#x5B3D;",
    "嬾" => "&#x5B3E;",
    "嬿" => "&#x5B3F;",
    "孀" => "&#x5B40;",
    "孁" => "&#x5B41;",
    "孂" => "&#x5B42;",
    "孃" => "&#x5B43;",
    "孄" => "&#x5B44;",
    "孅" => "&#x5B45;",
    "孆" => "&#x5B46;",
    "孇" => "&#x5B47;",
    "孈" => "&#x5B48;",
    "孉" => "&#x5B49;",
    "孊" => "&#x5B4A;",
    "孋" => "&#x5B4B;",
    "孌" => "&#x5B4C;",
    "孍" => "&#x5B4D;",
    "孎" => "&#x5B4E;",
    "孏" => "&#x5B4F;",
    "子" => "&#x5B50;",
    "孑" => "&#x5B51;",
    "孒" => "&#x5B52;",
    "孓" => "&#x5B53;",
    "孔" => "&#x5B54;",
    "孕" => "&#x5B55;",
    "孖" => "&#x5B56;",
    "字" => "&#x5B57;",
    "存" => "&#x5B58;",
    "孙" => "&#x5B59;",
    "孚" => "&#x5B5A;",
    "孛" => "&#x5B5B;",
    "孜" => "&#x5B5C;",
    "孝" => "&#x5B5D;",
    "孞" => "&#x5B5E;",
    "孟" => "&#x5B5F;",
    "孠" => "&#x5B60;",
    "孡" => "&#x5B61;",
    "孢" => "&#x5B62;",
    "季" => "&#x5B63;",
    "孤" => "&#x5B64;",
    "孥" => "&#x5B65;",
    "学" => "&#x5B66;",
    "孧" => "&#x5B67;",
    "孨" => "&#x5B68;",
    "孩" => "&#x5B69;",
    "孪" => "&#x5B6A;",
    "孫" => "&#x5B6B;",
    "孬" => "&#x5B6C;",
    "孭" => "&#x5B6D;",
    "孮" => "&#x5B6E;",
    "孯" => "&#x5B6F;",
    "孰" => "&#x5B70;",
    "孱" => "&#x5B71;",
    "孲" => "&#x5B72;",
    "孳" => "&#x5B73;",
    "孴" => "&#x5B74;",
    "孵" => "&#x5B75;",
    "孶" => "&#x5B76;",
    "孷" => "&#x5B77;",
    "學" => "&#x5B78;",
    "孹" => "&#x5B79;",
    "孺" => "&#x5B7A;",
    "孻" => "&#x5B7B;",
    "孼" => "&#x5B7C;",
    "孽" => "&#x5B7D;",
    "孾" => "&#x5B7E;",
    "孿" => "&#x5B7F;",
    "宀" => "&#x5B80;",
    "宁" => "&#x5B81;",
    "宂" => "&#x5B82;",
    "它" => "&#x5B83;",
    "宄" => "&#x5B84;",
    "宅" => "&#x5B85;",
    "宆" => "&#x5B86;",
    "宇" => "&#x5B87;",
    "守" => "&#x5B88;",
    "安" => "&#x5B89;",
    "宊" => "&#x5B8A;",
    "宋" => "&#x5B8B;",
    "完" => "&#x5B8C;",
    "宍" => "&#x5B8D;",
    "宎" => "&#x5B8E;",
    "宏" => "&#x5B8F;",
    "宐" => "&#x5B90;",
    "宑" => "&#x5B91;",
    "宒" => "&#x5B92;",
    "宓" => "&#x5B93;",
    "宔" => "&#x5B94;",
    "宕" => "&#x5B95;",
    "宖" => "&#x5B96;",
    "宗" => "&#x5B97;",
    "官" => "&#x5B98;",
    "宙" => "&#x5B99;",
    "定" => "&#x5B9A;",
    "宛" => "&#x5B9B;",
    "宜" => "&#x5B9C;",
    "宝" => "&#x5B9D;",
    "实" => "&#x5B9E;",
    "実" => "&#x5B9F;",
    "宠" => "&#x5BA0;",
    "审" => "&#x5BA1;",
    "客" => "&#x5BA2;",
    "宣" => "&#x5BA3;",
    "室" => "&#x5BA4;",
    "宥" => "&#x5BA5;",
    "宦" => "&#x5BA6;",
    "宧" => "&#x5BA7;",
    "宨" => "&#x5BA8;",
    "宩" => "&#x5BA9;",
    "宪" => "&#x5BAA;",
    "宫" => "&#x5BAB;",
    "宬" => "&#x5BAC;",
    "宭" => "&#x5BAD;",
    "宮" => "&#x5BAE;",
    "宯" => "&#x5BAF;",
    "宰" => "&#x5BB0;",
    "宱" => "&#x5BB1;",
    "宲" => "&#x5BB2;",
    "害" => "&#x5BB3;",
    "宴" => "&#x5BB4;",
    "宵" => "&#x5BB5;",
    "家" => "&#x5BB6;",
    "宷" => "&#x5BB7;",
    "宸" => "&#x5BB8;",
    "容" => "&#x5BB9;",
    "宺" => "&#x5BBA;",
    "宻" => "&#x5BBB;",
    "宼" => "&#x5BBC;",
    "宽" => "&#x5BBD;",
    "宾" => "&#x5BBE;",
    "宿" => "&#x5BBF;",
    "寀" => "&#x5BC0;",
    "寁" => "&#x5BC1;",
    "寂" => "&#x5BC2;",
    "寃" => "&#x5BC3;",
    "寄" => "&#x5BC4;",
    "寅" => "&#x5BC5;",
    "密" => "&#x5BC6;",
    "寇" => "&#x5BC7;",
    "寈" => "&#x5BC8;",
    "寉" => "&#x5BC9;",
    "寊" => "&#x5BCA;",
    "寋" => "&#x5BCB;",
    "富" => "&#x5BCC;",
    "寍" => "&#x5BCD;",
    "寎" => "&#x5BCE;",
    "寏" => "&#x5BCF;",
    "寐" => "&#x5BD0;",
    "寑" => "&#x5BD1;",
    "寒" => "&#x5BD2;",
    "寓" => "&#x5BD3;",
    "寔" => "&#x5BD4;",
    "寕" => "&#x5BD5;",
    "寖" => "&#x5BD6;",
    "寗" => "&#x5BD7;",
    "寘" => "&#x5BD8;",
    "寙" => "&#x5BD9;",
    "寚" => "&#x5BDA;",
    "寛" => "&#x5BDB;",
    "寜" => "&#x5BDC;",
    "寝" => "&#x5BDD;",
    "寞" => "&#x5BDE;",
    "察" => "&#x5BDF;",
    "寠" => "&#x5BE0;",
    "寡" => "&#x5BE1;",
    "寢" => "&#x5BE2;",
    "寣" => "&#x5BE3;",
    "寤" => "&#x5BE4;",
    "寥" => "&#x5BE5;",
    "實" => "&#x5BE6;",
    "寧" => "&#x5BE7;",
    "寨" => "&#x5BE8;",
    "審" => "&#x5BE9;",
    "寪" => "&#x5BEA;",
    "寫" => "&#x5BEB;",
    "寬" => "&#x5BEC;",
    "寭" => "&#x5BED;",
    "寮" => "&#x5BEE;",
    "寯" => "&#x5BEF;",
    "寰" => "&#x5BF0;",
    "寱" => "&#x5BF1;",
    "寲" => "&#x5BF2;",
    "寳" => "&#x5BF3;",
    "寴" => "&#x5BF4;",
    "寵" => "&#x5BF5;",
    "寶" => "&#x5BF6;",
    "寷" => "&#x5BF7;",
    "寸" => "&#x5BF8;",
    "对" => "&#x5BF9;",
    "寺" => "&#x5BFA;",
    "寻" => "&#x5BFB;",
    "导" => "&#x5BFC;",
    "寽" => "&#x5BFD;",
    "対" => "&#x5BFE;",
    "寿" => "&#x5BFF;",
    "尀" => "&#x5C00;",
    "封" => "&#x5C01;",
    "専" => "&#x5C02;",
    "尃" => "&#x5C03;",
    "射" => "&#x5C04;",
    "尅" => "&#x5C05;",
    "将" => "&#x5C06;",
    "將" => "&#x5C07;",
    "專" => "&#x5C08;",
    "尉" => "&#x5C09;",
    "尊" => "&#x5C0A;",
    "尋" => "&#x5C0B;",
    "尌" => "&#x5C0C;",
    "對" => "&#x5C0D;",
    "導" => "&#x5C0E;",
    "小" => "&#x5C0F;",
    "尐" => "&#x5C10;",
    "少" => "&#x5C11;",
    "尒" => "&#x5C12;",
    "尓" => "&#x5C13;",
    "尔" => "&#x5C14;",
    "尕" => "&#x5C15;",
    "尖" => "&#x5C16;",
    "尗" => "&#x5C17;",
    "尘" => "&#x5C18;",
    "尙" => "&#x5C19;",
    "尚" => "&#x5C1A;",
    "尛" => "&#x5C1B;",
    "尜" => "&#x5C1C;",
    "尝" => "&#x5C1D;",
    "尞" => "&#x5C1E;",
    "尟" => "&#x5C1F;",
    "尠" => "&#x5C20;",
    "尡" => "&#x5C21;",
    "尢" => "&#x5C22;",
    "尣" => "&#x5C23;",
    "尤" => "&#x5C24;",
    "尥" => "&#x5C25;",
    "尦" => "&#x5C26;",
    "尧" => "&#x5C27;",
    "尨" => "&#x5C28;",
    "尩" => "&#x5C29;",
    "尪" => "&#x5C2A;",
    "尫" => "&#x5C2B;",
    "尬" => "&#x5C2C;",
    "尭" => "&#x5C2D;",
    "尮" => "&#x5C2E;",
    "尯" => "&#x5C2F;",
    "尰" => "&#x5C30;",
    "就" => "&#x5C31;",
    "尲" => "&#x5C32;",
    "尳" => "&#x5C33;",
    "尴" => "&#x5C34;",
    "尵" => "&#x5C35;",
    "尶" => "&#x5C36;",
    "尷" => "&#x5C37;",
    "尸" => "&#x5C38;",
    "尹" => "&#x5C39;",
    "尺" => "&#x5C3A;",
    "尻" => "&#x5C3B;",
    "尼" => "&#x5C3C;",
    "尽" => "&#x5C3D;",
    "尾" => "&#x5C3E;",
    "尿" => "&#x5C3F;",
    "局" => "&#x5C40;",
    "屁" => "&#x5C41;",
    "层" => "&#x5C42;",
    "屃" => "&#x5C43;",
    "屄" => "&#x5C44;",
    "居" => "&#x5C45;",
    "屆" => "&#x5C46;",
    "屇" => "&#x5C47;",
    "屈" => "&#x5C48;",
    "屉" => "&#x5C49;",
    "届" => "&#x5C4A;",
    "屋" => "&#x5C4B;",
    "屌" => "&#x5C4C;",
    "屍" => "&#x5C4D;",
    "屎" => "&#x5C4E;",
    "屏" => "&#x5C4F;",
    "屐" => "&#x5C50;",
    "屑" => "&#x5C51;",
    "屒" => "&#x5C52;",
    "屓" => "&#x5C53;",
    "屔" => "&#x5C54;",
    "展" => "&#x5C55;",
    "屖" => "&#x5C56;",
    "屗" => "&#x5C57;",
    "屘" => "&#x5C58;",
    "屙" => "&#x5C59;",
    "屚" => "&#x5C5A;",
    "屛" => "&#x5C5B;",
    "屜" => "&#x5C5C;",
    "屝" => "&#x5C5D;",
    "属" => "&#x5C5E;",
    "屟" => "&#x5C5F;",
    "屠" => "&#x5C60;",
    "屡" => "&#x5C61;",
    "屢" => "&#x5C62;",
    "屣" => "&#x5C63;",
    "層" => "&#x5C64;",
    "履" => "&#x5C65;",
    "屦" => "&#x5C66;",
    "屧" => "&#x5C67;",
    "屨" => "&#x5C68;",
    "屩" => "&#x5C69;",
    "屪" => "&#x5C6A;",
    "屫" => "&#x5C6B;",
    "屬" => "&#x5C6C;",
    "屭" => "&#x5C6D;",
    "屮" => "&#x5C6E;",
    "屯" => "&#x5C6F;",
    "屰" => "&#x5C70;",
    "山" => "&#x5C71;",
    "屲" => "&#x5C72;",
    "屳" => "&#x5C73;",
    "屴" => "&#x5C74;",
    "屵" => "&#x5C75;",
    "屶" => "&#x5C76;",
    "屷" => "&#x5C77;",
    "屸" => "&#x5C78;",
    "屹" => "&#x5C79;",
    "屺" => "&#x5C7A;",
    "屻" => "&#x5C7B;",
    "屼" => "&#x5C7C;",
    "屽" => "&#x5C7D;",
    "屾" => "&#x5C7E;",
    "屿" => "&#x5C7F;",
    "岀" => "&#x5C80;",
    "岁" => "&#x5C81;",
    "岂" => "&#x5C82;",
    "岃" => "&#x5C83;",
    "岄" => "&#x5C84;",
    "岅" => "&#x5C85;",
    "岆" => "&#x5C86;",
    "岇" => "&#x5C87;",
    "岈" => "&#x5C88;",
    "岉" => "&#x5C89;",
    "岊" => "&#x5C8A;",
    "岋" => "&#x5C8B;",
    "岌" => "&#x5C8C;",
    "岍" => "&#x5C8D;",
    "岎" => "&#x5C8E;",
    "岏" => "&#x5C8F;",
    "岐" => "&#x5C90;",
    "岑" => "&#x5C91;",
    "岒" => "&#x5C92;",
    "岓" => "&#x5C93;",
    "岔" => "&#x5C94;",
    "岕" => "&#x5C95;",
    "岖" => "&#x5C96;",
    "岗" => "&#x5C97;",
    "岘" => "&#x5C98;",
    "岙" => "&#x5C99;",
    "岚" => "&#x5C9A;",
    "岛" => "&#x5C9B;",
    "岜" => "&#x5C9C;",
    "岝" => "&#x5C9D;",
    "岞" => "&#x5C9E;",
    "岟" => "&#x5C9F;",
    "岠" => "&#x5CA0;",
    "岡" => "&#x5CA1;",
    "岢" => "&#x5CA2;",
    "岣" => "&#x5CA3;",
    "岤" => "&#x5CA4;",
    "岥" => "&#x5CA5;",
    "岦" => "&#x5CA6;",
    "岧" => "&#x5CA7;",
    "岨" => "&#x5CA8;",
    "岩" => "&#x5CA9;",
    "岪" => "&#x5CAA;",
    "岫" => "&#x5CAB;",
    "岬" => "&#x5CAC;",
    "岭" => "&#x5CAD;",
    "岮" => "&#x5CAE;",
    "岯" => "&#x5CAF;",
    "岰" => "&#x5CB0;",
    "岱" => "&#x5CB1;",
    "岲" => "&#x5CB2;",
    "岳" => "&#x5CB3;",
    "岴" => "&#x5CB4;",
    "岵" => "&#x5CB5;",
    "岶" => "&#x5CB6;",
    "岷" => "&#x5CB7;",
    "岸" => "&#x5CB8;",
    "岹" => "&#x5CB9;",
    "岺" => "&#x5CBA;",
    "岻" => "&#x5CBB;",
    "岼" => "&#x5CBC;",
    "岽" => "&#x5CBD;",
    "岾" => "&#x5CBE;",
    "岿" => "&#x5CBF;",
    "峀" => "&#x5CC0;",
    "峁" => "&#x5CC1;",
    "峂" => "&#x5CC2;",
    "峃" => "&#x5CC3;",
    "峄" => "&#x5CC4;",
    "峅" => "&#x5CC5;",
    "峆" => "&#x5CC6;",
    "峇" => "&#x5CC7;",
    "峈" => "&#x5CC8;",
    "峉" => "&#x5CC9;",
    "峊" => "&#x5CCA;",
    "峋" => "&#x5CCB;",
    "峌" => "&#x5CCC;",
    "峍" => "&#x5CCD;",
    "峎" => "&#x5CCE;",
    "峏" => "&#x5CCF;",
    "峐" => "&#x5CD0;",
    "峑" => "&#x5CD1;",
    "峒" => "&#x5CD2;",
    "峓" => "&#x5CD3;",
    "峔" => "&#x5CD4;",
    "峕" => "&#x5CD5;",
    "峖" => "&#x5CD6;",
    "峗" => "&#x5CD7;",
    "峘" => "&#x5CD8;",
    "峙" => "&#x5CD9;",
    "峚" => "&#x5CDA;",
    "峛" => "&#x5CDB;",
    "峜" => "&#x5CDC;",
    "峝" => "&#x5CDD;",
    "峞" => "&#x5CDE;",
    "峟" => "&#x5CDF;",
    "峠" => "&#x5CE0;",
    "峡" => "&#x5CE1;",
    "峢" => "&#x5CE2;",
    "峣" => "&#x5CE3;",
    "峤" => "&#x5CE4;",
    "峥" => "&#x5CE5;",
    "峦" => "&#x5CE6;",
    "峧" => "&#x5CE7;",
    "峨" => "&#x5CE8;",
    "峩" => "&#x5CE9;",
    "峪" => "&#x5CEA;",
    "峫" => "&#x5CEB;",
    "峬" => "&#x5CEC;",
    "峭" => "&#x5CED;",
    "峮" => "&#x5CEE;",
    "峯" => "&#x5CEF;",
    "峰" => "&#x5CF0;",
    "峱" => "&#x5CF1;",
    "峲" => "&#x5CF2;",
    "峳" => "&#x5CF3;",
    "峴" => "&#x5CF4;",
    "峵" => "&#x5CF5;",
    "島" => "&#x5CF6;",
    "峷" => "&#x5CF7;",
    "峸" => "&#x5CF8;",
    "峹" => "&#x5CF9;",
    "峺" => "&#x5CFA;",
    "峻" => "&#x5CFB;",
    "峼" => "&#x5CFC;",
    "峽" => "&#x5CFD;",
    "峾" => "&#x5CFE;",
    "峿" => "&#x5CFF;",
    "崀" => "&#x5D00;",
    "崁" => "&#x5D01;",
    "崂" => "&#x5D02;",
    "崃" => "&#x5D03;",
    "崄" => "&#x5D04;",
    "崅" => "&#x5D05;",
    "崆" => "&#x5D06;",
    "崇" => "&#x5D07;",
    "崈" => "&#x5D08;",
    "崉" => "&#x5D09;",
    "崊" => "&#x5D0A;",
    "崋" => "&#x5D0B;",
    "崌" => "&#x5D0C;",
    "崍" => "&#x5D0D;",
    "崎" => "&#x5D0E;",
    "崏" => "&#x5D0F;",
    "崐" => "&#x5D10;",
    "崑" => "&#x5D11;",
    "崒" => "&#x5D12;",
    "崓" => "&#x5D13;",
    "崔" => "&#x5D14;",
    "崕" => "&#x5D15;",
    "崖" => "&#x5D16;",
    "崗" => "&#x5D17;",
    "崘" => "&#x5D18;",
    "崙" => "&#x5D19;",
    "崚" => "&#x5D1A;",
    "崛" => "&#x5D1B;",
    "崜" => "&#x5D1C;",
    "崝" => "&#x5D1D;",
    "崞" => "&#x5D1E;",
    "崟" => "&#x5D1F;",
    "崠" => "&#x5D20;",
    "崡" => "&#x5D21;",
    "崢" => "&#x5D22;",
    "崣" => "&#x5D23;",
    "崤" => "&#x5D24;",
    "崥" => "&#x5D25;",
    "崦" => "&#x5D26;",
    "崧" => "&#x5D27;",
    "崨" => "&#x5D28;",
    "崩" => "&#x5D29;",
    "崪" => "&#x5D2A;",
    "崫" => "&#x5D2B;",
    "崬" => "&#x5D2C;",
    "崭" => "&#x5D2D;",
    "崮" => "&#x5D2E;",
    "崯" => "&#x5D2F;",
    "崰" => "&#x5D30;",
    "崱" => "&#x5D31;",
    "崲" => "&#x5D32;",
    "崳" => "&#x5D33;",
    "崴" => "&#x5D34;",
    "崵" => "&#x5D35;",
    "崶" => "&#x5D36;",
    "崷" => "&#x5D37;",
    "崸" => "&#x5D38;",
    "崹" => "&#x5D39;",
    "崺" => "&#x5D3A;",
    "崻" => "&#x5D3B;",
    "崼" => "&#x5D3C;",
    "崽" => "&#x5D3D;",
    "崾" => "&#x5D3E;",
    "崿" => "&#x5D3F;",
    "嵀" => "&#x5D40;",
    "嵁" => "&#x5D41;",
    "嵂" => "&#x5D42;",
    "嵃" => "&#x5D43;",
    "嵄" => "&#x5D44;",
    "嵅" => "&#x5D45;",
    "嵆" => "&#x5D46;",
    "嵇" => "&#x5D47;",
    "嵈" => "&#x5D48;",
    "嵉" => "&#x5D49;",
    "嵊" => "&#x5D4A;",
    "嵋" => "&#x5D4B;",
    "嵌" => "&#x5D4C;",
    "嵍" => "&#x5D4D;",
    "嵎" => "&#x5D4E;",
    "嵏" => "&#x5D4F;",
    "嵐" => "&#x5D50;",
    "嵑" => "&#x5D51;",
    "嵒" => "&#x5D52;",
    "嵓" => "&#x5D53;",
    "嵔" => "&#x5D54;",
    "嵕" => "&#x5D55;",
    "嵖" => "&#x5D56;",
    "嵗" => "&#x5D57;",
    "嵘" => "&#x5D58;",
    "嵙" => "&#x5D59;",
    "嵚" => "&#x5D5A;",
    "嵛" => "&#x5D5B;",
    "嵜" => "&#x5D5C;",
    "嵝" => "&#x5D5D;",
    "嵞" => "&#x5D5E;",
    "嵟" => "&#x5D5F;",
    "嵠" => "&#x5D60;",
    "嵡" => "&#x5D61;",
    "嵢" => "&#x5D62;",
    "嵣" => "&#x5D63;",
    "嵤" => "&#x5D64;",
    "嵥" => "&#x5D65;",
    "嵦" => "&#x5D66;",
    "嵧" => "&#x5D67;",
    "嵨" => "&#x5D68;",
    "嵩" => "&#x5D69;",
    "嵪" => "&#x5D6A;",
    "嵫" => "&#x5D6B;",
    "嵬" => "&#x5D6C;",
    "嵭" => "&#x5D6D;",
    "嵮" => "&#x5D6E;",
    "嵯" => "&#x5D6F;",
    "嵰" => "&#x5D70;",
    "嵱" => "&#x5D71;",
    "嵲" => "&#x5D72;",
    "嵳" => "&#x5D73;",
    "嵴" => "&#x5D74;",
    "嵵" => "&#x5D75;",
    "嵶" => "&#x5D76;",
    "嵷" => "&#x5D77;",
    "嵸" => "&#x5D78;",
    "嵹" => "&#x5D79;",
    "嵺" => "&#x5D7A;",
    "嵻" => "&#x5D7B;",
    "嵼" => "&#x5D7C;",
    "嵽" => "&#x5D7D;",
    "嵾" => "&#x5D7E;",
    "嵿" => "&#x5D7F;",
    "嶀" => "&#x5D80;",
    "嶁" => "&#x5D81;",
    "嶂" => "&#x5D82;",
    "嶃" => "&#x5D83;",
    "嶄" => "&#x5D84;",
    "嶅" => "&#x5D85;",
    "嶆" => "&#x5D86;",
    "嶇" => "&#x5D87;",
    "嶈" => "&#x5D88;",
    "嶉" => "&#x5D89;",
    "嶊" => "&#x5D8A;",
    "嶋" => "&#x5D8B;",
    "嶌" => "&#x5D8C;",
    "嶍" => "&#x5D8D;",
    "嶎" => "&#x5D8E;",
    "嶏" => "&#x5D8F;",
    "嶐" => "&#x5D90;",
    "嶑" => "&#x5D91;",
    "嶒" => "&#x5D92;",
    "嶓" => "&#x5D93;",
    "嶔" => "&#x5D94;",
    "嶕" => "&#x5D95;",
    "嶖" => "&#x5D96;",
    "嶗" => "&#x5D97;",
    "嶘" => "&#x5D98;",
    "嶙" => "&#x5D99;",
    "嶚" => "&#x5D9A;",
    "嶛" => "&#x5D9B;",
    "嶜" => "&#x5D9C;",
    "嶝" => "&#x5D9D;",
    "嶞" => "&#x5D9E;",
    "嶟" => "&#x5D9F;",
    "嶠" => "&#x5DA0;",
    "嶡" => "&#x5DA1;",
    "嶢" => "&#x5DA2;",
    "嶣" => "&#x5DA3;",
    "嶤" => "&#x5DA4;",
    "嶥" => "&#x5DA5;",
    "嶦" => "&#x5DA6;",
    "嶧" => "&#x5DA7;",
    "嶨" => "&#x5DA8;",
    "嶩" => "&#x5DA9;",
    "嶪" => "&#x5DAA;",
    "嶫" => "&#x5DAB;",
    "嶬" => "&#x5DAC;",
    "嶭" => "&#x5DAD;",
    "嶮" => "&#x5DAE;",
    "嶯" => "&#x5DAF;",
    "嶰" => "&#x5DB0;",
    "嶱" => "&#x5DB1;",
    "嶲" => "&#x5DB2;",
    "嶳" => "&#x5DB3;",
    "嶴" => "&#x5DB4;",
    "嶵" => "&#x5DB5;",
    "嶶" => "&#x5DB6;",
    "嶷" => "&#x5DB7;",
    "嶸" => "&#x5DB8;",
    "嶹" => "&#x5DB9;",
    "嶺" => "&#x5DBA;",
    "嶻" => "&#x5DBB;",
    "嶼" => "&#x5DBC;",
    "嶽" => "&#x5DBD;",
    "嶾" => "&#x5DBE;",
    "嶿" => "&#x5DBF;",
    "巀" => "&#x5DC0;",
    "巁" => "&#x5DC1;",
    "巂" => "&#x5DC2;",
    "巃" => "&#x5DC3;",
    "巄" => "&#x5DC4;",
    "巅" => "&#x5DC5;",
    "巆" => "&#x5DC6;",
    "巇" => "&#x5DC7;",
    "巈" => "&#x5DC8;",
    "巉" => "&#x5DC9;",
    "巊" => "&#x5DCA;",
    "巋" => "&#x5DCB;",
    "巌" => "&#x5DCC;",
    "巍" => "&#x5DCD;",
    "巎" => "&#x5DCE;",
    "巏" => "&#x5DCF;",
    "巐" => "&#x5DD0;",
    "巑" => "&#x5DD1;",
    "巒" => "&#x5DD2;",
    "巓" => "&#x5DD3;",
    "巔" => "&#x5DD4;",
    "巕" => "&#x5DD5;",
    "巖" => "&#x5DD6;",
    "巗" => "&#x5DD7;",
    "巘" => "&#x5DD8;",
    "巙" => "&#x5DD9;",
    "巚" => "&#x5DDA;",
    "巛" => "&#x5DDB;",
    "巜" => "&#x5DDC;",
    "川" => "&#x5DDD;",
    "州" => "&#x5DDE;",
    "巟" => "&#x5DDF;",
    "巠" => "&#x5DE0;",
    "巡" => "&#x5DE1;",
    "巢" => "&#x5DE2;",
    "巣" => "&#x5DE3;",
    "巤" => "&#x5DE4;",
    "工" => "&#x5DE5;",
    "左" => "&#x5DE6;",
    "巧" => "&#x5DE7;",
    "巨" => "&#x5DE8;",
    "巩" => "&#x5DE9;",
    "巪" => "&#x5DEA;",
    "巫" => "&#x5DEB;",
    "巬" => "&#x5DEC;",
    "巭" => "&#x5DED;",
    "差" => "&#x5DEE;",
    "巯" => "&#x5DEF;",
    "巰" => "&#x5DF0;",
    "己" => "&#x5DF1;",
    "已" => "&#x5DF2;",
    "巳" => "&#x5DF3;",
    "巴" => "&#x5DF4;",
    "巵" => "&#x5DF5;",
    "巶" => "&#x5DF6;",
    "巷" => "&#x5DF7;",
    "巸" => "&#x5DF8;",
    "巹" => "&#x5DF9;",
    "巺" => "&#x5DFA;",
    "巻" => "&#x5DFB;",
    "巼" => "&#x5DFC;",
    "巽" => "&#x5DFD;",
    "巾" => "&#x5DFE;",
    "巿" => "&#x5DFF;",
    "帀" => "&#x5E00;",
    "币" => "&#x5E01;",
    "市" => "&#x5E02;",
    "布" => "&#x5E03;",
    "帄" => "&#x5E04;",
    "帅" => "&#x5E05;",
    "帆" => "&#x5E06;",
    "帇" => "&#x5E07;",
    "师" => "&#x5E08;",
    "帉" => "&#x5E09;",
    "帊" => "&#x5E0A;",
    "帋" => "&#x5E0B;",
    "希" => "&#x5E0C;",
    "帍" => "&#x5E0D;",
    "帎" => "&#x5E0E;",
    "帏" => "&#x5E0F;",
    "帐" => "&#x5E10;",
    "帑" => "&#x5E11;",
    "帒" => "&#x5E12;",
    "帓" => "&#x5E13;",
    "帔" => "&#x5E14;",
    "帕" => "&#x5E15;",
    "帖" => "&#x5E16;",
    "帗" => "&#x5E17;",
    "帘" => "&#x5E18;",
    "帙" => "&#x5E19;",
    "帚" => "&#x5E1A;",
    "帛" => "&#x5E1B;",
    "帜" => "&#x5E1C;",
    "帝" => "&#x5E1D;",
    "帞" => "&#x5E1E;",
    "帟" => "&#x5E1F;",
    "帠" => "&#x5E20;",
    "帡" => "&#x5E21;",
    "帢" => "&#x5E22;",
    "帣" => "&#x5E23;",
    "帤" => "&#x5E24;",
    "帥" => "&#x5E25;",
    "带" => "&#x5E26;",
    "帧" => "&#x5E27;",
    "帨" => "&#x5E28;",
    "帩" => "&#x5E29;",
    "帪" => "&#x5E2A;",
    "師" => "&#x5E2B;",
    "帬" => "&#x5E2C;",
    "席" => "&#x5E2D;",
    "帮" => "&#x5E2E;",
    "帯" => "&#x5E2F;",
    "帰" => "&#x5E30;",
    "帱" => "&#x5E31;",
    "帲" => "&#x5E32;",
    "帳" => "&#x5E33;",
    "帴" => "&#x5E34;",
    "帵" => "&#x5E35;",
    "帶" => "&#x5E36;",
    "帷" => "&#x5E37;",
    "常" => "&#x5E38;",
    "帹" => "&#x5E39;",
    "帺" => "&#x5E3A;",
    "帻" => "&#x5E3B;",
    "帼" => "&#x5E3C;",
    "帽" => "&#x5E3D;",
    "帾" => "&#x5E3E;",
    "帿" => "&#x5E3F;",
    "幀" => "&#x5E40;",
    "幁" => "&#x5E41;",
    "幂" => "&#x5E42;",
    "幃" => "&#x5E43;",
    "幄" => "&#x5E44;",
    "幅" => "&#x5E45;",
    "幆" => "&#x5E46;",
    "幇" => "&#x5E47;",
    "幈" => "&#x5E48;",
    "幉" => "&#x5E49;",
    "幊" => "&#x5E4A;",
    "幋" => "&#x5E4B;",
    "幌" => "&#x5E4C;",
    "幍" => "&#x5E4D;",
    "幎" => "&#x5E4E;",
    "幏" => "&#x5E4F;",
    "幐" => "&#x5E50;",
    "幑" => "&#x5E51;",
    "幒" => "&#x5E52;",
    "幓" => "&#x5E53;",
    "幔" => "&#x5E54;",
    "幕" => "&#x5E55;",
    "幖" => "&#x5E56;",
    "幗" => "&#x5E57;",
    "幘" => "&#x5E58;",
    "幙" => "&#x5E59;",
    "幚" => "&#x5E5A;",
    "幛" => "&#x5E5B;",
    "幜" => "&#x5E5C;",
    "幝" => "&#x5E5D;",
    "幞" => "&#x5E5E;",
    "幟" => "&#x5E5F;",
    "幠" => "&#x5E60;",
    "幡" => "&#x5E61;",
    "幢" => "&#x5E62;",
    "幣" => "&#x5E63;",
    "幤" => "&#x5E64;",
    "幥" => "&#x5E65;",
    "幦" => "&#x5E66;",
    "幧" => "&#x5E67;",
    "幨" => "&#x5E68;",
    "幩" => "&#x5E69;",
    "幪" => "&#x5E6A;",
    "幫" => "&#x5E6B;",
    "幬" => "&#x5E6C;",
    "幭" => "&#x5E6D;",
    "幮" => "&#x5E6E;",
    "幯" => "&#x5E6F;",
    "幰" => "&#x5E70;",
    "幱" => "&#x5E71;",
    "干" => "&#x5E72;",
    "平" => "&#x5E73;",
    "年" => "&#x5E74;",
    "幵" => "&#x5E75;",
    "并" => "&#x5E76;",
    "幷" => "&#x5E77;",
    "幸" => "&#x5E78;",
    "幹" => "&#x5E79;",
    "幺" => "&#x5E7A;",
    "幻" => "&#x5E7B;",
    "幼" => "&#x5E7C;",
    "幽" => "&#x5E7D;",
    "幾" => "&#x5E7E;",
    "广" => "&#x5E7F;",
    "庀" => "&#x5E80;",
    "庁" => "&#x5E81;",
    "庂" => "&#x5E82;",
    "広" => "&#x5E83;",
    "庄" => "&#x5E84;",
    "庅" => "&#x5E85;",
    "庆" => "&#x5E86;",
    "庇" => "&#x5E87;",
    "庈" => "&#x5E88;",
    "庉" => "&#x5E89;",
    "床" => "&#x5E8A;",
    "庋" => "&#x5E8B;",
    "庌" => "&#x5E8C;",
    "庍" => "&#x5E8D;",
    "庎" => "&#x5E8E;",
    "序" => "&#x5E8F;",
    "庐" => "&#x5E90;",
    "庑" => "&#x5E91;",
    "庒" => "&#x5E92;",
    "库" => "&#x5E93;",
    "应" => "&#x5E94;",
    "底" => "&#x5E95;",
    "庖" => "&#x5E96;",
    "店" => "&#x5E97;",
    "庘" => "&#x5E98;",
    "庙" => "&#x5E99;",
    "庚" => "&#x5E9A;",
    "庛" => "&#x5E9B;",
    "府" => "&#x5E9C;",
    "庝" => "&#x5E9D;",
    "庞" => "&#x5E9E;",
    "废" => "&#x5E9F;",
    "庠" => "&#x5EA0;",
    "庡" => "&#x5EA1;",
    "庢" => "&#x5EA2;",
    "庣" => "&#x5EA3;",
    "庤" => "&#x5EA4;",
    "庥" => "&#x5EA5;",
    "度" => "&#x5EA6;",
    "座" => "&#x5EA7;",
    "庨" => "&#x5EA8;",
    "庩" => "&#x5EA9;",
    "庪" => "&#x5EAA;",
    "庫" => "&#x5EAB;",
    "庬" => "&#x5EAC;",
    "庭" => "&#x5EAD;",
    "庮" => "&#x5EAE;",
    "庯" => "&#x5EAF;",
    "庰" => "&#x5EB0;",
    "庱" => "&#x5EB1;",
    "庲" => "&#x5EB2;",
    "庳" => "&#x5EB3;",
    "庴" => "&#x5EB4;",
    "庵" => "&#x5EB5;",
    "庶" => "&#x5EB6;",
    "康" => "&#x5EB7;",
    "庸" => "&#x5EB8;",
    "庹" => "&#x5EB9;",
    "庺" => "&#x5EBA;",
    "庻" => "&#x5EBB;",
    "庼" => "&#x5EBC;",
    "庽" => "&#x5EBD;",
    "庾" => "&#x5EBE;",
    "庿" => "&#x5EBF;",
    "廀" => "&#x5EC0;",
    "廁" => "&#x5EC1;",
    "廂" => "&#x5EC2;",
    "廃" => "&#x5EC3;",
    "廄" => "&#x5EC4;",
    "廅" => "&#x5EC5;",
    "廆" => "&#x5EC6;",
    "廇" => "&#x5EC7;",
    "廈" => "&#x5EC8;",
    "廉" => "&#x5EC9;",
    "廊" => "&#x5ECA;",
    "廋" => "&#x5ECB;",
    "廌" => "&#x5ECC;",
    "廍" => "&#x5ECD;",
    "廎" => "&#x5ECE;",
    "廏" => "&#x5ECF;",
    "廐" => "&#x5ED0;",
    "廑" => "&#x5ED1;",
    "廒" => "&#x5ED2;",
    "廓" => "&#x5ED3;",
    "廔" => "&#x5ED4;",
    "廕" => "&#x5ED5;",
    "廖" => "&#x5ED6;",
    "廗" => "&#x5ED7;",
    "廘" => "&#x5ED8;",
    "廙" => "&#x5ED9;",
    "廚" => "&#x5EDA;",
    "廛" => "&#x5EDB;",
    "廜" => "&#x5EDC;",
    "廝" => "&#x5EDD;",
    "廞" => "&#x5EDE;",
    "廟" => "&#x5EDF;",
    "廠" => "&#x5EE0;",
    "廡" => "&#x5EE1;",
    "廢" => "&#x5EE2;",
    "廣" => "&#x5EE3;",
    "廤" => "&#x5EE4;",
    "廥" => "&#x5EE5;",
    "廦" => "&#x5EE6;",
    "廧" => "&#x5EE7;",
    "廨" => "&#x5EE8;",
    "廩" => "&#x5EE9;",
    "廪" => "&#x5EEA;",
    "廫" => "&#x5EEB;",
    "廬" => "&#x5EEC;",
    "廭" => "&#x5EED;",
    "廮" => "&#x5EEE;",
    "廯" => "&#x5EEF;",
    "廰" => "&#x5EF0;",
    "廱" => "&#x5EF1;",
    "廲" => "&#x5EF2;",
    "廳" => "&#x5EF3;",
    "廴" => "&#x5EF4;",
    "廵" => "&#x5EF5;",
    "延" => "&#x5EF6;",
    "廷" => "&#x5EF7;",
    "廸" => "&#x5EF8;",
    "廹" => "&#x5EF9;",
    "建" => "&#x5EFA;",
    "廻" => "&#x5EFB;",
    "廼" => "&#x5EFC;",
    "廽" => "&#x5EFD;",
    "廾" => "&#x5EFE;",
    "廿" => "&#x5EFF;",
    "开" => "&#x5F00;",
    "弁" => "&#x5F01;",
    "异" => "&#x5F02;",
    "弃" => "&#x5F03;",
    "弄" => "&#x5F04;",
    "弅" => "&#x5F05;",
    "弆" => "&#x5F06;",
    "弇" => "&#x5F07;",
    "弈" => "&#x5F08;",
    "弉" => "&#x5F09;",
    "弊" => "&#x5F0A;",
    "弋" => "&#x5F0B;",
    "弌" => "&#x5F0C;",
    "弍" => "&#x5F0D;",
    "弎" => "&#x5F0E;",
    "式" => "&#x5F0F;",
    "弐" => "&#x5F10;",
    "弑" => "&#x5F11;",
    "弒" => "&#x5F12;",
    "弓" => "&#x5F13;",
    "弔" => "&#x5F14;",
    "引" => "&#x5F15;",
    "弖" => "&#x5F16;",
    "弗" => "&#x5F17;",
    "弘" => "&#x5F18;",
    "弙" => "&#x5F19;",
    "弚" => "&#x5F1A;",
    "弛" => "&#x5F1B;",
    "弜" => "&#x5F1C;",
    "弝" => "&#x5F1D;",
    "弞" => "&#x5F1E;",
    "弟" => "&#x5F1F;",
    "张" => "&#x5F20;",
    "弡" => "&#x5F21;",
    "弢" => "&#x5F22;",
    "弣" => "&#x5F23;",
    "弤" => "&#x5F24;",
    "弥" => "&#x5F25;",
    "弦" => "&#x5F26;",
    "弧" => "&#x5F27;",
    "弨" => "&#x5F28;",
    "弩" => "&#x5F29;",
    "弪" => "&#x5F2A;",
    "弫" => "&#x5F2B;",
    "弬" => "&#x5F2C;",
    "弭" => "&#x5F2D;",
    "弮" => "&#x5F2E;",
    "弯" => "&#x5F2F;",
    "弰" => "&#x5F30;",
    "弱" => "&#x5F31;",
    "弲" => "&#x5F32;",
    "弳" => "&#x5F33;",
    "弴" => "&#x5F34;",
    "張" => "&#x5F35;",
    "弶" => "&#x5F36;",
    "強" => "&#x5F37;",
    "弸" => "&#x5F38;",
    "弹" => "&#x5F39;",
    "强" => "&#x5F3A;",
    "弻" => "&#x5F3B;",
    "弼" => "&#x5F3C;",
    "弽" => "&#x5F3D;",
    "弾" => "&#x5F3E;",
    "弿" => "&#x5F3F;",
    "彀" => "&#x5F40;",
    "彁" => "&#x5F41;",
    "彂" => "&#x5F42;",
    "彃" => "&#x5F43;",
    "彄" => "&#x5F44;",
    "彅" => "&#x5F45;",
    "彆" => "&#x5F46;",
    "彇" => "&#x5F47;",
    "彈" => "&#x5F48;",
    "彉" => "&#x5F49;",
    "彊" => "&#x5F4A;",
    "彋" => "&#x5F4B;",
    "彌" => "&#x5F4C;",
    "彍" => "&#x5F4D;",
    "彎" => "&#x5F4E;",
    "彏" => "&#x5F4F;",
    "彐" => "&#x5F50;",
    "彑" => "&#x5F51;",
    "归" => "&#x5F52;",
    "当" => "&#x5F53;",
    "彔" => "&#x5F54;",
    "录" => "&#x5F55;",
    "彖" => "&#x5F56;",
    "彗" => "&#x5F57;",
    "彘" => "&#x5F58;",
    "彙" => "&#x5F59;",
    "彚" => "&#x5F5A;",
    "彛" => "&#x5F5B;",
    "彜" => "&#x5F5C;",
    "彝" => "&#x5F5D;",
    "彞" => "&#x5F5E;",
    "彟" => "&#x5F5F;",
    "彠" => "&#x5F60;",
    "彡" => "&#x5F61;",
    "形" => "&#x5F62;",
    "彣" => "&#x5F63;",
    "彤" => "&#x5F64;",
    "彥" => "&#x5F65;",
    "彦" => "&#x5F66;",
    "彧" => "&#x5F67;",
    "彨" => "&#x5F68;",
    "彩" => "&#x5F69;",
    "彪" => "&#x5F6A;",
    "彫" => "&#x5F6B;",
    "彬" => "&#x5F6C;",
    "彭" => "&#x5F6D;",
    "彮" => "&#x5F6E;",
    "彯" => "&#x5F6F;",
    "彰" => "&#x5F70;",
    "影" => "&#x5F71;",
    "彲" => "&#x5F72;",
    "彳" => "&#x5F73;",
    "彴" => "&#x5F74;",
    "彵" => "&#x5F75;",
    "彶" => "&#x5F76;",
    "彷" => "&#x5F77;",
    "彸" => "&#x5F78;",
    "役" => "&#x5F79;",
    "彺" => "&#x5F7A;",
    "彻" => "&#x5F7B;",
    "彼" => "&#x5F7C;",
    "彽" => "&#x5F7D;",
    "彾" => "&#x5F7E;",
    "彿" => "&#x5F7F;",
    "往" => "&#x5F80;",
    "征" => "&#x5F81;",
    "徂" => "&#x5F82;",
    "徃" => "&#x5F83;",
    "径" => "&#x5F84;",
    "待" => "&#x5F85;",
    "徆" => "&#x5F86;",
    "徇" => "&#x5F87;",
    "很" => "&#x5F88;",
    "徉" => "&#x5F89;",
    "徊" => "&#x5F8A;",
    "律" => "&#x5F8B;",
    "後" => "&#x5F8C;",
    "徍" => "&#x5F8D;",
    "徎" => "&#x5F8E;",
    "徏" => "&#x5F8F;",
    "徐" => "&#x5F90;",
    "徑" => "&#x5F91;",
    "徒" => "&#x5F92;",
    "従" => "&#x5F93;",
    "徔" => "&#x5F94;",
    "徕" => "&#x5F95;",
    "徖" => "&#x5F96;",
    "得" => "&#x5F97;",
    "徘" => "&#x5F98;",
    "徙" => "&#x5F99;",
    "徚" => "&#x5F9A;",
    "徛" => "&#x5F9B;",
    "徜" => "&#x5F9C;",
    "徝" => "&#x5F9D;",
    "從" => "&#x5F9E;",
    "徟" => "&#x5F9F;",
    "徠" => "&#x5FA0;",
    "御" => "&#x5FA1;",
    "徢" => "&#x5FA2;",
    "徣" => "&#x5FA3;",
    "徤" => "&#x5FA4;",
    "徥" => "&#x5FA5;",
    "徦" => "&#x5FA6;",
    "徧" => "&#x5FA7;",
    "徨" => "&#x5FA8;",
    "復" => "&#x5FA9;",
    "循" => "&#x5FAA;",
    "徫" => "&#x5FAB;",
    "徬" => "&#x5FAC;",
    "徭" => "&#x5FAD;",
    "微" => "&#x5FAE;",
    "徯" => "&#x5FAF;",
    "徰" => "&#x5FB0;",
    "徱" => "&#x5FB1;",
    "徲" => "&#x5FB2;",
    "徳" => "&#x5FB3;",
    "徴" => "&#x5FB4;",
    "徵" => "&#x5FB5;",
    "徶" => "&#x5FB6;",
    "德" => "&#x5FB7;",
    "徸" => "&#x5FB8;",
    "徹" => "&#x5FB9;",
    "徺" => "&#x5FBA;",
    "徻" => "&#x5FBB;",
    "徼" => "&#x5FBC;",
    "徽" => "&#x5FBD;",
    "徾" => "&#x5FBE;",
    "徿" => "&#x5FBF;",
    "忀" => "&#x5FC0;",
    "忁" => "&#x5FC1;",
    "忂" => "&#x5FC2;",
    "心" => "&#x5FC3;",
    "忄" => "&#x5FC4;",
    "必" => "&#x5FC5;",
    "忆" => "&#x5FC6;",
    "忇" => "&#x5FC7;",
    "忈" => "&#x5FC8;",
    "忉" => "&#x5FC9;",
    "忊" => "&#x5FCA;",
    "忋" => "&#x5FCB;",
    "忌" => "&#x5FCC;",
    "忍" => "&#x5FCD;",
    "忎" => "&#x5FCE;",
    "忏" => "&#x5FCF;",
    "忐" => "&#x5FD0;",
    "忑" => "&#x5FD1;",
    "忒" => "&#x5FD2;",
    "忓" => "&#x5FD3;",
    "忔" => "&#x5FD4;",
    "忕" => "&#x5FD5;",
    "忖" => "&#x5FD6;",
    "志" => "&#x5FD7;",
    "忘" => "&#x5FD8;",
    "忙" => "&#x5FD9;",
    "忚" => "&#x5FDA;",
    "忛" => "&#x5FDB;",
    "応" => "&#x5FDC;",
    "忝" => "&#x5FDD;",
    "忞" => "&#x5FDE;",
    "忟" => "&#x5FDF;",
    "忠" => "&#x5FE0;",
    "忡" => "&#x5FE1;",
    "忢" => "&#x5FE2;",
    "忣" => "&#x5FE3;",
    "忤" => "&#x5FE4;",
    "忥" => "&#x5FE5;",
    "忦" => "&#x5FE6;",
    "忧" => "&#x5FE7;",
    "忨" => "&#x5FE8;",
    "忩" => "&#x5FE9;",
    "忪" => "&#x5FEA;",
    "快" => "&#x5FEB;",
    "忬" => "&#x5FEC;",
    "忭" => "&#x5FED;",
    "忮" => "&#x5FEE;",
    "忯" => "&#x5FEF;",
    "忰" => "&#x5FF0;",
    "忱" => "&#x5FF1;",
    "忲" => "&#x5FF2;",
    "忳" => "&#x5FF3;",
    "忴" => "&#x5FF4;",
    "念" => "&#x5FF5;",
    "忶" => "&#x5FF6;",
    "忷" => "&#x5FF7;",
    "忸" => "&#x5FF8;",
    "忹" => "&#x5FF9;",
    "忺" => "&#x5FFA;",
    "忻" => "&#x5FFB;",
    "忼" => "&#x5FFC;",
    "忽" => "&#x5FFD;",
    "忾" => "&#x5FFE;",
    "忿" => "&#x5FFF;",
    "怀" => "&#x6000;",
    "态" => "&#x6001;",
    "怂" => "&#x6002;",
    "怃" => "&#x6003;",
    "怄" => "&#x6004;",
    "怅" => "&#x6005;",
    "怆" => "&#x6006;",
    "怇" => "&#x6007;",
    "怈" => "&#x6008;",
    "怉" => "&#x6009;",
    "怊" => "&#x600A;",
    "怋" => "&#x600B;",
    "怌" => "&#x600C;",
    "怍" => "&#x600D;",
    "怎" => "&#x600E;",
    "怏" => "&#x600F;",
    "怐" => "&#x6010;",
    "怑" => "&#x6011;",
    "怒" => "&#x6012;",
    "怓" => "&#x6013;",
    "怔" => "&#x6014;",
    "怕" => "&#x6015;",
    "怖" => "&#x6016;",
    "怗" => "&#x6017;",
    "怘" => "&#x6018;",
    "怙" => "&#x6019;",
    "怚" => "&#x601A;",
    "怛" => "&#x601B;",
    "怜" => "&#x601C;",
    "思" => "&#x601D;",
    "怞" => "&#x601E;",
    "怟" => "&#x601F;",
    "怠" => "&#x6020;",
    "怡" => "&#x6021;",
    "怢" => "&#x6022;",
    "怣" => "&#x6023;",
    "怤" => "&#x6024;",
    "急" => "&#x6025;",
    "怦" => "&#x6026;",
    "性" => "&#x6027;",
    "怨" => "&#x6028;",
    "怩" => "&#x6029;",
    "怪" => "&#x602A;",
    "怫" => "&#x602B;",
    "怬" => "&#x602C;",
    "怭" => "&#x602D;",
    "怮" => "&#x602E;",
    "怯" => "&#x602F;",
    "怰" => "&#x6030;",
    "怱" => "&#x6031;",
    "怲" => "&#x6032;",
    "怳" => "&#x6033;",
    "怴" => "&#x6034;",
    "怵" => "&#x6035;",
    "怶" => "&#x6036;",
    "怷" => "&#x6037;",
    "怸" => "&#x6038;",
    "怹" => "&#x6039;",
    "怺" => "&#x603A;",
    "总" => "&#x603B;",
    "怼" => "&#x603C;",
    "怽" => "&#x603D;",
    "怾" => "&#x603E;",
    "怿" => "&#x603F;",
    "恀" => "&#x6040;",
    "恁" => "&#x6041;",
    "恂" => "&#x6042;",
    "恃" => "&#x6043;",
    "恄" => "&#x6044;",
    "恅" => "&#x6045;",
    "恆" => "&#x6046;",
    "恇" => "&#x6047;",
    "恈" => "&#x6048;",
    "恉" => "&#x6049;",
    "恊" => "&#x604A;",
    "恋" => "&#x604B;",
    "恌" => "&#x604C;",
    "恍" => "&#x604D;",
    "恎" => "&#x604E;",
    "恏" => "&#x604F;",
    "恐" => "&#x6050;",
    "恑" => "&#x6051;",
    "恒" => "&#x6052;",
    "恓" => "&#x6053;",
    "恔" => "&#x6054;",
    "恕" => "&#x6055;",
    "恖" => "&#x6056;",
    "恗" => "&#x6057;",
    "恘" => "&#x6058;",
    "恙" => "&#x6059;",
    "恚" => "&#x605A;",
    "恛" => "&#x605B;",
    "恜" => "&#x605C;",
    "恝" => "&#x605D;",
    "恞" => "&#x605E;",
    "恟" => "&#x605F;",
    "恠" => "&#x6060;",
    "恡" => "&#x6061;",
    "恢" => "&#x6062;",
    "恣" => "&#x6063;",
    "恤" => "&#x6064;",
    "恥" => "&#x6065;",
    "恦" => "&#x6066;",
    "恧" => "&#x6067;",
    "恨" => "&#x6068;",
    "恩" => "&#x6069;",
    "恪" => "&#x606A;",
    "恫" => "&#x606B;",
    "恬" => "&#x606C;",
    "恭" => "&#x606D;",
    "恮" => "&#x606E;",
    "息" => "&#x606F;",
    "恰" => "&#x6070;",
    "恱" => "&#x6071;",
    "恲" => "&#x6072;",
    "恳" => "&#x6073;",
    "恴" => "&#x6074;",
    "恵" => "&#x6075;",
    "恶" => "&#x6076;",
    "恷" => "&#x6077;",
    "恸" => "&#x6078;",
    "恹" => "&#x6079;",
    "恺" => "&#x607A;",
    "恻" => "&#x607B;",
    "恼" => "&#x607C;",
    "恽" => "&#x607D;",
    "恾" => "&#x607E;",
    "恿" => "&#x607F;",
    "悀" => "&#x6080;",
    "悁" => "&#x6081;",
    "悂" => "&#x6082;",
    "悃" => "&#x6083;",
    "悄" => "&#x6084;",
    "悅" => "&#x6085;",
    "悆" => "&#x6086;",
    "悇" => "&#x6087;",
    "悈" => "&#x6088;",
    "悉" => "&#x6089;",
    "悊" => "&#x608A;",
    "悋" => "&#x608B;",
    "悌" => "&#x608C;",
    "悍" => "&#x608D;",
    "悎" => "&#x608E;",
    "悏" => "&#x608F;",
    "悐" => "&#x6090;",
    "悑" => "&#x6091;",
    "悒" => "&#x6092;",
    "悓" => "&#x6093;",
    "悔" => "&#x6094;",
    "悕" => "&#x6095;",
    "悖" => "&#x6096;",
    "悗" => "&#x6097;",
    "悘" => "&#x6098;",
    "悙" => "&#x6099;",
    "悚" => "&#x609A;",
    "悛" => "&#x609B;",
    "悜" => "&#x609C;",
    "悝" => "&#x609D;",
    "悞" => "&#x609E;",
    "悟" => "&#x609F;",
    "悠" => "&#x60A0;",
    "悡" => "&#x60A1;",
    "悢" => "&#x60A2;",
    "患" => "&#x60A3;",
    "悤" => "&#x60A4;",
    "悥" => "&#x60A5;",
    "悦" => "&#x60A6;",
    "悧" => "&#x60A7;",
    "您" => "&#x60A8;",
    "悩" => "&#x60A9;",
    "悪" => "&#x60AA;",
    "悫" => "&#x60AB;",
    "悬" => "&#x60AC;",
    "悭" => "&#x60AD;",
    "悮" => "&#x60AE;",
    "悯" => "&#x60AF;",
    "悰" => "&#x60B0;",
    "悱" => "&#x60B1;",
    "悲" => "&#x60B2;",
    "悳" => "&#x60B3;",
    "悴" => "&#x60B4;",
    "悵" => "&#x60B5;",
    "悶" => "&#x60B6;",
    "悷" => "&#x60B7;",
    "悸" => "&#x60B8;",
    "悹" => "&#x60B9;",
    "悺" => "&#x60BA;",
    "悻" => "&#x60BB;",
    "悼" => "&#x60BC;",
    "悽" => "&#x60BD;",
    "悾" => "&#x60BE;",
    "悿" => "&#x60BF;",
    "惀" => "&#x60C0;",
    "惁" => "&#x60C1;",
    "惂" => "&#x60C2;",
    "惃" => "&#x60C3;",
    "惄" => "&#x60C4;",
    "情" => "&#x60C5;",
    "惆" => "&#x60C6;",
    "惇" => "&#x60C7;",
    "惈" => "&#x60C8;",
    "惉" => "&#x60C9;",
    "惊" => "&#x60CA;",
    "惋" => "&#x60CB;",
    "惌" => "&#x60CC;",
    "惍" => "&#x60CD;",
    "惎" => "&#x60CE;",
    "惏" => "&#x60CF;",
    "惐" => "&#x60D0;",
    "惑" => "&#x60D1;",
    "惒" => "&#x60D2;",
    "惓" => "&#x60D3;",
    "惔" => "&#x60D4;",
    "惕" => "&#x60D5;",
    "惖" => "&#x60D6;",
    "惗" => "&#x60D7;",
    "惘" => "&#x60D8;",
    "惙" => "&#x60D9;",
    "惚" => "&#x60DA;",
    "惛" => "&#x60DB;",
    "惜" => "&#x60DC;",
    "惝" => "&#x60DD;",
    "惞" => "&#x60DE;",
    "惟" => "&#x60DF;",
    "惠" => "&#x60E0;",
    "惡" => "&#x60E1;",
    "惢" => "&#x60E2;",
    "惣" => "&#x60E3;",
    "惤" => "&#x60E4;",
    "惥" => "&#x60E5;",
    "惦" => "&#x60E6;",
    "惧" => "&#x60E7;",
    "惨" => "&#x60E8;",
    "惩" => "&#x60E9;",
    "惪" => "&#x60EA;",
    "惫" => "&#x60EB;",
    "惬" => "&#x60EC;",
    "惭" => "&#x60ED;",
    "惮" => "&#x60EE;",
    "惯" => "&#x60EF;",
    "惰" => "&#x60F0;",
    "惱" => "&#x60F1;",
    "惲" => "&#x60F2;",
    "想" => "&#x60F3;",
    "惴" => "&#x60F4;",
    "惵" => "&#x60F5;",
    "惶" => "&#x60F6;",
    "惷" => "&#x60F7;",
    "惸" => "&#x60F8;",
    "惹" => "&#x60F9;",
    "惺" => "&#x60FA;",
    "惻" => "&#x60FB;",
    "惼" => "&#x60FC;",
    "惽" => "&#x60FD;",
    "惾" => "&#x60FE;",
    "惿" => "&#x60FF;",
    "愀" => "&#x6100;",
    "愁" => "&#x6101;",
    "愂" => "&#x6102;",
    "愃" => "&#x6103;",
    "愄" => "&#x6104;",
    "愅" => "&#x6105;",
    "愆" => "&#x6106;",
    "愇" => "&#x6107;",
    "愈" => "&#x6108;",
    "愉" => "&#x6109;",
    "愊" => "&#x610A;",
    "愋" => "&#x610B;",
    "愌" => "&#x610C;",
    "愍" => "&#x610D;",
    "愎" => "&#x610E;",
    "意" => "&#x610F;",
    "愐" => "&#x6110;",
    "愑" => "&#x6111;",
    "愒" => "&#x6112;",
    "愓" => "&#x6113;",
    "愔" => "&#x6114;",
    "愕" => "&#x6115;",
    "愖" => "&#x6116;",
    "愗" => "&#x6117;",
    "愘" => "&#x6118;",
    "愙" => "&#x6119;",
    "愚" => "&#x611A;",
    "愛" => "&#x611B;",
    "愜" => "&#x611C;",
    "愝" => "&#x611D;",
    "愞" => "&#x611E;",
    "感" => "&#x611F;",
    "愠" => "&#x6120;",
    "愡" => "&#x6121;",
    "愢" => "&#x6122;",
    "愣" => "&#x6123;",
    "愤" => "&#x6124;",
    "愥" => "&#x6125;",
    "愦" => "&#x6126;",
    "愧" => "&#x6127;",
    "愨" => "&#x6128;",
    "愩" => "&#x6129;",
    "愪" => "&#x612A;",
    "愫" => "&#x612B;",
    "愬" => "&#x612C;",
    "愭" => "&#x612D;",
    "愮" => "&#x612E;",
    "愯" => "&#x612F;",
    "愰" => "&#x6130;",
    "愱" => "&#x6131;",
    "愲" => "&#x6132;",
    "愳" => "&#x6133;",
    "愴" => "&#x6134;",
    "愵" => "&#x6135;",
    "愶" => "&#x6136;",
    "愷" => "&#x6137;",
    "愸" => "&#x6138;",
    "愹" => "&#x6139;",
    "愺" => "&#x613A;",
    "愻" => "&#x613B;",
    "愼" => "&#x613C;",
    "愽" => "&#x613D;",
    "愾" => "&#x613E;",
    "愿" => "&#x613F;",
    "慀" => "&#x6140;",
    "慁" => "&#x6141;",
    "慂" => "&#x6142;",
    "慃" => "&#x6143;",
    "慄" => "&#x6144;",
    "慅" => "&#x6145;",
    "慆" => "&#x6146;",
    "慇" => "&#x6147;",
    "慈" => "&#x6148;",
    "慉" => "&#x6149;",
    "慊" => "&#x614A;",
    "態" => "&#x614B;",
    "慌" => "&#x614C;",
    "慍" => "&#x614D;",
    "慎" => "&#x614E;",
    "慏" => "&#x614F;",
    "慐" => "&#x6150;",
    "慑" => "&#x6151;",
    "慒" => "&#x6152;",
    "慓" => "&#x6153;",
    "慔" => "&#x6154;",
    "慕" => "&#x6155;",
    "慖" => "&#x6156;",
    "慗" => "&#x6157;",
    "慘" => "&#x6158;",
    "慙" => "&#x6159;",
    "慚" => "&#x615A;",
    "慛" => "&#x615B;",
    "慜" => "&#x615C;",
    "慝" => "&#x615D;",
    "慞" => "&#x615E;",
    "慟" => "&#x615F;",
    "慠" => "&#x6160;",
    "慡" => "&#x6161;",
    "慢" => "&#x6162;",
    "慣" => "&#x6163;",
    "慤" => "&#x6164;",
    "慥" => "&#x6165;",
    "慦" => "&#x6166;",
    "慧" => "&#x6167;",
    "慨" => "&#x6168;",
    "慩" => "&#x6169;",
    "慪" => "&#x616A;",
    "慫" => "&#x616B;",
    "慬" => "&#x616C;",
    "慭" => "&#x616D;",
    "慮" => "&#x616E;",
    "慯" => "&#x616F;",
    "慰" => "&#x6170;",
    "慱" => "&#x6171;",
    "慲" => "&#x6172;",
    "慳" => "&#x6173;",
    "慴" => "&#x6174;",
    "慵" => "&#x6175;",
    "慶" => "&#x6176;",
    "慷" => "&#x6177;",
    "慸" => "&#x6178;",
    "慹" => "&#x6179;",
    "慺" => "&#x617A;",
    "慻" => "&#x617B;",
    "慼" => "&#x617C;",
    "慽" => "&#x617D;",
    "慾" => "&#x617E;",
    "慿" => "&#x617F;",
    "憀" => "&#x6180;",
    "憁" => "&#x6181;",
    "憂" => "&#x6182;",
    "憃" => "&#x6183;",
    "憄" => "&#x6184;",
    "憅" => "&#x6185;",
    "憆" => "&#x6186;",
    "憇" => "&#x6187;",
    "憈" => "&#x6188;",
    "憉" => "&#x6189;",
    "憊" => "&#x618A;",
    "憋" => "&#x618B;",
    "憌" => "&#x618C;",
    "憍" => "&#x618D;",
    "憎" => "&#x618E;",
    "憏" => "&#x618F;",
    "憐" => "&#x6190;",
    "憑" => "&#x6191;",
    "憒" => "&#x6192;",
    "憓" => "&#x6193;",
    "憔" => "&#x6194;",
    "憕" => "&#x6195;",
    "憖" => "&#x6196;",
    "憗" => "&#x6197;",
    "憘" => "&#x6198;",
    "憙" => "&#x6199;",
    "憚" => "&#x619A;",
    "憛" => "&#x619B;",
    "憜" => "&#x619C;",
    "憝" => "&#x619D;",
    "憞" => "&#x619E;",
    "憟" => "&#x619F;",
    "憠" => "&#x61A0;",
    "憡" => "&#x61A1;",
    "憢" => "&#x61A2;",
    "憣" => "&#x61A3;",
    "憤" => "&#x61A4;",
    "憥" => "&#x61A5;",
    "憦" => "&#x61A6;",
    "憧" => "&#x61A7;",
    "憨" => "&#x61A8;",
    "憩" => "&#x61A9;",
    "憪" => "&#x61AA;",
    "憫" => "&#x61AB;",
    "憬" => "&#x61AC;",
    "憭" => "&#x61AD;",
    "憮" => "&#x61AE;",
    "憯" => "&#x61AF;",
    "憰" => "&#x61B0;",
    "憱" => "&#x61B1;",
    "憲" => "&#x61B2;",
    "憳" => "&#x61B3;",
    "憴" => "&#x61B4;",
    "憵" => "&#x61B5;",
    "憶" => "&#x61B6;",
    "憷" => "&#x61B7;",
    "憸" => "&#x61B8;",
    "憹" => "&#x61B9;",
    "憺" => "&#x61BA;",
    "憻" => "&#x61BB;",
    "憼" => "&#x61BC;",
    "憽" => "&#x61BD;",
    "憾" => "&#x61BE;",
    "憿" => "&#x61BF;",
    "懀" => "&#x61C0;",
    "懁" => "&#x61C1;",
    "懂" => "&#x61C2;",
    "懃" => "&#x61C3;",
    "懄" => "&#x61C4;",
    "懅" => "&#x61C5;",
    "懆" => "&#x61C6;",
    "懇" => "&#x61C7;",
    "懈" => "&#x61C8;",
    "應" => "&#x61C9;",
    "懊" => "&#x61CA;",
    "懋" => "&#x61CB;",
    "懌" => "&#x61CC;",
    "懍" => "&#x61CD;",
    "懎" => "&#x61CE;",
    "懏" => "&#x61CF;",
    "懐" => "&#x61D0;",
    "懑" => "&#x61D1;",
    "懒" => "&#x61D2;",
    "懓" => "&#x61D3;",
    "懔" => "&#x61D4;",
    "懕" => "&#x61D5;",
    "懖" => "&#x61D6;",
    "懗" => "&#x61D7;",
    "懘" => "&#x61D8;",
    "懙" => "&#x61D9;",
    "懚" => "&#x61DA;",
    "懛" => "&#x61DB;",
    "懜" => "&#x61DC;",
    "懝" => "&#x61DD;",
    "懞" => "&#x61DE;",
    "懟" => "&#x61DF;",
    "懠" => "&#x61E0;",
    "懡" => "&#x61E1;",
    "懢" => "&#x61E2;",
    "懣" => "&#x61E3;",
    "懤" => "&#x61E4;",
    "懥" => "&#x61E5;",
    "懦" => "&#x61E6;",
    "懧" => "&#x61E7;",
    "懨" => "&#x61E8;",
    "懩" => "&#x61E9;",
    "懪" => "&#x61EA;",
    "懫" => "&#x61EB;",
    "懬" => "&#x61EC;",
    "懭" => "&#x61ED;",
    "懮" => "&#x61EE;",
    "懯" => "&#x61EF;",
    "懰" => "&#x61F0;",
    "懱" => "&#x61F1;",
    "懲" => "&#x61F2;",
    "懳" => "&#x61F3;",
    "懴" => "&#x61F4;",
    "懵" => "&#x61F5;",
    "懶" => "&#x61F6;",
    "懷" => "&#x61F7;",
    "懸" => "&#x61F8;",
    "懹" => "&#x61F9;",
    "懺" => "&#x61FA;",
    "懻" => "&#x61FB;",
    "懼" => "&#x61FC;",
    "懽" => "&#x61FD;",
    "懾" => "&#x61FE;",
    "懿" => "&#x61FF;",
    "戀" => "&#x6200;",
    "戁" => "&#x6201;",
    "戂" => "&#x6202;",
    "戃" => "&#x6203;",
    "戄" => "&#x6204;",
    "戅" => "&#x6205;",
    "戆" => "&#x6206;",
    "戇" => "&#x6207;",
    "戈" => "&#x6208;",
    "戉" => "&#x6209;",
    "戊" => "&#x620A;",
    "戋" => "&#x620B;",
    "戌" => "&#x620C;",
    "戍" => "&#x620D;",
    "戎" => "&#x620E;",
    "戏" => "&#x620F;",
    "成" => "&#x6210;",
    "我" => "&#x6211;",
    "戒" => "&#x6212;",
    "戓" => "&#x6213;",
    "戔" => "&#x6214;",
    "戕" => "&#x6215;",
    "或" => "&#x6216;",
    "戗" => "&#x6217;",
    "战" => "&#x6218;",
    "戙" => "&#x6219;",
    "戚" => "&#x621A;",
    "戛" => "&#x621B;",
    "戜" => "&#x621C;",
    "戝" => "&#x621D;",
    "戞" => "&#x621E;",
    "戟" => "&#x621F;",
    "戠" => "&#x6220;",
    "戡" => "&#x6221;",
    "戢" => "&#x6222;",
    "戣" => "&#x6223;",
    "戤" => "&#x6224;",
    "戥" => "&#x6225;",
    "戦" => "&#x6226;",
    "戧" => "&#x6227;",
    "戨" => "&#x6228;",
    "戩" => "&#x6229;",
    "截" => "&#x622A;",
    "戫" => "&#x622B;",
    "戬" => "&#x622C;",
    "戭" => "&#x622D;",
    "戮" => "&#x622E;",
    "戯" => "&#x622F;",
    "戰" => "&#x6230;",
    "戱" => "&#x6231;",
    "戲" => "&#x6232;",
    "戳" => "&#x6233;",
    "戴" => "&#x6234;",
    "戵" => "&#x6235;",
    "戶" => "&#x6236;",
    "户" => "&#x6237;",
    "戸" => "&#x6238;",
    "戹" => "&#x6239;",
    "戺" => "&#x623A;",
    "戻" => "&#x623B;",
    "戼" => "&#x623C;",
    "戽" => "&#x623D;",
    "戾" => "&#x623E;",
    "房" => "&#x623F;",
    "所" => "&#x6240;",
    "扁" => "&#x6241;",
    "扂" => "&#x6242;",
    "扃" => "&#x6243;",
    "扄" => "&#x6244;",
    "扅" => "&#x6245;",
    "扆" => "&#x6246;",
    "扇" => "&#x6247;",
    "扈" => "&#x6248;",
    "扉" => "&#x6249;",
    "扊" => "&#x624A;",
    "手" => "&#x624B;",
    "扌" => "&#x624C;",
    "才" => "&#x624D;",
    "扎" => "&#x624E;",
    "扏" => "&#x624F;",
    "扐" => "&#x6250;",
    "扑" => "&#x6251;",
    "扒" => "&#x6252;",
    "打" => "&#x6253;",
    "扔" => "&#x6254;",
    "払" => "&#x6255;",
    "扖" => "&#x6256;",
    "扗" => "&#x6257;",
    "托" => "&#x6258;",
    "扙" => "&#x6259;",
    "扚" => "&#x625A;",
    "扛" => "&#x625B;",
    "扜" => "&#x625C;",
    "扝" => "&#x625D;",
    "扞" => "&#x625E;",
    "扟" => "&#x625F;",
    "扠" => "&#x6260;",
    "扡" => "&#x6261;",
    "扢" => "&#x6262;",
    "扣" => "&#x6263;",
    "扤" => "&#x6264;",
    "扥" => "&#x6265;",
    "扦" => "&#x6266;",
    "执" => "&#x6267;",
    "扨" => "&#x6268;",
    "扩" => "&#x6269;",
    "扪" => "&#x626A;",
    "扫" => "&#x626B;",
    "扬" => "&#x626C;",
    "扭" => "&#x626D;",
    "扮" => "&#x626E;",
    "扯" => "&#x626F;",
    "扰" => "&#x6270;",
    "扱" => "&#x6271;",
    "扲" => "&#x6272;",
    "扳" => "&#x6273;",
    "扴" => "&#x6274;",
    "扵" => "&#x6275;",
    "扶" => "&#x6276;",
    "扷" => "&#x6277;",
    "扸" => "&#x6278;",
    "批" => "&#x6279;",
    "扺" => "&#x627A;",
    "扻" => "&#x627B;",
    "扼" => "&#x627C;",
    "扽" => "&#x627D;",
    "找" => "&#x627E;",
    "承" => "&#x627F;",
    "技" => "&#x6280;",
    "抁" => "&#x6281;",
    "抂" => "&#x6282;",
    "抃" => "&#x6283;",
    "抄" => "&#x6284;",
    "抅" => "&#x6285;",
    "抆" => "&#x6286;",
    "抇" => "&#x6287;",
    "抈" => "&#x6288;",
    "抉" => "&#x6289;",
    "把" => "&#x628A;",
    "抋" => "&#x628B;",
    "抌" => "&#x628C;",
    "抍" => "&#x628D;",
    "抎" => "&#x628E;",
    "抏" => "&#x628F;",
    "抐" => "&#x6290;",
    "抑" => "&#x6291;",
    "抒" => "&#x6292;",
    "抓" => "&#x6293;",
    "抔" => "&#x6294;",
    "投" => "&#x6295;",
    "抖" => "&#x6296;",
    "抗" => "&#x6297;",
    "折" => "&#x6298;",
    "抙" => "&#x6299;",
    "抚" => "&#x629A;",
    "抛" => "&#x629B;",
    "抜" => "&#x629C;",
    "抝" => "&#x629D;",
    "択" => "&#x629E;",
    "抟" => "&#x629F;",
    "抠" => "&#x62A0;",
    "抡" => "&#x62A1;",
    "抢" => "&#x62A2;",
    "抣" => "&#x62A3;",
    "护" => "&#x62A4;",
    "报" => "&#x62A5;",
    "抦" => "&#x62A6;",
    "抧" => "&#x62A7;",
    "抨" => "&#x62A8;",
    "抩" => "&#x62A9;",
    "抪" => "&#x62AA;",
    "披" => "&#x62AB;",
    "抬" => "&#x62AC;",
    "抭" => "&#x62AD;",
    "抮" => "&#x62AE;",
    "抯" => "&#x62AF;",
    "抰" => "&#x62B0;",
    "抱" => "&#x62B1;",
    "抲" => "&#x62B2;",
    "抳" => "&#x62B3;",
    "抴" => "&#x62B4;",
    "抵" => "&#x62B5;",
    "抶" => "&#x62B6;",
    "抷" => "&#x62B7;",
    "抸" => "&#x62B8;",
    "抹" => "&#x62B9;",
    "抺" => "&#x62BA;",
    "抻" => "&#x62BB;",
    "押" => "&#x62BC;",
    "抽" => "&#x62BD;",
    "抾" => "&#x62BE;",
    "抿" => "&#x62BF;",
    "拀" => "&#x62C0;",
    "拁" => "&#x62C1;",
    "拂" => "&#x62C2;",
    "拃" => "&#x62C3;",
    "拄" => "&#x62C4;",
    "担" => "&#x62C5;",
    "拆" => "&#x62C6;",
    "拇" => "&#x62C7;",
    "拈" => "&#x62C8;",
    "拉" => "&#x62C9;",
    "拊" => "&#x62CA;",
    "拋" => "&#x62CB;",
    "拌" => "&#x62CC;",
    "拍" => "&#x62CD;",
    "拎" => "&#x62CE;",
    "拏" => "&#x62CF;",
    "拐" => "&#x62D0;",
    "拑" => "&#x62D1;",
    "拒" => "&#x62D2;",
    "拓" => "&#x62D3;",
    "拔" => "&#x62D4;",
    "拕" => "&#x62D5;",
    "拖" => "&#x62D6;",
    "拗" => "&#x62D7;",
    "拘" => "&#x62D8;",
    "拙" => "&#x62D9;",
    "拚" => "&#x62DA;",
    "招" => "&#x62DB;",
    "拜" => "&#x62DC;",
    "拝" => "&#x62DD;",
    "拞" => "&#x62DE;",
    "拟" => "&#x62DF;",
    "拠" => "&#x62E0;",
    "拡" => "&#x62E1;",
    "拢" => "&#x62E2;",
    "拣" => "&#x62E3;",
    "拤" => "&#x62E4;",
    "拥" => "&#x62E5;",
    "拦" => "&#x62E6;",
    "拧" => "&#x62E7;",
    "拨" => "&#x62E8;",
    "择" => "&#x62E9;",
    "拪" => "&#x62EA;",
    "拫" => "&#x62EB;",
    "括" => "&#x62EC;",
    "拭" => "&#x62ED;",
    "拮" => "&#x62EE;",
    "拯" => "&#x62EF;",
    "拰" => "&#x62F0;",
    "拱" => "&#x62F1;",
    "拲" => "&#x62F2;",
    "拳" => "&#x62F3;",
    "拴" => "&#x62F4;",
    "拵" => "&#x62F5;",
    "拶" => "&#x62F6;",
    "拷" => "&#x62F7;",
    "拸" => "&#x62F8;",
    "拹" => "&#x62F9;",
    "拺" => "&#x62FA;",
    "拻" => "&#x62FB;",
    "拼" => "&#x62FC;",
    "拽" => "&#x62FD;",
    "拾" => "&#x62FE;",
    "拿" => "&#x62FF;",
    "挀" => "&#x6300;",
    "持" => "&#x6301;",
    "挂" => "&#x6302;",
    "挃" => "&#x6303;",
    "挄" => "&#x6304;",
    "挅" => "&#x6305;",
    "挆" => "&#x6306;",
    "指" => "&#x6307;",
    "挈" => "&#x6308;",
    "按" => "&#x6309;",
    "挊" => "&#x630A;",
    "挋" => "&#x630B;",
    "挌" => "&#x630C;",
    "挍" => "&#x630D;",
    "挎" => "&#x630E;",
    "挏" => "&#x630F;",
    "挐" => "&#x6310;",
    "挑" => "&#x6311;",
    "挒" => "&#x6312;",
    "挓" => "&#x6313;",
    "挔" => "&#x6314;",
    "挕" => "&#x6315;",
    "挖" => "&#x6316;",
    "挗" => "&#x6317;",
    "挘" => "&#x6318;",
    "挙" => "&#x6319;",
    "挚" => "&#x631A;",
    "挛" => "&#x631B;",
    "挜" => "&#x631C;",
    "挝" => "&#x631D;",
    "挞" => "&#x631E;",
    "挟" => "&#x631F;",
    "挠" => "&#x6320;",
    "挡" => "&#x6321;",
    "挢" => "&#x6322;",
    "挣" => "&#x6323;",
    "挤" => "&#x6324;",
    "挥" => "&#x6325;",
    "挦" => "&#x6326;",
    "挧" => "&#x6327;",
    "挨" => "&#x6328;",
    "挩" => "&#x6329;",
    "挪" => "&#x632A;",
    "挫" => "&#x632B;",
    "挬" => "&#x632C;",
    "挭" => "&#x632D;",
    "挮" => "&#x632E;",
    "振" => "&#x632F;",
    "挰" => "&#x6330;",
    "挱" => "&#x6331;",
    "挲" => "&#x6332;",
    "挳" => "&#x6333;",
    "挴" => "&#x6334;",
    "挵" => "&#x6335;",
    "挶" => "&#x6336;",
    "挷" => "&#x6337;",
    "挸" => "&#x6338;",
    "挹" => "&#x6339;",
    "挺" => "&#x633A;",
    "挻" => "&#x633B;",
    "挼" => "&#x633C;",
    "挽" => "&#x633D;",
    "挾" => "&#x633E;",
    "挿" => "&#x633F;",
    "捀" => "&#x6340;",
    "捁" => "&#x6341;",
    "捂" => "&#x6342;",
    "捃" => "&#x6343;",
    "捄" => "&#x6344;",
    "捅" => "&#x6345;",
    "捆" => "&#x6346;",
    "捇" => "&#x6347;",
    "捈" => "&#x6348;",
    "捉" => "&#x6349;",
    "捊" => "&#x634A;",
    "捋" => "&#x634B;",
    "捌" => "&#x634C;",
    "捍" => "&#x634D;",
    "捎" => "&#x634E;",
    "捏" => "&#x634F;",
    "捐" => "&#x6350;",
    "捑" => "&#x6351;",
    "捒" => "&#x6352;",
    "捓" => "&#x6353;",
    "捔" => "&#x6354;",
    "捕" => "&#x6355;",
    "捖" => "&#x6356;",
    "捗" => "&#x6357;",
    "捘" => "&#x6358;",
    "捙" => "&#x6359;",
    "捚" => "&#x635A;",
    "捛" => "&#x635B;",
    "捜" => "&#x635C;",
    "捝" => "&#x635D;",
    "捞" => "&#x635E;",
    "损" => "&#x635F;",
    "捠" => "&#x6360;",
    "捡" => "&#x6361;",
    "换" => "&#x6362;",
    "捣" => "&#x6363;",
    "捤" => "&#x6364;",
    "捥" => "&#x6365;",
    "捦" => "&#x6366;",
    "捧" => "&#x6367;",
    "捨" => "&#x6368;",
    "捩" => "&#x6369;",
    "捪" => "&#x636A;",
    "捫" => "&#x636B;",
    "捬" => "&#x636C;",
    "捭" => "&#x636D;",
    "据" => "&#x636E;",
    "捯" => "&#x636F;",
    "捰" => "&#x6370;",
    "捱" => "&#x6371;",
    "捲" => "&#x6372;",
    "捳" => "&#x6373;",
    "捴" => "&#x6374;",
    "捵" => "&#x6375;",
    "捶" => "&#x6376;",
    "捷" => "&#x6377;",
    "捸" => "&#x6378;",
    "捹" => "&#x6379;",
    "捺" => "&#x637A;",
    "捻" => "&#x637B;",
    "捼" => "&#x637C;",
    "捽" => "&#x637D;",
    "捾" => "&#x637E;",
    "捿" => "&#x637F;",
    "掀" => "&#x6380;",
    "掁" => "&#x6381;",
    "掂" => "&#x6382;",
    "掃" => "&#x6383;",
    "掄" => "&#x6384;",
    "掅" => "&#x6385;",
    "掆" => "&#x6386;",
    "掇" => "&#x6387;",
    "授" => "&#x6388;",
    "掉" => "&#x6389;",
    "掊" => "&#x638A;",
    "掋" => "&#x638B;",
    "掌" => "&#x638C;",
    "掍" => "&#x638D;",
    "掎" => "&#x638E;",
    "掏" => "&#x638F;",
    "掐" => "&#x6390;",
    "掑" => "&#x6391;",
    "排" => "&#x6392;",
    "掓" => "&#x6393;",
    "掔" => "&#x6394;",
    "掕" => "&#x6395;",
    "掖" => "&#x6396;",
    "掗" => "&#x6397;",
    "掘" => "&#x6398;",
    "掙" => "&#x6399;",
    "掚" => "&#x639A;",
    "掛" => "&#x639B;",
    "掜" => "&#x639C;",
    "掝" => "&#x639D;",
    "掞" => "&#x639E;",
    "掟" => "&#x639F;",
    "掠" => "&#x63A0;",
    "採" => "&#x63A1;",
    "探" => "&#x63A2;",
    "掣" => "&#x63A3;",
    "掤" => "&#x63A4;",
    "接" => "&#x63A5;",
    "掦" => "&#x63A6;",
    "控" => "&#x63A7;",
    "推" => "&#x63A8;",
    "掩" => "&#x63A9;",
    "措" => "&#x63AA;",
    "掫" => "&#x63AB;",
    "掬" => "&#x63AC;",
    "掭" => "&#x63AD;",
    "掮" => "&#x63AE;",
    "掯" => "&#x63AF;",
    "掰" => "&#x63B0;",
    "掱" => "&#x63B1;",
    "掲" => "&#x63B2;",
    "掳" => "&#x63B3;",
    "掴" => "&#x63B4;",
    "掵" => "&#x63B5;",
    "掶" => "&#x63B6;",
    "掷" => "&#x63B7;",
    "掸" => "&#x63B8;",
    "掹" => "&#x63B9;",
    "掺" => "&#x63BA;",
    "掻" => "&#x63BB;",
    "掼" => "&#x63BC;",
    "掽" => "&#x63BD;",
    "掾" => "&#x63BE;",
    "掿" => "&#x63BF;",
    "揀" => "&#x63C0;",
    "揁" => "&#x63C1;",
    "揂" => "&#x63C2;",
    "揃" => "&#x63C3;",
    "揄" => "&#x63C4;",
    "揅" => "&#x63C5;",
    "揆" => "&#x63C6;",
    "揇" => "&#x63C7;",
    "揈" => "&#x63C8;",
    "揉" => "&#x63C9;",
    "揊" => "&#x63CA;",
    "揋" => "&#x63CB;",
    "揌" => "&#x63CC;",
    "揍" => "&#x63CD;",
    "揎" => "&#x63CE;",
    "描" => "&#x63CF;",
    "提" => "&#x63D0;",
    "揑" => "&#x63D1;",
    "插" => "&#x63D2;",
    "揓" => "&#x63D3;",
    "揔" => "&#x63D4;",
    "揕" => "&#x63D5;",
    "揖" => "&#x63D6;",
    "揗" => "&#x63D7;",
    "揘" => "&#x63D8;",
    "揙" => "&#x63D9;",
    "揚" => "&#x63DA;",
    "換" => "&#x63DB;",
    "揜" => "&#x63DC;",
    "揝" => "&#x63DD;",
    "揞" => "&#x63DE;",
    "揟" => "&#x63DF;",
    "揠" => "&#x63E0;",
    "握" => "&#x63E1;",
    "揢" => "&#x63E2;",
    "揣" => "&#x63E3;",
    "揤" => "&#x63E4;",
    "揥" => "&#x63E5;",
    "揦" => "&#x63E6;",
    "揧" => "&#x63E7;",
    "揨" => "&#x63E8;",
    "揩" => "&#x63E9;",
    "揪" => "&#x63EA;",
    "揫" => "&#x63EB;",
    "揬" => "&#x63EC;",
    "揭" => "&#x63ED;",
    "揮" => "&#x63EE;",
    "揯" => "&#x63EF;",
    "揰" => "&#x63F0;",
    "揱" => "&#x63F1;",
    "揲" => "&#x63F2;",
    "揳" => "&#x63F3;",
    "援" => "&#x63F4;",
    "揵" => "&#x63F5;",
    "揶" => "&#x63F6;",
    "揷" => "&#x63F7;",
    "揸" => "&#x63F8;",
    "揹" => "&#x63F9;",
    "揺" => "&#x63FA;",
    "揻" => "&#x63FB;",
    "揼" => "&#x63FC;",
    "揽" => "&#x63FD;",
    "揾" => "&#x63FE;",
    "揿" => "&#x63FF;",
    "搀" => "&#x6400;",
    "搁" => "&#x6401;",
    "搂" => "&#x6402;",
    "搃" => "&#x6403;",
    "搄" => "&#x6404;",
    "搅" => "&#x6405;",
    "搆" => "&#x6406;",
    "搇" => "&#x6407;",
    "搈" => "&#x6408;",
    "搉" => "&#x6409;",
    "搊" => "&#x640A;",
    "搋" => "&#x640B;",
    "搌" => "&#x640C;",
    "損" => "&#x640D;",
    "搎" => "&#x640E;",
    "搏" => "&#x640F;",
    "搐" => "&#x6410;",
    "搑" => "&#x6411;",
    "搒" => "&#x6412;",
    "搓" => "&#x6413;",
    "搔" => "&#x6414;",
    "搕" => "&#x6415;",
    "搖" => "&#x6416;",
    "搗" => "&#x6417;",
    "搘" => "&#x6418;",
    "搙" => "&#x6419;",
    "搚" => "&#x641A;",
    "搛" => "&#x641B;",
    "搜" => "&#x641C;",
    "搝" => "&#x641D;",
    "搞" => "&#x641E;",
    "搟" => "&#x641F;",
    "搠" => "&#x6420;",
    "搡" => "&#x6421;",
    "搢" => "&#x6422;",
    "搣" => "&#x6423;",
    "搤" => "&#x6424;",
    "搥" => "&#x6425;",
    "搦" => "&#x6426;",
    "搧" => "&#x6427;",
    "搨" => "&#x6428;",
    "搩" => "&#x6429;",
    "搪" => "&#x642A;",
    "搫" => "&#x642B;",
    "搬" => "&#x642C;",
    "搭" => "&#x642D;",
    "搮" => "&#x642E;",
    "搯" => "&#x642F;",
    "搰" => "&#x6430;",
    "搱" => "&#x6431;",
    "搲" => "&#x6432;",
    "搳" => "&#x6433;",
    "搴" => "&#x6434;",
    "搵" => "&#x6435;",
    "搶" => "&#x6436;",
    "搷" => "&#x6437;",
    "搸" => "&#x6438;",
    "搹" => "&#x6439;",
    "携" => "&#x643A;",
    "搻" => "&#x643B;",
    "搼" => "&#x643C;",
    "搽" => "&#x643D;",
    "搾" => "&#x643E;",
    "搿" => "&#x643F;",
    "摀" => "&#x6440;",
    "摁" => "&#x6441;",
    "摂" => "&#x6442;",
    "摃" => "&#x6443;",
    "摄" => "&#x6444;",
    "摅" => "&#x6445;",
    "摆" => "&#x6446;",
    "摇" => "&#x6447;",
    "摈" => "&#x6448;",
    "摉" => "&#x6449;",
    "摊" => "&#x644A;",
    "摋" => "&#x644B;",
    "摌" => "&#x644C;",
    "摍" => "&#x644D;",
    "摎" => "&#x644E;",
    "摏" => "&#x644F;",
    "摐" => "&#x6450;",
    "摑" => "&#x6451;",
    "摒" => "&#x6452;",
    "摓" => "&#x6453;",
    "摔" => "&#x6454;",
    "摕" => "&#x6455;",
    "摖" => "&#x6456;",
    "摗" => "&#x6457;",
    "摘" => "&#x6458;",
    "摙" => "&#x6459;",
    "摚" => "&#x645A;",
    "摛" => "&#x645B;",
    "摜" => "&#x645C;",
    "摝" => "&#x645D;",
    "摞" => "&#x645E;",
    "摟" => "&#x645F;",
    "摠" => "&#x6460;",
    "摡" => "&#x6461;",
    "摢" => "&#x6462;",
    "摣" => "&#x6463;",
    "摤" => "&#x6464;",
    "摥" => "&#x6465;",
    "摦" => "&#x6466;",
    "摧" => "&#x6467;",
    "摨" => "&#x6468;",
    "摩" => "&#x6469;",
    "摪" => "&#x646A;",
    "摫" => "&#x646B;",
    "摬" => "&#x646C;",
    "摭" => "&#x646D;",
    "摮" => "&#x646E;",
    "摯" => "&#x646F;",
    "摰" => "&#x6470;",
    "摱" => "&#x6471;",
    "摲" => "&#x6472;",
    "摳" => "&#x6473;",
    "摴" => "&#x6474;",
    "摵" => "&#x6475;",
    "摶" => "&#x6476;",
    "摷" => "&#x6477;",
    "摸" => "&#x6478;",
    "摹" => "&#x6479;",
    "摺" => "&#x647A;",
    "摻" => "&#x647B;",
    "摼" => "&#x647C;",
    "摽" => "&#x647D;",
    "摾" => "&#x647E;",
    "摿" => "&#x647F;",
    "撀" => "&#x6480;",
    "撁" => "&#x6481;",
    "撂" => "&#x6482;",
    "撃" => "&#x6483;",
    "撄" => "&#x6484;",
    "撅" => "&#x6485;",
    "撆" => "&#x6486;",
    "撇" => "&#x6487;",
    "撈" => "&#x6488;",
    "撉" => "&#x6489;",
    "撊" => "&#x648A;",
    "撋" => "&#x648B;",
    "撌" => "&#x648C;",
    "撍" => "&#x648D;",
    "撎" => "&#x648E;",
    "撏" => "&#x648F;",
    "撐" => "&#x6490;",
    "撑" => "&#x6491;",
    "撒" => "&#x6492;",
    "撓" => "&#x6493;",
    "撔" => "&#x6494;",
    "撕" => "&#x6495;",
    "撖" => "&#x6496;",
    "撗" => "&#x6497;",
    "撘" => "&#x6498;",
    "撙" => "&#x6499;",
    "撚" => "&#x649A;",
    "撛" => "&#x649B;",
    "撜" => "&#x649C;",
    "撝" => "&#x649D;",
    "撞" => "&#x649E;",
    "撟" => "&#x649F;",
    "撠" => "&#x64A0;",
    "撡" => "&#x64A1;",
    "撢" => "&#x64A2;",
    "撣" => "&#x64A3;",
    "撤" => "&#x64A4;",
    "撥" => "&#x64A5;",
    "撦" => "&#x64A6;",
    "撧" => "&#x64A7;",
    "撨" => "&#x64A8;",
    "撩" => "&#x64A9;",
    "撪" => "&#x64AA;",
    "撫" => "&#x64AB;",
    "撬" => "&#x64AC;",
    "播" => "&#x64AD;",
    "撮" => "&#x64AE;",
    "撯" => "&#x64AF;",
    "撰" => "&#x64B0;",
    "撱" => "&#x64B1;",
    "撲" => "&#x64B2;",
    "撳" => "&#x64B3;",
    "撴" => "&#x64B4;",
    "撵" => "&#x64B5;",
    "撶" => "&#x64B6;",
    "撷" => "&#x64B7;",
    "撸" => "&#x64B8;",
    "撹" => "&#x64B9;",
    "撺" => "&#x64BA;",
    "撻" => "&#x64BB;",
    "撼" => "&#x64BC;",
    "撽" => "&#x64BD;",
    "撾" => "&#x64BE;",
    "撿" => "&#x64BF;",
    "擀" => "&#x64C0;",
    "擁" => "&#x64C1;",
    "擂" => "&#x64C2;",
    "擃" => "&#x64C3;",
    "擄" => "&#x64C4;",
    "擅" => "&#x64C5;",
    "擆" => "&#x64C6;",
    "擇" => "&#x64C7;",
    "擈" => "&#x64C8;",
    "擉" => "&#x64C9;",
    "擊" => "&#x64CA;",
    "擋" => "&#x64CB;",
    "擌" => "&#x64CC;",
    "操" => "&#x64CD;",
    "擎" => "&#x64CE;",
    "擏" => "&#x64CF;",
    "擐" => "&#x64D0;",
    "擑" => "&#x64D1;",
    "擒" => "&#x64D2;",
    "擓" => "&#x64D3;",
    "擔" => "&#x64D4;",
    "擕" => "&#x64D5;",
    "擖" => "&#x64D6;",
    "擗" => "&#x64D7;",
    "擘" => "&#x64D8;",
    "擙" => "&#x64D9;",
    "據" => "&#x64DA;",
    "擛" => "&#x64DB;",
    "擜" => "&#x64DC;",
    "擝" => "&#x64DD;",
    "擞" => "&#x64DE;",
    "擟" => "&#x64DF;",
    "擠" => "&#x64E0;",
    "擡" => "&#x64E1;",
    "擢" => "&#x64E2;",
    "擣" => "&#x64E3;",
    "擤" => "&#x64E4;",
    "擥" => "&#x64E5;",
    "擦" => "&#x64E6;",
    "擧" => "&#x64E7;",
    "擨" => "&#x64E8;",
    "擩" => "&#x64E9;",
    "擪" => "&#x64EA;",
    "擫" => "&#x64EB;",
    "擬" => "&#x64EC;",
    "擭" => "&#x64ED;",
    "擮" => "&#x64EE;",
    "擯" => "&#x64EF;",
    "擰" => "&#x64F0;",
    "擱" => "&#x64F1;",
    "擲" => "&#x64F2;",
    "擳" => "&#x64F3;",
    "擴" => "&#x64F4;",
    "擵" => "&#x64F5;",
    "擶" => "&#x64F6;",
    "擷" => "&#x64F7;",
    "擸" => "&#x64F8;",
    "擹" => "&#x64F9;",
    "擺" => "&#x64FA;",
    "擻" => "&#x64FB;",
    "擼" => "&#x64FC;",
    "擽" => "&#x64FD;",
    "擾" => "&#x64FE;",
    "擿" => "&#x64FF;",
    "攀" => "&#x6500;",
    "攁" => "&#x6501;",
    "攂" => "&#x6502;",
    "攃" => "&#x6503;",
    "攄" => "&#x6504;",
    "攅" => "&#x6505;",
    "攆" => "&#x6506;",
    "攇" => "&#x6507;",
    "攈" => "&#x6508;",
    "攉" => "&#x6509;",
    "攊" => "&#x650A;",
    "攋" => "&#x650B;",
    "攌" => "&#x650C;",
    "攍" => "&#x650D;",
    "攎" => "&#x650E;",
    "攏" => "&#x650F;",
    "攐" => "&#x6510;",
    "攑" => "&#x6511;",
    "攒" => "&#x6512;",
    "攓" => "&#x6513;",
    "攔" => "&#x6514;",
    "攕" => "&#x6515;",
    "攖" => "&#x6516;",
    "攗" => "&#x6517;",
    "攘" => "&#x6518;",
    "攙" => "&#x6519;",
    "攚" => "&#x651A;",
    "攛" => "&#x651B;",
    "攜" => "&#x651C;",
    "攝" => "&#x651D;",
    "攞" => "&#x651E;",
    "攟" => "&#x651F;",
    "攠" => "&#x6520;",
    "攡" => "&#x6521;",
    "攢" => "&#x6522;",
    "攣" => "&#x6523;",
    "攤" => "&#x6524;",
    "攥" => "&#x6525;",
    "攦" => "&#x6526;",
    "攧" => "&#x6527;",
    "攨" => "&#x6528;",
    "攩" => "&#x6529;",
    "攪" => "&#x652A;",
    "攫" => "&#x652B;",
    "攬" => "&#x652C;",
    "攭" => "&#x652D;",
    "攮" => "&#x652E;",
    "支" => "&#x652F;",
    "攰" => "&#x6530;",
    "攱" => "&#x6531;",
    "攲" => "&#x6532;",
    "攳" => "&#x6533;",
    "攴" => "&#x6534;",
    "攵" => "&#x6535;",
    "收" => "&#x6536;",
    "攷" => "&#x6537;",
    "攸" => "&#x6538;",
    "改" => "&#x6539;",
    "攺" => "&#x653A;",
    "攻" => "&#x653B;",
    "攼" => "&#x653C;",
    "攽" => "&#x653D;",
    "放" => "&#x653E;",
    "政" => "&#x653F;",
    "敀" => "&#x6540;",
    "敁" => "&#x6541;",
    "敂" => "&#x6542;",
    "敃" => "&#x6543;",
    "敄" => "&#x6544;",
    "故" => "&#x6545;",
    "敆" => "&#x6546;",
    "敇" => "&#x6547;",
    "效" => "&#x6548;",
    "敉" => "&#x6549;",
    "敊" => "&#x654A;",
    "敋" => "&#x654B;",
    "敌" => "&#x654C;",
    "敍" => "&#x654D;",
    "敎" => "&#x654E;",
    "敏" => "&#x654F;",
    "敐" => "&#x6550;",
    "救" => "&#x6551;",
    "敒" => "&#x6552;",
    "敓" => "&#x6553;",
    "敔" => "&#x6554;",
    "敕" => "&#x6555;",
    "敖" => "&#x6556;",
    "敗" => "&#x6557;",
    "敘" => "&#x6558;",
    "教" => "&#x6559;",
    "敚" => "&#x655A;",
    "敛" => "&#x655B;",
    "敜" => "&#x655C;",
    "敝" => "&#x655D;",
    "敞" => "&#x655E;",
    "敟" => "&#x655F;",
    "敠" => "&#x6560;",
    "敡" => "&#x6561;",
    "敢" => "&#x6562;",
    "散" => "&#x6563;",
    "敤" => "&#x6564;",
    "敥" => "&#x6565;",
    "敦" => "&#x6566;",
    "敧" => "&#x6567;",
    "敨" => "&#x6568;",
    "敩" => "&#x6569;",
    "敪" => "&#x656A;",
    "敫" => "&#x656B;",
    "敬" => "&#x656C;",
    "敭" => "&#x656D;",
    "敮" => "&#x656E;",
    "敯" => "&#x656F;",
    "数" => "&#x6570;",
    "敱" => "&#x6571;",
    "敲" => "&#x6572;",
    "敳" => "&#x6573;",
    "整" => "&#x6574;",
    "敵" => "&#x6575;",
    "敶" => "&#x6576;",
    "敷" => "&#x6577;",
    "數" => "&#x6578;",
    "敹" => "&#x6579;",
    "敺" => "&#x657A;",
    "敻" => "&#x657B;",
    "敼" => "&#x657C;",
    "敽" => "&#x657D;",
    "敾" => "&#x657E;",
    "敿" => "&#x657F;",
    "斀" => "&#x6580;",
    "斁" => "&#x6581;",
    "斂" => "&#x6582;",
    "斃" => "&#x6583;",
    "斄" => "&#x6584;",
    "斅" => "&#x6585;",
    "斆" => "&#x6586;",
    "文" => "&#x6587;",
    "斈" => "&#x6588;",
    "斉" => "&#x6589;",
    "斊" => "&#x658A;",
    "斋" => "&#x658B;",
    "斌" => "&#x658C;",
    "斍" => "&#x658D;",
    "斎" => "&#x658E;",
    "斏" => "&#x658F;",
    "斐" => "&#x6590;",
    "斑" => "&#x6591;",
    "斒" => "&#x6592;",
    "斓" => "&#x6593;",
    "斔" => "&#x6594;",
    "斕" => "&#x6595;",
    "斖" => "&#x6596;",
    "斗" => "&#x6597;",
    "斘" => "&#x6598;",
    "料" => "&#x6599;",
    "斚" => "&#x659A;",
    "斛" => "&#x659B;",
    "斜" => "&#x659C;",
    "斝" => "&#x659D;",
    "斞" => "&#x659E;",
    "斟" => "&#x659F;",
    "斠" => "&#x65A0;",
    "斡" => "&#x65A1;",
    "斢" => "&#x65A2;",
    "斣" => "&#x65A3;",
    "斤" => "&#x65A4;",
    "斥" => "&#x65A5;",
    "斦" => "&#x65A6;",
    "斧" => "&#x65A7;",
    "斨" => "&#x65A8;",
    "斩" => "&#x65A9;",
    "斪" => "&#x65AA;",
    "斫" => "&#x65AB;",
    "斬" => "&#x65AC;",
    "断" => "&#x65AD;",
    "斮" => "&#x65AE;",
    "斯" => "&#x65AF;",
    "新" => "&#x65B0;",
    "斱" => "&#x65B1;",
    "斲" => "&#x65B2;",
    "斳" => "&#x65B3;",
    "斴" => "&#x65B4;",
    "斵" => "&#x65B5;",
    "斶" => "&#x65B6;",
    "斷" => "&#x65B7;",
    "斸" => "&#x65B8;",
    "方" => "&#x65B9;",
    "斺" => "&#x65BA;",
    "斻" => "&#x65BB;",
    "於" => "&#x65BC;",
    "施" => "&#x65BD;",
    "斾" => "&#x65BE;",
    "斿" => "&#x65BF;",
    "旀" => "&#x65C0;",
    "旁" => "&#x65C1;",
    "旂" => "&#x65C2;",
    "旃" => "&#x65C3;",
    "旄" => "&#x65C4;",
    "旅" => "&#x65C5;",
    "旆" => "&#x65C6;",
    "旇" => "&#x65C7;",
    "旈" => "&#x65C8;",
    "旉" => "&#x65C9;",
    "旊" => "&#x65CA;",
    "旋" => "&#x65CB;",
    "旌" => "&#x65CC;",
    "旍" => "&#x65CD;",
    "旎" => "&#x65CE;",
    "族" => "&#x65CF;",
    "旐" => "&#x65D0;",
    "旑" => "&#x65D1;",
    "旒" => "&#x65D2;",
    "旓" => "&#x65D3;",
    "旔" => "&#x65D4;",
    "旕" => "&#x65D5;",
    "旖" => "&#x65D6;",
    "旗" => "&#x65D7;",
    "旘" => "&#x65D8;",
    "旙" => "&#x65D9;",
    "旚" => "&#x65DA;",
    "旛" => "&#x65DB;",
    "旜" => "&#x65DC;",
    "旝" => "&#x65DD;",
    "旞" => "&#x65DE;",
    "旟" => "&#x65DF;",
    "无" => "&#x65E0;",
    "旡" => "&#x65E1;",
    "既" => "&#x65E2;",
    "旣" => "&#x65E3;",
    "旤" => "&#x65E4;",
    "日" => "&#x65E5;",
    "旦" => "&#x65E6;",
    "旧" => "&#x65E7;",
    "旨" => "&#x65E8;",
    "早" => "&#x65E9;",
    "旪" => "&#x65EA;",
    "旫" => "&#x65EB;",
    "旬" => "&#x65EC;",
    "旭" => "&#x65ED;",
    "旮" => "&#x65EE;",
    "旯" => "&#x65EF;",
    "旰" => "&#x65F0;",
    "旱" => "&#x65F1;",
    "旲" => "&#x65F2;",
    "旳" => "&#x65F3;",
    "旴" => "&#x65F4;",
    "旵" => "&#x65F5;",
    "时" => "&#x65F6;",
    "旷" => "&#x65F7;",
    "旸" => "&#x65F8;",
    "旹" => "&#x65F9;",
    "旺" => "&#x65FA;",
    "旻" => "&#x65FB;",
    "旼" => "&#x65FC;",
    "旽" => "&#x65FD;",
    "旾" => "&#x65FE;",
    "旿" => "&#x65FF;",
    "昀" => "&#x6600;",
    "昁" => "&#x6601;",
    "昂" => "&#x6602;",
    "昃" => "&#x6603;",
    "昄" => "&#x6604;",
    "昅" => "&#x6605;",
    "昆" => "&#x6606;",
    "昇" => "&#x6607;",
    "昈" => "&#x6608;",
    "昉" => "&#x6609;",
    "昊" => "&#x660A;",
    "昋" => "&#x660B;",
    "昌" => "&#x660C;",
    "昍" => "&#x660D;",
    "明" => "&#x660E;",
    "昏" => "&#x660F;",
    "昐" => "&#x6610;",
    "昑" => "&#x6611;",
    "昒" => "&#x6612;",
    "易" => "&#x6613;",
    "昔" => "&#x6614;",
    "昕" => "&#x6615;",
    "昖" => "&#x6616;",
    "昗" => "&#x6617;",
    "昘" => "&#x6618;",
    "昙" => "&#x6619;",
    "昚" => "&#x661A;",
    "昛" => "&#x661B;",
    "昜" => "&#x661C;",
    "昝" => "&#x661D;",
    "昞" => "&#x661E;",
    "星" => "&#x661F;",
    "映" => "&#x6620;",
    "昡" => "&#x6621;",
    "昢" => "&#x6622;",
    "昣" => "&#x6623;",
    "昤" => "&#x6624;",
    "春" => "&#x6625;",
    "昦" => "&#x6626;",
    "昧" => "&#x6627;",
    "昨" => "&#x6628;",
    "昩" => "&#x6629;",
    "昪" => "&#x662A;",
    "昫" => "&#x662B;",
    "昬" => "&#x662C;",
    "昭" => "&#x662D;",
    "昮" => "&#x662E;",
    "是" => "&#x662F;",
    "昰" => "&#x6630;",
    "昱" => "&#x6631;",
    "昲" => "&#x6632;",
    "昳" => "&#x6633;",
    "昴" => "&#x6634;",
    "昵" => "&#x6635;",
    "昶" => "&#x6636;",
    "昷" => "&#x6637;",
    "昸" => "&#x6638;",
    "昹" => "&#x6639;",
    "昺" => "&#x663A;",
    "昻" => "&#x663B;",
    "昼" => "&#x663C;",
    "昽" => "&#x663D;",
    "显" => "&#x663E;",
    "昿" => "&#x663F;",
    "晀" => "&#x6640;",
    "晁" => "&#x6641;",
    "時" => "&#x6642;",
    "晃" => "&#x6643;",
    "晄" => "&#x6644;",
    "晅" => "&#x6645;",
    "晆" => "&#x6646;",
    "晇" => "&#x6647;",
    "晈" => "&#x6648;",
    "晉" => "&#x6649;",
    "晊" => "&#x664A;",
    "晋" => "&#x664B;",
    "晌" => "&#x664C;",
    "晍" => "&#x664D;",
    "晎" => "&#x664E;",
    "晏" => "&#x664F;",
    "晐" => "&#x6650;",
    "晑" => "&#x6651;",
    "晒" => "&#x6652;",
    "晓" => "&#x6653;",
    "晔" => "&#x6654;",
    "晕" => "&#x6655;",
    "晖" => "&#x6656;",
    "晗" => "&#x6657;",
    "晘" => "&#x6658;",
    "晙" => "&#x6659;",
    "晚" => "&#x665A;",
    "晛" => "&#x665B;",
    "晜" => "&#x665C;",
    "晝" => "&#x665D;",
    "晞" => "&#x665E;",
    "晟" => "&#x665F;",
    "晠" => "&#x6660;",
    "晡" => "&#x6661;",
    "晢" => "&#x6662;",
    "晣" => "&#x6663;",
    "晤" => "&#x6664;",
    "晥" => "&#x6665;",
    "晦" => "&#x6666;",
    "晧" => "&#x6667;",
    "晨" => "&#x6668;",
    "晩" => "&#x6669;",
    "晪" => "&#x666A;",
    "晫" => "&#x666B;",
    "晬" => "&#x666C;",
    "晭" => "&#x666D;",
    "普" => "&#x666E;",
    "景" => "&#x666F;",
    "晰" => "&#x6670;",
    "晱" => "&#x6671;",
    "晲" => "&#x6672;",
    "晳" => "&#x6673;",
    "晴" => "&#x6674;",
    "晵" => "&#x6675;",
    "晶" => "&#x6676;",
    "晷" => "&#x6677;",
    "晸" => "&#x6678;",
    "晹" => "&#x6679;",
    "智" => "&#x667A;",
    "晻" => "&#x667B;",
    "晼" => "&#x667C;",
    "晽" => "&#x667D;",
    "晾" => "&#x667E;",
    "晿" => "&#x667F;",
    "暀" => "&#x6680;",
    "暁" => "&#x6681;",
    "暂" => "&#x6682;",
    "暃" => "&#x6683;",
    "暄" => "&#x6684;",
    "暅" => "&#x6685;",
    "暆" => "&#x6686;",
    "暇" => "&#x6687;",
    "暈" => "&#x6688;",
    "暉" => "&#x6689;",
    "暊" => "&#x668A;",
    "暋" => "&#x668B;",
    "暌" => "&#x668C;",
    "暍" => "&#x668D;",
    "暎" => "&#x668E;",
    "暏" => "&#x668F;",
    "暐" => "&#x6690;",
    "暑" => "&#x6691;",
    "暒" => "&#x6692;",
    "暓" => "&#x6693;",
    "暔" => "&#x6694;",
    "暕" => "&#x6695;",
    "暖" => "&#x6696;",
    "暗" => "&#x6697;",
    "暘" => "&#x6698;",
    "暙" => "&#x6699;",
    "暚" => "&#x669A;",
    "暛" => "&#x669B;",
    "暜" => "&#x669C;",
    "暝" => "&#x669D;",
    "暞" => "&#x669E;",
    "暟" => "&#x669F;",
    "暠" => "&#x66A0;",
    "暡" => "&#x66A1;",
    "暢" => "&#x66A2;",
    "暣" => "&#x66A3;",
    "暤" => "&#x66A4;",
    "暥" => "&#x66A5;",
    "暦" => "&#x66A6;",
    "暧" => "&#x66A7;",
    "暨" => "&#x66A8;",
    "暩" => "&#x66A9;",
    "暪" => "&#x66AA;",
    "暫" => "&#x66AB;",
    "暬" => "&#x66AC;",
    "暭" => "&#x66AD;",
    "暮" => "&#x66AE;",
    "暯" => "&#x66AF;",
    "暰" => "&#x66B0;",
    "暱" => "&#x66B1;",
    "暲" => "&#x66B2;",
    "暳" => "&#x66B3;",
    "暴" => "&#x66B4;",
    "暵" => "&#x66B5;",
    "暶" => "&#x66B6;",
    "暷" => "&#x66B7;",
    "暸" => "&#x66B8;",
    "暹" => "&#x66B9;",
    "暺" => "&#x66BA;",
    "暻" => "&#x66BB;",
    "暼" => "&#x66BC;",
    "暽" => "&#x66BD;",
    "暾" => "&#x66BE;",
    "暿" => "&#x66BF;",
    "曀" => "&#x66C0;",
    "曁" => "&#x66C1;",
    "曂" => "&#x66C2;",
    "曃" => "&#x66C3;",
    "曄" => "&#x66C4;",
    "曅" => "&#x66C5;",
    "曆" => "&#x66C6;",
    "曇" => "&#x66C7;",
    "曈" => "&#x66C8;",
    "曉" => "&#x66C9;",
    "曊" => "&#x66CA;",
    "曋" => "&#x66CB;",
    "曌" => "&#x66CC;",
    "曍" => "&#x66CD;",
    "曎" => "&#x66CE;",
    "曏" => "&#x66CF;",
    "曐" => "&#x66D0;",
    "曑" => "&#x66D1;",
    "曒" => "&#x66D2;",
    "曓" => "&#x66D3;",
    "曔" => "&#x66D4;",
    "曕" => "&#x66D5;",
    "曖" => "&#x66D6;",
    "曗" => "&#x66D7;",
    "曘" => "&#x66D8;",
    "曙" => "&#x66D9;",
    "曚" => "&#x66DA;",
    "曛" => "&#x66DB;",
    "曜" => "&#x66DC;",
    "曝" => "&#x66DD;",
    "曞" => "&#x66DE;",
    "曟" => "&#x66DF;",
    "曠" => "&#x66E0;",
    "曡" => "&#x66E1;",
    "曢" => "&#x66E2;",
    "曣" => "&#x66E3;",
    "曤" => "&#x66E4;",
    "曥" => "&#x66E5;",
    "曦" => "&#x66E6;",
    "曧" => "&#x66E7;",
    "曨" => "&#x66E8;",
    "曩" => "&#x66E9;",
    "曪" => "&#x66EA;",
    "曫" => "&#x66EB;",
    "曬" => "&#x66EC;",
    "曭" => "&#x66ED;",
    "曮" => "&#x66EE;",
    "曯" => "&#x66EF;",
    "曰" => "&#x66F0;",
    "曱" => "&#x66F1;",
    "曲" => "&#x66F2;",
    "曳" => "&#x66F3;",
    "更" => "&#x66F4;",
    "曵" => "&#x66F5;",
    "曶" => "&#x66F6;",
    "曷" => "&#x66F7;",
    "書" => "&#x66F8;",
    "曹" => "&#x66F9;",
    "曺" => "&#x66FA;",
    "曻" => "&#x66FB;",
    "曼" => "&#x66FC;",
    "曽" => "&#x66FD;",
    "曾" => "&#x66FE;",
    "替" => "&#x66FF;",
    "最" => "&#x6700;",
    "朁" => "&#x6701;",
    "朂" => "&#x6702;",
    "會" => "&#x6703;",
    "朄" => "&#x6704;",
    "朅" => "&#x6705;",
    "朆" => "&#x6706;",
    "朇" => "&#x6707;",
    "月" => "&#x6708;",
    "有" => "&#x6709;",
    "朊" => "&#x670A;",
    "朋" => "&#x670B;",
    "朌" => "&#x670C;",
    "服" => "&#x670D;",
    "朎" => "&#x670E;",
    "朏" => "&#x670F;",
    "朐" => "&#x6710;",
    "朑" => "&#x6711;",
    "朒" => "&#x6712;",
    "朓" => "&#x6713;",
    "朔" => "&#x6714;",
    "朕" => "&#x6715;",
    "朖" => "&#x6716;",
    "朗" => "&#x6717;",
    "朘" => "&#x6718;",
    "朙" => "&#x6719;",
    "朚" => "&#x671A;",
    "望" => "&#x671B;",
    "朜" => "&#x671C;",
    "朝" => "&#x671D;",
    "朞" => "&#x671E;",
    "期" => "&#x671F;",
    "朠" => "&#x6720;",
    "朡" => "&#x6721;",
    "朢" => "&#x6722;",
    "朣" => "&#x6723;",
    "朤" => "&#x6724;",
    "朥" => "&#x6725;",
    "朦" => "&#x6726;",
    "朧" => "&#x6727;",
    "木" => "&#x6728;",
    "朩" => "&#x6729;",
    "未" => "&#x672A;",
    "末" => "&#x672B;",
    "本" => "&#x672C;",
    "札" => "&#x672D;",
    "朮" => "&#x672E;",
    "术" => "&#x672F;",
    "朰" => "&#x6730;",
    "朱" => "&#x6731;",
    "朲" => "&#x6732;",
    "朳" => "&#x6733;",
    "朴" => "&#x6734;",
    "朵" => "&#x6735;",
    "朶" => "&#x6736;",
    "朷" => "&#x6737;",
    "朸" => "&#x6738;",
    "朹" => "&#x6739;",
    "机" => "&#x673A;",
    "朻" => "&#x673B;",
    "朼" => "&#x673C;",
    "朽" => "&#x673D;",
    "朾" => "&#x673E;",
    "朿" => "&#x673F;",
    "杀" => "&#x6740;",
    "杁" => "&#x6741;",
    "杂" => "&#x6742;",
    "权" => "&#x6743;",
    "杄" => "&#x6744;",
    "杅" => "&#x6745;",
    "杆" => "&#x6746;",
    "杇" => "&#x6747;",
    "杈" => "&#x6748;",
    "杉" => "&#x6749;",
    "杊" => "&#x674A;",
    "杋" => "&#x674B;",
    "杌" => "&#x674C;",
    "杍" => "&#x674D;",
    "李" => "&#x674E;",
    "杏" => "&#x674F;",
    "材" => "&#x6750;",
    "村" => "&#x6751;",
    "杒" => "&#x6752;",
    "杓" => "&#x6753;",
    "杔" => "&#x6754;",
    "杕" => "&#x6755;",
    "杖" => "&#x6756;",
    "杗" => "&#x6757;",
    "杘" => "&#x6758;",
    "杙" => "&#x6759;",
    "杚" => "&#x675A;",
    "杛" => "&#x675B;",
    "杜" => "&#x675C;",
    "杝" => "&#x675D;",
    "杞" => "&#x675E;",
    "束" => "&#x675F;",
    "杠" => "&#x6760;",
    "条" => "&#x6761;",
    "杢" => "&#x6762;",
    "杣" => "&#x6763;",
    "杤" => "&#x6764;",
    "来" => "&#x6765;",
    "杦" => "&#x6766;",
    "杧" => "&#x6767;",
    "杨" => "&#x6768;",
    "杩" => "&#x6769;",
    "杪" => "&#x676A;",
    "杫" => "&#x676B;",
    "杬" => "&#x676C;",
    "杭" => "&#x676D;",
    "杮" => "&#x676E;",
    "杯" => "&#x676F;",
    "杰" => "&#x6770;",
    "東" => "&#x6771;",
    "杲" => "&#x6772;",
    "杳" => "&#x6773;",
    "杴" => "&#x6774;",
    "杵" => "&#x6775;",
    "杶" => "&#x6776;",
    "杷" => "&#x6777;",
    "杸" => "&#x6778;",
    "杹" => "&#x6779;",
    "杺" => "&#x677A;",
    "杻" => "&#x677B;",
    "杼" => "&#x677C;",
    "杽" => "&#x677D;",
    "松" => "&#x677E;",
    "板" => "&#x677F;",
    "枀" => "&#x6780;",
    "极" => "&#x6781;",
    "枂" => "&#x6782;",
    "枃" => "&#x6783;",
    "构" => "&#x6784;",
    "枅" => "&#x6785;",
    "枆" => "&#x6786;",
    "枇" => "&#x6787;",
    "枈" => "&#x6788;",
    "枉" => "&#x6789;",
    "枊" => "&#x678A;",
    "枋" => "&#x678B;",
    "枌" => "&#x678C;",
    "枍" => "&#x678D;",
    "枎" => "&#x678E;",
    "枏" => "&#x678F;",
    "析" => "&#x6790;",
    "枑" => "&#x6791;",
    "枒" => "&#x6792;",
    "枓" => "&#x6793;",
    "枔" => "&#x6794;",
    "枕" => "&#x6795;",
    "枖" => "&#x6796;",
    "林" => "&#x6797;",
    "枘" => "&#x6798;",
    "枙" => "&#x6799;",
    "枚" => "&#x679A;",
    "枛" => "&#x679B;",
    "果" => "&#x679C;",
    "枝" => "&#x679D;",
    "枞" => "&#x679E;",
    "枟" => "&#x679F;",
    "枠" => "&#x67A0;",
    "枡" => "&#x67A1;",
    "枢" => "&#x67A2;",
    "枣" => "&#x67A3;",
    "枤" => "&#x67A4;",
    "枥" => "&#x67A5;",
    "枦" => "&#x67A6;",
    "枧" => "&#x67A7;",
    "枨" => "&#x67A8;",
    "枩" => "&#x67A9;",
    "枪" => "&#x67AA;",
    "枫" => "&#x67AB;",
    "枬" => "&#x67AC;",
    "枭" => "&#x67AD;",
    "枮" => "&#x67AE;",
    "枯" => "&#x67AF;",
    "枰" => "&#x67B0;",
    "枱" => "&#x67B1;",
    "枲" => "&#x67B2;",
    "枳" => "&#x67B3;",
    "枴" => "&#x67B4;",
    "枵" => "&#x67B5;",
    "架" => "&#x67B6;",
    "枷" => "&#x67B7;",
    "枸" => "&#x67B8;",
    "枹" => "&#x67B9;",
    "枺" => "&#x67BA;",
    "枻" => "&#x67BB;",
    "枼" => "&#x67BC;",
    "枽" => "&#x67BD;",
    "枾" => "&#x67BE;",
    "枿" => "&#x67BF;",
    "柀" => "&#x67C0;",
    "柁" => "&#x67C1;",
    "柂" => "&#x67C2;",
    "柃" => "&#x67C3;",
    "柄" => "&#x67C4;",
    "柅" => "&#x67C5;",
    "柆" => "&#x67C6;",
    "柇" => "&#x67C7;",
    "柈" => "&#x67C8;",
    "柉" => "&#x67C9;",
    "柊" => "&#x67CA;",
    "柋" => "&#x67CB;",
    "柌" => "&#x67CC;",
    "柍" => "&#x67CD;",
    "柎" => "&#x67CE;",
    "柏" => "&#x67CF;",
    "某" => "&#x67D0;",
    "柑" => "&#x67D1;",
    "柒" => "&#x67D2;",
    "染" => "&#x67D3;",
    "柔" => "&#x67D4;",
    "柕" => "&#x67D5;",
    "柖" => "&#x67D6;",
    "柗" => "&#x67D7;",
    "柘" => "&#x67D8;",
    "柙" => "&#x67D9;",
    "柚" => "&#x67DA;",
    "柛" => "&#x67DB;",
    "柜" => "&#x67DC;",
    "柝" => "&#x67DD;",
    "柞" => "&#x67DE;",
    "柟" => "&#x67DF;",
    "柠" => "&#x67E0;",
    "柡" => "&#x67E1;",
    "柢" => "&#x67E2;",
    "柣" => "&#x67E3;",
    "柤" => "&#x67E4;",
    "查" => "&#x67E5;",
    "柦" => "&#x67E6;",
    "柧" => "&#x67E7;",
    "柨" => "&#x67E8;",
    "柩" => "&#x67E9;",
    "柪" => "&#x67EA;",
    "柫" => "&#x67EB;",
    "柬" => "&#x67EC;",
    "柭" => "&#x67ED;",
    "柮" => "&#x67EE;",
    "柯" => "&#x67EF;",
    "柰" => "&#x67F0;",
    "柱" => "&#x67F1;",
    "柲" => "&#x67F2;",
    "柳" => "&#x67F3;",
    "柴" => "&#x67F4;",
    "柵" => "&#x67F5;",
    "柶" => "&#x67F6;",
    "柷" => "&#x67F7;",
    "柸" => "&#x67F8;",
    "柹" => "&#x67F9;",
    "柺" => "&#x67FA;",
    "査" => "&#x67FB;",
    "柼" => "&#x67FC;",
    "柽" => "&#x67FD;",
    "柾" => "&#x67FE;",
    "柿" => "&#x67FF;",
    "栀" => "&#x6800;",
    "栁" => "&#x6801;",
    "栂" => "&#x6802;",
    "栃" => "&#x6803;",
    "栄" => "&#x6804;",
    "栅" => "&#x6805;",
    "栆" => "&#x6806;",
    "标" => "&#x6807;",
    "栈" => "&#x6808;",
    "栉" => "&#x6809;",
    "栊" => "&#x680A;",
    "栋" => "&#x680B;",
    "栌" => "&#x680C;",
    "栍" => "&#x680D;",
    "栎" => "&#x680E;",
    "栏" => "&#x680F;",
    "栐" => "&#x6810;",
    "树" => "&#x6811;",
    "栒" => "&#x6812;",
    "栓" => "&#x6813;",
    "栔" => "&#x6814;",
    "栕" => "&#x6815;",
    "栖" => "&#x6816;",
    "栗" => "&#x6817;",
    "栘" => "&#x6818;",
    "栙" => "&#x6819;",
    "栚" => "&#x681A;",
    "栛" => "&#x681B;",
    "栜" => "&#x681C;",
    "栝" => "&#x681D;",
    "栞" => "&#x681E;",
    "栟" => "&#x681F;",
    "栠" => "&#x6820;",
    "校" => "&#x6821;",
    "栢" => "&#x6822;",
    "栣" => "&#x6823;",
    "栤" => "&#x6824;",
    "栥" => "&#x6825;",
    "栦" => "&#x6826;",
    "栧" => "&#x6827;",
    "栨" => "&#x6828;",
    "栩" => "&#x6829;",
    "株" => "&#x682A;",
    "栫" => "&#x682B;",
    "栬" => "&#x682C;",
    "栭" => "&#x682D;",
    "栮" => "&#x682E;",
    "栯" => "&#x682F;",
    "栰" => "&#x6830;",
    "栱" => "&#x6831;",
    "栲" => "&#x6832;",
    "栳" => "&#x6833;",
    "栴" => "&#x6834;",
    "栵" => "&#x6835;",
    "栶" => "&#x6836;",
    "样" => "&#x6837;",
    "核" => "&#x6838;",
    "根" => "&#x6839;",
    "栺" => "&#x683A;",
    "栻" => "&#x683B;",
    "格" => "&#x683C;",
    "栽" => "&#x683D;",
    "栾" => "&#x683E;",
    "栿" => "&#x683F;",
    "桀" => "&#x6840;",
    "桁" => "&#x6841;",
    "桂" => "&#x6842;",
    "桃" => "&#x6843;",
    "桄" => "&#x6844;",
    "桅" => "&#x6845;",
    "框" => "&#x6846;",
    "桇" => "&#x6847;",
    "案" => "&#x6848;",
    "桉" => "&#x6849;",
    "桊" => "&#x684A;",
    "桋" => "&#x684B;",
    "桌" => "&#x684C;",
    "桍" => "&#x684D;",
    "桎" => "&#x684E;",
    "桏" => "&#x684F;",
    "桐" => "&#x6850;",
    "桑" => "&#x6851;",
    "桒" => "&#x6852;",
    "桓" => "&#x6853;",
    "桔" => "&#x6854;",
    "桕" => "&#x6855;",
    "桖" => "&#x6856;",
    "桗" => "&#x6857;",
    "桘" => "&#x6858;",
    "桙" => "&#x6859;",
    "桚" => "&#x685A;",
    "桛" => "&#x685B;",
    "桜" => "&#x685C;",
    "桝" => "&#x685D;",
    "桞" => "&#x685E;",
    "桟" => "&#x685F;",
    "桠" => "&#x6860;",
    "桡" => "&#x6861;",
    "桢" => "&#x6862;",
    "档" => "&#x6863;",
    "桤" => "&#x6864;",
    "桥" => "&#x6865;",
    "桦" => "&#x6866;",
    "桧" => "&#x6867;",
    "桨" => "&#x6868;",
    "桩" => "&#x6869;",
    "桪" => "&#x686A;",
    "桫" => "&#x686B;",
    "桬" => "&#x686C;",
    "桭" => "&#x686D;",
    "桮" => "&#x686E;",
    "桯" => "&#x686F;",
    "桰" => "&#x6870;",
    "桱" => "&#x6871;",
    "桲" => "&#x6872;",
    "桳" => "&#x6873;",
    "桴" => "&#x6874;",
    "桵" => "&#x6875;",
    "桶" => "&#x6876;",
    "桷" => "&#x6877;",
    "桸" => "&#x6878;",
    "桹" => "&#x6879;",
    "桺" => "&#x687A;",
    "桻" => "&#x687B;",
    "桼" => "&#x687C;",
    "桽" => "&#x687D;",
    "桾" => "&#x687E;",
    "桿" => "&#x687F;",
    "梀" => "&#x6880;",
    "梁" => "&#x6881;",
    "梂" => "&#x6882;",
    "梃" => "&#x6883;",
    "梄" => "&#x6884;",
    "梅" => "&#x6885;",
    "梆" => "&#x6886;",
    "梇" => "&#x6887;",
    "梈" => "&#x6888;",
    "梉" => "&#x6889;",
    "梊" => "&#x688A;",
    "梋" => "&#x688B;",
    "梌" => "&#x688C;",
    "梍" => "&#x688D;",
    "梎" => "&#x688E;",
    "梏" => "&#x688F;",
    "梐" => "&#x6890;",
    "梑" => "&#x6891;",
    "梒" => "&#x6892;",
    "梓" => "&#x6893;",
    "梔" => "&#x6894;",
    "梕" => "&#x6895;",
    "梖" => "&#x6896;",
    "梗" => "&#x6897;",
    "梘" => "&#x6898;",
    "梙" => "&#x6899;",
    "梚" => "&#x689A;",
    "梛" => "&#x689B;",
    "梜" => "&#x689C;",
    "條" => "&#x689D;",
    "梞" => "&#x689E;",
    "梟" => "&#x689F;",
    "梠" => "&#x68A0;",
    "梡" => "&#x68A1;",
    "梢" => "&#x68A2;",
    "梣" => "&#x68A3;",
    "梤" => "&#x68A4;",
    "梥" => "&#x68A5;",
    "梦" => "&#x68A6;",
    "梧" => "&#x68A7;",
    "梨" => "&#x68A8;",
    "梩" => "&#x68A9;",
    "梪" => "&#x68AA;",
    "梫" => "&#x68AB;",
    "梬" => "&#x68AC;",
    "梭" => "&#x68AD;",
    "梮" => "&#x68AE;",
    "梯" => "&#x68AF;",
    "械" => "&#x68B0;",
    "梱" => "&#x68B1;",
    "梲" => "&#x68B2;",
    "梳" => "&#x68B3;",
    "梴" => "&#x68B4;",
    "梵" => "&#x68B5;",
    "梶" => "&#x68B6;",
    "梷" => "&#x68B7;",
    "梸" => "&#x68B8;",
    "梹" => "&#x68B9;",
    "梺" => "&#x68BA;",
    "梻" => "&#x68BB;",
    "梼" => "&#x68BC;",
    "梽" => "&#x68BD;",
    "梾" => "&#x68BE;",
    "梿" => "&#x68BF;",
    "检" => "&#x68C0;",
    "棁" => "&#x68C1;",
    "棂" => "&#x68C2;",
    "棃" => "&#x68C3;",
    "棄" => "&#x68C4;",
    "棅" => "&#x68C5;",
    "棆" => "&#x68C6;",
    "棇" => "&#x68C7;",
    "棈" => "&#x68C8;",
    "棉" => "&#x68C9;",
    "棊" => "&#x68CA;",
    "棋" => "&#x68CB;",
    "棌" => "&#x68CC;",
    "棍" => "&#x68CD;",
    "棎" => "&#x68CE;",
    "棏" => "&#x68CF;",
    "棐" => "&#x68D0;",
    "棑" => "&#x68D1;",
    "棒" => "&#x68D2;",
    "棓" => "&#x68D3;",
    "棔" => "&#x68D4;",
    "棕" => "&#x68D5;",
    "棖" => "&#x68D6;",
    "棗" => "&#x68D7;",
    "棘" => "&#x68D8;",
    "棙" => "&#x68D9;",
    "棚" => "&#x68DA;",
    "棛" => "&#x68DB;",
    "棜" => "&#x68DC;",
    "棝" => "&#x68DD;",
    "棞" => "&#x68DE;",
    "棟" => "&#x68DF;",
    "棠" => "&#x68E0;",
    "棡" => "&#x68E1;",
    "棢" => "&#x68E2;",
    "棣" => "&#x68E3;",
    "棤" => "&#x68E4;",
    "棥" => "&#x68E5;",
    "棦" => "&#x68E6;",
    "棧" => "&#x68E7;",
    "棨" => "&#x68E8;",
    "棩" => "&#x68E9;",
    "棪" => "&#x68EA;",
    "棫" => "&#x68EB;",
    "棬" => "&#x68EC;",
    "棭" => "&#x68ED;",
    "森" => "&#x68EE;",
    "棯" => "&#x68EF;",
    "棰" => "&#x68F0;",
    "棱" => "&#x68F1;",
    "棲" => "&#x68F2;",
    "棳" => "&#x68F3;",
    "棴" => "&#x68F4;",
    "棵" => "&#x68F5;",
    "棶" => "&#x68F6;",
    "棷" => "&#x68F7;",
    "棸" => "&#x68F8;",
    "棹" => "&#x68F9;",
    "棺" => "&#x68FA;",
    "棻" => "&#x68FB;",
    "棼" => "&#x68FC;",
    "棽" => "&#x68FD;",
    "棾" => "&#x68FE;",
    "棿" => "&#x68FF;",
    "椀" => "&#x6900;",
    "椁" => "&#x6901;",
    "椂" => "&#x6902;",
    "椃" => "&#x6903;",
    "椄" => "&#x6904;",
    "椅" => "&#x6905;",
    "椆" => "&#x6906;",
    "椇" => "&#x6907;",
    "椈" => "&#x6908;",
    "椉" => "&#x6909;",
    "椊" => "&#x690A;",
    "椋" => "&#x690B;",
    "椌" => "&#x690C;",
    "植" => "&#x690D;",
    "椎" => "&#x690E;",
    "椏" => "&#x690F;",
    "椐" => "&#x6910;",
    "椑" => "&#x6911;",
    "椒" => "&#x6912;",
    "椓" => "&#x6913;",
    "椔" => "&#x6914;",
    "椕" => "&#x6915;",
    "椖" => "&#x6916;",
    "椗" => "&#x6917;",
    "椘" => "&#x6918;",
    "椙" => "&#x6919;",
    "椚" => "&#x691A;",
    "椛" => "&#x691B;",
    "検" => "&#x691C;",
    "椝" => "&#x691D;",
    "椞" => "&#x691E;",
    "椟" => "&#x691F;",
    "椠" => "&#x6920;",
    "椡" => "&#x6921;",
    "椢" => "&#x6922;",
    "椣" => "&#x6923;",
    "椤" => "&#x6924;",
    "椥" => "&#x6925;",
    "椦" => "&#x6926;",
    "椧" => "&#x6927;",
    "椨" => "&#x6928;",
    "椩" => "&#x6929;",
    "椪" => "&#x692A;",
    "椫" => "&#x692B;",
    "椬" => "&#x692C;",
    "椭" => "&#x692D;",
    "椮" => "&#x692E;",
    "椯" => "&#x692F;",
    "椰" => "&#x6930;",
    "椱" => "&#x6931;",
    "椲" => "&#x6932;",
    "椳" => "&#x6933;",
    "椴" => "&#x6934;",
    "椵" => "&#x6935;",
    "椶" => "&#x6936;",
    "椷" => "&#x6937;",
    "椸" => "&#x6938;",
    "椹" => "&#x6939;",
    "椺" => "&#x693A;",
    "椻" => "&#x693B;",
    "椼" => "&#x693C;",
    "椽" => "&#x693D;",
    "椾" => "&#x693E;",
    "椿" => "&#x693F;",
    "楀" => "&#x6940;",
    "楁" => "&#x6941;",
    "楂" => "&#x6942;",
    "楃" => "&#x6943;",
    "楄" => "&#x6944;",
    "楅" => "&#x6945;",
    "楆" => "&#x6946;",
    "楇" => "&#x6947;",
    "楈" => "&#x6948;",
    "楉" => "&#x6949;",
    "楊" => "&#x694A;",
    "楋" => "&#x694B;",
    "楌" => "&#x694C;",
    "楍" => "&#x694D;",
    "楎" => "&#x694E;",
    "楏" => "&#x694F;",
    "楐" => "&#x6950;",
    "楑" => "&#x6951;",
    "楒" => "&#x6952;",
    "楓" => "&#x6953;",
    "楔" => "&#x6954;",
    "楕" => "&#x6955;",
    "楖" => "&#x6956;",
    "楗" => "&#x6957;",
    "楘" => "&#x6958;",
    "楙" => "&#x6959;",
    "楚" => "&#x695A;",
    "楛" => "&#x695B;",
    "楜" => "&#x695C;",
    "楝" => "&#x695D;",
    "楞" => "&#x695E;",
    "楟" => "&#x695F;",
    "楠" => "&#x6960;",
    "楡" => "&#x6961;",
    "楢" => "&#x6962;",
    "楣" => "&#x6963;",
    "楤" => "&#x6964;",
    "楥" => "&#x6965;",
    "楦" => "&#x6966;",
    "楧" => "&#x6967;",
    "楨" => "&#x6968;",
    "楩" => "&#x6969;",
    "楪" => "&#x696A;",
    "楫" => "&#x696B;",
    "楬" => "&#x696C;",
    "業" => "&#x696D;",
    "楮" => "&#x696E;",
    "楯" => "&#x696F;",
    "楰" => "&#x6970;",
    "楱" => "&#x6971;",
    "楲" => "&#x6972;",
    "楳" => "&#x6973;",
    "楴" => "&#x6974;",
    "極" => "&#x6975;",
    "楶" => "&#x6976;",
    "楷" => "&#x6977;",
    "楸" => "&#x6978;",
    "楹" => "&#x6979;",
    "楺" => "&#x697A;",
    "楻" => "&#x697B;",
    "楼" => "&#x697C;",
    "楽" => "&#x697D;",
    "楾" => "&#x697E;",
    "楿" => "&#x697F;",
    "榀" => "&#x6980;",
    "榁" => "&#x6981;",
    "概" => "&#x6982;",
    "榃" => "&#x6983;",
    "榄" => "&#x6984;",
    "榅" => "&#x6985;",
    "榆" => "&#x6986;",
    "榇" => "&#x6987;",
    "榈" => "&#x6988;",
    "榉" => "&#x6989;",
    "榊" => "&#x698A;",
    "榋" => "&#x698B;",
    "榌" => "&#x698C;",
    "榍" => "&#x698D;",
    "榎" => "&#x698E;",
    "榏" => "&#x698F;",
    "榐" => "&#x6990;",
    "榑" => "&#x6991;",
    "榒" => "&#x6992;",
    "榓" => "&#x6993;",
    "榔" => "&#x6994;",
    "榕" => "&#x6995;",
    "榖" => "&#x6996;",
    "榗" => "&#x6997;",
    "榘" => "&#x6998;",
    "榙" => "&#x6999;",
    "榚" => "&#x699A;",
    "榛" => "&#x699B;",
    "榜" => "&#x699C;",
    "榝" => "&#x699D;",
    "榞" => "&#x699E;",
    "榟" => "&#x699F;",
    "榠" => "&#x69A0;",
    "榡" => "&#x69A1;",
    "榢" => "&#x69A2;",
    "榣" => "&#x69A3;",
    "榤" => "&#x69A4;",
    "榥" => "&#x69A5;",
    "榦" => "&#x69A6;",
    "榧" => "&#x69A7;",
    "榨" => "&#x69A8;",
    "榩" => "&#x69A9;",
    "榪" => "&#x69AA;",
    "榫" => "&#x69AB;",
    "榬" => "&#x69AC;",
    "榭" => "&#x69AD;",
    "榮" => "&#x69AE;",
    "榯" => "&#x69AF;",
    "榰" => "&#x69B0;",
    "榱" => "&#x69B1;",
    "榲" => "&#x69B2;",
    "榳" => "&#x69B3;",
    "榴" => "&#x69B4;",
    "榵" => "&#x69B5;",
    "榶" => "&#x69B6;",
    "榷" => "&#x69B7;",
    "榸" => "&#x69B8;",
    "榹" => "&#x69B9;",
    "榺" => "&#x69BA;",
    "榻" => "&#x69BB;",
    "榼" => "&#x69BC;",
    "榽" => "&#x69BD;",
    "榾" => "&#x69BE;",
    "榿" => "&#x69BF;",
    "槀" => "&#x69C0;",
    "槁" => "&#x69C1;",
    "槂" => "&#x69C2;",
    "槃" => "&#x69C3;",
    "槄" => "&#x69C4;",
    "槅" => "&#x69C5;",
    "槆" => "&#x69C6;",
    "槇" => "&#x69C7;",
    "槈" => "&#x69C8;",
    "槉" => "&#x69C9;",
    "槊" => "&#x69CA;",
    "構" => "&#x69CB;",
    "槌" => "&#x69CC;",
    "槍" => "&#x69CD;",
    "槎" => "&#x69CE;",
    "槏" => "&#x69CF;",
    "槐" => "&#x69D0;",
    "槑" => "&#x69D1;",
    "槒" => "&#x69D2;",
    "槓" => "&#x69D3;",
    "槔" => "&#x69D4;",
    "槕" => "&#x69D5;",
    "槖" => "&#x69D6;",
    "槗" => "&#x69D7;",
    "様" => "&#x69D8;",
    "槙" => "&#x69D9;",
    "槚" => "&#x69DA;",
    "槛" => "&#x69DB;",
    "槜" => "&#x69DC;",
    "槝" => "&#x69DD;",
    "槞" => "&#x69DE;",
    "槟" => "&#x69DF;",
    "槠" => "&#x69E0;",
    "槡" => "&#x69E1;",
    "槢" => "&#x69E2;",
    "槣" => "&#x69E3;",
    "槤" => "&#x69E4;",
    "槥" => "&#x69E5;",
    "槦" => "&#x69E6;",
    "槧" => "&#x69E7;",
    "槨" => "&#x69E8;",
    "槩" => "&#x69E9;",
    "槪" => "&#x69EA;",
    "槫" => "&#x69EB;",
    "槬" => "&#x69EC;",
    "槭" => "&#x69ED;",
    "槮" => "&#x69EE;",
    "槯" => "&#x69EF;",
    "槰" => "&#x69F0;",
    "槱" => "&#x69F1;",
    "槲" => "&#x69F2;",
    "槳" => "&#x69F3;",
    "槴" => "&#x69F4;",
    "槵" => "&#x69F5;",
    "槶" => "&#x69F6;",
    "槷" => "&#x69F7;",
    "槸" => "&#x69F8;",
    "槹" => "&#x69F9;",
    "槺" => "&#x69FA;",
    "槻" => "&#x69FB;",
    "槼" => "&#x69FC;",
    "槽" => "&#x69FD;",
    "槾" => "&#x69FE;",
    "槿" => "&#x69FF;",
    "樀" => "&#x6A00;",
    "樁" => "&#x6A01;",
    "樂" => "&#x6A02;",
    "樃" => "&#x6A03;",
    "樄" => "&#x6A04;",
    "樅" => "&#x6A05;",
    "樆" => "&#x6A06;",
    "樇" => "&#x6A07;",
    "樈" => "&#x6A08;",
    "樉" => "&#x6A09;",
    "樊" => "&#x6A0A;",
    "樋" => "&#x6A0B;",
    "樌" => "&#x6A0C;",
    "樍" => "&#x6A0D;",
    "樎" => "&#x6A0E;",
    "樏" => "&#x6A0F;",
    "樐" => "&#x6A10;",
    "樑" => "&#x6A11;",
    "樒" => "&#x6A12;",
    "樓" => "&#x6A13;",
    "樔" => "&#x6A14;",
    "樕" => "&#x6A15;",
    "樖" => "&#x6A16;",
    "樗" => "&#x6A17;",
    "樘" => "&#x6A18;",
    "標" => "&#x6A19;",
    "樚" => "&#x6A1A;",
    "樛" => "&#x6A1B;",
    "樜" => "&#x6A1C;",
    "樝" => "&#x6A1D;",
    "樞" => "&#x6A1E;",
    "樟" => "&#x6A1F;",
    "樠" => "&#x6A20;",
    "模" => "&#x6A21;",
    "樢" => "&#x6A22;",
    "樣" => "&#x6A23;",
    "樤" => "&#x6A24;",
    "樥" => "&#x6A25;",
    "樦" => "&#x6A26;",
    "樧" => "&#x6A27;",
    "樨" => "&#x6A28;",
    "権" => "&#x6A29;",
    "横" => "&#x6A2A;",
    "樫" => "&#x6A2B;",
    "樬" => "&#x6A2C;",
    "樭" => "&#x6A2D;",
    "樮" => "&#x6A2E;",
    "樯" => "&#x6A2F;",
    "樰" => "&#x6A30;",
    "樱" => "&#x6A31;",
    "樲" => "&#x6A32;",
    "樳" => "&#x6A33;",
    "樴" => "&#x6A34;",
    "樵" => "&#x6A35;",
    "樶" => "&#x6A36;",
    "樷" => "&#x6A37;",
    "樸" => "&#x6A38;",
    "樹" => "&#x6A39;",
    "樺" => "&#x6A3A;",
    "樻" => "&#x6A3B;",
    "樼" => "&#x6A3C;",
    "樽" => "&#x6A3D;",
    "樾" => "&#x6A3E;",
    "樿" => "&#x6A3F;",
    "橀" => "&#x6A40;",
    "橁" => "&#x6A41;",
    "橂" => "&#x6A42;",
    "橃" => "&#x6A43;",
    "橄" => "&#x6A44;",
    "橅" => "&#x6A45;",
    "橆" => "&#x6A46;",
    "橇" => "&#x6A47;",
    "橈" => "&#x6A48;",
    "橉" => "&#x6A49;",
    "橊" => "&#x6A4A;",
    "橋" => "&#x6A4B;",
    "橌" => "&#x6A4C;",
    "橍" => "&#x6A4D;",
    "橎" => "&#x6A4E;",
    "橏" => "&#x6A4F;",
    "橐" => "&#x6A50;",
    "橑" => "&#x6A51;",
    "橒" => "&#x6A52;",
    "橓" => "&#x6A53;",
    "橔" => "&#x6A54;",
    "橕" => "&#x6A55;",
    "橖" => "&#x6A56;",
    "橗" => "&#x6A57;",
    "橘" => "&#x6A58;",
    "橙" => "&#x6A59;",
    "橚" => "&#x6A5A;",
    "橛" => "&#x6A5B;",
    "橜" => "&#x6A5C;",
    "橝" => "&#x6A5D;",
    "橞" => "&#x6A5E;",
    "機" => "&#x6A5F;",
    "橠" => "&#x6A60;",
    "橡" => "&#x6A61;",
    "橢" => "&#x6A62;",
    "橣" => "&#x6A63;",
    "橤" => "&#x6A64;",
    "橥" => "&#x6A65;",
    "橦" => "&#x6A66;",
    "橧" => "&#x6A67;",
    "橨" => "&#x6A68;",
    "橩" => "&#x6A69;",
    "橪" => "&#x6A6A;",
    "橫" => "&#x6A6B;",
    "橬" => "&#x6A6C;",
    "橭" => "&#x6A6D;",
    "橮" => "&#x6A6E;",
    "橯" => "&#x6A6F;",
    "橰" => "&#x6A70;",
    "橱" => "&#x6A71;",
    "橲" => "&#x6A72;",
    "橳" => "&#x6A73;",
    "橴" => "&#x6A74;",
    "橵" => "&#x6A75;",
    "橶" => "&#x6A76;",
    "橷" => "&#x6A77;",
    "橸" => "&#x6A78;",
    "橹" => "&#x6A79;",
    "橺" => "&#x6A7A;",
    "橻" => "&#x6A7B;",
    "橼" => "&#x6A7C;",
    "橽" => "&#x6A7D;",
    "橾" => "&#x6A7E;",
    "橿" => "&#x6A7F;",
    "檀" => "&#x6A80;",
    "檁" => "&#x6A81;",
    "檂" => "&#x6A82;",
    "檃" => "&#x6A83;",
    "檄" => "&#x6A84;",
    "檅" => "&#x6A85;",
    "檆" => "&#x6A86;",
    "檇" => "&#x6A87;",
    "檈" => "&#x6A88;",
    "檉" => "&#x6A89;",
    "檊" => "&#x6A8A;",
    "檋" => "&#x6A8B;",
    "檌" => "&#x6A8C;",
    "檍" => "&#x6A8D;",
    "檎" => "&#x6A8E;",
    "檏" => "&#x6A8F;",
    "檐" => "&#x6A90;",
    "檑" => "&#x6A91;",
    "檒" => "&#x6A92;",
    "檓" => "&#x6A93;",
    "檔" => "&#x6A94;",
    "檕" => "&#x6A95;",
    "檖" => "&#x6A96;",
    "檗" => "&#x6A97;",
    "檘" => "&#x6A98;",
    "檙" => "&#x6A99;",
    "檚" => "&#x6A9A;",
    "檛" => "&#x6A9B;",
    "檜" => "&#x6A9C;",
    "檝" => "&#x6A9D;",
    "檞" => "&#x6A9E;",
    "檟" => "&#x6A9F;",
    "檠" => "&#x6AA0;",
    "檡" => "&#x6AA1;",
    "檢" => "&#x6AA2;",
    "檣" => "&#x6AA3;",
    "檤" => "&#x6AA4;",
    "檥" => "&#x6AA5;",
    "檦" => "&#x6AA6;",
    "檧" => "&#x6AA7;",
    "檨" => "&#x6AA8;",
    "檩" => "&#x6AA9;",
    "檪" => "&#x6AAA;",
    "檫" => "&#x6AAB;",
    "檬" => "&#x6AAC;",
    "檭" => "&#x6AAD;",
    "檮" => "&#x6AAE;",
    "檯" => "&#x6AAF;",
    "檰" => "&#x6AB0;",
    "檱" => "&#x6AB1;",
    "檲" => "&#x6AB2;",
    "檳" => "&#x6AB3;",
    "檴" => "&#x6AB4;",
    "檵" => "&#x6AB5;",
    "檶" => "&#x6AB6;",
    "檷" => "&#x6AB7;",
    "檸" => "&#x6AB8;",
    "檹" => "&#x6AB9;",
    "檺" => "&#x6ABA;",
    "檻" => "&#x6ABB;",
    "檼" => "&#x6ABC;",
    "檽" => "&#x6ABD;",
    "檾" => "&#x6ABE;",
    "檿" => "&#x6ABF;",
    "櫀" => "&#x6AC0;",
    "櫁" => "&#x6AC1;",
    "櫂" => "&#x6AC2;",
    "櫃" => "&#x6AC3;",
    "櫄" => "&#x6AC4;",
    "櫅" => "&#x6AC5;",
    "櫆" => "&#x6AC6;",
    "櫇" => "&#x6AC7;",
    "櫈" => "&#x6AC8;",
    "櫉" => "&#x6AC9;",
    "櫊" => "&#x6ACA;",
    "櫋" => "&#x6ACB;",
    "櫌" => "&#x6ACC;",
    "櫍" => "&#x6ACD;",
    "櫎" => "&#x6ACE;",
    "櫏" => "&#x6ACF;",
    "櫐" => "&#x6AD0;",
    "櫑" => "&#x6AD1;",
    "櫒" => "&#x6AD2;",
    "櫓" => "&#x6AD3;",
    "櫔" => "&#x6AD4;",
    "櫕" => "&#x6AD5;",
    "櫖" => "&#x6AD6;",
    "櫗" => "&#x6AD7;",
    "櫘" => "&#x6AD8;",
    "櫙" => "&#x6AD9;",
    "櫚" => "&#x6ADA;",
    "櫛" => "&#x6ADB;",
    "櫜" => "&#x6ADC;",
    "櫝" => "&#x6ADD;",
    "櫞" => "&#x6ADE;",
    "櫟" => "&#x6ADF;",
    "櫠" => "&#x6AE0;",
    "櫡" => "&#x6AE1;",
    "櫢" => "&#x6AE2;",
    "櫣" => "&#x6AE3;",
    "櫤" => "&#x6AE4;",
    "櫥" => "&#x6AE5;",
    "櫦" => "&#x6AE6;",
    "櫧" => "&#x6AE7;",
    "櫨" => "&#x6AE8;",
    "櫩" => "&#x6AE9;",
    "櫪" => "&#x6AEA;",
    "櫫" => "&#x6AEB;",
    "櫬" => "&#x6AEC;",
    "櫭" => "&#x6AED;",
    "櫮" => "&#x6AEE;",
    "櫯" => "&#x6AEF;",
    "櫰" => "&#x6AF0;",
    "櫱" => "&#x6AF1;",
    "櫲" => "&#x6AF2;",
    "櫳" => "&#x6AF3;",
    "櫴" => "&#x6AF4;",
    "櫵" => "&#x6AF5;",
    "櫶" => "&#x6AF6;",
    "櫷" => "&#x6AF7;",
    "櫸" => "&#x6AF8;",
    "櫹" => "&#x6AF9;",
    "櫺" => "&#x6AFA;",
    "櫻" => "&#x6AFB;",
    "櫼" => "&#x6AFC;",
    "櫽" => "&#x6AFD;",
    "櫾" => "&#x6AFE;",
    "櫿" => "&#x6AFF;",
    "欀" => "&#x6B00;",
    "欁" => "&#x6B01;",
    "欂" => "&#x6B02;",
    "欃" => "&#x6B03;",
    "欄" => "&#x6B04;",
    "欅" => "&#x6B05;",
    "欆" => "&#x6B06;",
    "欇" => "&#x6B07;",
    "欈" => "&#x6B08;",
    "欉" => "&#x6B09;",
    "權" => "&#x6B0A;",
    "欋" => "&#x6B0B;",
    "欌" => "&#x6B0C;",
    "欍" => "&#x6B0D;",
    "欎" => "&#x6B0E;",
    "欏" => "&#x6B0F;",
    "欐" => "&#x6B10;",
    "欑" => "&#x6B11;",
    "欒" => "&#x6B12;",
    "欓" => "&#x6B13;",
    "欔" => "&#x6B14;",
    "欕" => "&#x6B15;",
    "欖" => "&#x6B16;",
    "欗" => "&#x6B17;",
    "欘" => "&#x6B18;",
    "欙" => "&#x6B19;",
    "欚" => "&#x6B1A;",
    "欛" => "&#x6B1B;",
    "欜" => "&#x6B1C;",
    "欝" => "&#x6B1D;",
    "欞" => "&#x6B1E;",
    "欟" => "&#x6B1F;",
    "欠" => "&#x6B20;",
    "次" => "&#x6B21;",
    "欢" => "&#x6B22;",
    "欣" => "&#x6B23;",
    "欤" => "&#x6B24;",
    "欥" => "&#x6B25;",
    "欦" => "&#x6B26;",
    "欧" => "&#x6B27;",
    "欨" => "&#x6B28;",
    "欩" => "&#x6B29;",
    "欪" => "&#x6B2A;",
    "欫" => "&#x6B2B;",
    "欬" => "&#x6B2C;",
    "欭" => "&#x6B2D;",
    "欮" => "&#x6B2E;",
    "欯" => "&#x6B2F;",
    "欰" => "&#x6B30;",
    "欱" => "&#x6B31;",
    "欲" => "&#x6B32;",
    "欳" => "&#x6B33;",
    "欴" => "&#x6B34;",
    "欵" => "&#x6B35;",
    "欶" => "&#x6B36;",
    "欷" => "&#x6B37;",
    "欸" => "&#x6B38;",
    "欹" => "&#x6B39;",
    "欺" => "&#x6B3A;",
    "欻" => "&#x6B3B;",
    "欼" => "&#x6B3C;",
    "欽" => "&#x6B3D;",
    "款" => "&#x6B3E;",
    "欿" => "&#x6B3F;",
    "歀" => "&#x6B40;",
    "歁" => "&#x6B41;",
    "歂" => "&#x6B42;",
    "歃" => "&#x6B43;",
    "歄" => "&#x6B44;",
    "歅" => "&#x6B45;",
    "歆" => "&#x6B46;",
    "歇" => "&#x6B47;",
    "歈" => "&#x6B48;",
    "歉" => "&#x6B49;",
    "歊" => "&#x6B4A;",
    "歋" => "&#x6B4B;",
    "歌" => "&#x6B4C;",
    "歍" => "&#x6B4D;",
    "歎" => "&#x6B4E;",
    "歏" => "&#x6B4F;",
    "歐" => "&#x6B50;",
    "歑" => "&#x6B51;",
    "歒" => "&#x6B52;",
    "歓" => "&#x6B53;",
    "歔" => "&#x6B54;",
    "歕" => "&#x6B55;",
    "歖" => "&#x6B56;",
    "歗" => "&#x6B57;",
    "歘" => "&#x6B58;",
    "歙" => "&#x6B59;",
    "歚" => "&#x6B5A;",
    "歛" => "&#x6B5B;",
    "歜" => "&#x6B5C;",
    "歝" => "&#x6B5D;",
    "歞" => "&#x6B5E;",
    "歟" => "&#x6B5F;",
    "歠" => "&#x6B60;",
    "歡" => "&#x6B61;",
    "止" => "&#x6B62;",
    "正" => "&#x6B63;",
    "此" => "&#x6B64;",
    "步" => "&#x6B65;",
    "武" => "&#x6B66;",
    "歧" => "&#x6B67;",
    "歨" => "&#x6B68;",
    "歩" => "&#x6B69;",
    "歪" => "&#x6B6A;",
    "歫" => "&#x6B6B;",
    "歬" => "&#x6B6C;",
    "歭" => "&#x6B6D;",
    "歮" => "&#x6B6E;",
    "歯" => "&#x6B6F;",
    "歰" => "&#x6B70;",
    "歱" => "&#x6B71;",
    "歲" => "&#x6B72;",
    "歳" => "&#x6B73;",
    "歴" => "&#x6B74;",
    "歵" => "&#x6B75;",
    "歶" => "&#x6B76;",
    "歷" => "&#x6B77;",
    "歸" => "&#x6B78;",
    "歹" => "&#x6B79;",
    "歺" => "&#x6B7A;",
    "死" => "&#x6B7B;",
    "歼" => "&#x6B7C;",
    "歽" => "&#x6B7D;",
    "歾" => "&#x6B7E;",
    "歿" => "&#x6B7F;",
    "殀" => "&#x6B80;",
    "殁" => "&#x6B81;",
    "殂" => "&#x6B82;",
    "殃" => "&#x6B83;",
    "殄" => "&#x6B84;",
    "殅" => "&#x6B85;",
    "殆" => "&#x6B86;",
    "殇" => "&#x6B87;",
    "殈" => "&#x6B88;",
    "殉" => "&#x6B89;",
    "殊" => "&#x6B8A;",
    "残" => "&#x6B8B;",
    "殌" => "&#x6B8C;",
    "殍" => "&#x6B8D;",
    "殎" => "&#x6B8E;",
    "殏" => "&#x6B8F;",
    "殐" => "&#x6B90;",
    "殑" => "&#x6B91;",
    "殒" => "&#x6B92;",
    "殓" => "&#x6B93;",
    "殔" => "&#x6B94;",
    "殕" => "&#x6B95;",
    "殖" => "&#x6B96;",
    "殗" => "&#x6B97;",
    "殘" => "&#x6B98;",
    "殙" => "&#x6B99;",
    "殚" => "&#x6B9A;",
    "殛" => "&#x6B9B;",
    "殜" => "&#x6B9C;",
    "殝" => "&#x6B9D;",
    "殞" => "&#x6B9E;",
    "殟" => "&#x6B9F;",
    "殠" => "&#x6BA0;",
    "殡" => "&#x6BA1;",
    "殢" => "&#x6BA2;",
    "殣" => "&#x6BA3;",
    "殤" => "&#x6BA4;",
    "殥" => "&#x6BA5;",
    "殦" => "&#x6BA6;",
    "殧" => "&#x6BA7;",
    "殨" => "&#x6BA8;",
    "殩" => "&#x6BA9;",
    "殪" => "&#x6BAA;",
    "殫" => "&#x6BAB;",
    "殬" => "&#x6BAC;",
    "殭" => "&#x6BAD;",
    "殮" => "&#x6BAE;",
    "殯" => "&#x6BAF;",
    "殰" => "&#x6BB0;",
    "殱" => "&#x6BB1;",
    "殲" => "&#x6BB2;",
    "殳" => "&#x6BB3;",
    "殴" => "&#x6BB4;",
    "段" => "&#x6BB5;",
    "殶" => "&#x6BB6;",
    "殷" => "&#x6BB7;",
    "殸" => "&#x6BB8;",
    "殹" => "&#x6BB9;",
    "殺" => "&#x6BBA;",
    "殻" => "&#x6BBB;",
    "殼" => "&#x6BBC;",
    "殽" => "&#x6BBD;",
    "殾" => "&#x6BBE;",
    "殿" => "&#x6BBF;",
    "毀" => "&#x6BC0;",
    "毁" => "&#x6BC1;",
    "毂" => "&#x6BC2;",
    "毃" => "&#x6BC3;",
    "毄" => "&#x6BC4;",
    "毅" => "&#x6BC5;",
    "毆" => "&#x6BC6;",
    "毇" => "&#x6BC7;",
    "毈" => "&#x6BC8;",
    "毉" => "&#x6BC9;",
    "毊" => "&#x6BCA;",
    "毋" => "&#x6BCB;",
    "毌" => "&#x6BCC;",
    "母" => "&#x6BCD;",
    "毎" => "&#x6BCE;",
    "每" => "&#x6BCF;",
    "毐" => "&#x6BD0;",
    "毑" => "&#x6BD1;",
    "毒" => "&#x6BD2;",
    "毓" => "&#x6BD3;",
    "比" => "&#x6BD4;",
    "毕" => "&#x6BD5;",
    "毖" => "&#x6BD6;",
    "毗" => "&#x6BD7;",
    "毘" => "&#x6BD8;",
    "毙" => "&#x6BD9;",
    "毚" => "&#x6BDA;",
    "毛" => "&#x6BDB;",
    "毜" => "&#x6BDC;",
    "毝" => "&#x6BDD;",
    "毞" => "&#x6BDE;",
    "毟" => "&#x6BDF;",
    "毠" => "&#x6BE0;",
    "毡" => "&#x6BE1;",
    "毢" => "&#x6BE2;",
    "毣" => "&#x6BE3;",
    "毤" => "&#x6BE4;",
    "毥" => "&#x6BE5;",
    "毦" => "&#x6BE6;",
    "毧" => "&#x6BE7;",
    "毨" => "&#x6BE8;",
    "毩" => "&#x6BE9;",
    "毪" => "&#x6BEA;",
    "毫" => "&#x6BEB;",
    "毬" => "&#x6BEC;",
    "毭" => "&#x6BED;",
    "毮" => "&#x6BEE;",
    "毯" => "&#x6BEF;",
    "毰" => "&#x6BF0;",
    "毱" => "&#x6BF1;",
    "毲" => "&#x6BF2;",
    "毳" => "&#x6BF3;",
    "毴" => "&#x6BF4;",
    "毵" => "&#x6BF5;",
    "毶" => "&#x6BF6;",
    "毷" => "&#x6BF7;",
    "毸" => "&#x6BF8;",
    "毹" => "&#x6BF9;",
    "毺" => "&#x6BFA;",
    "毻" => "&#x6BFB;",
    "毼" => "&#x6BFC;",
    "毽" => "&#x6BFD;",
    "毾" => "&#x6BFE;",
    "毿" => "&#x6BFF;",
    "氀" => "&#x6C00;",
    "氁" => "&#x6C01;",
    "氂" => "&#x6C02;",
    "氃" => "&#x6C03;",
    "氄" => "&#x6C04;",
    "氅" => "&#x6C05;",
    "氆" => "&#x6C06;",
    "氇" => "&#x6C07;",
    "氈" => "&#x6C08;",
    "氉" => "&#x6C09;",
    "氊" => "&#x6C0A;",
    "氋" => "&#x6C0B;",
    "氌" => "&#x6C0C;",
    "氍" => "&#x6C0D;",
    "氎" => "&#x6C0E;",
    "氏" => "&#x6C0F;",
    "氐" => "&#x6C10;",
    "民" => "&#x6C11;",
    "氒" => "&#x6C12;",
    "氓" => "&#x6C13;",
    "气" => "&#x6C14;",
    "氕" => "&#x6C15;",
    "氖" => "&#x6C16;",
    "気" => "&#x6C17;",
    "氘" => "&#x6C18;",
    "氙" => "&#x6C19;",
    "氚" => "&#x6C1A;",
    "氛" => "&#x6C1B;",
    "氜" => "&#x6C1C;",
    "氝" => "&#x6C1D;",
    "氞" => "&#x6C1E;",
    "氟" => "&#x6C1F;",
    "氠" => "&#x6C20;",
    "氡" => "&#x6C21;",
    "氢" => "&#x6C22;",
    "氣" => "&#x6C23;",
    "氤" => "&#x6C24;",
    "氥" => "&#x6C25;",
    "氦" => "&#x6C26;",
    "氧" => "&#x6C27;",
    "氨" => "&#x6C28;",
    "氩" => "&#x6C29;",
    "氪" => "&#x6C2A;",
    "氫" => "&#x6C2B;",
    "氬" => "&#x6C2C;",
    "氭" => "&#x6C2D;",
    "氮" => "&#x6C2E;",
    "氯" => "&#x6C2F;",
    "氰" => "&#x6C30;",
    "氱" => "&#x6C31;",
    "氲" => "&#x6C32;",
    "氳" => "&#x6C33;",
    "水" => "&#x6C34;",
    "氵" => "&#x6C35;",
    "氶" => "&#x6C36;",
    "氷" => "&#x6C37;",
    "永" => "&#x6C38;",
    "氹" => "&#x6C39;",
    "氺" => "&#x6C3A;",
    "氻" => "&#x6C3B;",
    "氼" => "&#x6C3C;",
    "氽" => "&#x6C3D;",
    "氾" => "&#x6C3E;",
    "氿" => "&#x6C3F;",
    "汀" => "&#x6C40;",
    "汁" => "&#x6C41;",
    "求" => "&#x6C42;",
    "汃" => "&#x6C43;",
    "汄" => "&#x6C44;",
    "汅" => "&#x6C45;",
    "汆" => "&#x6C46;",
    "汇" => "&#x6C47;",
    "汈" => "&#x6C48;",
    "汉" => "&#x6C49;",
    "汊" => "&#x6C4A;",
    "汋" => "&#x6C4B;",
    "汌" => "&#x6C4C;",
    "汍" => "&#x6C4D;",
    "汎" => "&#x6C4E;",
    "汏" => "&#x6C4F;",
    "汐" => "&#x6C50;",
    "汑" => "&#x6C51;",
    "汒" => "&#x6C52;",
    "汓" => "&#x6C53;",
    "汔" => "&#x6C54;",
    "汕" => "&#x6C55;",
    "汖" => "&#x6C56;",
    "汗" => "&#x6C57;",
    "汘" => "&#x6C58;",
    "汙" => "&#x6C59;",
    "汚" => "&#x6C5A;",
    "汛" => "&#x6C5B;",
    "汜" => "&#x6C5C;",
    "汝" => "&#x6C5D;",
    "汞" => "&#x6C5E;",
    "江" => "&#x6C5F;",
    "池" => "&#x6C60;",
    "污" => "&#x6C61;",
    "汢" => "&#x6C62;",
    "汣" => "&#x6C63;",
    "汤" => "&#x6C64;",
    "汥" => "&#x6C65;",
    "汦" => "&#x6C66;",
    "汧" => "&#x6C67;",
    "汨" => "&#x6C68;",
    "汩" => "&#x6C69;",
    "汪" => "&#x6C6A;",
    "汫" => "&#x6C6B;",
    "汬" => "&#x6C6C;",
    "汭" => "&#x6C6D;",
    "汮" => "&#x6C6E;",
    "汯" => "&#x6C6F;",
    "汰" => "&#x6C70;",
    "汱" => "&#x6C71;",
    "汲" => "&#x6C72;",
    "汳" => "&#x6C73;",
    "汴" => "&#x6C74;",
    "汵" => "&#x6C75;",
    "汶" => "&#x6C76;",
    "汷" => "&#x6C77;",
    "汸" => "&#x6C78;",
    "汹" => "&#x6C79;",
    "決" => "&#x6C7A;",
    "汻" => "&#x6C7B;",
    "汼" => "&#x6C7C;",
    "汽" => "&#x6C7D;",
    "汾" => "&#x6C7E;",
    "汿" => "&#x6C7F;",
    "沀" => "&#x6C80;",
    "沁" => "&#x6C81;",
    "沂" => "&#x6C82;",
    "沃" => "&#x6C83;",
    "沄" => "&#x6C84;",
    "沅" => "&#x6C85;",
    "沆" => "&#x6C86;",
    "沇" => "&#x6C87;",
    "沈" => "&#x6C88;",
    "沉" => "&#x6C89;",
    "沊" => "&#x6C8A;",
    "沋" => "&#x6C8B;",
    "沌" => "&#x6C8C;",
    "沍" => "&#x6C8D;",
    "沎" => "&#x6C8E;",
    "沏" => "&#x6C8F;",
    "沐" => "&#x6C90;",
    "沑" => "&#x6C91;",
    "沒" => "&#x6C92;",
    "沓" => "&#x6C93;",
    "沔" => "&#x6C94;",
    "沕" => "&#x6C95;",
    "沖" => "&#x6C96;",
    "沗" => "&#x6C97;",
    "沘" => "&#x6C98;",
    "沙" => "&#x6C99;",
    "沚" => "&#x6C9A;",
    "沛" => "&#x6C9B;",
    "沜" => "&#x6C9C;",
    "沝" => "&#x6C9D;",
    "沞" => "&#x6C9E;",
    "沟" => "&#x6C9F;",
    "沠" => "&#x6CA0;",
    "没" => "&#x6CA1;",
    "沢" => "&#x6CA2;",
    "沣" => "&#x6CA3;",
    "沤" => "&#x6CA4;",
    "沥" => "&#x6CA5;",
    "沦" => "&#x6CA6;",
    "沧" => "&#x6CA7;",
    "沨" => "&#x6CA8;",
    "沩" => "&#x6CA9;",
    "沪" => "&#x6CAA;",
    "沫" => "&#x6CAB;",
    "沬" => "&#x6CAC;",
    "沭" => "&#x6CAD;",
    "沮" => "&#x6CAE;",
    "沯" => "&#x6CAF;",
    "沰" => "&#x6CB0;",
    "沱" => "&#x6CB1;",
    "沲" => "&#x6CB2;",
    "河" => "&#x6CB3;",
    "沴" => "&#x6CB4;",
    "沵" => "&#x6CB5;",
    "沶" => "&#x6CB6;",
    "沷" => "&#x6CB7;",
    "沸" => "&#x6CB8;",
    "油" => "&#x6CB9;",
    "沺" => "&#x6CBA;",
    "治" => "&#x6CBB;",
    "沼" => "&#x6CBC;",
    "沽" => "&#x6CBD;",
    "沾" => "&#x6CBE;",
    "沿" => "&#x6CBF;",
    "泀" => "&#x6CC0;",
    "況" => "&#x6CC1;",
    "泂" => "&#x6CC2;",
    "泃" => "&#x6CC3;",
    "泄" => "&#x6CC4;",
    "泅" => "&#x6CC5;",
    "泆" => "&#x6CC6;",
    "泇" => "&#x6CC7;",
    "泈" => "&#x6CC8;",
    "泉" => "&#x6CC9;",
    "泊" => "&#x6CCA;",
    "泋" => "&#x6CCB;",
    "泌" => "&#x6CCC;",
    "泍" => "&#x6CCD;",
    "泎" => "&#x6CCE;",
    "泏" => "&#x6CCF;",
    "泐" => "&#x6CD0;",
    "泑" => "&#x6CD1;",
    "泒" => "&#x6CD2;",
    "泓" => "&#x6CD3;",
    "泔" => "&#x6CD4;",
    "法" => "&#x6CD5;",
    "泖" => "&#x6CD6;",
    "泗" => "&#x6CD7;",
    "泘" => "&#x6CD8;",
    "泙" => "&#x6CD9;",
    "泚" => "&#x6CDA;",
    "泛" => "&#x6CDB;",
    "泜" => "&#x6CDC;",
    "泝" => "&#x6CDD;",
    "泞" => "&#x6CDE;",
    "泟" => "&#x6CDF;",
    "泠" => "&#x6CE0;",
    "泡" => "&#x6CE1;",
    "波" => "&#x6CE2;",
    "泣" => "&#x6CE3;",
    "泤" => "&#x6CE4;",
    "泥" => "&#x6CE5;",
    "泦" => "&#x6CE6;",
    "泧" => "&#x6CE7;",
    "注" => "&#x6CE8;",
    "泩" => "&#x6CE9;",
    "泪" => "&#x6CEA;",
    "泫" => "&#x6CEB;",
    "泬" => "&#x6CEC;",
    "泭" => "&#x6CED;",
    "泮" => "&#x6CEE;",
    "泯" => "&#x6CEF;",
    "泰" => "&#x6CF0;",
    "泱" => "&#x6CF1;",
    "泲" => "&#x6CF2;",
    "泳" => "&#x6CF3;",
    "泴" => "&#x6CF4;",
    "泵" => "&#x6CF5;",
    "泶" => "&#x6CF6;",
    "泷" => "&#x6CF7;",
    "泸" => "&#x6CF8;",
    "泹" => "&#x6CF9;",
    "泺" => "&#x6CFA;",
    "泻" => "&#x6CFB;",
    "泼" => "&#x6CFC;",
    "泽" => "&#x6CFD;",
    "泾" => "&#x6CFE;",
    "泿" => "&#x6CFF;",
    "洀" => "&#x6D00;",
    "洁" => "&#x6D01;",
    "洂" => "&#x6D02;",
    "洃" => "&#x6D03;",
    "洄" => "&#x6D04;",
    "洅" => "&#x6D05;",
    "洆" => "&#x6D06;",
    "洇" => "&#x6D07;",
    "洈" => "&#x6D08;",
    "洉" => "&#x6D09;",
    "洊" => "&#x6D0A;",
    "洋" => "&#x6D0B;",
    "洌" => "&#x6D0C;",
    "洍" => "&#x6D0D;",
    "洎" => "&#x6D0E;",
    "洏" => "&#x6D0F;",
    "洐" => "&#x6D10;",
    "洑" => "&#x6D11;",
    "洒" => "&#x6D12;",
    "洓" => "&#x6D13;",
    "洔" => "&#x6D14;",
    "洕" => "&#x6D15;",
    "洖" => "&#x6D16;",
    "洗" => "&#x6D17;",
    "洘" => "&#x6D18;",
    "洙" => "&#x6D19;",
    "洚" => "&#x6D1A;",
    "洛" => "&#x6D1B;",
    "洜" => "&#x6D1C;",
    "洝" => "&#x6D1D;",
    "洞" => "&#x6D1E;",
    "洟" => "&#x6D1F;",
    "洠" => "&#x6D20;",
    "洡" => "&#x6D21;",
    "洢" => "&#x6D22;",
    "洣" => "&#x6D23;",
    "洤" => "&#x6D24;",
    "津" => "&#x6D25;",
    "洦" => "&#x6D26;",
    "洧" => "&#x6D27;",
    "洨" => "&#x6D28;",
    "洩" => "&#x6D29;",
    "洪" => "&#x6D2A;",
    "洫" => "&#x6D2B;",
    "洬" => "&#x6D2C;",
    "洭" => "&#x6D2D;",
    "洮" => "&#x6D2E;",
    "洯" => "&#x6D2F;",
    "洰" => "&#x6D30;",
    "洱" => "&#x6D31;",
    "洲" => "&#x6D32;",
    "洳" => "&#x6D33;",
    "洴" => "&#x6D34;",
    "洵" => "&#x6D35;",
    "洶" => "&#x6D36;",
    "洷" => "&#x6D37;",
    "洸" => "&#x6D38;",
    "洹" => "&#x6D39;",
    "洺" => "&#x6D3A;",
    "活" => "&#x6D3B;",
    "洼" => "&#x6D3C;",
    "洽" => "&#x6D3D;",
    "派" => "&#x6D3E;",
    "洿" => "&#x6D3F;",
    "浀" => "&#x6D40;",
    "流" => "&#x6D41;",
    "浂" => "&#x6D42;",
    "浃" => "&#x6D43;",
    "浄" => "&#x6D44;",
    "浅" => "&#x6D45;",
    "浆" => "&#x6D46;",
    "浇" => "&#x6D47;",
    "浈" => "&#x6D48;",
    "浉" => "&#x6D49;",
    "浊" => "&#x6D4A;",
    "测" => "&#x6D4B;",
    "浌" => "&#x6D4C;",
    "浍" => "&#x6D4D;",
    "济" => "&#x6D4E;",
    "浏" => "&#x6D4F;",
    "浐" => "&#x6D50;",
    "浑" => "&#x6D51;",
    "浒" => "&#x6D52;",
    "浓" => "&#x6D53;",
    "浔" => "&#x6D54;",
    "浕" => "&#x6D55;",
    "浖" => "&#x6D56;",
    "浗" => "&#x6D57;",
    "浘" => "&#x6D58;",
    "浙" => "&#x6D59;",
    "浚" => "&#x6D5A;",
    "浛" => "&#x6D5B;",
    "浜" => "&#x6D5C;",
    "浝" => "&#x6D5D;",
    "浞" => "&#x6D5E;",
    "浟" => "&#x6D5F;",
    "浠" => "&#x6D60;",
    "浡" => "&#x6D61;",
    "浢" => "&#x6D62;",
    "浣" => "&#x6D63;",
    "浤" => "&#x6D64;",
    "浥" => "&#x6D65;",
    "浦" => "&#x6D66;",
    "浧" => "&#x6D67;",
    "浨" => "&#x6D68;",
    "浩" => "&#x6D69;",
    "浪" => "&#x6D6A;",
    "浫" => "&#x6D6B;",
    "浬" => "&#x6D6C;",
    "浭" => "&#x6D6D;",
    "浮" => "&#x6D6E;",
    "浯" => "&#x6D6F;",
    "浰" => "&#x6D70;",
    "浱" => "&#x6D71;",
    "浲" => "&#x6D72;",
    "浳" => "&#x6D73;",
    "浴" => "&#x6D74;",
    "浵" => "&#x6D75;",
    "浶" => "&#x6D76;",
    "海" => "&#x6D77;",
    "浸" => "&#x6D78;",
    "浹" => "&#x6D79;",
    "浺" => "&#x6D7A;",
    "浻" => "&#x6D7B;",
    "浼" => "&#x6D7C;",
    "浽" => "&#x6D7D;",
    "浾" => "&#x6D7E;",
    "浿" => "&#x6D7F;",
    "涀" => "&#x6D80;",
    "涁" => "&#x6D81;",
    "涂" => "&#x6D82;",
    "涃" => "&#x6D83;",
    "涄" => "&#x6D84;",
    "涅" => "&#x6D85;",
    "涆" => "&#x6D86;",
    "涇" => "&#x6D87;",
    "消" => "&#x6D88;",
    "涉" => "&#x6D89;",
    "涊" => "&#x6D8A;",
    "涋" => "&#x6D8B;",
    "涌" => "&#x6D8C;",
    "涍" => "&#x6D8D;",
    "涎" => "&#x6D8E;",
    "涏" => "&#x6D8F;",
    "涐" => "&#x6D90;",
    "涑" => "&#x6D91;",
    "涒" => "&#x6D92;",
    "涓" => "&#x6D93;",
    "涔" => "&#x6D94;",
    "涕" => "&#x6D95;",
    "涖" => "&#x6D96;",
    "涗" => "&#x6D97;",
    "涘" => "&#x6D98;",
    "涙" => "&#x6D99;",
    "涚" => "&#x6D9A;",
    "涛" => "&#x6D9B;",
    "涜" => "&#x6D9C;",
    "涝" => "&#x6D9D;",
    "涞" => "&#x6D9E;",
    "涟" => "&#x6D9F;",
    "涠" => "&#x6DA0;",
    "涡" => "&#x6DA1;",
    "涢" => "&#x6DA2;",
    "涣" => "&#x6DA3;",
    "涤" => "&#x6DA4;",
    "涥" => "&#x6DA5;",
    "润" => "&#x6DA6;",
    "涧" => "&#x6DA7;",
    "涨" => "&#x6DA8;",
    "涩" => "&#x6DA9;",
    "涪" => "&#x6DAA;",
    "涫" => "&#x6DAB;",
    "涬" => "&#x6DAC;",
    "涭" => "&#x6DAD;",
    "涮" => "&#x6DAE;",
    "涯" => "&#x6DAF;",
    "涰" => "&#x6DB0;",
    "涱" => "&#x6DB1;",
    "液" => "&#x6DB2;",
    "涳" => "&#x6DB3;",
    "涴" => "&#x6DB4;",
    "涵" => "&#x6DB5;",
    "涶" => "&#x6DB6;",
    "涷" => "&#x6DB7;",
    "涸" => "&#x6DB8;",
    "涹" => "&#x6DB9;",
    "涺" => "&#x6DBA;",
    "涻" => "&#x6DBB;",
    "涼" => "&#x6DBC;",
    "涽" => "&#x6DBD;",
    "涾" => "&#x6DBE;",
    "涿" => "&#x6DBF;",
    "淀" => "&#x6DC0;",
    "淁" => "&#x6DC1;",
    "淂" => "&#x6DC2;",
    "淃" => "&#x6DC3;",
    "淄" => "&#x6DC4;",
    "淅" => "&#x6DC5;",
    "淆" => "&#x6DC6;",
    "淇" => "&#x6DC7;",
    "淈" => "&#x6DC8;",
    "淉" => "&#x6DC9;",
    "淊" => "&#x6DCA;",
    "淋" => "&#x6DCB;",
    "淌" => "&#x6DCC;",
    "淍" => "&#x6DCD;",
    "淎" => "&#x6DCE;",
    "淏" => "&#x6DCF;",
    "淐" => "&#x6DD0;",
    "淑" => "&#x6DD1;",
    "淒" => "&#x6DD2;",
    "淓" => "&#x6DD3;",
    "淔" => "&#x6DD4;",
    "淕" => "&#x6DD5;",
    "淖" => "&#x6DD6;",
    "淗" => "&#x6DD7;",
    "淘" => "&#x6DD8;",
    "淙" => "&#x6DD9;",
    "淚" => "&#x6DDA;",
    "淛" => "&#x6DDB;",
    "淜" => "&#x6DDC;",
    "淝" => "&#x6DDD;",
    "淞" => "&#x6DDE;",
    "淟" => "&#x6DDF;",
    "淠" => "&#x6DE0;",
    "淡" => "&#x6DE1;",
    "淢" => "&#x6DE2;",
    "淣" => "&#x6DE3;",
    "淤" => "&#x6DE4;",
    "淥" => "&#x6DE5;",
    "淦" => "&#x6DE6;",
    "淧" => "&#x6DE7;",
    "淨" => "&#x6DE8;",
    "淩" => "&#x6DE9;",
    "淪" => "&#x6DEA;",
    "淫" => "&#x6DEB;",
    "淬" => "&#x6DEC;",
    "淭" => "&#x6DED;",
    "淮" => "&#x6DEE;",
    "淯" => "&#x6DEF;",
    "淰" => "&#x6DF0;",
    "深" => "&#x6DF1;",
    "淲" => "&#x6DF2;",
    "淳" => "&#x6DF3;",
    "淴" => "&#x6DF4;",
    "淵" => "&#x6DF5;",
    "淶" => "&#x6DF6;",
    "混" => "&#x6DF7;",
    "淸" => "&#x6DF8;",
    "淹" => "&#x6DF9;",
    "淺" => "&#x6DFA;",
    "添" => "&#x6DFB;",
    "淼" => "&#x6DFC;",
    "淽" => "&#x6DFD;",
    "淾" => "&#x6DFE;",
    "淿" => "&#x6DFF;",
    "渀" => "&#x6E00;",
    "渁" => "&#x6E01;",
    "渂" => "&#x6E02;",
    "渃" => "&#x6E03;",
    "渄" => "&#x6E04;",
    "清" => "&#x6E05;",
    "渆" => "&#x6E06;",
    "渇" => "&#x6E07;",
    "済" => "&#x6E08;",
    "渉" => "&#x6E09;",
    "渊" => "&#x6E0A;",
    "渋" => "&#x6E0B;",
    "渌" => "&#x6E0C;",
    "渍" => "&#x6E0D;",
    "渎" => "&#x6E0E;",
    "渏" => "&#x6E0F;",
    "渐" => "&#x6E10;",
    "渑" => "&#x6E11;",
    "渒" => "&#x6E12;",
    "渓" => "&#x6E13;",
    "渔" => "&#x6E14;",
    "渕" => "&#x6E15;",
    "渖" => "&#x6E16;",
    "渗" => "&#x6E17;",
    "渘" => "&#x6E18;",
    "渙" => "&#x6E19;",
    "渚" => "&#x6E1A;",
    "減" => "&#x6E1B;",
    "渜" => "&#x6E1C;",
    "渝" => "&#x6E1D;",
    "渞" => "&#x6E1E;",
    "渟" => "&#x6E1F;",
    "渠" => "&#x6E20;",
    "渡" => "&#x6E21;",
    "渢" => "&#x6E22;",
    "渣" => "&#x6E23;",
    "渤" => "&#x6E24;",
    "渥" => "&#x6E25;",
    "渦" => "&#x6E26;",
    "渧" => "&#x6E27;",
    "渨" => "&#x6E28;",
    "温" => "&#x6E29;",
    "渪" => "&#x6E2A;",
    "渫" => "&#x6E2B;",
    "測" => "&#x6E2C;",
    "渭" => "&#x6E2D;",
    "渮" => "&#x6E2E;",
    "港" => "&#x6E2F;",
    "渰" => "&#x6E30;",
    "渱" => "&#x6E31;",
    "渲" => "&#x6E32;",
    "渳" => "&#x6E33;",
    "渴" => "&#x6E34;",
    "渵" => "&#x6E35;",
    "渶" => "&#x6E36;",
    "渷" => "&#x6E37;",
    "游" => "&#x6E38;",
    "渹" => "&#x6E39;",
    "渺" => "&#x6E3A;",
    "渻" => "&#x6E3B;",
    "渼" => "&#x6E3C;",
    "渽" => "&#x6E3D;",
    "渾" => "&#x6E3E;",
    "渿" => "&#x6E3F;",
    "湀" => "&#x6E40;",
    "湁" => "&#x6E41;",
    "湂" => "&#x6E42;",
    "湃" => "&#x6E43;",
    "湄" => "&#x6E44;",
    "湅" => "&#x6E45;",
    "湆" => "&#x6E46;",
    "湇" => "&#x6E47;",
    "湈" => "&#x6E48;",
    "湉" => "&#x6E49;",
    "湊" => "&#x6E4A;",
    "湋" => "&#x6E4B;",
    "湌" => "&#x6E4C;",
    "湍" => "&#x6E4D;",
    "湎" => "&#x6E4E;",
    "湏" => "&#x6E4F;",
    "湐" => "&#x6E50;",
    "湑" => "&#x6E51;",
    "湒" => "&#x6E52;",
    "湓" => "&#x6E53;",
    "湔" => "&#x6E54;",
    "湕" => "&#x6E55;",
    "湖" => "&#x6E56;",
    "湗" => "&#x6E57;",
    "湘" => "&#x6E58;",
    "湙" => "&#x6E59;",
    "湚" => "&#x6E5A;",
    "湛" => "&#x6E5B;",
    "湜" => "&#x6E5C;",
    "湝" => "&#x6E5D;",
    "湞" => "&#x6E5E;",
    "湟" => "&#x6E5F;",
    "湠" => "&#x6E60;",
    "湡" => "&#x6E61;",
    "湢" => "&#x6E62;",
    "湣" => "&#x6E63;",
    "湤" => "&#x6E64;",
    "湥" => "&#x6E65;",
    "湦" => "&#x6E66;",
    "湧" => "&#x6E67;",
    "湨" => "&#x6E68;",
    "湩" => "&#x6E69;",
    "湪" => "&#x6E6A;",
    "湫" => "&#x6E6B;",
    "湬" => "&#x6E6C;",
    "湭" => "&#x6E6D;",
    "湮" => "&#x6E6E;",
    "湯" => "&#x6E6F;",
    "湰" => "&#x6E70;",
    "湱" => "&#x6E71;",
    "湲" => "&#x6E72;",
    "湳" => "&#x6E73;",
    "湴" => "&#x6E74;",
    "湵" => "&#x6E75;",
    "湶" => "&#x6E76;",
    "湷" => "&#x6E77;",
    "湸" => "&#x6E78;",
    "湹" => "&#x6E79;",
    "湺" => "&#x6E7A;",
    "湻" => "&#x6E7B;",
    "湼" => "&#x6E7C;",
    "湽" => "&#x6E7D;",
    "湾" => "&#x6E7E;",
    "湿" => "&#x6E7F;",
    "満" => "&#x6E80;",
    "溁" => "&#x6E81;",
    "溂" => "&#x6E82;",
    "溃" => "&#x6E83;",
    "溄" => "&#x6E84;",
    "溅" => "&#x6E85;",
    "溆" => "&#x6E86;",
    "溇" => "&#x6E87;",
    "溈" => "&#x6E88;",
    "溉" => "&#x6E89;",
    "溊" => "&#x6E8A;",
    "溋" => "&#x6E8B;",
    "溌" => "&#x6E8C;",
    "溍" => "&#x6E8D;",
    "溎" => "&#x6E8E;",
    "溏" => "&#x6E8F;",
    "源" => "&#x6E90;",
    "溑" => "&#x6E91;",
    "溒" => "&#x6E92;",
    "溓" => "&#x6E93;",
    "溔" => "&#x6E94;",
    "溕" => "&#x6E95;",
    "準" => "&#x6E96;",
    "溗" => "&#x6E97;",
    "溘" => "&#x6E98;",
    "溙" => "&#x6E99;",
    "溚" => "&#x6E9A;",
    "溛" => "&#x6E9B;",
    "溜" => "&#x6E9C;",
    "溝" => "&#x6E9D;",
    "溞" => "&#x6E9E;",
    "溟" => "&#x6E9F;",
    "溠" => "&#x6EA0;",
    "溡" => "&#x6EA1;",
    "溢" => "&#x6EA2;",
    "溣" => "&#x6EA3;",
    "溤" => "&#x6EA4;",
    "溥" => "&#x6EA5;",
    "溦" => "&#x6EA6;",
    "溧" => "&#x6EA7;",
    "溨" => "&#x6EA8;",
    "溩" => "&#x6EA9;",
    "溪" => "&#x6EAA;",
    "溫" => "&#x6EAB;",
    "溬" => "&#x6EAC;",
    "溭" => "&#x6EAD;",
    "溮" => "&#x6EAE;",
    "溯" => "&#x6EAF;",
    "溰" => "&#x6EB0;",
    "溱" => "&#x6EB1;",
    "溲" => "&#x6EB2;",
    "溳" => "&#x6EB3;",
    "溴" => "&#x6EB4;",
    "溵" => "&#x6EB5;",
    "溶" => "&#x6EB6;",
    "溷" => "&#x6EB7;",
    "溸" => "&#x6EB8;",
    "溹" => "&#x6EB9;",
    "溺" => "&#x6EBA;",
    "溻" => "&#x6EBB;",
    "溼" => "&#x6EBC;",
    "溽" => "&#x6EBD;",
    "溾" => "&#x6EBE;",
    "溿" => "&#x6EBF;",
    "滀" => "&#x6EC0;",
    "滁" => "&#x6EC1;",
    "滂" => "&#x6EC2;",
    "滃" => "&#x6EC3;",
    "滄" => "&#x6EC4;",
    "滅" => "&#x6EC5;",
    "滆" => "&#x6EC6;",
    "滇" => "&#x6EC7;",
    "滈" => "&#x6EC8;",
    "滉" => "&#x6EC9;",
    "滊" => "&#x6ECA;",
    "滋" => "&#x6ECB;",
    "滌" => "&#x6ECC;",
    "滍" => "&#x6ECD;",
    "滎" => "&#x6ECE;",
    "滏" => "&#x6ECF;",
    "滐" => "&#x6ED0;",
    "滑" => "&#x6ED1;",
    "滒" => "&#x6ED2;",
    "滓" => "&#x6ED3;",
    "滔" => "&#x6ED4;",
    "滕" => "&#x6ED5;",
    "滖" => "&#x6ED6;",
    "滗" => "&#x6ED7;",
    "滘" => "&#x6ED8;",
    "滙" => "&#x6ED9;",
    "滚" => "&#x6EDA;",
    "滛" => "&#x6EDB;",
    "滜" => "&#x6EDC;",
    "滝" => "&#x6EDD;",
    "滞" => "&#x6EDE;",
    "滟" => "&#x6EDF;",
    "滠" => "&#x6EE0;",
    "满" => "&#x6EE1;",
    "滢" => "&#x6EE2;",
    "滣" => "&#x6EE3;",
    "滤" => "&#x6EE4;",
    "滥" => "&#x6EE5;",
    "滦" => "&#x6EE6;",
    "滧" => "&#x6EE7;",
    "滨" => "&#x6EE8;",
    "滩" => "&#x6EE9;",
    "滪" => "&#x6EEA;",
    "滫" => "&#x6EEB;",
    "滬" => "&#x6EEC;",
    "滭" => "&#x6EED;",
    "滮" => "&#x6EEE;",
    "滯" => "&#x6EEF;",
    "滰" => "&#x6EF0;",
    "滱" => "&#x6EF1;",
    "滲" => "&#x6EF2;",
    "滳" => "&#x6EF3;",
    "滴" => "&#x6EF4;",
    "滵" => "&#x6EF5;",
    "滶" => "&#x6EF6;",
    "滷" => "&#x6EF7;",
    "滸" => "&#x6EF8;",
    "滹" => "&#x6EF9;",
    "滺" => "&#x6EFA;",
    "滻" => "&#x6EFB;",
    "滼" => "&#x6EFC;",
    "滽" => "&#x6EFD;",
    "滾" => "&#x6EFE;",
    "滿" => "&#x6EFF;",
    "漀" => "&#x6F00;",
    "漁" => "&#x6F01;",
    "漂" => "&#x6F02;",
    "漃" => "&#x6F03;",
    "漄" => "&#x6F04;",
    "漅" => "&#x6F05;",
    "漆" => "&#x6F06;",
    "漇" => "&#x6F07;",
    "漈" => "&#x6F08;",
    "漉" => "&#x6F09;",
    "漊" => "&#x6F0A;",
    "漋" => "&#x6F0B;",
    "漌" => "&#x6F0C;",
    "漍" => "&#x6F0D;",
    "漎" => "&#x6F0E;",
    "漏" => "&#x6F0F;",
    "漐" => "&#x6F10;",
    "漑" => "&#x6F11;",
    "漒" => "&#x6F12;",
    "漓" => "&#x6F13;",
    "演" => "&#x6F14;",
    "漕" => "&#x6F15;",
    "漖" => "&#x6F16;",
    "漗" => "&#x6F17;",
    "漘" => "&#x6F18;",
    "漙" => "&#x6F19;",
    "漚" => "&#x6F1A;",
    "漛" => "&#x6F1B;",
    "漜" => "&#x6F1C;",
    "漝" => "&#x6F1D;",
    "漞" => "&#x6F1E;",
    "漟" => "&#x6F1F;",
    "漠" => "&#x6F20;",
    "漡" => "&#x6F21;",
    "漢" => "&#x6F22;",
    "漣" => "&#x6F23;",
    "漤" => "&#x6F24;",
    "漥" => "&#x6F25;",
    "漦" => "&#x6F26;",
    "漧" => "&#x6F27;",
    "漨" => "&#x6F28;",
    "漩" => "&#x6F29;",
    "漪" => "&#x6F2A;",
    "漫" => "&#x6F2B;",
    "漬" => "&#x6F2C;",
    "漭" => "&#x6F2D;",
    "漮" => "&#x6F2E;",
    "漯" => "&#x6F2F;",
    "漰" => "&#x6F30;",
    "漱" => "&#x6F31;",
    "漲" => "&#x6F32;",
    "漳" => "&#x6F33;",
    "漴" => "&#x6F34;",
    "漵" => "&#x6F35;",
    "漶" => "&#x6F36;",
    "漷" => "&#x6F37;",
    "漸" => "&#x6F38;",
    "漹" => "&#x6F39;",
    "漺" => "&#x6F3A;",
    "漻" => "&#x6F3B;",
    "漼" => "&#x6F3C;",
    "漽" => "&#x6F3D;",
    "漾" => "&#x6F3E;",
    "漿" => "&#x6F3F;",
    "潀" => "&#x6F40;",
    "潁" => "&#x6F41;",
    "潂" => "&#x6F42;",
    "潃" => "&#x6F43;",
    "潄" => "&#x6F44;",
    "潅" => "&#x6F45;",
    "潆" => "&#x6F46;",
    "潇" => "&#x6F47;",
    "潈" => "&#x6F48;",
    "潉" => "&#x6F49;",
    "潊" => "&#x6F4A;",
    "潋" => "&#x6F4B;",
    "潌" => "&#x6F4C;",
    "潍" => "&#x6F4D;",
    "潎" => "&#x6F4E;",
    "潏" => "&#x6F4F;",
    "潐" => "&#x6F50;",
    "潑" => "&#x6F51;",
    "潒" => "&#x6F52;",
    "潓" => "&#x6F53;",
    "潔" => "&#x6F54;",
    "潕" => "&#x6F55;",
    "潖" => "&#x6F56;",
    "潗" => "&#x6F57;",
    "潘" => "&#x6F58;",
    "潙" => "&#x6F59;",
    "潚" => "&#x6F5A;",
    "潛" => "&#x6F5B;",
    "潜" => "&#x6F5C;",
    "潝" => "&#x6F5D;",
    "潞" => "&#x6F5E;",
    "潟" => "&#x6F5F;",
    "潠" => "&#x6F60;",
    "潡" => "&#x6F61;",
    "潢" => "&#x6F62;",
    "潣" => "&#x6F63;",
    "潤" => "&#x6F64;",
    "潥" => "&#x6F65;",
    "潦" => "&#x6F66;",
    "潧" => "&#x6F67;",
    "潨" => "&#x6F68;",
    "潩" => "&#x6F69;",
    "潪" => "&#x6F6A;",
    "潫" => "&#x6F6B;",
    "潬" => "&#x6F6C;",
    "潭" => "&#x6F6D;",
    "潮" => "&#x6F6E;",
    "潯" => "&#x6F6F;",
    "潰" => "&#x6F70;",
    "潱" => "&#x6F71;",
    "潲" => "&#x6F72;",
    "潳" => "&#x6F73;",
    "潴" => "&#x6F74;",
    "潵" => "&#x6F75;",
    "潶" => "&#x6F76;",
    "潷" => "&#x6F77;",
    "潸" => "&#x6F78;",
    "潹" => "&#x6F79;",
    "潺" => "&#x6F7A;",
    "潻" => "&#x6F7B;",
    "潼" => "&#x6F7C;",
    "潽" => "&#x6F7D;",
    "潾" => "&#x6F7E;",
    "潿" => "&#x6F7F;",
    "澀" => "&#x6F80;",
    "澁" => "&#x6F81;",
    "澂" => "&#x6F82;",
    "澃" => "&#x6F83;",
    "澄" => "&#x6F84;",
    "澅" => "&#x6F85;",
    "澆" => "&#x6F86;",
    "澇" => "&#x6F87;",
    "澈" => "&#x6F88;",
    "澉" => "&#x6F89;",
    "澊" => "&#x6F8A;",
    "澋" => "&#x6F8B;",
    "澌" => "&#x6F8C;",
    "澍" => "&#x6F8D;",
    "澎" => "&#x6F8E;",
    "澏" => "&#x6F8F;",
    "澐" => "&#x6F90;",
    "澑" => "&#x6F91;",
    "澒" => "&#x6F92;",
    "澓" => "&#x6F93;",
    "澔" => "&#x6F94;",
    "澕" => "&#x6F95;",
    "澖" => "&#x6F96;",
    "澗" => "&#x6F97;",
    "澘" => "&#x6F98;",
    "澙" => "&#x6F99;",
    "澚" => "&#x6F9A;",
    "澛" => "&#x6F9B;",
    "澜" => "&#x6F9C;",
    "澝" => "&#x6F9D;",
    "澞" => "&#x6F9E;",
    "澟" => "&#x6F9F;",
    "澠" => "&#x6FA0;",
    "澡" => "&#x6FA1;",
    "澢" => "&#x6FA2;",
    "澣" => "&#x6FA3;",
    "澤" => "&#x6FA4;",
    "澥" => "&#x6FA5;",
    "澦" => "&#x6FA6;",
    "澧" => "&#x6FA7;",
    "澨" => "&#x6FA8;",
    "澩" => "&#x6FA9;",
    "澪" => "&#x6FAA;",
    "澫" => "&#x6FAB;",
    "澬" => "&#x6FAC;",
    "澭" => "&#x6FAD;",
    "澮" => "&#x6FAE;",
    "澯" => "&#x6FAF;",
    "澰" => "&#x6FB0;",
    "澱" => "&#x6FB1;",
    "澲" => "&#x6FB2;",
    "澳" => "&#x6FB3;",
    "澴" => "&#x6FB4;",
    "澵" => "&#x6FB5;",
    "澶" => "&#x6FB6;",
    "澷" => "&#x6FB7;",
    "澸" => "&#x6FB8;",
    "澹" => "&#x6FB9;",
    "澺" => "&#x6FBA;",
    "澻" => "&#x6FBB;",
    "澼" => "&#x6FBC;",
    "澽" => "&#x6FBD;",
    "澾" => "&#x6FBE;",
    "澿" => "&#x6FBF;",
    "激" => "&#x6FC0;",
    "濁" => "&#x6FC1;",
    "濂" => "&#x6FC2;",
    "濃" => "&#x6FC3;",
    "濄" => "&#x6FC4;",
    "濅" => "&#x6FC5;",
    "濆" => "&#x6FC6;",
    "濇" => "&#x6FC7;",
    "濈" => "&#x6FC8;",
    "濉" => "&#x6FC9;",
    "濊" => "&#x6FCA;",
    "濋" => "&#x6FCB;",
    "濌" => "&#x6FCC;",
    "濍" => "&#x6FCD;",
    "濎" => "&#x6FCE;",
    "濏" => "&#x6FCF;",
    "濐" => "&#x6FD0;",
    "濑" => "&#x6FD1;",
    "濒" => "&#x6FD2;",
    "濓" => "&#x6FD3;",
    "濔" => "&#x6FD4;",
    "濕" => "&#x6FD5;",
    "濖" => "&#x6FD6;",
    "濗" => "&#x6FD7;",
    "濘" => "&#x6FD8;",
    "濙" => "&#x6FD9;",
    "濚" => "&#x6FDA;",
    "濛" => "&#x6FDB;",
    "濜" => "&#x6FDC;",
    "濝" => "&#x6FDD;",
    "濞" => "&#x6FDE;",
    "濟" => "&#x6FDF;",
    "濠" => "&#x6FE0;",
    "濡" => "&#x6FE1;",
    "濢" => "&#x6FE2;",
    "濣" => "&#x6FE3;",
    "濤" => "&#x6FE4;",
    "濥" => "&#x6FE5;",
    "濦" => "&#x6FE6;",
    "濧" => "&#x6FE7;",
    "濨" => "&#x6FE8;",
    "濩" => "&#x6FE9;",
    "濪" => "&#x6FEA;",
    "濫" => "&#x6FEB;",
    "濬" => "&#x6FEC;",
    "濭" => "&#x6FED;",
    "濮" => "&#x6FEE;",
    "濯" => "&#x6FEF;",
    "濰" => "&#x6FF0;",
    "濱" => "&#x6FF1;",
    "濲" => "&#x6FF2;",
    "濳" => "&#x6FF3;",
    "濴" => "&#x6FF4;",
    "濵" => "&#x6FF5;",
    "濶" => "&#x6FF6;",
    "濷" => "&#x6FF7;",
    "濸" => "&#x6FF8;",
    "濹" => "&#x6FF9;",
    "濺" => "&#x6FFA;",
    "濻" => "&#x6FFB;",
    "濼" => "&#x6FFC;",
    "濽" => "&#x6FFD;",
    "濾" => "&#x6FFE;",
    "濿" => "&#x6FFF;",
    "瀀" => "&#x7000;",
    "瀁" => "&#x7001;",
    "瀂" => "&#x7002;",
    "瀃" => "&#x7003;",
    "瀄" => "&#x7004;",
    "瀅" => "&#x7005;",
    "瀆" => "&#x7006;",
    "瀇" => "&#x7007;",
    "瀈" => "&#x7008;",
    "瀉" => "&#x7009;",
    "瀊" => "&#x700A;",
    "瀋" => "&#x700B;",
    "瀌" => "&#x700C;",
    "瀍" => "&#x700D;",
    "瀎" => "&#x700E;",
    "瀏" => "&#x700F;",
    "瀐" => "&#x7010;",
    "瀑" => "&#x7011;",
    "瀒" => "&#x7012;",
    "瀓" => "&#x7013;",
    "瀔" => "&#x7014;",
    "瀕" => "&#x7015;",
    "瀖" => "&#x7016;",
    "瀗" => "&#x7017;",
    "瀘" => "&#x7018;",
    "瀙" => "&#x7019;",
    "瀚" => "&#x701A;",
    "瀛" => "&#x701B;",
    "瀜" => "&#x701C;",
    "瀝" => "&#x701D;",
    "瀞" => "&#x701E;",
    "瀟" => "&#x701F;",
    "瀠" => "&#x7020;",
    "瀡" => "&#x7021;",
    "瀢" => "&#x7022;",
    "瀣" => "&#x7023;",
    "瀤" => "&#x7024;",
    "瀥" => "&#x7025;",
    "瀦" => "&#x7026;",
    "瀧" => "&#x7027;",
    "瀨" => "&#x7028;",
    "瀩" => "&#x7029;",
    "瀪" => "&#x702A;",
    "瀫" => "&#x702B;",
    "瀬" => "&#x702C;",
    "瀭" => "&#x702D;",
    "瀮" => "&#x702E;",
    "瀯" => "&#x702F;",
    "瀰" => "&#x7030;",
    "瀱" => "&#x7031;",
    "瀲" => "&#x7032;",
    "瀳" => "&#x7033;",
    "瀴" => "&#x7034;",
    "瀵" => "&#x7035;",
    "瀶" => "&#x7036;",
    "瀷" => "&#x7037;",
    "瀸" => "&#x7038;",
    "瀹" => "&#x7039;",
    "瀺" => "&#x703A;",
    "瀻" => "&#x703B;",
    "瀼" => "&#x703C;",
    "瀽" => "&#x703D;",
    "瀾" => "&#x703E;",
    "瀿" => "&#x703F;",
    "灀" => "&#x7040;",
    "灁" => "&#x7041;",
    "灂" => "&#x7042;",
    "灃" => "&#x7043;",
    "灄" => "&#x7044;",
    "灅" => "&#x7045;",
    "灆" => "&#x7046;",
    "灇" => "&#x7047;",
    "灈" => "&#x7048;",
    "灉" => "&#x7049;",
    "灊" => "&#x704A;",
    "灋" => "&#x704B;",
    "灌" => "&#x704C;",
    "灍" => "&#x704D;",
    "灎" => "&#x704E;",
    "灏" => "&#x704F;",
    "灐" => "&#x7050;",
    "灑" => "&#x7051;",
    "灒" => "&#x7052;",
    "灓" => "&#x7053;",
    "灔" => "&#x7054;",
    "灕" => "&#x7055;",
    "灖" => "&#x7056;",
    "灗" => "&#x7057;",
    "灘" => "&#x7058;",
    "灙" => "&#x7059;",
    "灚" => "&#x705A;",
    "灛" => "&#x705B;",
    "灜" => "&#x705C;",
    "灝" => "&#x705D;",
    "灞" => "&#x705E;",
    "灟" => "&#x705F;",
    "灠" => "&#x7060;",
    "灡" => "&#x7061;",
    "灢" => "&#x7062;",
    "灣" => "&#x7063;",
    "灤" => "&#x7064;",
    "灥" => "&#x7065;",
    "灦" => "&#x7066;",
    "灧" => "&#x7067;",
    "灨" => "&#x7068;",
    "灩" => "&#x7069;",
    "灪" => "&#x706A;",
    "火" => "&#x706B;",
    "灬" => "&#x706C;",
    "灭" => "&#x706D;",
    "灮" => "&#x706E;",
    "灯" => "&#x706F;",
    "灰" => "&#x7070;",
    "灱" => "&#x7071;",
    "灲" => "&#x7072;",
    "灳" => "&#x7073;",
    "灴" => "&#x7074;",
    "灵" => "&#x7075;",
    "灶" => "&#x7076;",
    "灷" => "&#x7077;",
    "灸" => "&#x7078;",
    "灹" => "&#x7079;",
    "灺" => "&#x707A;",
    "灻" => "&#x707B;",
    "灼" => "&#x707C;",
    "災" => "&#x707D;",
    "灾" => "&#x707E;",
    "灿" => "&#x707F;",
    "炀" => "&#x7080;",
    "炁" => "&#x7081;",
    "炂" => "&#x7082;",
    "炃" => "&#x7083;",
    "炄" => "&#x7084;",
    "炅" => "&#x7085;",
    "炆" => "&#x7086;",
    "炇" => "&#x7087;",
    "炈" => "&#x7088;",
    "炉" => "&#x7089;",
    "炊" => "&#x708A;",
    "炋" => "&#x708B;",
    "炌" => "&#x708C;",
    "炍" => "&#x708D;",
    "炎" => "&#x708E;",
    "炏" => "&#x708F;",
    "炐" => "&#x7090;",
    "炑" => "&#x7091;",
    "炒" => "&#x7092;",
    "炓" => "&#x7093;",
    "炔" => "&#x7094;",
    "炕" => "&#x7095;",
    "炖" => "&#x7096;",
    "炗" => "&#x7097;",
    "炘" => "&#x7098;",
    "炙" => "&#x7099;",
    "炚" => "&#x709A;",
    "炛" => "&#x709B;",
    "炜" => "&#x709C;",
    "炝" => "&#x709D;",
    "炞" => "&#x709E;",
    "炟" => "&#x709F;",
    "炠" => "&#x70A0;",
    "炡" => "&#x70A1;",
    "炢" => "&#x70A2;",
    "炣" => "&#x70A3;",
    "炤" => "&#x70A4;",
    "炥" => "&#x70A5;",
    "炦" => "&#x70A6;",
    "炧" => "&#x70A7;",
    "炨" => "&#x70A8;",
    "炩" => "&#x70A9;",
    "炪" => "&#x70AA;",
    "炫" => "&#x70AB;",
    "炬" => "&#x70AC;",
    "炭" => "&#x70AD;",
    "炮" => "&#x70AE;",
    "炯" => "&#x70AF;",
    "炰" => "&#x70B0;",
    "炱" => "&#x70B1;",
    "炲" => "&#x70B2;",
    "炳" => "&#x70B3;",
    "炴" => "&#x70B4;",
    "炵" => "&#x70B5;",
    "炶" => "&#x70B6;",
    "炷" => "&#x70B7;",
    "炸" => "&#x70B8;",
    "点" => "&#x70B9;",
    "為" => "&#x70BA;",
    "炻" => "&#x70BB;",
    "炼" => "&#x70BC;",
    "炽" => "&#x70BD;",
    "炾" => "&#x70BE;",
    "炿" => "&#x70BF;",
    "烀" => "&#x70C0;",
    "烁" => "&#x70C1;",
    "烂" => "&#x70C2;",
    "烃" => "&#x70C3;",
    "烄" => "&#x70C4;",
    "烅" => "&#x70C5;",
    "烆" => "&#x70C6;",
    "烇" => "&#x70C7;",
    "烈" => "&#x70C8;",
    "烉" => "&#x70C9;",
    "烊" => "&#x70CA;",
    "烋" => "&#x70CB;",
    "烌" => "&#x70CC;",
    "烍" => "&#x70CD;",
    "烎" => "&#x70CE;",
    "烏" => "&#x70CF;",
    "烐" => "&#x70D0;",
    "烑" => "&#x70D1;",
    "烒" => "&#x70D2;",
    "烓" => "&#x70D3;",
    "烔" => "&#x70D4;",
    "烕" => "&#x70D5;",
    "烖" => "&#x70D6;",
    "烗" => "&#x70D7;",
    "烘" => "&#x70D8;",
    "烙" => "&#x70D9;",
    "烚" => "&#x70DA;",
    "烛" => "&#x70DB;",
    "烜" => "&#x70DC;",
    "烝" => "&#x70DD;",
    "烞" => "&#x70DE;",
    "烟" => "&#x70DF;",
    "烠" => "&#x70E0;",
    "烡" => "&#x70E1;",
    "烢" => "&#x70E2;",
    "烣" => "&#x70E3;",
    "烤" => "&#x70E4;",
    "烥" => "&#x70E5;",
    "烦" => "&#x70E6;",
    "烧" => "&#x70E7;",
    "烨" => "&#x70E8;",
    "烩" => "&#x70E9;",
    "烪" => "&#x70EA;",
    "烫" => "&#x70EB;",
    "烬" => "&#x70EC;",
    "热" => "&#x70ED;",
    "烮" => "&#x70EE;",
    "烯" => "&#x70EF;",
    "烰" => "&#x70F0;",
    "烱" => "&#x70F1;",
    "烲" => "&#x70F2;",
    "烳" => "&#x70F3;",
    "烴" => "&#x70F4;",
    "烵" => "&#x70F5;",
    "烶" => "&#x70F6;",
    "烷" => "&#x70F7;",
    "烸" => "&#x70F8;",
    "烹" => "&#x70F9;",
    "烺" => "&#x70FA;",
    "烻" => "&#x70FB;",
    "烼" => "&#x70FC;",
    "烽" => "&#x70FD;",
    "烾" => "&#x70FE;",
    "烿" => "&#x70FF;",
    "焀" => "&#x7100;",
    "焁" => "&#x7101;",
    "焂" => "&#x7102;",
    "焃" => "&#x7103;",
    "焄" => "&#x7104;",
    "焅" => "&#x7105;",
    "焆" => "&#x7106;",
    "焇" => "&#x7107;",
    "焈" => "&#x7108;",
    "焉" => "&#x7109;",
    "焊" => "&#x710A;",
    "焋" => "&#x710B;",
    "焌" => "&#x710C;",
    "焍" => "&#x710D;",
    "焎" => "&#x710E;",
    "焏" => "&#x710F;",
    "焐" => "&#x7110;",
    "焑" => "&#x7111;",
    "焒" => "&#x7112;",
    "焓" => "&#x7113;",
    "焔" => "&#x7114;",
    "焕" => "&#x7115;",
    "焖" => "&#x7116;",
    "焗" => "&#x7117;",
    "焘" => "&#x7118;",
    "焙" => "&#x7119;",
    "焚" => "&#x711A;",
    "焛" => "&#x711B;",
    "焜" => "&#x711C;",
    "焝" => "&#x711D;",
    "焞" => "&#x711E;",
    "焟" => "&#x711F;",
    "焠" => "&#x7120;",
    "無" => "&#x7121;",
    "焢" => "&#x7122;",
    "焣" => "&#x7123;",
    "焤" => "&#x7124;",
    "焥" => "&#x7125;",
    "焦" => "&#x7126;",
    "焧" => "&#x7127;",
    "焨" => "&#x7128;",
    "焩" => "&#x7129;",
    "焪" => "&#x712A;",
    "焫" => "&#x712B;",
    "焬" => "&#x712C;",
    "焭" => "&#x712D;",
    "焮" => "&#x712E;",
    "焯" => "&#x712F;",
    "焰" => "&#x7130;",
    "焱" => "&#x7131;",
    "焲" => "&#x7132;",
    "焳" => "&#x7133;",
    "焴" => "&#x7134;",
    "焵" => "&#x7135;",
    "然" => "&#x7136;",
    "焷" => "&#x7137;",
    "焸" => "&#x7138;",
    "焹" => "&#x7139;",
    "焺" => "&#x713A;",
    "焻" => "&#x713B;",
    "焼" => "&#x713C;",
    "焽" => "&#x713D;",
    "焾" => "&#x713E;",
    "焿" => "&#x713F;",
    "煀" => "&#x7140;",
    "煁" => "&#x7141;",
    "煂" => "&#x7142;",
    "煃" => "&#x7143;",
    "煄" => "&#x7144;",
    "煅" => "&#x7145;",
    "煆" => "&#x7146;",
    "煇" => "&#x7147;",
    "煈" => "&#x7148;",
    "煉" => "&#x7149;",
    "煊" => "&#x714A;",
    "煋" => "&#x714B;",
    "煌" => "&#x714C;",
    "煍" => "&#x714D;",
    "煎" => "&#x714E;",
    "煏" => "&#x714F;",
    "煐" => "&#x7150;",
    "煑" => "&#x7151;",
    "煒" => "&#x7152;",
    "煓" => "&#x7153;",
    "煔" => "&#x7154;",
    "煕" => "&#x7155;",
    "煖" => "&#x7156;",
    "煗" => "&#x7157;",
    "煘" => "&#x7158;",
    "煙" => "&#x7159;",
    "煚" => "&#x715A;",
    "煛" => "&#x715B;",
    "煜" => "&#x715C;",
    "煝" => "&#x715D;",
    "煞" => "&#x715E;",
    "煟" => "&#x715F;",
    "煠" => "&#x7160;",
    "煡" => "&#x7161;",
    "煢" => "&#x7162;",
    "煣" => "&#x7163;",
    "煤" => "&#x7164;",
    "煥" => "&#x7165;",
    "煦" => "&#x7166;",
    "照" => "&#x7167;",
    "煨" => "&#x7168;",
    "煩" => "&#x7169;",
    "煪" => "&#x716A;",
    "煫" => "&#x716B;",
    "煬" => "&#x716C;",
    "煭" => "&#x716D;",
    "煮" => "&#x716E;",
    "煯" => "&#x716F;",
    "煰" => "&#x7170;",
    "煱" => "&#x7171;",
    "煲" => "&#x7172;",
    "煳" => "&#x7173;",
    "煴" => "&#x7174;",
    "煵" => "&#x7175;",
    "煶" => "&#x7176;",
    "煷" => "&#x7177;",
    "煸" => "&#x7178;",
    "煹" => "&#x7179;",
    "煺" => "&#x717A;",
    "煻" => "&#x717B;",
    "煼" => "&#x717C;",
    "煽" => "&#x717D;",
    "煾" => "&#x717E;",
    "煿" => "&#x717F;",
    "熀" => "&#x7180;",
    "熁" => "&#x7181;",
    "熂" => "&#x7182;",
    "熃" => "&#x7183;",
    "熄" => "&#x7184;",
    "熅" => "&#x7185;",
    "熆" => "&#x7186;",
    "熇" => "&#x7187;",
    "熈" => "&#x7188;",
    "熉" => "&#x7189;",
    "熊" => "&#x718A;",
    "熋" => "&#x718B;",
    "熌" => "&#x718C;",
    "熍" => "&#x718D;",
    "熎" => "&#x718E;",
    "熏" => "&#x718F;",
    "熐" => "&#x7190;",
    "熑" => "&#x7191;",
    "熒" => "&#x7192;",
    "熓" => "&#x7193;",
    "熔" => "&#x7194;",
    "熕" => "&#x7195;",
    "熖" => "&#x7196;",
    "熗" => "&#x7197;",
    "熘" => "&#x7198;",
    "熙" => "&#x7199;",
    "熚" => "&#x719A;",
    "熛" => "&#x719B;",
    "熜" => "&#x719C;",
    "熝" => "&#x719D;",
    "熞" => "&#x719E;",
    "熟" => "&#x719F;",
    "熠" => "&#x71A0;",
    "熡" => "&#x71A1;",
    "熢" => "&#x71A2;",
    "熣" => "&#x71A3;",
    "熤" => "&#x71A4;",
    "熥" => "&#x71A5;",
    "熦" => "&#x71A6;",
    "熧" => "&#x71A7;",
    "熨" => "&#x71A8;",
    "熩" => "&#x71A9;",
    "熪" => "&#x71AA;",
    "熫" => "&#x71AB;",
    "熬" => "&#x71AC;",
    "熭" => "&#x71AD;",
    "熮" => "&#x71AE;",
    "熯" => "&#x71AF;",
    "熰" => "&#x71B0;",
    "熱" => "&#x71B1;",
    "熲" => "&#x71B2;",
    "熳" => "&#x71B3;",
    "熴" => "&#x71B4;",
    "熵" => "&#x71B5;",
    "熶" => "&#x71B6;",
    "熷" => "&#x71B7;",
    "熸" => "&#x71B8;",
    "熹" => "&#x71B9;",
    "熺" => "&#x71BA;",
    "熻" => "&#x71BB;",
    "熼" => "&#x71BC;",
    "熽" => "&#x71BD;",
    "熾" => "&#x71BE;",
    "熿" => "&#x71BF;",
    "燀" => "&#x71C0;",
    "燁" => "&#x71C1;",
    "燂" => "&#x71C2;",
    "燃" => "&#x71C3;",
    "燄" => "&#x71C4;",
    "燅" => "&#x71C5;",
    "燆" => "&#x71C6;",
    "燇" => "&#x71C7;",
    "燈" => "&#x71C8;",
    "燉" => "&#x71C9;",
    "燊" => "&#x71CA;",
    "燋" => "&#x71CB;",
    "燌" => "&#x71CC;",
    "燍" => "&#x71CD;",
    "燎" => "&#x71CE;",
    "燏" => "&#x71CF;",
    "燐" => "&#x71D0;",
    "燑" => "&#x71D1;",
    "燒" => "&#x71D2;",
    "燓" => "&#x71D3;",
    "燔" => "&#x71D4;",
    "燕" => "&#x71D5;",
    "燖" => "&#x71D6;",
    "燗" => "&#x71D7;",
    "燘" => "&#x71D8;",
    "燙" => "&#x71D9;",
    "燚" => "&#x71DA;",
    "燛" => "&#x71DB;",
    "燜" => "&#x71DC;",
    "燝" => "&#x71DD;",
    "燞" => "&#x71DE;",
    "營" => "&#x71DF;",
    "燠" => "&#x71E0;",
    "燡" => "&#x71E1;",
    "燢" => "&#x71E2;",
    "燣" => "&#x71E3;",
    "燤" => "&#x71E4;",
    "燥" => "&#x71E5;",
    "燦" => "&#x71E6;",
    "燧" => "&#x71E7;",
    "燨" => "&#x71E8;",
    "燩" => "&#x71E9;",
    "燪" => "&#x71EA;",
    "燫" => "&#x71EB;",
    "燬" => "&#x71EC;",
    "燭" => "&#x71ED;",
    "燮" => "&#x71EE;",
    "燯" => "&#x71EF;",
    "燰" => "&#x71F0;",
    "燱" => "&#x71F1;",
    "燲" => "&#x71F2;",
    "燳" => "&#x71F3;",
    "燴" => "&#x71F4;",
    "燵" => "&#x71F5;",
    "燶" => "&#x71F6;",
    "燷" => "&#x71F7;",
    "燸" => "&#x71F8;",
    "燹" => "&#x71F9;",
    "燺" => "&#x71FA;",
    "燻" => "&#x71FB;",
    "燼" => "&#x71FC;",
    "燽" => "&#x71FD;",
    "燾" => "&#x71FE;",
    "燿" => "&#x71FF;",
    "爀" => "&#x7200;",
    "爁" => "&#x7201;",
    "爂" => "&#x7202;",
    "爃" => "&#x7203;",
    "爄" => "&#x7204;",
    "爅" => "&#x7205;",
    "爆" => "&#x7206;",
    "爇" => "&#x7207;",
    "爈" => "&#x7208;",
    "爉" => "&#x7209;",
    "爊" => "&#x720A;",
    "爋" => "&#x720B;",
    "爌" => "&#x720C;",
    "爍" => "&#x720D;",
    "爎" => "&#x720E;",
    "爏" => "&#x720F;",
    "爐" => "&#x7210;",
    "爑" => "&#x7211;",
    "爒" => "&#x7212;",
    "爓" => "&#x7213;",
    "爔" => "&#x7214;",
    "爕" => "&#x7215;",
    "爖" => "&#x7216;",
    "爗" => "&#x7217;",
    "爘" => "&#x7218;",
    "爙" => "&#x7219;",
    "爚" => "&#x721A;",
    "爛" => "&#x721B;",
    "爜" => "&#x721C;",
    "爝" => "&#x721D;",
    "爞" => "&#x721E;",
    "爟" => "&#x721F;",
    "爠" => "&#x7220;",
    "爡" => "&#x7221;",
    "爢" => "&#x7222;",
    "爣" => "&#x7223;",
    "爤" => "&#x7224;",
    "爥" => "&#x7225;",
    "爦" => "&#x7226;",
    "爧" => "&#x7227;",
    "爨" => "&#x7228;",
    "爩" => "&#x7229;",
    "爪" => "&#x722A;",
    "爫" => "&#x722B;",
    "爬" => "&#x722C;",
    "爭" => "&#x722D;",
    "爮" => "&#x722E;",
    "爯" => "&#x722F;",
    "爰" => "&#x7230;",
    "爱" => "&#x7231;",
    "爲" => "&#x7232;",
    "爳" => "&#x7233;",
    "爴" => "&#x7234;",
    "爵" => "&#x7235;",
    "父" => "&#x7236;",
    "爷" => "&#x7237;",
    "爸" => "&#x7238;",
    "爹" => "&#x7239;",
    "爺" => "&#x723A;",
    "爻" => "&#x723B;",
    "爼" => "&#x723C;",
    "爽" => "&#x723D;",
    "爾" => "&#x723E;",
    "爿" => "&#x723F;",
    "牀" => "&#x7240;",
    "牁" => "&#x7241;",
    "牂" => "&#x7242;",
    "牃" => "&#x7243;",
    "牄" => "&#x7244;",
    "牅" => "&#x7245;",
    "牆" => "&#x7246;",
    "片" => "&#x7247;",
    "版" => "&#x7248;",
    "牉" => "&#x7249;",
    "牊" => "&#x724A;",
    "牋" => "&#x724B;",
    "牌" => "&#x724C;",
    "牍" => "&#x724D;",
    "牎" => "&#x724E;",
    "牏" => "&#x724F;",
    "牐" => "&#x7250;",
    "牑" => "&#x7251;",
    "牒" => "&#x7252;",
    "牓" => "&#x7253;",
    "牔" => "&#x7254;",
    "牕" => "&#x7255;",
    "牖" => "&#x7256;",
    "牗" => "&#x7257;",
    "牘" => "&#x7258;",
    "牙" => "&#x7259;",
    "牚" => "&#x725A;",
    "牛" => "&#x725B;",
    "牜" => "&#x725C;",
    "牝" => "&#x725D;",
    "牞" => "&#x725E;",
    "牟" => "&#x725F;",
    "牠" => "&#x7260;",
    "牡" => "&#x7261;",
    "牢" => "&#x7262;",
    "牣" => "&#x7263;",
    "牤" => "&#x7264;",
    "牥" => "&#x7265;",
    "牦" => "&#x7266;",
    "牧" => "&#x7267;",
    "牨" => "&#x7268;",
    "物" => "&#x7269;",
    "牪" => "&#x726A;",
    "牫" => "&#x726B;",
    "牬" => "&#x726C;",
    "牭" => "&#x726D;",
    "牮" => "&#x726E;",
    "牯" => "&#x726F;",
    "牰" => "&#x7270;",
    "牱" => "&#x7271;",
    "牲" => "&#x7272;",
    "牳" => "&#x7273;",
    "牴" => "&#x7274;",
    "牵" => "&#x7275;",
    "牶" => "&#x7276;",
    "牷" => "&#x7277;",
    "牸" => "&#x7278;",
    "特" => "&#x7279;",
    "牺" => "&#x727A;",
    "牻" => "&#x727B;",
    "牼" => "&#x727C;",
    "牽" => "&#x727D;",
    "牾" => "&#x727E;",
    "牿" => "&#x727F;",
    "犀" => "&#x7280;",
    "犁" => "&#x7281;",
    "犂" => "&#x7282;",
    "犃" => "&#x7283;",
    "犄" => "&#x7284;",
    "犅" => "&#x7285;",
    "犆" => "&#x7286;",
    "犇" => "&#x7287;",
    "犈" => "&#x7288;",
    "犉" => "&#x7289;",
    "犊" => "&#x728A;",
    "犋" => "&#x728B;",
    "犌" => "&#x728C;",
    "犍" => "&#x728D;",
    "犎" => "&#x728E;",
    "犏" => "&#x728F;",
    "犐" => "&#x7290;",
    "犑" => "&#x7291;",
    "犒" => "&#x7292;",
    "犓" => "&#x7293;",
    "犔" => "&#x7294;",
    "犕" => "&#x7295;",
    "犖" => "&#x7296;",
    "犗" => "&#x7297;",
    "犘" => "&#x7298;",
    "犙" => "&#x7299;",
    "犚" => "&#x729A;",
    "犛" => "&#x729B;",
    "犜" => "&#x729C;",
    "犝" => "&#x729D;",
    "犞" => "&#x729E;",
    "犟" => "&#x729F;",
    "犠" => "&#x72A0;",
    "犡" => "&#x72A1;",
    "犢" => "&#x72A2;",
    "犣" => "&#x72A3;",
    "犤" => "&#x72A4;",
    "犥" => "&#x72A5;",
    "犦" => "&#x72A6;",
    "犧" => "&#x72A7;",
    "犨" => "&#x72A8;",
    "犩" => "&#x72A9;",
    "犪" => "&#x72AA;",
    "犫" => "&#x72AB;",
    "犬" => "&#x72AC;",
    "犭" => "&#x72AD;",
    "犮" => "&#x72AE;",
    "犯" => "&#x72AF;",
    "犰" => "&#x72B0;",
    "犱" => "&#x72B1;",
    "犲" => "&#x72B2;",
    "犳" => "&#x72B3;",
    "犴" => "&#x72B4;",
    "犵" => "&#x72B5;",
    "状" => "&#x72B6;",
    "犷" => "&#x72B7;",
    "犸" => "&#x72B8;",
    "犹" => "&#x72B9;",
    "犺" => "&#x72BA;",
    "犻" => "&#x72BB;",
    "犼" => "&#x72BC;",
    "犽" => "&#x72BD;",
    "犾" => "&#x72BE;",
    "犿" => "&#x72BF;",
    "狀" => "&#x72C0;",
    "狁" => "&#x72C1;",
    "狂" => "&#x72C2;",
    "狃" => "&#x72C3;",
    "狄" => "&#x72C4;",
    "狅" => "&#x72C5;",
    "狆" => "&#x72C6;",
    "狇" => "&#x72C7;",
    "狈" => "&#x72C8;",
    "狉" => "&#x72C9;",
    "狊" => "&#x72CA;",
    "狋" => "&#x72CB;",
    "狌" => "&#x72CC;",
    "狍" => "&#x72CD;",
    "狎" => "&#x72CE;",
    "狏" => "&#x72CF;",
    "狐" => "&#x72D0;",
    "狑" => "&#x72D1;",
    "狒" => "&#x72D2;",
    "狓" => "&#x72D3;",
    "狔" => "&#x72D4;",
    "狕" => "&#x72D5;",
    "狖" => "&#x72D6;",
    "狗" => "&#x72D7;",
    "狘" => "&#x72D8;",
    "狙" => "&#x72D9;",
    "狚" => "&#x72DA;",
    "狛" => "&#x72DB;",
    "狜" => "&#x72DC;",
    "狝" => "&#x72DD;",
    "狞" => "&#x72DE;",
    "狟" => "&#x72DF;",
    "狠" => "&#x72E0;",
    "狡" => "&#x72E1;",
    "狢" => "&#x72E2;",
    "狣" => "&#x72E3;",
    "狤" => "&#x72E4;",
    "狥" => "&#x72E5;",
    "狦" => "&#x72E6;",
    "狧" => "&#x72E7;",
    "狨" => "&#x72E8;",
    "狩" => "&#x72E9;",
    "狪" => "&#x72EA;",
    "狫" => "&#x72EB;",
    "独" => "&#x72EC;",
    "狭" => "&#x72ED;",
    "狮" => "&#x72EE;",
    "狯" => "&#x72EF;",
    "狰" => "&#x72F0;",
    "狱" => "&#x72F1;",
    "狲" => "&#x72F2;",
    "狳" => "&#x72F3;",
    "狴" => "&#x72F4;",
    "狵" => "&#x72F5;",
    "狶" => "&#x72F6;",
    "狷" => "&#x72F7;",
    "狸" => "&#x72F8;",
    "狹" => "&#x72F9;",
    "狺" => "&#x72FA;",
    "狻" => "&#x72FB;",
    "狼" => "&#x72FC;",
    "狽" => "&#x72FD;",
    "狾" => "&#x72FE;",
    "狿" => "&#x72FF;",
    "猀" => "&#x7300;",
    "猁" => "&#x7301;",
    "猂" => "&#x7302;",
    "猃" => "&#x7303;",
    "猄" => "&#x7304;",
    "猅" => "&#x7305;",
    "猆" => "&#x7306;",
    "猇" => "&#x7307;",
    "猈" => "&#x7308;",
    "猉" => "&#x7309;",
    "猊" => "&#x730A;",
    "猋" => "&#x730B;",
    "猌" => "&#x730C;",
    "猍" => "&#x730D;",
    "猎" => "&#x730E;",
    "猏" => "&#x730F;",
    "猐" => "&#x7310;",
    "猑" => "&#x7311;",
    "猒" => "&#x7312;",
    "猓" => "&#x7313;",
    "猔" => "&#x7314;",
    "猕" => "&#x7315;",
    "猖" => "&#x7316;",
    "猗" => "&#x7317;",
    "猘" => "&#x7318;",
    "猙" => "&#x7319;",
    "猚" => "&#x731A;",
    "猛" => "&#x731B;",
    "猜" => "&#x731C;",
    "猝" => "&#x731D;",
    "猞" => "&#x731E;",
    "猟" => "&#x731F;",
    "猠" => "&#x7320;",
    "猡" => "&#x7321;",
    "猢" => "&#x7322;",
    "猣" => "&#x7323;",
    "猤" => "&#x7324;",
    "猥" => "&#x7325;",
    "猦" => "&#x7326;",
    "猧" => "&#x7327;",
    "猨" => "&#x7328;",
    "猩" => "&#x7329;",
    "猪" => "&#x732A;",
    "猫" => "&#x732B;",
    "猬" => "&#x732C;",
    "猭" => "&#x732D;",
    "献" => "&#x732E;",
    "猯" => "&#x732F;",
    "猰" => "&#x7330;",
    "猱" => "&#x7331;",
    "猲" => "&#x7332;",
    "猳" => "&#x7333;",
    "猴" => "&#x7334;",
    "猵" => "&#x7335;",
    "猶" => "&#x7336;",
    "猷" => "&#x7337;",
    "猸" => "&#x7338;",
    "猹" => "&#x7339;",
    "猺" => "&#x733A;",
    "猻" => "&#x733B;",
    "猼" => "&#x733C;",
    "猽" => "&#x733D;",
    "猾" => "&#x733E;",
    "猿" => "&#x733F;",
    "獀" => "&#x7340;",
    "獁" => "&#x7341;",
    "獂" => "&#x7342;",
    "獃" => "&#x7343;",
    "獄" => "&#x7344;",
    "獅" => "&#x7345;",
    "獆" => "&#x7346;",
    "獇" => "&#x7347;",
    "獈" => "&#x7348;",
    "獉" => "&#x7349;",
    "獊" => "&#x734A;",
    "獋" => "&#x734B;",
    "獌" => "&#x734C;",
    "獍" => "&#x734D;",
    "獎" => "&#x734E;",
    "獏" => "&#x734F;",
    "獐" => "&#x7350;",
    "獑" => "&#x7351;",
    "獒" => "&#x7352;",
    "獓" => "&#x7353;",
    "獔" => "&#x7354;",
    "獕" => "&#x7355;",
    "獖" => "&#x7356;",
    "獗" => "&#x7357;",
    "獘" => "&#x7358;",
    "獙" => "&#x7359;",
    "獚" => "&#x735A;",
    "獛" => "&#x735B;",
    "獜" => "&#x735C;",
    "獝" => "&#x735D;",
    "獞" => "&#x735E;",
    "獟" => "&#x735F;",
    "獠" => "&#x7360;",
    "獡" => "&#x7361;",
    "獢" => "&#x7362;",
    "獣" => "&#x7363;",
    "獤" => "&#x7364;",
    "獥" => "&#x7365;",
    "獦" => "&#x7366;",
    "獧" => "&#x7367;",
    "獨" => "&#x7368;",
    "獩" => "&#x7369;",
    "獪" => "&#x736A;",
    "獫" => "&#x736B;",
    "獬" => "&#x736C;",
    "獭" => "&#x736D;",
    "獮" => "&#x736E;",
    "獯" => "&#x736F;",
    "獰" => "&#x7370;",
    "獱" => "&#x7371;",
    "獲" => "&#x7372;",
    "獳" => "&#x7373;",
    "獴" => "&#x7374;",
    "獵" => "&#x7375;",
    "獶" => "&#x7376;",
    "獷" => "&#x7377;",
    "獸" => "&#x7378;",
    "獹" => "&#x7379;",
    "獺" => "&#x737A;",
    "獻" => "&#x737B;",
    "獼" => "&#x737C;",
    "獽" => "&#x737D;",
    "獾" => "&#x737E;",
    "獿" => "&#x737F;",
    "玀" => "&#x7380;",
    "玁" => "&#x7381;",
    "玂" => "&#x7382;",
    "玃" => "&#x7383;",
    "玄" => "&#x7384;",
    "玅" => "&#x7385;",
    "玆" => "&#x7386;",
    "率" => "&#x7387;",
    "玈" => "&#x7388;",
    "玉" => "&#x7389;",
    "玊" => "&#x738A;",
    "王" => "&#x738B;",
    "玌" => "&#x738C;",
    "玍" => "&#x738D;",
    "玎" => "&#x738E;",
    "玏" => "&#x738F;",
    "玐" => "&#x7390;",
    "玑" => "&#x7391;",
    "玒" => "&#x7392;",
    "玓" => "&#x7393;",
    "玔" => "&#x7394;",
    "玕" => "&#x7395;",
    "玖" => "&#x7396;",
    "玗" => "&#x7397;",
    "玘" => "&#x7398;",
    "玙" => "&#x7399;",
    "玚" => "&#x739A;",
    "玛" => "&#x739B;",
    "玜" => "&#x739C;",
    "玝" => "&#x739D;",
    "玞" => "&#x739E;",
    "玟" => "&#x739F;",
    "玠" => "&#x73A0;",
    "玡" => "&#x73A1;",
    "玢" => "&#x73A2;",
    "玣" => "&#x73A3;",
    "玤" => "&#x73A4;",
    "玥" => "&#x73A5;",
    "玦" => "&#x73A6;",
    "玧" => "&#x73A7;",
    "玨" => "&#x73A8;",
    "玩" => "&#x73A9;",
    "玪" => "&#x73AA;",
    "玫" => "&#x73AB;",
    "玬" => "&#x73AC;",
    "玭" => "&#x73AD;",
    "玮" => "&#x73AE;",
    "环" => "&#x73AF;",
    "现" => "&#x73B0;",
    "玱" => "&#x73B1;",
    "玲" => "&#x73B2;",
    "玳" => "&#x73B3;",
    "玴" => "&#x73B4;",
    "玵" => "&#x73B5;",
    "玶" => "&#x73B6;",
    "玷" => "&#x73B7;",
    "玸" => "&#x73B8;",
    "玹" => "&#x73B9;",
    "玺" => "&#x73BA;",
    "玻" => "&#x73BB;",
    "玼" => "&#x73BC;",
    "玽" => "&#x73BD;",
    "玾" => "&#x73BE;",
    "玿" => "&#x73BF;",
    "珀" => "&#x73C0;",
    "珁" => "&#x73C1;",
    "珂" => "&#x73C2;",
    "珃" => "&#x73C3;",
    "珄" => "&#x73C4;",
    "珅" => "&#x73C5;",
    "珆" => "&#x73C6;",
    "珇" => "&#x73C7;",
    "珈" => "&#x73C8;",
    "珉" => "&#x73C9;",
    "珊" => "&#x73CA;",
    "珋" => "&#x73CB;",
    "珌" => "&#x73CC;",
    "珍" => "&#x73CD;",
    "珎" => "&#x73CE;",
    "珏" => "&#x73CF;",
    "珐" => "&#x73D0;",
    "珑" => "&#x73D1;",
    "珒" => "&#x73D2;",
    "珓" => "&#x73D3;",
    "珔" => "&#x73D4;",
    "珕" => "&#x73D5;",
    "珖" => "&#x73D6;",
    "珗" => "&#x73D7;",
    "珘" => "&#x73D8;",
    "珙" => "&#x73D9;",
    "珚" => "&#x73DA;",
    "珛" => "&#x73DB;",
    "珜" => "&#x73DC;",
    "珝" => "&#x73DD;",
    "珞" => "&#x73DE;",
    "珟" => "&#x73DF;",
    "珠" => "&#x73E0;",
    "珡" => "&#x73E1;",
    "珢" => "&#x73E2;",
    "珣" => "&#x73E3;",
    "珤" => "&#x73E4;",
    "珥" => "&#x73E5;",
    "珦" => "&#x73E6;",
    "珧" => "&#x73E7;",
    "珨" => "&#x73E8;",
    "珩" => "&#x73E9;",
    "珪" => "&#x73EA;",
    "珫" => "&#x73EB;",
    "珬" => "&#x73EC;",
    "班" => "&#x73ED;",
    "珮" => "&#x73EE;",
    "珯" => "&#x73EF;",
    "珰" => "&#x73F0;",
    "珱" => "&#x73F1;",
    "珲" => "&#x73F2;",
    "珳" => "&#x73F3;",
    "珴" => "&#x73F4;",
    "珵" => "&#x73F5;",
    "珶" => "&#x73F6;",
    "珷" => "&#x73F7;",
    "珸" => "&#x73F8;",
    "珹" => "&#x73F9;",
    "珺" => "&#x73FA;",
    "珻" => "&#x73FB;",
    "珼" => "&#x73FC;",
    "珽" => "&#x73FD;",
    "現" => "&#x73FE;",
    "珿" => "&#x73FF;",
    "琀" => "&#x7400;",
    "琁" => "&#x7401;",
    "琂" => "&#x7402;",
    "球" => "&#x7403;",
    "琄" => "&#x7404;",
    "琅" => "&#x7405;",
    "理" => "&#x7406;",
    "琇" => "&#x7407;",
    "琈" => "&#x7408;",
    "琉" => "&#x7409;",
    "琊" => "&#x740A;",
    "琋" => "&#x740B;",
    "琌" => "&#x740C;",
    "琍" => "&#x740D;",
    "琎" => "&#x740E;",
    "琏" => "&#x740F;",
    "琐" => "&#x7410;",
    "琑" => "&#x7411;",
    "琒" => "&#x7412;",
    "琓" => "&#x7413;",
    "琔" => "&#x7414;",
    "琕" => "&#x7415;",
    "琖" => "&#x7416;",
    "琗" => "&#x7417;",
    "琘" => "&#x7418;",
    "琙" => "&#x7419;",
    "琚" => "&#x741A;",
    "琛" => "&#x741B;",
    "琜" => "&#x741C;",
    "琝" => "&#x741D;",
    "琞" => "&#x741E;",
    "琟" => "&#x741F;",
    "琠" => "&#x7420;",
    "琡" => "&#x7421;",
    "琢" => "&#x7422;",
    "琣" => "&#x7423;",
    "琤" => "&#x7424;",
    "琥" => "&#x7425;",
    "琦" => "&#x7426;",
    "琧" => "&#x7427;",
    "琨" => "&#x7428;",
    "琩" => "&#x7429;",
    "琪" => "&#x742A;",
    "琫" => "&#x742B;",
    "琬" => "&#x742C;",
    "琭" => "&#x742D;",
    "琮" => "&#x742E;",
    "琯" => "&#x742F;",
    "琰" => "&#x7430;",
    "琱" => "&#x7431;",
    "琲" => "&#x7432;",
    "琳" => "&#x7433;",
    "琴" => "&#x7434;",
    "琵" => "&#x7435;",
    "琶" => "&#x7436;",
    "琷" => "&#x7437;",
    "琸" => "&#x7438;",
    "琹" => "&#x7439;",
    "琺" => "&#x743A;",
    "琻" => "&#x743B;",
    "琼" => "&#x743C;",
    "琽" => "&#x743D;",
    "琾" => "&#x743E;",
    "琿" => "&#x743F;",
    "瑀" => "&#x7440;",
    "瑁" => "&#x7441;",
    "瑂" => "&#x7442;",
    "瑃" => "&#x7443;",
    "瑄" => "&#x7444;",
    "瑅" => "&#x7445;",
    "瑆" => "&#x7446;",
    "瑇" => "&#x7447;",
    "瑈" => "&#x7448;",
    "瑉" => "&#x7449;",
    "瑊" => "&#x744A;",
    "瑋" => "&#x744B;",
    "瑌" => "&#x744C;",
    "瑍" => "&#x744D;",
    "瑎" => "&#x744E;",
    "瑏" => "&#x744F;",
    "瑐" => "&#x7450;",
    "瑑" => "&#x7451;",
    "瑒" => "&#x7452;",
    "瑓" => "&#x7453;",
    "瑔" => "&#x7454;",
    "瑕" => "&#x7455;",
    "瑖" => "&#x7456;",
    "瑗" => "&#x7457;",
    "瑘" => "&#x7458;",
    "瑙" => "&#x7459;",
    "瑚" => "&#x745A;",
    "瑛" => "&#x745B;",
    "瑜" => "&#x745C;",
    "瑝" => "&#x745D;",
    "瑞" => "&#x745E;",
    "瑟" => "&#x745F;",
    "瑠" => "&#x7460;",
    "瑡" => "&#x7461;",
    "瑢" => "&#x7462;",
    "瑣" => "&#x7463;",
    "瑤" => "&#x7464;",
    "瑥" => "&#x7465;",
    "瑦" => "&#x7466;",
    "瑧" => "&#x7467;",
    "瑨" => "&#x7468;",
    "瑩" => "&#x7469;",
    "瑪" => "&#x746A;",
    "瑫" => "&#x746B;",
    "瑬" => "&#x746C;",
    "瑭" => "&#x746D;",
    "瑮" => "&#x746E;",
    "瑯" => "&#x746F;",
    "瑰" => "&#x7470;",
    "瑱" => "&#x7471;",
    "瑲" => "&#x7472;",
    "瑳" => "&#x7473;",
    "瑴" => "&#x7474;",
    "瑵" => "&#x7475;",
    "瑶" => "&#x7476;",
    "瑷" => "&#x7477;",
    "瑸" => "&#x7478;",
    "瑹" => "&#x7479;",
    "瑺" => "&#x747A;",
    "瑻" => "&#x747B;",
    "瑼" => "&#x747C;",
    "瑽" => "&#x747D;",
    "瑾" => "&#x747E;",
    "瑿" => "&#x747F;",
    "璀" => "&#x7480;",
    "璁" => "&#x7481;",
    "璂" => "&#x7482;",
    "璃" => "&#x7483;",
    "璄" => "&#x7484;",
    "璅" => "&#x7485;",
    "璆" => "&#x7486;",
    "璇" => "&#x7487;",
    "璈" => "&#x7488;",
    "璉" => "&#x7489;",
    "璊" => "&#x748A;",
    "璋" => "&#x748B;",
    "璌" => "&#x748C;",
    "璍" => "&#x748D;",
    "璎" => "&#x748E;",
    "璏" => "&#x748F;",
    "璐" => "&#x7490;",
    "璑" => "&#x7491;",
    "璒" => "&#x7492;",
    "璓" => "&#x7493;",
    "璔" => "&#x7494;",
    "璕" => "&#x7495;",
    "璖" => "&#x7496;",
    "璗" => "&#x7497;",
    "璘" => "&#x7498;",
    "璙" => "&#x7499;",
    "璚" => "&#x749A;",
    "璛" => "&#x749B;",
    "璜" => "&#x749C;",
    "璝" => "&#x749D;",
    "璞" => "&#x749E;",
    "璟" => "&#x749F;",
    "璠" => "&#x74A0;",
    "璡" => "&#x74A1;",
    "璢" => "&#x74A2;",
    "璣" => "&#x74A3;",
    "璤" => "&#x74A4;",
    "璥" => "&#x74A5;",
    "璦" => "&#x74A6;",
    "璧" => "&#x74A7;",
    "璨" => "&#x74A8;",
    "璩" => "&#x74A9;",
    "璪" => "&#x74AA;",
    "璫" => "&#x74AB;",
    "璬" => "&#x74AC;",
    "璭" => "&#x74AD;",
    "璮" => "&#x74AE;",
    "璯" => "&#x74AF;",
    "環" => "&#x74B0;",
    "璱" => "&#x74B1;",
    "璲" => "&#x74B2;",
    "璳" => "&#x74B3;",
    "璴" => "&#x74B4;",
    "璵" => "&#x74B5;",
    "璶" => "&#x74B6;",
    "璷" => "&#x74B7;",
    "璸" => "&#x74B8;",
    "璹" => "&#x74B9;",
    "璺" => "&#x74BA;",
    "璻" => "&#x74BB;",
    "璼" => "&#x74BC;",
    "璽" => "&#x74BD;",
    "璾" => "&#x74BE;",
    "璿" => "&#x74BF;",
    "瓀" => "&#x74C0;",
    "瓁" => "&#x74C1;",
    "瓂" => "&#x74C2;",
    "瓃" => "&#x74C3;",
    "瓄" => "&#x74C4;",
    "瓅" => "&#x74C5;",
    "瓆" => "&#x74C6;",
    "瓇" => "&#x74C7;",
    "瓈" => "&#x74C8;",
    "瓉" => "&#x74C9;",
    "瓊" => "&#x74CA;",
    "瓋" => "&#x74CB;",
    "瓌" => "&#x74CC;",
    "瓍" => "&#x74CD;",
    "瓎" => "&#x74CE;",
    "瓏" => "&#x74CF;",
    "瓐" => "&#x74D0;",
    "瓑" => "&#x74D1;",
    "瓒" => "&#x74D2;",
    "瓓" => "&#x74D3;",
    "瓔" => "&#x74D4;",
    "瓕" => "&#x74D5;",
    "瓖" => "&#x74D6;",
    "瓗" => "&#x74D7;",
    "瓘" => "&#x74D8;",
    "瓙" => "&#x74D9;",
    "瓚" => "&#x74DA;",
    "瓛" => "&#x74DB;",
    "瓜" => "&#x74DC;",
    "瓝" => "&#x74DD;",
    "瓞" => "&#x74DE;",
    "瓟" => "&#x74DF;",
    "瓠" => "&#x74E0;",
    "瓡" => "&#x74E1;",
    "瓢" => "&#x74E2;",
    "瓣" => "&#x74E3;",
    "瓤" => "&#x74E4;",
    "瓥" => "&#x74E5;",
    "瓦" => "&#x74E6;",
    "瓧" => "&#x74E7;",
    "瓨" => "&#x74E8;",
    "瓩" => "&#x74E9;",
    "瓪" => "&#x74EA;",
    "瓫" => "&#x74EB;",
    "瓬" => "&#x74EC;",
    "瓭" => "&#x74ED;",
    "瓮" => "&#x74EE;",
    "瓯" => "&#x74EF;",
    "瓰" => "&#x74F0;",
    "瓱" => "&#x74F1;",
    "瓲" => "&#x74F2;",
    "瓳" => "&#x74F3;",
    "瓴" => "&#x74F4;",
    "瓵" => "&#x74F5;",
    "瓶" => "&#x74F6;",
    "瓷" => "&#x74F7;",
    "瓸" => "&#x74F8;",
    "瓹" => "&#x74F9;",
    "瓺" => "&#x74FA;",
    "瓻" => "&#x74FB;",
    "瓼" => "&#x74FC;",
    "瓽" => "&#x74FD;",
    "瓾" => "&#x74FE;",
    "瓿" => "&#x74FF;",
    "甀" => "&#x7500;",
    "甁" => "&#x7501;",
    "甂" => "&#x7502;",
    "甃" => "&#x7503;",
    "甄" => "&#x7504;",
    "甅" => "&#x7505;",
    "甆" => "&#x7506;",
    "甇" => "&#x7507;",
    "甈" => "&#x7508;",
    "甉" => "&#x7509;",
    "甊" => "&#x750A;",
    "甋" => "&#x750B;",
    "甌" => "&#x750C;",
    "甍" => "&#x750D;",
    "甎" => "&#x750E;",
    "甏" => "&#x750F;",
    "甐" => "&#x7510;",
    "甑" => "&#x7511;",
    "甒" => "&#x7512;",
    "甓" => "&#x7513;",
    "甔" => "&#x7514;",
    "甕" => "&#x7515;",
    "甖" => "&#x7516;",
    "甗" => "&#x7517;",
    "甘" => "&#x7518;",
    "甙" => "&#x7519;",
    "甚" => "&#x751A;",
    "甛" => "&#x751B;",
    "甜" => "&#x751C;",
    "甝" => "&#x751D;",
    "甞" => "&#x751E;",
    "生" => "&#x751F;",
    "甠" => "&#x7520;",
    "甡" => "&#x7521;",
    "產" => "&#x7522;",
    "産" => "&#x7523;",
    "甤" => "&#x7524;",
    "甥" => "&#x7525;",
    "甦" => "&#x7526;",
    "甧" => "&#x7527;",
    "用" => "&#x7528;",
    "甩" => "&#x7529;",
    "甪" => "&#x752A;",
    "甫" => "&#x752B;",
    "甬" => "&#x752C;",
    "甭" => "&#x752D;",
    "甮" => "&#x752E;",
    "甯" => "&#x752F;",
    "田" => "&#x7530;",
    "由" => "&#x7531;",
    "甲" => "&#x7532;",
    "申" => "&#x7533;",
    "甴" => "&#x7534;",
    "电" => "&#x7535;",
    "甶" => "&#x7536;",
    "男" => "&#x7537;",
    "甸" => "&#x7538;",
    "甹" => "&#x7539;",
    "町" => "&#x753A;",
    "画" => "&#x753B;",
    "甼" => "&#x753C;",
    "甽" => "&#x753D;",
    "甾" => "&#x753E;",
    "甿" => "&#x753F;",
    "畀" => "&#x7540;",
    "畁" => "&#x7541;",
    "畂" => "&#x7542;",
    "畃" => "&#x7543;",
    "畄" => "&#x7544;",
    "畅" => "&#x7545;",
    "畆" => "&#x7546;",
    "畇" => "&#x7547;",
    "畈" => "&#x7548;",
    "畉" => "&#x7549;",
    "畊" => "&#x754A;",
    "畋" => "&#x754B;",
    "界" => "&#x754C;",
    "畍" => "&#x754D;",
    "畎" => "&#x754E;",
    "畏" => "&#x754F;",
    "畐" => "&#x7550;",
    "畑" => "&#x7551;",
    "畒" => "&#x7552;",
    "畓" => "&#x7553;",
    "畔" => "&#x7554;",
    "畕" => "&#x7555;",
    "畖" => "&#x7556;",
    "畗" => "&#x7557;",
    "畘" => "&#x7558;",
    "留" => "&#x7559;",
    "畚" => "&#x755A;",
    "畛" => "&#x755B;",
    "畜" => "&#x755C;",
    "畝" => "&#x755D;",
    "畞" => "&#x755E;",
    "畟" => "&#x755F;",
    "畠" => "&#x7560;",
    "畡" => "&#x7561;",
    "畢" => "&#x7562;",
    "畣" => "&#x7563;",
    "畤" => "&#x7564;",
    "略" => "&#x7565;",
    "畦" => "&#x7566;",
    "畧" => "&#x7567;",
    "畨" => "&#x7568;",
    "畩" => "&#x7569;",
    "番" => "&#x756A;",
    "畫" => "&#x756B;",
    "畬" => "&#x756C;",
    "畭" => "&#x756D;",
    "畮" => "&#x756E;",
    "畯" => "&#x756F;",
    "異" => "&#x7570;",
    "畱" => "&#x7571;",
    "畲" => "&#x7572;",
    "畳" => "&#x7573;",
    "畴" => "&#x7574;",
    "畵" => "&#x7575;",
    "當" => "&#x7576;",
    "畷" => "&#x7577;",
    "畸" => "&#x7578;",
    "畹" => "&#x7579;",
    "畺" => "&#x757A;",
    "畻" => "&#x757B;",
    "畼" => "&#x757C;",
    "畽" => "&#x757D;",
    "畾" => "&#x757E;",
    "畿" => "&#x757F;",
    "疀" => "&#x7580;",
    "疁" => "&#x7581;",
    "疂" => "&#x7582;",
    "疃" => "&#x7583;",
    "疄" => "&#x7584;",
    "疅" => "&#x7585;",
    "疆" => "&#x7586;",
    "疇" => "&#x7587;",
    "疈" => "&#x7588;",
    "疉" => "&#x7589;",
    "疊" => "&#x758A;",
    "疋" => "&#x758B;",
    "疌" => "&#x758C;",
    "疍" => "&#x758D;",
    "疎" => "&#x758E;",
    "疏" => "&#x758F;",
    "疐" => "&#x7590;",
    "疑" => "&#x7591;",
    "疒" => "&#x7592;",
    "疓" => "&#x7593;",
    "疔" => "&#x7594;",
    "疕" => "&#x7595;",
    "疖" => "&#x7596;",
    "疗" => "&#x7597;",
    "疘" => "&#x7598;",
    "疙" => "&#x7599;",
    "疚" => "&#x759A;",
    "疛" => "&#x759B;",
    "疜" => "&#x759C;",
    "疝" => "&#x759D;",
    "疞" => "&#x759E;",
    "疟" => "&#x759F;",
    "疠" => "&#x75A0;",
    "疡" => "&#x75A1;",
    "疢" => "&#x75A2;",
    "疣" => "&#x75A3;",
    "疤" => "&#x75A4;",
    "疥" => "&#x75A5;",
    "疦" => "&#x75A6;",
    "疧" => "&#x75A7;",
    "疨" => "&#x75A8;",
    "疩" => "&#x75A9;",
    "疪" => "&#x75AA;",
    "疫" => "&#x75AB;",
    "疬" => "&#x75AC;",
    "疭" => "&#x75AD;",
    "疮" => "&#x75AE;",
    "疯" => "&#x75AF;",
    "疰" => "&#x75B0;",
    "疱" => "&#x75B1;",
    "疲" => "&#x75B2;",
    "疳" => "&#x75B3;",
    "疴" => "&#x75B4;",
    "疵" => "&#x75B5;",
    "疶" => "&#x75B6;",
    "疷" => "&#x75B7;",
    "疸" => "&#x75B8;",
    "疹" => "&#x75B9;",
    "疺" => "&#x75BA;",
    "疻" => "&#x75BB;",
    "疼" => "&#x75BC;",
    "疽" => "&#x75BD;",
    "疾" => "&#x75BE;",
    "疿" => "&#x75BF;",
    "痀" => "&#x75C0;",
    "痁" => "&#x75C1;",
    "痂" => "&#x75C2;",
    "痃" => "&#x75C3;",
    "痄" => "&#x75C4;",
    "病" => "&#x75C5;",
    "痆" => "&#x75C6;",
    "症" => "&#x75C7;",
    "痈" => "&#x75C8;",
    "痉" => "&#x75C9;",
    "痊" => "&#x75CA;",
    "痋" => "&#x75CB;",
    "痌" => "&#x75CC;",
    "痍" => "&#x75CD;",
    "痎" => "&#x75CE;",
    "痏" => "&#x75CF;",
    "痐" => "&#x75D0;",
    "痑" => "&#x75D1;",
    "痒" => "&#x75D2;",
    "痓" => "&#x75D3;",
    "痔" => "&#x75D4;",
    "痕" => "&#x75D5;",
    "痖" => "&#x75D6;",
    "痗" => "&#x75D7;",
    "痘" => "&#x75D8;",
    "痙" => "&#x75D9;",
    "痚" => "&#x75DA;",
    "痛" => "&#x75DB;",
    "痜" => "&#x75DC;",
    "痝" => "&#x75DD;",
    "痞" => "&#x75DE;",
    "痟" => "&#x75DF;",
    "痠" => "&#x75E0;",
    "痡" => "&#x75E1;",
    "痢" => "&#x75E2;",
    "痣" => "&#x75E3;",
    "痤" => "&#x75E4;",
    "痥" => "&#x75E5;",
    "痦" => "&#x75E6;",
    "痧" => "&#x75E7;",
    "痨" => "&#x75E8;",
    "痩" => "&#x75E9;",
    "痪" => "&#x75EA;",
    "痫" => "&#x75EB;",
    "痬" => "&#x75EC;",
    "痭" => "&#x75ED;",
    "痮" => "&#x75EE;",
    "痯" => "&#x75EF;",
    "痰" => "&#x75F0;",
    "痱" => "&#x75F1;",
    "痲" => "&#x75F2;",
    "痳" => "&#x75F3;",
    "痴" => "&#x75F4;",
    "痵" => "&#x75F5;",
    "痶" => "&#x75F6;",
    "痷" => "&#x75F7;",
    "痸" => "&#x75F8;",
    "痹" => "&#x75F9;",
    "痺" => "&#x75FA;",
    "痻" => "&#x75FB;",
    "痼" => "&#x75FC;",
    "痽" => "&#x75FD;",
    "痾" => "&#x75FE;",
    "痿" => "&#x75FF;",
    "瘀" => "&#x7600;",
    "瘁" => "&#x7601;",
    "瘂" => "&#x7602;",
    "瘃" => "&#x7603;",
    "瘄" => "&#x7604;",
    "瘅" => "&#x7605;",
    "瘆" => "&#x7606;",
    "瘇" => "&#x7607;",
    "瘈" => "&#x7608;",
    "瘉" => "&#x7609;",
    "瘊" => "&#x760A;",
    "瘋" => "&#x760B;",
    "瘌" => "&#x760C;",
    "瘍" => "&#x760D;",
    "瘎" => "&#x760E;",
    "瘏" => "&#x760F;",
    "瘐" => "&#x7610;",
    "瘑" => "&#x7611;",
    "瘒" => "&#x7612;",
    "瘓" => "&#x7613;",
    "瘔" => "&#x7614;",
    "瘕" => "&#x7615;",
    "瘖" => "&#x7616;",
    "瘗" => "&#x7617;",
    "瘘" => "&#x7618;",
    "瘙" => "&#x7619;",
    "瘚" => "&#x761A;",
    "瘛" => "&#x761B;",
    "瘜" => "&#x761C;",
    "瘝" => "&#x761D;",
    "瘞" => "&#x761E;",
    "瘟" => "&#x761F;",
    "瘠" => "&#x7620;",
    "瘡" => "&#x7621;",
    "瘢" => "&#x7622;",
    "瘣" => "&#x7623;",
    "瘤" => "&#x7624;",
    "瘥" => "&#x7625;",
    "瘦" => "&#x7626;",
    "瘧" => "&#x7627;",
    "瘨" => "&#x7628;",
    "瘩" => "&#x7629;",
    "瘪" => "&#x762A;",
    "瘫" => "&#x762B;",
    "瘬" => "&#x762C;",
    "瘭" => "&#x762D;",
    "瘮" => "&#x762E;",
    "瘯" => "&#x762F;",
    "瘰" => "&#x7630;",
    "瘱" => "&#x7631;",
    "瘲" => "&#x7632;",
    "瘳" => "&#x7633;",
    "瘴" => "&#x7634;",
    "瘵" => "&#x7635;",
    "瘶" => "&#x7636;",
    "瘷" => "&#x7637;",
    "瘸" => "&#x7638;",
    "瘹" => "&#x7639;",
    "瘺" => "&#x763A;",
    "瘻" => "&#x763B;",
    "瘼" => "&#x763C;",
    "瘽" => "&#x763D;",
    "瘾" => "&#x763E;",
    "瘿" => "&#x763F;",
    "癀" => "&#x7640;",
    "癁" => "&#x7641;",
    "療" => "&#x7642;",
    "癃" => "&#x7643;",
    "癄" => "&#x7644;",
    "癅" => "&#x7645;",
    "癆" => "&#x7646;",
    "癇" => "&#x7647;",
    "癈" => "&#x7648;",
    "癉" => "&#x7649;",
    "癊" => "&#x764A;",
    "癋" => "&#x764B;",
    "癌" => "&#x764C;",
    "癍" => "&#x764D;",
    "癎" => "&#x764E;",
    "癏" => "&#x764F;",
    "癐" => "&#x7650;",
    "癑" => "&#x7651;",
    "癒" => "&#x7652;",
    "癓" => "&#x7653;",
    "癔" => "&#x7654;",
    "癕" => "&#x7655;",
    "癖" => "&#x7656;",
    "癗" => "&#x7657;",
    "癘" => "&#x7658;",
    "癙" => "&#x7659;",
    "癚" => "&#x765A;",
    "癛" => "&#x765B;",
    "癜" => "&#x765C;",
    "癝" => "&#x765D;",
    "癞" => "&#x765E;",
    "癟" => "&#x765F;",
    "癠" => "&#x7660;",
    "癡" => "&#x7661;",
    "癢" => "&#x7662;",
    "癣" => "&#x7663;",
    "癤" => "&#x7664;",
    "癥" => "&#x7665;",
    "癦" => "&#x7666;",
    "癧" => "&#x7667;",
    "癨" => "&#x7668;",
    "癩" => "&#x7669;",
    "癪" => "&#x766A;",
    "癫" => "&#x766B;",
    "癬" => "&#x766C;",
    "癭" => "&#x766D;",
    "癮" => "&#x766E;",
    "癯" => "&#x766F;",
    "癰" => "&#x7670;",
    "癱" => "&#x7671;",
    "癲" => "&#x7672;",
    "癳" => "&#x7673;",
    "癴" => "&#x7674;",
    "癵" => "&#x7675;",
    "癶" => "&#x7676;",
    "癷" => "&#x7677;",
    "癸" => "&#x7678;",
    "癹" => "&#x7679;",
    "発" => "&#x767A;",
    "登" => "&#x767B;",
    "發" => "&#x767C;",
    "白" => "&#x767D;",
    "百" => "&#x767E;",
    "癿" => "&#x767F;",
    "皀" => "&#x7680;",
    "皁" => "&#x7681;",
    "皂" => "&#x7682;",
    "皃" => "&#x7683;",
    "的" => "&#x7684;",
    "皅" => "&#x7685;",
    "皆" => "&#x7686;",
    "皇" => "&#x7687;",
    "皈" => "&#x7688;",
    "皉" => "&#x7689;",
    "皊" => "&#x768A;",
    "皋" => "&#x768B;",
    "皌" => "&#x768C;",
    "皍" => "&#x768D;",
    "皎" => "&#x768E;",
    "皏" => "&#x768F;",
    "皐" => "&#x7690;",
    "皑" => "&#x7691;",
    "皒" => "&#x7692;",
    "皓" => "&#x7693;",
    "皔" => "&#x7694;",
    "皕" => "&#x7695;",
    "皖" => "&#x7696;",
    "皗" => "&#x7697;",
    "皘" => "&#x7698;",
    "皙" => "&#x7699;",
    "皚" => "&#x769A;",
    "皛" => "&#x769B;",
    "皜" => "&#x769C;",
    "皝" => "&#x769D;",
    "皞" => "&#x769E;",
    "皟" => "&#x769F;",
    "皠" => "&#x76A0;",
    "皡" => "&#x76A1;",
    "皢" => "&#x76A2;",
    "皣" => "&#x76A3;",
    "皤" => "&#x76A4;",
    "皥" => "&#x76A5;",
    "皦" => "&#x76A6;",
    "皧" => "&#x76A7;",
    "皨" => "&#x76A8;",
    "皩" => "&#x76A9;",
    "皪" => "&#x76AA;",
    "皫" => "&#x76AB;",
    "皬" => "&#x76AC;",
    "皭" => "&#x76AD;",
    "皮" => "&#x76AE;",
    "皯" => "&#x76AF;",
    "皰" => "&#x76B0;",
    "皱" => "&#x76B1;",
    "皲" => "&#x76B2;",
    "皳" => "&#x76B3;",
    "皴" => "&#x76B4;",
    "皵" => "&#x76B5;",
    "皶" => "&#x76B6;",
    "皷" => "&#x76B7;",
    "皸" => "&#x76B8;",
    "皹" => "&#x76B9;",
    "皺" => "&#x76BA;",
    "皻" => "&#x76BB;",
    "皼" => "&#x76BC;",
    "皽" => "&#x76BD;",
    "皾" => "&#x76BE;",
    "皿" => "&#x76BF;",
    "盀" => "&#x76C0;",
    "盁" => "&#x76C1;",
    "盂" => "&#x76C2;",
    "盃" => "&#x76C3;",
    "盄" => "&#x76C4;",
    "盅" => "&#x76C5;",
    "盆" => "&#x76C6;",
    "盇" => "&#x76C7;",
    "盈" => "&#x76C8;",
    "盉" => "&#x76C9;",
    "益" => "&#x76CA;",
    "盋" => "&#x76CB;",
    "盌" => "&#x76CC;",
    "盍" => "&#x76CD;",
    "盎" => "&#x76CE;",
    "盏" => "&#x76CF;",
    "盐" => "&#x76D0;",
    "监" => "&#x76D1;",
    "盒" => "&#x76D2;",
    "盓" => "&#x76D3;",
    "盔" => "&#x76D4;",
    "盕" => "&#x76D5;",
    "盖" => "&#x76D6;",
    "盗" => "&#x76D7;",
    "盘" => "&#x76D8;",
    "盙" => "&#x76D9;",
    "盚" => "&#x76DA;",
    "盛" => "&#x76DB;",
    "盜" => "&#x76DC;",
    "盝" => "&#x76DD;",
    "盞" => "&#x76DE;",
    "盟" => "&#x76DF;",
    "盠" => "&#x76E0;",
    "盡" => "&#x76E1;",
    "盢" => "&#x76E2;",
    "監" => "&#x76E3;",
    "盤" => "&#x76E4;",
    "盥" => "&#x76E5;",
    "盦" => "&#x76E6;",
    "盧" => "&#x76E7;",
    "盨" => "&#x76E8;",
    "盩" => "&#x76E9;",
    "盪" => "&#x76EA;",
    "盫" => "&#x76EB;",
    "盬" => "&#x76EC;",
    "盭" => "&#x76ED;",
    "目" => "&#x76EE;",
    "盯" => "&#x76EF;",
    "盰" => "&#x76F0;",
    "盱" => "&#x76F1;",
    "盲" => "&#x76F2;",
    "盳" => "&#x76F3;",
    "直" => "&#x76F4;",
    "盵" => "&#x76F5;",
    "盶" => "&#x76F6;",
    "盷" => "&#x76F7;",
    "相" => "&#x76F8;",
    "盹" => "&#x76F9;",
    "盺" => "&#x76FA;",
    "盻" => "&#x76FB;",
    "盼" => "&#x76FC;",
    "盽" => "&#x76FD;",
    "盾" => "&#x76FE;",
    "盿" => "&#x76FF;",
    "眀" => "&#x7700;",
    "省" => "&#x7701;",
    "眂" => "&#x7702;",
    "眃" => "&#x7703;",
    "眄" => "&#x7704;",
    "眅" => "&#x7705;",
    "眆" => "&#x7706;",
    "眇" => "&#x7707;",
    "眈" => "&#x7708;",
    "眉" => "&#x7709;",
    "眊" => "&#x770A;",
    "看" => "&#x770B;",
    "県" => "&#x770C;",
    "眍" => "&#x770D;",
    "眎" => "&#x770E;",
    "眏" => "&#x770F;",
    "眐" => "&#x7710;",
    "眑" => "&#x7711;",
    "眒" => "&#x7712;",
    "眓" => "&#x7713;",
    "眔" => "&#x7714;",
    "眕" => "&#x7715;",
    "眖" => "&#x7716;",
    "眗" => "&#x7717;",
    "眘" => "&#x7718;",
    "眙" => "&#x7719;",
    "眚" => "&#x771A;",
    "眛" => "&#x771B;",
    "眜" => "&#x771C;",
    "眝" => "&#x771D;",
    "眞" => "&#x771E;",
    "真" => "&#x771F;",
    "眠" => "&#x7720;",
    "眡" => "&#x7721;",
    "眢" => "&#x7722;",
    "眣" => "&#x7723;",
    "眤" => "&#x7724;",
    "眥" => "&#x7725;",
    "眦" => "&#x7726;",
    "眧" => "&#x7727;",
    "眨" => "&#x7728;",
    "眩" => "&#x7729;",
    "眪" => "&#x772A;",
    "眫" => "&#x772B;",
    "眬" => "&#x772C;",
    "眭" => "&#x772D;",
    "眮" => "&#x772E;",
    "眯" => "&#x772F;",
    "眰" => "&#x7730;",
    "眱" => "&#x7731;",
    "眲" => "&#x7732;",
    "眳" => "&#x7733;",
    "眴" => "&#x7734;",
    "眵" => "&#x7735;",
    "眶" => "&#x7736;",
    "眷" => "&#x7737;",
    "眸" => "&#x7738;",
    "眹" => "&#x7739;",
    "眺" => "&#x773A;",
    "眻" => "&#x773B;",
    "眼" => "&#x773C;",
    "眽" => "&#x773D;",
    "眾" => "&#x773E;",
    "眿" => "&#x773F;",
    "着" => "&#x7740;",
    "睁" => "&#x7741;",
    "睂" => "&#x7742;",
    "睃" => "&#x7743;",
    "睄" => "&#x7744;",
    "睅" => "&#x7745;",
    "睆" => "&#x7746;",
    "睇" => "&#x7747;",
    "睈" => "&#x7748;",
    "睉" => "&#x7749;",
    "睊" => "&#x774A;",
    "睋" => "&#x774B;",
    "睌" => "&#x774C;",
    "睍" => "&#x774D;",
    "睎" => "&#x774E;",
    "睏" => "&#x774F;",
    "睐" => "&#x7750;",
    "睑" => "&#x7751;",
    "睒" => "&#x7752;",
    "睓" => "&#x7753;",
    "睔" => "&#x7754;",
    "睕" => "&#x7755;",
    "睖" => "&#x7756;",
    "睗" => "&#x7757;",
    "睘" => "&#x7758;",
    "睙" => "&#x7759;",
    "睚" => "&#x775A;",
    "睛" => "&#x775B;",
    "睜" => "&#x775C;",
    "睝" => "&#x775D;",
    "睞" => "&#x775E;",
    "睟" => "&#x775F;",
    "睠" => "&#x7760;",
    "睡" => "&#x7761;",
    "睢" => "&#x7762;",
    "督" => "&#x7763;",
    "睤" => "&#x7764;",
    "睥" => "&#x7765;",
    "睦" => "&#x7766;",
    "睧" => "&#x7767;",
    "睨" => "&#x7768;",
    "睩" => "&#x7769;",
    "睪" => "&#x776A;",
    "睫" => "&#x776B;",
    "睬" => "&#x776C;",
    "睭" => "&#x776D;",
    "睮" => "&#x776E;",
    "睯" => "&#x776F;",
    "睰" => "&#x7770;",
    "睱" => "&#x7771;",
    "睲" => "&#x7772;",
    "睳" => "&#x7773;",
    "睴" => "&#x7774;",
    "睵" => "&#x7775;",
    "睶" => "&#x7776;",
    "睷" => "&#x7777;",
    "睸" => "&#x7778;",
    "睹" => "&#x7779;",
    "睺" => "&#x777A;",
    "睻" => "&#x777B;",
    "睼" => "&#x777C;",
    "睽" => "&#x777D;",
    "睾" => "&#x777E;",
    "睿" => "&#x777F;",
    "瞀" => "&#x7780;",
    "瞁" => "&#x7781;",
    "瞂" => "&#x7782;",
    "瞃" => "&#x7783;",
    "瞄" => "&#x7784;",
    "瞅" => "&#x7785;",
    "瞆" => "&#x7786;",
    "瞇" => "&#x7787;",
    "瞈" => "&#x7788;",
    "瞉" => "&#x7789;",
    "瞊" => "&#x778A;",
    "瞋" => "&#x778B;",
    "瞌" => "&#x778C;",
    "瞍" => "&#x778D;",
    "瞎" => "&#x778E;",
    "瞏" => "&#x778F;",
    "瞐" => "&#x7790;",
    "瞑" => "&#x7791;",
    "瞒" => "&#x7792;",
    "瞓" => "&#x7793;",
    "瞔" => "&#x7794;",
    "瞕" => "&#x7795;",
    "瞖" => "&#x7796;",
    "瞗" => "&#x7797;",
    "瞘" => "&#x7798;",
    "瞙" => "&#x7799;",
    "瞚" => "&#x779A;",
    "瞛" => "&#x779B;",
    "瞜" => "&#x779C;",
    "瞝" => "&#x779D;",
    "瞞" => "&#x779E;",
    "瞟" => "&#x779F;",
    "瞠" => "&#x77A0;",
    "瞡" => "&#x77A1;",
    "瞢" => "&#x77A2;",
    "瞣" => "&#x77A3;",
    "瞤" => "&#x77A4;",
    "瞥" => "&#x77A5;",
    "瞦" => "&#x77A6;",
    "瞧" => "&#x77A7;",
    "瞨" => "&#x77A8;",
    "瞩" => "&#x77A9;",
    "瞪" => "&#x77AA;",
    "瞫" => "&#x77AB;",
    "瞬" => "&#x77AC;",
    "瞭" => "&#x77AD;",
    "瞮" => "&#x77AE;",
    "瞯" => "&#x77AF;",
    "瞰" => "&#x77B0;",
    "瞱" => "&#x77B1;",
    "瞲" => "&#x77B2;",
    "瞳" => "&#x77B3;",
    "瞴" => "&#x77B4;",
    "瞵" => "&#x77B5;",
    "瞶" => "&#x77B6;",
    "瞷" => "&#x77B7;",
    "瞸" => "&#x77B8;",
    "瞹" => "&#x77B9;",
    "瞺" => "&#x77BA;",
    "瞻" => "&#x77BB;",
    "瞼" => "&#x77BC;",
    "瞽" => "&#x77BD;",
    "瞾" => "&#x77BE;",
    "瞿" => "&#x77BF;",
    "矀" => "&#x77C0;",
    "矁" => "&#x77C1;",
    "矂" => "&#x77C2;",
    "矃" => "&#x77C3;",
    "矄" => "&#x77C4;",
    "矅" => "&#x77C5;",
    "矆" => "&#x77C6;",
    "矇" => "&#x77C7;",
    "矈" => "&#x77C8;",
    "矉" => "&#x77C9;",
    "矊" => "&#x77CA;",
    "矋" => "&#x77CB;",
    "矌" => "&#x77CC;",
    "矍" => "&#x77CD;",
    "矎" => "&#x77CE;",
    "矏" => "&#x77CF;",
    "矐" => "&#x77D0;",
    "矑" => "&#x77D1;",
    "矒" => "&#x77D2;",
    "矓" => "&#x77D3;",
    "矔" => "&#x77D4;",
    "矕" => "&#x77D5;",
    "矖" => "&#x77D6;",
    "矗" => "&#x77D7;",
    "矘" => "&#x77D8;",
    "矙" => "&#x77D9;",
    "矚" => "&#x77DA;",
    "矛" => "&#x77DB;",
    "矜" => "&#x77DC;",
    "矝" => "&#x77DD;",
    "矞" => "&#x77DE;",
    "矟" => "&#x77DF;",
    "矠" => "&#x77E0;",
    "矡" => "&#x77E1;",
    "矢" => "&#x77E2;",
    "矣" => "&#x77E3;",
    "矤" => "&#x77E4;",
    "知" => "&#x77E5;",
    "矦" => "&#x77E6;",
    "矧" => "&#x77E7;",
    "矨" => "&#x77E8;",
    "矩" => "&#x77E9;",
    "矪" => "&#x77EA;",
    "矫" => "&#x77EB;",
    "矬" => "&#x77EC;",
    "短" => "&#x77ED;",
    "矮" => "&#x77EE;",
    "矯" => "&#x77EF;",
    "矰" => "&#x77F0;",
    "矱" => "&#x77F1;",
    "矲" => "&#x77F2;",
    "石" => "&#x77F3;",
    "矴" => "&#x77F4;",
    "矵" => "&#x77F5;",
    "矶" => "&#x77F6;",
    "矷" => "&#x77F7;",
    "矸" => "&#x77F8;",
    "矹" => "&#x77F9;",
    "矺" => "&#x77FA;",
    "矻" => "&#x77FB;",
    "矼" => "&#x77FC;",
    "矽" => "&#x77FD;",
    "矾" => "&#x77FE;",
    "矿" => "&#x77FF;",
    "砀" => "&#x7800;",
    "码" => "&#x7801;",
    "砂" => "&#x7802;",
    "砃" => "&#x7803;",
    "砄" => "&#x7804;",
    "砅" => "&#x7805;",
    "砆" => "&#x7806;",
    "砇" => "&#x7807;",
    "砈" => "&#x7808;",
    "砉" => "&#x7809;",
    "砊" => "&#x780A;",
    "砋" => "&#x780B;",
    "砌" => "&#x780C;",
    "砍" => "&#x780D;",
    "砎" => "&#x780E;",
    "砏" => "&#x780F;",
    "砐" => "&#x7810;",
    "砑" => "&#x7811;",
    "砒" => "&#x7812;",
    "砓" => "&#x7813;",
    "研" => "&#x7814;",
    "砕" => "&#x7815;",
    "砖" => "&#x7816;",
    "砗" => "&#x7817;",
    "砘" => "&#x7818;",
    "砙" => "&#x7819;",
    "砚" => "&#x781A;",
    "砛" => "&#x781B;",
    "砜" => "&#x781C;",
    "砝" => "&#x781D;",
    "砞" => "&#x781E;",
    "砟" => "&#x781F;",
    "砠" => "&#x7820;",
    "砡" => "&#x7821;",
    "砢" => "&#x7822;",
    "砣" => "&#x7823;",
    "砤" => "&#x7824;",
    "砥" => "&#x7825;",
    "砦" => "&#x7826;",
    "砧" => "&#x7827;",
    "砨" => "&#x7828;",
    "砩" => "&#x7829;",
    "砪" => "&#x782A;",
    "砫" => "&#x782B;",
    "砬" => "&#x782C;",
    "砭" => "&#x782D;",
    "砮" => "&#x782E;",
    "砯" => "&#x782F;",
    "砰" => "&#x7830;",
    "砱" => "&#x7831;",
    "砲" => "&#x7832;",
    "砳" => "&#x7833;",
    "破" => "&#x7834;",
    "砵" => "&#x7835;",
    "砶" => "&#x7836;",
    "砷" => "&#x7837;",
    "砸" => "&#x7838;",
    "砹" => "&#x7839;",
    "砺" => "&#x783A;",
    "砻" => "&#x783B;",
    "砼" => "&#x783C;",
    "砽" => "&#x783D;",
    "砾" => "&#x783E;",
    "砿" => "&#x783F;",
    "础" => "&#x7840;",
    "硁" => "&#x7841;",
    "硂" => "&#x7842;",
    "硃" => "&#x7843;",
    "硄" => "&#x7844;",
    "硅" => "&#x7845;",
    "硆" => "&#x7846;",
    "硇" => "&#x7847;",
    "硈" => "&#x7848;",
    "硉" => "&#x7849;",
    "硊" => "&#x784A;",
    "硋" => "&#x784B;",
    "硌" => "&#x784C;",
    "硍" => "&#x784D;",
    "硎" => "&#x784E;",
    "硏" => "&#x784F;",
    "硐" => "&#x7850;",
    "硑" => "&#x7851;",
    "硒" => "&#x7852;",
    "硓" => "&#x7853;",
    "硔" => "&#x7854;",
    "硕" => "&#x7855;",
    "硖" => "&#x7856;",
    "硗" => "&#x7857;",
    "硘" => "&#x7858;",
    "硙" => "&#x7859;",
    "硚" => "&#x785A;",
    "硛" => "&#x785B;",
    "硜" => "&#x785C;",
    "硝" => "&#x785D;",
    "硞" => "&#x785E;",
    "硟" => "&#x785F;",
    "硠" => "&#x7860;",
    "硡" => "&#x7861;",
    "硢" => "&#x7862;",
    "硣" => "&#x7863;",
    "硤" => "&#x7864;",
    "硥" => "&#x7865;",
    "硦" => "&#x7866;",
    "硧" => "&#x7867;",
    "硨" => "&#x7868;",
    "硩" => "&#x7869;",
    "硪" => "&#x786A;",
    "硫" => "&#x786B;",
    "硬" => "&#x786C;",
    "硭" => "&#x786D;",
    "确" => "&#x786E;",
    "硯" => "&#x786F;",
    "硰" => "&#x7870;",
    "硱" => "&#x7871;",
    "硲" => "&#x7872;",
    "硳" => "&#x7873;",
    "硴" => "&#x7874;",
    "硵" => "&#x7875;",
    "硶" => "&#x7876;",
    "硷" => "&#x7877;",
    "硸" => "&#x7878;",
    "硹" => "&#x7879;",
    "硺" => "&#x787A;",
    "硻" => "&#x787B;",
    "硼" => "&#x787C;",
    "硽" => "&#x787D;",
    "硾" => "&#x787E;",
    "硿" => "&#x787F;",
    "碀" => "&#x7880;",
    "碁" => "&#x7881;",
    "碂" => "&#x7882;",
    "碃" => "&#x7883;",
    "碄" => "&#x7884;",
    "碅" => "&#x7885;",
    "碆" => "&#x7886;",
    "碇" => "&#x7887;",
    "碈" => "&#x7888;",
    "碉" => "&#x7889;",
    "碊" => "&#x788A;",
    "碋" => "&#x788B;",
    "碌" => "&#x788C;",
    "碍" => "&#x788D;",
    "碎" => "&#x788E;",
    "碏" => "&#x788F;",
    "碐" => "&#x7890;",
    "碑" => "&#x7891;",
    "碒" => "&#x7892;",
    "碓" => "&#x7893;",
    "碔" => "&#x7894;",
    "碕" => "&#x7895;",
    "碖" => "&#x7896;",
    "碗" => "&#x7897;",
    "碘" => "&#x7898;",
    "碙" => "&#x7899;",
    "碚" => "&#x789A;",
    "碛" => "&#x789B;",
    "碜" => "&#x789C;",
    "碝" => "&#x789D;",
    "碞" => "&#x789E;",
    "碟" => "&#x789F;",
    "碠" => "&#x78A0;",
    "碡" => "&#x78A1;",
    "碢" => "&#x78A2;",
    "碣" => "&#x78A3;",
    "碤" => "&#x78A4;",
    "碥" => "&#x78A5;",
    "碦" => "&#x78A6;",
    "碧" => "&#x78A7;",
    "碨" => "&#x78A8;",
    "碩" => "&#x78A9;",
    "碪" => "&#x78AA;",
    "碫" => "&#x78AB;",
    "碬" => "&#x78AC;",
    "碭" => "&#x78AD;",
    "碮" => "&#x78AE;",
    "碯" => "&#x78AF;",
    "碰" => "&#x78B0;",
    "碱" => "&#x78B1;",
    "碲" => "&#x78B2;",
    "碳" => "&#x78B3;",
    "碴" => "&#x78B4;",
    "碵" => "&#x78B5;",
    "碶" => "&#x78B6;",
    "碷" => "&#x78B7;",
    "碸" => "&#x78B8;",
    "碹" => "&#x78B9;",
    "確" => "&#x78BA;",
    "碻" => "&#x78BB;",
    "碼" => "&#x78BC;",
    "碽" => "&#x78BD;",
    "碾" => "&#x78BE;",
    "碿" => "&#x78BF;",
    "磀" => "&#x78C0;",
    "磁" => "&#x78C1;",
    "磂" => "&#x78C2;",
    "磃" => "&#x78C3;",
    "磄" => "&#x78C4;",
    "磅" => "&#x78C5;",
    "磆" => "&#x78C6;",
    "磇" => "&#x78C7;",
    "磈" => "&#x78C8;",
    "磉" => "&#x78C9;",
    "磊" => "&#x78CA;",
    "磋" => "&#x78CB;",
    "磌" => "&#x78CC;",
    "磍" => "&#x78CD;",
    "磎" => "&#x78CE;",
    "磏" => "&#x78CF;",
    "磐" => "&#x78D0;",
    "磑" => "&#x78D1;",
    "磒" => "&#x78D2;",
    "磓" => "&#x78D3;",
    "磔" => "&#x78D4;",
    "磕" => "&#x78D5;",
    "磖" => "&#x78D6;",
    "磗" => "&#x78D7;",
    "磘" => "&#x78D8;",
    "磙" => "&#x78D9;",
    "磚" => "&#x78DA;",
    "磛" => "&#x78DB;",
    "磜" => "&#x78DC;",
    "磝" => "&#x78DD;",
    "磞" => "&#x78DE;",
    "磟" => "&#x78DF;",
    "磠" => "&#x78E0;",
    "磡" => "&#x78E1;",
    "磢" => "&#x78E2;",
    "磣" => "&#x78E3;",
    "磤" => "&#x78E4;",
    "磥" => "&#x78E5;",
    "磦" => "&#x78E6;",
    "磧" => "&#x78E7;",
    "磨" => "&#x78E8;",
    "磩" => "&#x78E9;",
    "磪" => "&#x78EA;",
    "磫" => "&#x78EB;",
    "磬" => "&#x78EC;",
    "磭" => "&#x78ED;",
    "磮" => "&#x78EE;",
    "磯" => "&#x78EF;",
    "磰" => "&#x78F0;",
    "磱" => "&#x78F1;",
    "磲" => "&#x78F2;",
    "磳" => "&#x78F3;",
    "磴" => "&#x78F4;",
    "磵" => "&#x78F5;",
    "磶" => "&#x78F6;",
    "磷" => "&#x78F7;",
    "磸" => "&#x78F8;",
    "磹" => "&#x78F9;",
    "磺" => "&#x78FA;",
    "磻" => "&#x78FB;",
    "磼" => "&#x78FC;",
    "磽" => "&#x78FD;",
    "磾" => "&#x78FE;",
    "磿" => "&#x78FF;",
    "礀" => "&#x7900;",
    "礁" => "&#x7901;",
    "礂" => "&#x7902;",
    "礃" => "&#x7903;",
    "礄" => "&#x7904;",
    "礅" => "&#x7905;",
    "礆" => "&#x7906;",
    "礇" => "&#x7907;",
    "礈" => "&#x7908;",
    "礉" => "&#x7909;",
    "礊" => "&#x790A;",
    "礋" => "&#x790B;",
    "礌" => "&#x790C;",
    "礍" => "&#x790D;",
    "礎" => "&#x790E;",
    "礏" => "&#x790F;",
    "礐" => "&#x7910;",
    "礑" => "&#x7911;",
    "礒" => "&#x7912;",
    "礓" => "&#x7913;",
    "礔" => "&#x7914;",
    "礕" => "&#x7915;",
    "礖" => "&#x7916;",
    "礗" => "&#x7917;",
    "礘" => "&#x7918;",
    "礙" => "&#x7919;",
    "礚" => "&#x791A;",
    "礛" => "&#x791B;",
    "礜" => "&#x791C;",
    "礝" => "&#x791D;",
    "礞" => "&#x791E;",
    "礟" => "&#x791F;",
    "礠" => "&#x7920;",
    "礡" => "&#x7921;",
    "礢" => "&#x7922;",
    "礣" => "&#x7923;",
    "礤" => "&#x7924;",
    "礥" => "&#x7925;",
    "礦" => "&#x7926;",
    "礧" => "&#x7927;",
    "礨" => "&#x7928;",
    "礩" => "&#x7929;",
    "礪" => "&#x792A;",
    "礫" => "&#x792B;",
    "礬" => "&#x792C;",
    "礭" => "&#x792D;",
    "礮" => "&#x792E;",
    "礯" => "&#x792F;",
    "礰" => "&#x7930;",
    "礱" => "&#x7931;",
    "礲" => "&#x7932;",
    "礳" => "&#x7933;",
    "礴" => "&#x7934;",
    "礵" => "&#x7935;",
    "礶" => "&#x7936;",
    "礷" => "&#x7937;",
    "礸" => "&#x7938;",
    "礹" => "&#x7939;",
    "示" => "&#x793A;",
    "礻" => "&#x793B;",
    "礼" => "&#x793C;",
    "礽" => "&#x793D;",
    "社" => "&#x793E;",
    "礿" => "&#x793F;",
    "祀" => "&#x7940;",
    "祁" => "&#x7941;",
    "祂" => "&#x7942;",
    "祃" => "&#x7943;",
    "祄" => "&#x7944;",
    "祅" => "&#x7945;",
    "祆" => "&#x7946;",
    "祇" => "&#x7947;",
    "祈" => "&#x7948;",
    "祉" => "&#x7949;",
    "祊" => "&#x794A;",
    "祋" => "&#x794B;",
    "祌" => "&#x794C;",
    "祍" => "&#x794D;",
    "祎" => "&#x794E;",
    "祏" => "&#x794F;",
    "祐" => "&#x7950;",
    "祑" => "&#x7951;",
    "祒" => "&#x7952;",
    "祓" => "&#x7953;",
    "祔" => "&#x7954;",
    "祕" => "&#x7955;",
    "祖" => "&#x7956;",
    "祗" => "&#x7957;",
    "祘" => "&#x7958;",
    "祙" => "&#x7959;",
    "祚" => "&#x795A;",
    "祛" => "&#x795B;",
    "祜" => "&#x795C;",
    "祝" => "&#x795D;",
    "神" => "&#x795E;",
    "祟" => "&#x795F;",
    "祠" => "&#x7960;",
    "祡" => "&#x7961;",
    "祢" => "&#x7962;",
    "祣" => "&#x7963;",
    "祤" => "&#x7964;",
    "祥" => "&#x7965;",
    "祦" => "&#x7966;",
    "祧" => "&#x7967;",
    "票" => "&#x7968;",
    "祩" => "&#x7969;",
    "祪" => "&#x796A;",
    "祫" => "&#x796B;",
    "祬" => "&#x796C;",
    "祭" => "&#x796D;",
    "祮" => "&#x796E;",
    "祯" => "&#x796F;",
    "祰" => "&#x7970;",
    "祱" => "&#x7971;",
    "祲" => "&#x7972;",
    "祳" => "&#x7973;",
    "祴" => "&#x7974;",
    "祵" => "&#x7975;",
    "祶" => "&#x7976;",
    "祷" => "&#x7977;",
    "祸" => "&#x7978;",
    "祹" => "&#x7979;",
    "祺" => "&#x797A;",
    "祻" => "&#x797B;",
    "祼" => "&#x797C;",
    "祽" => "&#x797D;",
    "祾" => "&#x797E;",
    "祿" => "&#x797F;",
    "禀" => "&#x7980;",
    "禁" => "&#x7981;",
    "禂" => "&#x7982;",
    "禃" => "&#x7983;",
    "禄" => "&#x7984;",
    "禅" => "&#x7985;",
    "禆" => "&#x7986;",
    "禇" => "&#x7987;",
    "禈" => "&#x7988;",
    "禉" => "&#x7989;",
    "禊" => "&#x798A;",
    "禋" => "&#x798B;",
    "禌" => "&#x798C;",
    "禍" => "&#x798D;",
    "禎" => "&#x798E;",
    "福" => "&#x798F;",
    "禐" => "&#x7990;",
    "禑" => "&#x7991;",
    "禒" => "&#x7992;",
    "禓" => "&#x7993;",
    "禔" => "&#x7994;",
    "禕" => "&#x7995;",
    "禖" => "&#x7996;",
    "禗" => "&#x7997;",
    "禘" => "&#x7998;",
    "禙" => "&#x7999;",
    "禚" => "&#x799A;",
    "禛" => "&#x799B;",
    "禜" => "&#x799C;",
    "禝" => "&#x799D;",
    "禞" => "&#x799E;",
    "禟" => "&#x799F;",
    "禠" => "&#x79A0;",
    "禡" => "&#x79A1;",
    "禢" => "&#x79A2;",
    "禣" => "&#x79A3;",
    "禤" => "&#x79A4;",
    "禥" => "&#x79A5;",
    "禦" => "&#x79A6;",
    "禧" => "&#x79A7;",
    "禨" => "&#x79A8;",
    "禩" => "&#x79A9;",
    "禪" => "&#x79AA;",
    "禫" => "&#x79AB;",
    "禬" => "&#x79AC;",
    "禭" => "&#x79AD;",
    "禮" => "&#x79AE;",
    "禯" => "&#x79AF;",
    "禰" => "&#x79B0;",
    "禱" => "&#x79B1;",
    "禲" => "&#x79B2;",
    "禳" => "&#x79B3;",
    "禴" => "&#x79B4;",
    "禵" => "&#x79B5;",
    "禶" => "&#x79B6;",
    "禷" => "&#x79B7;",
    "禸" => "&#x79B8;",
    "禹" => "&#x79B9;",
    "禺" => "&#x79BA;",
    "离" => "&#x79BB;",
    "禼" => "&#x79BC;",
    "禽" => "&#x79BD;",
    "禾" => "&#x79BE;",
    "禿" => "&#x79BF;",
    "秀" => "&#x79C0;",
    "私" => "&#x79C1;",
    "秂" => "&#x79C2;",
    "秃" => "&#x79C3;",
    "秄" => "&#x79C4;",
    "秅" => "&#x79C5;",
    "秆" => "&#x79C6;",
    "秇" => "&#x79C7;",
    "秈" => "&#x79C8;",
    "秉" => "&#x79C9;",
    "秊" => "&#x79CA;",
    "秋" => "&#x79CB;",
    "秌" => "&#x79CC;",
    "种" => "&#x79CD;",
    "秎" => "&#x79CE;",
    "秏" => "&#x79CF;",
    "秐" => "&#x79D0;",
    "科" => "&#x79D1;",
    "秒" => "&#x79D2;",
    "秓" => "&#x79D3;",
    "秔" => "&#x79D4;",
    "秕" => "&#x79D5;",
    "秖" => "&#x79D6;",
    "秗" => "&#x79D7;",
    "秘" => "&#x79D8;",
    "秙" => "&#x79D9;",
    "秚" => "&#x79DA;",
    "秛" => "&#x79DB;",
    "秜" => "&#x79DC;",
    "秝" => "&#x79DD;",
    "秞" => "&#x79DE;",
    "租" => "&#x79DF;",
    "秠" => "&#x79E0;",
    "秡" => "&#x79E1;",
    "秢" => "&#x79E2;",
    "秣" => "&#x79E3;",
    "秤" => "&#x79E4;",
    "秥" => "&#x79E5;",
    "秦" => "&#x79E6;",
    "秧" => "&#x79E7;",
    "秨" => "&#x79E8;",
    "秩" => "&#x79E9;",
    "秪" => "&#x79EA;",
    "秫" => "&#x79EB;",
    "秬" => "&#x79EC;",
    "秭" => "&#x79ED;",
    "秮" => "&#x79EE;",
    "积" => "&#x79EF;",
    "称" => "&#x79F0;",
    "秱" => "&#x79F1;",
    "秲" => "&#x79F2;",
    "秳" => "&#x79F3;",
    "秴" => "&#x79F4;",
    "秵" => "&#x79F5;",
    "秶" => "&#x79F6;",
    "秷" => "&#x79F7;",
    "秸" => "&#x79F8;",
    "秹" => "&#x79F9;",
    "秺" => "&#x79FA;",
    "移" => "&#x79FB;",
    "秼" => "&#x79FC;",
    "秽" => "&#x79FD;",
    "秾" => "&#x79FE;",
    "秿" => "&#x79FF;",
    "稀" => "&#x7A00;",
    "稁" => "&#x7A01;",
    "稂" => "&#x7A02;",
    "稃" => "&#x7A03;",
    "稄" => "&#x7A04;",
    "稅" => "&#x7A05;",
    "稆" => "&#x7A06;",
    "稇" => "&#x7A07;",
    "稈" => "&#x7A08;",
    "稉" => "&#x7A09;",
    "稊" => "&#x7A0A;",
    "程" => "&#x7A0B;",
    "稌" => "&#x7A0C;",
    "稍" => "&#x7A0D;",
    "税" => "&#x7A0E;",
    "稏" => "&#x7A0F;",
    "稐" => "&#x7A10;",
    "稑" => "&#x7A11;",
    "稒" => "&#x7A12;",
    "稓" => "&#x7A13;",
    "稔" => "&#x7A14;",
    "稕" => "&#x7A15;",
    "稖" => "&#x7A16;",
    "稗" => "&#x7A17;",
    "稘" => "&#x7A18;",
    "稙" => "&#x7A19;",
    "稚" => "&#x7A1A;",
    "稛" => "&#x7A1B;",
    "稜" => "&#x7A1C;",
    "稝" => "&#x7A1D;",
    "稞" => "&#x7A1E;",
    "稟" => "&#x7A1F;",
    "稠" => "&#x7A20;",
    "稡" => "&#x7A21;",
    "稢" => "&#x7A22;",
    "稣" => "&#x7A23;",
    "稤" => "&#x7A24;",
    "稥" => "&#x7A25;",
    "稦" => "&#x7A26;",
    "稧" => "&#x7A27;",
    "稨" => "&#x7A28;",
    "稩" => "&#x7A29;",
    "稪" => "&#x7A2A;",
    "稫" => "&#x7A2B;",
    "稬" => "&#x7A2C;",
    "稭" => "&#x7A2D;",
    "種" => "&#x7A2E;",
    "稯" => "&#x7A2F;",
    "稰" => "&#x7A30;",
    "稱" => "&#x7A31;",
    "稲" => "&#x7A32;",
    "稳" => "&#x7A33;",
    "稴" => "&#x7A34;",
    "稵" => "&#x7A35;",
    "稶" => "&#x7A36;",
    "稷" => "&#x7A37;",
    "稸" => "&#x7A38;",
    "稹" => "&#x7A39;",
    "稺" => "&#x7A3A;",
    "稻" => "&#x7A3B;",
    "稼" => "&#x7A3C;",
    "稽" => "&#x7A3D;",
    "稾" => "&#x7A3E;",
    "稿" => "&#x7A3F;",
    "穀" => "&#x7A40;",
    "穁" => "&#x7A41;",
    "穂" => "&#x7A42;",
    "穃" => "&#x7A43;",
    "穄" => "&#x7A44;",
    "穅" => "&#x7A45;",
    "穆" => "&#x7A46;",
    "穇" => "&#x7A47;",
    "穈" => "&#x7A48;",
    "穉" => "&#x7A49;",
    "穊" => "&#x7A4A;",
    "穋" => "&#x7A4B;",
    "穌" => "&#x7A4C;",
    "積" => "&#x7A4D;",
    "穎" => "&#x7A4E;",
    "穏" => "&#x7A4F;",
    "穐" => "&#x7A50;",
    "穑" => "&#x7A51;",
    "穒" => "&#x7A52;",
    "穓" => "&#x7A53;",
    "穔" => "&#x7A54;",
    "穕" => "&#x7A55;",
    "穖" => "&#x7A56;",
    "穗" => "&#x7A57;",
    "穘" => "&#x7A58;",
    "穙" => "&#x7A59;",
    "穚" => "&#x7A5A;",
    "穛" => "&#x7A5B;",
    "穜" => "&#x7A5C;",
    "穝" => "&#x7A5D;",
    "穞" => "&#x7A5E;",
    "穟" => "&#x7A5F;",
    "穠" => "&#x7A60;",
    "穡" => "&#x7A61;",
    "穢" => "&#x7A62;",
    "穣" => "&#x7A63;",
    "穤" => "&#x7A64;",
    "穥" => "&#x7A65;",
    "穦" => "&#x7A66;",
    "穧" => "&#x7A67;",
    "穨" => "&#x7A68;",
    "穩" => "&#x7A69;",
    "穪" => "&#x7A6A;",
    "穫" => "&#x7A6B;",
    "穬" => "&#x7A6C;",
    "穭" => "&#x7A6D;",
    "穮" => "&#x7A6E;",
    "穯" => "&#x7A6F;",
    "穰" => "&#x7A70;",
    "穱" => "&#x7A71;",
    "穲" => "&#x7A72;",
    "穳" => "&#x7A73;",
    "穴" => "&#x7A74;",
    "穵" => "&#x7A75;",
    "究" => "&#x7A76;",
    "穷" => "&#x7A77;",
    "穸" => "&#x7A78;",
    "穹" => "&#x7A79;",
    "空" => "&#x7A7A;",
    "穻" => "&#x7A7B;",
    "穼" => "&#x7A7C;",
    "穽" => "&#x7A7D;",
    "穾" => "&#x7A7E;",
    "穿" => "&#x7A7F;",
    "窀" => "&#x7A80;",
    "突" => "&#x7A81;",
    "窂" => "&#x7A82;",
    "窃" => "&#x7A83;",
    "窄" => "&#x7A84;",
    "窅" => "&#x7A85;",
    "窆" => "&#x7A86;",
    "窇" => "&#x7A87;",
    "窈" => "&#x7A88;",
    "窉" => "&#x7A89;",
    "窊" => "&#x7A8A;",
    "窋" => "&#x7A8B;",
    "窌" => "&#x7A8C;",
    "窍" => "&#x7A8D;",
    "窎" => "&#x7A8E;",
    "窏" => "&#x7A8F;",
    "窐" => "&#x7A90;",
    "窑" => "&#x7A91;",
    "窒" => "&#x7A92;",
    "窓" => "&#x7A93;",
    "窔" => "&#x7A94;",
    "窕" => "&#x7A95;",
    "窖" => "&#x7A96;",
    "窗" => "&#x7A97;",
    "窘" => "&#x7A98;",
    "窙" => "&#x7A99;",
    "窚" => "&#x7A9A;",
    "窛" => "&#x7A9B;",
    "窜" => "&#x7A9C;",
    "窝" => "&#x7A9D;",
    "窞" => "&#x7A9E;",
    "窟" => "&#x7A9F;",
    "窠" => "&#x7AA0;",
    "窡" => "&#x7AA1;",
    "窢" => "&#x7AA2;",
    "窣" => "&#x7AA3;",
    "窤" => "&#x7AA4;",
    "窥" => "&#x7AA5;",
    "窦" => "&#x7AA6;",
    "窧" => "&#x7AA7;",
    "窨" => "&#x7AA8;",
    "窩" => "&#x7AA9;",
    "窪" => "&#x7AAA;",
    "窫" => "&#x7AAB;",
    "窬" => "&#x7AAC;",
    "窭" => "&#x7AAD;",
    "窮" => "&#x7AAE;",
    "窯" => "&#x7AAF;",
    "窰" => "&#x7AB0;",
    "窱" => "&#x7AB1;",
    "窲" => "&#x7AB2;",
    "窳" => "&#x7AB3;",
    "窴" => "&#x7AB4;",
    "窵" => "&#x7AB5;",
    "窶" => "&#x7AB6;",
    "窷" => "&#x7AB7;",
    "窸" => "&#x7AB8;",
    "窹" => "&#x7AB9;",
    "窺" => "&#x7ABA;",
    "窻" => "&#x7ABB;",
    "窼" => "&#x7ABC;",
    "窽" => "&#x7ABD;",
    "窾" => "&#x7ABE;",
    "窿" => "&#x7ABF;",
    "竀" => "&#x7AC0;",
    "竁" => "&#x7AC1;",
    "竂" => "&#x7AC2;",
    "竃" => "&#x7AC3;",
    "竄" => "&#x7AC4;",
    "竅" => "&#x7AC5;",
    "竆" => "&#x7AC6;",
    "竇" => "&#x7AC7;",
    "竈" => "&#x7AC8;",
    "竉" => "&#x7AC9;",
    "竊" => "&#x7ACA;",
    "立" => "&#x7ACB;",
    "竌" => "&#x7ACC;",
    "竍" => "&#x7ACD;",
    "竎" => "&#x7ACE;",
    "竏" => "&#x7ACF;",
    "竐" => "&#x7AD0;",
    "竑" => "&#x7AD1;",
    "竒" => "&#x7AD2;",
    "竓" => "&#x7AD3;",
    "竔" => "&#x7AD4;",
    "竕" => "&#x7AD5;",
    "竖" => "&#x7AD6;",
    "竗" => "&#x7AD7;",
    "竘" => "&#x7AD8;",
    "站" => "&#x7AD9;",
    "竚" => "&#x7ADA;",
    "竛" => "&#x7ADB;",
    "竜" => "&#x7ADC;",
    "竝" => "&#x7ADD;",
    "竞" => "&#x7ADE;",
    "竟" => "&#x7ADF;",
    "章" => "&#x7AE0;",
    "竡" => "&#x7AE1;",
    "竢" => "&#x7AE2;",
    "竣" => "&#x7AE3;",
    "竤" => "&#x7AE4;",
    "童" => "&#x7AE5;",
    "竦" => "&#x7AE6;",
    "竧" => "&#x7AE7;",
    "竨" => "&#x7AE8;",
    "竩" => "&#x7AE9;",
    "竪" => "&#x7AEA;",
    "竫" => "&#x7AEB;",
    "竬" => "&#x7AEC;",
    "竭" => "&#x7AED;",
    "竮" => "&#x7AEE;",
    "端" => "&#x7AEF;",
    "竰" => "&#x7AF0;",
    "竱" => "&#x7AF1;",
    "竲" => "&#x7AF2;",
    "竳" => "&#x7AF3;",
    "竴" => "&#x7AF4;",
    "竵" => "&#x7AF5;",
    "競" => "&#x7AF6;",
    "竷" => "&#x7AF7;",
    "竸" => "&#x7AF8;",
    "竹" => "&#x7AF9;",
    "竺" => "&#x7AFA;",
    "竻" => "&#x7AFB;",
    "竼" => "&#x7AFC;",
    "竽" => "&#x7AFD;",
    "竾" => "&#x7AFE;",
    "竿" => "&#x7AFF;",
    "笀" => "&#x7B00;",
    "笁" => "&#x7B01;",
    "笂" => "&#x7B02;",
    "笃" => "&#x7B03;",
    "笄" => "&#x7B04;",
    "笅" => "&#x7B05;",
    "笆" => "&#x7B06;",
    "笇" => "&#x7B07;",
    "笈" => "&#x7B08;",
    "笉" => "&#x7B09;",
    "笊" => "&#x7B0A;",
    "笋" => "&#x7B0B;",
    "笌" => "&#x7B0C;",
    "笍" => "&#x7B0D;",
    "笎" => "&#x7B0E;",
    "笏" => "&#x7B0F;",
    "笐" => "&#x7B10;",
    "笑" => "&#x7B11;",
    "笒" => "&#x7B12;",
    "笓" => "&#x7B13;",
    "笔" => "&#x7B14;",
    "笕" => "&#x7B15;",
    "笖" => "&#x7B16;",
    "笗" => "&#x7B17;",
    "笘" => "&#x7B18;",
    "笙" => "&#x7B19;",
    "笚" => "&#x7B1A;",
    "笛" => "&#x7B1B;",
    "笜" => "&#x7B1C;",
    "笝" => "&#x7B1D;",
    "笞" => "&#x7B1E;",
    "笟" => "&#x7B1F;",
    "笠" => "&#x7B20;",
    "笡" => "&#x7B21;",
    "笢" => "&#x7B22;",
    "笣" => "&#x7B23;",
    "笤" => "&#x7B24;",
    "笥" => "&#x7B25;",
    "符" => "&#x7B26;",
    "笧" => "&#x7B27;",
    "笨" => "&#x7B28;",
    "笩" => "&#x7B29;",
    "笪" => "&#x7B2A;",
    "笫" => "&#x7B2B;",
    "第" => "&#x7B2C;",
    "笭" => "&#x7B2D;",
    "笮" => "&#x7B2E;",
    "笯" => "&#x7B2F;",
    "笰" => "&#x7B30;",
    "笱" => "&#x7B31;",
    "笲" => "&#x7B32;",
    "笳" => "&#x7B33;",
    "笴" => "&#x7B34;",
    "笵" => "&#x7B35;",
    "笶" => "&#x7B36;",
    "笷" => "&#x7B37;",
    "笸" => "&#x7B38;",
    "笹" => "&#x7B39;",
    "笺" => "&#x7B3A;",
    "笻" => "&#x7B3B;",
    "笼" => "&#x7B3C;",
    "笽" => "&#x7B3D;",
    "笾" => "&#x7B3E;",
    "笿" => "&#x7B3F;",
    "筀" => "&#x7B40;",
    "筁" => "&#x7B41;",
    "筂" => "&#x7B42;",
    "筃" => "&#x7B43;",
    "筄" => "&#x7B44;",
    "筅" => "&#x7B45;",
    "筆" => "&#x7B46;",
    "筇" => "&#x7B47;",
    "筈" => "&#x7B48;",
    "等" => "&#x7B49;",
    "筊" => "&#x7B4A;",
    "筋" => "&#x7B4B;",
    "筌" => "&#x7B4C;",
    "筍" => "&#x7B4D;",
    "筎" => "&#x7B4E;",
    "筏" => "&#x7B4F;",
    "筐" => "&#x7B50;",
    "筑" => "&#x7B51;",
    "筒" => "&#x7B52;",
    "筓" => "&#x7B53;",
    "答" => "&#x7B54;",
    "筕" => "&#x7B55;",
    "策" => "&#x7B56;",
    "筗" => "&#x7B57;",
    "筘" => "&#x7B58;",
    "筙" => "&#x7B59;",
    "筚" => "&#x7B5A;",
    "筛" => "&#x7B5B;",
    "筜" => "&#x7B5C;",
    "筝" => "&#x7B5D;",
    "筞" => "&#x7B5E;",
    "筟" => "&#x7B5F;",
    "筠" => "&#x7B60;",
    "筡" => "&#x7B61;",
    "筢" => "&#x7B62;",
    "筣" => "&#x7B63;",
    "筤" => "&#x7B64;",
    "筥" => "&#x7B65;",
    "筦" => "&#x7B66;",
    "筧" => "&#x7B67;",
    "筨" => "&#x7B68;",
    "筩" => "&#x7B69;",
    "筪" => "&#x7B6A;",
    "筫" => "&#x7B6B;",
    "筬" => "&#x7B6C;",
    "筭" => "&#x7B6D;",
    "筮" => "&#x7B6E;",
    "筯" => "&#x7B6F;",
    "筰" => "&#x7B70;",
    "筱" => "&#x7B71;",
    "筲" => "&#x7B72;",
    "筳" => "&#x7B73;",
    "筴" => "&#x7B74;",
    "筵" => "&#x7B75;",
    "筶" => "&#x7B76;",
    "筷" => "&#x7B77;",
    "筸" => "&#x7B78;",
    "筹" => "&#x7B79;",
    "筺" => "&#x7B7A;",
    "筻" => "&#x7B7B;",
    "筼" => "&#x7B7C;",
    "筽" => "&#x7B7D;",
    "签" => "&#x7B7E;",
    "筿" => "&#x7B7F;",
    "简" => "&#x7B80;",
    "箁" => "&#x7B81;",
    "箂" => "&#x7B82;",
    "箃" => "&#x7B83;",
    "箄" => "&#x7B84;",
    "箅" => "&#x7B85;",
    "箆" => "&#x7B86;",
    "箇" => "&#x7B87;",
    "箈" => "&#x7B88;",
    "箉" => "&#x7B89;",
    "箊" => "&#x7B8A;",
    "箋" => "&#x7B8B;",
    "箌" => "&#x7B8C;",
    "箍" => "&#x7B8D;",
    "箎" => "&#x7B8E;",
    "箏" => "&#x7B8F;",
    "箐" => "&#x7B90;",
    "箑" => "&#x7B91;",
    "箒" => "&#x7B92;",
    "箓" => "&#x7B93;",
    "箔" => "&#x7B94;",
    "箕" => "&#x7B95;",
    "箖" => "&#x7B96;",
    "算" => "&#x7B97;",
    "箘" => "&#x7B98;",
    "箙" => "&#x7B99;",
    "箚" => "&#x7B9A;",
    "箛" => "&#x7B9B;",
    "箜" => "&#x7B9C;",
    "箝" => "&#x7B9D;",
    "箞" => "&#x7B9E;",
    "箟" => "&#x7B9F;",
    "箠" => "&#x7BA0;",
    "管" => "&#x7BA1;",
    "箢" => "&#x7BA2;",
    "箣" => "&#x7BA3;",
    "箤" => "&#x7BA4;",
    "箥" => "&#x7BA5;",
    "箦" => "&#x7BA6;",
    "箧" => "&#x7BA7;",
    "箨" => "&#x7BA8;",
    "箩" => "&#x7BA9;",
    "箪" => "&#x7BAA;",
    "箫" => "&#x7BAB;",
    "箬" => "&#x7BAC;",
    "箭" => "&#x7BAD;",
    "箮" => "&#x7BAE;",
    "箯" => "&#x7BAF;",
    "箰" => "&#x7BB0;",
    "箱" => "&#x7BB1;",
    "箲" => "&#x7BB2;",
    "箳" => "&#x7BB3;",
    "箴" => "&#x7BB4;",
    "箵" => "&#x7BB5;",
    "箶" => "&#x7BB6;",
    "箷" => "&#x7BB7;",
    "箸" => "&#x7BB8;",
    "箹" => "&#x7BB9;",
    "箺" => "&#x7BBA;",
    "箻" => "&#x7BBB;",
    "箼" => "&#x7BBC;",
    "箽" => "&#x7BBD;",
    "箾" => "&#x7BBE;",
    "箿" => "&#x7BBF;",
    "節" => "&#x7BC0;",
    "篁" => "&#x7BC1;",
    "篂" => "&#x7BC2;",
    "篃" => "&#x7BC3;",
    "範" => "&#x7BC4;",
    "篅" => "&#x7BC5;",
    "篆" => "&#x7BC6;",
    "篇" => "&#x7BC7;",
    "篈" => "&#x7BC8;",
    "築" => "&#x7BC9;",
    "篊" => "&#x7BCA;",
    "篋" => "&#x7BCB;",
    "篌" => "&#x7BCC;",
    "篍" => "&#x7BCD;",
    "篎" => "&#x7BCE;",
    "篏" => "&#x7BCF;",
    "篐" => "&#x7BD0;",
    "篑" => "&#x7BD1;",
    "篒" => "&#x7BD2;",
    "篓" => "&#x7BD3;",
    "篔" => "&#x7BD4;",
    "篕" => "&#x7BD5;",
    "篖" => "&#x7BD6;",
    "篗" => "&#x7BD7;",
    "篘" => "&#x7BD8;",
    "篙" => "&#x7BD9;",
    "篚" => "&#x7BDA;",
    "篛" => "&#x7BDB;",
    "篜" => "&#x7BDC;",
    "篝" => "&#x7BDD;",
    "篞" => "&#x7BDE;",
    "篟" => "&#x7BDF;",
    "篠" => "&#x7BE0;",
    "篡" => "&#x7BE1;",
    "篢" => "&#x7BE2;",
    "篣" => "&#x7BE3;",
    "篤" => "&#x7BE4;",
    "篥" => "&#x7BE5;",
    "篦" => "&#x7BE6;",
    "篧" => "&#x7BE7;",
    "篨" => "&#x7BE8;",
    "篩" => "&#x7BE9;",
    "篪" => "&#x7BEA;",
    "篫" => "&#x7BEB;",
    "篬" => "&#x7BEC;",
    "篭" => "&#x7BED;",
    "篮" => "&#x7BEE;",
    "篯" => "&#x7BEF;",
    "篰" => "&#x7BF0;",
    "篱" => "&#x7BF1;",
    "篲" => "&#x7BF2;",
    "篳" => "&#x7BF3;",
    "篴" => "&#x7BF4;",
    "篵" => "&#x7BF5;",
    "篶" => "&#x7BF6;",
    "篷" => "&#x7BF7;",
    "篸" => "&#x7BF8;",
    "篹" => "&#x7BF9;",
    "篺" => "&#x7BFA;",
    "篻" => "&#x7BFB;",
    "篼" => "&#x7BFC;",
    "篽" => "&#x7BFD;",
    "篾" => "&#x7BFE;",
    "篿" => "&#x7BFF;",
    "簀" => "&#x7C00;",
    "簁" => "&#x7C01;",
    "簂" => "&#x7C02;",
    "簃" => "&#x7C03;",
    "簄" => "&#x7C04;",
    "簅" => "&#x7C05;",
    "簆" => "&#x7C06;",
    "簇" => "&#x7C07;",
    "簈" => "&#x7C08;",
    "簉" => "&#x7C09;",
    "簊" => "&#x7C0A;",
    "簋" => "&#x7C0B;",
    "簌" => "&#x7C0C;",
    "簍" => "&#x7C0D;",
    "簎" => "&#x7C0E;",
    "簏" => "&#x7C0F;",
    "簐" => "&#x7C10;",
    "簑" => "&#x7C11;",
    "簒" => "&#x7C12;",
    "簓" => "&#x7C13;",
    "簔" => "&#x7C14;",
    "簕" => "&#x7C15;",
    "簖" => "&#x7C16;",
    "簗" => "&#x7C17;",
    "簘" => "&#x7C18;",
    "簙" => "&#x7C19;",
    "簚" => "&#x7C1A;",
    "簛" => "&#x7C1B;",
    "簜" => "&#x7C1C;",
    "簝" => "&#x7C1D;",
    "簞" => "&#x7C1E;",
    "簟" => "&#x7C1F;",
    "簠" => "&#x7C20;",
    "簡" => "&#x7C21;",
    "簢" => "&#x7C22;",
    "簣" => "&#x7C23;",
    "簤" => "&#x7C24;",
    "簥" => "&#x7C25;",
    "簦" => "&#x7C26;",
    "簧" => "&#x7C27;",
    "簨" => "&#x7C28;",
    "簩" => "&#x7C29;",
    "簪" => "&#x7C2A;",
    "簫" => "&#x7C2B;",
    "簬" => "&#x7C2C;",
    "簭" => "&#x7C2D;",
    "簮" => "&#x7C2E;",
    "簯" => "&#x7C2F;",
    "簰" => "&#x7C30;",
    "簱" => "&#x7C31;",
    "簲" => "&#x7C32;",
    "簳" => "&#x7C33;",
    "簴" => "&#x7C34;",
    "簵" => "&#x7C35;",
    "簶" => "&#x7C36;",
    "簷" => "&#x7C37;",
    "簸" => "&#x7C38;",
    "簹" => "&#x7C39;",
    "簺" => "&#x7C3A;",
    "簻" => "&#x7C3B;",
    "簼" => "&#x7C3C;",
    "簽" => "&#x7C3D;",
    "簾" => "&#x7C3E;",
    "簿" => "&#x7C3F;",
    "籀" => "&#x7C40;",
    "籁" => "&#x7C41;",
    "籂" => "&#x7C42;",
    "籃" => "&#x7C43;",
    "籄" => "&#x7C44;",
    "籅" => "&#x7C45;",
    "籆" => "&#x7C46;",
    "籇" => "&#x7C47;",
    "籈" => "&#x7C48;",
    "籉" => "&#x7C49;",
    "籊" => "&#x7C4A;",
    "籋" => "&#x7C4B;",
    "籌" => "&#x7C4C;",
    "籍" => "&#x7C4D;",
    "籎" => "&#x7C4E;",
    "籏" => "&#x7C4F;",
    "籐" => "&#x7C50;",
    "籑" => "&#x7C51;",
    "籒" => "&#x7C52;",
    "籓" => "&#x7C53;",
    "籔" => "&#x7C54;",
    "籕" => "&#x7C55;",
    "籖" => "&#x7C56;",
    "籗" => "&#x7C57;",
    "籘" => "&#x7C58;",
    "籙" => "&#x7C59;",
    "籚" => "&#x7C5A;",
    "籛" => "&#x7C5B;",
    "籜" => "&#x7C5C;",
    "籝" => "&#x7C5D;",
    "籞" => "&#x7C5E;",
    "籟" => "&#x7C5F;",
    "籠" => "&#x7C60;",
    "籡" => "&#x7C61;",
    "籢" => "&#x7C62;",
    "籣" => "&#x7C63;",
    "籤" => "&#x7C64;",
    "籥" => "&#x7C65;",
    "籦" => "&#x7C66;",
    "籧" => "&#x7C67;",
    "籨" => "&#x7C68;",
    "籩" => "&#x7C69;",
    "籪" => "&#x7C6A;",
    "籫" => "&#x7C6B;",
    "籬" => "&#x7C6C;",
    "籭" => "&#x7C6D;",
    "籮" => "&#x7C6E;",
    "籯" => "&#x7C6F;",
    "籰" => "&#x7C70;",
    "籱" => "&#x7C71;",
    "籲" => "&#x7C72;",
    "米" => "&#x7C73;",
    "籴" => "&#x7C74;",
    "籵" => "&#x7C75;",
    "籶" => "&#x7C76;",
    "籷" => "&#x7C77;",
    "籸" => "&#x7C78;",
    "籹" => "&#x7C79;",
    "籺" => "&#x7C7A;",
    "类" => "&#x7C7B;",
    "籼" => "&#x7C7C;",
    "籽" => "&#x7C7D;",
    "籾" => "&#x7C7E;",
    "籿" => "&#x7C7F;",
    "粀" => "&#x7C80;",
    "粁" => "&#x7C81;",
    "粂" => "&#x7C82;",
    "粃" => "&#x7C83;",
    "粄" => "&#x7C84;",
    "粅" => "&#x7C85;",
    "粆" => "&#x7C86;",
    "粇" => "&#x7C87;",
    "粈" => "&#x7C88;",
    "粉" => "&#x7C89;",
    "粊" => "&#x7C8A;",
    "粋" => "&#x7C8B;",
    "粌" => "&#x7C8C;",
    "粍" => "&#x7C8D;",
    "粎" => "&#x7C8E;",
    "粏" => "&#x7C8F;",
    "粐" => "&#x7C90;",
    "粑" => "&#x7C91;",
    "粒" => "&#x7C92;",
    "粓" => "&#x7C93;",
    "粔" => "&#x7C94;",
    "粕" => "&#x7C95;",
    "粖" => "&#x7C96;",
    "粗" => "&#x7C97;",
    "粘" => "&#x7C98;",
    "粙" => "&#x7C99;",
    "粚" => "&#x7C9A;",
    "粛" => "&#x7C9B;",
    "粜" => "&#x7C9C;",
    "粝" => "&#x7C9D;",
    "粞" => "&#x7C9E;",
    "粟" => "&#x7C9F;",
    "粠" => "&#x7CA0;",
    "粡" => "&#x7CA1;",
    "粢" => "&#x7CA2;",
    "粣" => "&#x7CA3;",
    "粤" => "&#x7CA4;",
    "粥" => "&#x7CA5;",
    "粦" => "&#x7CA6;",
    "粧" => "&#x7CA7;",
    "粨" => "&#x7CA8;",
    "粩" => "&#x7CA9;",
    "粪" => "&#x7CAA;",
    "粫" => "&#x7CAB;",
    "粬" => "&#x7CAC;",
    "粭" => "&#x7CAD;",
    "粮" => "&#x7CAE;",
    "粯" => "&#x7CAF;",
    "粰" => "&#x7CB0;",
    "粱" => "&#x7CB1;",
    "粲" => "&#x7CB2;",
    "粳" => "&#x7CB3;",
    "粴" => "&#x7CB4;",
    "粵" => "&#x7CB5;",
    "粶" => "&#x7CB6;",
    "粷" => "&#x7CB7;",
    "粸" => "&#x7CB8;",
    "粹" => "&#x7CB9;",
    "粺" => "&#x7CBA;",
    "粻" => "&#x7CBB;",
    "粼" => "&#x7CBC;",
    "粽" => "&#x7CBD;",
    "精" => "&#x7CBE;",
    "粿" => "&#x7CBF;",
    "糀" => "&#x7CC0;",
    "糁" => "&#x7CC1;",
    "糂" => "&#x7CC2;",
    "糃" => "&#x7CC3;",
    "糄" => "&#x7CC4;",
    "糅" => "&#x7CC5;",
    "糆" => "&#x7CC6;",
    "糇" => "&#x7CC7;",
    "糈" => "&#x7CC8;",
    "糉" => "&#x7CC9;",
    "糊" => "&#x7CCA;",
    "糋" => "&#x7CCB;",
    "糌" => "&#x7CCC;",
    "糍" => "&#x7CCD;",
    "糎" => "&#x7CCE;",
    "糏" => "&#x7CCF;",
    "糐" => "&#x7CD0;",
    "糑" => "&#x7CD1;",
    "糒" => "&#x7CD2;",
    "糓" => "&#x7CD3;",
    "糔" => "&#x7CD4;",
    "糕" => "&#x7CD5;",
    "糖" => "&#x7CD6;",
    "糗" => "&#x7CD7;",
    "糘" => "&#x7CD8;",
    "糙" => "&#x7CD9;",
    "糚" => "&#x7CDA;",
    "糛" => "&#x7CDB;",
    "糜" => "&#x7CDC;",
    "糝" => "&#x7CDD;",
    "糞" => "&#x7CDE;",
    "糟" => "&#x7CDF;",
    "糠" => "&#x7CE0;",
    "糡" => "&#x7CE1;",
    "糢" => "&#x7CE2;",
    "糣" => "&#x7CE3;",
    "糤" => "&#x7CE4;",
    "糥" => "&#x7CE5;",
    "糦" => "&#x7CE6;",
    "糧" => "&#x7CE7;",
    "糨" => "&#x7CE8;",
    "糩" => "&#x7CE9;",
    "糪" => "&#x7CEA;",
    "糫" => "&#x7CEB;",
    "糬" => "&#x7CEC;",
    "糭" => "&#x7CED;",
    "糮" => "&#x7CEE;",
    "糯" => "&#x7CEF;",
    "糰" => "&#x7CF0;",
    "糱" => "&#x7CF1;",
    "糲" => "&#x7CF2;",
    "糳" => "&#x7CF3;",
    "糴" => "&#x7CF4;",
    "糵" => "&#x7CF5;",
    "糶" => "&#x7CF6;",
    "糷" => "&#x7CF7;",
    "糸" => "&#x7CF8;",
    "糹" => "&#x7CF9;",
    "糺" => "&#x7CFA;",
    "系" => "&#x7CFB;",
    "糼" => "&#x7CFC;",
    "糽" => "&#x7CFD;",
    "糾" => "&#x7CFE;",
    "糿" => "&#x7CFF;",
    "紀" => "&#x7D00;",
    "紁" => "&#x7D01;",
    "紂" => "&#x7D02;",
    "紃" => "&#x7D03;",
    "約" => "&#x7D04;",
    "紅" => "&#x7D05;",
    "紆" => "&#x7D06;",
    "紇" => "&#x7D07;",
    "紈" => "&#x7D08;",
    "紉" => "&#x7D09;",
    "紊" => "&#x7D0A;",
    "紋" => "&#x7D0B;",
    "紌" => "&#x7D0C;",
    "納" => "&#x7D0D;",
    "紎" => "&#x7D0E;",
    "紏" => "&#x7D0F;",
    "紐" => "&#x7D10;",
    "紑" => "&#x7D11;",
    "紒" => "&#x7D12;",
    "紓" => "&#x7D13;",
    "純" => "&#x7D14;",
    "紕" => "&#x7D15;",
    "紖" => "&#x7D16;",
    "紗" => "&#x7D17;",
    "紘" => "&#x7D18;",
    "紙" => "&#x7D19;",
    "級" => "&#x7D1A;",
    "紛" => "&#x7D1B;",
    "紜" => "&#x7D1C;",
    "紝" => "&#x7D1D;",
    "紞" => "&#x7D1E;",
    "紟" => "&#x7D1F;",
    "素" => "&#x7D20;",
    "紡" => "&#x7D21;",
    "索" => "&#x7D22;",
    "紣" => "&#x7D23;",
    "紤" => "&#x7D24;",
    "紥" => "&#x7D25;",
    "紦" => "&#x7D26;",
    "紧" => "&#x7D27;",
    "紨" => "&#x7D28;",
    "紩" => "&#x7D29;",
    "紪" => "&#x7D2A;",
    "紫" => "&#x7D2B;",
    "紬" => "&#x7D2C;",
    "紭" => "&#x7D2D;",
    "紮" => "&#x7D2E;",
    "累" => "&#x7D2F;",
    "細" => "&#x7D30;",
    "紱" => "&#x7D31;",
    "紲" => "&#x7D32;",
    "紳" => "&#x7D33;",
    "紴" => "&#x7D34;",
    "紵" => "&#x7D35;",
    "紶" => "&#x7D36;",
    "紷" => "&#x7D37;",
    "紸" => "&#x7D38;",
    "紹" => "&#x7D39;",
    "紺" => "&#x7D3A;",
    "紻" => "&#x7D3B;",
    "紼" => "&#x7D3C;",
    "紽" => "&#x7D3D;",
    "紾" => "&#x7D3E;",
    "紿" => "&#x7D3F;",
    "絀" => "&#x7D40;",
    "絁" => "&#x7D41;",
    "終" => "&#x7D42;",
    "絃" => "&#x7D43;",
    "組" => "&#x7D44;",
    "絅" => "&#x7D45;",
    "絆" => "&#x7D46;",
    "絇" => "&#x7D47;",
    "絈" => "&#x7D48;",
    "絉" => "&#x7D49;",
    "絊" => "&#x7D4A;",
    "絋" => "&#x7D4B;",
    "経" => "&#x7D4C;",
    "絍" => "&#x7D4D;",
    "絎" => "&#x7D4E;",
    "絏" => "&#x7D4F;",
    "結" => "&#x7D50;",
    "絑" => "&#x7D51;",
    "絒" => "&#x7D52;",
    "絓" => "&#x7D53;",
    "絔" => "&#x7D54;",
    "絕" => "&#x7D55;",
    "絖" => "&#x7D56;",
    "絗" => "&#x7D57;",
    "絘" => "&#x7D58;",
    "絙" => "&#x7D59;",
    "絚" => "&#x7D5A;",
    "絛" => "&#x7D5B;",
    "絜" => "&#x7D5C;",
    "絝" => "&#x7D5D;",
    "絞" => "&#x7D5E;",
    "絟" => "&#x7D5F;",
    "絠" => "&#x7D60;",
    "絡" => "&#x7D61;",
    "絢" => "&#x7D62;",
    "絣" => "&#x7D63;",
    "絤" => "&#x7D64;",
    "絥" => "&#x7D65;",
    "給" => "&#x7D66;",
    "絧" => "&#x7D67;",
    "絨" => "&#x7D68;",
    "絩" => "&#x7D69;",
    "絪" => "&#x7D6A;",
    "絫" => "&#x7D6B;",
    "絬" => "&#x7D6C;",
    "絭" => "&#x7D6D;",
    "絮" => "&#x7D6E;",
    "絯" => "&#x7D6F;",
    "絰" => "&#x7D70;",
    "統" => "&#x7D71;",
    "絲" => "&#x7D72;",
    "絳" => "&#x7D73;",
    "絴" => "&#x7D74;",
    "絵" => "&#x7D75;",
    "絶" => "&#x7D76;",
    "絷" => "&#x7D77;",
    "絸" => "&#x7D78;",
    "絹" => "&#x7D79;",
    "絺" => "&#x7D7A;",
    "絻" => "&#x7D7B;",
    "絼" => "&#x7D7C;",
    "絽" => "&#x7D7D;",
    "絾" => "&#x7D7E;",
    "絿" => "&#x7D7F;",
    "綀" => "&#x7D80;",
    "綁" => "&#x7D81;",
    "綂" => "&#x7D82;",
    "綃" => "&#x7D83;",
    "綄" => "&#x7D84;",
    "綅" => "&#x7D85;",
    "綆" => "&#x7D86;",
    "綇" => "&#x7D87;",
    "綈" => "&#x7D88;",
    "綉" => "&#x7D89;",
    "綊" => "&#x7D8A;",
    "綋" => "&#x7D8B;",
    "綌" => "&#x7D8C;",
    "綍" => "&#x7D8D;",
    "綎" => "&#x7D8E;",
    "綏" => "&#x7D8F;",
    "綐" => "&#x7D90;",
    "綑" => "&#x7D91;",
    "綒" => "&#x7D92;",
    "經" => "&#x7D93;",
    "綔" => "&#x7D94;",
    "綕" => "&#x7D95;",
    "綖" => "&#x7D96;",
    "綗" => "&#x7D97;",
    "綘" => "&#x7D98;",
    "継" => "&#x7D99;",
    "続" => "&#x7D9A;",
    "綛" => "&#x7D9B;",
    "綜" => "&#x7D9C;",
    "綝" => "&#x7D9D;",
    "綞" => "&#x7D9E;",
    "綟" => "&#x7D9F;",
    "綠" => "&#x7DA0;",
    "綡" => "&#x7DA1;",
    "綢" => "&#x7DA2;",
    "綣" => "&#x7DA3;",
    "綤" => "&#x7DA4;",
    "綥" => "&#x7DA5;",
    "綦" => "&#x7DA6;",
    "綧" => "&#x7DA7;",
    "綨" => "&#x7DA8;",
    "綩" => "&#x7DA9;",
    "綪" => "&#x7DAA;",
    "綫" => "&#x7DAB;",
    "綬" => "&#x7DAC;",
    "維" => "&#x7DAD;",
    "綮" => "&#x7DAE;",
    "綯" => "&#x7DAF;",
    "綰" => "&#x7DB0;",
    "綱" => "&#x7DB1;",
    "網" => "&#x7DB2;",
    "綳" => "&#x7DB3;",
    "綴" => "&#x7DB4;",
    "綵" => "&#x7DB5;",
    "綶" => "&#x7DB6;",
    "綷" => "&#x7DB7;",
    "綸" => "&#x7DB8;",
    "綹" => "&#x7DB9;",
    "綺" => "&#x7DBA;",
    "綻" => "&#x7DBB;",
    "綼" => "&#x7DBC;",
    "綽" => "&#x7DBD;",
    "綾" => "&#x7DBE;",
    "綿" => "&#x7DBF;",
    "緀" => "&#x7DC0;",
    "緁" => "&#x7DC1;",
    "緂" => "&#x7DC2;",
    "緃" => "&#x7DC3;",
    "緄" => "&#x7DC4;",
    "緅" => "&#x7DC5;",
    "緆" => "&#x7DC6;",
    "緇" => "&#x7DC7;",
    "緈" => "&#x7DC8;",
    "緉" => "&#x7DC9;",
    "緊" => "&#x7DCA;",
    "緋" => "&#x7DCB;",
    "緌" => "&#x7DCC;",
    "緍" => "&#x7DCD;",
    "緎" => "&#x7DCE;",
    "総" => "&#x7DCF;",
    "緐" => "&#x7DD0;",
    "緑" => "&#x7DD1;",
    "緒" => "&#x7DD2;",
    "緓" => "&#x7DD3;",
    "緔" => "&#x7DD4;",
    "緕" => "&#x7DD5;",
    "緖" => "&#x7DD6;",
    "緗" => "&#x7DD7;",
    "緘" => "&#x7DD8;",
    "緙" => "&#x7DD9;",
    "線" => "&#x7DDA;",
    "緛" => "&#x7DDB;",
    "緜" => "&#x7DDC;",
    "緝" => "&#x7DDD;",
    "緞" => "&#x7DDE;",
    "緟" => "&#x7DDF;",
    "締" => "&#x7DE0;",
    "緡" => "&#x7DE1;",
    "緢" => "&#x7DE2;",
    "緣" => "&#x7DE3;",
    "緤" => "&#x7DE4;",
    "緥" => "&#x7DE5;",
    "緦" => "&#x7DE6;",
    "緧" => "&#x7DE7;",
    "編" => "&#x7DE8;",
    "緩" => "&#x7DE9;",
    "緪" => "&#x7DEA;",
    "緫" => "&#x7DEB;",
    "緬" => "&#x7DEC;",
    "緭" => "&#x7DED;",
    "緮" => "&#x7DEE;",
    "緯" => "&#x7DEF;",
    "緰" => "&#x7DF0;",
    "緱" => "&#x7DF1;",
    "緲" => "&#x7DF2;",
    "緳" => "&#x7DF3;",
    "練" => "&#x7DF4;",
    "緵" => "&#x7DF5;",
    "緶" => "&#x7DF6;",
    "緷" => "&#x7DF7;",
    "緸" => "&#x7DF8;",
    "緹" => "&#x7DF9;",
    "緺" => "&#x7DFA;",
    "緻" => "&#x7DFB;",
    "緼" => "&#x7DFC;",
    "緽" => "&#x7DFD;",
    "緾" => "&#x7DFE;",
    "緿" => "&#x7DFF;",
    "縀" => "&#x7E00;",
    "縁" => "&#x7E01;",
    "縂" => "&#x7E02;",
    "縃" => "&#x7E03;",
    "縄" => "&#x7E04;",
    "縅" => "&#x7E05;",
    "縆" => "&#x7E06;",
    "縇" => "&#x7E07;",
    "縈" => "&#x7E08;",
    "縉" => "&#x7E09;",
    "縊" => "&#x7E0A;",
    "縋" => "&#x7E0B;",
    "縌" => "&#x7E0C;",
    "縍" => "&#x7E0D;",
    "縎" => "&#x7E0E;",
    "縏" => "&#x7E0F;",
    "縐" => "&#x7E10;",
    "縑" => "&#x7E11;",
    "縒" => "&#x7E12;",
    "縓" => "&#x7E13;",
    "縔" => "&#x7E14;",
    "縕" => "&#x7E15;",
    "縖" => "&#x7E16;",
    "縗" => "&#x7E17;",
    "縘" => "&#x7E18;",
    "縙" => "&#x7E19;",
    "縚" => "&#x7E1A;",
    "縛" => "&#x7E1B;",
    "縜" => "&#x7E1C;",
    "縝" => "&#x7E1D;",
    "縞" => "&#x7E1E;",
    "縟" => "&#x7E1F;",
    "縠" => "&#x7E20;",
    "縡" => "&#x7E21;",
    "縢" => "&#x7E22;",
    "縣" => "&#x7E23;",
    "縤" => "&#x7E24;",
    "縥" => "&#x7E25;",
    "縦" => "&#x7E26;",
    "縧" => "&#x7E27;",
    "縨" => "&#x7E28;",
    "縩" => "&#x7E29;",
    "縪" => "&#x7E2A;",
    "縫" => "&#x7E2B;",
    "縬" => "&#x7E2C;",
    "縭" => "&#x7E2D;",
    "縮" => "&#x7E2E;",
    "縯" => "&#x7E2F;",
    "縰" => "&#x7E30;",
    "縱" => "&#x7E31;",
    "縲" => "&#x7E32;",
    "縳" => "&#x7E33;",
    "縴" => "&#x7E34;",
    "縵" => "&#x7E35;",
    "縶" => "&#x7E36;",
    "縷" => "&#x7E37;",
    "縸" => "&#x7E38;",
    "縹" => "&#x7E39;",
    "縺" => "&#x7E3A;",
    "縻" => "&#x7E3B;",
    "縼" => "&#x7E3C;",
    "總" => "&#x7E3D;",
    "績" => "&#x7E3E;",
    "縿" => "&#x7E3F;",
    "繀" => "&#x7E40;",
    "繁" => "&#x7E41;",
    "繂" => "&#x7E42;",
    "繃" => "&#x7E43;",
    "繄" => "&#x7E44;",
    "繅" => "&#x7E45;",
    "繆" => "&#x7E46;",
    "繇" => "&#x7E47;",
    "繈" => "&#x7E48;",
    "繉" => "&#x7E49;",
    "繊" => "&#x7E4A;",
    "繋" => "&#x7E4B;",
    "繌" => "&#x7E4C;",
    "繍" => "&#x7E4D;",
    "繎" => "&#x7E4E;",
    "繏" => "&#x7E4F;",
    "繐" => "&#x7E50;",
    "繑" => "&#x7E51;",
    "繒" => "&#x7E52;",
    "繓" => "&#x7E53;",
    "織" => "&#x7E54;",
    "繕" => "&#x7E55;",
    "繖" => "&#x7E56;",
    "繗" => "&#x7E57;",
    "繘" => "&#x7E58;",
    "繙" => "&#x7E59;",
    "繚" => "&#x7E5A;",
    "繛" => "&#x7E5B;",
    "繜" => "&#x7E5C;",
    "繝" => "&#x7E5D;",
    "繞" => "&#x7E5E;",
    "繟" => "&#x7E5F;",
    "繠" => "&#x7E60;",
    "繡" => "&#x7E61;",
    "繢" => "&#x7E62;",
    "繣" => "&#x7E63;",
    "繤" => "&#x7E64;",
    "繥" => "&#x7E65;",
    "繦" => "&#x7E66;",
    "繧" => "&#x7E67;",
    "繨" => "&#x7E68;",
    "繩" => "&#x7E69;",
    "繪" => "&#x7E6A;",
    "繫" => "&#x7E6B;",
    "繬" => "&#x7E6C;",
    "繭" => "&#x7E6D;",
    "繮" => "&#x7E6E;",
    "繯" => "&#x7E6F;",
    "繰" => "&#x7E70;",
    "繱" => "&#x7E71;",
    "繲" => "&#x7E72;",
    "繳" => "&#x7E73;",
    "繴" => "&#x7E74;",
    "繵" => "&#x7E75;",
    "繶" => "&#x7E76;",
    "繷" => "&#x7E77;",
    "繸" => "&#x7E78;",
    "繹" => "&#x7E79;",
    "繺" => "&#x7E7A;",
    "繻" => "&#x7E7B;",
    "繼" => "&#x7E7C;",
    "繽" => "&#x7E7D;",
    "繾" => "&#x7E7E;",
    "繿" => "&#x7E7F;",
    "纀" => "&#x7E80;",
    "纁" => "&#x7E81;",
    "纂" => "&#x7E82;",
    "纃" => "&#x7E83;",
    "纄" => "&#x7E84;",
    "纅" => "&#x7E85;",
    "纆" => "&#x7E86;",
    "纇" => "&#x7E87;",
    "纈" => "&#x7E88;",
    "纉" => "&#x7E89;",
    "纊" => "&#x7E8A;",
    "纋" => "&#x7E8B;",
    "續" => "&#x7E8C;",
    "纍" => "&#x7E8D;",
    "纎" => "&#x7E8E;",
    "纏" => "&#x7E8F;",
    "纐" => "&#x7E90;",
    "纑" => "&#x7E91;",
    "纒" => "&#x7E92;",
    "纓" => "&#x7E93;",
    "纔" => "&#x7E94;",
    "纕" => "&#x7E95;",
    "纖" => "&#x7E96;",
    "纗" => "&#x7E97;",
    "纘" => "&#x7E98;",
    "纙" => "&#x7E99;",
    "纚" => "&#x7E9A;",
    "纛" => "&#x7E9B;",
    "纜" => "&#x7E9C;",
    "纝" => "&#x7E9D;",
    "纞" => "&#x7E9E;",
    "纟" => "&#x7E9F;",
    "纠" => "&#x7EA0;",
    "纡" => "&#x7EA1;",
    "红" => "&#x7EA2;",
    "纣" => "&#x7EA3;",
    "纤" => "&#x7EA4;",
    "纥" => "&#x7EA5;",
    "约" => "&#x7EA6;",
    "级" => "&#x7EA7;",
    "纨" => "&#x7EA8;",
    "纩" => "&#x7EA9;",
    "纪" => "&#x7EAA;",
    "纫" => "&#x7EAB;",
    "纬" => "&#x7EAC;",
    "纭" => "&#x7EAD;",
    "纮" => "&#x7EAE;",
    "纯" => "&#x7EAF;",
    "纰" => "&#x7EB0;",
    "纱" => "&#x7EB1;",
    "纲" => "&#x7EB2;",
    "纳" => "&#x7EB3;",
    "纴" => "&#x7EB4;",
    "纵" => "&#x7EB5;",
    "纶" => "&#x7EB6;",
    "纷" => "&#x7EB7;",
    "纸" => "&#x7EB8;",
    "纹" => "&#x7EB9;",
    "纺" => "&#x7EBA;",
    "纻" => "&#x7EBB;",
    "纼" => "&#x7EBC;",
    "纽" => "&#x7EBD;",
    "纾" => "&#x7EBE;",
    "线" => "&#x7EBF;",
    "绀" => "&#x7EC0;",
    "绁" => "&#x7EC1;",
    "绂" => "&#x7EC2;",
    "练" => "&#x7EC3;",
    "组" => "&#x7EC4;",
    "绅" => "&#x7EC5;",
    "细" => "&#x7EC6;",
    "织" => "&#x7EC7;",
    "终" => "&#x7EC8;",
    "绉" => "&#x7EC9;",
    "绊" => "&#x7ECA;",
    "绋" => "&#x7ECB;",
    "绌" => "&#x7ECC;",
    "绍" => "&#x7ECD;",
    "绎" => "&#x7ECE;",
    "经" => "&#x7ECF;",
    "绐" => "&#x7ED0;",
    "绑" => "&#x7ED1;",
    "绒" => "&#x7ED2;",
    "结" => "&#x7ED3;",
    "绔" => "&#x7ED4;",
    "绕" => "&#x7ED5;",
    "绖" => "&#x7ED6;",
    "绗" => "&#x7ED7;",
    "绘" => "&#x7ED8;",
    "给" => "&#x7ED9;",
    "绚" => "&#x7EDA;",
    "绛" => "&#x7EDB;",
    "络" => "&#x7EDC;",
    "绝" => "&#x7EDD;",
    "绞" => "&#x7EDE;",
    "统" => "&#x7EDF;",
    "绠" => "&#x7EE0;",
    "绡" => "&#x7EE1;",
    "绢" => "&#x7EE2;",
    "绣" => "&#x7EE3;",
    "绤" => "&#x7EE4;",
    "绥" => "&#x7EE5;",
    "绦" => "&#x7EE6;",
    "继" => "&#x7EE7;",
    "绨" => "&#x7EE8;",
    "绩" => "&#x7EE9;",
    "绪" => "&#x7EEA;",
    "绫" => "&#x7EEB;",
    "绬" => "&#x7EEC;",
    "续" => "&#x7EED;",
    "绮" => "&#x7EEE;",
    "绯" => "&#x7EEF;",
    "绰" => "&#x7EF0;",
    "绱" => "&#x7EF1;",
    "绲" => "&#x7EF2;",
    "绳" => "&#x7EF3;",
    "维" => "&#x7EF4;",
    "绵" => "&#x7EF5;",
    "绶" => "&#x7EF6;",
    "绷" => "&#x7EF7;",
    "绸" => "&#x7EF8;",
    "绹" => "&#x7EF9;",
    "绺" => "&#x7EFA;",
    "绻" => "&#x7EFB;",
    "综" => "&#x7EFC;",
    "绽" => "&#x7EFD;",
    "绾" => "&#x7EFE;",
    "绿" => "&#x7EFF;",
    "缀" => "&#x7F00;",
    "缁" => "&#x7F01;",
    "缂" => "&#x7F02;",
    "缃" => "&#x7F03;",
    "缄" => "&#x7F04;",
    "缅" => "&#x7F05;",
    "缆" => "&#x7F06;",
    "缇" => "&#x7F07;",
    "缈" => "&#x7F08;",
    "缉" => "&#x7F09;",
    "缊" => "&#x7F0A;",
    "缋" => "&#x7F0B;",
    "缌" => "&#x7F0C;",
    "缍" => "&#x7F0D;",
    "缎" => "&#x7F0E;",
    "缏" => "&#x7F0F;",
    "缐" => "&#x7F10;",
    "缑" => "&#x7F11;",
    "缒" => "&#x7F12;",
    "缓" => "&#x7F13;",
    "缔" => "&#x7F14;",
    "缕" => "&#x7F15;",
    "编" => "&#x7F16;",
    "缗" => "&#x7F17;",
    "缘" => "&#x7F18;",
    "缙" => "&#x7F19;",
    "缚" => "&#x7F1A;",
    "缛" => "&#x7F1B;",
    "缜" => "&#x7F1C;",
    "缝" => "&#x7F1D;",
    "缞" => "&#x7F1E;",
    "缟" => "&#x7F1F;",
    "缠" => "&#x7F20;",
    "缡" => "&#x7F21;",
    "缢" => "&#x7F22;",
    "缣" => "&#x7F23;",
    "缤" => "&#x7F24;",
    "缥" => "&#x7F25;",
    "缦" => "&#x7F26;",
    "缧" => "&#x7F27;",
    "缨" => "&#x7F28;",
    "缩" => "&#x7F29;",
    "缪" => "&#x7F2A;",
    "缫" => "&#x7F2B;",
    "缬" => "&#x7F2C;",
    "缭" => "&#x7F2D;",
    "缮" => "&#x7F2E;",
    "缯" => "&#x7F2F;",
    "缰" => "&#x7F30;",
    "缱" => "&#x7F31;",
    "缲" => "&#x7F32;",
    "缳" => "&#x7F33;",
    "缴" => "&#x7F34;",
    "缵" => "&#x7F35;",
    "缶" => "&#x7F36;",
    "缷" => "&#x7F37;",
    "缸" => "&#x7F38;",
    "缹" => "&#x7F39;",
    "缺" => "&#x7F3A;",
    "缻" => "&#x7F3B;",
    "缼" => "&#x7F3C;",
    "缽" => "&#x7F3D;",
    "缾" => "&#x7F3E;",
    "缿" => "&#x7F3F;",
    "罀" => "&#x7F40;",
    "罁" => "&#x7F41;",
    "罂" => "&#x7F42;",
    "罃" => "&#x7F43;",
    "罄" => "&#x7F44;",
    "罅" => "&#x7F45;",
    "罆" => "&#x7F46;",
    "罇" => "&#x7F47;",
    "罈" => "&#x7F48;",
    "罉" => "&#x7F49;",
    "罊" => "&#x7F4A;",
    "罋" => "&#x7F4B;",
    "罌" => "&#x7F4C;",
    "罍" => "&#x7F4D;",
    "罎" => "&#x7F4E;",
    "罏" => "&#x7F4F;",
    "罐" => "&#x7F50;",
    "网" => "&#x7F51;",
    "罒" => "&#x7F52;",
    "罓" => "&#x7F53;",
    "罔" => "&#x7F54;",
    "罕" => "&#x7F55;",
    "罖" => "&#x7F56;",
    "罗" => "&#x7F57;",
    "罘" => "&#x7F58;",
    "罙" => "&#x7F59;",
    "罚" => "&#x7F5A;",
    "罛" => "&#x7F5B;",
    "罜" => "&#x7F5C;",
    "罝" => "&#x7F5D;",
    "罞" => "&#x7F5E;",
    "罟" => "&#x7F5F;",
    "罠" => "&#x7F60;",
    "罡" => "&#x7F61;",
    "罢" => "&#x7F62;",
    "罣" => "&#x7F63;",
    "罤" => "&#x7F64;",
    "罥" => "&#x7F65;",
    "罦" => "&#x7F66;",
    "罧" => "&#x7F67;",
    "罨" => "&#x7F68;",
    "罩" => "&#x7F69;",
    "罪" => "&#x7F6A;",
    "罫" => "&#x7F6B;",
    "罬" => "&#x7F6C;",
    "罭" => "&#x7F6D;",
    "置" => "&#x7F6E;",
    "罯" => "&#x7F6F;",
    "罰" => "&#x7F70;",
    "罱" => "&#x7F71;",
    "署" => "&#x7F72;",
    "罳" => "&#x7F73;",
    "罴" => "&#x7F74;",
    "罵" => "&#x7F75;",
    "罶" => "&#x7F76;",
    "罷" => "&#x7F77;",
    "罸" => "&#x7F78;",
    "罹" => "&#x7F79;",
    "罺" => "&#x7F7A;",
    "罻" => "&#x7F7B;",
    "罼" => "&#x7F7C;",
    "罽" => "&#x7F7D;",
    "罾" => "&#x7F7E;",
    "罿" => "&#x7F7F;",
    "羀" => "&#x7F80;",
    "羁" => "&#x7F81;",
    "羂" => "&#x7F82;",
    "羃" => "&#x7F83;",
    "羄" => "&#x7F84;",
    "羅" => "&#x7F85;",
    "羆" => "&#x7F86;",
    "羇" => "&#x7F87;",
    "羈" => "&#x7F88;",
    "羉" => "&#x7F89;",
    "羊" => "&#x7F8A;",
    "羋" => "&#x7F8B;",
    "羌" => "&#x7F8C;",
    "羍" => "&#x7F8D;",
    "美" => "&#x7F8E;",
    "羏" => "&#x7F8F;",
    "羐" => "&#x7F90;",
    "羑" => "&#x7F91;",
    "羒" => "&#x7F92;",
    "羓" => "&#x7F93;",
    "羔" => "&#x7F94;",
    "羕" => "&#x7F95;",
    "羖" => "&#x7F96;",
    "羗" => "&#x7F97;",
    "羘" => "&#x7F98;",
    "羙" => "&#x7F99;",
    "羚" => "&#x7F9A;",
    "羛" => "&#x7F9B;",
    "羜" => "&#x7F9C;",
    "羝" => "&#x7F9D;",
    "羞" => "&#x7F9E;",
    "羟" => "&#x7F9F;",
    "羠" => "&#x7FA0;",
    "羡" => "&#x7FA1;",
    "羢" => "&#x7FA2;",
    "羣" => "&#x7FA3;",
    "群" => "&#x7FA4;",
    "羥" => "&#x7FA5;",
    "羦" => "&#x7FA6;",
    "羧" => "&#x7FA7;",
    "羨" => "&#x7FA8;",
    "義" => "&#x7FA9;",
    "羪" => "&#x7FAA;",
    "羫" => "&#x7FAB;",
    "羬" => "&#x7FAC;",
    "羭" => "&#x7FAD;",
    "羮" => "&#x7FAE;",
    "羯" => "&#x7FAF;",
    "羰" => "&#x7FB0;",
    "羱" => "&#x7FB1;",
    "羲" => "&#x7FB2;",
    "羳" => "&#x7FB3;",
    "羴" => "&#x7FB4;",
    "羵" => "&#x7FB5;",
    "羶" => "&#x7FB6;",
    "羷" => "&#x7FB7;",
    "羸" => "&#x7FB8;",
    "羹" => "&#x7FB9;",
    "羺" => "&#x7FBA;",
    "羻" => "&#x7FBB;",
    "羼" => "&#x7FBC;",
    "羽" => "&#x7FBD;",
    "羾" => "&#x7FBE;",
    "羿" => "&#x7FBF;",
    "翀" => "&#x7FC0;",
    "翁" => "&#x7FC1;",
    "翂" => "&#x7FC2;",
    "翃" => "&#x7FC3;",
    "翄" => "&#x7FC4;",
    "翅" => "&#x7FC5;",
    "翆" => "&#x7FC6;",
    "翇" => "&#x7FC7;",
    "翈" => "&#x7FC8;",
    "翉" => "&#x7FC9;",
    "翊" => "&#x7FCA;",
    "翋" => "&#x7FCB;",
    "翌" => "&#x7FCC;",
    "翍" => "&#x7FCD;",
    "翎" => "&#x7FCE;",
    "翏" => "&#x7FCF;",
    "翐" => "&#x7FD0;",
    "翑" => "&#x7FD1;",
    "習" => "&#x7FD2;",
    "翓" => "&#x7FD3;",
    "翔" => "&#x7FD4;",
    "翕" => "&#x7FD5;",
    "翖" => "&#x7FD6;",
    "翗" => "&#x7FD7;",
    "翘" => "&#x7FD8;",
    "翙" => "&#x7FD9;",
    "翚" => "&#x7FDA;",
    "翛" => "&#x7FDB;",
    "翜" => "&#x7FDC;",
    "翝" => "&#x7FDD;",
    "翞" => "&#x7FDE;",
    "翟" => "&#x7FDF;",
    "翠" => "&#x7FE0;",
    "翡" => "&#x7FE1;",
    "翢" => "&#x7FE2;",
    "翣" => "&#x7FE3;",
    "翤" => "&#x7FE4;",
    "翥" => "&#x7FE5;",
    "翦" => "&#x7FE6;",
    "翧" => "&#x7FE7;",
    "翨" => "&#x7FE8;",
    "翩" => "&#x7FE9;",
    "翪" => "&#x7FEA;",
    "翫" => "&#x7FEB;",
    "翬" => "&#x7FEC;",
    "翭" => "&#x7FED;",
    "翮" => "&#x7FEE;",
    "翯" => "&#x7FEF;",
    "翰" => "&#x7FF0;",
    "翱" => "&#x7FF1;",
    "翲" => "&#x7FF2;",
    "翳" => "&#x7FF3;",
    "翴" => "&#x7FF4;",
    "翵" => "&#x7FF5;",
    "翶" => "&#x7FF6;",
    "翷" => "&#x7FF7;",
    "翸" => "&#x7FF8;",
    "翹" => "&#x7FF9;",
    "翺" => "&#x7FFA;",
    "翻" => "&#x7FFB;",
    "翼" => "&#x7FFC;",
    "翽" => "&#x7FFD;",
    "翾" => "&#x7FFE;",
    "翿" => "&#x7FFF;",
    "耀" => "&#x8000;",
    "老" => "&#x8001;",
    "耂" => "&#x8002;",
    "考" => "&#x8003;",
    "耄" => "&#x8004;",
    "者" => "&#x8005;",
    "耆" => "&#x8006;",
    "耇" => "&#x8007;",
    "耈" => "&#x8008;",
    "耉" => "&#x8009;",
    "耊" => "&#x800A;",
    "耋" => "&#x800B;",
    "而" => "&#x800C;",
    "耍" => "&#x800D;",
    "耎" => "&#x800E;",
    "耏" => "&#x800F;",
    "耐" => "&#x8010;",
    "耑" => "&#x8011;",
    "耒" => "&#x8012;",
    "耓" => "&#x8013;",
    "耔" => "&#x8014;",
    "耕" => "&#x8015;",
    "耖" => "&#x8016;",
    "耗" => "&#x8017;",
    "耘" => "&#x8018;",
    "耙" => "&#x8019;",
    "耚" => "&#x801A;",
    "耛" => "&#x801B;",
    "耜" => "&#x801C;",
    "耝" => "&#x801D;",
    "耞" => "&#x801E;",
    "耟" => "&#x801F;",
    "耠" => "&#x8020;",
    "耡" => "&#x8021;",
    "耢" => "&#x8022;",
    "耣" => "&#x8023;",
    "耤" => "&#x8024;",
    "耥" => "&#x8025;",
    "耦" => "&#x8026;",
    "耧" => "&#x8027;",
    "耨" => "&#x8028;",
    "耩" => "&#x8029;",
    "耪" => "&#x802A;",
    "耫" => "&#x802B;",
    "耬" => "&#x802C;",
    "耭" => "&#x802D;",
    "耮" => "&#x802E;",
    "耯" => "&#x802F;",
    "耰" => "&#x8030;",
    "耱" => "&#x8031;",
    "耲" => "&#x8032;",
    "耳" => "&#x8033;",
    "耴" => "&#x8034;",
    "耵" => "&#x8035;",
    "耶" => "&#x8036;",
    "耷" => "&#x8037;",
    "耸" => "&#x8038;",
    "耹" => "&#x8039;",
    "耺" => "&#x803A;",
    "耻" => "&#x803B;",
    "耼" => "&#x803C;",
    "耽" => "&#x803D;",
    "耾" => "&#x803E;",
    "耿" => "&#x803F;",
    "聀" => "&#x8040;",
    "聁" => "&#x8041;",
    "聂" => "&#x8042;",
    "聃" => "&#x8043;",
    "聄" => "&#x8044;",
    "聅" => "&#x8045;",
    "聆" => "&#x8046;",
    "聇" => "&#x8047;",
    "聈" => "&#x8048;",
    "聉" => "&#x8049;",
    "聊" => "&#x804A;",
    "聋" => "&#x804B;",
    "职" => "&#x804C;",
    "聍" => "&#x804D;",
    "聎" => "&#x804E;",
    "聏" => "&#x804F;",
    "聐" => "&#x8050;",
    "聑" => "&#x8051;",
    "聒" => "&#x8052;",
    "聓" => "&#x8053;",
    "联" => "&#x8054;",
    "聕" => "&#x8055;",
    "聖" => "&#x8056;",
    "聗" => "&#x8057;",
    "聘" => "&#x8058;",
    "聙" => "&#x8059;",
    "聚" => "&#x805A;",
    "聛" => "&#x805B;",
    "聜" => "&#x805C;",
    "聝" => "&#x805D;",
    "聞" => "&#x805E;",
    "聟" => "&#x805F;",
    "聠" => "&#x8060;",
    "聡" => "&#x8061;",
    "聢" => "&#x8062;",
    "聣" => "&#x8063;",
    "聤" => "&#x8064;",
    "聥" => "&#x8065;",
    "聦" => "&#x8066;",
    "聧" => "&#x8067;",
    "聨" => "&#x8068;",
    "聩" => "&#x8069;",
    "聪" => "&#x806A;",
    "聫" => "&#x806B;",
    "聬" => "&#x806C;",
    "聭" => "&#x806D;",
    "聮" => "&#x806E;",
    "聯" => "&#x806F;",
    "聰" => "&#x8070;",
    "聱" => "&#x8071;",
    "聲" => "&#x8072;",
    "聳" => "&#x8073;",
    "聴" => "&#x8074;",
    "聵" => "&#x8075;",
    "聶" => "&#x8076;",
    "職" => "&#x8077;",
    "聸" => "&#x8078;",
    "聹" => "&#x8079;",
    "聺" => "&#x807A;",
    "聻" => "&#x807B;",
    "聼" => "&#x807C;",
    "聽" => "&#x807D;",
    "聾" => "&#x807E;",
    "聿" => "&#x807F;",
    "肀" => "&#x8080;",
    "肁" => "&#x8081;",
    "肂" => "&#x8082;",
    "肃" => "&#x8083;",
    "肄" => "&#x8084;",
    "肅" => "&#x8085;",
    "肆" => "&#x8086;",
    "肇" => "&#x8087;",
    "肈" => "&#x8088;",
    "肉" => "&#x8089;",
    "肊" => "&#x808A;",
    "肋" => "&#x808B;",
    "肌" => "&#x808C;",
    "肍" => "&#x808D;",
    "肎" => "&#x808E;",
    "肏" => "&#x808F;",
    "肐" => "&#x8090;",
    "肑" => "&#x8091;",
    "肒" => "&#x8092;",
    "肓" => "&#x8093;",
    "肔" => "&#x8094;",
    "肕" => "&#x8095;",
    "肖" => "&#x8096;",
    "肗" => "&#x8097;",
    "肘" => "&#x8098;",
    "肙" => "&#x8099;",
    "肚" => "&#x809A;",
    "肛" => "&#x809B;",
    "肜" => "&#x809C;",
    "肝" => "&#x809D;",
    "肞" => "&#x809E;",
    "肟" => "&#x809F;",
    "肠" => "&#x80A0;",
    "股" => "&#x80A1;",
    "肢" => "&#x80A2;",
    "肣" => "&#x80A3;",
    "肤" => "&#x80A4;",
    "肥" => "&#x80A5;",
    "肦" => "&#x80A6;",
    "肧" => "&#x80A7;",
    "肨" => "&#x80A8;",
    "肩" => "&#x80A9;",
    "肪" => "&#x80AA;",
    "肫" => "&#x80AB;",
    "肬" => "&#x80AC;",
    "肭" => "&#x80AD;",
    "肮" => "&#x80AE;",
    "肯" => "&#x80AF;",
    "肰" => "&#x80B0;",
    "肱" => "&#x80B1;",
    "育" => "&#x80B2;",
    "肳" => "&#x80B3;",
    "肴" => "&#x80B4;",
    "肵" => "&#x80B5;",
    "肶" => "&#x80B6;",
    "肷" => "&#x80B7;",
    "肸" => "&#x80B8;",
    "肹" => "&#x80B9;",
    "肺" => "&#x80BA;",
    "肻" => "&#x80BB;",
    "肼" => "&#x80BC;",
    "肽" => "&#x80BD;",
    "肾" => "&#x80BE;",
    "肿" => "&#x80BF;",
    "胀" => "&#x80C0;",
    "胁" => "&#x80C1;",
    "胂" => "&#x80C2;",
    "胃" => "&#x80C3;",
    "胄" => "&#x80C4;",
    "胅" => "&#x80C5;",
    "胆" => "&#x80C6;",
    "胇" => "&#x80C7;",
    "胈" => "&#x80C8;",
    "胉" => "&#x80C9;",
    "胊" => "&#x80CA;",
    "胋" => "&#x80CB;",
    "背" => "&#x80CC;",
    "胍" => "&#x80CD;",
    "胎" => "&#x80CE;",
    "胏" => "&#x80CF;",
    "胐" => "&#x80D0;",
    "胑" => "&#x80D1;",
    "胒" => "&#x80D2;",
    "胓" => "&#x80D3;",
    "胔" => "&#x80D4;",
    "胕" => "&#x80D5;",
    "胖" => "&#x80D6;",
    "胗" => "&#x80D7;",
    "胘" => "&#x80D8;",
    "胙" => "&#x80D9;",
    "胚" => "&#x80DA;",
    "胛" => "&#x80DB;",
    "胜" => "&#x80DC;",
    "胝" => "&#x80DD;",
    "胞" => "&#x80DE;",
    "胟" => "&#x80DF;",
    "胠" => "&#x80E0;",
    "胡" => "&#x80E1;",
    "胢" => "&#x80E2;",
    "胣" => "&#x80E3;",
    "胤" => "&#x80E4;",
    "胥" => "&#x80E5;",
    "胦" => "&#x80E6;",
    "胧" => "&#x80E7;",
    "胨" => "&#x80E8;",
    "胩" => "&#x80E9;",
    "胪" => "&#x80EA;",
    "胫" => "&#x80EB;",
    "胬" => "&#x80EC;",
    "胭" => "&#x80ED;",
    "胮" => "&#x80EE;",
    "胯" => "&#x80EF;",
    "胰" => "&#x80F0;",
    "胱" => "&#x80F1;",
    "胲" => "&#x80F2;",
    "胳" => "&#x80F3;",
    "胴" => "&#x80F4;",
    "胵" => "&#x80F5;",
    "胶" => "&#x80F6;",
    "胷" => "&#x80F7;",
    "胸" => "&#x80F8;",
    "胹" => "&#x80F9;",
    "胺" => "&#x80FA;",
    "胻" => "&#x80FB;",
    "胼" => "&#x80FC;",
    "能" => "&#x80FD;",
    "胾" => "&#x80FE;",
    "胿" => "&#x80FF;",
    "脀" => "&#x8100;",
    "脁" => "&#x8101;",
    "脂" => "&#x8102;",
    "脃" => "&#x8103;",
    "脄" => "&#x8104;",
    "脅" => "&#x8105;",
    "脆" => "&#x8106;",
    "脇" => "&#x8107;",
    "脈" => "&#x8108;",
    "脉" => "&#x8109;",
    "脊" => "&#x810A;",
    "脋" => "&#x810B;",
    "脌" => "&#x810C;",
    "脍" => "&#x810D;",
    "脎" => "&#x810E;",
    "脏" => "&#x810F;",
    "脐" => "&#x8110;",
    "脑" => "&#x8111;",
    "脒" => "&#x8112;",
    "脓" => "&#x8113;",
    "脔" => "&#x8114;",
    "脕" => "&#x8115;",
    "脖" => "&#x8116;",
    "脗" => "&#x8117;",
    "脘" => "&#x8118;",
    "脙" => "&#x8119;",
    "脚" => "&#x811A;",
    "脛" => "&#x811B;",
    "脜" => "&#x811C;",
    "脝" => "&#x811D;",
    "脞" => "&#x811E;",
    "脟" => "&#x811F;",
    "脠" => "&#x8120;",
    "脡" => "&#x8121;",
    "脢" => "&#x8122;",
    "脣" => "&#x8123;",
    "脤" => "&#x8124;",
    "脥" => "&#x8125;",
    "脦" => "&#x8126;",
    "脧" => "&#x8127;",
    "脨" => "&#x8128;",
    "脩" => "&#x8129;",
    "脪" => "&#x812A;",
    "脫" => "&#x812B;",
    "脬" => "&#x812C;",
    "脭" => "&#x812D;",
    "脮" => "&#x812E;",
    "脯" => "&#x812F;",
    "脰" => "&#x8130;",
    "脱" => "&#x8131;",
    "脲" => "&#x8132;",
    "脳" => "&#x8133;",
    "脴" => "&#x8134;",
    "脵" => "&#x8135;",
    "脶" => "&#x8136;",
    "脷" => "&#x8137;",
    "脸" => "&#x8138;",
    "脹" => "&#x8139;",
    "脺" => "&#x813A;",
    "脻" => "&#x813B;",
    "脼" => "&#x813C;",
    "脽" => "&#x813D;",
    "脾" => "&#x813E;",
    "脿" => "&#x813F;",
    "腀" => "&#x8140;",
    "腁" => "&#x8141;",
    "腂" => "&#x8142;",
    "腃" => "&#x8143;",
    "腄" => "&#x8144;",
    "腅" => "&#x8145;",
    "腆" => "&#x8146;",
    "腇" => "&#x8147;",
    "腈" => "&#x8148;",
    "腉" => "&#x8149;",
    "腊" => "&#x814A;",
    "腋" => "&#x814B;",
    "腌" => "&#x814C;",
    "腍" => "&#x814D;",
    "腎" => "&#x814E;",
    "腏" => "&#x814F;",
    "腐" => "&#x8150;",
    "腑" => "&#x8151;",
    "腒" => "&#x8152;",
    "腓" => "&#x8153;",
    "腔" => "&#x8154;",
    "腕" => "&#x8155;",
    "腖" => "&#x8156;",
    "腗" => "&#x8157;",
    "腘" => "&#x8158;",
    "腙" => "&#x8159;",
    "腚" => "&#x815A;",
    "腛" => "&#x815B;",
    "腜" => "&#x815C;",
    "腝" => "&#x815D;",
    "腞" => "&#x815E;",
    "腟" => "&#x815F;",
    "腠" => "&#x8160;",
    "腡" => "&#x8161;",
    "腢" => "&#x8162;",
    "腣" => "&#x8163;",
    "腤" => "&#x8164;",
    "腥" => "&#x8165;",
    "腦" => "&#x8166;",
    "腧" => "&#x8167;",
    "腨" => "&#x8168;",
    "腩" => "&#x8169;",
    "腪" => "&#x816A;",
    "腫" => "&#x816B;",
    "腬" => "&#x816C;",
    "腭" => "&#x816D;",
    "腮" => "&#x816E;",
    "腯" => "&#x816F;",
    "腰" => "&#x8170;",
    "腱" => "&#x8171;",
    "腲" => "&#x8172;",
    "腳" => "&#x8173;",
    "腴" => "&#x8174;",
    "腵" => "&#x8175;",
    "腶" => "&#x8176;",
    "腷" => "&#x8177;",
    "腸" => "&#x8178;",
    "腹" => "&#x8179;",
    "腺" => "&#x817A;",
    "腻" => "&#x817B;",
    "腼" => "&#x817C;",
    "腽" => "&#x817D;",
    "腾" => "&#x817E;",
    "腿" => "&#x817F;",
    "膀" => "&#x8180;",
    "膁" => "&#x8181;",
    "膂" => "&#x8182;",
    "膃" => "&#x8183;",
    "膄" => "&#x8184;",
    "膅" => "&#x8185;",
    "膆" => "&#x8186;",
    "膇" => "&#x8187;",
    "膈" => "&#x8188;",
    "膉" => "&#x8189;",
    "膊" => "&#x818A;",
    "膋" => "&#x818B;",
    "膌" => "&#x818C;",
    "膍" => "&#x818D;",
    "膎" => "&#x818E;",
    "膏" => "&#x818F;",
    "膐" => "&#x8190;",
    "膑" => "&#x8191;",
    "膒" => "&#x8192;",
    "膓" => "&#x8193;",
    "膔" => "&#x8194;",
    "膕" => "&#x8195;",
    "膖" => "&#x8196;",
    "膗" => "&#x8197;",
    "膘" => "&#x8198;",
    "膙" => "&#x8199;",
    "膚" => "&#x819A;",
    "膛" => "&#x819B;",
    "膜" => "&#x819C;",
    "膝" => "&#x819D;",
    "膞" => "&#x819E;",
    "膟" => "&#x819F;",
    "膠" => "&#x81A0;",
    "膡" => "&#x81A1;",
    "膢" => "&#x81A2;",
    "膣" => "&#x81A3;",
    "膤" => "&#x81A4;",
    "膥" => "&#x81A5;",
    "膦" => "&#x81A6;",
    "膧" => "&#x81A7;",
    "膨" => "&#x81A8;",
    "膩" => "&#x81A9;",
    "膪" => "&#x81AA;",
    "膫" => "&#x81AB;",
    "膬" => "&#x81AC;",
    "膭" => "&#x81AD;",
    "膮" => "&#x81AE;",
    "膯" => "&#x81AF;",
    "膰" => "&#x81B0;",
    "膱" => "&#x81B1;",
    "膲" => "&#x81B2;",
    "膳" => "&#x81B3;",
    "膴" => "&#x81B4;",
    "膵" => "&#x81B5;",
    "膶" => "&#x81B6;",
    "膷" => "&#x81B7;",
    "膸" => "&#x81B8;",
    "膹" => "&#x81B9;",
    "膺" => "&#x81BA;",
    "膻" => "&#x81BB;",
    "膼" => "&#x81BC;",
    "膽" => "&#x81BD;",
    "膾" => "&#x81BE;",
    "膿" => "&#x81BF;",
    "臀" => "&#x81C0;",
    "臁" => "&#x81C1;",
    "臂" => "&#x81C2;",
    "臃" => "&#x81C3;",
    "臄" => "&#x81C4;",
    "臅" => "&#x81C5;",
    "臆" => "&#x81C6;",
    "臇" => "&#x81C7;",
    "臈" => "&#x81C8;",
    "臉" => "&#x81C9;",
    "臊" => "&#x81CA;",
    "臋" => "&#x81CB;",
    "臌" => "&#x81CC;",
    "臍" => "&#x81CD;",
    "臎" => "&#x81CE;",
    "臏" => "&#x81CF;",
    "臐" => "&#x81D0;",
    "臑" => "&#x81D1;",
    "臒" => "&#x81D2;",
    "臓" => "&#x81D3;",
    "臔" => "&#x81D4;",
    "臕" => "&#x81D5;",
    "臖" => "&#x81D6;",
    "臗" => "&#x81D7;",
    "臘" => "&#x81D8;",
    "臙" => "&#x81D9;",
    "臚" => "&#x81DA;",
    "臛" => "&#x81DB;",
    "臜" => "&#x81DC;",
    "臝" => "&#x81DD;",
    "臞" => "&#x81DE;",
    "臟" => "&#x81DF;",
    "臠" => "&#x81E0;",
    "臡" => "&#x81E1;",
    "臢" => "&#x81E2;",
    "臣" => "&#x81E3;",
    "臤" => "&#x81E4;",
    "臥" => "&#x81E5;",
    "臦" => "&#x81E6;",
    "臧" => "&#x81E7;",
    "臨" => "&#x81E8;",
    "臩" => "&#x81E9;",
    "自" => "&#x81EA;",
    "臫" => "&#x81EB;",
    "臬" => "&#x81EC;",
    "臭" => "&#x81ED;",
    "臮" => "&#x81EE;",
    "臯" => "&#x81EF;",
    "臰" => "&#x81F0;",
    "臱" => "&#x81F1;",
    "臲" => "&#x81F2;",
    "至" => "&#x81F3;",
    "致" => "&#x81F4;",
    "臵" => "&#x81F5;",
    "臶" => "&#x81F6;",
    "臷" => "&#x81F7;",
    "臸" => "&#x81F8;",
    "臹" => "&#x81F9;",
    "臺" => "&#x81FA;",
    "臻" => "&#x81FB;",
    "臼" => "&#x81FC;",
    "臽" => "&#x81FD;",
    "臾" => "&#x81FE;",
    "臿" => "&#x81FF;",
    "舀" => "&#x8200;",
    "舁" => "&#x8201;",
    "舂" => "&#x8202;",
    "舃" => "&#x8203;",
    "舄" => "&#x8204;",
    "舅" => "&#x8205;",
    "舆" => "&#x8206;",
    "與" => "&#x8207;",
    "興" => "&#x8208;",
    "舉" => "&#x8209;",
    "舊" => "&#x820A;",
    "舋" => "&#x820B;",
    "舌" => "&#x820C;",
    "舍" => "&#x820D;",
    "舎" => "&#x820E;",
    "舏" => "&#x820F;",
    "舐" => "&#x8210;",
    "舑" => "&#x8211;",
    "舒" => "&#x8212;",
    "舓" => "&#x8213;",
    "舔" => "&#x8214;",
    "舕" => "&#x8215;",
    "舖" => "&#x8216;",
    "舗" => "&#x8217;",
    "舘" => "&#x8218;",
    "舙" => "&#x8219;",
    "舚" => "&#x821A;",
    "舛" => "&#x821B;",
    "舜" => "&#x821C;",
    "舝" => "&#x821D;",
    "舞" => "&#x821E;",
    "舟" => "&#x821F;",
    "舠" => "&#x8220;",
    "舡" => "&#x8221;",
    "舢" => "&#x8222;",
    "舣" => "&#x8223;",
    "舤" => "&#x8224;",
    "舥" => "&#x8225;",
    "舦" => "&#x8226;",
    "舧" => "&#x8227;",
    "舨" => "&#x8228;",
    "舩" => "&#x8229;",
    "航" => "&#x822A;",
    "舫" => "&#x822B;",
    "般" => "&#x822C;",
    "舭" => "&#x822D;",
    "舮" => "&#x822E;",
    "舯" => "&#x822F;",
    "舰" => "&#x8230;",
    "舱" => "&#x8231;",
    "舲" => "&#x8232;",
    "舳" => "&#x8233;",
    "舴" => "&#x8234;",
    "舵" => "&#x8235;",
    "舶" => "&#x8236;",
    "舷" => "&#x8237;",
    "舸" => "&#x8238;",
    "船" => "&#x8239;",
    "舺" => "&#x823A;",
    "舻" => "&#x823B;",
    "舼" => "&#x823C;",
    "舽" => "&#x823D;",
    "舾" => "&#x823E;",
    "舿" => "&#x823F;",
    "艀" => "&#x8240;",
    "艁" => "&#x8241;",
    "艂" => "&#x8242;",
    "艃" => "&#x8243;",
    "艄" => "&#x8244;",
    "艅" => "&#x8245;",
    "艆" => "&#x8246;",
    "艇" => "&#x8247;",
    "艈" => "&#x8248;",
    "艉" => "&#x8249;",
    "艊" => "&#x824A;",
    "艋" => "&#x824B;",
    "艌" => "&#x824C;",
    "艍" => "&#x824D;",
    "艎" => "&#x824E;",
    "艏" => "&#x824F;",
    "艐" => "&#x8250;",
    "艑" => "&#x8251;",
    "艒" => "&#x8252;",
    "艓" => "&#x8253;",
    "艔" => "&#x8254;",
    "艕" => "&#x8255;",
    "艖" => "&#x8256;",
    "艗" => "&#x8257;",
    "艘" => "&#x8258;",
    "艙" => "&#x8259;",
    "艚" => "&#x825A;",
    "艛" => "&#x825B;",
    "艜" => "&#x825C;",
    "艝" => "&#x825D;",
    "艞" => "&#x825E;",
    "艟" => "&#x825F;",
    "艠" => "&#x8260;",
    "艡" => "&#x8261;",
    "艢" => "&#x8262;",
    "艣" => "&#x8263;",
    "艤" => "&#x8264;",
    "艥" => "&#x8265;",
    "艦" => "&#x8266;",
    "艧" => "&#x8267;",
    "艨" => "&#x8268;",
    "艩" => "&#x8269;",
    "艪" => "&#x826A;",
    "艫" => "&#x826B;",
    "艬" => "&#x826C;",
    "艭" => "&#x826D;",
    "艮" => "&#x826E;",
    "良" => "&#x826F;",
    "艰" => "&#x8270;",
    "艱" => "&#x8271;",
    "色" => "&#x8272;",
    "艳" => "&#x8273;",
    "艴" => "&#x8274;",
    "艵" => "&#x8275;",
    "艶" => "&#x8276;",
    "艷" => "&#x8277;",
    "艸" => "&#x8278;",
    "艹" => "&#x8279;",
    "艺" => "&#x827A;",
    "艻" => "&#x827B;",
    "艼" => "&#x827C;",
    "艽" => "&#x827D;",
    "艾" => "&#x827E;",
    "艿" => "&#x827F;",
    "芀" => "&#x8280;",
    "芁" => "&#x8281;",
    "节" => "&#x8282;",
    "芃" => "&#x8283;",
    "芄" => "&#x8284;",
    "芅" => "&#x8285;",
    "芆" => "&#x8286;",
    "芇" => "&#x8287;",
    "芈" => "&#x8288;",
    "芉" => "&#x8289;",
    "芊" => "&#x828A;",
    "芋" => "&#x828B;",
    "芌" => "&#x828C;",
    "芍" => "&#x828D;",
    "芎" => "&#x828E;",
    "芏" => "&#x828F;",
    "芐" => "&#x8290;",
    "芑" => "&#x8291;",
    "芒" => "&#x8292;",
    "芓" => "&#x8293;",
    "芔" => "&#x8294;",
    "芕" => "&#x8295;",
    "芖" => "&#x8296;",
    "芗" => "&#x8297;",
    "芘" => "&#x8298;",
    "芙" => "&#x8299;",
    "芚" => "&#x829A;",
    "芛" => "&#x829B;",
    "芜" => "&#x829C;",
    "芝" => "&#x829D;",
    "芞" => "&#x829E;",
    "芟" => "&#x829F;",
    "芠" => "&#x82A0;",
    "芡" => "&#x82A1;",
    "芢" => "&#x82A2;",
    "芣" => "&#x82A3;",
    "芤" => "&#x82A4;",
    "芥" => "&#x82A5;",
    "芦" => "&#x82A6;",
    "芧" => "&#x82A7;",
    "芨" => "&#x82A8;",
    "芩" => "&#x82A9;",
    "芪" => "&#x82AA;",
    "芫" => "&#x82AB;",
    "芬" => "&#x82AC;",
    "芭" => "&#x82AD;",
    "芮" => "&#x82AE;",
    "芯" => "&#x82AF;",
    "芰" => "&#x82B0;",
    "花" => "&#x82B1;",
    "芲" => "&#x82B2;",
    "芳" => "&#x82B3;",
    "芴" => "&#x82B4;",
    "芵" => "&#x82B5;",
    "芶" => "&#x82B6;",
    "芷" => "&#x82B7;",
    "芸" => "&#x82B8;",
    "芹" => "&#x82B9;",
    "芺" => "&#x82BA;",
    "芻" => "&#x82BB;",
    "芼" => "&#x82BC;",
    "芽" => "&#x82BD;",
    "芾" => "&#x82BE;",
    "芿" => "&#x82BF;",
    "苀" => "&#x82C0;",
    "苁" => "&#x82C1;",
    "苂" => "&#x82C2;",
    "苃" => "&#x82C3;",
    "苄" => "&#x82C4;",
    "苅" => "&#x82C5;",
    "苆" => "&#x82C6;",
    "苇" => "&#x82C7;",
    "苈" => "&#x82C8;",
    "苉" => "&#x82C9;",
    "苊" => "&#x82CA;",
    "苋" => "&#x82CB;",
    "苌" => "&#x82CC;",
    "苍" => "&#x82CD;",
    "苎" => "&#x82CE;",
    "苏" => "&#x82CF;",
    "苐" => "&#x82D0;",
    "苑" => "&#x82D1;",
    "苒" => "&#x82D2;",
    "苓" => "&#x82D3;",
    "苔" => "&#x82D4;",
    "苕" => "&#x82D5;",
    "苖" => "&#x82D6;",
    "苗" => "&#x82D7;",
    "苘" => "&#x82D8;",
    "苙" => "&#x82D9;",
    "苚" => "&#x82DA;",
    "苛" => "&#x82DB;",
    "苜" => "&#x82DC;",
    "苝" => "&#x82DD;",
    "苞" => "&#x82DE;",
    "苟" => "&#x82DF;",
    "苠" => "&#x82E0;",
    "苡" => "&#x82E1;",
    "苢" => "&#x82E2;",
    "苣" => "&#x82E3;",
    "苤" => "&#x82E4;",
    "若" => "&#x82E5;",
    "苦" => "&#x82E6;",
    "苧" => "&#x82E7;",
    "苨" => "&#x82E8;",
    "苩" => "&#x82E9;",
    "苪" => "&#x82EA;",
    "苫" => "&#x82EB;",
    "苬" => "&#x82EC;",
    "苭" => "&#x82ED;",
    "苮" => "&#x82EE;",
    "苯" => "&#x82EF;",
    "苰" => "&#x82F0;",
    "英" => "&#x82F1;",
    "苲" => "&#x82F2;",
    "苳" => "&#x82F3;",
    "苴" => "&#x82F4;",
    "苵" => "&#x82F5;",
    "苶" => "&#x82F6;",
    "苷" => "&#x82F7;",
    "苸" => "&#x82F8;",
    "苹" => "&#x82F9;",
    "苺" => "&#x82FA;",
    "苻" => "&#x82FB;",
    "苼" => "&#x82FC;",
    "苽" => "&#x82FD;",
    "苾" => "&#x82FE;",
    "苿" => "&#x82FF;",
    "茀" => "&#x8300;",
    "茁" => "&#x8301;",
    "茂" => "&#x8302;",
    "范" => "&#x8303;",
    "茄" => "&#x8304;",
    "茅" => "&#x8305;",
    "茆" => "&#x8306;",
    "茇" => "&#x8307;",
    "茈" => "&#x8308;",
    "茉" => "&#x8309;",
    "茊" => "&#x830A;",
    "茋" => "&#x830B;",
    "茌" => "&#x830C;",
    "茍" => "&#x830D;",
    "茎" => "&#x830E;",
    "茏" => "&#x830F;",
    "茐" => "&#x8310;",
    "茑" => "&#x8311;",
    "茒" => "&#x8312;",
    "茓" => "&#x8313;",
    "茔" => "&#x8314;",
    "茕" => "&#x8315;",
    "茖" => "&#x8316;",
    "茗" => "&#x8317;",
    "茘" => "&#x8318;",
    "茙" => "&#x8319;",
    "茚" => "&#x831A;",
    "茛" => "&#x831B;",
    "茜" => "&#x831C;",
    "茝" => "&#x831D;",
    "茞" => "&#x831E;",
    "茟" => "&#x831F;",
    "茠" => "&#x8320;",
    "茡" => "&#x8321;",
    "茢" => "&#x8322;",
    "茣" => "&#x8323;",
    "茤" => "&#x8324;",
    "茥" => "&#x8325;",
    "茦" => "&#x8326;",
    "茧" => "&#x8327;",
    "茨" => "&#x8328;",
    "茩" => "&#x8329;",
    "茪" => "&#x832A;",
    "茫" => "&#x832B;",
    "茬" => "&#x832C;",
    "茭" => "&#x832D;",
    "茮" => "&#x832E;",
    "茯" => "&#x832F;",
    "茰" => "&#x8330;",
    "茱" => "&#x8331;",
    "茲" => "&#x8332;",
    "茳" => "&#x8333;",
    "茴" => "&#x8334;",
    "茵" => "&#x8335;",
    "茶" => "&#x8336;",
    "茷" => "&#x8337;",
    "茸" => "&#x8338;",
    "茹" => "&#x8339;",
    "茺" => "&#x833A;",
    "茻" => "&#x833B;",
    "茼" => "&#x833C;",
    "茽" => "&#x833D;",
    "茾" => "&#x833E;",
    "茿" => "&#x833F;",
    "荀" => "&#x8340;",
    "荁" => "&#x8341;",
    "荂" => "&#x8342;",
    "荃" => "&#x8343;",
    "荄" => "&#x8344;",
    "荅" => "&#x8345;",
    "荆" => "&#x8346;",
    "荇" => "&#x8347;",
    "荈" => "&#x8348;",
    "草" => "&#x8349;",
    "荊" => "&#x834A;",
    "荋" => "&#x834B;",
    "荌" => "&#x834C;",
    "荍" => "&#x834D;",
    "荎" => "&#x834E;",
    "荏" => "&#x834F;",
    "荐" => "&#x8350;",
    "荑" => "&#x8351;",
    "荒" => "&#x8352;",
    "荓" => "&#x8353;",
    "荔" => "&#x8354;",
    "荕" => "&#x8355;",
    "荖" => "&#x8356;",
    "荗" => "&#x8357;",
    "荘" => "&#x8358;",
    "荙" => "&#x8359;",
    "荚" => "&#x835A;",
    "荛" => "&#x835B;",
    "荜" => "&#x835C;",
    "荝" => "&#x835D;",
    "荞" => "&#x835E;",
    "荟" => "&#x835F;",
    "荠" => "&#x8360;",
    "荡" => "&#x8361;",
    "荢" => "&#x8362;",
    "荣" => "&#x8363;",
    "荤" => "&#x8364;",
    "荥" => "&#x8365;",
    "荦" => "&#x8366;",
    "荧" => "&#x8367;",
    "荨" => "&#x8368;",
    "荩" => "&#x8369;",
    "荪" => "&#x836A;",
    "荫" => "&#x836B;",
    "荬" => "&#x836C;",
    "荭" => "&#x836D;",
    "荮" => "&#x836E;",
    "药" => "&#x836F;",
    "荰" => "&#x8370;",
    "荱" => "&#x8371;",
    "荲" => "&#x8372;",
    "荳" => "&#x8373;",
    "荴" => "&#x8374;",
    "荵" => "&#x8375;",
    "荶" => "&#x8376;",
    "荷" => "&#x8377;",
    "荸" => "&#x8378;",
    "荹" => "&#x8379;",
    "荺" => "&#x837A;",
    "荻" => "&#x837B;",
    "荼" => "&#x837C;",
    "荽" => "&#x837D;",
    "荾" => "&#x837E;",
    "荿" => "&#x837F;",
    "莀" => "&#x8380;",
    "莁" => "&#x8381;",
    "莂" => "&#x8382;",
    "莃" => "&#x8383;",
    "莄" => "&#x8384;",
    "莅" => "&#x8385;",
    "莆" => "&#x8386;",
    "莇" => "&#x8387;",
    "莈" => "&#x8388;",
    "莉" => "&#x8389;",
    "莊" => "&#x838A;",
    "莋" => "&#x838B;",
    "莌" => "&#x838C;",
    "莍" => "&#x838D;",
    "莎" => "&#x838E;",
    "莏" => "&#x838F;",
    "莐" => "&#x8390;",
    "莑" => "&#x8391;",
    "莒" => "&#x8392;",
    "莓" => "&#x8393;",
    "莔" => "&#x8394;",
    "莕" => "&#x8395;",
    "莖" => "&#x8396;",
    "莗" => "&#x8397;",
    "莘" => "&#x8398;",
    "莙" => "&#x8399;",
    "莚" => "&#x839A;",
    "莛" => "&#x839B;",
    "莜" => "&#x839C;",
    "莝" => "&#x839D;",
    "莞" => "&#x839E;",
    "莟" => "&#x839F;",
    "莠" => "&#x83A0;",
    "莡" => "&#x83A1;",
    "莢" => "&#x83A2;",
    "莣" => "&#x83A3;",
    "莤" => "&#x83A4;",
    "莥" => "&#x83A5;",
    "莦" => "&#x83A6;",
    "莧" => "&#x83A7;",
    "莨" => "&#x83A8;",
    "莩" => "&#x83A9;",
    "莪" => "&#x83AA;",
    "莫" => "&#x83AB;",
    "莬" => "&#x83AC;",
    "莭" => "&#x83AD;",
    "莮" => "&#x83AE;",
    "莯" => "&#x83AF;",
    "莰" => "&#x83B0;",
    "莱" => "&#x83B1;",
    "莲" => "&#x83B2;",
    "莳" => "&#x83B3;",
    "莴" => "&#x83B4;",
    "莵" => "&#x83B5;",
    "莶" => "&#x83B6;",
    "获" => "&#x83B7;",
    "莸" => "&#x83B8;",
    "莹" => "&#x83B9;",
    "莺" => "&#x83BA;",
    "莻" => "&#x83BB;",
    "莼" => "&#x83BC;",
    "莽" => "&#x83BD;",
    "莾" => "&#x83BE;",
    "莿" => "&#x83BF;",
    "菀" => "&#x83C0;",
    "菁" => "&#x83C1;",
    "菂" => "&#x83C2;",
    "菃" => "&#x83C3;",
    "菄" => "&#x83C4;",
    "菅" => "&#x83C5;",
    "菆" => "&#x83C6;",
    "菇" => "&#x83C7;",
    "菈" => "&#x83C8;",
    "菉" => "&#x83C9;",
    "菊" => "&#x83CA;",
    "菋" => "&#x83CB;",
    "菌" => "&#x83CC;",
    "菍" => "&#x83CD;",
    "菎" => "&#x83CE;",
    "菏" => "&#x83CF;",
    "菐" => "&#x83D0;",
    "菑" => "&#x83D1;",
    "菒" => "&#x83D2;",
    "菓" => "&#x83D3;",
    "菔" => "&#x83D4;",
    "菕" => "&#x83D5;",
    "菖" => "&#x83D6;",
    "菗" => "&#x83D7;",
    "菘" => "&#x83D8;",
    "菙" => "&#x83D9;",
    "菚" => "&#x83DA;",
    "菛" => "&#x83DB;",
    "菜" => "&#x83DC;",
    "菝" => "&#x83DD;",
    "菞" => "&#x83DE;",
    "菟" => "&#x83DF;",
    "菠" => "&#x83E0;",
    "菡" => "&#x83E1;",
    "菢" => "&#x83E2;",
    "菣" => "&#x83E3;",
    "菤" => "&#x83E4;",
    "菥" => "&#x83E5;",
    "菦" => "&#x83E6;",
    "菧" => "&#x83E7;",
    "菨" => "&#x83E8;",
    "菩" => "&#x83E9;",
    "菪" => "&#x83EA;",
    "菫" => "&#x83EB;",
    "菬" => "&#x83EC;",
    "菭" => "&#x83ED;",
    "菮" => "&#x83EE;",
    "華" => "&#x83EF;",
    "菰" => "&#x83F0;",
    "菱" => "&#x83F1;",
    "菲" => "&#x83F2;",
    "菳" => "&#x83F3;",
    "菴" => "&#x83F4;",
    "菵" => "&#x83F5;",
    "菶" => "&#x83F6;",
    "菷" => "&#x83F7;",
    "菸" => "&#x83F8;",
    "菹" => "&#x83F9;",
    "菺" => "&#x83FA;",
    "菻" => "&#x83FB;",
    "菼" => "&#x83FC;",
    "菽" => "&#x83FD;",
    "菾" => "&#x83FE;",
    "菿" => "&#x83FF;",
    "萀" => "&#x8400;",
    "萁" => "&#x8401;",
    "萂" => "&#x8402;",
    "萃" => "&#x8403;",
    "萄" => "&#x8404;",
    "萅" => "&#x8405;",
    "萆" => "&#x8406;",
    "萇" => "&#x8407;",
    "萈" => "&#x8408;",
    "萉" => "&#x8409;",
    "萊" => "&#x840A;",
    "萋" => "&#x840B;",
    "萌" => "&#x840C;",
    "萍" => "&#x840D;",
    "萎" => "&#x840E;",
    "萏" => "&#x840F;",
    "萐" => "&#x8410;",
    "萑" => "&#x8411;",
    "萒" => "&#x8412;",
    "萓" => "&#x8413;",
    "萔" => "&#x8414;",
    "萕" => "&#x8415;",
    "萖" => "&#x8416;",
    "萗" => "&#x8417;",
    "萘" => "&#x8418;",
    "萙" => "&#x8419;",
    "萚" => "&#x841A;",
    "萛" => "&#x841B;",
    "萜" => "&#x841C;",
    "萝" => "&#x841D;",
    "萞" => "&#x841E;",
    "萟" => "&#x841F;",
    "萠" => "&#x8420;",
    "萡" => "&#x8421;",
    "萢" => "&#x8422;",
    "萣" => "&#x8423;",
    "萤" => "&#x8424;",
    "营" => "&#x8425;",
    "萦" => "&#x8426;",
    "萧" => "&#x8427;",
    "萨" => "&#x8428;",
    "萩" => "&#x8429;",
    "萪" => "&#x842A;",
    "萫" => "&#x842B;",
    "萬" => "&#x842C;",
    "萭" => "&#x842D;",
    "萮" => "&#x842E;",
    "萯" => "&#x842F;",
    "萰" => "&#x8430;",
    "萱" => "&#x8431;",
    "萲" => "&#x8432;",
    "萳" => "&#x8433;",
    "萴" => "&#x8434;",
    "萵" => "&#x8435;",
    "萶" => "&#x8436;",
    "萷" => "&#x8437;",
    "萸" => "&#x8438;",
    "萹" => "&#x8439;",
    "萺" => "&#x843A;",
    "萻" => "&#x843B;",
    "萼" => "&#x843C;",
    "落" => "&#x843D;",
    "萾" => "&#x843E;",
    "萿" => "&#x843F;",
    "葀" => "&#x8440;",
    "葁" => "&#x8441;",
    "葂" => "&#x8442;",
    "葃" => "&#x8443;",
    "葄" => "&#x8444;",
    "葅" => "&#x8445;",
    "葆" => "&#x8446;",
    "葇" => "&#x8447;",
    "葈" => "&#x8448;",
    "葉" => "&#x8449;",
    "葊" => "&#x844A;",
    "葋" => "&#x844B;",
    "葌" => "&#x844C;",
    "葍" => "&#x844D;",
    "葎" => "&#x844E;",
    "葏" => "&#x844F;",
    "葐" => "&#x8450;",
    "葑" => "&#x8451;",
    "葒" => "&#x8452;",
    "葓" => "&#x8453;",
    "葔" => "&#x8454;",
    "葕" => "&#x8455;",
    "葖" => "&#x8456;",
    "著" => "&#x8457;",
    "葘" => "&#x8458;",
    "葙" => "&#x8459;",
    "葚" => "&#x845A;",
    "葛" => "&#x845B;",
    "葜" => "&#x845C;",
    "葝" => "&#x845D;",
    "葞" => "&#x845E;",
    "葟" => "&#x845F;",
    "葠" => "&#x8460;",
    "葡" => "&#x8461;",
    "葢" => "&#x8462;",
    "董" => "&#x8463;",
    "葤" => "&#x8464;",
    "葥" => "&#x8465;",
    "葦" => "&#x8466;",
    "葧" => "&#x8467;",
    "葨" => "&#x8468;",
    "葩" => "&#x8469;",
    "葪" => "&#x846A;",
    "葫" => "&#x846B;",
    "葬" => "&#x846C;",
    "葭" => "&#x846D;",
    "葮" => "&#x846E;",
    "葯" => "&#x846F;",
    "葰" => "&#x8470;",
    "葱" => "&#x8471;",
    "葲" => "&#x8472;",
    "葳" => "&#x8473;",
    "葴" => "&#x8474;",
    "葵" => "&#x8475;",
    "葶" => "&#x8476;",
    "葷" => "&#x8477;",
    "葸" => "&#x8478;",
    "葹" => "&#x8479;",
    "葺" => "&#x847A;",
    "葻" => "&#x847B;",
    "葼" => "&#x847C;",
    "葽" => "&#x847D;",
    "葾" => "&#x847E;",
    "葿" => "&#x847F;",
    "蒀" => "&#x8480;",
    "蒁" => "&#x8481;",
    "蒂" => "&#x8482;",
    "蒃" => "&#x8483;",
    "蒄" => "&#x8484;",
    "蒅" => "&#x8485;",
    "蒆" => "&#x8486;",
    "蒇" => "&#x8487;",
    "蒈" => "&#x8488;",
    "蒉" => "&#x8489;",
    "蒊" => "&#x848A;",
    "蒋" => "&#x848B;",
    "蒌" => "&#x848C;",
    "蒍" => "&#x848D;",
    "蒎" => "&#x848E;",
    "蒏" => "&#x848F;",
    "蒐" => "&#x8490;",
    "蒑" => "&#x8491;",
    "蒒" => "&#x8492;",
    "蒓" => "&#x8493;",
    "蒔" => "&#x8494;",
    "蒕" => "&#x8495;",
    "蒖" => "&#x8496;",
    "蒗" => "&#x8497;",
    "蒘" => "&#x8498;",
    "蒙" => "&#x8499;",
    "蒚" => "&#x849A;",
    "蒛" => "&#x849B;",
    "蒜" => "&#x849C;",
    "蒝" => "&#x849D;",
    "蒞" => "&#x849E;",
    "蒟" => "&#x849F;",
    "蒠" => "&#x84A0;",
    "蒡" => "&#x84A1;",
    "蒢" => "&#x84A2;",
    "蒣" => "&#x84A3;",
    "蒤" => "&#x84A4;",
    "蒥" => "&#x84A5;",
    "蒦" => "&#x84A6;",
    "蒧" => "&#x84A7;",
    "蒨" => "&#x84A8;",
    "蒩" => "&#x84A9;",
    "蒪" => "&#x84AA;",
    "蒫" => "&#x84AB;",
    "蒬" => "&#x84AC;",
    "蒭" => "&#x84AD;",
    "蒮" => "&#x84AE;",
    "蒯" => "&#x84AF;",
    "蒰" => "&#x84B0;",
    "蒱" => "&#x84B1;",
    "蒲" => "&#x84B2;",
    "蒳" => "&#x84B3;",
    "蒴" => "&#x84B4;",
    "蒵" => "&#x84B5;",
    "蒶" => "&#x84B6;",
    "蒷" => "&#x84B7;",
    "蒸" => "&#x84B8;",
    "蒹" => "&#x84B9;",
    "蒺" => "&#x84BA;",
    "蒻" => "&#x84BB;",
    "蒼" => "&#x84BC;",
    "蒽" => "&#x84BD;",
    "蒾" => "&#x84BE;",
    "蒿" => "&#x84BF;",
    "蓀" => "&#x84C0;",
    "蓁" => "&#x84C1;",
    "蓂" => "&#x84C2;",
    "蓃" => "&#x84C3;",
    "蓄" => "&#x84C4;",
    "蓅" => "&#x84C5;",
    "蓆" => "&#x84C6;",
    "蓇" => "&#x84C7;",
    "蓈" => "&#x84C8;",
    "蓉" => "&#x84C9;",
    "蓊" => "&#x84CA;",
    "蓋" => "&#x84CB;",
    "蓌" => "&#x84CC;",
    "蓍" => "&#x84CD;",
    "蓎" => "&#x84CE;",
    "蓏" => "&#x84CF;",
    "蓐" => "&#x84D0;",
    "蓑" => "&#x84D1;",
    "蓒" => "&#x84D2;",
    "蓓" => "&#x84D3;",
    "蓔" => "&#x84D4;",
    "蓕" => "&#x84D5;",
    "蓖" => "&#x84D6;",
    "蓗" => "&#x84D7;",
    "蓘" => "&#x84D8;",
    "蓙" => "&#x84D9;",
    "蓚" => "&#x84DA;",
    "蓛" => "&#x84DB;",
    "蓜" => "&#x84DC;",
    "蓝" => "&#x84DD;",
    "蓞" => "&#x84DE;",
    "蓟" => "&#x84DF;",
    "蓠" => "&#x84E0;",
    "蓡" => "&#x84E1;",
    "蓢" => "&#x84E2;",
    "蓣" => "&#x84E3;",
    "蓤" => "&#x84E4;",
    "蓥" => "&#x84E5;",
    "蓦" => "&#x84E6;",
    "蓧" => "&#x84E7;",
    "蓨" => "&#x84E8;",
    "蓩" => "&#x84E9;",
    "蓪" => "&#x84EA;",
    "蓫" => "&#x84EB;",
    "蓬" => "&#x84EC;",
    "蓭" => "&#x84ED;",
    "蓮" => "&#x84EE;",
    "蓯" => "&#x84EF;",
    "蓰" => "&#x84F0;",
    "蓱" => "&#x84F1;",
    "蓲" => "&#x84F2;",
    "蓳" => "&#x84F3;",
    "蓴" => "&#x84F4;",
    "蓵" => "&#x84F5;",
    "蓶" => "&#x84F6;",
    "蓷" => "&#x84F7;",
    "蓸" => "&#x84F8;",
    "蓹" => "&#x84F9;",
    "蓺" => "&#x84FA;",
    "蓻" => "&#x84FB;",
    "蓼" => "&#x84FC;",
    "蓽" => "&#x84FD;",
    "蓾" => "&#x84FE;",
    "蓿" => "&#x84FF;",
    "蔀" => "&#x8500;",
    "蔁" => "&#x8501;",
    "蔂" => "&#x8502;",
    "蔃" => "&#x8503;",
    "蔄" => "&#x8504;",
    "蔅" => "&#x8505;",
    "蔆" => "&#x8506;",
    "蔇" => "&#x8507;",
    "蔈" => "&#x8508;",
    "蔉" => "&#x8509;",
    "蔊" => "&#x850A;",
    "蔋" => "&#x850B;",
    "蔌" => "&#x850C;",
    "蔍" => "&#x850D;",
    "蔎" => "&#x850E;",
    "蔏" => "&#x850F;",
    "蔐" => "&#x8510;",
    "蔑" => "&#x8511;",
    "蔒" => "&#x8512;",
    "蔓" => "&#x8513;",
    "蔔" => "&#x8514;",
    "蔕" => "&#x8515;",
    "蔖" => "&#x8516;",
    "蔗" => "&#x8517;",
    "蔘" => "&#x8518;",
    "蔙" => "&#x8519;",
    "蔚" => "&#x851A;",
    "蔛" => "&#x851B;",
    "蔜" => "&#x851C;",
    "蔝" => "&#x851D;",
    "蔞" => "&#x851E;",
    "蔟" => "&#x851F;",
    "蔠" => "&#x8520;",
    "蔡" => "&#x8521;",
    "蔢" => "&#x8522;",
    "蔣" => "&#x8523;",
    "蔤" => "&#x8524;",
    "蔥" => "&#x8525;",
    "蔦" => "&#x8526;",
    "蔧" => "&#x8527;",
    "蔨" => "&#x8528;",
    "蔩" => "&#x8529;",
    "蔪" => "&#x852A;",
    "蔫" => "&#x852B;",
    "蔬" => "&#x852C;",
    "蔭" => "&#x852D;",
    "蔮" => "&#x852E;",
    "蔯" => "&#x852F;",
    "蔰" => "&#x8530;",
    "蔱" => "&#x8531;",
    "蔲" => "&#x8532;",
    "蔳" => "&#x8533;",
    "蔴" => "&#x8534;",
    "蔵" => "&#x8535;",
    "蔶" => "&#x8536;",
    "蔷" => "&#x8537;",
    "蔸" => "&#x8538;",
    "蔹" => "&#x8539;",
    "蔺" => "&#x853A;",
    "蔻" => "&#x853B;",
    "蔼" => "&#x853C;",
    "蔽" => "&#x853D;",
    "蔾" => "&#x853E;",
    "蔿" => "&#x853F;",
    "蕀" => "&#x8540;",
    "蕁" => "&#x8541;",
    "蕂" => "&#x8542;",
    "蕃" => "&#x8543;",
    "蕄" => "&#x8544;",
    "蕅" => "&#x8545;",
    "蕆" => "&#x8546;",
    "蕇" => "&#x8547;",
    "蕈" => "&#x8548;",
    "蕉" => "&#x8549;",
    "蕊" => "&#x854A;",
    "蕋" => "&#x854B;",
    "蕌" => "&#x854C;",
    "蕍" => "&#x854D;",
    "蕎" => "&#x854E;",
    "蕏" => "&#x854F;",
    "蕐" => "&#x8550;",
    "蕑" => "&#x8551;",
    "蕒" => "&#x8552;",
    "蕓" => "&#x8553;",
    "蕔" => "&#x8554;",
    "蕕" => "&#x8555;",
    "蕖" => "&#x8556;",
    "蕗" => "&#x8557;",
    "蕘" => "&#x8558;",
    "蕙" => "&#x8559;",
    "蕚" => "&#x855A;",
    "蕛" => "&#x855B;",
    "蕜" => "&#x855C;",
    "蕝" => "&#x855D;",
    "蕞" => "&#x855E;",
    "蕟" => "&#x855F;",
    "蕠" => "&#x8560;",
    "蕡" => "&#x8561;",
    "蕢" => "&#x8562;",
    "蕣" => "&#x8563;",
    "蕤" => "&#x8564;",
    "蕥" => "&#x8565;",
    "蕦" => "&#x8566;",
    "蕧" => "&#x8567;",
    "蕨" => "&#x8568;",
    "蕩" => "&#x8569;",
    "蕪" => "&#x856A;",
    "蕫" => "&#x856B;",
    "蕬" => "&#x856C;",
    "蕭" => "&#x856D;",
    "蕮" => "&#x856E;",
    "蕯" => "&#x856F;",
    "蕰" => "&#x8570;",
    "蕱" => "&#x8571;",
    "蕲" => "&#x8572;",
    "蕳" => "&#x8573;",
    "蕴" => "&#x8574;",
    "蕵" => "&#x8575;",
    "蕶" => "&#x8576;",
    "蕷" => "&#x8577;",
    "蕸" => "&#x8578;",
    "蕹" => "&#x8579;",
    "蕺" => "&#x857A;",
    "蕻" => "&#x857B;",
    "蕼" => "&#x857C;",
    "蕽" => "&#x857D;",
    "蕾" => "&#x857E;",
    "蕿" => "&#x857F;",
    "薀" => "&#x8580;",
    "薁" => "&#x8581;",
    "薂" => "&#x8582;",
    "薃" => "&#x8583;",
    "薄" => "&#x8584;",
    "薅" => "&#x8585;",
    "薆" => "&#x8586;",
    "薇" => "&#x8587;",
    "薈" => "&#x8588;",
    "薉" => "&#x8589;",
    "薊" => "&#x858A;",
    "薋" => "&#x858B;",
    "薌" => "&#x858C;",
    "薍" => "&#x858D;",
    "薎" => "&#x858E;",
    "薏" => "&#x858F;",
    "薐" => "&#x8590;",
    "薑" => "&#x8591;",
    "薒" => "&#x8592;",
    "薓" => "&#x8593;",
    "薔" => "&#x8594;",
    "薕" => "&#x8595;",
    "薖" => "&#x8596;",
    "薗" => "&#x8597;",
    "薘" => "&#x8598;",
    "薙" => "&#x8599;",
    "薚" => "&#x859A;",
    "薛" => "&#x859B;",
    "薜" => "&#x859C;",
    "薝" => "&#x859D;",
    "薞" => "&#x859E;",
    "薟" => "&#x859F;",
    "薠" => "&#x85A0;",
    "薡" => "&#x85A1;",
    "薢" => "&#x85A2;",
    "薣" => "&#x85A3;",
    "薤" => "&#x85A4;",
    "薥" => "&#x85A5;",
    "薦" => "&#x85A6;",
    "薧" => "&#x85A7;",
    "薨" => "&#x85A8;",
    "薩" => "&#x85A9;",
    "薪" => "&#x85AA;",
    "薫" => "&#x85AB;",
    "薬" => "&#x85AC;",
    "薭" => "&#x85AD;",
    "薮" => "&#x85AE;",
    "薯" => "&#x85AF;",
    "薰" => "&#x85B0;",
    "薱" => "&#x85B1;",
    "薲" => "&#x85B2;",
    "薳" => "&#x85B3;",
    "薴" => "&#x85B4;",
    "薵" => "&#x85B5;",
    "薶" => "&#x85B6;",
    "薷" => "&#x85B7;",
    "薸" => "&#x85B8;",
    "薹" => "&#x85B9;",
    "薺" => "&#x85BA;",
    "薻" => "&#x85BB;",
    "薼" => "&#x85BC;",
    "薽" => "&#x85BD;",
    "薾" => "&#x85BE;",
    "薿" => "&#x85BF;",
    "藀" => "&#x85C0;",
    "藁" => "&#x85C1;",
    "藂" => "&#x85C2;",
    "藃" => "&#x85C3;",
    "藄" => "&#x85C4;",
    "藅" => "&#x85C5;",
    "藆" => "&#x85C6;",
    "藇" => "&#x85C7;",
    "藈" => "&#x85C8;",
    "藉" => "&#x85C9;",
    "藊" => "&#x85CA;",
    "藋" => "&#x85CB;",
    "藌" => "&#x85CC;",
    "藍" => "&#x85CD;",
    "藎" => "&#x85CE;",
    "藏" => "&#x85CF;",
    "藐" => "&#x85D0;",
    "藑" => "&#x85D1;",
    "藒" => "&#x85D2;",
    "藓" => "&#x85D3;",
    "藔" => "&#x85D4;",
    "藕" => "&#x85D5;",
    "藖" => "&#x85D6;",
    "藗" => "&#x85D7;",
    "藘" => "&#x85D8;",
    "藙" => "&#x85D9;",
    "藚" => "&#x85DA;",
    "藛" => "&#x85DB;",
    "藜" => "&#x85DC;",
    "藝" => "&#x85DD;",
    "藞" => "&#x85DE;",
    "藟" => "&#x85DF;",
    "藠" => "&#x85E0;",
    "藡" => "&#x85E1;",
    "藢" => "&#x85E2;",
    "藣" => "&#x85E3;",
    "藤" => "&#x85E4;",
    "藥" => "&#x85E5;",
    "藦" => "&#x85E6;",
    "藧" => "&#x85E7;",
    "藨" => "&#x85E8;",
    "藩" => "&#x85E9;",
    "藪" => "&#x85EA;",
    "藫" => "&#x85EB;",
    "藬" => "&#x85EC;",
    "藭" => "&#x85ED;",
    "藮" => "&#x85EE;",
    "藯" => "&#x85EF;",
    "藰" => "&#x85F0;",
    "藱" => "&#x85F1;",
    "藲" => "&#x85F2;",
    "藳" => "&#x85F3;",
    "藴" => "&#x85F4;",
    "藵" => "&#x85F5;",
    "藶" => "&#x85F6;",
    "藷" => "&#x85F7;",
    "藸" => "&#x85F8;",
    "藹" => "&#x85F9;",
    "藺" => "&#x85FA;",
    "藻" => "&#x85FB;",
    "藼" => "&#x85FC;",
    "藽" => "&#x85FD;",
    "藾" => "&#x85FE;",
    "藿" => "&#x85FF;",
    "蘀" => "&#x8600;",
    "蘁" => "&#x8601;",
    "蘂" => "&#x8602;",
    "蘃" => "&#x8603;",
    "蘄" => "&#x8604;",
    "蘅" => "&#x8605;",
    "蘆" => "&#x8606;",
    "蘇" => "&#x8607;",
    "蘈" => "&#x8608;",
    "蘉" => "&#x8609;",
    "蘊" => "&#x860A;",
    "蘋" => "&#x860B;",
    "蘌" => "&#x860C;",
    "蘍" => "&#x860D;",
    "蘎" => "&#x860E;",
    "蘏" => "&#x860F;",
    "蘐" => "&#x8610;",
    "蘑" => "&#x8611;",
    "蘒" => "&#x8612;",
    "蘓" => "&#x8613;",
    "蘔" => "&#x8614;",
    "蘕" => "&#x8615;",
    "蘖" => "&#x8616;",
    "蘗" => "&#x8617;",
    "蘘" => "&#x8618;",
    "蘙" => "&#x8619;",
    "蘚" => "&#x861A;",
    "蘛" => "&#x861B;",
    "蘜" => "&#x861C;",
    "蘝" => "&#x861D;",
    "蘞" => "&#x861E;",
    "蘟" => "&#x861F;",
    "蘠" => "&#x8620;",
    "蘡" => "&#x8621;",
    "蘢" => "&#x8622;",
    "蘣" => "&#x8623;",
    "蘤" => "&#x8624;",
    "蘥" => "&#x8625;",
    "蘦" => "&#x8626;",
    "蘧" => "&#x8627;",
    "蘨" => "&#x8628;",
    "蘩" => "&#x8629;",
    "蘪" => "&#x862A;",
    "蘫" => "&#x862B;",
    "蘬" => "&#x862C;",
    "蘭" => "&#x862D;",
    "蘮" => "&#x862E;",
    "蘯" => "&#x862F;",
    "蘰" => "&#x8630;",
    "蘱" => "&#x8631;",
    "蘲" => "&#x8632;",
    "蘳" => "&#x8633;",
    "蘴" => "&#x8634;",
    "蘵" => "&#x8635;",
    "蘶" => "&#x8636;",
    "蘷" => "&#x8637;",
    "蘸" => "&#x8638;",
    "蘹" => "&#x8639;",
    "蘺" => "&#x863A;",
    "蘻" => "&#x863B;",
    "蘼" => "&#x863C;",
    "蘽" => "&#x863D;",
    "蘾" => "&#x863E;",
    "蘿" => "&#x863F;",
    "虀" => "&#x8640;",
    "虁" => "&#x8641;",
    "虂" => "&#x8642;",
    "虃" => "&#x8643;",
    "虄" => "&#x8644;",
    "虅" => "&#x8645;",
    "虆" => "&#x8646;",
    "虇" => "&#x8647;",
    "虈" => "&#x8648;",
    "虉" => "&#x8649;",
    "虊" => "&#x864A;",
    "虋" => "&#x864B;",
    "虌" => "&#x864C;",
    "虍" => "&#x864D;",
    "虎" => "&#x864E;",
    "虏" => "&#x864F;",
    "虐" => "&#x8650;",
    "虑" => "&#x8651;",
    "虒" => "&#x8652;",
    "虓" => "&#x8653;",
    "虔" => "&#x8654;",
    "處" => "&#x8655;",
    "虖" => "&#x8656;",
    "虗" => "&#x8657;",
    "虘" => "&#x8658;",
    "虙" => "&#x8659;",
    "虚" => "&#x865A;",
    "虛" => "&#x865B;",
    "虜" => "&#x865C;",
    "虝" => "&#x865D;",
    "虞" => "&#x865E;",
    "號" => "&#x865F;",
    "虠" => "&#x8660;",
    "虡" => "&#x8661;",
    "虢" => "&#x8662;",
    "虣" => "&#x8663;",
    "虤" => "&#x8664;",
    "虥" => "&#x8665;",
    "虦" => "&#x8666;",
    "虧" => "&#x8667;",
    "虨" => "&#x8668;",
    "虩" => "&#x8669;",
    "虪" => "&#x866A;",
    "虫" => "&#x866B;",
    "虬" => "&#x866C;",
    "虭" => "&#x866D;",
    "虮" => "&#x866E;",
    "虯" => "&#x866F;",
    "虰" => "&#x8670;",
    "虱" => "&#x8671;",
    "虲" => "&#x8672;",
    "虳" => "&#x8673;",
    "虴" => "&#x8674;",
    "虵" => "&#x8675;",
    "虶" => "&#x8676;",
    "虷" => "&#x8677;",
    "虸" => "&#x8678;",
    "虹" => "&#x8679;",
    "虺" => "&#x867A;",
    "虻" => "&#x867B;",
    "虼" => "&#x867C;",
    "虽" => "&#x867D;",
    "虾" => "&#x867E;",
    "虿" => "&#x867F;",
    "蚀" => "&#x8680;",
    "蚁" => "&#x8681;",
    "蚂" => "&#x8682;",
    "蚃" => "&#x8683;",
    "蚄" => "&#x8684;",
    "蚅" => "&#x8685;",
    "蚆" => "&#x8686;",
    "蚇" => "&#x8687;",
    "蚈" => "&#x8688;",
    "蚉" => "&#x8689;",
    "蚊" => "&#x868A;",
    "蚋" => "&#x868B;",
    "蚌" => "&#x868C;",
    "蚍" => "&#x868D;",
    "蚎" => "&#x868E;",
    "蚏" => "&#x868F;",
    "蚐" => "&#x8690;",
    "蚑" => "&#x8691;",
    "蚒" => "&#x8692;",
    "蚓" => "&#x8693;",
    "蚔" => "&#x8694;",
    "蚕" => "&#x8695;",
    "蚖" => "&#x8696;",
    "蚗" => "&#x8697;",
    "蚘" => "&#x8698;",
    "蚙" => "&#x8699;",
    "蚚" => "&#x869A;",
    "蚛" => "&#x869B;",
    "蚜" => "&#x869C;",
    "蚝" => "&#x869D;",
    "蚞" => "&#x869E;",
    "蚟" => "&#x869F;",
    "蚠" => "&#x86A0;",
    "蚡" => "&#x86A1;",
    "蚢" => "&#x86A2;",
    "蚣" => "&#x86A3;",
    "蚤" => "&#x86A4;",
    "蚥" => "&#x86A5;",
    "蚦" => "&#x86A6;",
    "蚧" => "&#x86A7;",
    "蚨" => "&#x86A8;",
    "蚩" => "&#x86A9;",
    "蚪" => "&#x86AA;",
    "蚫" => "&#x86AB;",
    "蚬" => "&#x86AC;",
    "蚭" => "&#x86AD;",
    "蚮" => "&#x86AE;",
    "蚯" => "&#x86AF;",
    "蚰" => "&#x86B0;",
    "蚱" => "&#x86B1;",
    "蚲" => "&#x86B2;",
    "蚳" => "&#x86B3;",
    "蚴" => "&#x86B4;",
    "蚵" => "&#x86B5;",
    "蚶" => "&#x86B6;",
    "蚷" => "&#x86B7;",
    "蚸" => "&#x86B8;",
    "蚹" => "&#x86B9;",
    "蚺" => "&#x86BA;",
    "蚻" => "&#x86BB;",
    "蚼" => "&#x86BC;",
    "蚽" => "&#x86BD;",
    "蚾" => "&#x86BE;",
    "蚿" => "&#x86BF;",
    "蛀" => "&#x86C0;",
    "蛁" => "&#x86C1;",
    "蛂" => "&#x86C2;",
    "蛃" => "&#x86C3;",
    "蛄" => "&#x86C4;",
    "蛅" => "&#x86C5;",
    "蛆" => "&#x86C6;",
    "蛇" => "&#x86C7;",
    "蛈" => "&#x86C8;",
    "蛉" => "&#x86C9;",
    "蛊" => "&#x86CA;",
    "蛋" => "&#x86CB;",
    "蛌" => "&#x86CC;",
    "蛍" => "&#x86CD;",
    "蛎" => "&#x86CE;",
    "蛏" => "&#x86CF;",
    "蛐" => "&#x86D0;",
    "蛑" => "&#x86D1;",
    "蛒" => "&#x86D2;",
    "蛓" => "&#x86D3;",
    "蛔" => "&#x86D4;",
    "蛕" => "&#x86D5;",
    "蛖" => "&#x86D6;",
    "蛗" => "&#x86D7;",
    "蛘" => "&#x86D8;",
    "蛙" => "&#x86D9;",
    "蛚" => "&#x86DA;",
    "蛛" => "&#x86DB;",
    "蛜" => "&#x86DC;",
    "蛝" => "&#x86DD;",
    "蛞" => "&#x86DE;",
    "蛟" => "&#x86DF;",
    "蛠" => "&#x86E0;",
    "蛡" => "&#x86E1;",
    "蛢" => "&#x86E2;",
    "蛣" => "&#x86E3;",
    "蛤" => "&#x86E4;",
    "蛥" => "&#x86E5;",
    "蛦" => "&#x86E6;",
    "蛧" => "&#x86E7;",
    "蛨" => "&#x86E8;",
    "蛩" => "&#x86E9;",
    "蛪" => "&#x86EA;",
    "蛫" => "&#x86EB;",
    "蛬" => "&#x86EC;",
    "蛭" => "&#x86ED;",
    "蛮" => "&#x86EE;",
    "蛯" => "&#x86EF;",
    "蛰" => "&#x86F0;",
    "蛱" => "&#x86F1;",
    "蛲" => "&#x86F2;",
    "蛳" => "&#x86F3;",
    "蛴" => "&#x86F4;",
    "蛵" => "&#x86F5;",
    "蛶" => "&#x86F6;",
    "蛷" => "&#x86F7;",
    "蛸" => "&#x86F8;",
    "蛹" => "&#x86F9;",
    "蛺" => "&#x86FA;",
    "蛻" => "&#x86FB;",
    "蛼" => "&#x86FC;",
    "蛽" => "&#x86FD;",
    "蛾" => "&#x86FE;",
    "蛿" => "&#x86FF;",
    "蜀" => "&#x8700;",
    "蜁" => "&#x8701;",
    "蜂" => "&#x8702;",
    "蜃" => "&#x8703;",
    "蜄" => "&#x8704;",
    "蜅" => "&#x8705;",
    "蜆" => "&#x8706;",
    "蜇" => "&#x8707;",
    "蜈" => "&#x8708;",
    "蜉" => "&#x8709;",
    "蜊" => "&#x870A;",
    "蜋" => "&#x870B;",
    "蜌" => "&#x870C;",
    "蜍" => "&#x870D;",
    "蜎" => "&#x870E;",
    "蜏" => "&#x870F;",
    "蜐" => "&#x8710;",
    "蜑" => "&#x8711;",
    "蜒" => "&#x8712;",
    "蜓" => "&#x8713;",
    "蜔" => "&#x8714;",
    "蜕" => "&#x8715;",
    "蜖" => "&#x8716;",
    "蜗" => "&#x8717;",
    "蜘" => "&#x8718;",
    "蜙" => "&#x8719;",
    "蜚" => "&#x871A;",
    "蜛" => "&#x871B;",
    "蜜" => "&#x871C;",
    "蜝" => "&#x871D;",
    "蜞" => "&#x871E;",
    "蜟" => "&#x871F;",
    "蜠" => "&#x8720;",
    "蜡" => "&#x8721;",
    "蜢" => "&#x8722;",
    "蜣" => "&#x8723;",
    "蜤" => "&#x8724;",
    "蜥" => "&#x8725;",
    "蜦" => "&#x8726;",
    "蜧" => "&#x8727;",
    "蜨" => "&#x8728;",
    "蜩" => "&#x8729;",
    "蜪" => "&#x872A;",
    "蜫" => "&#x872B;",
    "蜬" => "&#x872C;",
    "蜭" => "&#x872D;",
    "蜮" => "&#x872E;",
    "蜯" => "&#x872F;",
    "蜰" => "&#x8730;",
    "蜱" => "&#x8731;",
    "蜲" => "&#x8732;",
    "蜳" => "&#x8733;",
    "蜴" => "&#x8734;",
    "蜵" => "&#x8735;",
    "蜶" => "&#x8736;",
    "蜷" => "&#x8737;",
    "蜸" => "&#x8738;",
    "蜹" => "&#x8739;",
    "蜺" => "&#x873A;",
    "蜻" => "&#x873B;",
    "蜼" => "&#x873C;",
    "蜽" => "&#x873D;",
    "蜾" => "&#x873E;",
    "蜿" => "&#x873F;",
    "蝀" => "&#x8740;",
    "蝁" => "&#x8741;",
    "蝂" => "&#x8742;",
    "蝃" => "&#x8743;",
    "蝄" => "&#x8744;",
    "蝅" => "&#x8745;",
    "蝆" => "&#x8746;",
    "蝇" => "&#x8747;",
    "蝈" => "&#x8748;",
    "蝉" => "&#x8749;",
    "蝊" => "&#x874A;",
    "蝋" => "&#x874B;",
    "蝌" => "&#x874C;",
    "蝍" => "&#x874D;",
    "蝎" => "&#x874E;",
    "蝏" => "&#x874F;",
    "蝐" => "&#x8750;",
    "蝑" => "&#x8751;",
    "蝒" => "&#x8752;",
    "蝓" => "&#x8753;",
    "蝔" => "&#x8754;",
    "蝕" => "&#x8755;",
    "蝖" => "&#x8756;",
    "蝗" => "&#x8757;",
    "蝘" => "&#x8758;",
    "蝙" => "&#x8759;",
    "蝚" => "&#x875A;",
    "蝛" => "&#x875B;",
    "蝜" => "&#x875C;",
    "蝝" => "&#x875D;",
    "蝞" => "&#x875E;",
    "蝟" => "&#x875F;",
    "蝠" => "&#x8760;",
    "蝡" => "&#x8761;",
    "蝢" => "&#x8762;",
    "蝣" => "&#x8763;",
    "蝤" => "&#x8764;",
    "蝥" => "&#x8765;",
    "蝦" => "&#x8766;",
    "蝧" => "&#x8767;",
    "蝨" => "&#x8768;",
    "蝩" => "&#x8769;",
    "蝪" => "&#x876A;",
    "蝫" => "&#x876B;",
    "蝬" => "&#x876C;",
    "蝭" => "&#x876D;",
    "蝮" => "&#x876E;",
    "蝯" => "&#x876F;",
    "蝰" => "&#x8770;",
    "蝱" => "&#x8771;",
    "蝲" => "&#x8772;",
    "蝳" => "&#x8773;",
    "蝴" => "&#x8774;",
    "蝵" => "&#x8775;",
    "蝶" => "&#x8776;",
    "蝷" => "&#x8777;",
    "蝸" => "&#x8778;",
    "蝹" => "&#x8779;",
    "蝺" => "&#x877A;",
    "蝻" => "&#x877B;",
    "蝼" => "&#x877C;",
    "蝽" => "&#x877D;",
    "蝾" => "&#x877E;",
    "蝿" => "&#x877F;",
    "螀" => "&#x8780;",
    "螁" => "&#x8781;",
    "螂" => "&#x8782;",
    "螃" => "&#x8783;",
    "螄" => "&#x8784;",
    "螅" => "&#x8785;",
    "螆" => "&#x8786;",
    "螇" => "&#x8787;",
    "螈" => "&#x8788;",
    "螉" => "&#x8789;",
    "螊" => "&#x878A;",
    "螋" => "&#x878B;",
    "螌" => "&#x878C;",
    "融" => "&#x878D;",
    "螎" => "&#x878E;",
    "螏" => "&#x878F;",
    "螐" => "&#x8790;",
    "螑" => "&#x8791;",
    "螒" => "&#x8792;",
    "螓" => "&#x8793;",
    "螔" => "&#x8794;",
    "螕" => "&#x8795;",
    "螖" => "&#x8796;",
    "螗" => "&#x8797;",
    "螘" => "&#x8798;",
    "螙" => "&#x8799;",
    "螚" => "&#x879A;",
    "螛" => "&#x879B;",
    "螜" => "&#x879C;",
    "螝" => "&#x879D;",
    "螞" => "&#x879E;",
    "螟" => "&#x879F;",
    "螠" => "&#x87A0;",
    "螡" => "&#x87A1;",
    "螢" => "&#x87A2;",
    "螣" => "&#x87A3;",
    "螤" => "&#x87A4;",
    "螥" => "&#x87A5;",
    "螦" => "&#x87A6;",
    "螧" => "&#x87A7;",
    "螨" => "&#x87A8;",
    "螩" => "&#x87A9;",
    "螪" => "&#x87AA;",
    "螫" => "&#x87AB;",
    "螬" => "&#x87AC;",
    "螭" => "&#x87AD;",
    "螮" => "&#x87AE;",
    "螯" => "&#x87AF;",
    "螰" => "&#x87B0;",
    "螱" => "&#x87B1;",
    "螲" => "&#x87B2;",
    "螳" => "&#x87B3;",
    "螴" => "&#x87B4;",
    "螵" => "&#x87B5;",
    "螶" => "&#x87B6;",
    "螷" => "&#x87B7;",
    "螸" => "&#x87B8;",
    "螹" => "&#x87B9;",
    "螺" => "&#x87BA;",
    "螻" => "&#x87BB;",
    "螼" => "&#x87BC;",
    "螽" => "&#x87BD;",
    "螾" => "&#x87BE;",
    "螿" => "&#x87BF;",
    "蟀" => "&#x87C0;",
    "蟁" => "&#x87C1;",
    "蟂" => "&#x87C2;",
    "蟃" => "&#x87C3;",
    "蟄" => "&#x87C4;",
    "蟅" => "&#x87C5;",
    "蟆" => "&#x87C6;",
    "蟇" => "&#x87C7;",
    "蟈" => "&#x87C8;",
    "蟉" => "&#x87C9;",
    "蟊" => "&#x87CA;",
    "蟋" => "&#x87CB;",
    "蟌" => "&#x87CC;",
    "蟍" => "&#x87CD;",
    "蟎" => "&#x87CE;",
    "蟏" => "&#x87CF;",
    "蟐" => "&#x87D0;",
    "蟑" => "&#x87D1;",
    "蟒" => "&#x87D2;",
    "蟓" => "&#x87D3;",
    "蟔" => "&#x87D4;",
    "蟕" => "&#x87D5;",
    "蟖" => "&#x87D6;",
    "蟗" => "&#x87D7;",
    "蟘" => "&#x87D8;",
    "蟙" => "&#x87D9;",
    "蟚" => "&#x87DA;",
    "蟛" => "&#x87DB;",
    "蟜" => "&#x87DC;",
    "蟝" => "&#x87DD;",
    "蟞" => "&#x87DE;",
    "蟟" => "&#x87DF;",
    "蟠" => "&#x87E0;",
    "蟡" => "&#x87E1;",
    "蟢" => "&#x87E2;",
    "蟣" => "&#x87E3;",
    "蟤" => "&#x87E4;",
    "蟥" => "&#x87E5;",
    "蟦" => "&#x87E6;",
    "蟧" => "&#x87E7;",
    "蟨" => "&#x87E8;",
    "蟩" => "&#x87E9;",
    "蟪" => "&#x87EA;",
    "蟫" => "&#x87EB;",
    "蟬" => "&#x87EC;",
    "蟭" => "&#x87ED;",
    "蟮" => "&#x87EE;",
    "蟯" => "&#x87EF;",
    "蟰" => "&#x87F0;",
    "蟱" => "&#x87F1;",
    "蟲" => "&#x87F2;",
    "蟳" => "&#x87F3;",
    "蟴" => "&#x87F4;",
    "蟵" => "&#x87F5;",
    "蟶" => "&#x87F6;",
    "蟷" => "&#x87F7;",
    "蟸" => "&#x87F8;",
    "蟹" => "&#x87F9;",
    "蟺" => "&#x87FA;",
    "蟻" => "&#x87FB;",
    "蟼" => "&#x87FC;",
    "蟽" => "&#x87FD;",
    "蟾" => "&#x87FE;",
    "蟿" => "&#x87FF;",
    "蠀" => "&#x8800;",
    "蠁" => "&#x8801;",
    "蠂" => "&#x8802;",
    "蠃" => "&#x8803;",
    "蠄" => "&#x8804;",
    "蠅" => "&#x8805;",
    "蠆" => "&#x8806;",
    "蠇" => "&#x8807;",
    "蠈" => "&#x8808;",
    "蠉" => "&#x8809;",
    "蠊" => "&#x880A;",
    "蠋" => "&#x880B;",
    "蠌" => "&#x880C;",
    "蠍" => "&#x880D;",
    "蠎" => "&#x880E;",
    "蠏" => "&#x880F;",
    "蠐" => "&#x8810;",
    "蠑" => "&#x8811;",
    "蠒" => "&#x8812;",
    "蠓" => "&#x8813;",
    "蠔" => "&#x8814;",
    "蠕" => "&#x8815;",
    "蠖" => "&#x8816;",
    "蠗" => "&#x8817;",
    "蠘" => "&#x8818;",
    "蠙" => "&#x8819;",
    "蠚" => "&#x881A;",
    "蠛" => "&#x881B;",
    "蠜" => "&#x881C;",
    "蠝" => "&#x881D;",
    "蠞" => "&#x881E;",
    "蠟" => "&#x881F;",
    "蠠" => "&#x8820;",
    "蠡" => "&#x8821;",
    "蠢" => "&#x8822;",
    "蠣" => "&#x8823;",
    "蠤" => "&#x8824;",
    "蠥" => "&#x8825;",
    "蠦" => "&#x8826;",
    "蠧" => "&#x8827;",
    "蠨" => "&#x8828;",
    "蠩" => "&#x8829;",
    "蠪" => "&#x882A;",
    "蠫" => "&#x882B;",
    "蠬" => "&#x882C;",
    "蠭" => "&#x882D;",
    "蠮" => "&#x882E;",
    "蠯" => "&#x882F;",
    "蠰" => "&#x8830;",
    "蠱" => "&#x8831;",
    "蠲" => "&#x8832;",
    "蠳" => "&#x8833;",
    "蠴" => "&#x8834;",
    "蠵" => "&#x8835;",
    "蠶" => "&#x8836;",
    "蠷" => "&#x8837;",
    "蠸" => "&#x8838;",
    "蠹" => "&#x8839;",
    "蠺" => "&#x883A;",
    "蠻" => "&#x883B;",
    "蠼" => "&#x883C;",
    "蠽" => "&#x883D;",
    "蠾" => "&#x883E;",
    "蠿" => "&#x883F;",
    "血" => "&#x8840;",
    "衁" => "&#x8841;",
    "衂" => "&#x8842;",
    "衃" => "&#x8843;",
    "衄" => "&#x8844;",
    "衅" => "&#x8845;",
    "衆" => "&#x8846;",
    "衇" => "&#x8847;",
    "衈" => "&#x8848;",
    "衉" => "&#x8849;",
    "衊" => "&#x884A;",
    "衋" => "&#x884B;",
    "行" => "&#x884C;",
    "衍" => "&#x884D;",
    "衎" => "&#x884E;",
    "衏" => "&#x884F;",
    "衐" => "&#x8850;",
    "衑" => "&#x8851;",
    "衒" => "&#x8852;",
    "術" => "&#x8853;",
    "衔" => "&#x8854;",
    "衕" => "&#x8855;",
    "衖" => "&#x8856;",
    "街" => "&#x8857;",
    "衘" => "&#x8858;",
    "衙" => "&#x8859;",
    "衚" => "&#x885A;",
    "衛" => "&#x885B;",
    "衜" => "&#x885C;",
    "衝" => "&#x885D;",
    "衞" => "&#x885E;",
    "衟" => "&#x885F;",
    "衠" => "&#x8860;",
    "衡" => "&#x8861;",
    "衢" => "&#x8862;",
    "衣" => "&#x8863;",
    "衤" => "&#x8864;",
    "补" => "&#x8865;",
    "衦" => "&#x8866;",
    "衧" => "&#x8867;",
    "表" => "&#x8868;",
    "衩" => "&#x8869;",
    "衪" => "&#x886A;",
    "衫" => "&#x886B;",
    "衬" => "&#x886C;",
    "衭" => "&#x886D;",
    "衮" => "&#x886E;",
    "衯" => "&#x886F;",
    "衰" => "&#x8870;",
    "衱" => "&#x8871;",
    "衲" => "&#x8872;",
    "衳" => "&#x8873;",
    "衴" => "&#x8874;",
    "衵" => "&#x8875;",
    "衶" => "&#x8876;",
    "衷" => "&#x8877;",
    "衸" => "&#x8878;",
    "衹" => "&#x8879;",
    "衺" => "&#x887A;",
    "衻" => "&#x887B;",
    "衼" => "&#x887C;",
    "衽" => "&#x887D;",
    "衾" => "&#x887E;",
    "衿" => "&#x887F;",
    "袀" => "&#x8880;",
    "袁" => "&#x8881;",
    "袂" => "&#x8882;",
    "袃" => "&#x8883;",
    "袄" => "&#x8884;",
    "袅" => "&#x8885;",
    "袆" => "&#x8886;",
    "袇" => "&#x8887;",
    "袈" => "&#x8888;",
    "袉" => "&#x8889;",
    "袊" => "&#x888A;",
    "袋" => "&#x888B;",
    "袌" => "&#x888C;",
    "袍" => "&#x888D;",
    "袎" => "&#x888E;",
    "袏" => "&#x888F;",
    "袐" => "&#x8890;",
    "袑" => "&#x8891;",
    "袒" => "&#x8892;",
    "袓" => "&#x8893;",
    "袔" => "&#x8894;",
    "袕" => "&#x8895;",
    "袖" => "&#x8896;",
    "袗" => "&#x8897;",
    "袘" => "&#x8898;",
    "袙" => "&#x8899;",
    "袚" => "&#x889A;",
    "袛" => "&#x889B;",
    "袜" => "&#x889C;",
    "袝" => "&#x889D;",
    "袞" => "&#x889E;",
    "袟" => "&#x889F;",
    "袠" => "&#x88A0;",
    "袡" => "&#x88A1;",
    "袢" => "&#x88A2;",
    "袣" => "&#x88A3;",
    "袤" => "&#x88A4;",
    "袥" => "&#x88A5;",
    "袦" => "&#x88A6;",
    "袧" => "&#x88A7;",
    "袨" => "&#x88A8;",
    "袩" => "&#x88A9;",
    "袪" => "&#x88AA;",
    "被" => "&#x88AB;",
    "袬" => "&#x88AC;",
    "袭" => "&#x88AD;",
    "袮" => "&#x88AE;",
    "袯" => "&#x88AF;",
    "袰" => "&#x88B0;",
    "袱" => "&#x88B1;",
    "袲" => "&#x88B2;",
    "袳" => "&#x88B3;",
    "袴" => "&#x88B4;",
    "袵" => "&#x88B5;",
    "袶" => "&#x88B6;",
    "袷" => "&#x88B7;",
    "袸" => "&#x88B8;",
    "袹" => "&#x88B9;",
    "袺" => "&#x88BA;",
    "袻" => "&#x88BB;",
    "袼" => "&#x88BC;",
    "袽" => "&#x88BD;",
    "袾" => "&#x88BE;",
    "袿" => "&#x88BF;",
    "裀" => "&#x88C0;",
    "裁" => "&#x88C1;",
    "裂" => "&#x88C2;",
    "裃" => "&#x88C3;",
    "裄" => "&#x88C4;",
    "装" => "&#x88C5;",
    "裆" => "&#x88C6;",
    "裇" => "&#x88C7;",
    "裈" => "&#x88C8;",
    "裉" => "&#x88C9;",
    "裊" => "&#x88CA;",
    "裋" => "&#x88CB;",
    "裌" => "&#x88CC;",
    "裍" => "&#x88CD;",
    "裎" => "&#x88CE;",
    "裏" => "&#x88CF;",
    "裐" => "&#x88D0;",
    "裑" => "&#x88D1;",
    "裒" => "&#x88D2;",
    "裓" => "&#x88D3;",
    "裔" => "&#x88D4;",
    "裕" => "&#x88D5;",
    "裖" => "&#x88D6;",
    "裗" => "&#x88D7;",
    "裘" => "&#x88D8;",
    "裙" => "&#x88D9;",
    "裚" => "&#x88DA;",
    "裛" => "&#x88DB;",
    "補" => "&#x88DC;",
    "裝" => "&#x88DD;",
    "裞" => "&#x88DE;",
    "裟" => "&#x88DF;",
    "裠" => "&#x88E0;",
    "裡" => "&#x88E1;",
    "裢" => "&#x88E2;",
    "裣" => "&#x88E3;",
    "裤" => "&#x88E4;",
    "裥" => "&#x88E5;",
    "裦" => "&#x88E6;",
    "裧" => "&#x88E7;",
    "裨" => "&#x88E8;",
    "裩" => "&#x88E9;",
    "裪" => "&#x88EA;",
    "裫" => "&#x88EB;",
    "裬" => "&#x88EC;",
    "裭" => "&#x88ED;",
    "裮" => "&#x88EE;",
    "裯" => "&#x88EF;",
    "裰" => "&#x88F0;",
    "裱" => "&#x88F1;",
    "裲" => "&#x88F2;",
    "裳" => "&#x88F3;",
    "裴" => "&#x88F4;",
    "裵" => "&#x88F5;",
    "裶" => "&#x88F6;",
    "裷" => "&#x88F7;",
    "裸" => "&#x88F8;",
    "裹" => "&#x88F9;",
    "裺" => "&#x88FA;",
    "裻" => "&#x88FB;",
    "裼" => "&#x88FC;",
    "製" => "&#x88FD;",
    "裾" => "&#x88FE;",
    "裿" => "&#x88FF;",
    "褀" => "&#x8900;",
    "褁" => "&#x8901;",
    "褂" => "&#x8902;",
    "褃" => "&#x8903;",
    "褄" => "&#x8904;",
    "褅" => "&#x8905;",
    "褆" => "&#x8906;",
    "複" => "&#x8907;",
    "褈" => "&#x8908;",
    "褉" => "&#x8909;",
    "褊" => "&#x890A;",
    "褋" => "&#x890B;",
    "褌" => "&#x890C;",
    "褍" => "&#x890D;",
    "褎" => "&#x890E;",
    "褏" => "&#x890F;",
    "褐" => "&#x8910;",
    "褑" => "&#x8911;",
    "褒" => "&#x8912;",
    "褓" => "&#x8913;",
    "褔" => "&#x8914;",
    "褕" => "&#x8915;",
    "褖" => "&#x8916;",
    "褗" => "&#x8917;",
    "褘" => "&#x8918;",
    "褙" => "&#x8919;",
    "褚" => "&#x891A;",
    "褛" => "&#x891B;",
    "褜" => "&#x891C;",
    "褝" => "&#x891D;",
    "褞" => "&#x891E;",
    "褟" => "&#x891F;",
    "褠" => "&#x8920;",
    "褡" => "&#x8921;",
    "褢" => "&#x8922;",
    "褣" => "&#x8923;",
    "褤" => "&#x8924;",
    "褥" => "&#x8925;",
    "褦" => "&#x8926;",
    "褧" => "&#x8927;",
    "褨" => "&#x8928;",
    "褩" => "&#x8929;",
    "褪" => "&#x892A;",
    "褫" => "&#x892B;",
    "褬" => "&#x892C;",
    "褭" => "&#x892D;",
    "褮" => "&#x892E;",
    "褯" => "&#x892F;",
    "褰" => "&#x8930;",
    "褱" => "&#x8931;",
    "褲" => "&#x8932;",
    "褳" => "&#x8933;",
    "褴" => "&#x8934;",
    "褵" => "&#x8935;",
    "褶" => "&#x8936;",
    "褷" => "&#x8937;",
    "褸" => "&#x8938;",
    "褹" => "&#x8939;",
    "褺" => "&#x893A;",
    "褻" => "&#x893B;",
    "褼" => "&#x893C;",
    "褽" => "&#x893D;",
    "褾" => "&#x893E;",
    "褿" => "&#x893F;",
    "襀" => "&#x8940;",
    "襁" => "&#x8941;",
    "襂" => "&#x8942;",
    "襃" => "&#x8943;",
    "襄" => "&#x8944;",
    "襅" => "&#x8945;",
    "襆" => "&#x8946;",
    "襇" => "&#x8947;",
    "襈" => "&#x8948;",
    "襉" => "&#x8949;",
    "襊" => "&#x894A;",
    "襋" => "&#x894B;",
    "襌" => "&#x894C;",
    "襍" => "&#x894D;",
    "襎" => "&#x894E;",
    "襏" => "&#x894F;",
    "襐" => "&#x8950;",
    "襑" => "&#x8951;",
    "襒" => "&#x8952;",
    "襓" => "&#x8953;",
    "襔" => "&#x8954;",
    "襕" => "&#x8955;",
    "襖" => "&#x8956;",
    "襗" => "&#x8957;",
    "襘" => "&#x8958;",
    "襙" => "&#x8959;",
    "襚" => "&#x895A;",
    "襛" => "&#x895B;",
    "襜" => "&#x895C;",
    "襝" => "&#x895D;",
    "襞" => "&#x895E;",
    "襟" => "&#x895F;",
    "襠" => "&#x8960;",
    "襡" => "&#x8961;",
    "襢" => "&#x8962;",
    "襣" => "&#x8963;",
    "襤" => "&#x8964;",
    "襥" => "&#x8965;",
    "襦" => "&#x8966;",
    "襧" => "&#x8967;",
    "襨" => "&#x8968;",
    "襩" => "&#x8969;",
    "襪" => "&#x896A;",
    "襫" => "&#x896B;",
    "襬" => "&#x896C;",
    "襭" => "&#x896D;",
    "襮" => "&#x896E;",
    "襯" => "&#x896F;",
    "襰" => "&#x8970;",
    "襱" => "&#x8971;",
    "襲" => "&#x8972;",
    "襳" => "&#x8973;",
    "襴" => "&#x8974;",
    "襵" => "&#x8975;",
    "襶" => "&#x8976;",
    "襷" => "&#x8977;",
    "襸" => "&#x8978;",
    "襹" => "&#x8979;",
    "襺" => "&#x897A;",
    "襻" => "&#x897B;",
    "襼" => "&#x897C;",
    "襽" => "&#x897D;",
    "襾" => "&#x897E;",
    "西" => "&#x897F;",
    "覀" => "&#x8980;",
    "要" => "&#x8981;",
    "覂" => "&#x8982;",
    "覃" => "&#x8983;",
    "覄" => "&#x8984;",
    "覅" => "&#x8985;",
    "覆" => "&#x8986;",
    "覇" => "&#x8987;",
    "覈" => "&#x8988;",
    "覉" => "&#x8989;",
    "覊" => "&#x898A;",
    "見" => "&#x898B;",
    "覌" => "&#x898C;",
    "覍" => "&#x898D;",
    "覎" => "&#x898E;",
    "規" => "&#x898F;",
    "覐" => "&#x8990;",
    "覑" => "&#x8991;",
    "覒" => "&#x8992;",
    "覓" => "&#x8993;",
    "覔" => "&#x8994;",
    "覕" => "&#x8995;",
    "視" => "&#x8996;",
    "覗" => "&#x8997;",
    "覘" => "&#x8998;",
    "覙" => "&#x8999;",
    "覚" => "&#x899A;",
    "覛" => "&#x899B;",
    "覜" => "&#x899C;",
    "覝" => "&#x899D;",
    "覞" => "&#x899E;",
    "覟" => "&#x899F;",
    "覠" => "&#x89A0;",
    "覡" => "&#x89A1;",
    "覢" => "&#x89A2;",
    "覣" => "&#x89A3;",
    "覤" => "&#x89A4;",
    "覥" => "&#x89A5;",
    "覦" => "&#x89A6;",
    "覧" => "&#x89A7;",
    "覨" => "&#x89A8;",
    "覩" => "&#x89A9;",
    "親" => "&#x89AA;",
    "覫" => "&#x89AB;",
    "覬" => "&#x89AC;",
    "覭" => "&#x89AD;",
    "覮" => "&#x89AE;",
    "覯" => "&#x89AF;",
    "覰" => "&#x89B0;",
    "覱" => "&#x89B1;",
    "覲" => "&#x89B2;",
    "観" => "&#x89B3;",
    "覴" => "&#x89B4;",
    "覵" => "&#x89B5;",
    "覶" => "&#x89B6;",
    "覷" => "&#x89B7;",
    "覸" => "&#x89B8;",
    "覹" => "&#x89B9;",
    "覺" => "&#x89BA;",
    "覻" => "&#x89BB;",
    "覼" => "&#x89BC;",
    "覽" => "&#x89BD;",
    "覾" => "&#x89BE;",
    "覿" => "&#x89BF;",
    "觀" => "&#x89C0;",
    "见" => "&#x89C1;",
    "观" => "&#x89C2;",
    "觃" => "&#x89C3;",
    "规" => "&#x89C4;",
    "觅" => "&#x89C5;",
    "视" => "&#x89C6;",
    "觇" => "&#x89C7;",
    "览" => "&#x89C8;",
    "觉" => "&#x89C9;",
    "觊" => "&#x89CA;",
    "觋" => "&#x89CB;",
    "觌" => "&#x89CC;",
    "觍" => "&#x89CD;",
    "觎" => "&#x89CE;",
    "觏" => "&#x89CF;",
    "觐" => "&#x89D0;",
    "觑" => "&#x89D1;",
    "角" => "&#x89D2;",
    "觓" => "&#x89D3;",
    "觔" => "&#x89D4;",
    "觕" => "&#x89D5;",
    "觖" => "&#x89D6;",
    "觗" => "&#x89D7;",
    "觘" => "&#x89D8;",
    "觙" => "&#x89D9;",
    "觚" => "&#x89DA;",
    "觛" => "&#x89DB;",
    "觜" => "&#x89DC;",
    "觝" => "&#x89DD;",
    "觞" => "&#x89DE;",
    "觟" => "&#x89DF;",
    "觠" => "&#x89E0;",
    "觡" => "&#x89E1;",
    "觢" => "&#x89E2;",
    "解" => "&#x89E3;",
    "觤" => "&#x89E4;",
    "觥" => "&#x89E5;",
    "触" => "&#x89E6;",
    "觧" => "&#x89E7;",
    "觨" => "&#x89E8;",
    "觩" => "&#x89E9;",
    "觪" => "&#x89EA;",
    "觫" => "&#x89EB;",
    "觬" => "&#x89EC;",
    "觭" => "&#x89ED;",
    "觮" => "&#x89EE;",
    "觯" => "&#x89EF;",
    "觰" => "&#x89F0;",
    "觱" => "&#x89F1;",
    "觲" => "&#x89F2;",
    "觳" => "&#x89F3;",
    "觴" => "&#x89F4;",
    "觵" => "&#x89F5;",
    "觶" => "&#x89F6;",
    "觷" => "&#x89F7;",
    "觸" => "&#x89F8;",
    "觹" => "&#x89F9;",
    "觺" => "&#x89FA;",
    "觻" => "&#x89FB;",
    "觼" => "&#x89FC;",
    "觽" => "&#x89FD;",
    "觾" => "&#x89FE;",
    "觿" => "&#x89FF;",
    "言" => "&#x8A00;",
    "訁" => "&#x8A01;",
    "訂" => "&#x8A02;",
    "訃" => "&#x8A03;",
    "訄" => "&#x8A04;",
    "訅" => "&#x8A05;",
    "訆" => "&#x8A06;",
    "訇" => "&#x8A07;",
    "計" => "&#x8A08;",
    "訉" => "&#x8A09;",
    "訊" => "&#x8A0A;",
    "訋" => "&#x8A0B;",
    "訌" => "&#x8A0C;",
    "訍" => "&#x8A0D;",
    "討" => "&#x8A0E;",
    "訏" => "&#x8A0F;",
    "訐" => "&#x8A10;",
    "訑" => "&#x8A11;",
    "訒" => "&#x8A12;",
    "訓" => "&#x8A13;",
    "訔" => "&#x8A14;",
    "訕" => "&#x8A15;",
    "訖" => "&#x8A16;",
    "託" => "&#x8A17;",
    "記" => "&#x8A18;",
    "訙" => "&#x8A19;",
    "訚" => "&#x8A1A;",
    "訛" => "&#x8A1B;",
    "訜" => "&#x8A1C;",
    "訝" => "&#x8A1D;",
    "訞" => "&#x8A1E;",
    "訟" => "&#x8A1F;",
    "訠" => "&#x8A20;",
    "訡" => "&#x8A21;",
    "訢" => "&#x8A22;",
    "訣" => "&#x8A23;",
    "訤" => "&#x8A24;",
    "訥" => "&#x8A25;",
    "訦" => "&#x8A26;",
    "訧" => "&#x8A27;",
    "訨" => "&#x8A28;",
    "訩" => "&#x8A29;",
    "訪" => "&#x8A2A;",
    "訫" => "&#x8A2B;",
    "訬" => "&#x8A2C;",
    "設" => "&#x8A2D;",
    "訮" => "&#x8A2E;",
    "訯" => "&#x8A2F;",
    "訰" => "&#x8A30;",
    "許" => "&#x8A31;",
    "訲" => "&#x8A32;",
    "訳" => "&#x8A33;",
    "訴" => "&#x8A34;",
    "訵" => "&#x8A35;",
    "訶" => "&#x8A36;",
    "訷" => "&#x8A37;",
    "訸" => "&#x8A38;",
    "訹" => "&#x8A39;",
    "診" => "&#x8A3A;",
    "註" => "&#x8A3B;",
    "証" => "&#x8A3C;",
    "訽" => "&#x8A3D;",
    "訾" => "&#x8A3E;",
    "訿" => "&#x8A3F;",
    "詀" => "&#x8A40;",
    "詁" => "&#x8A41;",
    "詂" => "&#x8A42;",
    "詃" => "&#x8A43;",
    "詄" => "&#x8A44;",
    "詅" => "&#x8A45;",
    "詆" => "&#x8A46;",
    "詇" => "&#x8A47;",
    "詈" => "&#x8A48;",
    "詉" => "&#x8A49;",
    "詊" => "&#x8A4A;",
    "詋" => "&#x8A4B;",
    "詌" => "&#x8A4C;",
    "詍" => "&#x8A4D;",
    "詎" => "&#x8A4E;",
    "詏" => "&#x8A4F;",
    "詐" => "&#x8A50;",
    "詑" => "&#x8A51;",
    "詒" => "&#x8A52;",
    "詓" => "&#x8A53;",
    "詔" => "&#x8A54;",
    "評" => "&#x8A55;",
    "詖" => "&#x8A56;",
    "詗" => "&#x8A57;",
    "詘" => "&#x8A58;",
    "詙" => "&#x8A59;",
    "詚" => "&#x8A5A;",
    "詛" => "&#x8A5B;",
    "詜" => "&#x8A5C;",
    "詝" => "&#x8A5D;",
    "詞" => "&#x8A5E;",
    "詟" => "&#x8A5F;",
    "詠" => "&#x8A60;",
    "詡" => "&#x8A61;",
    "詢" => "&#x8A62;",
    "詣" => "&#x8A63;",
    "詤" => "&#x8A64;",
    "詥" => "&#x8A65;",
    "試" => "&#x8A66;",
    "詧" => "&#x8A67;",
    "詨" => "&#x8A68;",
    "詩" => "&#x8A69;",
    "詪" => "&#x8A6A;",
    "詫" => "&#x8A6B;",
    "詬" => "&#x8A6C;",
    "詭" => "&#x8A6D;",
    "詮" => "&#x8A6E;",
    "詯" => "&#x8A6F;",
    "詰" => "&#x8A70;",
    "話" => "&#x8A71;",
    "該" => "&#x8A72;",
    "詳" => "&#x8A73;",
    "詴" => "&#x8A74;",
    "詵" => "&#x8A75;",
    "詶" => "&#x8A76;",
    "詷" => "&#x8A77;",
    "詸" => "&#x8A78;",
    "詹" => "&#x8A79;",
    "詺" => "&#x8A7A;",
    "詻" => "&#x8A7B;",
    "詼" => "&#x8A7C;",
    "詽" => "&#x8A7D;",
    "詾" => "&#x8A7E;",
    "詿" => "&#x8A7F;",
    "誀" => "&#x8A80;",
    "誁" => "&#x8A81;",
    "誂" => "&#x8A82;",
    "誃" => "&#x8A83;",
    "誄" => "&#x8A84;",
    "誅" => "&#x8A85;",
    "誆" => "&#x8A86;",
    "誇" => "&#x8A87;",
    "誈" => "&#x8A88;",
    "誉" => "&#x8A89;",
    "誊" => "&#x8A8A;",
    "誋" => "&#x8A8B;",
    "誌" => "&#x8A8C;",
    "認" => "&#x8A8D;",
    "誎" => "&#x8A8E;",
    "誏" => "&#x8A8F;",
    "誐" => "&#x8A90;",
    "誑" => "&#x8A91;",
    "誒" => "&#x8A92;",
    "誓" => "&#x8A93;",
    "誔" => "&#x8A94;",
    "誕" => "&#x8A95;",
    "誖" => "&#x8A96;",
    "誗" => "&#x8A97;",
    "誘" => "&#x8A98;",
    "誙" => "&#x8A99;",
    "誚" => "&#x8A9A;",
    "誛" => "&#x8A9B;",
    "誜" => "&#x8A9C;",
    "誝" => "&#x8A9D;",
    "語" => "&#x8A9E;",
    "誟" => "&#x8A9F;",
    "誠" => "&#x8AA0;",
    "誡" => "&#x8AA1;",
    "誢" => "&#x8AA2;",
    "誣" => "&#x8AA3;",
    "誤" => "&#x8AA4;",
    "誥" => "&#x8AA5;",
    "誦" => "&#x8AA6;",
    "誧" => "&#x8AA7;",
    "誨" => "&#x8AA8;",
    "誩" => "&#x8AA9;",
    "說" => "&#x8AAA;",
    "誫" => "&#x8AAB;",
    "説" => "&#x8AAC;",
    "読" => "&#x8AAD;",
    "誮" => "&#x8AAE;",
    "誯" => "&#x8AAF;",
    "誰" => "&#x8AB0;",
    "誱" => "&#x8AB1;",
    "課" => "&#x8AB2;",
    "誳" => "&#x8AB3;",
    "誴" => "&#x8AB4;",
    "誵" => "&#x8AB5;",
    "誶" => "&#x8AB6;",
    "誷" => "&#x8AB7;",
    "誸" => "&#x8AB8;",
    "誹" => "&#x8AB9;",
    "誺" => "&#x8ABA;",
    "誻" => "&#x8ABB;",
    "誼" => "&#x8ABC;",
    "誽" => "&#x8ABD;",
    "誾" => "&#x8ABE;",
    "調" => "&#x8ABF;",
    "諀" => "&#x8AC0;",
    "諁" => "&#x8AC1;",
    "諂" => "&#x8AC2;",
    "諃" => "&#x8AC3;",
    "諄" => "&#x8AC4;",
    "諅" => "&#x8AC5;",
    "諆" => "&#x8AC6;",
    "談" => "&#x8AC7;",
    "諈" => "&#x8AC8;",
    "諉" => "&#x8AC9;",
    "諊" => "&#x8ACA;",
    "請" => "&#x8ACB;",
    "諌" => "&#x8ACC;",
    "諍" => "&#x8ACD;",
    "諎" => "&#x8ACE;",
    "諏" => "&#x8ACF;",
    "諐" => "&#x8AD0;",
    "諑" => "&#x8AD1;",
    "諒" => "&#x8AD2;",
    "諓" => "&#x8AD3;",
    "諔" => "&#x8AD4;",
    "諕" => "&#x8AD5;",
    "論" => "&#x8AD6;",
    "諗" => "&#x8AD7;",
    "諘" => "&#x8AD8;",
    "諙" => "&#x8AD9;",
    "諚" => "&#x8ADA;",
    "諛" => "&#x8ADB;",
    "諜" => "&#x8ADC;",
    "諝" => "&#x8ADD;",
    "諞" => "&#x8ADE;",
    "諟" => "&#x8ADF;",
    "諠" => "&#x8AE0;",
    "諡" => "&#x8AE1;",
    "諢" => "&#x8AE2;",
    "諣" => "&#x8AE3;",
    "諤" => "&#x8AE4;",
    "諥" => "&#x8AE5;",
    "諦" => "&#x8AE6;",
    "諧" => "&#x8AE7;",
    "諨" => "&#x8AE8;",
    "諩" => "&#x8AE9;",
    "諪" => "&#x8AEA;",
    "諫" => "&#x8AEB;",
    "諬" => "&#x8AEC;",
    "諭" => "&#x8AED;",
    "諮" => "&#x8AEE;",
    "諯" => "&#x8AEF;",
    "諰" => "&#x8AF0;",
    "諱" => "&#x8AF1;",
    "諲" => "&#x8AF2;",
    "諳" => "&#x8AF3;",
    "諴" => "&#x8AF4;",
    "諵" => "&#x8AF5;",
    "諶" => "&#x8AF6;",
    "諷" => "&#x8AF7;",
    "諸" => "&#x8AF8;",
    "諹" => "&#x8AF9;",
    "諺" => "&#x8AFA;",
    "諻" => "&#x8AFB;",
    "諼" => "&#x8AFC;",
    "諽" => "&#x8AFD;",
    "諾" => "&#x8AFE;",
    "諿" => "&#x8AFF;",
    "謀" => "&#x8B00;",
    "謁" => "&#x8B01;",
    "謂" => "&#x8B02;",
    "謃" => "&#x8B03;",
    "謄" => "&#x8B04;",
    "謅" => "&#x8B05;",
    "謆" => "&#x8B06;",
    "謇" => "&#x8B07;",
    "謈" => "&#x8B08;",
    "謉" => "&#x8B09;",
    "謊" => "&#x8B0A;",
    "謋" => "&#x8B0B;",
    "謌" => "&#x8B0C;",
    "謍" => "&#x8B0D;",
    "謎" => "&#x8B0E;",
    "謏" => "&#x8B0F;",
    "謐" => "&#x8B10;",
    "謑" => "&#x8B11;",
    "謒" => "&#x8B12;",
    "謓" => "&#x8B13;",
    "謔" => "&#x8B14;",
    "謕" => "&#x8B15;",
    "謖" => "&#x8B16;",
    "謗" => "&#x8B17;",
    "謘" => "&#x8B18;",
    "謙" => "&#x8B19;",
    "謚" => "&#x8B1A;",
    "講" => "&#x8B1B;",
    "謜" => "&#x8B1C;",
    "謝" => "&#x8B1D;",
    "謞" => "&#x8B1E;",
    "謟" => "&#x8B1F;",
    "謠" => "&#x8B20;",
    "謡" => "&#x8B21;",
    "謢" => "&#x8B22;",
    "謣" => "&#x8B23;",
    "謤" => "&#x8B24;",
    "謥" => "&#x8B25;",
    "謦" => "&#x8B26;",
    "謧" => "&#x8B27;",
    "謨" => "&#x8B28;",
    "謩" => "&#x8B29;",
    "謪" => "&#x8B2A;",
    "謫" => "&#x8B2B;",
    "謬" => "&#x8B2C;",
    "謭" => "&#x8B2D;",
    "謮" => "&#x8B2E;",
    "謯" => "&#x8B2F;",
    "謰" => "&#x8B30;",
    "謱" => "&#x8B31;",
    "謲" => "&#x8B32;",
    "謳" => "&#x8B33;",
    "謴" => "&#x8B34;",
    "謵" => "&#x8B35;",
    "謶" => "&#x8B36;",
    "謷" => "&#x8B37;",
    "謸" => "&#x8B38;",
    "謹" => "&#x8B39;",
    "謺" => "&#x8B3A;",
    "謻" => "&#x8B3B;",
    "謼" => "&#x8B3C;",
    "謽" => "&#x8B3D;",
    "謾" => "&#x8B3E;",
    "謿" => "&#x8B3F;",
    "譀" => "&#x8B40;",
    "譁" => "&#x8B41;",
    "譂" => "&#x8B42;",
    "譃" => "&#x8B43;",
    "譄" => "&#x8B44;",
    "譅" => "&#x8B45;",
    "譆" => "&#x8B46;",
    "譇" => "&#x8B47;",
    "譈" => "&#x8B48;",
    "證" => "&#x8B49;",
    "譊" => "&#x8B4A;",
    "譋" => "&#x8B4B;",
    "譌" => "&#x8B4C;",
    "譍" => "&#x8B4D;",
    "譎" => "&#x8B4E;",
    "譏" => "&#x8B4F;",
    "譐" => "&#x8B50;",
    "譑" => "&#x8B51;",
    "譒" => "&#x8B52;",
    "譓" => "&#x8B53;",
    "譔" => "&#x8B54;",
    "譕" => "&#x8B55;",
    "譖" => "&#x8B56;",
    "譗" => "&#x8B57;",
    "識" => "&#x8B58;",
    "譙" => "&#x8B59;",
    "譚" => "&#x8B5A;",
    "譛" => "&#x8B5B;",
    "譜" => "&#x8B5C;",
    "譝" => "&#x8B5D;",
    "譞" => "&#x8B5E;",
    "譟" => "&#x8B5F;",
    "譠" => "&#x8B60;",
    "譡" => "&#x8B61;",
    "譢" => "&#x8B62;",
    "譣" => "&#x8B63;",
    "譤" => "&#x8B64;",
    "譥" => "&#x8B65;",
    "警" => "&#x8B66;",
    "譧" => "&#x8B67;",
    "譨" => "&#x8B68;",
    "譩" => "&#x8B69;",
    "譪" => "&#x8B6A;",
    "譫" => "&#x8B6B;",
    "譬" => "&#x8B6C;",
    "譭" => "&#x8B6D;",
    "譮" => "&#x8B6E;",
    "譯" => "&#x8B6F;",
    "議" => "&#x8B70;",
    "譱" => "&#x8B71;",
    "譲" => "&#x8B72;",
    "譳" => "&#x8B73;",
    "譴" => "&#x8B74;",
    "譵" => "&#x8B75;",
    "譶" => "&#x8B76;",
    "護" => "&#x8B77;",
    "譸" => "&#x8B78;",
    "譹" => "&#x8B79;",
    "譺" => "&#x8B7A;",
    "譻" => "&#x8B7B;",
    "譼" => "&#x8B7C;",
    "譽" => "&#x8B7D;",
    "譾" => "&#x8B7E;",
    "譿" => "&#x8B7F;",
    "讀" => "&#x8B80;",
    "讁" => "&#x8B81;",
    "讂" => "&#x8B82;",
    "讃" => "&#x8B83;",
    "讄" => "&#x8B84;",
    "讅" => "&#x8B85;",
    "讆" => "&#x8B86;",
    "讇" => "&#x8B87;",
    "讈" => "&#x8B88;",
    "讉" => "&#x8B89;",
    "變" => "&#x8B8A;",
    "讋" => "&#x8B8B;",
    "讌" => "&#x8B8C;",
    "讍" => "&#x8B8D;",
    "讎" => "&#x8B8E;",
    "讏" => "&#x8B8F;",
    "讐" => "&#x8B90;",
    "讑" => "&#x8B91;",
    "讒" => "&#x8B92;",
    "讓" => "&#x8B93;",
    "讔" => "&#x8B94;",
    "讕" => "&#x8B95;",
    "讖" => "&#x8B96;",
    "讗" => "&#x8B97;",
    "讘" => "&#x8B98;",
    "讙" => "&#x8B99;",
    "讚" => "&#x8B9A;",
    "讛" => "&#x8B9B;",
    "讜" => "&#x8B9C;",
    "讝" => "&#x8B9D;",
    "讞" => "&#x8B9E;",
    "讟" => "&#x8B9F;",
    "讠" => "&#x8BA0;",
    "计" => "&#x8BA1;",
    "订" => "&#x8BA2;",
    "讣" => "&#x8BA3;",
    "认" => "&#x8BA4;",
    "讥" => "&#x8BA5;",
    "讦" => "&#x8BA6;",
    "讧" => "&#x8BA7;",
    "讨" => "&#x8BA8;",
    "让" => "&#x8BA9;",
    "讪" => "&#x8BAA;",
    "讫" => "&#x8BAB;",
    "讬" => "&#x8BAC;",
    "训" => "&#x8BAD;",
    "议" => "&#x8BAE;",
    "讯" => "&#x8BAF;",
    "记" => "&#x8BB0;",
    "讱" => "&#x8BB1;",
    "讲" => "&#x8BB2;",
    "讳" => "&#x8BB3;",
    "讴" => "&#x8BB4;",
    "讵" => "&#x8BB5;",
    "讶" => "&#x8BB6;",
    "讷" => "&#x8BB7;",
    "许" => "&#x8BB8;",
    "讹" => "&#x8BB9;",
    "论" => "&#x8BBA;",
    "讻" => "&#x8BBB;",
    "讼" => "&#x8BBC;",
    "讽" => "&#x8BBD;",
    "设" => "&#x8BBE;",
    "访" => "&#x8BBF;",
    "诀" => "&#x8BC0;",
    "证" => "&#x8BC1;",
    "诂" => "&#x8BC2;",
    "诃" => "&#x8BC3;",
    "评" => "&#x8BC4;",
    "诅" => "&#x8BC5;",
    "识" => "&#x8BC6;",
    "诇" => "&#x8BC7;",
    "诈" => "&#x8BC8;",
    "诉" => "&#x8BC9;",
    "诊" => "&#x8BCA;",
    "诋" => "&#x8BCB;",
    "诌" => "&#x8BCC;",
    "词" => "&#x8BCD;",
    "诎" => "&#x8BCE;",
    "诏" => "&#x8BCF;",
    "诐" => "&#x8BD0;",
    "译" => "&#x8BD1;",
    "诒" => "&#x8BD2;",
    "诓" => "&#x8BD3;",
    "诔" => "&#x8BD4;",
    "试" => "&#x8BD5;",
    "诖" => "&#x8BD6;",
    "诗" => "&#x8BD7;",
    "诘" => "&#x8BD8;",
    "诙" => "&#x8BD9;",
    "诚" => "&#x8BDA;",
    "诛" => "&#x8BDB;",
    "诜" => "&#x8BDC;",
    "话" => "&#x8BDD;",
    "诞" => "&#x8BDE;",
    "诟" => "&#x8BDF;",
    "诠" => "&#x8BE0;",
    "诡" => "&#x8BE1;",
    "询" => "&#x8BE2;",
    "诣" => "&#x8BE3;",
    "诤" => "&#x8BE4;",
    "该" => "&#x8BE5;",
    "详" => "&#x8BE6;",
    "诧" => "&#x8BE7;",
    "诨" => "&#x8BE8;",
    "诩" => "&#x8BE9;",
    "诪" => "&#x8BEA;",
    "诫" => "&#x8BEB;",
    "诬" => "&#x8BEC;",
    "语" => "&#x8BED;",
    "诮" => "&#x8BEE;",
    "误" => "&#x8BEF;",
    "诰" => "&#x8BF0;",
    "诱" => "&#x8BF1;",
    "诲" => "&#x8BF2;",
    "诳" => "&#x8BF3;",
    "说" => "&#x8BF4;",
    "诵" => "&#x8BF5;",
    "诶" => "&#x8BF6;",
    "请" => "&#x8BF7;",
    "诸" => "&#x8BF8;",
    "诹" => "&#x8BF9;",
    "诺" => "&#x8BFA;",
    "读" => "&#x8BFB;",
    "诼" => "&#x8BFC;",
    "诽" => "&#x8BFD;",
    "课" => "&#x8BFE;",
    "诿" => "&#x8BFF;",
    "谀" => "&#x8C00;",
    "谁" => "&#x8C01;",
    "谂" => "&#x8C02;",
    "调" => "&#x8C03;",
    "谄" => "&#x8C04;",
    "谅" => "&#x8C05;",
    "谆" => "&#x8C06;",
    "谇" => "&#x8C07;",
    "谈" => "&#x8C08;",
    "谉" => "&#x8C09;",
    "谊" => "&#x8C0A;",
    "谋" => "&#x8C0B;",
    "谌" => "&#x8C0C;",
    "谍" => "&#x8C0D;",
    "谎" => "&#x8C0E;",
    "谏" => "&#x8C0F;",
    "谐" => "&#x8C10;",
    "谑" => "&#x8C11;",
    "谒" => "&#x8C12;",
    "谓" => "&#x8C13;",
    "谔" => "&#x8C14;",
    "谕" => "&#x8C15;",
    "谖" => "&#x8C16;",
    "谗" => "&#x8C17;",
    "谘" => "&#x8C18;",
    "谙" => "&#x8C19;",
    "谚" => "&#x8C1A;",
    "谛" => "&#x8C1B;",
    "谜" => "&#x8C1C;",
    "谝" => "&#x8C1D;",
    "谞" => "&#x8C1E;",
    "谟" => "&#x8C1F;",
    "谠" => "&#x8C20;",
    "谡" => "&#x8C21;",
    "谢" => "&#x8C22;",
    "谣" => "&#x8C23;",
    "谤" => "&#x8C24;",
    "谥" => "&#x8C25;",
    "谦" => "&#x8C26;",
    "谧" => "&#x8C27;",
    "谨" => "&#x8C28;",
    "谩" => "&#x8C29;",
    "谪" => "&#x8C2A;",
    "谫" => "&#x8C2B;",
    "谬" => "&#x8C2C;",
    "谭" => "&#x8C2D;",
    "谮" => "&#x8C2E;",
    "谯" => "&#x8C2F;",
    "谰" => "&#x8C30;",
    "谱" => "&#x8C31;",
    "谲" => "&#x8C32;",
    "谳" => "&#x8C33;",
    "谴" => "&#x8C34;",
    "谵" => "&#x8C35;",
    "谶" => "&#x8C36;",
    "谷" => "&#x8C37;",
    "谸" => "&#x8C38;",
    "谹" => "&#x8C39;",
    "谺" => "&#x8C3A;",
    "谻" => "&#x8C3B;",
    "谼" => "&#x8C3C;",
    "谽" => "&#x8C3D;",
    "谾" => "&#x8C3E;",
    "谿" => "&#x8C3F;",
    "豀" => "&#x8C40;",
    "豁" => "&#x8C41;",
    "豂" => "&#x8C42;",
    "豃" => "&#x8C43;",
    "豄" => "&#x8C44;",
    "豅" => "&#x8C45;",
    "豆" => "&#x8C46;",
    "豇" => "&#x8C47;",
    "豈" => "&#x8C48;",
    "豉" => "&#x8C49;",
    "豊" => "&#x8C4A;",
    "豋" => "&#x8C4B;",
    "豌" => "&#x8C4C;",
    "豍" => "&#x8C4D;",
    "豎" => "&#x8C4E;",
    "豏" => "&#x8C4F;",
    "豐" => "&#x8C50;",
    "豑" => "&#x8C51;",
    "豒" => "&#x8C52;",
    "豓" => "&#x8C53;",
    "豔" => "&#x8C54;",
    "豕" => "&#x8C55;",
    "豖" => "&#x8C56;",
    "豗" => "&#x8C57;",
    "豘" => "&#x8C58;",
    "豙" => "&#x8C59;",
    "豚" => "&#x8C5A;",
    "豛" => "&#x8C5B;",
    "豜" => "&#x8C5C;",
    "豝" => "&#x8C5D;",
    "豞" => "&#x8C5E;",
    "豟" => "&#x8C5F;",
    "豠" => "&#x8C60;",
    "象" => "&#x8C61;",
    "豢" => "&#x8C62;",
    "豣" => "&#x8C63;",
    "豤" => "&#x8C64;",
    "豥" => "&#x8C65;",
    "豦" => "&#x8C66;",
    "豧" => "&#x8C67;",
    "豨" => "&#x8C68;",
    "豩" => "&#x8C69;",
    "豪" => "&#x8C6A;",
    "豫" => "&#x8C6B;",
    "豬" => "&#x8C6C;",
    "豭" => "&#x8C6D;",
    "豮" => "&#x8C6E;",
    "豯" => "&#x8C6F;",
    "豰" => "&#x8C70;",
    "豱" => "&#x8C71;",
    "豲" => "&#x8C72;",
    "豳" => "&#x8C73;",
    "豴" => "&#x8C74;",
    "豵" => "&#x8C75;",
    "豶" => "&#x8C76;",
    "豷" => "&#x8C77;",
    "豸" => "&#x8C78;",
    "豹" => "&#x8C79;",
    "豺" => "&#x8C7A;",
    "豻" => "&#x8C7B;",
    "豼" => "&#x8C7C;",
    "豽" => "&#x8C7D;",
    "豾" => "&#x8C7E;",
    "豿" => "&#x8C7F;",
    "貀" => "&#x8C80;",
    "貁" => "&#x8C81;",
    "貂" => "&#x8C82;",
    "貃" => "&#x8C83;",
    "貄" => "&#x8C84;",
    "貅" => "&#x8C85;",
    "貆" => "&#x8C86;",
    "貇" => "&#x8C87;",
    "貈" => "&#x8C88;",
    "貉" => "&#x8C89;",
    "貊" => "&#x8C8A;",
    "貋" => "&#x8C8B;",
    "貌" => "&#x8C8C;",
    "貍" => "&#x8C8D;",
    "貎" => "&#x8C8E;",
    "貏" => "&#x8C8F;",
    "貐" => "&#x8C90;",
    "貑" => "&#x8C91;",
    "貒" => "&#x8C92;",
    "貓" => "&#x8C93;",
    "貔" => "&#x8C94;",
    "貕" => "&#x8C95;",
    "貖" => "&#x8C96;",
    "貗" => "&#x8C97;",
    "貘" => "&#x8C98;",
    "貙" => "&#x8C99;",
    "貚" => "&#x8C9A;",
    "貛" => "&#x8C9B;",
    "貜" => "&#x8C9C;",
    "貝" => "&#x8C9D;",
    "貞" => "&#x8C9E;",
    "貟" => "&#x8C9F;",
    "負" => "&#x8CA0;",
    "財" => "&#x8CA1;",
    "貢" => "&#x8CA2;",
    "貣" => "&#x8CA3;",
    "貤" => "&#x8CA4;",
    "貥" => "&#x8CA5;",
    "貦" => "&#x8CA6;",
    "貧" => "&#x8CA7;",
    "貨" => "&#x8CA8;",
    "販" => "&#x8CA9;",
    "貪" => "&#x8CAA;",
    "貫" => "&#x8CAB;",
    "責" => "&#x8CAC;",
    "貭" => "&#x8CAD;",
    "貮" => "&#x8CAE;",
    "貯" => "&#x8CAF;",
    "貰" => "&#x8CB0;",
    "貱" => "&#x8CB1;",
    "貲" => "&#x8CB2;",
    "貳" => "&#x8CB3;",
    "貴" => "&#x8CB4;",
    "貵" => "&#x8CB5;",
    "貶" => "&#x8CB6;",
    "買" => "&#x8CB7;",
    "貸" => "&#x8CB8;",
    "貹" => "&#x8CB9;",
    "貺" => "&#x8CBA;",
    "費" => "&#x8CBB;",
    "貼" => "&#x8CBC;",
    "貽" => "&#x8CBD;",
    "貾" => "&#x8CBE;",
    "貿" => "&#x8CBF;",
    "賀" => "&#x8CC0;",
    "賁" => "&#x8CC1;",
    "賂" => "&#x8CC2;",
    "賃" => "&#x8CC3;",
    "賄" => "&#x8CC4;",
    "賅" => "&#x8CC5;",
    "賆" => "&#x8CC6;",
    "資" => "&#x8CC7;",
    "賈" => "&#x8CC8;",
    "賉" => "&#x8CC9;",
    "賊" => "&#x8CCA;",
    "賋" => "&#x8CCB;",
    "賌" => "&#x8CCC;",
    "賍" => "&#x8CCD;",
    "賎" => "&#x8CCE;",
    "賏" => "&#x8CCF;",
    "賐" => "&#x8CD0;",
    "賑" => "&#x8CD1;",
    "賒" => "&#x8CD2;",
    "賓" => "&#x8CD3;",
    "賔" => "&#x8CD4;",
    "賕" => "&#x8CD5;",
    "賖" => "&#x8CD6;",
    "賗" => "&#x8CD7;",
    "賘" => "&#x8CD8;",
    "賙" => "&#x8CD9;",
    "賚" => "&#x8CDA;",
    "賛" => "&#x8CDB;",
    "賜" => "&#x8CDC;",
    "賝" => "&#x8CDD;",
    "賞" => "&#x8CDE;",
    "賟" => "&#x8CDF;",
    "賠" => "&#x8CE0;",
    "賡" => "&#x8CE1;",
    "賢" => "&#x8CE2;",
    "賣" => "&#x8CE3;",
    "賤" => "&#x8CE4;",
    "賥" => "&#x8CE5;",
    "賦" => "&#x8CE6;",
    "賧" => "&#x8CE7;",
    "賨" => "&#x8CE8;",
    "賩" => "&#x8CE9;",
    "質" => "&#x8CEA;",
    "賫" => "&#x8CEB;",
    "賬" => "&#x8CEC;",
    "賭" => "&#x8CED;",
    "賮" => "&#x8CEE;",
    "賯" => "&#x8CEF;",
    "賰" => "&#x8CF0;",
    "賱" => "&#x8CF1;",
    "賲" => "&#x8CF2;",
    "賳" => "&#x8CF3;",
    "賴" => "&#x8CF4;",
    "賵" => "&#x8CF5;",
    "賶" => "&#x8CF6;",
    "賷" => "&#x8CF7;",
    "賸" => "&#x8CF8;",
    "賹" => "&#x8CF9;",
    "賺" => "&#x8CFA;",
    "賻" => "&#x8CFB;",
    "購" => "&#x8CFC;",
    "賽" => "&#x8CFD;",
    "賾" => "&#x8CFE;",
    "賿" => "&#x8CFF;",
    "贀" => "&#x8D00;",
    "贁" => "&#x8D01;",
    "贂" => "&#x8D02;",
    "贃" => "&#x8D03;",
    "贄" => "&#x8D04;",
    "贅" => "&#x8D05;",
    "贆" => "&#x8D06;",
    "贇" => "&#x8D07;",
    "贈" => "&#x8D08;",
    "贉" => "&#x8D09;",
    "贊" => "&#x8D0A;",
    "贋" => "&#x8D0B;",
    "贌" => "&#x8D0C;",
    "贍" => "&#x8D0D;",
    "贎" => "&#x8D0E;",
    "贏" => "&#x8D0F;",
    "贐" => "&#x8D10;",
    "贑" => "&#x8D11;",
    "贒" => "&#x8D12;",
    "贓" => "&#x8D13;",
    "贔" => "&#x8D14;",
    "贕" => "&#x8D15;",
    "贖" => "&#x8D16;",
    "贗" => "&#x8D17;",
    "贘" => "&#x8D18;",
    "贙" => "&#x8D19;",
    "贚" => "&#x8D1A;",
    "贛" => "&#x8D1B;",
    "贜" => "&#x8D1C;",
    "贝" => "&#x8D1D;",
    "贞" => "&#x8D1E;",
    "负" => "&#x8D1F;",
    "贠" => "&#x8D20;",
    "贡" => "&#x8D21;",
    "财" => "&#x8D22;",
    "责" => "&#x8D23;",
    "贤" => "&#x8D24;",
    "败" => "&#x8D25;",
    "账" => "&#x8D26;",
    "货" => "&#x8D27;",
    "质" => "&#x8D28;",
    "贩" => "&#x8D29;",
    "贪" => "&#x8D2A;",
    "贫" => "&#x8D2B;",
    "贬" => "&#x8D2C;",
    "购" => "&#x8D2D;",
    "贮" => "&#x8D2E;",
    "贯" => "&#x8D2F;",
    "贰" => "&#x8D30;",
    "贱" => "&#x8D31;",
    "贲" => "&#x8D32;",
    "贳" => "&#x8D33;",
    "贴" => "&#x8D34;",
    "贵" => "&#x8D35;",
    "贶" => "&#x8D36;",
    "贷" => "&#x8D37;",
    "贸" => "&#x8D38;",
    "费" => "&#x8D39;",
    "贺" => "&#x8D3A;",
    "贻" => "&#x8D3B;",
    "贼" => "&#x8D3C;",
    "贽" => "&#x8D3D;",
    "贾" => "&#x8D3E;",
    "贿" => "&#x8D3F;",
    "赀" => "&#x8D40;",
    "赁" => "&#x8D41;",
    "赂" => "&#x8D42;",
    "赃" => "&#x8D43;",
    "资" => "&#x8D44;",
    "赅" => "&#x8D45;",
    "赆" => "&#x8D46;",
    "赇" => "&#x8D47;",
    "赈" => "&#x8D48;",
    "赉" => "&#x8D49;",
    "赊" => "&#x8D4A;",
    "赋" => "&#x8D4B;",
    "赌" => "&#x8D4C;",
    "赍" => "&#x8D4D;",
    "赎" => "&#x8D4E;",
    "赏" => "&#x8D4F;",
    "赐" => "&#x8D50;",
    "赑" => "&#x8D51;",
    "赒" => "&#x8D52;",
    "赓" => "&#x8D53;",
    "赔" => "&#x8D54;",
    "赕" => "&#x8D55;",
    "赖" => "&#x8D56;",
    "赗" => "&#x8D57;",
    "赘" => "&#x8D58;",
    "赙" => "&#x8D59;",
    "赚" => "&#x8D5A;",
    "赛" => "&#x8D5B;",
    "赜" => "&#x8D5C;",
    "赝" => "&#x8D5D;",
    "赞" => "&#x8D5E;",
    "赟" => "&#x8D5F;",
    "赠" => "&#x8D60;",
    "赡" => "&#x8D61;",
    "赢" => "&#x8D62;",
    "赣" => "&#x8D63;",
    "赤" => "&#x8D64;",
    "赥" => "&#x8D65;",
    "赦" => "&#x8D66;",
    "赧" => "&#x8D67;",
    "赨" => "&#x8D68;",
    "赩" => "&#x8D69;",
    "赪" => "&#x8D6A;",
    "赫" => "&#x8D6B;",
    "赬" => "&#x8D6C;",
    "赭" => "&#x8D6D;",
    "赮" => "&#x8D6E;",
    "赯" => "&#x8D6F;",
    "走" => "&#x8D70;",
    "赱" => "&#x8D71;",
    "赲" => "&#x8D72;",
    "赳" => "&#x8D73;",
    "赴" => "&#x8D74;",
    "赵" => "&#x8D75;",
    "赶" => "&#x8D76;",
    "起" => "&#x8D77;",
    "赸" => "&#x8D78;",
    "赹" => "&#x8D79;",
    "赺" => "&#x8D7A;",
    "赻" => "&#x8D7B;",
    "赼" => "&#x8D7C;",
    "赽" => "&#x8D7D;",
    "赾" => "&#x8D7E;",
    "赿" => "&#x8D7F;",
    "趀" => "&#x8D80;",
    "趁" => "&#x8D81;",
    "趂" => "&#x8D82;",
    "趃" => "&#x8D83;",
    "趄" => "&#x8D84;",
    "超" => "&#x8D85;",
    "趆" => "&#x8D86;",
    "趇" => "&#x8D87;",
    "趈" => "&#x8D88;",
    "趉" => "&#x8D89;",
    "越" => "&#x8D8A;",
    "趋" => "&#x8D8B;",
    "趌" => "&#x8D8C;",
    "趍" => "&#x8D8D;",
    "趎" => "&#x8D8E;",
    "趏" => "&#x8D8F;",
    "趐" => "&#x8D90;",
    "趑" => "&#x8D91;",
    "趒" => "&#x8D92;",
    "趓" => "&#x8D93;",
    "趔" => "&#x8D94;",
    "趕" => "&#x8D95;",
    "趖" => "&#x8D96;",
    "趗" => "&#x8D97;",
    "趘" => "&#x8D98;",
    "趙" => "&#x8D99;",
    "趚" => "&#x8D9A;",
    "趛" => "&#x8D9B;",
    "趜" => "&#x8D9C;",
    "趝" => "&#x8D9D;",
    "趞" => "&#x8D9E;",
    "趟" => "&#x8D9F;",
    "趠" => "&#x8DA0;",
    "趡" => "&#x8DA1;",
    "趢" => "&#x8DA2;",
    "趣" => "&#x8DA3;",
    "趤" => "&#x8DA4;",
    "趥" => "&#x8DA5;",
    "趦" => "&#x8DA6;",
    "趧" => "&#x8DA7;",
    "趨" => "&#x8DA8;",
    "趩" => "&#x8DA9;",
    "趪" => "&#x8DAA;",
    "趫" => "&#x8DAB;",
    "趬" => "&#x8DAC;",
    "趭" => "&#x8DAD;",
    "趮" => "&#x8DAE;",
    "趯" => "&#x8DAF;",
    "趰" => "&#x8DB0;",
    "趱" => "&#x8DB1;",
    "趲" => "&#x8DB2;",
    "足" => "&#x8DB3;",
    "趴" => "&#x8DB4;",
    "趵" => "&#x8DB5;",
    "趶" => "&#x8DB6;",
    "趷" => "&#x8DB7;",
    "趸" => "&#x8DB8;",
    "趹" => "&#x8DB9;",
    "趺" => "&#x8DBA;",
    "趻" => "&#x8DBB;",
    "趼" => "&#x8DBC;",
    "趽" => "&#x8DBD;",
    "趾" => "&#x8DBE;",
    "趿" => "&#x8DBF;",
    "跀" => "&#x8DC0;",
    "跁" => "&#x8DC1;",
    "跂" => "&#x8DC2;",
    "跃" => "&#x8DC3;",
    "跄" => "&#x8DC4;",
    "跅" => "&#x8DC5;",
    "跆" => "&#x8DC6;",
    "跇" => "&#x8DC7;",
    "跈" => "&#x8DC8;",
    "跉" => "&#x8DC9;",
    "跊" => "&#x8DCA;",
    "跋" => "&#x8DCB;",
    "跌" => "&#x8DCC;",
    "跍" => "&#x8DCD;",
    "跎" => "&#x8DCE;",
    "跏" => "&#x8DCF;",
    "跐" => "&#x8DD0;",
    "跑" => "&#x8DD1;",
    "跒" => "&#x8DD2;",
    "跓" => "&#x8DD3;",
    "跔" => "&#x8DD4;",
    "跕" => "&#x8DD5;",
    "跖" => "&#x8DD6;",
    "跗" => "&#x8DD7;",
    "跘" => "&#x8DD8;",
    "跙" => "&#x8DD9;",
    "跚" => "&#x8DDA;",
    "跛" => "&#x8DDB;",
    "跜" => "&#x8DDC;",
    "距" => "&#x8DDD;",
    "跞" => "&#x8DDE;",
    "跟" => "&#x8DDF;",
    "跠" => "&#x8DE0;",
    "跡" => "&#x8DE1;",
    "跢" => "&#x8DE2;",
    "跣" => "&#x8DE3;",
    "跤" => "&#x8DE4;",
    "跥" => "&#x8DE5;",
    "跦" => "&#x8DE6;",
    "跧" => "&#x8DE7;",
    "跨" => "&#x8DE8;",
    "跩" => "&#x8DE9;",
    "跪" => "&#x8DEA;",
    "跫" => "&#x8DEB;",
    "跬" => "&#x8DEC;",
    "跭" => "&#x8DED;",
    "跮" => "&#x8DEE;",
    "路" => "&#x8DEF;",
    "跰" => "&#x8DF0;",
    "跱" => "&#x8DF1;",
    "跲" => "&#x8DF2;",
    "跳" => "&#x8DF3;",
    "跴" => "&#x8DF4;",
    "践" => "&#x8DF5;",
    "跶" => "&#x8DF6;",
    "跷" => "&#x8DF7;",
    "跸" => "&#x8DF8;",
    "跹" => "&#x8DF9;",
    "跺" => "&#x8DFA;",
    "跻" => "&#x8DFB;",
    "跼" => "&#x8DFC;",
    "跽" => "&#x8DFD;",
    "跾" => "&#x8DFE;",
    "跿" => "&#x8DFF;",
    "踀" => "&#x8E00;",
    "踁" => "&#x8E01;",
    "踂" => "&#x8E02;",
    "踃" => "&#x8E03;",
    "踄" => "&#x8E04;",
    "踅" => "&#x8E05;",
    "踆" => "&#x8E06;",
    "踇" => "&#x8E07;",
    "踈" => "&#x8E08;",
    "踉" => "&#x8E09;",
    "踊" => "&#x8E0A;",
    "踋" => "&#x8E0B;",
    "踌" => "&#x8E0C;",
    "踍" => "&#x8E0D;",
    "踎" => "&#x8E0E;",
    "踏" => "&#x8E0F;",
    "踐" => "&#x8E10;",
    "踑" => "&#x8E11;",
    "踒" => "&#x8E12;",
    "踓" => "&#x8E13;",
    "踔" => "&#x8E14;",
    "踕" => "&#x8E15;",
    "踖" => "&#x8E16;",
    "踗" => "&#x8E17;",
    "踘" => "&#x8E18;",
    "踙" => "&#x8E19;",
    "踚" => "&#x8E1A;",
    "踛" => "&#x8E1B;",
    "踜" => "&#x8E1C;",
    "踝" => "&#x8E1D;",
    "踞" => "&#x8E1E;",
    "踟" => "&#x8E1F;",
    "踠" => "&#x8E20;",
    "踡" => "&#x8E21;",
    "踢" => "&#x8E22;",
    "踣" => "&#x8E23;",
    "踤" => "&#x8E24;",
    "踥" => "&#x8E25;",
    "踦" => "&#x8E26;",
    "踧" => "&#x8E27;",
    "踨" => "&#x8E28;",
    "踩" => "&#x8E29;",
    "踪" => "&#x8E2A;",
    "踫" => "&#x8E2B;",
    "踬" => "&#x8E2C;",
    "踭" => "&#x8E2D;",
    "踮" => "&#x8E2E;",
    "踯" => "&#x8E2F;",
    "踰" => "&#x8E30;",
    "踱" => "&#x8E31;",
    "踲" => "&#x8E32;",
    "踳" => "&#x8E33;",
    "踴" => "&#x8E34;",
    "踵" => "&#x8E35;",
    "踶" => "&#x8E36;",
    "踷" => "&#x8E37;",
    "踸" => "&#x8E38;",
    "踹" => "&#x8E39;",
    "踺" => "&#x8E3A;",
    "踻" => "&#x8E3B;",
    "踼" => "&#x8E3C;",
    "踽" => "&#x8E3D;",
    "踾" => "&#x8E3E;",
    "踿" => "&#x8E3F;",
    "蹀" => "&#x8E40;",
    "蹁" => "&#x8E41;",
    "蹂" => "&#x8E42;",
    "蹃" => "&#x8E43;",
    "蹄" => "&#x8E44;",
    "蹅" => "&#x8E45;",
    "蹆" => "&#x8E46;",
    "蹇" => "&#x8E47;",
    "蹈" => "&#x8E48;",
    "蹉" => "&#x8E49;",
    "蹊" => "&#x8E4A;",
    "蹋" => "&#x8E4B;",
    "蹌" => "&#x8E4C;",
    "蹍" => "&#x8E4D;",
    "蹎" => "&#x8E4E;",
    "蹏" => "&#x8E4F;",
    "蹐" => "&#x8E50;",
    "蹑" => "&#x8E51;",
    "蹒" => "&#x8E52;",
    "蹓" => "&#x8E53;",
    "蹔" => "&#x8E54;",
    "蹕" => "&#x8E55;",
    "蹖" => "&#x8E56;",
    "蹗" => "&#x8E57;",
    "蹘" => "&#x8E58;",
    "蹙" => "&#x8E59;",
    "蹚" => "&#x8E5A;",
    "蹛" => "&#x8E5B;",
    "蹜" => "&#x8E5C;",
    "蹝" => "&#x8E5D;",
    "蹞" => "&#x8E5E;",
    "蹟" => "&#x8E5F;",
    "蹠" => "&#x8E60;",
    "蹡" => "&#x8E61;",
    "蹢" => "&#x8E62;",
    "蹣" => "&#x8E63;",
    "蹤" => "&#x8E64;",
    "蹥" => "&#x8E65;",
    "蹦" => "&#x8E66;",
    "蹧" => "&#x8E67;",
    "蹨" => "&#x8E68;",
    "蹩" => "&#x8E69;",
    "蹪" => "&#x8E6A;",
    "蹫" => "&#x8E6B;",
    "蹬" => "&#x8E6C;",
    "蹭" => "&#x8E6D;",
    "蹮" => "&#x8E6E;",
    "蹯" => "&#x8E6F;",
    "蹰" => "&#x8E70;",
    "蹱" => "&#x8E71;",
    "蹲" => "&#x8E72;",
    "蹳" => "&#x8E73;",
    "蹴" => "&#x8E74;",
    "蹵" => "&#x8E75;",
    "蹶" => "&#x8E76;",
    "蹷" => "&#x8E77;",
    "蹸" => "&#x8E78;",
    "蹹" => "&#x8E79;",
    "蹺" => "&#x8E7A;",
    "蹻" => "&#x8E7B;",
    "蹼" => "&#x8E7C;",
    "蹽" => "&#x8E7D;",
    "蹾" => "&#x8E7E;",
    "蹿" => "&#x8E7F;",
    "躀" => "&#x8E80;",
    "躁" => "&#x8E81;",
    "躂" => "&#x8E82;",
    "躃" => "&#x8E83;",
    "躄" => "&#x8E84;",
    "躅" => "&#x8E85;",
    "躆" => "&#x8E86;",
    "躇" => "&#x8E87;",
    "躈" => "&#x8E88;",
    "躉" => "&#x8E89;",
    "躊" => "&#x8E8A;",
    "躋" => "&#x8E8B;",
    "躌" => "&#x8E8C;",
    "躍" => "&#x8E8D;",
    "躎" => "&#x8E8E;",
    "躏" => "&#x8E8F;",
    "躐" => "&#x8E90;",
    "躑" => "&#x8E91;",
    "躒" => "&#x8E92;",
    "躓" => "&#x8E93;",
    "躔" => "&#x8E94;",
    "躕" => "&#x8E95;",
    "躖" => "&#x8E96;",
    "躗" => "&#x8E97;",
    "躘" => "&#x8E98;",
    "躙" => "&#x8E99;",
    "躚" => "&#x8E9A;",
    "躛" => "&#x8E9B;",
    "躜" => "&#x8E9C;",
    "躝" => "&#x8E9D;",
    "躞" => "&#x8E9E;",
    "躟" => "&#x8E9F;",
    "躠" => "&#x8EA0;",
    "躡" => "&#x8EA1;",
    "躢" => "&#x8EA2;",
    "躣" => "&#x8EA3;",
    "躤" => "&#x8EA4;",
    "躥" => "&#x8EA5;",
    "躦" => "&#x8EA6;",
    "躧" => "&#x8EA7;",
    "躨" => "&#x8EA8;",
    "躩" => "&#x8EA9;",
    "躪" => "&#x8EAA;",
    "身" => "&#x8EAB;",
    "躬" => "&#x8EAC;",
    "躭" => "&#x8EAD;",
    "躮" => "&#x8EAE;",
    "躯" => "&#x8EAF;",
    "躰" => "&#x8EB0;",
    "躱" => "&#x8EB1;",
    "躲" => "&#x8EB2;",
    "躳" => "&#x8EB3;",
    "躴" => "&#x8EB4;",
    "躵" => "&#x8EB5;",
    "躶" => "&#x8EB6;",
    "躷" => "&#x8EB7;",
    "躸" => "&#x8EB8;",
    "躹" => "&#x8EB9;",
    "躺" => "&#x8EBA;",
    "躻" => "&#x8EBB;",
    "躼" => "&#x8EBC;",
    "躽" => "&#x8EBD;",
    "躾" => "&#x8EBE;",
    "躿" => "&#x8EBF;",
    "軀" => "&#x8EC0;",
    "軁" => "&#x8EC1;",
    "軂" => "&#x8EC2;",
    "軃" => "&#x8EC3;",
    "軄" => "&#x8EC4;",
    "軅" => "&#x8EC5;",
    "軆" => "&#x8EC6;",
    "軇" => "&#x8EC7;",
    "軈" => "&#x8EC8;",
    "軉" => "&#x8EC9;",
    "車" => "&#x8ECA;",
    "軋" => "&#x8ECB;",
    "軌" => "&#x8ECC;",
    "軍" => "&#x8ECD;",
    "軎" => "&#x8ECE;",
    "軏" => "&#x8ECF;",
    "軐" => "&#x8ED0;",
    "軑" => "&#x8ED1;",
    "軒" => "&#x8ED2;",
    "軓" => "&#x8ED3;",
    "軔" => "&#x8ED4;",
    "軕" => "&#x8ED5;",
    "軖" => "&#x8ED6;",
    "軗" => "&#x8ED7;",
    "軘" => "&#x8ED8;",
    "軙" => "&#x8ED9;",
    "軚" => "&#x8EDA;",
    "軛" => "&#x8EDB;",
    "軜" => "&#x8EDC;",
    "軝" => "&#x8EDD;",
    "軞" => "&#x8EDE;",
    "軟" => "&#x8EDF;",
    "軠" => "&#x8EE0;",
    "軡" => "&#x8EE1;",
    "転" => "&#x8EE2;",
    "軣" => "&#x8EE3;",
    "軤" => "&#x8EE4;",
    "軥" => "&#x8EE5;",
    "軦" => "&#x8EE6;",
    "軧" => "&#x8EE7;",
    "軨" => "&#x8EE8;",
    "軩" => "&#x8EE9;",
    "軪" => "&#x8EEA;",
    "軫" => "&#x8EEB;",
    "軬" => "&#x8EEC;",
    "軭" => "&#x8EED;",
    "軮" => "&#x8EEE;",
    "軯" => "&#x8EEF;",
    "軰" => "&#x8EF0;",
    "軱" => "&#x8EF1;",
    "軲" => "&#x8EF2;",
    "軳" => "&#x8EF3;",
    "軴" => "&#x8EF4;",
    "軵" => "&#x8EF5;",
    "軶" => "&#x8EF6;",
    "軷" => "&#x8EF7;",
    "軸" => "&#x8EF8;",
    "軹" => "&#x8EF9;",
    "軺" => "&#x8EFA;",
    "軻" => "&#x8EFB;",
    "軼" => "&#x8EFC;",
    "軽" => "&#x8EFD;",
    "軾" => "&#x8EFE;",
    "軿" => "&#x8EFF;",
    "輀" => "&#x8F00;",
    "輁" => "&#x8F01;",
    "輂" => "&#x8F02;",
    "較" => "&#x8F03;",
    "輄" => "&#x8F04;",
    "輅" => "&#x8F05;",
    "輆" => "&#x8F06;",
    "輇" => "&#x8F07;",
    "輈" => "&#x8F08;",
    "載" => "&#x8F09;",
    "輊" => "&#x8F0A;",
    "輋" => "&#x8F0B;",
    "輌" => "&#x8F0C;",
    "輍" => "&#x8F0D;",
    "輎" => "&#x8F0E;",
    "輏" => "&#x8F0F;",
    "輐" => "&#x8F10;",
    "輑" => "&#x8F11;",
    "輒" => "&#x8F12;",
    "輓" => "&#x8F13;",
    "輔" => "&#x8F14;",
    "輕" => "&#x8F15;",
    "輖" => "&#x8F16;",
    "輗" => "&#x8F17;",
    "輘" => "&#x8F18;",
    "輙" => "&#x8F19;",
    "輚" => "&#x8F1A;",
    "輛" => "&#x8F1B;",
    "輜" => "&#x8F1C;",
    "輝" => "&#x8F1D;",
    "輞" => "&#x8F1E;",
    "輟" => "&#x8F1F;",
    "輠" => "&#x8F20;",
    "輡" => "&#x8F21;",
    "輢" => "&#x8F22;",
    "輣" => "&#x8F23;",
    "輤" => "&#x8F24;",
    "輥" => "&#x8F25;",
    "輦" => "&#x8F26;",
    "輧" => "&#x8F27;",
    "輨" => "&#x8F28;",
    "輩" => "&#x8F29;",
    "輪" => "&#x8F2A;",
    "輫" => "&#x8F2B;",
    "輬" => "&#x8F2C;",
    "輭" => "&#x8F2D;",
    "輮" => "&#x8F2E;",
    "輯" => "&#x8F2F;",
    "輰" => "&#x8F30;",
    "輱" => "&#x8F31;",
    "輲" => "&#x8F32;",
    "輳" => "&#x8F33;",
    "輴" => "&#x8F34;",
    "輵" => "&#x8F35;",
    "輶" => "&#x8F36;",
    "輷" => "&#x8F37;",
    "輸" => "&#x8F38;",
    "輹" => "&#x8F39;",
    "輺" => "&#x8F3A;",
    "輻" => "&#x8F3B;",
    "輼" => "&#x8F3C;",
    "輽" => "&#x8F3D;",
    "輾" => "&#x8F3E;",
    "輿" => "&#x8F3F;",
    "轀" => "&#x8F40;",
    "轁" => "&#x8F41;",
    "轂" => "&#x8F42;",
    "轃" => "&#x8F43;",
    "轄" => "&#x8F44;",
    "轅" => "&#x8F45;",
    "轆" => "&#x8F46;",
    "轇" => "&#x8F47;",
    "轈" => "&#x8F48;",
    "轉" => "&#x8F49;",
    "轊" => "&#x8F4A;",
    "轋" => "&#x8F4B;",
    "轌" => "&#x8F4C;",
    "轍" => "&#x8F4D;",
    "轎" => "&#x8F4E;",
    "轏" => "&#x8F4F;",
    "轐" => "&#x8F50;",
    "轑" => "&#x8F51;",
    "轒" => "&#x8F52;",
    "轓" => "&#x8F53;",
    "轔" => "&#x8F54;",
    "轕" => "&#x8F55;",
    "轖" => "&#x8F56;",
    "轗" => "&#x8F57;",
    "轘" => "&#x8F58;",
    "轙" => "&#x8F59;",
    "轚" => "&#x8F5A;",
    "轛" => "&#x8F5B;",
    "轜" => "&#x8F5C;",
    "轝" => "&#x8F5D;",
    "轞" => "&#x8F5E;",
    "轟" => "&#x8F5F;",
    "轠" => "&#x8F60;",
    "轡" => "&#x8F61;",
    "轢" => "&#x8F62;",
    "轣" => "&#x8F63;",
    "轤" => "&#x8F64;",
    "轥" => "&#x8F65;",
    "车" => "&#x8F66;",
    "轧" => "&#x8F67;",
    "轨" => "&#x8F68;",
    "轩" => "&#x8F69;",
    "轪" => "&#x8F6A;",
    "轫" => "&#x8F6B;",
    "转" => "&#x8F6C;",
    "轭" => "&#x8F6D;",
    "轮" => "&#x8F6E;",
    "软" => "&#x8F6F;",
    "轰" => "&#x8F70;",
    "轱" => "&#x8F71;",
    "轲" => "&#x8F72;",
    "轳" => "&#x8F73;",
    "轴" => "&#x8F74;",
    "轵" => "&#x8F75;",
    "轶" => "&#x8F76;",
    "轷" => "&#x8F77;",
    "轸" => "&#x8F78;",
    "轹" => "&#x8F79;",
    "轺" => "&#x8F7A;",
    "轻" => "&#x8F7B;",
    "轼" => "&#x8F7C;",
    "载" => "&#x8F7D;",
    "轾" => "&#x8F7E;",
    "轿" => "&#x8F7F;",
    "辀" => "&#x8F80;",
    "辁" => "&#x8F81;",
    "辂" => "&#x8F82;",
    "较" => "&#x8F83;",
    "辄" => "&#x8F84;",
    "辅" => "&#x8F85;",
    "辆" => "&#x8F86;",
    "辇" => "&#x8F87;",
    "辈" => "&#x8F88;",
    "辉" => "&#x8F89;",
    "辊" => "&#x8F8A;",
    "辋" => "&#x8F8B;",
    "辌" => "&#x8F8C;",
    "辍" => "&#x8F8D;",
    "辎" => "&#x8F8E;",
    "辏" => "&#x8F8F;",
    "辐" => "&#x8F90;",
    "辑" => "&#x8F91;",
    "辒" => "&#x8F92;",
    "输" => "&#x8F93;",
    "辔" => "&#x8F94;",
    "辕" => "&#x8F95;",
    "辖" => "&#x8F96;",
    "辗" => "&#x8F97;",
    "辘" => "&#x8F98;",
    "辙" => "&#x8F99;",
    "辚" => "&#x8F9A;",
    "辛" => "&#x8F9B;",
    "辜" => "&#x8F9C;",
    "辝" => "&#x8F9D;",
    "辞" => "&#x8F9E;",
    "辟" => "&#x8F9F;",
    "辠" => "&#x8FA0;",
    "辡" => "&#x8FA1;",
    "辢" => "&#x8FA2;",
    "辣" => "&#x8FA3;",
    "辤" => "&#x8FA4;",
    "辥" => "&#x8FA5;",
    "辦" => "&#x8FA6;",
    "辧" => "&#x8FA7;",
    "辨" => "&#x8FA8;",
    "辩" => "&#x8FA9;",
    "辪" => "&#x8FAA;",
    "辫" => "&#x8FAB;",
    "辬" => "&#x8FAC;",
    "辭" => "&#x8FAD;",
    "辮" => "&#x8FAE;",
    "辯" => "&#x8FAF;",
    "辰" => "&#x8FB0;",
    "辱" => "&#x8FB1;",
    "農" => "&#x8FB2;",
    "辳" => "&#x8FB3;",
    "辴" => "&#x8FB4;",
    "辵" => "&#x8FB5;",
    "辶" => "&#x8FB6;",
    "辷" => "&#x8FB7;",
    "辸" => "&#x8FB8;",
    "边" => "&#x8FB9;",
    "辺" => "&#x8FBA;",
    "辻" => "&#x8FBB;",
    "込" => "&#x8FBC;",
    "辽" => "&#x8FBD;",
    "达" => "&#x8FBE;",
    "辿" => "&#x8FBF;",
    "迀" => "&#x8FC0;",
    "迁" => "&#x8FC1;",
    "迂" => "&#x8FC2;",
    "迃" => "&#x8FC3;",
    "迄" => "&#x8FC4;",
    "迅" => "&#x8FC5;",
    "迆" => "&#x8FC6;",
    "过" => "&#x8FC7;",
    "迈" => "&#x8FC8;",
    "迉" => "&#x8FC9;",
    "迊" => "&#x8FCA;",
    "迋" => "&#x8FCB;",
    "迌" => "&#x8FCC;",
    "迍" => "&#x8FCD;",
    "迎" => "&#x8FCE;",
    "迏" => "&#x8FCF;",
    "运" => "&#x8FD0;",
    "近" => "&#x8FD1;",
    "迒" => "&#x8FD2;",
    "迓" => "&#x8FD3;",
    "返" => "&#x8FD4;",
    "迕" => "&#x8FD5;",
    "迖" => "&#x8FD6;",
    "迗" => "&#x8FD7;",
    "还" => "&#x8FD8;",
    "这" => "&#x8FD9;",
    "迚" => "&#x8FDA;",
    "进" => "&#x8FDB;",
    "远" => "&#x8FDC;",
    "违" => "&#x8FDD;",
    "连" => "&#x8FDE;",
    "迟" => "&#x8FDF;",
    "迠" => "&#x8FE0;",
    "迡" => "&#x8FE1;",
    "迢" => "&#x8FE2;",
    "迣" => "&#x8FE3;",
    "迤" => "&#x8FE4;",
    "迥" => "&#x8FE5;",
    "迦" => "&#x8FE6;",
    "迧" => "&#x8FE7;",
    "迨" => "&#x8FE8;",
    "迩" => "&#x8FE9;",
    "迪" => "&#x8FEA;",
    "迫" => "&#x8FEB;",
    "迬" => "&#x8FEC;",
    "迭" => "&#x8FED;",
    "迮" => "&#x8FEE;",
    "迯" => "&#x8FEF;",
    "述" => "&#x8FF0;",
    "迱" => "&#x8FF1;",
    "迲" => "&#x8FF2;",
    "迳" => "&#x8FF3;",
    "迴" => "&#x8FF4;",
    "迵" => "&#x8FF5;",
    "迶" => "&#x8FF6;",
    "迷" => "&#x8FF7;",
    "迸" => "&#x8FF8;",
    "迹" => "&#x8FF9;",
    "迺" => "&#x8FFA;",
    "迻" => "&#x8FFB;",
    "迼" => "&#x8FFC;",
    "追" => "&#x8FFD;",
    "迾" => "&#x8FFE;",
    "迿" => "&#x8FFF;",
    "退" => "&#x9000;",
    "送" => "&#x9001;",
    "适" => "&#x9002;",
    "逃" => "&#x9003;",
    "逄" => "&#x9004;",
    "逅" => "&#x9005;",
    "逆" => "&#x9006;",
    "逇" => "&#x9007;",
    "逈" => "&#x9008;",
    "选" => "&#x9009;",
    "逊" => "&#x900A;",
    "逋" => "&#x900B;",
    "逌" => "&#x900C;",
    "逍" => "&#x900D;",
    "逎" => "&#x900E;",
    "透" => "&#x900F;",
    "逐" => "&#x9010;",
    "逑" => "&#x9011;",
    "递" => "&#x9012;",
    "逓" => "&#x9013;",
    "途" => "&#x9014;",
    "逕" => "&#x9015;",
    "逖" => "&#x9016;",
    "逗" => "&#x9017;",
    "逘" => "&#x9018;",
    "這" => "&#x9019;",
    "通" => "&#x901A;",
    "逛" => "&#x901B;",
    "逜" => "&#x901C;",
    "逝" => "&#x901D;",
    "逞" => "&#x901E;",
    "速" => "&#x901F;",
    "造" => "&#x9020;",
    "逡" => "&#x9021;",
    "逢" => "&#x9022;",
    "連" => "&#x9023;",
    "逤" => "&#x9024;",
    "逥" => "&#x9025;",
    "逦" => "&#x9026;",
    "逧" => "&#x9027;",
    "逨" => "&#x9028;",
    "逩" => "&#x9029;",
    "逪" => "&#x902A;",
    "逫" => "&#x902B;",
    "逬" => "&#x902C;",
    "逭" => "&#x902D;",
    "逮" => "&#x902E;",
    "逯" => "&#x902F;",
    "逰" => "&#x9030;",
    "週" => "&#x9031;",
    "進" => "&#x9032;",
    "逳" => "&#x9033;",
    "逴" => "&#x9034;",
    "逵" => "&#x9035;",
    "逶" => "&#x9036;",
    "逷" => "&#x9037;",
    "逸" => "&#x9038;",
    "逹" => "&#x9039;",
    "逺" => "&#x903A;",
    "逻" => "&#x903B;",
    "逼" => "&#x903C;",
    "逽" => "&#x903D;",
    "逾" => "&#x903E;",
    "逿" => "&#x903F;",
    "遀" => "&#x9040;",
    "遁" => "&#x9041;",
    "遂" => "&#x9042;",
    "遃" => "&#x9043;",
    "遄" => "&#x9044;",
    "遅" => "&#x9045;",
    "遆" => "&#x9046;",
    "遇" => "&#x9047;",
    "遈" => "&#x9048;",
    "遉" => "&#x9049;",
    "遊" => "&#x904A;",
    "運" => "&#x904B;",
    "遌" => "&#x904C;",
    "遍" => "&#x904D;",
    "過" => "&#x904E;",
    "遏" => "&#x904F;",
    "遐" => "&#x9050;",
    "遑" => "&#x9051;",
    "遒" => "&#x9052;",
    "道" => "&#x9053;",
    "達" => "&#x9054;",
    "違" => "&#x9055;",
    "遖" => "&#x9056;",
    "遗" => "&#x9057;",
    "遘" => "&#x9058;",
    "遙" => "&#x9059;",
    "遚" => "&#x905A;",
    "遛" => "&#x905B;",
    "遜" => "&#x905C;",
    "遝" => "&#x905D;",
    "遞" => "&#x905E;",
    "遟" => "&#x905F;",
    "遠" => "&#x9060;",
    "遡" => "&#x9061;",
    "遢" => "&#x9062;",
    "遣" => "&#x9063;",
    "遤" => "&#x9064;",
    "遥" => "&#x9065;",
    "遦" => "&#x9066;",
    "遧" => "&#x9067;",
    "遨" => "&#x9068;",
    "適" => "&#x9069;",
    "遪" => "&#x906A;",
    "遫" => "&#x906B;",
    "遬" => "&#x906C;",
    "遭" => "&#x906D;",
    "遮" => "&#x906E;",
    "遯" => "&#x906F;",
    "遰" => "&#x9070;",
    "遱" => "&#x9071;",
    "遲" => "&#x9072;",
    "遳" => "&#x9073;",
    "遴" => "&#x9074;",
    "遵" => "&#x9075;",
    "遶" => "&#x9076;",
    "遷" => "&#x9077;",
    "選" => "&#x9078;",
    "遹" => "&#x9079;",
    "遺" => "&#x907A;",
    "遻" => "&#x907B;",
    "遼" => "&#x907C;",
    "遽" => "&#x907D;",
    "遾" => "&#x907E;",
    "避" => "&#x907F;",
    "邀" => "&#x9080;",
    "邁" => "&#x9081;",
    "邂" => "&#x9082;",
    "邃" => "&#x9083;",
    "還" => "&#x9084;",
    "邅" => "&#x9085;",
    "邆" => "&#x9086;",
    "邇" => "&#x9087;",
    "邈" => "&#x9088;",
    "邉" => "&#x9089;",
    "邊" => "&#x908A;",
    "邋" => "&#x908B;",
    "邌" => "&#x908C;",
    "邍" => "&#x908D;",
    "邎" => "&#x908E;",
    "邏" => "&#x908F;",
    "邐" => "&#x9090;",
    "邑" => "&#x9091;",
    "邒" => "&#x9092;",
    "邓" => "&#x9093;",
    "邔" => "&#x9094;",
    "邕" => "&#x9095;",
    "邖" => "&#x9096;",
    "邗" => "&#x9097;",
    "邘" => "&#x9098;",
    "邙" => "&#x9099;",
    "邚" => "&#x909A;",
    "邛" => "&#x909B;",
    "邜" => "&#x909C;",
    "邝" => "&#x909D;",
    "邞" => "&#x909E;",
    "邟" => "&#x909F;",
    "邠" => "&#x90A0;",
    "邡" => "&#x90A1;",
    "邢" => "&#x90A2;",
    "那" => "&#x90A3;",
    "邤" => "&#x90A4;",
    "邥" => "&#x90A5;",
    "邦" => "&#x90A6;",
    "邧" => "&#x90A7;",
    "邨" => "&#x90A8;",
    "邩" => "&#x90A9;",
    "邪" => "&#x90AA;",
    "邫" => "&#x90AB;",
    "邬" => "&#x90AC;",
    "邭" => "&#x90AD;",
    "邮" => "&#x90AE;",
    "邯" => "&#x90AF;",
    "邰" => "&#x90B0;",
    "邱" => "&#x90B1;",
    "邲" => "&#x90B2;",
    "邳" => "&#x90B3;",
    "邴" => "&#x90B4;",
    "邵" => "&#x90B5;",
    "邶" => "&#x90B6;",
    "邷" => "&#x90B7;",
    "邸" => "&#x90B8;",
    "邹" => "&#x90B9;",
    "邺" => "&#x90BA;",
    "邻" => "&#x90BB;",
    "邼" => "&#x90BC;",
    "邽" => "&#x90BD;",
    "邾" => "&#x90BE;",
    "邿" => "&#x90BF;",
    "郀" => "&#x90C0;",
    "郁" => "&#x90C1;",
    "郂" => "&#x90C2;",
    "郃" => "&#x90C3;",
    "郄" => "&#x90C4;",
    "郅" => "&#x90C5;",
    "郆" => "&#x90C6;",
    "郇" => "&#x90C7;",
    "郈" => "&#x90C8;",
    "郉" => "&#x90C9;",
    "郊" => "&#x90CA;",
    "郋" => "&#x90CB;",
    "郌" => "&#x90CC;",
    "郍" => "&#x90CD;",
    "郎" => "&#x90CE;",
    "郏" => "&#x90CF;",
    "郐" => "&#x90D0;",
    "郑" => "&#x90D1;",
    "郒" => "&#x90D2;",
    "郓" => "&#x90D3;",
    "郔" => "&#x90D4;",
    "郕" => "&#x90D5;",
    "郖" => "&#x90D6;",
    "郗" => "&#x90D7;",
    "郘" => "&#x90D8;",
    "郙" => "&#x90D9;",
    "郚" => "&#x90DA;",
    "郛" => "&#x90DB;",
    "郜" => "&#x90DC;",
    "郝" => "&#x90DD;",
    "郞" => "&#x90DE;",
    "郟" => "&#x90DF;",
    "郠" => "&#x90E0;",
    "郡" => "&#x90E1;",
    "郢" => "&#x90E2;",
    "郣" => "&#x90E3;",
    "郤" => "&#x90E4;",
    "郥" => "&#x90E5;",
    "郦" => "&#x90E6;",
    "郧" => "&#x90E7;",
    "部" => "&#x90E8;",
    "郩" => "&#x90E9;",
    "郪" => "&#x90EA;",
    "郫" => "&#x90EB;",
    "郬" => "&#x90EC;",
    "郭" => "&#x90ED;",
    "郮" => "&#x90EE;",
    "郯" => "&#x90EF;",
    "郰" => "&#x90F0;",
    "郱" => "&#x90F1;",
    "郲" => "&#x90F2;",
    "郳" => "&#x90F3;",
    "郴" => "&#x90F4;",
    "郵" => "&#x90F5;",
    "郶" => "&#x90F6;",
    "郷" => "&#x90F7;",
    "郸" => "&#x90F8;",
    "郹" => "&#x90F9;",
    "郺" => "&#x90FA;",
    "郻" => "&#x90FB;",
    "郼" => "&#x90FC;",
    "都" => "&#x90FD;",
    "郾" => "&#x90FE;",
    "郿" => "&#x90FF;",
    "鄀" => "&#x9100;",
    "鄁" => "&#x9101;",
    "鄂" => "&#x9102;",
    "鄃" => "&#x9103;",
    "鄄" => "&#x9104;",
    "鄅" => "&#x9105;",
    "鄆" => "&#x9106;",
    "鄇" => "&#x9107;",
    "鄈" => "&#x9108;",
    "鄉" => "&#x9109;",
    "鄊" => "&#x910A;",
    "鄋" => "&#x910B;",
    "鄌" => "&#x910C;",
    "鄍" => "&#x910D;",
    "鄎" => "&#x910E;",
    "鄏" => "&#x910F;",
    "鄐" => "&#x9110;",
    "鄑" => "&#x9111;",
    "鄒" => "&#x9112;",
    "鄓" => "&#x9113;",
    "鄔" => "&#x9114;",
    "鄕" => "&#x9115;",
    "鄖" => "&#x9116;",
    "鄗" => "&#x9117;",
    "鄘" => "&#x9118;",
    "鄙" => "&#x9119;",
    "鄚" => "&#x911A;",
    "鄛" => "&#x911B;",
    "鄜" => "&#x911C;",
    "鄝" => "&#x911D;",
    "鄞" => "&#x911E;",
    "鄟" => "&#x911F;",
    "鄠" => "&#x9120;",
    "鄡" => "&#x9121;",
    "鄢" => "&#x9122;",
    "鄣" => "&#x9123;",
    "鄤" => "&#x9124;",
    "鄥" => "&#x9125;",
    "鄦" => "&#x9126;",
    "鄧" => "&#x9127;",
    "鄨" => "&#x9128;",
    "鄩" => "&#x9129;",
    "鄪" => "&#x912A;",
    "鄫" => "&#x912B;",
    "鄬" => "&#x912C;",
    "鄭" => "&#x912D;",
    "鄮" => "&#x912E;",
    "鄯" => "&#x912F;",
    "鄰" => "&#x9130;",
    "鄱" => "&#x9131;",
    "鄲" => "&#x9132;",
    "鄳" => "&#x9133;",
    "鄴" => "&#x9134;",
    "鄵" => "&#x9135;",
    "鄶" => "&#x9136;",
    "鄷" => "&#x9137;",
    "鄸" => "&#x9138;",
    "鄹" => "&#x9139;",
    "鄺" => "&#x913A;",
    "鄻" => "&#x913B;",
    "鄼" => "&#x913C;",
    "鄽" => "&#x913D;",
    "鄾" => "&#x913E;",
    "鄿" => "&#x913F;",
    "酀" => "&#x9140;",
    "酁" => "&#x9141;",
    "酂" => "&#x9142;",
    "酃" => "&#x9143;",
    "酄" => "&#x9144;",
    "酅" => "&#x9145;",
    "酆" => "&#x9146;",
    "酇" => "&#x9147;",
    "酈" => "&#x9148;",
    "酉" => "&#x9149;",
    "酊" => "&#x914A;",
    "酋" => "&#x914B;",
    "酌" => "&#x914C;",
    "配" => "&#x914D;",
    "酎" => "&#x914E;",
    "酏" => "&#x914F;",
    "酐" => "&#x9150;",
    "酑" => "&#x9151;",
    "酒" => "&#x9152;",
    "酓" => "&#x9153;",
    "酔" => "&#x9154;",
    "酕" => "&#x9155;",
    "酖" => "&#x9156;",
    "酗" => "&#x9157;",
    "酘" => "&#x9158;",
    "酙" => "&#x9159;",
    "酚" => "&#x915A;",
    "酛" => "&#x915B;",
    "酜" => "&#x915C;",
    "酝" => "&#x915D;",
    "酞" => "&#x915E;",
    "酟" => "&#x915F;",
    "酠" => "&#x9160;",
    "酡" => "&#x9161;",
    "酢" => "&#x9162;",
    "酣" => "&#x9163;",
    "酤" => "&#x9164;",
    "酥" => "&#x9165;",
    "酦" => "&#x9166;",
    "酧" => "&#x9167;",
    "酨" => "&#x9168;",
    "酩" => "&#x9169;",
    "酪" => "&#x916A;",
    "酫" => "&#x916B;",
    "酬" => "&#x916C;",
    "酭" => "&#x916D;",
    "酮" => "&#x916E;",
    "酯" => "&#x916F;",
    "酰" => "&#x9170;",
    "酱" => "&#x9171;",
    "酲" => "&#x9172;",
    "酳" => "&#x9173;",
    "酴" => "&#x9174;",
    "酵" => "&#x9175;",
    "酶" => "&#x9176;",
    "酷" => "&#x9177;",
    "酸" => "&#x9178;",
    "酹" => "&#x9179;",
    "酺" => "&#x917A;",
    "酻" => "&#x917B;",
    "酼" => "&#x917C;",
    "酽" => "&#x917D;",
    "酾" => "&#x917E;",
    "酿" => "&#x917F;",
    "醀" => "&#x9180;",
    "醁" => "&#x9181;",
    "醂" => "&#x9182;",
    "醃" => "&#x9183;",
    "醄" => "&#x9184;",
    "醅" => "&#x9185;",
    "醆" => "&#x9186;",
    "醇" => "&#x9187;",
    "醈" => "&#x9188;",
    "醉" => "&#x9189;",
    "醊" => "&#x918A;",
    "醋" => "&#x918B;",
    "醌" => "&#x918C;",
    "醍" => "&#x918D;",
    "醎" => "&#x918E;",
    "醏" => "&#x918F;",
    "醐" => "&#x9190;",
    "醑" => "&#x9191;",
    "醒" => "&#x9192;",
    "醓" => "&#x9193;",
    "醔" => "&#x9194;",
    "醕" => "&#x9195;",
    "醖" => "&#x9196;",
    "醗" => "&#x9197;",
    "醘" => "&#x9198;",
    "醙" => "&#x9199;",
    "醚" => "&#x919A;",
    "醛" => "&#x919B;",
    "醜" => "&#x919C;",
    "醝" => "&#x919D;",
    "醞" => "&#x919E;",
    "醟" => "&#x919F;",
    "醠" => "&#x91A0;",
    "醡" => "&#x91A1;",
    "醢" => "&#x91A2;",
    "醣" => "&#x91A3;",
    "醤" => "&#x91A4;",
    "醥" => "&#x91A5;",
    "醦" => "&#x91A6;",
    "醧" => "&#x91A7;",
    "醨" => "&#x91A8;",
    "醩" => "&#x91A9;",
    "醪" => "&#x91AA;",
    "醫" => "&#x91AB;",
    "醬" => "&#x91AC;",
    "醭" => "&#x91AD;",
    "醮" => "&#x91AE;",
    "醯" => "&#x91AF;",
    "醰" => "&#x91B0;",
    "醱" => "&#x91B1;",
    "醲" => "&#x91B2;",
    "醳" => "&#x91B3;",
    "醴" => "&#x91B4;",
    "醵" => "&#x91B5;",
    "醶" => "&#x91B6;",
    "醷" => "&#x91B7;",
    "醸" => "&#x91B8;",
    "醹" => "&#x91B9;",
    "醺" => "&#x91BA;",
    "醻" => "&#x91BB;",
    "醼" => "&#x91BC;",
    "醽" => "&#x91BD;",
    "醾" => "&#x91BE;",
    "醿" => "&#x91BF;",
    "釀" => "&#x91C0;",
    "釁" => "&#x91C1;",
    "釂" => "&#x91C2;",
    "釃" => "&#x91C3;",
    "釄" => "&#x91C4;",
    "釅" => "&#x91C5;",
    "釆" => "&#x91C6;",
    "采" => "&#x91C7;",
    "釈" => "&#x91C8;",
    "釉" => "&#x91C9;",
    "释" => "&#x91CA;",
    "釋" => "&#x91CB;",
    "里" => "&#x91CC;",
    "重" => "&#x91CD;",
    "野" => "&#x91CE;",
    "量" => "&#x91CF;",
    "釐" => "&#x91D0;",
    "金" => "&#x91D1;",
    "釒" => "&#x91D2;",
    "釓" => "&#x91D3;",
    "釔" => "&#x91D4;",
    "釕" => "&#x91D5;",
    "釖" => "&#x91D6;",
    "釗" => "&#x91D7;",
    "釘" => "&#x91D8;",
    "釙" => "&#x91D9;",
    "釚" => "&#x91DA;",
    "釛" => "&#x91DB;",
    "釜" => "&#x91DC;",
    "針" => "&#x91DD;",
    "釞" => "&#x91DE;",
    "釟" => "&#x91DF;",
    "釠" => "&#x91E0;",
    "釡" => "&#x91E1;",
    "釢" => "&#x91E2;",
    "釣" => "&#x91E3;",
    "釤" => "&#x91E4;",
    "釥" => "&#x91E5;",
    "釦" => "&#x91E6;",
    "釧" => "&#x91E7;",
    "釨" => "&#x91E8;",
    "釩" => "&#x91E9;",
    "釪" => "&#x91EA;",
    "釫" => "&#x91EB;",
    "釬" => "&#x91EC;",
    "釭" => "&#x91ED;",
    "釮" => "&#x91EE;",
    "釯" => "&#x91EF;",
    "釰" => "&#x91F0;",
    "釱" => "&#x91F1;",
    "釲" => "&#x91F2;",
    "釳" => "&#x91F3;",
    "釴" => "&#x91F4;",
    "釵" => "&#x91F5;",
    "釶" => "&#x91F6;",
    "釷" => "&#x91F7;",
    "釸" => "&#x91F8;",
    "釹" => "&#x91F9;",
    "釺" => "&#x91FA;",
    "釻" => "&#x91FB;",
    "釼" => "&#x91FC;",
    "釽" => "&#x91FD;",
    "釾" => "&#x91FE;",
    "釿" => "&#x91FF;",
    "鈀" => "&#x9200;",
    "鈁" => "&#x9201;",
    "鈂" => "&#x9202;",
    "鈃" => "&#x9203;",
    "鈄" => "&#x9204;",
    "鈅" => "&#x9205;",
    "鈆" => "&#x9206;",
    "鈇" => "&#x9207;",
    "鈈" => "&#x9208;",
    "鈉" => "&#x9209;",
    "鈊" => "&#x920A;",
    "鈋" => "&#x920B;",
    "鈌" => "&#x920C;",
    "鈍" => "&#x920D;",
    "鈎" => "&#x920E;",
    "鈏" => "&#x920F;",
    "鈐" => "&#x9210;",
    "鈑" => "&#x9211;",
    "鈒" => "&#x9212;",
    "鈓" => "&#x9213;",
    "鈔" => "&#x9214;",
    "鈕" => "&#x9215;",
    "鈖" => "&#x9216;",
    "鈗" => "&#x9217;",
    "鈘" => "&#x9218;",
    "鈙" => "&#x9219;",
    "鈚" => "&#x921A;",
    "鈛" => "&#x921B;",
    "鈜" => "&#x921C;",
    "鈝" => "&#x921D;",
    "鈞" => "&#x921E;",
    "鈟" => "&#x921F;",
    "鈠" => "&#x9220;",
    "鈡" => "&#x9221;",
    "鈢" => "&#x9222;",
    "鈣" => "&#x9223;",
    "鈤" => "&#x9224;",
    "鈥" => "&#x9225;",
    "鈦" => "&#x9226;",
    "鈧" => "&#x9227;",
    "鈨" => "&#x9228;",
    "鈩" => "&#x9229;",
    "鈪" => "&#x922A;",
    "鈫" => "&#x922B;",
    "鈬" => "&#x922C;",
    "鈭" => "&#x922D;",
    "鈮" => "&#x922E;",
    "鈯" => "&#x922F;",
    "鈰" => "&#x9230;",
    "鈱" => "&#x9231;",
    "鈲" => "&#x9232;",
    "鈳" => "&#x9233;",
    "鈴" => "&#x9234;",
    "鈵" => "&#x9235;",
    "鈶" => "&#x9236;",
    "鈷" => "&#x9237;",
    "鈸" => "&#x9238;",
    "鈹" => "&#x9239;",
    "鈺" => "&#x923A;",
    "鈻" => "&#x923B;",
    "鈼" => "&#x923C;",
    "鈽" => "&#x923D;",
    "鈾" => "&#x923E;",
    "鈿" => "&#x923F;",
    "鉀" => "&#x9240;",
    "鉁" => "&#x9241;",
    "鉂" => "&#x9242;",
    "鉃" => "&#x9243;",
    "鉄" => "&#x9244;",
    "鉅" => "&#x9245;",
    "鉆" => "&#x9246;",
    "鉇" => "&#x9247;",
    "鉈" => "&#x9248;",
    "鉉" => "&#x9249;",
    "鉊" => "&#x924A;",
    "鉋" => "&#x924B;",
    "鉌" => "&#x924C;",
    "鉍" => "&#x924D;",
    "鉎" => "&#x924E;",
    "鉏" => "&#x924F;",
    "鉐" => "&#x9250;",
    "鉑" => "&#x9251;",
    "鉒" => "&#x9252;",
    "鉓" => "&#x9253;",
    "鉔" => "&#x9254;",
    "鉕" => "&#x9255;",
    "鉖" => "&#x9256;",
    "鉗" => "&#x9257;",
    "鉘" => "&#x9258;",
    "鉙" => "&#x9259;",
    "鉚" => "&#x925A;",
    "鉛" => "&#x925B;",
    "鉜" => "&#x925C;",
    "鉝" => "&#x925D;",
    "鉞" => "&#x925E;",
    "鉟" => "&#x925F;",
    "鉠" => "&#x9260;",
    "鉡" => "&#x9261;",
    "鉢" => "&#x9262;",
    "鉣" => "&#x9263;",
    "鉤" => "&#x9264;",
    "鉥" => "&#x9265;",
    "鉦" => "&#x9266;",
    "鉧" => "&#x9267;",
    "鉨" => "&#x9268;",
    "鉩" => "&#x9269;",
    "鉪" => "&#x926A;",
    "鉫" => "&#x926B;",
    "鉬" => "&#x926C;",
    "鉭" => "&#x926D;",
    "鉮" => "&#x926E;",
    "鉯" => "&#x926F;",
    "鉰" => "&#x9270;",
    "鉱" => "&#x9271;",
    "鉲" => "&#x9272;",
    "鉳" => "&#x9273;",
    "鉴" => "&#x9274;",
    "鉵" => "&#x9275;",
    "鉶" => "&#x9276;",
    "鉷" => "&#x9277;",
    "鉸" => "&#x9278;",
    "鉹" => "&#x9279;",
    "鉺" => "&#x927A;",
    "鉻" => "&#x927B;",
    "鉼" => "&#x927C;",
    "鉽" => "&#x927D;",
    "鉾" => "&#x927E;",
    "鉿" => "&#x927F;",
    "銀" => "&#x9280;",
    "銁" => "&#x9281;",
    "銂" => "&#x9282;",
    "銃" => "&#x9283;",
    "銄" => "&#x9284;",
    "銅" => "&#x9285;",
    "銆" => "&#x9286;",
    "銇" => "&#x9287;",
    "銈" => "&#x9288;",
    "銉" => "&#x9289;",
    "銊" => "&#x928A;",
    "銋" => "&#x928B;",
    "銌" => "&#x928C;",
    "銍" => "&#x928D;",
    "銎" => "&#x928E;",
    "銏" => "&#x928F;",
    "銐" => "&#x9290;",
    "銑" => "&#x9291;",
    "銒" => "&#x9292;",
    "銓" => "&#x9293;",
    "銔" => "&#x9294;",
    "銕" => "&#x9295;",
    "銖" => "&#x9296;",
    "銗" => "&#x9297;",
    "銘" => "&#x9298;",
    "銙" => "&#x9299;",
    "銚" => "&#x929A;",
    "銛" => "&#x929B;",
    "銜" => "&#x929C;",
    "銝" => "&#x929D;",
    "銞" => "&#x929E;",
    "銟" => "&#x929F;",
    "銠" => "&#x92A0;",
    "銡" => "&#x92A1;",
    "銢" => "&#x92A2;",
    "銣" => "&#x92A3;",
    "銤" => "&#x92A4;",
    "銥" => "&#x92A5;",
    "銦" => "&#x92A6;",
    "銧" => "&#x92A7;",
    "銨" => "&#x92A8;",
    "銩" => "&#x92A9;",
    "銪" => "&#x92AA;",
    "銫" => "&#x92AB;",
    "銬" => "&#x92AC;",
    "銭" => "&#x92AD;",
    "銮" => "&#x92AE;",
    "銯" => "&#x92AF;",
    "銰" => "&#x92B0;",
    "銱" => "&#x92B1;",
    "銲" => "&#x92B2;",
    "銳" => "&#x92B3;",
    "銴" => "&#x92B4;",
    "銵" => "&#x92B5;",
    "銶" => "&#x92B6;",
    "銷" => "&#x92B7;",
    "銸" => "&#x92B8;",
    "銹" => "&#x92B9;",
    "銺" => "&#x92BA;",
    "銻" => "&#x92BB;",
    "銼" => "&#x92BC;",
    "銽" => "&#x92BD;",
    "銾" => "&#x92BE;",
    "銿" => "&#x92BF;",
    "鋀" => "&#x92C0;",
    "鋁" => "&#x92C1;",
    "鋂" => "&#x92C2;",
    "鋃" => "&#x92C3;",
    "鋄" => "&#x92C4;",
    "鋅" => "&#x92C5;",
    "鋆" => "&#x92C6;",
    "鋇" => "&#x92C7;",
    "鋈" => "&#x92C8;",
    "鋉" => "&#x92C9;",
    "鋊" => "&#x92CA;",
    "鋋" => "&#x92CB;",
    "鋌" => "&#x92CC;",
    "鋍" => "&#x92CD;",
    "鋎" => "&#x92CE;",
    "鋏" => "&#x92CF;",
    "鋐" => "&#x92D0;",
    "鋑" => "&#x92D1;",
    "鋒" => "&#x92D2;",
    "鋓" => "&#x92D3;",
    "鋔" => "&#x92D4;",
    "鋕" => "&#x92D5;",
    "鋖" => "&#x92D6;",
    "鋗" => "&#x92D7;",
    "鋘" => "&#x92D8;",
    "鋙" => "&#x92D9;",
    "鋚" => "&#x92DA;",
    "鋛" => "&#x92DB;",
    "鋜" => "&#x92DC;",
    "鋝" => "&#x92DD;",
    "鋞" => "&#x92DE;",
    "鋟" => "&#x92DF;",
    "鋠" => "&#x92E0;",
    "鋡" => "&#x92E1;",
    "鋢" => "&#x92E2;",
    "鋣" => "&#x92E3;",
    "鋤" => "&#x92E4;",
    "鋥" => "&#x92E5;",
    "鋦" => "&#x92E6;",
    "鋧" => "&#x92E7;",
    "鋨" => "&#x92E8;",
    "鋩" => "&#x92E9;",
    "鋪" => "&#x92EA;",
    "鋫" => "&#x92EB;",
    "鋬" => "&#x92EC;",
    "鋭" => "&#x92ED;",
    "鋮" => "&#x92EE;",
    "鋯" => "&#x92EF;",
    "鋰" => "&#x92F0;",
    "鋱" => "&#x92F1;",
    "鋲" => "&#x92F2;",
    "鋳" => "&#x92F3;",
    "鋴" => "&#x92F4;",
    "鋵" => "&#x92F5;",
    "鋶" => "&#x92F6;",
    "鋷" => "&#x92F7;",
    "鋸" => "&#x92F8;",
    "鋹" => "&#x92F9;",
    "鋺" => "&#x92FA;",
    "鋻" => "&#x92FB;",
    "鋼" => "&#x92FC;",
    "鋽" => "&#x92FD;",
    "鋾" => "&#x92FE;",
    "鋿" => "&#x92FF;",
    "錀" => "&#x9300;",
    "錁" => "&#x9301;",
    "錂" => "&#x9302;",
    "錃" => "&#x9303;",
    "錄" => "&#x9304;",
    "錅" => "&#x9305;",
    "錆" => "&#x9306;",
    "錇" => "&#x9307;",
    "錈" => "&#x9308;",
    "錉" => "&#x9309;",
    "錊" => "&#x930A;",
    "錋" => "&#x930B;",
    "錌" => "&#x930C;",
    "錍" => "&#x930D;",
    "錎" => "&#x930E;",
    "錏" => "&#x930F;",
    "錐" => "&#x9310;",
    "錑" => "&#x9311;",
    "錒" => "&#x9312;",
    "錓" => "&#x9313;",
    "錔" => "&#x9314;",
    "錕" => "&#x9315;",
    "錖" => "&#x9316;",
    "錗" => "&#x9317;",
    "錘" => "&#x9318;",
    "錙" => "&#x9319;",
    "錚" => "&#x931A;",
    "錛" => "&#x931B;",
    "錜" => "&#x931C;",
    "錝" => "&#x931D;",
    "錞" => "&#x931E;",
    "錟" => "&#x931F;",
    "錠" => "&#x9320;",
    "錡" => "&#x9321;",
    "錢" => "&#x9322;",
    "錣" => "&#x9323;",
    "錤" => "&#x9324;",
    "錥" => "&#x9325;",
    "錦" => "&#x9326;",
    "錧" => "&#x9327;",
    "錨" => "&#x9328;",
    "錩" => "&#x9329;",
    "錪" => "&#x932A;",
    "錫" => "&#x932B;",
    "錬" => "&#x932C;",
    "錭" => "&#x932D;",
    "錮" => "&#x932E;",
    "錯" => "&#x932F;",
    "錰" => "&#x9330;",
    "錱" => "&#x9331;",
    "録" => "&#x9332;",
    "錳" => "&#x9333;",
    "錴" => "&#x9334;",
    "錵" => "&#x9335;",
    "錶" => "&#x9336;",
    "錷" => "&#x9337;",
    "錸" => "&#x9338;",
    "錹" => "&#x9339;",
    "錺" => "&#x933A;",
    "錻" => "&#x933B;",
    "錼" => "&#x933C;",
    "錽" => "&#x933D;",
    "錾" => "&#x933E;",
    "錿" => "&#x933F;",
    "鍀" => "&#x9340;",
    "鍁" => "&#x9341;",
    "鍂" => "&#x9342;",
    "鍃" => "&#x9343;",
    "鍄" => "&#x9344;",
    "鍅" => "&#x9345;",
    "鍆" => "&#x9346;",
    "鍇" => "&#x9347;",
    "鍈" => "&#x9348;",
    "鍉" => "&#x9349;",
    "鍊" => "&#x934A;",
    "鍋" => "&#x934B;",
    "鍌" => "&#x934C;",
    "鍍" => "&#x934D;",
    "鍎" => "&#x934E;",
    "鍏" => "&#x934F;",
    "鍐" => "&#x9350;",
    "鍑" => "&#x9351;",
    "鍒" => "&#x9352;",
    "鍓" => "&#x9353;",
    "鍔" => "&#x9354;",
    "鍕" => "&#x9355;",
    "鍖" => "&#x9356;",
    "鍗" => "&#x9357;",
    "鍘" => "&#x9358;",
    "鍙" => "&#x9359;",
    "鍚" => "&#x935A;",
    "鍛" => "&#x935B;",
    "鍜" => "&#x935C;",
    "鍝" => "&#x935D;",
    "鍞" => "&#x935E;",
    "鍟" => "&#x935F;",
    "鍠" => "&#x9360;",
    "鍡" => "&#x9361;",
    "鍢" => "&#x9362;",
    "鍣" => "&#x9363;",
    "鍤" => "&#x9364;",
    "鍥" => "&#x9365;",
    "鍦" => "&#x9366;",
    "鍧" => "&#x9367;",
    "鍨" => "&#x9368;",
    "鍩" => "&#x9369;",
    "鍪" => "&#x936A;",
    "鍫" => "&#x936B;",
    "鍬" => "&#x936C;",
    "鍭" => "&#x936D;",
    "鍮" => "&#x936E;",
    "鍯" => "&#x936F;",
    "鍰" => "&#x9370;",
    "鍱" => "&#x9371;",
    "鍲" => "&#x9372;",
    "鍳" => "&#x9373;",
    "鍴" => "&#x9374;",
    "鍵" => "&#x9375;",
    "鍶" => "&#x9376;",
    "鍷" => "&#x9377;",
    "鍸" => "&#x9378;",
    "鍹" => "&#x9379;",
    "鍺" => "&#x937A;",
    "鍻" => "&#x937B;",
    "鍼" => "&#x937C;",
    "鍽" => "&#x937D;",
    "鍾" => "&#x937E;",
    "鍿" => "&#x937F;",
    "鎀" => "&#x9380;",
    "鎁" => "&#x9381;",
    "鎂" => "&#x9382;",
    "鎃" => "&#x9383;",
    "鎄" => "&#x9384;",
    "鎅" => "&#x9385;",
    "鎆" => "&#x9386;",
    "鎇" => "&#x9387;",
    "鎈" => "&#x9388;",
    "鎉" => "&#x9389;",
    "鎊" => "&#x938A;",
    "鎋" => "&#x938B;",
    "鎌" => "&#x938C;",
    "鎍" => "&#x938D;",
    "鎎" => "&#x938E;",
    "鎏" => "&#x938F;",
    "鎐" => "&#x9390;",
    "鎑" => "&#x9391;",
    "鎒" => "&#x9392;",
    "鎓" => "&#x9393;",
    "鎔" => "&#x9394;",
    "鎕" => "&#x9395;",
    "鎖" => "&#x9396;",
    "鎗" => "&#x9397;",
    "鎘" => "&#x9398;",
    "鎙" => "&#x9399;",
    "鎚" => "&#x939A;",
    "鎛" => "&#x939B;",
    "鎜" => "&#x939C;",
    "鎝" => "&#x939D;",
    "鎞" => "&#x939E;",
    "鎟" => "&#x939F;",
    "鎠" => "&#x93A0;",
    "鎡" => "&#x93A1;",
    "鎢" => "&#x93A2;",
    "鎣" => "&#x93A3;",
    "鎤" => "&#x93A4;",
    "鎥" => "&#x93A5;",
    "鎦" => "&#x93A6;",
    "鎧" => "&#x93A7;",
    "鎨" => "&#x93A8;",
    "鎩" => "&#x93A9;",
    "鎪" => "&#x93AA;",
    "鎫" => "&#x93AB;",
    "鎬" => "&#x93AC;",
    "鎭" => "&#x93AD;",
    "鎮" => "&#x93AE;",
    "鎯" => "&#x93AF;",
    "鎰" => "&#x93B0;",
    "鎱" => "&#x93B1;",
    "鎲" => "&#x93B2;",
    "鎳" => "&#x93B3;",
    "鎴" => "&#x93B4;",
    "鎵" => "&#x93B5;",
    "鎶" => "&#x93B6;",
    "鎷" => "&#x93B7;",
    "鎸" => "&#x93B8;",
    "鎹" => "&#x93B9;",
    "鎺" => "&#x93BA;",
    "鎻" => "&#x93BB;",
    "鎼" => "&#x93BC;",
    "鎽" => "&#x93BD;",
    "鎾" => "&#x93BE;",
    "鎿" => "&#x93BF;",
    "鏀" => "&#x93C0;",
    "鏁" => "&#x93C1;",
    "鏂" => "&#x93C2;",
    "鏃" => "&#x93C3;",
    "鏄" => "&#x93C4;",
    "鏅" => "&#x93C5;",
    "鏆" => "&#x93C6;",
    "鏇" => "&#x93C7;",
    "鏈" => "&#x93C8;",
    "鏉" => "&#x93C9;",
    "鏊" => "&#x93CA;",
    "鏋" => "&#x93CB;",
    "鏌" => "&#x93CC;",
    "鏍" => "&#x93CD;",
    "鏎" => "&#x93CE;",
    "鏏" => "&#x93CF;",
    "鏐" => "&#x93D0;",
    "鏑" => "&#x93D1;",
    "鏒" => "&#x93D2;",
    "鏓" => "&#x93D3;",
    "鏔" => "&#x93D4;",
    "鏕" => "&#x93D5;",
    "鏖" => "&#x93D6;",
    "鏗" => "&#x93D7;",
    "鏘" => "&#x93D8;",
    "鏙" => "&#x93D9;",
    "鏚" => "&#x93DA;",
    "鏛" => "&#x93DB;",
    "鏜" => "&#x93DC;",
    "鏝" => "&#x93DD;",
    "鏞" => "&#x93DE;",
    "鏟" => "&#x93DF;",
    "鏠" => "&#x93E0;",
    "鏡" => "&#x93E1;",
    "鏢" => "&#x93E2;",
    "鏣" => "&#x93E3;",
    "鏤" => "&#x93E4;",
    "鏥" => "&#x93E5;",
    "鏦" => "&#x93E6;",
    "鏧" => "&#x93E7;",
    "鏨" => "&#x93E8;",
    "鏩" => "&#x93E9;",
    "鏪" => "&#x93EA;",
    "鏫" => "&#x93EB;",
    "鏬" => "&#x93EC;",
    "鏭" => "&#x93ED;",
    "鏮" => "&#x93EE;",
    "鏯" => "&#x93EF;",
    "鏰" => "&#x93F0;",
    "鏱" => "&#x93F1;",
    "鏲" => "&#x93F2;",
    "鏳" => "&#x93F3;",
    "鏴" => "&#x93F4;",
    "鏵" => "&#x93F5;",
    "鏶" => "&#x93F6;",
    "鏷" => "&#x93F7;",
    "鏸" => "&#x93F8;",
    "鏹" => "&#x93F9;",
    "鏺" => "&#x93FA;",
    "鏻" => "&#x93FB;",
    "鏼" => "&#x93FC;",
    "鏽" => "&#x93FD;",
    "鏾" => "&#x93FE;",
    "鏿" => "&#x93FF;",
    "鐀" => "&#x9400;",
    "鐁" => "&#x9401;",
    "鐂" => "&#x9402;",
    "鐃" => "&#x9403;",
    "鐄" => "&#x9404;",
    "鐅" => "&#x9405;",
    "鐆" => "&#x9406;",
    "鐇" => "&#x9407;",
    "鐈" => "&#x9408;",
    "鐉" => "&#x9409;",
    "鐊" => "&#x940A;",
    "鐋" => "&#x940B;",
    "鐌" => "&#x940C;",
    "鐍" => "&#x940D;",
    "鐎" => "&#x940E;",
    "鐏" => "&#x940F;",
    "鐐" => "&#x9410;",
    "鐑" => "&#x9411;",
    "鐒" => "&#x9412;",
    "鐓" => "&#x9413;",
    "鐔" => "&#x9414;",
    "鐕" => "&#x9415;",
    "鐖" => "&#x9416;",
    "鐗" => "&#x9417;",
    "鐘" => "&#x9418;",
    "鐙" => "&#x9419;",
    "鐚" => "&#x941A;",
    "鐛" => "&#x941B;",
    "鐜" => "&#x941C;",
    "鐝" => "&#x941D;",
    "鐞" => "&#x941E;",
    "鐟" => "&#x941F;",
    "鐠" => "&#x9420;",
    "鐡" => "&#x9421;",
    "鐢" => "&#x9422;",
    "鐣" => "&#x9423;",
    "鐤" => "&#x9424;",
    "鐥" => "&#x9425;",
    "鐦" => "&#x9426;",
    "鐧" => "&#x9427;",
    "鐨" => "&#x9428;",
    "鐩" => "&#x9429;",
    "鐪" => "&#x942A;",
    "鐫" => "&#x942B;",
    "鐬" => "&#x942C;",
    "鐭" => "&#x942D;",
    "鐮" => "&#x942E;",
    "鐯" => "&#x942F;",
    "鐰" => "&#x9430;",
    "鐱" => "&#x9431;",
    "鐲" => "&#x9432;",
    "鐳" => "&#x9433;",
    "鐴" => "&#x9434;",
    "鐵" => "&#x9435;",
    "鐶" => "&#x9436;",
    "鐷" => "&#x9437;",
    "鐸" => "&#x9438;",
    "鐹" => "&#x9439;",
    "鐺" => "&#x943A;",
    "鐻" => "&#x943B;",
    "鐼" => "&#x943C;",
    "鐽" => "&#x943D;",
    "鐾" => "&#x943E;",
    "鐿" => "&#x943F;",
    "鑀" => "&#x9440;",
    "鑁" => "&#x9441;",
    "鑂" => "&#x9442;",
    "鑃" => "&#x9443;",
    "鑄" => "&#x9444;",
    "鑅" => "&#x9445;",
    "鑆" => "&#x9446;",
    "鑇" => "&#x9447;",
    "鑈" => "&#x9448;",
    "鑉" => "&#x9449;",
    "鑊" => "&#x944A;",
    "鑋" => "&#x944B;",
    "鑌" => "&#x944C;",
    "鑍" => "&#x944D;",
    "鑎" => "&#x944E;",
    "鑏" => "&#x944F;",
    "鑐" => "&#x9450;",
    "鑑" => "&#x9451;",
    "鑒" => "&#x9452;",
    "鑓" => "&#x9453;",
    "鑔" => "&#x9454;",
    "鑕" => "&#x9455;",
    "鑖" => "&#x9456;",
    "鑗" => "&#x9457;",
    "鑘" => "&#x9458;",
    "鑙" => "&#x9459;",
    "鑚" => "&#x945A;",
    "鑛" => "&#x945B;",
    "鑜" => "&#x945C;",
    "鑝" => "&#x945D;",
    "鑞" => "&#x945E;",
    "鑟" => "&#x945F;",
    "鑠" => "&#x9460;",
    "鑡" => "&#x9461;",
    "鑢" => "&#x9462;",
    "鑣" => "&#x9463;",
    "鑤" => "&#x9464;",
    "鑥" => "&#x9465;",
    "鑦" => "&#x9466;",
    "鑧" => "&#x9467;",
    "鑨" => "&#x9468;",
    "鑩" => "&#x9469;",
    "鑪" => "&#x946A;",
    "鑫" => "&#x946B;",
    "鑬" => "&#x946C;",
    "鑭" => "&#x946D;",
    "鑮" => "&#x946E;",
    "鑯" => "&#x946F;",
    "鑰" => "&#x9470;",
    "鑱" => "&#x9471;",
    "鑲" => "&#x9472;",
    "鑳" => "&#x9473;",
    "鑴" => "&#x9474;",
    "鑵" => "&#x9475;",
    "鑶" => "&#x9476;",
    "鑷" => "&#x9477;",
    "鑸" => "&#x9478;",
    "鑹" => "&#x9479;",
    "鑺" => "&#x947A;",
    "鑻" => "&#x947B;",
    "鑼" => "&#x947C;",
    "鑽" => "&#x947D;",
    "鑾" => "&#x947E;",
    "鑿" => "&#x947F;",
    "钀" => "&#x9480;",
    "钁" => "&#x9481;",
    "钂" => "&#x9482;",
    "钃" => "&#x9483;",
    "钄" => "&#x9484;",
    "钅" => "&#x9485;",
    "钆" => "&#x9486;",
    "钇" => "&#x9487;",
    "针" => "&#x9488;",
    "钉" => "&#x9489;",
    "钊" => "&#x948A;",
    "钋" => "&#x948B;",
    "钌" => "&#x948C;",
    "钍" => "&#x948D;",
    "钎" => "&#x948E;",
    "钏" => "&#x948F;",
    "钐" => "&#x9490;",
    "钑" => "&#x9491;",
    "钒" => "&#x9492;",
    "钓" => "&#x9493;",
    "钔" => "&#x9494;",
    "钕" => "&#x9495;",
    "钖" => "&#x9496;",
    "钗" => "&#x9497;",
    "钘" => "&#x9498;",
    "钙" => "&#x9499;",
    "钚" => "&#x949A;",
    "钛" => "&#x949B;",
    "钜" => "&#x949C;",
    "钝" => "&#x949D;",
    "钞" => "&#x949E;",
    "钟" => "&#x949F;",
    "钠" => "&#x94A0;",
    "钡" => "&#x94A1;",
    "钢" => "&#x94A2;",
    "钣" => "&#x94A3;",
    "钤" => "&#x94A4;",
    "钥" => "&#x94A5;",
    "钦" => "&#x94A6;",
    "钧" => "&#x94A7;",
    "钨" => "&#x94A8;",
    "钩" => "&#x94A9;",
    "钪" => "&#x94AA;",
    "钫" => "&#x94AB;",
    "钬" => "&#x94AC;",
    "钭" => "&#x94AD;",
    "钮" => "&#x94AE;",
    "钯" => "&#x94AF;",
    "钰" => "&#x94B0;",
    "钱" => "&#x94B1;",
    "钲" => "&#x94B2;",
    "钳" => "&#x94B3;",
    "钴" => "&#x94B4;",
    "钵" => "&#x94B5;",
    "钶" => "&#x94B6;",
    "钷" => "&#x94B7;",
    "钸" => "&#x94B8;",
    "钹" => "&#x94B9;",
    "钺" => "&#x94BA;",
    "钻" => "&#x94BB;",
    "钼" => "&#x94BC;",
    "钽" => "&#x94BD;",
    "钾" => "&#x94BE;",
    "钿" => "&#x94BF;",
    "铀" => "&#x94C0;",
    "铁" => "&#x94C1;",
    "铂" => "&#x94C2;",
    "铃" => "&#x94C3;",
    "铄" => "&#x94C4;",
    "铅" => "&#x94C5;",
    "铆" => "&#x94C6;",
    "铇" => "&#x94C7;",
    "铈" => "&#x94C8;",
    "铉" => "&#x94C9;",
    "铊" => "&#x94CA;",
    "铋" => "&#x94CB;",
    "铌" => "&#x94CC;",
    "铍" => "&#x94CD;",
    "铎" => "&#x94CE;",
    "铏" => "&#x94CF;",
    "铐" => "&#x94D0;",
    "铑" => "&#x94D1;",
    "铒" => "&#x94D2;",
    "铓" => "&#x94D3;",
    "铔" => "&#x94D4;",
    "铕" => "&#x94D5;",
    "铖" => "&#x94D6;",
    "铗" => "&#x94D7;",
    "铘" => "&#x94D8;",
    "铙" => "&#x94D9;",
    "铚" => "&#x94DA;",
    "铛" => "&#x94DB;",
    "铜" => "&#x94DC;",
    "铝" => "&#x94DD;",
    "铞" => "&#x94DE;",
    "铟" => "&#x94DF;",
    "铠" => "&#x94E0;",
    "铡" => "&#x94E1;",
    "铢" => "&#x94E2;",
    "铣" => "&#x94E3;",
    "铤" => "&#x94E4;",
    "铥" => "&#x94E5;",
    "铦" => "&#x94E6;",
    "铧" => "&#x94E7;",
    "铨" => "&#x94E8;",
    "铩" => "&#x94E9;",
    "铪" => "&#x94EA;",
    "铫" => "&#x94EB;",
    "铬" => "&#x94EC;",
    "铭" => "&#x94ED;",
    "铮" => "&#x94EE;",
    "铯" => "&#x94EF;",
    "铰" => "&#x94F0;",
    "铱" => "&#x94F1;",
    "铲" => "&#x94F2;",
    "铳" => "&#x94F3;",
    "铴" => "&#x94F4;",
    "铵" => "&#x94F5;",
    "银" => "&#x94F6;",
    "铷" => "&#x94F7;",
    "铸" => "&#x94F8;",
    "铹" => "&#x94F9;",
    "铺" => "&#x94FA;",
    "铻" => "&#x94FB;",
    "铼" => "&#x94FC;",
    "铽" => "&#x94FD;",
    "链" => "&#x94FE;",
    "铿" => "&#x94FF;",
    "销" => "&#x9500;",
    "锁" => "&#x9501;",
    "锂" => "&#x9502;",
    "锃" => "&#x9503;",
    "锄" => "&#x9504;",
    "锅" => "&#x9505;",
    "锆" => "&#x9506;",
    "锇" => "&#x9507;",
    "锈" => "&#x9508;",
    "锉" => "&#x9509;",
    "锊" => "&#x950A;",
    "锋" => "&#x950B;",
    "锌" => "&#x950C;",
    "锍" => "&#x950D;",
    "锎" => "&#x950E;",
    "锏" => "&#x950F;",
    "锐" => "&#x9510;",
    "锑" => "&#x9511;",
    "锒" => "&#x9512;",
    "锓" => "&#x9513;",
    "锔" => "&#x9514;",
    "锕" => "&#x9515;",
    "锖" => "&#x9516;",
    "锗" => "&#x9517;",
    "锘" => "&#x9518;",
    "错" => "&#x9519;",
    "锚" => "&#x951A;",
    "锛" => "&#x951B;",
    "锜" => "&#x951C;",
    "锝" => "&#x951D;",
    "锞" => "&#x951E;",
    "锟" => "&#x951F;",
    "锠" => "&#x9520;",
    "锡" => "&#x9521;",
    "锢" => "&#x9522;",
    "锣" => "&#x9523;",
    "锤" => "&#x9524;",
    "锥" => "&#x9525;",
    "锦" => "&#x9526;",
    "锧" => "&#x9527;",
    "锨" => "&#x9528;",
    "锩" => "&#x9529;",
    "锪" => "&#x952A;",
    "锫" => "&#x952B;",
    "锬" => "&#x952C;",
    "锭" => "&#x952D;",
    "键" => "&#x952E;",
    "锯" => "&#x952F;",
    "锰" => "&#x9530;",
    "锱" => "&#x9531;",
    "锲" => "&#x9532;",
    "锳" => "&#x9533;",
    "锴" => "&#x9534;",
    "锵" => "&#x9535;",
    "锶" => "&#x9536;",
    "锷" => "&#x9537;",
    "锸" => "&#x9538;",
    "锹" => "&#x9539;",
    "锺" => "&#x953A;",
    "锻" => "&#x953B;",
    "锼" => "&#x953C;",
    "锽" => "&#x953D;",
    "锾" => "&#x953E;",
    "锿" => "&#x953F;",
    "镀" => "&#x9540;",
    "镁" => "&#x9541;",
    "镂" => "&#x9542;",
    "镃" => "&#x9543;",
    "镄" => "&#x9544;",
    "镅" => "&#x9545;",
    "镆" => "&#x9546;",
    "镇" => "&#x9547;",
    "镈" => "&#x9548;",
    "镉" => "&#x9549;",
    "镊" => "&#x954A;",
    "镋" => "&#x954B;",
    "镌" => "&#x954C;",
    "镍" => "&#x954D;",
    "镎" => "&#x954E;",
    "镏" => "&#x954F;",
    "镐" => "&#x9550;",
    "镑" => "&#x9551;",
    "镒" => "&#x9552;",
    "镓" => "&#x9553;",
    "镔" => "&#x9554;",
    "镕" => "&#x9555;",
    "镖" => "&#x9556;",
    "镗" => "&#x9557;",
    "镘" => "&#x9558;",
    "镙" => "&#x9559;",
    "镚" => "&#x955A;",
    "镛" => "&#x955B;",
    "镜" => "&#x955C;",
    "镝" => "&#x955D;",
    "镞" => "&#x955E;",
    "镟" => "&#x955F;",
    "镠" => "&#x9560;",
    "镡" => "&#x9561;",
    "镢" => "&#x9562;",
    "镣" => "&#x9563;",
    "镤" => "&#x9564;",
    "镥" => "&#x9565;",
    "镦" => "&#x9566;",
    "镧" => "&#x9567;",
    "镨" => "&#x9568;",
    "镩" => "&#x9569;",
    "镪" => "&#x956A;",
    "镫" => "&#x956B;",
    "镬" => "&#x956C;",
    "镭" => "&#x956D;",
    "镮" => "&#x956E;",
    "镯" => "&#x956F;",
    "镰" => "&#x9570;",
    "镱" => "&#x9571;",
    "镲" => "&#x9572;",
    "镳" => "&#x9573;",
    "镴" => "&#x9574;",
    "镵" => "&#x9575;",
    "镶" => "&#x9576;",
    "長" => "&#x9577;",
    "镸" => "&#x9578;",
    "镹" => "&#x9579;",
    "镺" => "&#x957A;",
    "镻" => "&#x957B;",
    "镼" => "&#x957C;",
    "镽" => "&#x957D;",
    "镾" => "&#x957E;",
    "长" => "&#x957F;",
    "門" => "&#x9580;",
    "閁" => "&#x9581;",
    "閂" => "&#x9582;",
    "閃" => "&#x9583;",
    "閄" => "&#x9584;",
    "閅" => "&#x9585;",
    "閆" => "&#x9586;",
    "閇" => "&#x9587;",
    "閈" => "&#x9588;",
    "閉" => "&#x9589;",
    "閊" => "&#x958A;",
    "開" => "&#x958B;",
    "閌" => "&#x958C;",
    "閍" => "&#x958D;",
    "閎" => "&#x958E;",
    "閏" => "&#x958F;",
    "閐" => "&#x9590;",
    "閑" => "&#x9591;",
    "閒" => "&#x9592;",
    "間" => "&#x9593;",
    "閔" => "&#x9594;",
    "閕" => "&#x9595;",
    "閖" => "&#x9596;",
    "閗" => "&#x9597;",
    "閘" => "&#x9598;",
    "閙" => "&#x9599;",
    "閚" => "&#x959A;",
    "閛" => "&#x959B;",
    "閜" => "&#x959C;",
    "閝" => "&#x959D;",
    "閞" => "&#x959E;",
    "閟" => "&#x959F;",
    "閠" => "&#x95A0;",
    "閡" => "&#x95A1;",
    "関" => "&#x95A2;",
    "閣" => "&#x95A3;",
    "閤" => "&#x95A4;",
    "閥" => "&#x95A5;",
    "閦" => "&#x95A6;",
    "閧" => "&#x95A7;",
    "閨" => "&#x95A8;",
    "閩" => "&#x95A9;",
    "閪" => "&#x95AA;",
    "閫" => "&#x95AB;",
    "閬" => "&#x95AC;",
    "閭" => "&#x95AD;",
    "閮" => "&#x95AE;",
    "閯" => "&#x95AF;",
    "閰" => "&#x95B0;",
    "閱" => "&#x95B1;",
    "閲" => "&#x95B2;",
    "閳" => "&#x95B3;",
    "閴" => "&#x95B4;",
    "閵" => "&#x95B5;",
    "閶" => "&#x95B6;",
    "閷" => "&#x95B7;",
    "閸" => "&#x95B8;",
    "閹" => "&#x95B9;",
    "閺" => "&#x95BA;",
    "閻" => "&#x95BB;",
    "閼" => "&#x95BC;",
    "閽" => "&#x95BD;",
    "閾" => "&#x95BE;",
    "閿" => "&#x95BF;",
    "闀" => "&#x95C0;",
    "闁" => "&#x95C1;",
    "闂" => "&#x95C2;",
    "闃" => "&#x95C3;",
    "闄" => "&#x95C4;",
    "闅" => "&#x95C5;",
    "闆" => "&#x95C6;",
    "闇" => "&#x95C7;",
    "闈" => "&#x95C8;",
    "闉" => "&#x95C9;",
    "闊" => "&#x95CA;",
    "闋" => "&#x95CB;",
    "闌" => "&#x95CC;",
    "闍" => "&#x95CD;",
    "闎" => "&#x95CE;",
    "闏" => "&#x95CF;",
    "闐" => "&#x95D0;",
    "闑" => "&#x95D1;",
    "闒" => "&#x95D2;",
    "闓" => "&#x95D3;",
    "闔" => "&#x95D4;",
    "闕" => "&#x95D5;",
    "闖" => "&#x95D6;",
    "闗" => "&#x95D7;",
    "闘" => "&#x95D8;",
    "闙" => "&#x95D9;",
    "闚" => "&#x95DA;",
    "闛" => "&#x95DB;",
    "關" => "&#x95DC;",
    "闝" => "&#x95DD;",
    "闞" => "&#x95DE;",
    "闟" => "&#x95DF;",
    "闠" => "&#x95E0;",
    "闡" => "&#x95E1;",
    "闢" => "&#x95E2;",
    "闣" => "&#x95E3;",
    "闤" => "&#x95E4;",
    "闥" => "&#x95E5;",
    "闦" => "&#x95E6;",
    "闧" => "&#x95E7;",
    "门" => "&#x95E8;",
    "闩" => "&#x95E9;",
    "闪" => "&#x95EA;",
    "闫" => "&#x95EB;",
    "闬" => "&#x95EC;",
    "闭" => "&#x95ED;",
    "问" => "&#x95EE;",
    "闯" => "&#x95EF;",
    "闰" => "&#x95F0;",
    "闱" => "&#x95F1;",
    "闲" => "&#x95F2;",
    "闳" => "&#x95F3;",
    "间" => "&#x95F4;",
    "闵" => "&#x95F5;",
    "闶" => "&#x95F6;",
    "闷" => "&#x95F7;",
    "闸" => "&#x95F8;",
    "闹" => "&#x95F9;",
    "闺" => "&#x95FA;",
    "闻" => "&#x95FB;",
    "闼" => "&#x95FC;",
    "闽" => "&#x95FD;",
    "闾" => "&#x95FE;",
    "闿" => "&#x95FF;",
    "阀" => "&#x9600;",
    "阁" => "&#x9601;",
    "阂" => "&#x9602;",
    "阃" => "&#x9603;",
    "阄" => "&#x9604;",
    "阅" => "&#x9605;",
    "阆" => "&#x9606;",
    "阇" => "&#x9607;",
    "阈" => "&#x9608;",
    "阉" => "&#x9609;",
    "阊" => "&#x960A;",
    "阋" => "&#x960B;",
    "阌" => "&#x960C;",
    "阍" => "&#x960D;",
    "阎" => "&#x960E;",
    "阏" => "&#x960F;",
    "阐" => "&#x9610;",
    "阑" => "&#x9611;",
    "阒" => "&#x9612;",
    "阓" => "&#x9613;",
    "阔" => "&#x9614;",
    "阕" => "&#x9615;",
    "阖" => "&#x9616;",
    "阗" => "&#x9617;",
    "阘" => "&#x9618;",
    "阙" => "&#x9619;",
    "阚" => "&#x961A;",
    "阛" => "&#x961B;",
    "阜" => "&#x961C;",
    "阝" => "&#x961D;",
    "阞" => "&#x961E;",
    "队" => "&#x961F;",
    "阠" => "&#x9620;",
    "阡" => "&#x9621;",
    "阢" => "&#x9622;",
    "阣" => "&#x9623;",
    "阤" => "&#x9624;",
    "阥" => "&#x9625;",
    "阦" => "&#x9626;",
    "阧" => "&#x9627;",
    "阨" => "&#x9628;",
    "阩" => "&#x9629;",
    "阪" => "&#x962A;",
    "阫" => "&#x962B;",
    "阬" => "&#x962C;",
    "阭" => "&#x962D;",
    "阮" => "&#x962E;",
    "阯" => "&#x962F;",
    "阰" => "&#x9630;",
    "阱" => "&#x9631;",
    "防" => "&#x9632;",
    "阳" => "&#x9633;",
    "阴" => "&#x9634;",
    "阵" => "&#x9635;",
    "阶" => "&#x9636;",
    "阷" => "&#x9637;",
    "阸" => "&#x9638;",
    "阹" => "&#x9639;",
    "阺" => "&#x963A;",
    "阻" => "&#x963B;",
    "阼" => "&#x963C;",
    "阽" => "&#x963D;",
    "阾" => "&#x963E;",
    "阿" => "&#x963F;",
    "陀" => "&#x9640;",
    "陁" => "&#x9641;",
    "陂" => "&#x9642;",
    "陃" => "&#x9643;",
    "附" => "&#x9644;",
    "际" => "&#x9645;",
    "陆" => "&#x9646;",
    "陇" => "&#x9647;",
    "陈" => "&#x9648;",
    "陉" => "&#x9649;",
    "陊" => "&#x964A;",
    "陋" => "&#x964B;",
    "陌" => "&#x964C;",
    "降" => "&#x964D;",
    "陎" => "&#x964E;",
    "陏" => "&#x964F;",
    "限" => "&#x9650;",
    "陑" => "&#x9651;",
    "陒" => "&#x9652;",
    "陓" => "&#x9653;",
    "陔" => "&#x9654;",
    "陕" => "&#x9655;",
    "陖" => "&#x9656;",
    "陗" => "&#x9657;",
    "陘" => "&#x9658;",
    "陙" => "&#x9659;",
    "陚" => "&#x965A;",
    "陛" => "&#x965B;",
    "陜" => "&#x965C;",
    "陝" => "&#x965D;",
    "陞" => "&#x965E;",
    "陟" => "&#x965F;",
    "陠" => "&#x9660;",
    "陡" => "&#x9661;",
    "院" => "&#x9662;",
    "陣" => "&#x9663;",
    "除" => "&#x9664;",
    "陥" => "&#x9665;",
    "陦" => "&#x9666;",
    "陧" => "&#x9667;",
    "陨" => "&#x9668;",
    "险" => "&#x9669;",
    "陪" => "&#x966A;",
    "陫" => "&#x966B;",
    "陬" => "&#x966C;",
    "陭" => "&#x966D;",
    "陮" => "&#x966E;",
    "陯" => "&#x966F;",
    "陰" => "&#x9670;",
    "陱" => "&#x9671;",
    "陲" => "&#x9672;",
    "陳" => "&#x9673;",
    "陴" => "&#x9674;",
    "陵" => "&#x9675;",
    "陶" => "&#x9676;",
    "陷" => "&#x9677;",
    "陸" => "&#x9678;",
    "陹" => "&#x9679;",
    "険" => "&#x967A;",
    "陻" => "&#x967B;",
    "陼" => "&#x967C;",
    "陽" => "&#x967D;",
    "陾" => "&#x967E;",
    "陿" => "&#x967F;",
    "隀" => "&#x9680;",
    "隁" => "&#x9681;",
    "隂" => "&#x9682;",
    "隃" => "&#x9683;",
    "隄" => "&#x9684;",
    "隅" => "&#x9685;",
    "隆" => "&#x9686;",
    "隇" => "&#x9687;",
    "隈" => "&#x9688;",
    "隉" => "&#x9689;",
    "隊" => "&#x968A;",
    "隋" => "&#x968B;",
    "隌" => "&#x968C;",
    "隍" => "&#x968D;",
    "階" => "&#x968E;",
    "随" => "&#x968F;",
    "隐" => "&#x9690;",
    "隑" => "&#x9691;",
    "隒" => "&#x9692;",
    "隓" => "&#x9693;",
    "隔" => "&#x9694;",
    "隕" => "&#x9695;",
    "隖" => "&#x9696;",
    "隗" => "&#x9697;",
    "隘" => "&#x9698;",
    "隙" => "&#x9699;",
    "隚" => "&#x969A;",
    "際" => "&#x969B;",
    "障" => "&#x969C;",
    "隝" => "&#x969D;",
    "隞" => "&#x969E;",
    "隟" => "&#x969F;",
    "隠" => "&#x96A0;",
    "隡" => "&#x96A1;",
    "隢" => "&#x96A2;",
    "隣" => "&#x96A3;",
    "隤" => "&#x96A4;",
    "隥" => "&#x96A5;",
    "隦" => "&#x96A6;",
    "隧" => "&#x96A7;",
    "隨" => "&#x96A8;",
    "隩" => "&#x96A9;",
    "險" => "&#x96AA;",
    "隫" => "&#x96AB;",
    "隬" => "&#x96AC;",
    "隭" => "&#x96AD;",
    "隮" => "&#x96AE;",
    "隯" => "&#x96AF;",
    "隰" => "&#x96B0;",
    "隱" => "&#x96B1;",
    "隲" => "&#x96B2;",
    "隳" => "&#x96B3;",
    "隴" => "&#x96B4;",
    "隵" => "&#x96B5;",
    "隶" => "&#x96B6;",
    "隷" => "&#x96B7;",
    "隸" => "&#x96B8;",
    "隹" => "&#x96B9;",
    "隺" => "&#x96BA;",
    "隻" => "&#x96BB;",
    "隼" => "&#x96BC;",
    "隽" => "&#x96BD;",
    "难" => "&#x96BE;",
    "隿" => "&#x96BF;",
    "雀" => "&#x96C0;",
    "雁" => "&#x96C1;",
    "雂" => "&#x96C2;",
    "雃" => "&#x96C3;",
    "雄" => "&#x96C4;",
    "雅" => "&#x96C5;",
    "集" => "&#x96C6;",
    "雇" => "&#x96C7;",
    "雈" => "&#x96C8;",
    "雉" => "&#x96C9;",
    "雊" => "&#x96CA;",
    "雋" => "&#x96CB;",
    "雌" => "&#x96CC;",
    "雍" => "&#x96CD;",
    "雎" => "&#x96CE;",
    "雏" => "&#x96CF;",
    "雐" => "&#x96D0;",
    "雑" => "&#x96D1;",
    "雒" => "&#x96D2;",
    "雓" => "&#x96D3;",
    "雔" => "&#x96D4;",
    "雕" => "&#x96D5;",
    "雖" => "&#x96D6;",
    "雗" => "&#x96D7;",
    "雘" => "&#x96D8;",
    "雙" => "&#x96D9;",
    "雚" => "&#x96DA;",
    "雛" => "&#x96DB;",
    "雜" => "&#x96DC;",
    "雝" => "&#x96DD;",
    "雞" => "&#x96DE;",
    "雟" => "&#x96DF;",
    "雠" => "&#x96E0;",
    "雡" => "&#x96E1;",
    "離" => "&#x96E2;",
    "難" => "&#x96E3;",
    "雤" => "&#x96E4;",
    "雥" => "&#x96E5;",
    "雦" => "&#x96E6;",
    "雧" => "&#x96E7;",
    "雨" => "&#x96E8;",
    "雩" => "&#x96E9;",
    "雪" => "&#x96EA;",
    "雫" => "&#x96EB;",
    "雬" => "&#x96EC;",
    "雭" => "&#x96ED;",
    "雮" => "&#x96EE;",
    "雯" => "&#x96EF;",
    "雰" => "&#x96F0;",
    "雱" => "&#x96F1;",
    "雲" => "&#x96F2;",
    "雳" => "&#x96F3;",
    "雴" => "&#x96F4;",
    "雵" => "&#x96F5;",
    "零" => "&#x96F6;",
    "雷" => "&#x96F7;",
    "雸" => "&#x96F8;",
    "雹" => "&#x96F9;",
    "雺" => "&#x96FA;",
    "電" => "&#x96FB;",
    "雼" => "&#x96FC;",
    "雽" => "&#x96FD;",
    "雾" => "&#x96FE;",
    "雿" => "&#x96FF;",
    "需" => "&#x9700;",
    "霁" => "&#x9701;",
    "霂" => "&#x9702;",
    "霃" => "&#x9703;",
    "霄" => "&#x9704;",
    "霅" => "&#x9705;",
    "霆" => "&#x9706;",
    "震" => "&#x9707;",
    "霈" => "&#x9708;",
    "霉" => "&#x9709;",
    "霊" => "&#x970A;",
    "霋" => "&#x970B;",
    "霌" => "&#x970C;",
    "霍" => "&#x970D;",
    "霎" => "&#x970E;",
    "霏" => "&#x970F;",
    "霐" => "&#x9710;",
    "霑" => "&#x9711;",
    "霒" => "&#x9712;",
    "霓" => "&#x9713;",
    "霔" => "&#x9714;",
    "霕" => "&#x9715;",
    "霖" => "&#x9716;",
    "霗" => "&#x9717;",
    "霘" => "&#x9718;",
    "霙" => "&#x9719;",
    "霚" => "&#x971A;",
    "霛" => "&#x971B;",
    "霜" => "&#x971C;",
    "霝" => "&#x971D;",
    "霞" => "&#x971E;",
    "霟" => "&#x971F;",
    "霠" => "&#x9720;",
    "霡" => "&#x9721;",
    "霢" => "&#x9722;",
    "霣" => "&#x9723;",
    "霤" => "&#x9724;",
    "霥" => "&#x9725;",
    "霦" => "&#x9726;",
    "霧" => "&#x9727;",
    "霨" => "&#x9728;",
    "霩" => "&#x9729;",
    "霪" => "&#x972A;",
    "霫" => "&#x972B;",
    "霬" => "&#x972C;",
    "霭" => "&#x972D;",
    "霮" => "&#x972E;",
    "霯" => "&#x972F;",
    "霰" => "&#x9730;",
    "霱" => "&#x9731;",
    "露" => "&#x9732;",
    "霳" => "&#x9733;",
    "霴" => "&#x9734;",
    "霵" => "&#x9735;",
    "霶" => "&#x9736;",
    "霷" => "&#x9737;",
    "霸" => "&#x9738;",
    "霹" => "&#x9739;",
    "霺" => "&#x973A;",
    "霻" => "&#x973B;",
    "霼" => "&#x973C;",
    "霽" => "&#x973D;",
    "霾" => "&#x973E;",
    "霿" => "&#x973F;",
    "靀" => "&#x9740;",
    "靁" => "&#x9741;",
    "靂" => "&#x9742;",
    "靃" => "&#x9743;",
    "靄" => "&#x9744;",
    "靅" => "&#x9745;",
    "靆" => "&#x9746;",
    "靇" => "&#x9747;",
    "靈" => "&#x9748;",
    "靉" => "&#x9749;",
    "靊" => "&#x974A;",
    "靋" => "&#x974B;",
    "靌" => "&#x974C;",
    "靍" => "&#x974D;",
    "靎" => "&#x974E;",
    "靏" => "&#x974F;",
    "靐" => "&#x9750;",
    "靑" => "&#x9751;",
    "青" => "&#x9752;",
    "靓" => "&#x9753;",
    "靔" => "&#x9754;",
    "靕" => "&#x9755;",
    "靖" => "&#x9756;",
    "靗" => "&#x9757;",
    "靘" => "&#x9758;",
    "静" => "&#x9759;",
    "靚" => "&#x975A;",
    "靛" => "&#x975B;",
    "靜" => "&#x975C;",
    "靝" => "&#x975D;",
    "非" => "&#x975E;",
    "靟" => "&#x975F;",
    "靠" => "&#x9760;",
    "靡" => "&#x9761;",
    "面" => "&#x9762;",
    "靣" => "&#x9763;",
    "靤" => "&#x9764;",
    "靥" => "&#x9765;",
    "靦" => "&#x9766;",
    "靧" => "&#x9767;",
    "靨" => "&#x9768;",
    "革" => "&#x9769;",
    "靪" => "&#x976A;",
    "靫" => "&#x976B;",
    "靬" => "&#x976C;",
    "靭" => "&#x976D;",
    "靮" => "&#x976E;",
    "靯" => "&#x976F;",
    "靰" => "&#x9770;",
    "靱" => "&#x9771;",
    "靲" => "&#x9772;",
    "靳" => "&#x9773;",
    "靴" => "&#x9774;",
    "靵" => "&#x9775;",
    "靶" => "&#x9776;",
    "靷" => "&#x9777;",
    "靸" => "&#x9778;",
    "靹" => "&#x9779;",
    "靺" => "&#x977A;",
    "靻" => "&#x977B;",
    "靼" => "&#x977C;",
    "靽" => "&#x977D;",
    "靾" => "&#x977E;",
    "靿" => "&#x977F;",
    "鞀" => "&#x9780;",
    "鞁" => "&#x9781;",
    "鞂" => "&#x9782;",
    "鞃" => "&#x9783;",
    "鞄" => "&#x9784;",
    "鞅" => "&#x9785;",
    "鞆" => "&#x9786;",
    "鞇" => "&#x9787;",
    "鞈" => "&#x9788;",
    "鞉" => "&#x9789;",
    "鞊" => "&#x978A;",
    "鞋" => "&#x978B;",
    "鞌" => "&#x978C;",
    "鞍" => "&#x978D;",
    "鞎" => "&#x978E;",
    "鞏" => "&#x978F;",
    "鞐" => "&#x9790;",
    "鞑" => "&#x9791;",
    "鞒" => "&#x9792;",
    "鞓" => "&#x9793;",
    "鞔" => "&#x9794;",
    "鞕" => "&#x9795;",
    "鞖" => "&#x9796;",
    "鞗" => "&#x9797;",
    "鞘" => "&#x9798;",
    "鞙" => "&#x9799;",
    "鞚" => "&#x979A;",
    "鞛" => "&#x979B;",
    "鞜" => "&#x979C;",
    "鞝" => "&#x979D;",
    "鞞" => "&#x979E;",
    "鞟" => "&#x979F;",
    "鞠" => "&#x97A0;",
    "鞡" => "&#x97A1;",
    "鞢" => "&#x97A2;",
    "鞣" => "&#x97A3;",
    "鞤" => "&#x97A4;",
    "鞥" => "&#x97A5;",
    "鞦" => "&#x97A6;",
    "鞧" => "&#x97A7;",
    "鞨" => "&#x97A8;",
    "鞩" => "&#x97A9;",
    "鞪" => "&#x97AA;",
    "鞫" => "&#x97AB;",
    "鞬" => "&#x97AC;",
    "鞭" => "&#x97AD;",
    "鞮" => "&#x97AE;",
    "鞯" => "&#x97AF;",
    "鞰" => "&#x97B0;",
    "鞱" => "&#x97B1;",
    "鞲" => "&#x97B2;",
    "鞳" => "&#x97B3;",
    "鞴" => "&#x97B4;",
    "鞵" => "&#x97B5;",
    "鞶" => "&#x97B6;",
    "鞷" => "&#x97B7;",
    "鞸" => "&#x97B8;",
    "鞹" => "&#x97B9;",
    "鞺" => "&#x97BA;",
    "鞻" => "&#x97BB;",
    "鞼" => "&#x97BC;",
    "鞽" => "&#x97BD;",
    "鞾" => "&#x97BE;",
    "鞿" => "&#x97BF;",
    "韀" => "&#x97C0;",
    "韁" => "&#x97C1;",
    "韂" => "&#x97C2;",
    "韃" => "&#x97C3;",
    "韄" => "&#x97C4;",
    "韅" => "&#x97C5;",
    "韆" => "&#x97C6;",
    "韇" => "&#x97C7;",
    "韈" => "&#x97C8;",
    "韉" => "&#x97C9;",
    "韊" => "&#x97CA;",
    "韋" => "&#x97CB;",
    "韌" => "&#x97CC;",
    "韍" => "&#x97CD;",
    "韎" => "&#x97CE;",
    "韏" => "&#x97CF;",
    "韐" => "&#x97D0;",
    "韑" => "&#x97D1;",
    "韒" => "&#x97D2;",
    "韓" => "&#x97D3;",
    "韔" => "&#x97D4;",
    "韕" => "&#x97D5;",
    "韖" => "&#x97D6;",
    "韗" => "&#x97D7;",
    "韘" => "&#x97D8;",
    "韙" => "&#x97D9;",
    "韚" => "&#x97DA;",
    "韛" => "&#x97DB;",
    "韜" => "&#x97DC;",
    "韝" => "&#x97DD;",
    "韞" => "&#x97DE;",
    "韟" => "&#x97DF;",
    "韠" => "&#x97E0;",
    "韡" => "&#x97E1;",
    "韢" => "&#x97E2;",
    "韣" => "&#x97E3;",
    "韤" => "&#x97E4;",
    "韥" => "&#x97E5;",
    "韦" => "&#x97E6;",
    "韧" => "&#x97E7;",
    "韨" => "&#x97E8;",
    "韩" => "&#x97E9;",
    "韪" => "&#x97EA;",
    "韫" => "&#x97EB;",
    "韬" => "&#x97EC;",
    "韭" => "&#x97ED;",
    "韮" => "&#x97EE;",
    "韯" => "&#x97EF;",
    "韰" => "&#x97F0;",
    "韱" => "&#x97F1;",
    "韲" => "&#x97F2;",
    "音" => "&#x97F3;",
    "韴" => "&#x97F4;",
    "韵" => "&#x97F5;",
    "韶" => "&#x97F6;",
    "韷" => "&#x97F7;",
    "韸" => "&#x97F8;",
    "韹" => "&#x97F9;",
    "韺" => "&#x97FA;",
    "韻" => "&#x97FB;",
    "韼" => "&#x97FC;",
    "韽" => "&#x97FD;",
    "韾" => "&#x97FE;",
    "響" => "&#x97FF;",
    "頀" => "&#x9800;",
    "頁" => "&#x9801;",
    "頂" => "&#x9802;",
    "頃" => "&#x9803;",
    "頄" => "&#x9804;",
    "項" => "&#x9805;",
    "順" => "&#x9806;",
    "頇" => "&#x9807;",
    "須" => "&#x9808;",
    "頉" => "&#x9809;",
    "頊" => "&#x980A;",
    "頋" => "&#x980B;",
    "頌" => "&#x980C;",
    "頍" => "&#x980D;",
    "頎" => "&#x980E;",
    "頏" => "&#x980F;",
    "預" => "&#x9810;",
    "頑" => "&#x9811;",
    "頒" => "&#x9812;",
    "頓" => "&#x9813;",
    "頔" => "&#x9814;",
    "頕" => "&#x9815;",
    "頖" => "&#x9816;",
    "頗" => "&#x9817;",
    "領" => "&#x9818;",
    "頙" => "&#x9819;",
    "頚" => "&#x981A;",
    "頛" => "&#x981B;",
    "頜" => "&#x981C;",
    "頝" => "&#x981D;",
    "頞" => "&#x981E;",
    "頟" => "&#x981F;",
    "頠" => "&#x9820;",
    "頡" => "&#x9821;",
    "頢" => "&#x9822;",
    "頣" => "&#x9823;",
    "頤" => "&#x9824;",
    "頥" => "&#x9825;",
    "頦" => "&#x9826;",
    "頧" => "&#x9827;",
    "頨" => "&#x9828;",
    "頩" => "&#x9829;",
    "頪" => "&#x982A;",
    "頫" => "&#x982B;",
    "頬" => "&#x982C;",
    "頭" => "&#x982D;",
    "頮" => "&#x982E;",
    "頯" => "&#x982F;",
    "頰" => "&#x9830;",
    "頱" => "&#x9831;",
    "頲" => "&#x9832;",
    "頳" => "&#x9833;",
    "頴" => "&#x9834;",
    "頵" => "&#x9835;",
    "頶" => "&#x9836;",
    "頷" => "&#x9837;",
    "頸" => "&#x9838;",
    "頹" => "&#x9839;",
    "頺" => "&#x983A;",
    "頻" => "&#x983B;",
    "頼" => "&#x983C;",
    "頽" => "&#x983D;",
    "頾" => "&#x983E;",
    "頿" => "&#x983F;",
    "顀" => "&#x9840;",
    "顁" => "&#x9841;",
    "顂" => "&#x9842;",
    "顃" => "&#x9843;",
    "顄" => "&#x9844;",
    "顅" => "&#x9845;",
    "顆" => "&#x9846;",
    "顇" => "&#x9847;",
    "顈" => "&#x9848;",
    "顉" => "&#x9849;",
    "顊" => "&#x984A;",
    "顋" => "&#x984B;",
    "題" => "&#x984C;",
    "額" => "&#x984D;",
    "顎" => "&#x984E;",
    "顏" => "&#x984F;",
    "顐" => "&#x9850;",
    "顑" => "&#x9851;",
    "顒" => "&#x9852;",
    "顓" => "&#x9853;",
    "顔" => "&#x9854;",
    "顕" => "&#x9855;",
    "顖" => "&#x9856;",
    "顗" => "&#x9857;",
    "願" => "&#x9858;",
    "顙" => "&#x9859;",
    "顚" => "&#x985A;",
    "顛" => "&#x985B;",
    "顜" => "&#x985C;",
    "顝" => "&#x985D;",
    "類" => "&#x985E;",
    "顟" => "&#x985F;",
    "顠" => "&#x9860;",
    "顡" => "&#x9861;",
    "顢" => "&#x9862;",
    "顣" => "&#x9863;",
    "顤" => "&#x9864;",
    "顥" => "&#x9865;",
    "顦" => "&#x9866;",
    "顧" => "&#x9867;",
    "顨" => "&#x9868;",
    "顩" => "&#x9869;",
    "顪" => "&#x986A;",
    "顫" => "&#x986B;",
    "顬" => "&#x986C;",
    "顭" => "&#x986D;",
    "顮" => "&#x986E;",
    "顯" => "&#x986F;",
    "顰" => "&#x9870;",
    "顱" => "&#x9871;",
    "顲" => "&#x9872;",
    "顳" => "&#x9873;",
    "顴" => "&#x9874;",
    "页" => "&#x9875;",
    "顶" => "&#x9876;",
    "顷" => "&#x9877;",
    "顸" => "&#x9878;",
    "项" => "&#x9879;",
    "顺" => "&#x987A;",
    "须" => "&#x987B;",
    "顼" => "&#x987C;",
    "顽" => "&#x987D;",
    "顾" => "&#x987E;",
    "顿" => "&#x987F;",
    "颀" => "&#x9880;",
    "颁" => "&#x9881;",
    "颂" => "&#x9882;",
    "颃" => "&#x9883;",
    "预" => "&#x9884;",
    "颅" => "&#x9885;",
    "领" => "&#x9886;",
    "颇" => "&#x9887;",
    "颈" => "&#x9888;",
    "颉" => "&#x9889;",
    "颊" => "&#x988A;",
    "颋" => "&#x988B;",
    "颌" => "&#x988C;",
    "颍" => "&#x988D;",
    "颎" => "&#x988E;",
    "颏" => "&#x988F;",
    "颐" => "&#x9890;",
    "频" => "&#x9891;",
    "颒" => "&#x9892;",
    "颓" => "&#x9893;",
    "颔" => "&#x9894;",
    "颕" => "&#x9895;",
    "颖" => "&#x9896;",
    "颗" => "&#x9897;",
    "题" => "&#x9898;",
    "颙" => "&#x9899;",
    "颚" => "&#x989A;",
    "颛" => "&#x989B;",
    "颜" => "&#x989C;",
    "额" => "&#x989D;",
    "颞" => "&#x989E;",
    "颟" => "&#x989F;",
    "颠" => "&#x98A0;",
    "颡" => "&#x98A1;",
    "颢" => "&#x98A2;",
    "颣" => "&#x98A3;",
    "颤" => "&#x98A4;",
    "颥" => "&#x98A5;",
    "颦" => "&#x98A6;",
    "颧" => "&#x98A7;",
    "風" => "&#x98A8;",
    "颩" => "&#x98A9;",
    "颪" => "&#x98AA;",
    "颫" => "&#x98AB;",
    "颬" => "&#x98AC;",
    "颭" => "&#x98AD;",
    "颮" => "&#x98AE;",
    "颯" => "&#x98AF;",
    "颰" => "&#x98B0;",
    "颱" => "&#x98B1;",
    "颲" => "&#x98B2;",
    "颳" => "&#x98B3;",
    "颴" => "&#x98B4;",
    "颵" => "&#x98B5;",
    "颶" => "&#x98B6;",
    "颷" => "&#x98B7;",
    "颸" => "&#x98B8;",
    "颹" => "&#x98B9;",
    "颺" => "&#x98BA;",
    "颻" => "&#x98BB;",
    "颼" => "&#x98BC;",
    "颽" => "&#x98BD;",
    "颾" => "&#x98BE;",
    "颿" => "&#x98BF;",
    "飀" => "&#x98C0;",
    "飁" => "&#x98C1;",
    "飂" => "&#x98C2;",
    "飃" => "&#x98C3;",
    "飄" => "&#x98C4;",
    "飅" => "&#x98C5;",
    "飆" => "&#x98C6;",
    "飇" => "&#x98C7;",
    "飈" => "&#x98C8;",
    "飉" => "&#x98C9;",
    "飊" => "&#x98CA;",
    "飋" => "&#x98CB;",
    "飌" => "&#x98CC;",
    "飍" => "&#x98CD;",
    "风" => "&#x98CE;",
    "飏" => "&#x98CF;",
    "飐" => "&#x98D0;",
    "飑" => "&#x98D1;",
    "飒" => "&#x98D2;",
    "飓" => "&#x98D3;",
    "飔" => "&#x98D4;",
    "飕" => "&#x98D5;",
    "飖" => "&#x98D6;",
    "飗" => "&#x98D7;",
    "飘" => "&#x98D8;",
    "飙" => "&#x98D9;",
    "飚" => "&#x98DA;",
    "飛" => "&#x98DB;",
    "飜" => "&#x98DC;",
    "飝" => "&#x98DD;",
    "飞" => "&#x98DE;",
    "食" => "&#x98DF;",
    "飠" => "&#x98E0;",
    "飡" => "&#x98E1;",
    "飢" => "&#x98E2;",
    "飣" => "&#x98E3;",
    "飤" => "&#x98E4;",
    "飥" => "&#x98E5;",
    "飦" => "&#x98E6;",
    "飧" => "&#x98E7;",
    "飨" => "&#x98E8;",
    "飩" => "&#x98E9;",
    "飪" => "&#x98EA;",
    "飫" => "&#x98EB;",
    "飬" => "&#x98EC;",
    "飭" => "&#x98ED;",
    "飮" => "&#x98EE;",
    "飯" => "&#x98EF;",
    "飰" => "&#x98F0;",
    "飱" => "&#x98F1;",
    "飲" => "&#x98F2;",
    "飳" => "&#x98F3;",
    "飴" => "&#x98F4;",
    "飵" => "&#x98F5;",
    "飶" => "&#x98F6;",
    "飷" => "&#x98F7;",
    "飸" => "&#x98F8;",
    "飹" => "&#x98F9;",
    "飺" => "&#x98FA;",
    "飻" => "&#x98FB;",
    "飼" => "&#x98FC;",
    "飽" => "&#x98FD;",
    "飾" => "&#x98FE;",
    "飿" => "&#x98FF;",
    "餀" => "&#x9900;",
    "餁" => "&#x9901;",
    "餂" => "&#x9902;",
    "餃" => "&#x9903;",
    "餄" => "&#x9904;",
    "餅" => "&#x9905;",
    "餆" => "&#x9906;",
    "餇" => "&#x9907;",
    "餈" => "&#x9908;",
    "餉" => "&#x9909;",
    "養" => "&#x990A;",
    "餋" => "&#x990B;",
    "餌" => "&#x990C;",
    "餍" => "&#x990D;",
    "餎" => "&#x990E;",
    "餏" => "&#x990F;",
    "餐" => "&#x9910;",
    "餑" => "&#x9911;",
    "餒" => "&#x9912;",
    "餓" => "&#x9913;",
    "餔" => "&#x9914;",
    "餕" => "&#x9915;",
    "餖" => "&#x9916;",
    "餗" => "&#x9917;",
    "餘" => "&#x9918;",
    "餙" => "&#x9919;",
    "餚" => "&#x991A;",
    "餛" => "&#x991B;",
    "餜" => "&#x991C;",
    "餝" => "&#x991D;",
    "餞" => "&#x991E;",
    "餟" => "&#x991F;",
    "餠" => "&#x9920;",
    "餡" => "&#x9921;",
    "餢" => "&#x9922;",
    "餣" => "&#x9923;",
    "餤" => "&#x9924;",
    "餥" => "&#x9925;",
    "餦" => "&#x9926;",
    "餧" => "&#x9927;",
    "館" => "&#x9928;",
    "餩" => "&#x9929;",
    "餪" => "&#x992A;",
    "餫" => "&#x992B;",
    "餬" => "&#x992C;",
    "餭" => "&#x992D;",
    "餮" => "&#x992E;",
    "餯" => "&#x992F;",
    "餰" => "&#x9930;",
    "餱" => "&#x9931;",
    "餲" => "&#x9932;",
    "餳" => "&#x9933;",
    "餴" => "&#x9934;",
    "餵" => "&#x9935;",
    "餶" => "&#x9936;",
    "餷" => "&#x9937;",
    "餸" => "&#x9938;",
    "餹" => "&#x9939;",
    "餺" => "&#x993A;",
    "餻" => "&#x993B;",
    "餼" => "&#x993C;",
    "餽" => "&#x993D;",
    "餾" => "&#x993E;",
    "餿" => "&#x993F;",
    "饀" => "&#x9940;",
    "饁" => "&#x9941;",
    "饂" => "&#x9942;",
    "饃" => "&#x9943;",
    "饄" => "&#x9944;",
    "饅" => "&#x9945;",
    "饆" => "&#x9946;",
    "饇" => "&#x9947;",
    "饈" => "&#x9948;",
    "饉" => "&#x9949;",
    "饊" => "&#x994A;",
    "饋" => "&#x994B;",
    "饌" => "&#x994C;",
    "饍" => "&#x994D;",
    "饎" => "&#x994E;",
    "饏" => "&#x994F;",
    "饐" => "&#x9950;",
    "饑" => "&#x9951;",
    "饒" => "&#x9952;",
    "饓" => "&#x9953;",
    "饔" => "&#x9954;",
    "饕" => "&#x9955;",
    "饖" => "&#x9956;",
    "饗" => "&#x9957;",
    "饘" => "&#x9958;",
    "饙" => "&#x9959;",
    "饚" => "&#x995A;",
    "饛" => "&#x995B;",
    "饜" => "&#x995C;",
    "饝" => "&#x995D;",
    "饞" => "&#x995E;",
    "饟" => "&#x995F;",
    "饠" => "&#x9960;",
    "饡" => "&#x9961;",
    "饢" => "&#x9962;",
    "饣" => "&#x9963;",
    "饤" => "&#x9964;",
    "饥" => "&#x9965;",
    "饦" => "&#x9966;",
    "饧" => "&#x9967;",
    "饨" => "&#x9968;",
    "饩" => "&#x9969;",
    "饪" => "&#x996A;",
    "饫" => "&#x996B;",
    "饬" => "&#x996C;",
    "饭" => "&#x996D;",
    "饮" => "&#x996E;",
    "饯" => "&#x996F;",
    "饰" => "&#x9970;",
    "饱" => "&#x9971;",
    "饲" => "&#x9972;",
    "饳" => "&#x9973;",
    "饴" => "&#x9974;",
    "饵" => "&#x9975;",
    "饶" => "&#x9976;",
    "饷" => "&#x9977;",
    "饸" => "&#x9978;",
    "饹" => "&#x9979;",
    "饺" => "&#x997A;",
    "饻" => "&#x997B;",
    "饼" => "&#x997C;",
    "饽" => "&#x997D;",
    "饾" => "&#x997E;",
    "饿" => "&#x997F;",
    "馀" => "&#x9980;",
    "馁" => "&#x9981;",
    "馂" => "&#x9982;",
    "馃" => "&#x9983;",
    "馄" => "&#x9984;",
    "馅" => "&#x9985;",
    "馆" => "&#x9986;",
    "馇" => "&#x9987;",
    "馈" => "&#x9988;",
    "馉" => "&#x9989;",
    "馊" => "&#x998A;",
    "馋" => "&#x998B;",
    "馌" => "&#x998C;",
    "馍" => "&#x998D;",
    "馎" => "&#x998E;",
    "馏" => "&#x998F;",
    "馐" => "&#x9990;",
    "馑" => "&#x9991;",
    "馒" => "&#x9992;",
    "馓" => "&#x9993;",
    "馔" => "&#x9994;",
    "馕" => "&#x9995;",
    "首" => "&#x9996;",
    "馗" => "&#x9997;",
    "馘" => "&#x9998;",
    "香" => "&#x9999;",
    "馚" => "&#x999A;",
    "馛" => "&#x999B;",
    "馜" => "&#x999C;",
    "馝" => "&#x999D;",
    "馞" => "&#x999E;",
    "馟" => "&#x999F;",
    "馠" => "&#x99A0;",
    "馡" => "&#x99A1;",
    "馢" => "&#x99A2;",
    "馣" => "&#x99A3;",
    "馤" => "&#x99A4;",
    "馥" => "&#x99A5;",
    "馦" => "&#x99A6;",
    "馧" => "&#x99A7;",
    "馨" => "&#x99A8;",
    "馩" => "&#x99A9;",
    "馪" => "&#x99AA;",
    "馫" => "&#x99AB;",
    "馬" => "&#x99AC;",
    "馭" => "&#x99AD;",
    "馮" => "&#x99AE;",
    "馯" => "&#x99AF;",
    "馰" => "&#x99B0;",
    "馱" => "&#x99B1;",
    "馲" => "&#x99B2;",
    "馳" => "&#x99B3;",
    "馴" => "&#x99B4;",
    "馵" => "&#x99B5;",
    "馶" => "&#x99B6;",
    "馷" => "&#x99B7;",
    "馸" => "&#x99B8;",
    "馹" => "&#x99B9;",
    "馺" => "&#x99BA;",
    "馻" => "&#x99BB;",
    "馼" => "&#x99BC;",
    "馽" => "&#x99BD;",
    "馾" => "&#x99BE;",
    "馿" => "&#x99BF;",
    "駀" => "&#x99C0;",
    "駁" => "&#x99C1;",
    "駂" => "&#x99C2;",
    "駃" => "&#x99C3;",
    "駄" => "&#x99C4;",
    "駅" => "&#x99C5;",
    "駆" => "&#x99C6;",
    "駇" => "&#x99C7;",
    "駈" => "&#x99C8;",
    "駉" => "&#x99C9;",
    "駊" => "&#x99CA;",
    "駋" => "&#x99CB;",
    "駌" => "&#x99CC;",
    "駍" => "&#x99CD;",
    "駎" => "&#x99CE;",
    "駏" => "&#x99CF;",
    "駐" => "&#x99D0;",
    "駑" => "&#x99D1;",
    "駒" => "&#x99D2;",
    "駓" => "&#x99D3;",
    "駔" => "&#x99D4;",
    "駕" => "&#x99D5;",
    "駖" => "&#x99D6;",
    "駗" => "&#x99D7;",
    "駘" => "&#x99D8;",
    "駙" => "&#x99D9;",
    "駚" => "&#x99DA;",
    "駛" => "&#x99DB;",
    "駜" => "&#x99DC;",
    "駝" => "&#x99DD;",
    "駞" => "&#x99DE;",
    "駟" => "&#x99DF;",
    "駠" => "&#x99E0;",
    "駡" => "&#x99E1;",
    "駢" => "&#x99E2;",
    "駣" => "&#x99E3;",
    "駤" => "&#x99E4;",
    "駥" => "&#x99E5;",
    "駦" => "&#x99E6;",
    "駧" => "&#x99E7;",
    "駨" => "&#x99E8;",
    "駩" => "&#x99E9;",
    "駪" => "&#x99EA;",
    "駫" => "&#x99EB;",
    "駬" => "&#x99EC;",
    "駭" => "&#x99ED;",
    "駮" => "&#x99EE;",
    "駯" => "&#x99EF;",
    "駰" => "&#x99F0;",
    "駱" => "&#x99F1;",
    "駲" => "&#x99F2;",
    "駳" => "&#x99F3;",
    "駴" => "&#x99F4;",
    "駵" => "&#x99F5;",
    "駶" => "&#x99F6;",
    "駷" => "&#x99F7;",
    "駸" => "&#x99F8;",
    "駹" => "&#x99F9;",
    "駺" => "&#x99FA;",
    "駻" => "&#x99FB;",
    "駼" => "&#x99FC;",
    "駽" => "&#x99FD;",
    "駾" => "&#x99FE;",
    "駿" => "&#x99FF;",
    "騀" => "&#x9A00;",
    "騁" => "&#x9A01;",
    "騂" => "&#x9A02;",
    "騃" => "&#x9A03;",
    "騄" => "&#x9A04;",
    "騅" => "&#x9A05;",
    "騆" => "&#x9A06;",
    "騇" => "&#x9A07;",
    "騈" => "&#x9A08;",
    "騉" => "&#x9A09;",
    "騊" => "&#x9A0A;",
    "騋" => "&#x9A0B;",
    "騌" => "&#x9A0C;",
    "騍" => "&#x9A0D;",
    "騎" => "&#x9A0E;",
    "騏" => "&#x9A0F;",
    "騐" => "&#x9A10;",
    "騑" => "&#x9A11;",
    "騒" => "&#x9A12;",
    "験" => "&#x9A13;",
    "騔" => "&#x9A14;",
    "騕" => "&#x9A15;",
    "騖" => "&#x9A16;",
    "騗" => "&#x9A17;",
    "騘" => "&#x9A18;",
    "騙" => "&#x9A19;",
    "騚" => "&#x9A1A;",
    "騛" => "&#x9A1B;",
    "騜" => "&#x9A1C;",
    "騝" => "&#x9A1D;",
    "騞" => "&#x9A1E;",
    "騟" => "&#x9A1F;",
    "騠" => "&#x9A20;",
    "騡" => "&#x9A21;",
    "騢" => "&#x9A22;",
    "騣" => "&#x9A23;",
    "騤" => "&#x9A24;",
    "騥" => "&#x9A25;",
    "騦" => "&#x9A26;",
    "騧" => "&#x9A27;",
    "騨" => "&#x9A28;",
    "騩" => "&#x9A29;",
    "騪" => "&#x9A2A;",
    "騫" => "&#x9A2B;",
    "騬" => "&#x9A2C;",
    "騭" => "&#x9A2D;",
    "騮" => "&#x9A2E;",
    "騯" => "&#x9A2F;",
    "騰" => "&#x9A30;",
    "騱" => "&#x9A31;",
    "騲" => "&#x9A32;",
    "騳" => "&#x9A33;",
    "騴" => "&#x9A34;",
    "騵" => "&#x9A35;",
    "騶" => "&#x9A36;",
    "騷" => "&#x9A37;",
    "騸" => "&#x9A38;",
    "騹" => "&#x9A39;",
    "騺" => "&#x9A3A;",
    "騻" => "&#x9A3B;",
    "騼" => "&#x9A3C;",
    "騽" => "&#x9A3D;",
    "騾" => "&#x9A3E;",
    "騿" => "&#x9A3F;",
    "驀" => "&#x9A40;",
    "驁" => "&#x9A41;",
    "驂" => "&#x9A42;",
    "驃" => "&#x9A43;",
    "驄" => "&#x9A44;",
    "驅" => "&#x9A45;",
    "驆" => "&#x9A46;",
    "驇" => "&#x9A47;",
    "驈" => "&#x9A48;",
    "驉" => "&#x9A49;",
    "驊" => "&#x9A4A;",
    "驋" => "&#x9A4B;",
    "驌" => "&#x9A4C;",
    "驍" => "&#x9A4D;",
    "驎" => "&#x9A4E;",
    "驏" => "&#x9A4F;",
    "驐" => "&#x9A50;",
    "驑" => "&#x9A51;",
    "驒" => "&#x9A52;",
    "驓" => "&#x9A53;",
    "驔" => "&#x9A54;",
    "驕" => "&#x9A55;",
    "驖" => "&#x9A56;",
    "驗" => "&#x9A57;",
    "驘" => "&#x9A58;",
    "驙" => "&#x9A59;",
    "驚" => "&#x9A5A;",
    "驛" => "&#x9A5B;",
    "驜" => "&#x9A5C;",
    "驝" => "&#x9A5D;",
    "驞" => "&#x9A5E;",
    "驟" => "&#x9A5F;",
    "驠" => "&#x9A60;",
    "驡" => "&#x9A61;",
    "驢" => "&#x9A62;",
    "驣" => "&#x9A63;",
    "驤" => "&#x9A64;",
    "驥" => "&#x9A65;",
    "驦" => "&#x9A66;",
    "驧" => "&#x9A67;",
    "驨" => "&#x9A68;",
    "驩" => "&#x9A69;",
    "驪" => "&#x9A6A;",
    "驫" => "&#x9A6B;",
    "马" => "&#x9A6C;",
    "驭" => "&#x9A6D;",
    "驮" => "&#x9A6E;",
    "驯" => "&#x9A6F;",
    "驰" => "&#x9A70;",
    "驱" => "&#x9A71;",
    "驲" => "&#x9A72;",
    "驳" => "&#x9A73;",
    "驴" => "&#x9A74;",
    "驵" => "&#x9A75;",
    "驶" => "&#x9A76;",
    "驷" => "&#x9A77;",
    "驸" => "&#x9A78;",
    "驹" => "&#x9A79;",
    "驺" => "&#x9A7A;",
    "驻" => "&#x9A7B;",
    "驼" => "&#x9A7C;",
    "驽" => "&#x9A7D;",
    "驾" => "&#x9A7E;",
    "驿" => "&#x9A7F;",
    "骀" => "&#x9A80;",
    "骁" => "&#x9A81;",
    "骂" => "&#x9A82;",
    "骃" => "&#x9A83;",
    "骄" => "&#x9A84;",
    "骅" => "&#x9A85;",
    "骆" => "&#x9A86;",
    "骇" => "&#x9A87;",
    "骈" => "&#x9A88;",
    "骉" => "&#x9A89;",
    "骊" => "&#x9A8A;",
    "骋" => "&#x9A8B;",
    "验" => "&#x9A8C;",
    "骍" => "&#x9A8D;",
    "骎" => "&#x9A8E;",
    "骏" => "&#x9A8F;",
    "骐" => "&#x9A90;",
    "骑" => "&#x9A91;",
    "骒" => "&#x9A92;",
    "骓" => "&#x9A93;",
    "骔" => "&#x9A94;",
    "骕" => "&#x9A95;",
    "骖" => "&#x9A96;",
    "骗" => "&#x9A97;",
    "骘" => "&#x9A98;",
    "骙" => "&#x9A99;",
    "骚" => "&#x9A9A;",
    "骛" => "&#x9A9B;",
    "骜" => "&#x9A9C;",
    "骝" => "&#x9A9D;",
    "骞" => "&#x9A9E;",
    "骟" => "&#x9A9F;",
    "骠" => "&#x9AA0;",
    "骡" => "&#x9AA1;",
    "骢" => "&#x9AA2;",
    "骣" => "&#x9AA3;",
    "骤" => "&#x9AA4;",
    "骥" => "&#x9AA5;",
    "骦" => "&#x9AA6;",
    "骧" => "&#x9AA7;",
    "骨" => "&#x9AA8;",
    "骩" => "&#x9AA9;",
    "骪" => "&#x9AAA;",
    "骫" => "&#x9AAB;",
    "骬" => "&#x9AAC;",
    "骭" => "&#x9AAD;",
    "骮" => "&#x9AAE;",
    "骯" => "&#x9AAF;",
    "骰" => "&#x9AB0;",
    "骱" => "&#x9AB1;",
    "骲" => "&#x9AB2;",
    "骳" => "&#x9AB3;",
    "骴" => "&#x9AB4;",
    "骵" => "&#x9AB5;",
    "骶" => "&#x9AB6;",
    "骷" => "&#x9AB7;",
    "骸" => "&#x9AB8;",
    "骹" => "&#x9AB9;",
    "骺" => "&#x9ABA;",
    "骻" => "&#x9ABB;",
    "骼" => "&#x9ABC;",
    "骽" => "&#x9ABD;",
    "骾" => "&#x9ABE;",
    "骿" => "&#x9ABF;",
    "髀" => "&#x9AC0;",
    "髁" => "&#x9AC1;",
    "髂" => "&#x9AC2;",
    "髃" => "&#x9AC3;",
    "髄" => "&#x9AC4;",
    "髅" => "&#x9AC5;",
    "髆" => "&#x9AC6;",
    "髇" => "&#x9AC7;",
    "髈" => "&#x9AC8;",
    "髉" => "&#x9AC9;",
    "髊" => "&#x9ACA;",
    "髋" => "&#x9ACB;",
    "髌" => "&#x9ACC;",
    "髍" => "&#x9ACD;",
    "髎" => "&#x9ACE;",
    "髏" => "&#x9ACF;",
    "髐" => "&#x9AD0;",
    "髑" => "&#x9AD1;",
    "髒" => "&#x9AD2;",
    "髓" => "&#x9AD3;",
    "體" => "&#x9AD4;",
    "髕" => "&#x9AD5;",
    "髖" => "&#x9AD6;",
    "髗" => "&#x9AD7;",
    "高" => "&#x9AD8;",
    "髙" => "&#x9AD9;",
    "髚" => "&#x9ADA;",
    "髛" => "&#x9ADB;",
    "髜" => "&#x9ADC;",
    "髝" => "&#x9ADD;",
    "髞" => "&#x9ADE;",
    "髟" => "&#x9ADF;",
    "髠" => "&#x9AE0;",
    "髡" => "&#x9AE1;",
    "髢" => "&#x9AE2;",
    "髣" => "&#x9AE3;",
    "髤" => "&#x9AE4;",
    "髥" => "&#x9AE5;",
    "髦" => "&#x9AE6;",
    "髧" => "&#x9AE7;",
    "髨" => "&#x9AE8;",
    "髩" => "&#x9AE9;",
    "髪" => "&#x9AEA;",
    "髫" => "&#x9AEB;",
    "髬" => "&#x9AEC;",
    "髭" => "&#x9AED;",
    "髮" => "&#x9AEE;",
    "髯" => "&#x9AEF;",
    "髰" => "&#x9AF0;",
    "髱" => "&#x9AF1;",
    "髲" => "&#x9AF2;",
    "髳" => "&#x9AF3;",
    "髴" => "&#x9AF4;",
    "髵" => "&#x9AF5;",
    "髶" => "&#x9AF6;",
    "髷" => "&#x9AF7;",
    "髸" => "&#x9AF8;",
    "髹" => "&#x9AF9;",
    "髺" => "&#x9AFA;",
    "髻" => "&#x9AFB;",
    "髼" => "&#x9AFC;",
    "髽" => "&#x9AFD;",
    "髾" => "&#x9AFE;",
    "髿" => "&#x9AFF;",
    "鬀" => "&#x9B00;",
    "鬁" => "&#x9B01;",
    "鬂" => "&#x9B02;",
    "鬃" => "&#x9B03;",
    "鬄" => "&#x9B04;",
    "鬅" => "&#x9B05;",
    "鬆" => "&#x9B06;",
    "鬇" => "&#x9B07;",
    "鬈" => "&#x9B08;",
    "鬉" => "&#x9B09;",
    "鬊" => "&#x9B0A;",
    "鬋" => "&#x9B0B;",
    "鬌" => "&#x9B0C;",
    "鬍" => "&#x9B0D;",
    "鬎" => "&#x9B0E;",
    "鬏" => "&#x9B0F;",
    "鬐" => "&#x9B10;",
    "鬑" => "&#x9B11;",
    "鬒" => "&#x9B12;",
    "鬓" => "&#x9B13;",
    "鬔" => "&#x9B14;",
    "鬕" => "&#x9B15;",
    "鬖" => "&#x9B16;",
    "鬗" => "&#x9B17;",
    "鬘" => "&#x9B18;",
    "鬙" => "&#x9B19;",
    "鬚" => "&#x9B1A;",
    "鬛" => "&#x9B1B;",
    "鬜" => "&#x9B1C;",
    "鬝" => "&#x9B1D;",
    "鬞" => "&#x9B1E;",
    "鬟" => "&#x9B1F;",
    "鬠" => "&#x9B20;",
    "鬡" => "&#x9B21;",
    "鬢" => "&#x9B22;",
    "鬣" => "&#x9B23;",
    "鬤" => "&#x9B24;",
    "鬥" => "&#x9B25;",
    "鬦" => "&#x9B26;",
    "鬧" => "&#x9B27;",
    "鬨" => "&#x9B28;",
    "鬩" => "&#x9B29;",
    "鬪" => "&#x9B2A;",
    "鬫" => "&#x9B2B;",
    "鬬" => "&#x9B2C;",
    "鬭" => "&#x9B2D;",
    "鬮" => "&#x9B2E;",
    "鬯" => "&#x9B2F;",
    "鬰" => "&#x9B30;",
    "鬱" => "&#x9B31;",
    "鬲" => "&#x9B32;",
    "鬳" => "&#x9B33;",
    "鬴" => "&#x9B34;",
    "鬵" => "&#x9B35;",
    "鬶" => "&#x9B36;",
    "鬷" => "&#x9B37;",
    "鬸" => "&#x9B38;",
    "鬹" => "&#x9B39;",
    "鬺" => "&#x9B3A;",
    "鬻" => "&#x9B3B;",
    "鬼" => "&#x9B3C;",
    "鬽" => "&#x9B3D;",
    "鬾" => "&#x9B3E;",
    "鬿" => "&#x9B3F;",
    "魀" => "&#x9B40;",
    "魁" => "&#x9B41;",
    "魂" => "&#x9B42;",
    "魃" => "&#x9B43;",
    "魄" => "&#x9B44;",
    "魅" => "&#x9B45;",
    "魆" => "&#x9B46;",
    "魇" => "&#x9B47;",
    "魈" => "&#x9B48;",
    "魉" => "&#x9B49;",
    "魊" => "&#x9B4A;",
    "魋" => "&#x9B4B;",
    "魌" => "&#x9B4C;",
    "魍" => "&#x9B4D;",
    "魎" => "&#x9B4E;",
    "魏" => "&#x9B4F;",
    "魐" => "&#x9B50;",
    "魑" => "&#x9B51;",
    "魒" => "&#x9B52;",
    "魓" => "&#x9B53;",
    "魔" => "&#x9B54;",
    "魕" => "&#x9B55;",
    "魖" => "&#x9B56;",
    "魗" => "&#x9B57;",
    "魘" => "&#x9B58;",
    "魙" => "&#x9B59;",
    "魚" => "&#x9B5A;",
    "魛" => "&#x9B5B;",
    "魜" => "&#x9B5C;",
    "魝" => "&#x9B5D;",
    "魞" => "&#x9B5E;",
    "魟" => "&#x9B5F;",
    "魠" => "&#x9B60;",
    "魡" => "&#x9B61;",
    "魢" => "&#x9B62;",
    "魣" => "&#x9B63;",
    "魤" => "&#x9B64;",
    "魥" => "&#x9B65;",
    "魦" => "&#x9B66;",
    "魧" => "&#x9B67;",
    "魨" => "&#x9B68;",
    "魩" => "&#x9B69;",
    "魪" => "&#x9B6A;",
    "魫" => "&#x9B6B;",
    "魬" => "&#x9B6C;",
    "魭" => "&#x9B6D;",
    "魮" => "&#x9B6E;",
    "魯" => "&#x9B6F;",
    "魰" => "&#x9B70;",
    "魱" => "&#x9B71;",
    "魲" => "&#x9B72;",
    "魳" => "&#x9B73;",
    "魴" => "&#x9B74;",
    "魵" => "&#x9B75;",
    "魶" => "&#x9B76;",
    "魷" => "&#x9B77;",
    "魸" => "&#x9B78;",
    "魹" => "&#x9B79;",
    "魺" => "&#x9B7A;",
    "魻" => "&#x9B7B;",
    "魼" => "&#x9B7C;",
    "魽" => "&#x9B7D;",
    "魾" => "&#x9B7E;",
    "魿" => "&#x9B7F;",
    "鮀" => "&#x9B80;",
    "鮁" => "&#x9B81;",
    "鮂" => "&#x9B82;",
    "鮃" => "&#x9B83;",
    "鮄" => "&#x9B84;",
    "鮅" => "&#x9B85;",
    "鮆" => "&#x9B86;",
    "鮇" => "&#x9B87;",
    "鮈" => "&#x9B88;",
    "鮉" => "&#x9B89;",
    "鮊" => "&#x9B8A;",
    "鮋" => "&#x9B8B;",
    "鮌" => "&#x9B8C;",
    "鮍" => "&#x9B8D;",
    "鮎" => "&#x9B8E;",
    "鮏" => "&#x9B8F;",
    "鮐" => "&#x9B90;",
    "鮑" => "&#x9B91;",
    "鮒" => "&#x9B92;",
    "鮓" => "&#x9B93;",
    "鮔" => "&#x9B94;",
    "鮕" => "&#x9B95;",
    "鮖" => "&#x9B96;",
    "鮗" => "&#x9B97;",
    "鮘" => "&#x9B98;",
    "鮙" => "&#x9B99;",
    "鮚" => "&#x9B9A;",
    "鮛" => "&#x9B9B;",
    "鮜" => "&#x9B9C;",
    "鮝" => "&#x9B9D;",
    "鮞" => "&#x9B9E;",
    "鮟" => "&#x9B9F;",
    "鮠" => "&#x9BA0;",
    "鮡" => "&#x9BA1;",
    "鮢" => "&#x9BA2;",
    "鮣" => "&#x9BA3;",
    "鮤" => "&#x9BA4;",
    "鮥" => "&#x9BA5;",
    "鮦" => "&#x9BA6;",
    "鮧" => "&#x9BA7;",
    "鮨" => "&#x9BA8;",
    "鮩" => "&#x9BA9;",
    "鮪" => "&#x9BAA;",
    "鮫" => "&#x9BAB;",
    "鮬" => "&#x9BAC;",
    "鮭" => "&#x9BAD;",
    "鮮" => "&#x9BAE;",
    "鮯" => "&#x9BAF;",
    "鮰" => "&#x9BB0;",
    "鮱" => "&#x9BB1;",
    "鮲" => "&#x9BB2;",
    "鮳" => "&#x9BB3;",
    "鮴" => "&#x9BB4;",
    "鮵" => "&#x9BB5;",
    "鮶" => "&#x9BB6;",
    "鮷" => "&#x9BB7;",
    "鮸" => "&#x9BB8;",
    "鮹" => "&#x9BB9;",
    "鮺" => "&#x9BBA;",
    "鮻" => "&#x9BBB;",
    "鮼" => "&#x9BBC;",
    "鮽" => "&#x9BBD;",
    "鮾" => "&#x9BBE;",
    "鮿" => "&#x9BBF;",
    "鯀" => "&#x9BC0;",
    "鯁" => "&#x9BC1;",
    "鯂" => "&#x9BC2;",
    "鯃" => "&#x9BC3;",
    "鯄" => "&#x9BC4;",
    "鯅" => "&#x9BC5;",
    "鯆" => "&#x9BC6;",
    "鯇" => "&#x9BC7;",
    "鯈" => "&#x9BC8;",
    "鯉" => "&#x9BC9;",
    "鯊" => "&#x9BCA;",
    "鯋" => "&#x9BCB;",
    "鯌" => "&#x9BCC;",
    "鯍" => "&#x9BCD;",
    "鯎" => "&#x9BCE;",
    "鯏" => "&#x9BCF;",
    "鯐" => "&#x9BD0;",
    "鯑" => "&#x9BD1;",
    "鯒" => "&#x9BD2;",
    "鯓" => "&#x9BD3;",
    "鯔" => "&#x9BD4;",
    "鯕" => "&#x9BD5;",
    "鯖" => "&#x9BD6;",
    "鯗" => "&#x9BD7;",
    "鯘" => "&#x9BD8;",
    "鯙" => "&#x9BD9;",
    "鯚" => "&#x9BDA;",
    "鯛" => "&#x9BDB;",
    "鯜" => "&#x9BDC;",
    "鯝" => "&#x9BDD;",
    "鯞" => "&#x9BDE;",
    "鯟" => "&#x9BDF;",
    "鯠" => "&#x9BE0;",
    "鯡" => "&#x9BE1;",
    "鯢" => "&#x9BE2;",
    "鯣" => "&#x9BE3;",
    "鯤" => "&#x9BE4;",
    "鯥" => "&#x9BE5;",
    "鯦" => "&#x9BE6;",
    "鯧" => "&#x9BE7;",
    "鯨" => "&#x9BE8;",
    "鯩" => "&#x9BE9;",
    "鯪" => "&#x9BEA;",
    "鯫" => "&#x9BEB;",
    "鯬" => "&#x9BEC;",
    "鯭" => "&#x9BED;",
    "鯮" => "&#x9BEE;",
    "鯯" => "&#x9BEF;",
    "鯰" => "&#x9BF0;",
    "鯱" => "&#x9BF1;",
    "鯲" => "&#x9BF2;",
    "鯳" => "&#x9BF3;",
    "鯴" => "&#x9BF4;",
    "鯵" => "&#x9BF5;",
    "鯶" => "&#x9BF6;",
    "鯷" => "&#x9BF7;",
    "鯸" => "&#x9BF8;",
    "鯹" => "&#x9BF9;",
    "鯺" => "&#x9BFA;",
    "鯻" => "&#x9BFB;",
    "鯼" => "&#x9BFC;",
    "鯽" => "&#x9BFD;",
    "鯾" => "&#x9BFE;",
    "鯿" => "&#x9BFF;",
    "鰀" => "&#x9C00;",
    "鰁" => "&#x9C01;",
    "鰂" => "&#x9C02;",
    "鰃" => "&#x9C03;",
    "鰄" => "&#x9C04;",
    "鰅" => "&#x9C05;",
    "鰆" => "&#x9C06;",
    "鰇" => "&#x9C07;",
    "鰈" => "&#x9C08;",
    "鰉" => "&#x9C09;",
    "鰊" => "&#x9C0A;",
    "鰋" => "&#x9C0B;",
    "鰌" => "&#x9C0C;",
    "鰍" => "&#x9C0D;",
    "鰎" => "&#x9C0E;",
    "鰏" => "&#x9C0F;",
    "鰐" => "&#x9C10;",
    "鰑" => "&#x9C11;",
    "鰒" => "&#x9C12;",
    "鰓" => "&#x9C13;",
    "鰔" => "&#x9C14;",
    "鰕" => "&#x9C15;",
    "鰖" => "&#x9C16;",
    "鰗" => "&#x9C17;",
    "鰘" => "&#x9C18;",
    "鰙" => "&#x9C19;",
    "鰚" => "&#x9C1A;",
    "鰛" => "&#x9C1B;",
    "鰜" => "&#x9C1C;",
    "鰝" => "&#x9C1D;",
    "鰞" => "&#x9C1E;",
    "鰟" => "&#x9C1F;",
    "鰠" => "&#x9C20;",
    "鰡" => "&#x9C21;",
    "鰢" => "&#x9C22;",
    "鰣" => "&#x9C23;",
    "鰤" => "&#x9C24;",
    "鰥" => "&#x9C25;",
    "鰦" => "&#x9C26;",
    "鰧" => "&#x9C27;",
    "鰨" => "&#x9C28;",
    "鰩" => "&#x9C29;",
    "鰪" => "&#x9C2A;",
    "鰫" => "&#x9C2B;",
    "鰬" => "&#x9C2C;",
    "鰭" => "&#x9C2D;",
    "鰮" => "&#x9C2E;",
    "鰯" => "&#x9C2F;",
    "鰰" => "&#x9C30;",
    "鰱" => "&#x9C31;",
    "鰲" => "&#x9C32;",
    "鰳" => "&#x9C33;",
    "鰴" => "&#x9C34;",
    "鰵" => "&#x9C35;",
    "鰶" => "&#x9C36;",
    "鰷" => "&#x9C37;",
    "鰸" => "&#x9C38;",
    "鰹" => "&#x9C39;",
    "鰺" => "&#x9C3A;",
    "鰻" => "&#x9C3B;",
    "鰼" => "&#x9C3C;",
    "鰽" => "&#x9C3D;",
    "鰾" => "&#x9C3E;",
    "鰿" => "&#x9C3F;",
    "鱀" => "&#x9C40;",
    "鱁" => "&#x9C41;",
    "鱂" => "&#x9C42;",
    "鱃" => "&#x9C43;",
    "鱄" => "&#x9C44;",
    "鱅" => "&#x9C45;",
    "鱆" => "&#x9C46;",
    "鱇" => "&#x9C47;",
    "鱈" => "&#x9C48;",
    "鱉" => "&#x9C49;",
    "鱊" => "&#x9C4A;",
    "鱋" => "&#x9C4B;",
    "鱌" => "&#x9C4C;",
    "鱍" => "&#x9C4D;",
    "鱎" => "&#x9C4E;",
    "鱏" => "&#x9C4F;",
    "鱐" => "&#x9C50;",
    "鱑" => "&#x9C51;",
    "鱒" => "&#x9C52;",
    "鱓" => "&#x9C53;",
    "鱔" => "&#x9C54;",
    "鱕" => "&#x9C55;",
    "鱖" => "&#x9C56;",
    "鱗" => "&#x9C57;",
    "鱘" => "&#x9C58;",
    "鱙" => "&#x9C59;",
    "鱚" => "&#x9C5A;",
    "鱛" => "&#x9C5B;",
    "鱜" => "&#x9C5C;",
    "鱝" => "&#x9C5D;",
    "鱞" => "&#x9C5E;",
    "鱟" => "&#x9C5F;",
    "鱠" => "&#x9C60;",
    "鱡" => "&#x9C61;",
    "鱢" => "&#x9C62;",
    "鱣" => "&#x9C63;",
    "鱤" => "&#x9C64;",
    "鱥" => "&#x9C65;",
    "鱦" => "&#x9C66;",
    "鱧" => "&#x9C67;",
    "鱨" => "&#x9C68;",
    "鱩" => "&#x9C69;",
    "鱪" => "&#x9C6A;",
    "鱫" => "&#x9C6B;",
    "鱬" => "&#x9C6C;",
    "鱭" => "&#x9C6D;",
    "鱮" => "&#x9C6E;",
    "鱯" => "&#x9C6F;",
    "鱰" => "&#x9C70;",
    "鱱" => "&#x9C71;",
    "鱲" => "&#x9C72;",
    "鱳" => "&#x9C73;",
    "鱴" => "&#x9C74;",
    "鱵" => "&#x9C75;",
    "鱶" => "&#x9C76;",
    "鱷" => "&#x9C77;",
    "鱸" => "&#x9C78;",
    "鱹" => "&#x9C79;",
    "鱺" => "&#x9C7A;",
    "鱻" => "&#x9C7B;",
    "鱼" => "&#x9C7C;",
    "鱽" => "&#x9C7D;",
    "鱾" => "&#x9C7E;",
    "鱿" => "&#x9C7F;",
    "鲀" => "&#x9C80;",
    "鲁" => "&#x9C81;",
    "鲂" => "&#x9C82;",
    "鲃" => "&#x9C83;",
    "鲄" => "&#x9C84;",
    "鲅" => "&#x9C85;",
    "鲆" => "&#x9C86;",
    "鲇" => "&#x9C87;",
    "鲈" => "&#x9C88;",
    "鲉" => "&#x9C89;",
    "鲊" => "&#x9C8A;",
    "鲋" => "&#x9C8B;",
    "鲌" => "&#x9C8C;",
    "鲍" => "&#x9C8D;",
    "鲎" => "&#x9C8E;",
    "鲏" => "&#x9C8F;",
    "鲐" => "&#x9C90;",
    "鲑" => "&#x9C91;",
    "鲒" => "&#x9C92;",
    "鲓" => "&#x9C93;",
    "鲔" => "&#x9C94;",
    "鲕" => "&#x9C95;",
    "鲖" => "&#x9C96;",
    "鲗" => "&#x9C97;",
    "鲘" => "&#x9C98;",
    "鲙" => "&#x9C99;",
    "鲚" => "&#x9C9A;",
    "鲛" => "&#x9C9B;",
    "鲜" => "&#x9C9C;",
    "鲝" => "&#x9C9D;",
    "鲞" => "&#x9C9E;",
    "鲟" => "&#x9C9F;",
    "鲠" => "&#x9CA0;",
    "鲡" => "&#x9CA1;",
    "鲢" => "&#x9CA2;",
    "鲣" => "&#x9CA3;",
    "鲤" => "&#x9CA4;",
    "鲥" => "&#x9CA5;",
    "鲦" => "&#x9CA6;",
    "鲧" => "&#x9CA7;",
    "鲨" => "&#x9CA8;",
    "鲩" => "&#x9CA9;",
    "鲪" => "&#x9CAA;",
    "鲫" => "&#x9CAB;",
    "鲬" => "&#x9CAC;",
    "鲭" => "&#x9CAD;",
    "鲮" => "&#x9CAE;",
    "鲯" => "&#x9CAF;",
    "鲰" => "&#x9CB0;",
    "鲱" => "&#x9CB1;",
    "鲲" => "&#x9CB2;",
    "鲳" => "&#x9CB3;",
    "鲴" => "&#x9CB4;",
    "鲵" => "&#x9CB5;",
    "鲶" => "&#x9CB6;",
    "鲷" => "&#x9CB7;",
    "鲸" => "&#x9CB8;",
    "鲹" => "&#x9CB9;",
    "鲺" => "&#x9CBA;",
    "鲻" => "&#x9CBB;",
    "鲼" => "&#x9CBC;",
    "鲽" => "&#x9CBD;",
    "鲾" => "&#x9CBE;",
    "鲿" => "&#x9CBF;",
    "鳀" => "&#x9CC0;",
    "鳁" => "&#x9CC1;",
    "鳂" => "&#x9CC2;",
    "鳃" => "&#x9CC3;",
    "鳄" => "&#x9CC4;",
    "鳅" => "&#x9CC5;",
    "鳆" => "&#x9CC6;",
    "鳇" => "&#x9CC7;",
    "鳈" => "&#x9CC8;",
    "鳉" => "&#x9CC9;",
    "鳊" => "&#x9CCA;",
    "鳋" => "&#x9CCB;",
    "鳌" => "&#x9CCC;",
    "鳍" => "&#x9CCD;",
    "鳎" => "&#x9CCE;",
    "鳏" => "&#x9CCF;",
    "鳐" => "&#x9CD0;",
    "鳑" => "&#x9CD1;",
    "鳒" => "&#x9CD2;",
    "鳓" => "&#x9CD3;",
    "鳔" => "&#x9CD4;",
    "鳕" => "&#x9CD5;",
    "鳖" => "&#x9CD6;",
    "鳗" => "&#x9CD7;",
    "鳘" => "&#x9CD8;",
    "鳙" => "&#x9CD9;",
    "鳚" => "&#x9CDA;",
    "鳛" => "&#x9CDB;",
    "鳜" => "&#x9CDC;",
    "鳝" => "&#x9CDD;",
    "鳞" => "&#x9CDE;",
    "鳟" => "&#x9CDF;",
    "鳠" => "&#x9CE0;",
    "鳡" => "&#x9CE1;",
    "鳢" => "&#x9CE2;",
    "鳣" => "&#x9CE3;",
    "鳤" => "&#x9CE4;",
    "鳥" => "&#x9CE5;",
    "鳦" => "&#x9CE6;",
    "鳧" => "&#x9CE7;",
    "鳨" => "&#x9CE8;",
    "鳩" => "&#x9CE9;",
    "鳪" => "&#x9CEA;",
    "鳫" => "&#x9CEB;",
    "鳬" => "&#x9CEC;",
    "鳭" => "&#x9CED;",
    "鳮" => "&#x9CEE;",
    "鳯" => "&#x9CEF;",
    "鳰" => "&#x9CF0;",
    "鳱" => "&#x9CF1;",
    "鳲" => "&#x9CF2;",
    "鳳" => "&#x9CF3;",
    "鳴" => "&#x9CF4;",
    "鳵" => "&#x9CF5;",
    "鳶" => "&#x9CF6;",
    "鳷" => "&#x9CF7;",
    "鳸" => "&#x9CF8;",
    "鳹" => "&#x9CF9;",
    "鳺" => "&#x9CFA;",
    "鳻" => "&#x9CFB;",
    "鳼" => "&#x9CFC;",
    "鳽" => "&#x9CFD;",
    "鳾" => "&#x9CFE;",
    "鳿" => "&#x9CFF;",
    "鴀" => "&#x9D00;",
    "鴁" => "&#x9D01;",
    "鴂" => "&#x9D02;",
    "鴃" => "&#x9D03;",
    "鴄" => "&#x9D04;",
    "鴅" => "&#x9D05;",
    "鴆" => "&#x9D06;",
    "鴇" => "&#x9D07;",
    "鴈" => "&#x9D08;",
    "鴉" => "&#x9D09;",
    "鴊" => "&#x9D0A;",
    "鴋" => "&#x9D0B;",
    "鴌" => "&#x9D0C;",
    "鴍" => "&#x9D0D;",
    "鴎" => "&#x9D0E;",
    "鴏" => "&#x9D0F;",
    "鴐" => "&#x9D10;",
    "鴑" => "&#x9D11;",
    "鴒" => "&#x9D12;",
    "鴓" => "&#x9D13;",
    "鴔" => "&#x9D14;",
    "鴕" => "&#x9D15;",
    "鴖" => "&#x9D16;",
    "鴗" => "&#x9D17;",
    "鴘" => "&#x9D18;",
    "鴙" => "&#x9D19;",
    "鴚" => "&#x9D1A;",
    "鴛" => "&#x9D1B;",
    "鴜" => "&#x9D1C;",
    "鴝" => "&#x9D1D;",
    "鴞" => "&#x9D1E;",
    "鴟" => "&#x9D1F;",
    "鴠" => "&#x9D20;",
    "鴡" => "&#x9D21;",
    "鴢" => "&#x9D22;",
    "鴣" => "&#x9D23;",
    "鴤" => "&#x9D24;",
    "鴥" => "&#x9D25;",
    "鴦" => "&#x9D26;",
    "鴧" => "&#x9D27;",
    "鴨" => "&#x9D28;",
    "鴩" => "&#x9D29;",
    "鴪" => "&#x9D2A;",
    "鴫" => "&#x9D2B;",
    "鴬" => "&#x9D2C;",
    "鴭" => "&#x9D2D;",
    "鴮" => "&#x9D2E;",
    "鴯" => "&#x9D2F;",
    "鴰" => "&#x9D30;",
    "鴱" => "&#x9D31;",
    "鴲" => "&#x9D32;",
    "鴳" => "&#x9D33;",
    "鴴" => "&#x9D34;",
    "鴵" => "&#x9D35;",
    "鴶" => "&#x9D36;",
    "鴷" => "&#x9D37;",
    "鴸" => "&#x9D38;",
    "鴹" => "&#x9D39;",
    "鴺" => "&#x9D3A;",
    "鴻" => "&#x9D3B;",
    "鴼" => "&#x9D3C;",
    "鴽" => "&#x9D3D;",
    "鴾" => "&#x9D3E;",
    "鴿" => "&#x9D3F;",
    "鵀" => "&#x9D40;",
    "鵁" => "&#x9D41;",
    "鵂" => "&#x9D42;",
    "鵃" => "&#x9D43;",
    "鵄" => "&#x9D44;",
    "鵅" => "&#x9D45;",
    "鵆" => "&#x9D46;",
    "鵇" => "&#x9D47;",
    "鵈" => "&#x9D48;",
    "鵉" => "&#x9D49;",
    "鵊" => "&#x9D4A;",
    "鵋" => "&#x9D4B;",
    "鵌" => "&#x9D4C;",
    "鵍" => "&#x9D4D;",
    "鵎" => "&#x9D4E;",
    "鵏" => "&#x9D4F;",
    "鵐" => "&#x9D50;",
    "鵑" => "&#x9D51;",
    "鵒" => "&#x9D52;",
    "鵓" => "&#x9D53;",
    "鵔" => "&#x9D54;",
    "鵕" => "&#x9D55;",
    "鵖" => "&#x9D56;",
    "鵗" => "&#x9D57;",
    "鵘" => "&#x9D58;",
    "鵙" => "&#x9D59;",
    "鵚" => "&#x9D5A;",
    "鵛" => "&#x9D5B;",
    "鵜" => "&#x9D5C;",
    "鵝" => "&#x9D5D;",
    "鵞" => "&#x9D5E;",
    "鵟" => "&#x9D5F;",
    "鵠" => "&#x9D60;",
    "鵡" => "&#x9D61;",
    "鵢" => "&#x9D62;",
    "鵣" => "&#x9D63;",
    "鵤" => "&#x9D64;",
    "鵥" => "&#x9D65;",
    "鵦" => "&#x9D66;",
    "鵧" => "&#x9D67;",
    "鵨" => "&#x9D68;",
    "鵩" => "&#x9D69;",
    "鵪" => "&#x9D6A;",
    "鵫" => "&#x9D6B;",
    "鵬" => "&#x9D6C;",
    "鵭" => "&#x9D6D;",
    "鵮" => "&#x9D6E;",
    "鵯" => "&#x9D6F;",
    "鵰" => "&#x9D70;",
    "鵱" => "&#x9D71;",
    "鵲" => "&#x9D72;",
    "鵳" => "&#x9D73;",
    "鵴" => "&#x9D74;",
    "鵵" => "&#x9D75;",
    "鵶" => "&#x9D76;",
    "鵷" => "&#x9D77;",
    "鵸" => "&#x9D78;",
    "鵹" => "&#x9D79;",
    "鵺" => "&#x9D7A;",
    "鵻" => "&#x9D7B;",
    "鵼" => "&#x9D7C;",
    "鵽" => "&#x9D7D;",
    "鵾" => "&#x9D7E;",
    "鵿" => "&#x9D7F;",
    "鶀" => "&#x9D80;",
    "鶁" => "&#x9D81;",
    "鶂" => "&#x9D82;",
    "鶃" => "&#x9D83;",
    "鶄" => "&#x9D84;",
    "鶅" => "&#x9D85;",
    "鶆" => "&#x9D86;",
    "鶇" => "&#x9D87;",
    "鶈" => "&#x9D88;",
    "鶉" => "&#x9D89;",
    "鶊" => "&#x9D8A;",
    "鶋" => "&#x9D8B;",
    "鶌" => "&#x9D8C;",
    "鶍" => "&#x9D8D;",
    "鶎" => "&#x9D8E;",
    "鶏" => "&#x9D8F;",
    "鶐" => "&#x9D90;",
    "鶑" => "&#x9D91;",
    "鶒" => "&#x9D92;",
    "鶓" => "&#x9D93;",
    "鶔" => "&#x9D94;",
    "鶕" => "&#x9D95;",
    "鶖" => "&#x9D96;",
    "鶗" => "&#x9D97;",
    "鶘" => "&#x9D98;",
    "鶙" => "&#x9D99;",
    "鶚" => "&#x9D9A;",
    "鶛" => "&#x9D9B;",
    "鶜" => "&#x9D9C;",
    "鶝" => "&#x9D9D;",
    "鶞" => "&#x9D9E;",
    "鶟" => "&#x9D9F;",
    "鶠" => "&#x9DA0;",
    "鶡" => "&#x9DA1;",
    "鶢" => "&#x9DA2;",
    "鶣" => "&#x9DA3;",
    "鶤" => "&#x9DA4;",
    "鶥" => "&#x9DA5;",
    "鶦" => "&#x9DA6;",
    "鶧" => "&#x9DA7;",
    "鶨" => "&#x9DA8;",
    "鶩" => "&#x9DA9;",
    "鶪" => "&#x9DAA;",
    "鶫" => "&#x9DAB;",
    "鶬" => "&#x9DAC;",
    "鶭" => "&#x9DAD;",
    "鶮" => "&#x9DAE;",
    "鶯" => "&#x9DAF;",
    "鶰" => "&#x9DB0;",
    "鶱" => "&#x9DB1;",
    "鶲" => "&#x9DB2;",
    "鶳" => "&#x9DB3;",
    "鶴" => "&#x9DB4;",
    "鶵" => "&#x9DB5;",
    "鶶" => "&#x9DB6;",
    "鶷" => "&#x9DB7;",
    "鶸" => "&#x9DB8;",
    "鶹" => "&#x9DB9;",
    "鶺" => "&#x9DBA;",
    "鶻" => "&#x9DBB;",
    "鶼" => "&#x9DBC;",
    "鶽" => "&#x9DBD;",
    "鶾" => "&#x9DBE;",
    "鶿" => "&#x9DBF;",
    "鷀" => "&#x9DC0;",
    "鷁" => "&#x9DC1;",
    "鷂" => "&#x9DC2;",
    "鷃" => "&#x9DC3;",
    "鷄" => "&#x9DC4;",
    "鷅" => "&#x9DC5;",
    "鷆" => "&#x9DC6;",
    "鷇" => "&#x9DC7;",
    "鷈" => "&#x9DC8;",
    "鷉" => "&#x9DC9;",
    "鷊" => "&#x9DCA;",
    "鷋" => "&#x9DCB;",
    "鷌" => "&#x9DCC;",
    "鷍" => "&#x9DCD;",
    "鷎" => "&#x9DCE;",
    "鷏" => "&#x9DCF;",
    "鷐" => "&#x9DD0;",
    "鷑" => "&#x9DD1;",
    "鷒" => "&#x9DD2;",
    "鷓" => "&#x9DD3;",
    "鷔" => "&#x9DD4;",
    "鷕" => "&#x9DD5;",
    "鷖" => "&#x9DD6;",
    "鷗" => "&#x9DD7;",
    "鷘" => "&#x9DD8;",
    "鷙" => "&#x9DD9;",
    "鷚" => "&#x9DDA;",
    "鷛" => "&#x9DDB;",
    "鷜" => "&#x9DDC;",
    "鷝" => "&#x9DDD;",
    "鷞" => "&#x9DDE;",
    "鷟" => "&#x9DDF;",
    "鷠" => "&#x9DE0;",
    "鷡" => "&#x9DE1;",
    "鷢" => "&#x9DE2;",
    "鷣" => "&#x9DE3;",
    "鷤" => "&#x9DE4;",
    "鷥" => "&#x9DE5;",
    "鷦" => "&#x9DE6;",
    "鷧" => "&#x9DE7;",
    "鷨" => "&#x9DE8;",
    "鷩" => "&#x9DE9;",
    "鷪" => "&#x9DEA;",
    "鷫" => "&#x9DEB;",
    "鷬" => "&#x9DEC;",
    "鷭" => "&#x9DED;",
    "鷮" => "&#x9DEE;",
    "鷯" => "&#x9DEF;",
    "鷰" => "&#x9DF0;",
    "鷱" => "&#x9DF1;",
    "鷲" => "&#x9DF2;",
    "鷳" => "&#x9DF3;",
    "鷴" => "&#x9DF4;",
    "鷵" => "&#x9DF5;",
    "鷶" => "&#x9DF6;",
    "鷷" => "&#x9DF7;",
    "鷸" => "&#x9DF8;",
    "鷹" => "&#x9DF9;",
    "鷺" => "&#x9DFA;",
    "鷻" => "&#x9DFB;",
    "鷼" => "&#x9DFC;",
    "鷽" => "&#x9DFD;",
    "鷾" => "&#x9DFE;",
    "鷿" => "&#x9DFF;",
    "鸀" => "&#x9E00;",
    "鸁" => "&#x9E01;",
    "鸂" => "&#x9E02;",
    "鸃" => "&#x9E03;",
    "鸄" => "&#x9E04;",
    "鸅" => "&#x9E05;",
    "鸆" => "&#x9E06;",
    "鸇" => "&#x9E07;",
    "鸈" => "&#x9E08;",
    "鸉" => "&#x9E09;",
    "鸊" => "&#x9E0A;",
    "鸋" => "&#x9E0B;",
    "鸌" => "&#x9E0C;",
    "鸍" => "&#x9E0D;",
    "鸎" => "&#x9E0E;",
    "鸏" => "&#x9E0F;",
    "鸐" => "&#x9E10;",
    "鸑" => "&#x9E11;",
    "鸒" => "&#x9E12;",
    "鸓" => "&#x9E13;",
    "鸔" => "&#x9E14;",
    "鸕" => "&#x9E15;",
    "鸖" => "&#x9E16;",
    "鸗" => "&#x9E17;",
    "鸘" => "&#x9E18;",
    "鸙" => "&#x9E19;",
    "鸚" => "&#x9E1A;",
    "鸛" => "&#x9E1B;",
    "鸜" => "&#x9E1C;",
    "鸝" => "&#x9E1D;",
    "鸞" => "&#x9E1E;",
    "鸟" => "&#x9E1F;",
    "鸠" => "&#x9E20;",
    "鸡" => "&#x9E21;",
    "鸢" => "&#x9E22;",
    "鸣" => "&#x9E23;",
    "鸤" => "&#x9E24;",
    "鸥" => "&#x9E25;",
    "鸦" => "&#x9E26;",
    "鸧" => "&#x9E27;",
    "鸨" => "&#x9E28;",
    "鸩" => "&#x9E29;",
    "鸪" => "&#x9E2A;",
    "鸫" => "&#x9E2B;",
    "鸬" => "&#x9E2C;",
    "鸭" => "&#x9E2D;",
    "鸮" => "&#x9E2E;",
    "鸯" => "&#x9E2F;",
    "鸰" => "&#x9E30;",
    "鸱" => "&#x9E31;",
    "鸲" => "&#x9E32;",
    "鸳" => "&#x9E33;",
    "鸴" => "&#x9E34;",
    "鸵" => "&#x9E35;",
    "鸶" => "&#x9E36;",
    "鸷" => "&#x9E37;",
    "鸸" => "&#x9E38;",
    "鸹" => "&#x9E39;",
    "鸺" => "&#x9E3A;",
    "鸻" => "&#x9E3B;",
    "鸼" => "&#x9E3C;",
    "鸽" => "&#x9E3D;",
    "鸾" => "&#x9E3E;",
    "鸿" => "&#x9E3F;",
    "鹀" => "&#x9E40;",
    "鹁" => "&#x9E41;",
    "鹂" => "&#x9E42;",
    "鹃" => "&#x9E43;",
    "鹄" => "&#x9E44;",
    "鹅" => "&#x9E45;",
    "鹆" => "&#x9E46;",
    "鹇" => "&#x9E47;",
    "鹈" => "&#x9E48;",
    "鹉" => "&#x9E49;",
    "鹊" => "&#x9E4A;",
    "鹋" => "&#x9E4B;",
    "鹌" => "&#x9E4C;",
    "鹍" => "&#x9E4D;",
    "鹎" => "&#x9E4E;",
    "鹏" => "&#x9E4F;",
    "鹐" => "&#x9E50;",
    "鹑" => "&#x9E51;",
    "鹒" => "&#x9E52;",
    "鹓" => "&#x9E53;",
    "鹔" => "&#x9E54;",
    "鹕" => "&#x9E55;",
    "鹖" => "&#x9E56;",
    "鹗" => "&#x9E57;",
    "鹘" => "&#x9E58;",
    "鹙" => "&#x9E59;",
    "鹚" => "&#x9E5A;",
    "鹛" => "&#x9E5B;",
    "鹜" => "&#x9E5C;",
    "鹝" => "&#x9E5D;",
    "鹞" => "&#x9E5E;",
    "鹟" => "&#x9E5F;",
    "鹠" => "&#x9E60;",
    "鹡" => "&#x9E61;",
    "鹢" => "&#x9E62;",
    "鹣" => "&#x9E63;",
    "鹤" => "&#x9E64;",
    "鹥" => "&#x9E65;",
    "鹦" => "&#x9E66;",
    "鹧" => "&#x9E67;",
    "鹨" => "&#x9E68;",
    "鹩" => "&#x9E69;",
    "鹪" => "&#x9E6A;",
    "鹫" => "&#x9E6B;",
    "鹬" => "&#x9E6C;",
    "鹭" => "&#x9E6D;",
    "鹮" => "&#x9E6E;",
    "鹯" => "&#x9E6F;",
    "鹰" => "&#x9E70;",
    "鹱" => "&#x9E71;",
    "鹲" => "&#x9E72;",
    "鹳" => "&#x9E73;",
    "鹴" => "&#x9E74;",
    "鹵" => "&#x9E75;",
    "鹶" => "&#x9E76;",
    "鹷" => "&#x9E77;",
    "鹸" => "&#x9E78;",
    "鹹" => "&#x9E79;",
    "鹺" => "&#x9E7A;",
    "鹻" => "&#x9E7B;",
    "鹼" => "&#x9E7C;",
    "鹽" => "&#x9E7D;",
    "鹾" => "&#x9E7E;",
    "鹿" => "&#x9E7F;",
    "麀" => "&#x9E80;",
    "麁" => "&#x9E81;",
    "麂" => "&#x9E82;",
    "麃" => "&#x9E83;",
    "麄" => "&#x9E84;",
    "麅" => "&#x9E85;",
    "麆" => "&#x9E86;",
    "麇" => "&#x9E87;",
    "麈" => "&#x9E88;",
    "麉" => "&#x9E89;",
    "麊" => "&#x9E8A;",
    "麋" => "&#x9E8B;",
    "麌" => "&#x9E8C;",
    "麍" => "&#x9E8D;",
    "麎" => "&#x9E8E;",
    "麏" => "&#x9E8F;",
    "麐" => "&#x9E90;",
    "麑" => "&#x9E91;",
    "麒" => "&#x9E92;",
    "麓" => "&#x9E93;",
    "麔" => "&#x9E94;",
    "麕" => "&#x9E95;",
    "麖" => "&#x9E96;",
    "麗" => "&#x9E97;",
    "麘" => "&#x9E98;",
    "麙" => "&#x9E99;",
    "麚" => "&#x9E9A;",
    "麛" => "&#x9E9B;",
    "麜" => "&#x9E9C;",
    "麝" => "&#x9E9D;",
    "麞" => "&#x9E9E;",
    "麟" => "&#x9E9F;",
    "麠" => "&#x9EA0;",
    "麡" => "&#x9EA1;",
    "麢" => "&#x9EA2;",
    "麣" => "&#x9EA3;",
    "麤" => "&#x9EA4;",
    "麥" => "&#x9EA5;",
    "麦" => "&#x9EA6;",
    "麧" => "&#x9EA7;",
    "麨" => "&#x9EA8;",
    "麩" => "&#x9EA9;",
    "麪" => "&#x9EAA;",
    "麫" => "&#x9EAB;",
    "麬" => "&#x9EAC;",
    "麭" => "&#x9EAD;",
    "麮" => "&#x9EAE;",
    "麯" => "&#x9EAF;",
    "麰" => "&#x9EB0;",
    "麱" => "&#x9EB1;",
    "麲" => "&#x9EB2;",
    "麳" => "&#x9EB3;",
    "麴" => "&#x9EB4;",
    "麵" => "&#x9EB5;",
    "麶" => "&#x9EB6;",
    "麷" => "&#x9EB7;",
    "麸" => "&#x9EB8;",
    "麹" => "&#x9EB9;",
    "麺" => "&#x9EBA;",
    "麻" => "&#x9EBB;",
    "麼" => "&#x9EBC;",
    "麽" => "&#x9EBD;",
    "麾" => "&#x9EBE;",
    "麿" => "&#x9EBF;",
    "黀" => "&#x9EC0;",
    "黁" => "&#x9EC1;",
    "黂" => "&#x9EC2;",
    "黃" => "&#x9EC3;",
    "黄" => "&#x9EC4;",
    "黅" => "&#x9EC5;",
    "黆" => "&#x9EC6;",
    "黇" => "&#x9EC7;",
    "黈" => "&#x9EC8;",
    "黉" => "&#x9EC9;",
    "黊" => "&#x9ECA;",
    "黋" => "&#x9ECB;",
    "黌" => "&#x9ECC;",
    "黍" => "&#x9ECD;",
    "黎" => "&#x9ECE;",
    "黏" => "&#x9ECF;",
    "黐" => "&#x9ED0;",
    "黑" => "&#x9ED1;",
    "黒" => "&#x9ED2;",
    "黓" => "&#x9ED3;",
    "黔" => "&#x9ED4;",
    "黕" => "&#x9ED5;",
    "黖" => "&#x9ED6;",
    "黗" => "&#x9ED7;",
    "默" => "&#x9ED8;",
    "黙" => "&#x9ED9;",
    "黚" => "&#x9EDA;",
    "黛" => "&#x9EDB;",
    "黜" => "&#x9EDC;",
    "黝" => "&#x9EDD;",
    "點" => "&#x9EDE;",
    "黟" => "&#x9EDF;",
    "黠" => "&#x9EE0;",
    "黡" => "&#x9EE1;",
    "黢" => "&#x9EE2;",
    "黣" => "&#x9EE3;",
    "黤" => "&#x9EE4;",
    "黥" => "&#x9EE5;",
    "黦" => "&#x9EE6;",
    "黧" => "&#x9EE7;",
    "黨" => "&#x9EE8;",
    "黩" => "&#x9EE9;",
    "黪" => "&#x9EEA;",
    "黫" => "&#x9EEB;",
    "黬" => "&#x9EEC;",
    "黭" => "&#x9EED;",
    "黮" => "&#x9EEE;",
    "黯" => "&#x9EEF;",
    "黰" => "&#x9EF0;",
    "黱" => "&#x9EF1;",
    "黲" => "&#x9EF2;",
    "黳" => "&#x9EF3;",
    "黴" => "&#x9EF4;",
    "黵" => "&#x9EF5;",
    "黶" => "&#x9EF6;",
    "黷" => "&#x9EF7;",
    "黸" => "&#x9EF8;",
    "黹" => "&#x9EF9;",
    "黺" => "&#x9EFA;",
    "黻" => "&#x9EFB;",
    "黼" => "&#x9EFC;",
    "黽" => "&#x9EFD;",
    "黾" => "&#x9EFE;",
    "黿" => "&#x9EFF;",
    "鼀" => "&#x9F00;",
    "鼁" => "&#x9F01;",
    "鼂" => "&#x9F02;",
    "鼃" => "&#x9F03;",
    "鼄" => "&#x9F04;",
    "鼅" => "&#x9F05;",
    "鼆" => "&#x9F06;",
    "鼇" => "&#x9F07;",
    "鼈" => "&#x9F08;",
    "鼉" => "&#x9F09;",
    "鼊" => "&#x9F0A;",
    "鼋" => "&#x9F0B;",
    "鼌" => "&#x9F0C;",
    "鼍" => "&#x9F0D;",
    "鼎" => "&#x9F0E;",
    "鼏" => "&#x9F0F;",
    "鼐" => "&#x9F10;",
    "鼑" => "&#x9F11;",
    "鼒" => "&#x9F12;",
    "鼓" => "&#x9F13;",
    "鼔" => "&#x9F14;",
    "鼕" => "&#x9F15;",
    "鼖" => "&#x9F16;",
    "鼗" => "&#x9F17;",
    "鼘" => "&#x9F18;",
    "鼙" => "&#x9F19;",
    "鼚" => "&#x9F1A;",
    "鼛" => "&#x9F1B;",
    "鼜" => "&#x9F1C;",
    "鼝" => "&#x9F1D;",
    "鼞" => "&#x9F1E;",
    "鼟" => "&#x9F1F;",
    "鼠" => "&#x9F20;",
    "鼡" => "&#x9F21;",
    "鼢" => "&#x9F22;",
    "鼣" => "&#x9F23;",
    "鼤" => "&#x9F24;",
    "鼥" => "&#x9F25;",
    "鼦" => "&#x9F26;",
    "鼧" => "&#x9F27;",
    "鼨" => "&#x9F28;",
    "鼩" => "&#x9F29;",
    "鼪" => "&#x9F2A;",
    "鼫" => "&#x9F2B;",
    "鼬" => "&#x9F2C;",
    "鼭" => "&#x9F2D;",
    "鼮" => "&#x9F2E;",
    "鼯" => "&#x9F2F;",
    "鼰" => "&#x9F30;",
    "鼱" => "&#x9F31;",
    "鼲" => "&#x9F32;",
    "鼳" => "&#x9F33;",
    "鼴" => "&#x9F34;",
    "鼵" => "&#x9F35;",
    "鼶" => "&#x9F36;",
    "鼷" => "&#x9F37;",
    "鼸" => "&#x9F38;",
    "鼹" => "&#x9F39;",
    "鼺" => "&#x9F3A;",
    "鼻" => "&#x9F3B;",
    "鼼" => "&#x9F3C;",
    "鼽" => "&#x9F3D;",
    "鼾" => "&#x9F3E;",
    "鼿" => "&#x9F3F;",
    "齀" => "&#x9F40;",
    "齁" => "&#x9F41;",
    "齂" => "&#x9F42;",
    "齃" => "&#x9F43;",
    "齄" => "&#x9F44;",
    "齅" => "&#x9F45;",
    "齆" => "&#x9F46;",
    "齇" => "&#x9F47;",
    "齈" => "&#x9F48;",
    "齉" => "&#x9F49;",
    "齊" => "&#x9F4A;",
    "齋" => "&#x9F4B;",
    "齌" => "&#x9F4C;",
    "齍" => "&#x9F4D;",
    "齎" => "&#x9F4E;",
    "齏" => "&#x9F4F;",
    "齐" => "&#x9F50;",
    "齑" => "&#x9F51;",
    "齒" => "&#x9F52;",
    "齓" => "&#x9F53;",
    "齔" => "&#x9F54;",
    "齕" => "&#x9F55;",
    "齖" => "&#x9F56;",
    "齗" => "&#x9F57;",
    "齘" => "&#x9F58;",
    "齙" => "&#x9F59;",
    "齚" => "&#x9F5A;",
    "齛" => "&#x9F5B;",
    "齜" => "&#x9F5C;",
    "齝" => "&#x9F5D;",
    "齞" => "&#x9F5E;",
    "齟" => "&#x9F5F;",
    "齠" => "&#x9F60;",
    "齡" => "&#x9F61;",
    "齢" => "&#x9F62;",
    "齣" => "&#x9F63;",
    "齤" => "&#x9F64;",
    "齥" => "&#x9F65;",
    "齦" => "&#x9F66;",
    "齧" => "&#x9F67;",
    "齨" => "&#x9F68;",
    "齩" => "&#x9F69;",
    "齪" => "&#x9F6A;",
    "齫" => "&#x9F6B;",
    "齬" => "&#x9F6C;",
    "齭" => "&#x9F6D;",
    "齮" => "&#x9F6E;",
    "齯" => "&#x9F6F;",
    "齰" => "&#x9F70;",
    "齱" => "&#x9F71;",
    "齲" => "&#x9F72;",
    "齳" => "&#x9F73;",
    "齴" => "&#x9F74;",
    "齵" => "&#x9F75;",
    "齶" => "&#x9F76;",
    "齷" => "&#x9F77;",
    "齸" => "&#x9F78;",
    "齹" => "&#x9F79;",
    "齺" => "&#x9F7A;",
    "齻" => "&#x9F7B;",
    "齼" => "&#x9F7C;",
    "齽" => "&#x9F7D;",
    "齾" => "&#x9F7E;",
    "齿" => "&#x9F7F;",
    "龀" => "&#x9F80;",
    "龁" => "&#x9F81;",
    "龂" => "&#x9F82;",
    "龃" => "&#x9F83;",
    "龄" => "&#x9F84;",
    "龅" => "&#x9F85;",
    "龆" => "&#x9F86;",
    "龇" => "&#x9F87;",
    "龈" => "&#x9F88;",
    "龉" => "&#x9F89;",
    "龊" => "&#x9F8A;",
    "龋" => "&#x9F8B;",
    "龌" => "&#x9F8C;",
    "龍" => "&#x9F8D;",
    "龎" => "&#x9F8E;",
    "龏" => "&#x9F8F;",
    "龐" => "&#x9F90;",
    "龑" => "&#x9F91;",
    "龒" => "&#x9F92;",
    "龓" => "&#x9F93;",
    "龔" => "&#x9F94;",
    "龕" => "&#x9F95;",
    "龖" => "&#x9F96;",
    "龗" => "&#x9F97;",
    "龘" => "&#x9F98;",
    "龙" => "&#x9F99;",
    "龚" => "&#x9F9A;",
    "龛" => "&#x9F9B;",
    "龜" => "&#x9F9C;",
    "龝" => "&#x9F9D;",
    "龞" => "&#x9F9E;",
    "龟" => "&#x9F9F;",
    "龠" => "&#x9FA0;",
    "龡" => "&#x9FA1;",
    "龢" => "&#x9FA2;",
    "龣" => "&#x9FA3;",
    "龤" => "&#x9FA4;",
    "龥" => "&#x9FA5;",
    "龦" => "&#x9FA6;",
    "龧" => "&#x9FA7;",
    "龨" => "&#x9FA8;",
    "龩" => "&#x9FA9;",
    "龪" => "&#x9FAA;",
    "龫" => "&#x9FAB;",
    "龬" => "&#x9FAC;",
    "龭" => "&#x9FAD;",
    "龮" => "&#x9FAE;",
    "龯" => "&#x9FAF;",
    "龰" => "&#x9FB0;",
    "龱" => "&#x9FB1;",
    "龲" => "&#x9FB2;",
    "龳" => "&#x9FB3;",
    "龴" => "&#x9FB4;",
    "龵" => "&#x9FB5;",
    "龶" => "&#x9FB6;",
    "龷" => "&#x9FB7;",
    "龸" => "&#x9FB8;",
    "龹" => "&#x9FB9;",
    "龺" => "&#x9FBA;",
    "龻" => "&#x9FBB;",
);